
module ATI_imem_rf_bfm(
  Reset,   
  RdClk, RdAdr, RdEn, RdDat,
  WrClk, WrAdr, WrEn, WrDat,
  WrEnMask,  //BYTE-write mask
  PipeStall,
  MemPowerCtrl
);

`ifdef ATI_HW_ACCEL

  ///
  ///  Stripped down implementation used by Hardware Accelerator
  ///

  // Parameters
  parameter NW = 8; //number of words
  parameter NB = 8; //bits per word
  parameter NA = 3; //number of addr bits
  parameter MX = 1; //mux option
  parameter BM = 0; //bit mask width
  parameter PM = 1; //pipe line mode,
  // 0->non-pipelined
  // 1->pipelined data with pipelined PipeStall
  // 2->pipelined data with unpipelined PipeStall
  
  // Inputs / Outputs
  input                        Reset;
  input                        RdClk;
  input                        RdEn;
  input                        WrClk;
  input                        WrEn;
  input [NA-1:0]               RdAdr, WrAdr;
  input [NB-1:0]               WrDat;
  input [((0==BM)?0:(BM-1)):0] WrEnMask;
  input                        PipeStall;
  input                        MemPowerCtrl;
  output [NB-1:0]              RdDat;

  // Flopped Address and Data
  reg [NB-1:0] RdDat0;
  reg [NB-1:0] RdDat1;
  reg          PipeStallQ;

  // This is a hack until all the wrappers can hook up MemPowerCtrl port
  wire _MemPowerCtrl = (MemPowerCtrl===1'bz) ? 1'b0 : MemPowerCtrl;

  // Two Dimensional Memory
  reg [NB-1:0] Mem [NW-1:0];

  // Output Wire
  wire [NB-1:0] RdDat;

  // Write Clock Interface
  always @(posedge WrClk) begin
    if(WrEn && !_MemPowerCtrl) begin
      if(BM == 0)
        Mem[WrAdr] <= WrDat;
      else
        Mem[WrAdr] <= (WrDat & WrEnMask) | (Mem[WrAdr] & ~WrEnMask); // Read Modify Write using WrEnMask
    end
  end

  // Read Clock Interface
  always @(posedge RdClk) begin
    if(!_MemPowerCtrl) begin
      PipeStallQ <= PipeStall;       // Pipe Signal
      if(RdEn)
        RdDat0 <= Mem[RdAdr];
    end
    RdDat1 <= ((PM<2)&&PipeStallQ || (PM==2)&&PipeStall ? RdDat0 : RdDat1); // In case of Stall Recirculate
  end

  assign RdDat = PM ? RdDat1 : RdDat0;

`else
  parameter NW = 8; //number of words
  parameter NB = 8; //bits per word
  parameter NA = 3; //number of addr bits
  parameter MX = 1; //mux option
  parameter BM = 0; //bit mask width
  parameter PM = 1; //pipe line mode,
  // 0->non-pipelined
  // 1->pipelined data with pipelined PipeStall
  // 2->pipelined data with unpipelined PipeStall

  input                        Reset;
  input                        RdClk, RdEn, WrClk, WrEn;
  input [NA-1:0]               RdAdr, WrAdr;
  input [NB-1:0]               WrDat;
  input [((0==BM)?0:(BM-1)):0] WrEnMask;
  input                        PipeStall;
  input                        MemPowerCtrl;

  output [NB-1:0] RdDat;

  reg [NA-1:0]                 WrAdrQ, RdAdrQ;
  reg [((0==BM)?0:(BM-1)):0]   WrEnMaskQ;
  reg                          WrEnQ;
  reg                          RdEnQ;
  reg [NB-1:0]                 RdDatQ1;
  reg                          PipeStallQ;
  reg [NB-1:0]                 Mem [NW-1:0];
  
  // This is a hack until all the wrappers can hook up MemPowerCtrl port
  wire _MemPowerCtrl = (MemPowerCtrl===1'bz) ? 1'b0 : MemPowerCtrl;

  integer msg_cnt = 0;

  wire same_address_exception;

  always @(posedge WrClk) begin : writeBlk
    if(!_MemPowerCtrl) begin
      WrAdrQ <= WrAdr;
      WrEnQ <= WrEn;
      WrEnMaskQ <= WrEnMask;

      if(WrEn) begin
  // VCS coverage off
        if(same_address_exception && $test$plusargs("corrupt_write_at_collision")) begin
          $display("time=%t: write operation corrupted@%x @%m\n", $time, WrAdr);
          Mem[WrAdr] <= {NB{1'bx}};
        end
  // VCS coverage on
        else begin
          if(BM == 0) begin
            Mem[WrAdr] <= WrDat;
          end
          else begin : a0
            Mem[WrAdr] <= (WrDat & WrEnMask) | (Mem[WrAdr] & ~WrEnMask);
          end
        end
      end
    end
  end

  // Read transactions
  // check for these illegal conditions -
  //
  // 1. Read Address == Write Address (R/W overlap contention)
  // 2. Read Address out of bounds
  //

  always @(posedge RdClk) begin
    if(!_MemPowerCtrl) begin
      RdEnQ <= RdEn;
      if(RdEn)
        RdAdrQ <= RdAdr;
        PipeStallQ <= PipeStall;
    end
  end

  wire out_of_bounds_exception = (RdAdrQ > NW);

 `ifdef NO_ASYNC_ADR_COLL

  reg  WrClk_posedge, RdClk_posedge;
  reg  same_address_exception_r;

  always@(posedge WrClk) begin
    WrClk_posedge <= 1;
    same_address_exception_r <= 0;
    #0.05 WrClk_posedge <= 0;
  end

  always@(posedge RdClk) begin
    RdClk_posedge <= 1;
    same_address_exception_r <= 0;
    #0.05 RdClk_posedge <= 0;
  end

  wire same_edge;
  assign same_edge = WrClk_posedge & RdClk_posedge;

  always@(posedge same_edge) begin
// VCS coverage off
    same_address_exception_r <= (WrAdrQ == RdAdrQ & (WrEnQ & RdEnQ) )? 1 : 0;
// VCS coverage on
  end

  assign same_address_exception = same_address_exception_r;
 `else
  assign same_address_exception = (WrAdrQ == RdAdrQ & (WrEnQ & RdEnQ) )? 1 : 0;
 `endif //NO_ASYNC_ADR_COLL

  reg [NB-1:0] exception_mask;

  always @(/*AUTOSENSE*/WrAdrQ or WrEnMaskQ
           or out_of_bounds_exception or same_address_exception) begin
    exception_mask={NB{1'b0}};

// VCS coverage off
    if(same_address_exception) begin
      if(msg_cnt < 5) begin
        if($test$plusargs("collision_msg"))
          $display("time=%t: Simultanuous read/write on the address=%x @%m\n", $time, WrAdrQ);
        msg_cnt = msg_cnt + 1;
      end

      if(BM==0) begin
        exception_mask={NB{1'bx}};
      end
      else begin: b0
        exception_mask={NB{1'bx}}&WrEnMaskQ;
      end
    end

    if(out_of_bounds_exception)
      exception_mask={NB{1'bx}};
// VCS coverage on

  end

  reg [NB-1:0] RdDatP;

  always @(posedge RdClk) begin
    if(!_MemPowerCtrl) begin
      if(RdEn)
        RdDatP  <= Mem[RdAdr];
    end
  end

  wire [NB-1:0] RdDatP_error;
  assign RdDatP_error = exception_mask ^ RdDatP;

  reg PipeStallQ_q;
  always @(posedge RdClk) begin
    PipeStallQ_q <= PipeStallQ;
  end
  
  always @(posedge RdClk) begin
    RdDatQ1 <= ((PM<2) && PipeStallQ) || ((PM==2) && PipeStall) ? RdDatP_error : RdDatQ1;
  end

  //assign RdDat = ((PM > 0) && PipeStallQ_q) ? RdDatP_error : RdDatQ1;
  assign RdDat =  PM ? RdDatQ1 : RdDatP_error;
  
`endif //ATI_HW_ACCEL

endmodule


module ATI_imem_pdp_bfm(
  Reset,    
  RdClk, RdAdr, RdEn, RdDat,
  WrClk, WrAdr, WrEn, WrDat,
  WrEnMask,  //BYTE-write mask
  PipeStall,
  MemPowerCtrl
);

  parameter NW = 8; //number of words
  parameter NB = 8; //bits per word
  parameter NA = 3; //number of addr bits
  parameter MX = 1; //mux option
  parameter BM = 0; //bit mask width
  parameter PM = 1; //pipe line mode,
  // 0->non-pipelined
  // 1->pipelined data with pipelined PipeStall
  // 2->pipelined data with unpipelined PipeStall

  input                        Reset;
  input                        RdClk, RdEn, WrClk, WrEn;
  input [NA-1:0]               RdAdr, WrAdr;
  input [NB-1:0]               WrDat;
  input [((0==BM)?0:(BM-1)):0] WrEnMask;
  input                        PipeStall;
  input                        MemPowerCtrl;

  output [NB-1:0]              RdDat;

  reg [NB-1:0]                 Mem [NW-1:0];
  
  // This is a hack until all the wrappers can hook up MemPowerCtrl port
  wire _MemPowerCtrl = (MemPowerCtrl===1'bz) ? 1'b0 : MemPowerCtrl;
  
  always @(posedge WrClk) begin : writeBlk
    if(WrEn && !_MemPowerCtrl) begin
      if(BM == 0)
        Mem[WrAdr] <= WrDat;
      else begin : a0
        Mem[WrAdr] <= (WrDat & WrEnMask) | (Mem[WrAdr] & ~WrEnMask);
      end
    end
  end

  reg [NB-1:0] RdDatQ; //This is equivalent to the sense amp output
  reg [NB-1:0] RdDatQQ;
  reg          PipeStallQ;

  always @(posedge RdClk) begin
    if(!_MemPowerCtrl) begin
      PipeStallQ <= PipeStall;
      if(RdEn)
        RdDatQ <= Mem[RdAdr];
    end
    if(((PM==1)&&PipeStallQ) || ((PM==2)&&PipeStall))
      RdDatQQ <= RdDatQ;
  end

  assign RdDat = PM ? RdDatQQ : RdDatQ;

endmodule


module ATI_imem_hd_bfm(
  Reset,    
  WrClk, WrEn, RdEn, Adr, WrDat, RdDat, WrEnMask, PipeStall, MemPowerCtrl
);

  parameter NW = 8; //number of words
  parameter NB = 8; //bits per word
  parameter NA = 3; //number of addr bits
  parameter MX = 1; //mux option
  parameter BM = 0; //BIT-write enable
  parameter PM = 1; 
  // 0->non-pipelined
  // 1->pipelined data with pipelined PipeStall
  // 2->pipelined data with unpipelined PipeStall

  input                        Reset;
  input                        RdEn, WrClk, WrEn;
  input [NA-1:0]               Adr;
  input [NB-1:0]               WrDat;
  input [((0==BM)?0:(BM-1)):0] WrEnMask;
  input                        PipeStall;
  input                        MemPowerCtrl;

  output [NB-1:0]              RdDat;

  reg                          PipeStallQ;
  reg [NB-1:0]                 Mem [NW-1:0];
  reg [NB-1:0]                 RdDatQ;
  reg [NB-1:0]                 RdDatQQ;

  // This is a hack until all the wrappers can hook up PipeStall and MemPowerCtrl port
  wire _PipeStall = (PipeStall===1'bz) ? 1'b1 : PipeStall;
  wire _MemPowerCtrl = (MemPowerCtrl===1'bz) ? 1'b0 : MemPowerCtrl;

  always @(posedge WrClk) begin
    if(WrEn && !_MemPowerCtrl) begin
      if(0 == BM) begin
        Mem[Adr] <= WrDat;
      end
      else begin
        Mem[Adr] <= (WrDat & WrEnMask) | (Mem[Adr] & ~WrEnMask);
      end
    end
  end

  always @(posedge WrClk) begin
    if(!_MemPowerCtrl) begin
      PipeStallQ <= _PipeStall;
      if(RdEn)
        RdDatQ <= Mem[Adr];
    end
    if(((PM==1)&&PipeStallQ) || ((PM==2)&&_PipeStall))
      RdDatQQ <= RdDatQ;
  end
  
  assign RdDat = PM ? RdDatQQ : RdDatQ;

// VCS coverage off
  
  integer i;
  reg [8*256-1:0] Hier;
  reg [8*256-1:0] InstanceName;
  reg check_psl;
  
  initial begin
    $sformat(Hier, "%m");
    InstanceName = Hier;
    
    for(i=0; (i < 256) && (Hier[7:0] != "."); i=i+1) begin
      Hier = Hier >> 8;
    end
    
    repeat(256-i) begin
      InstanceName = InstanceName << 8;
    end
    
    repeat(256-i) begin
      InstanceName = InstanceName >> 8;
    end
    
    check_psl = 1'b1;
    
    if($test$plusargs("ati_imem_bfm_psl_check=off"))
      check_psl = 1'b0;
    
    if($test$plusargs({InstanceName,"_psl_check=off"}))
      check_psl = 1'b0;
  end
  
  wire ME = RdEn;

// psl property write_mask_not_x_property = always((check_psl && ME && WrEn && (BM>0))->next[0]((WrEnMask^WrEnMask)=={NB{1'b0}})) @(posedge WrClk);
// psl write_mask_not_x : assert write_mask_not_x_property report "ERROR: Write mask contains x's in memory bfm" severity error;  
  
// psl property write_not_x_property = always ((check_psl && ME && WrEn)->next[0](
//                                            ((BM==0) && ((WrDat^WrDat)=={NB{1'b0}})) ||
//                                            ((BM>0) && (((WrDat|(~WrEnMask))^(WrDat|(~WrEnMask))) == {NB{1'b0}})))
//                                           ) @(posedge WrClk);
// psl write_not_x : assert write_not_x_property report "ERROR: Writing x's to memory bfm" severity error;
  
// psl property read_not_x_property = always ((check_psl && ME && !WrEn && ((PM!=1)||_PipeStall))->next[PM?2:1](
//                                            ((BM==0) && ((RdDat^RdDat)=={NB{1'b0}})) ||
//                                            ((BM>0) && (RdDat !== {NB{1'bX}})) ||
//                                            ((PM==2)&&!PipeStallQ))
//                                           ) @(posedge WrClk);
// psl read_not_x : assert read_not_x_property report "ERROR: Reading x's from memory bfm" severity error;
  
// psl property addr_out_of_range_property = always ((check_psl && ME)->next[0](Adr<NW)) @(posedge WrClk);
// psl addr_out_of_range : assert addr_out_of_range_property report "ERROR: Attempting to access out of bounds address in memory bfm" severity error;

// VCS coverage on  
endmodule


module ATI_imem_larr_bfm(
  RdAdr, RdDat,
  WrClk, WrAdr, WrEn, WrDat,
);

  parameter NW = 8; //number of words
  parameter NB = 8; //bits per word
  parameter NA = 3; //number of addr bits

  input                        WrClk, WrEn;
  input [NA-1:0]               RdAdr, WrAdr;
  input [NB-1:0]               WrDat;

  output [NB-1:0]              RdDat;

  reg [NB-1:0]                 Mem [NW-1:0];
  
  always @(posedge WrClk) begin : writeBlk
    if(WrEn) begin
        Mem[WrAdr] <= WrDat;
    end
  end
  assign RdDat = Mem[RdAdr];
    
endmodule

module ATI_edcmem_pdp_bfm (
  Reset,    
  RdClk, RdAdr, RdEn, RdDat,
  WrClk, WrAdr, WrEn, WrDat,
  WrEnMask,  //BYTE-write mask
  PipeStall,
  MemPowerCtrl,
  Fed, FedWrEnMask, Ded
);

  parameter NW = 8; //number of words
  parameter NB = 8; //bits per word
  parameter NA = 3; //number of addr bits
  parameter MX = 1; //mux option
  parameter BM = 0; //bit mask width
  parameter PM = 1; //pipe line mode,
  // 0->non-pipelined
  // 1->pipelined data with pipelined PipeStall
  // 2->pipelined data with unpipelined PipeStall
  parameter ED = 0; // error detect # of bits to store FED

  input                        Reset;
  input                        RdClk, RdEn, WrClk, WrEn;
  input [NA-1:0]               RdAdr, WrAdr;
  input [NB-1:0]               WrDat;
  input [((0==BM)?0:(BM-1)):0] WrEnMask;
  input                        PipeStall;
  input                        MemPowerCtrl;
  input [((0==ED)?0:(ED-1)):0] Fed;
  input [((0==ED)?0:(ED-1)):0] FedWrEnMask;

  output [NB-1:0]               RdDat;
  output [((0==ED)?0:(ED-1)):0] Ded;

  reg [NB-1:0]                 Mem [NW-1:0];
  reg [ED-1:0]                 ErrorBits [NW-1:0];
  
  // This is a hack until all the wrappers can hook up MemPowerCtrl port
  wire _MemPowerCtrl = (MemPowerCtrl===1'bz) ? 1'b0 : MemPowerCtrl;
 

  always @(posedge WrClk) begin : writeBlk
    if(WrEn && !_MemPowerCtrl) begin
      if(BM == 0)
        Mem[WrAdr] <= WrDat;
      else begin : a0
        Mem[WrAdr] <= (WrDat & WrEnMask) | (Mem[WrAdr] & ~WrEnMask);
      end
      if(ED == 0)
        ErrorBits[WrAdr] <= 1'b0;
      else
        ErrorBits[WrAdr] <= (Fed & FedWrEnMask) | (ErrorBits[WrAdr] & ~FedWrEnMask);
    end
  end

  reg [NB-1:0] RdDatQ; //This is equivalent to the sense amp output
  reg [NB-1:0] RdDatQQ;
  reg          PipeStallQ;
  reg [ED-1:0] DedQ;
  reg [ED-1:0] DedQQ;

  always @(posedge RdClk) begin
    if(!_MemPowerCtrl) begin
      PipeStallQ <= PipeStall;
      if(RdEn) begin
        RdDatQ <= Mem[RdAdr];
        DedQ   <= ErrorBits[RdAdr];
      end
    end
    if(((PM==1)&&PipeStallQ) || ((PM==2)&&PipeStall)) begin
      RdDatQQ <= RdDatQ;
      DedQQ <= DedQ;
    end
  end

  assign RdDat = PM ? RdDatQQ : RdDatQ;
  assign Ded   = PM ? DedQQ   : DedQ;

endmodule

