//---------------------------------------------------------------------------
//  TCC Disable Module
//  
//  Output is a mask of active TCCs.
//
//  Currently this module supports 16 TCCs.
//---------------------------------------------------------------------------

`include "gc__spi_reg.v"

module amd_tcc_disable (
    // outputs
    tcc_active_mask,
    reg_rd_valid,
    reg_rd_data,

    // inputs
    reg_wr,
    reg_wr_addr,
    reg_wr_data,
    reg_rd,
    reg_rd_addr,
    clk,
    hard_resetB
);

//----------------------------------------------
output [15:0] tcc_active_mask;  // bit mask per tcc. 1 = active, 0 = inactive
output        reg_rd_valid;     // register read address matches
output [31:0] reg_rd_data;      // register read data.  set to zero if no reg-addr match.

input         reg_wr;       // register write enable
input  [15:0] reg_wr_addr;  // register write addr [17:2]
input  [31:0] reg_wr_data;  // register write data
input         reg_rd;       // register read enable:  only RLC/SPI_M should use this, others tie to zero.
input  [15:0] reg_rd_addr;  // register read addr [17:2]
input         clk;
input         hard_resetB;   // active low reset -- this must be hard-reset, not soft reset

//----------------------------------------------

reg         write_dis;
reg  [15:0] inactive_tcc;
reg  [15:0] inactive_tcc_user;

reg  [15:0] tcc_active_mask;
reg         reg_rd_valid;
reg  [31:0] reg_rd_data;  
reg         reg_wr_d1;

// Register-write 
always @(posedge clk) begin
    if (!hard_resetB) begin
        write_dis          <= `CGTS_TCC_DISABLE_WRITE_DIS_DEFAULT;
        inactive_tcc       <= `CGTS_TCC_DISABLE_TCC_DISABLE_DEFAULT;
        inactive_tcc_user  <= `CGTS_USER_TCC_DISABLE_TCC_DISABLE_DEFAULT;
    end
    else if (reg_wr && !write_dis && (reg_wr_addr == `CGTS_TCC_DISABLE_ABSD_ADDR)) begin
        // this register can only be written once
        write_dis    <= reg_wr_data[`CGTS_TCC_DISABLE_WRITE_DIS];
        inactive_tcc <= reg_wr_data[`CGTS_TCC_DISABLE_TCC_DISABLE];
    end
    else if (reg_wr && (reg_wr_addr == `CGTS_USER_TCC_DISABLE_ABSD_ADDR)) begin
        inactive_tcc_user <= reg_wr_data[`CGTS_USER_TCC_DISABLE_TCC_DISABLE];
    end

    // Find the number of ACTIVE TCC (not inactive ones)
    reg_wr_d1 <= reg_wr;
    if (!hard_resetB || reg_wr_d1)
      tcc_active_mask <= ~(inactive_tcc | inactive_tcc_user);
end

// Register-read 
always @(*) begin
    reg_rd_valid = 1'b0;
    reg_rd_data  = 32'h0;

    if (reg_rd && (reg_rd_addr == `CGTS_TCC_DISABLE_ABSD_ADDR)) begin
        reg_rd_valid = 1'b1;
        reg_rd_data[`CGTS_TCC_DISABLE_WRITE_DIS]   = write_dis;
        reg_rd_data[`CGTS_TCC_DISABLE_TCC_DISABLE] = inactive_tcc;
    end
    else if (reg_rd && (reg_rd_addr == `CGTS_USER_TCC_DISABLE_ABSD_ADDR)) begin
        reg_rd_valid = 1'b1;
        reg_rd_data[`CGTS_USER_TCC_DISABLE_TCC_DISABLE] = inactive_tcc_user;
    end
end

`ifdef SIMULATION
//-------------------------------------------------------------------------------
// Write Disable Check
//-------------------------------------------------------------------------------
// VCS coverage off
localparam disable_psl_check = "disable_psl_check=";
localparam disable_psl_write_disable_check = "disable_psl_write_disable_check=";

reg check_all_psl;
reg check_write_disable_psl;

initial
begin
  check_all_psl = 1;
  check_write_disable_psl = 1;

  if ($test$plusargs({disable_psl_check,"on"})) check_all_psl = 0;
  if ($test$plusargs({disable_psl_write_disable_check,"on"})) check_write_disable_psl = 0;
end

// psl property write_disable_check = always ((hard_resetB && check_all_psl && check_write_disable_psl && reg_wr && (reg_wr_addr == `CGTS_TCC_DISABLE_ABSD_ADDR))->(~write_dis)) @(posedge clk);
// psl assert write_disable_check report "ERROR: Trying to write CGTS_TCC_DISABLE while write disable is set!";   
// VCS coverage on
`endif // SIMULATION
endmodule
