/*
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 *  NOTE: This *must* be instanced in the always-on power domain and attached to the static clk
 *
 *  Common module for DSM (Debug State Machine) irritator bus
 * 
 *  Each block has some number (M) of block irritators that can be driven by any of the irritator 
 *  input bits.
 * 
 *  Associated with each block, there is a N-bit (N=2*M) control register that controls whether a 
 *  given block irritator is enabled.
 * 
 *  Block irritators are used to affect functionality to aid in validation and debug.  Example uses 
 *  include, but are not limited to, fifo back-pressure, forcing EDC errors, and wavefront stalling.
 * 
 *  Circuit description of irritator functionality:
 * 
 *                                      +------+
 *  BLK_TARG_irritator_data[1:0]     [0]|      |
 *  ------------------------------+-----|      |
 *                                |     | and2 |--+  
 *  blk_dsm_cntl[N-1:0]           |  [0]|      |  |  +-----+
 *  -------------------------+----)-----|      |  |  |     |
 *                           |    |     +------+  +--|     |      blk_irritator[0]
 *                           |    |                  | or2 |----------------------
 *                           |    |     +------+     |     |
 *                           |    |  [1]|      |  +--|     |
 *                           |    +-----|      |  |  +-----+
 *                           |    |     | and2 |--+  
 *                           |    |  [1]|      |
 *                           +----)-----|      |
 *                           |    |     +------+
 *                           |    |
 *                           |    |     +------+
 *                           |    |  [0]|      |
 *                           |    +-----|      |
 *                           |    |     | and2 |--+  
 *                           |    |  [2]|      |  |  +-----+
 *                           +----)-----|      |  |  |     |
 *                           |    |     +------+  +--|     |      blk_irritator[1]
 *                           |    |                  | or2 |----------------------
 *                           |    |     +------+     |     |
 *                           |    |  [1]|      |  +--|     |
 *                           |    +-----|      |  |  +-----+
 *                           |    |     | and2 |--+  
 *                           |    |  [3]|      |
 *                           +----)-----|      |
 *                           |    |     +------+
 *                           .    .
 *                           .    .
 *                           .    .
 *                           |    |
 *                           |    |     +------+
 *                           |    |  [0]|      |
 *                           |    +-----|      |
 *                           |    |     | and2 |--+  
 *                           |    |[N-2]|      |  |  +-----+
 *                           +----)-----|      |  |  |     |
 *                           |    |     +------+  +--|     |    blk_irritator[M-1]
 *                           |    |                  | or2 |----------------------
 *                           |    |     +------+     |     |
 *                           |    |  [1]|      |  +--|     |
 *                           |    +-----|      |  |  +-----+
 *                           |          | and2 |--+  
 *                           |     [N-1]|      |
 *                           +----------|      |
 *                                      +------+
 * 
 */

// Include gc_defs.vh to get DSM_TARG_IRRITATOR_DATA defines
`include "gc_defs.vh"

module ati_dsm_irritator (
    clk,
    DSM_BLK_irritator_data,
    blk_dsm_cntl,
    BLK_TARG_irritator_data,
    blk_irritator
);

   parameter                               BLK_IRRITATORS = 1;
   parameter                               BLK_DSM_CNTL_WIDTH = BLK_IRRITATORS * `DSM_TARG_IRRITATOR_DATA_WIDTH;
   
   input                                   clk;
   input [`DSM_TARG_IRRITATOR_DATA_RANGE]  DSM_BLK_irritator_data;  // Top-level irritator chain input
   input [BLK_DSM_CNTL_WIDTH-1:0]          blk_dsm_cntl;            // Block-specific DSM irritator control from register bus (e.g. GRBM)

   output [`DSM_TARG_IRRITATOR_DATA_RANGE] BLK_TARG_irritator_data; // Top-level irritator chain output
   output [BLK_IRRITATORS-1:0]             blk_irritator;           // Local block irritator signal, enabled when corresponding control bit set

   reg [`DSM_TARG_IRRITATOR_DATA_RANGE]    BLK_TARG_irritator_data;
   reg [BLK_IRRITATORS-1:0]                blk_irritator;
   integer                                 i;
   integer                                 j;

   always @ (posedge clk) begin
      BLK_TARG_irritator_data <= DSM_BLK_irritator_data;
   end

   always @ * begin
      for (i = 0; i < BLK_IRRITATORS; i=i+1) begin
         blk_irritator[i] = 1'b0;
         for (j = 0; j < `DSM_TARG_IRRITATOR_DATA_WIDTH; j=j+1) begin
            blk_irritator[i] = blk_irritator[i] | (blk_dsm_cntl[i*`DSM_TARG_IRRITATOR_DATA_WIDTH+j] & BLK_TARG_irritator_data[j]);
         end
      end
   end
   
endmodule // ati_dsm_irritator
