//--------------------------------------------------------------------------
//
// $Id: //gfxip/gfx9/main/import/rtllib/src/rtl/ati_fifo_ctrl.v#2 $
//
// $Change: 1244568 $
//
//
// Notes:    This file is the control logic for a fifo with a registered
//           output. The output register is not included in this module and
//           must be provided externally. The enable for this output register
//           driven by this module. This control module assumes that the memory
//           element of the FIFO behaves like a virage memory (registered
//           read address, write address, and write data). The memory element
//           is also NOT provided in this module, but is driven directly
//           from registered signals produced by this module.
//           Writes to this module when the FULL signal is true will
//           cause this module to "go into the weeds".
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

module ati_fifo_ctrl (
    // -------------------------------------------
    // Chip Signals
    // -------------------------------------------
    CLK,
    RST,      // reset for this module is synchronous

    // signals at the FIFO level
    WE,
    RE,
    FULL,
    EMPTY,     // indicates that the output data register
               // does NOT have valid data
    BUSY,      // there is data in this fifo

    // signals to the MEM
    MEM_WADDR,
    MEM_WE,
    MEM_RADDR,
    MEM_RE,

    // signal to the external output register
    OUTREG_EN
);

    // parameterizations
    parameter ADDR_WIDTH = 0;        
    parameter DEPTH = 0;
    parameter SKID_WORDS = 0;
    parameter FIFO_NAME  = "";    


    parameter STOP_ON_ERROR = 1;
`ifdef SIMULATION
    parameter CYCLES_BEFORE_OVERFLOW_CHECK = 40;
    integer    cycle_count;
`endif

    // ***********************************
    // I/O Definitions
    // ***********************************
    // Chip Signals
    input                   CLK;
    input                   RST;

    // signals at the FIFO level
    input                   WE;
    input                   RE;
    output                  FULL;
    output                  EMPTY;
    output                  BUSY;

    // signals to the mem
    output [ADDR_WIDTH-1:0] MEM_WADDR;
    output                  MEM_WE;
    output [ADDR_WIDTH-1:0] MEM_RADDR;
    output                  MEM_RE;

    // signal to the external output register
    output                  OUTREG_EN;

    // signals used in this module
    // The address width of the FIFO_CNT_q signal will be 1 bit too wide
    // if the depth is not a power of two; however, this case works for all values.
    wire                    FIFO_WE;    // internal we, clocks write ptr
    wire                    FIFO_RE;    // internal re, clocks read ptr
    reg   [ADDR_WIDTH-1:0]  FIFO_WPTR_q, FIFO_RPTR_q;
    reg   [ADDR_WIDTH:0]    FIFO_CNT_q;

    reg                     FIFO_FULL_q;
    reg                     INT_FIFO_EMPTY_q;   // empty for the internal fifo
    reg                     EXT_EMPTY_q;   // output of rdata register is not valid
    reg                     MEM_EMPTY_q;        // output of mem is not valid

    wire                    READ_MEM;
    wire                    READ_FIFO;

    wire [ADDR_WIDTH:0]     nxt_FIFO_CNT_dec; 
    wire [ADDR_WIDTH:0]     nxt_FIFO_CNT_inc;

    wire [ADDR_WIDTH-1:0]     nxt_FIFO_WPTR_inc; 
    wire [ADDR_WIDTH-1:0]     nxt_FIFO_RPTR_inc; 
    // -------------------------------------------
    // concurrent assignments
    // -------------------------------------------
    assign FIFO_WE = WE;
    assign READ_MEM = RE | EXT_EMPTY_q;
    assign READ_FIFO = READ_MEM | MEM_EMPTY_q;
    assign FIFO_RE = READ_FIFO & ~INT_FIFO_EMPTY_q;
    assign OUTREG_EN = READ_MEM;
    assign EMPTY = EXT_EMPTY_q;
    assign FULL = FIFO_FULL_q;
    assign MEM_WADDR = FIFO_WPTR_q;
    assign MEM_RADDR = FIFO_RPTR_q;    
    assign MEM_RE = FIFO_RE;
    assign MEM_WE = WE;
    assign BUSY = ~(INT_FIFO_EMPTY_q & MEM_EMPTY_q & EXT_EMPTY_q);

    assign nxt_FIFO_CNT_dec  = FIFO_CNT_q - 1; 
    assign nxt_FIFO_CNT_inc  = FIFO_CNT_q + 1; 
    assign nxt_FIFO_WPTR_inc = FIFO_WPTR_q + 1; 
    assign nxt_FIFO_RPTR_inc = FIFO_RPTR_q + 1; 
    // -------------------------------------------
    // clock process
    // -------------------------------------------
    always @(posedge CLK) begin

        // -------------------------------------------
        // fifo registers
        // -------------------------------------------
        // clock the immediate data fifo registers on the write side
        if ( FIFO_WE == 1 ) begin
            if ( FIFO_WPTR_q == ( DEPTH - 1 ) ) begin
                FIFO_WPTR_q <= 0;
            end else begin
                FIFO_WPTR_q <= nxt_FIFO_WPTR_inc;
            end
        end

        // clock the immediate data fifo registers on the read side
        if ( FIFO_RE == 1 ) begin
            if ( FIFO_RPTR_q == ( DEPTH - 1 ) ) begin
                FIFO_RPTR_q <= 0;
            end else begin
                FIFO_RPTR_q <= nxt_FIFO_RPTR_inc;
            end
        end

        // ************  immediate data fifo empty/full logic  **************************
        // write, no read
        if (( FIFO_CNT_q >= DEPTH - SKID_WORDS - 1 ) && (FIFO_WE) && (!FIFO_RE ))  begin
           FIFO_FULL_q <= 1;
        end
        if ((FIFO_WE) && (!FIFO_RE)) begin 
           FIFO_CNT_q <= nxt_FIFO_CNT_inc;
           INT_FIFO_EMPTY_q <= 0;
        end 
        // write, no read

        // read, no write
        if (( FIFO_CNT_q == 1 ) && (!FIFO_WE) && (FIFO_RE)) begin
           INT_FIFO_EMPTY_q <= 1;
        end
        if ((FIFO_CNT_q == DEPTH - SKID_WORDS) && (!FIFO_WE) && (FIFO_RE)) begin
          FIFO_FULL_q <= 0;
        end
        if ((!FIFO_WE) && (FIFO_RE)) begin
           FIFO_CNT_q <= nxt_FIFO_CNT_dec;
        end
        // read, no write
        // ************  immediate data fifo empty/full logic  **************************

        if ( READ_FIFO ) begin
            MEM_EMPTY_q <= INT_FIFO_EMPTY_q;
        end
        
        if ( READ_MEM ) begin
            EXT_EMPTY_q <= MEM_EMPTY_q;
        end
        
        // -------------------------------------------
        // reset-able registers
        // -------------------------------------------
        // put this last in the clock process to override
        // anything that was assigned above
        if ( RST == 1 ) begin
            // immediate data fifo registers
            FIFO_WPTR_q <= 0;
            FIFO_RPTR_q <= 0;
            FIFO_CNT_q <= 0;
            FIFO_FULL_q <= 0;
            INT_FIFO_EMPTY_q <= 1;
            MEM_EMPTY_q <= 1;
            EXT_EMPTY_q <= 1;
        end

`ifdef SIMULATION

        // error checking
        if ( (cycle_count > CYCLES_BEFORE_OVERFLOW_CHECK) && (RST == 0) && FIFO_WE && (FIFO_CNT_q == DEPTH ) ) begin
// VCS coverage off        
            `MSGIO_DISPLAY("ATI_Fifo ERROR (%m): %s fifo written when full -- data dropped at time %t", FIFO_NAME, $time);
           if (STOP_ON_ERROR) begin
`ifdef MT
              $stop(0);
`else
              $finish(0);
`endif
// VCS coverage on        
           end
        end
`endif

    end

`ifdef SIMULATION
  initial begin
    cycle_count <= 0;
    forever begin
      @(posedge CLK);
      if (cycle_count != 32'hffffffff) begin
	cycle_count <= cycle_count + 1;
      end
    end
  end
`endif

endmodule // ati_fifo_ctrl
