//--------------------------------------------------------------------------
//
// $Id: $
//
// $Change: $
//
// Notes:     This is a 1-cycle read latency version of ati_fifo_ctrl_pipe_flopped
//            Supports FIFO depth up to and including 4095
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2005, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
// Author: Tien Wei (tien.wei@amd.com)
//
//--------------------------------------------------------------------------
module ati_fifo_ctrl_pipe_flopped_fast (/*AUTOARG*/
   // Outputs
   mem_we, mem_waddr, mem_re, mem_raddr, read_data, full, empty, busy,
   // Inputs
   clk, reset, we, write_data, re, mem_data
   );

   // Parameters
   parameter FIFO_WIDTH = 1;
   parameter FIFO_DEPTH = 2;
   parameter SKID_WORDS = 0;
   parameter SKID_FULL  = 0;
   parameter FIFO_NAME  = "";
   parameter ADDR_WIDTH = ((FIFO_DEPTH <= 4) ? 2 :
                           (FIFO_DEPTH <= 8) ? 3 :
                           (FIFO_DEPTH <= 16) ? 4 :
                           (FIFO_DEPTH <= 32) ? 5 :
                           (FIFO_DEPTH <= 64) ? 6 :
                           (FIFO_DEPTH <= 128) ? 7 :
                           (FIFO_DEPTH <= 256) ? 8 :
                           (FIFO_DEPTH <= 512) ? 9 : 
			   (FIFO_DEPTH <= 1024) ? 10 :
			   (FIFO_DEPTH <= 2048) ? 11 : 12);  // Bigger than 4095???  i hope not.
   // Equal to ADDR_WIDTH if NON-POWER-OF-2, +1 otherwise
   parameter COUNT_WIDTH = ((FIFO_DEPTH < 4) ? 2 :
                            (FIFO_DEPTH < 8) ? 3 :
                            (FIFO_DEPTH < 16) ? 4 :
                            (FIFO_DEPTH < 32) ? 5 :
                            (FIFO_DEPTH < 64) ? 6 :
                            (FIFO_DEPTH < 128) ? 7 :
                            (FIFO_DEPTH < 256) ? 8 :
                            (FIFO_DEPTH < 512) ? 9 : 
			    (FIFO_DEPTH < 1024) ? 10 :
			    (FIFO_DEPTH < 2048) ? 11 : 12);  // Bigger than 4095???  i hope not.
   
   // Base
   input 		   clk;           // synchronous read/write interfaces, both on clk  
   input 		   reset;         // active hi reset

   // Write interface
   input 		   we;            // Logical Rrite enable
   output 		   mem_we;        // Memory Write Enable (same as we)
   output [ADDR_WIDTH-1:0] mem_waddr;     // Write Address for RAM
   input [FIFO_WIDTH-1:0]  write_data;    // Write data for cases that don't use Virage MEM

   // Read interface
   input 		   re;            // Logical Read Enable
   output 		   mem_re;        // Read Enable for RAM
   output [ADDR_WIDTH-1:0] mem_raddr;     // Read Address for RAM
   input [FIFO_WIDTH-1:0]  mem_data;      // Data from Virage MEM  
   output [FIFO_WIDTH-1:0] read_data;     // Flopped data from Virage RAM
   
   // Status
   output 		   full;          // FIFO Full signal
   output 		   empty;         // FIFO Empty signal
   output 		   busy;
			      
   reg [COUNT_WIDTH-1:0]   cnt_d;
   reg [COUNT_WIDTH-1:0]   cnt_q;
   reg 			   empty_d;
   reg 			   empty_q;
   reg 			   full_d;
   reg 			   full_q;
   reg                     mem_re_q1,
			   mem_re_q2;

   // The logical count and empty flags as viewed from the outside world,
   // not knowing anything about RAM latency and the flops used to
   // achieve a 0-cycle read latency
   wire 		   cnt_q_gt_skid_limit = (cnt_q >  (FIFO_DEPTH - SKID_WORDS - 1));
   wire 		   cnt_q_eq_skid_limit = (cnt_q == (FIFO_DEPTH - SKID_WORDS - 1));

   always @ (*) begin : fifo_status
      
      // Logic for FULL is based on the different SKID parameters
      // Toggle FULL once in SKID territory if an entry is read
      if (SKID_FULL) begin
	 full_d = (cnt_q_eq_skid_limit & ~re & we) |  // hit limit, writing, and not reading
		  (cnt_q_gt_skid_limit & ~re);        // over limit and not reading. 
	 
	 // Keep FULL asserted once FIFO has entered the SKID entries
      end else if (SKID_WORDS > 0) begin
	 full_d = (cnt_q_eq_skid_limit & ~re & we) |  // hit limit, writing, and not reading
		  cnt_q_gt_skid_limit;                // over limit
	 
	 // No SKID entries in FIFO
      end else begin
	 full_d = (cnt_q_eq_skid_limit & ~re & we) |  // hit limit, writing, and not reading
		  (full_q & ~we & ~re) |              // store value (no read, no write)
		  (full_q &  we &  re);               // store value (read, write)
      end
	 
      if (reset) begin
	 cnt_d = 0;
	 empty_d = 1'h1;
      end else begin
	 case ({we, re})
	   2'h0,2'h3: begin cnt_d = cnt_q;                 empty_d = empty_q;      end // both or none, do the same
	   2'h1:      begin cnt_d = cnt_q - 1;             empty_d = (cnt_q == 1); end // read, no write
	   2'h2:      begin cnt_d = cnt_q + 1;             empty_d = 0;	    end // write, no read
	   default:   begin cnt_d = {(COUNT_WIDTH){1'hx}}; empty_d = 1'hx;	    end
	 endcase // case ({we, re})
      end
   end
	 

   // read from flops when
   //  there's something in them
   //  a read is happening
   //  something from the RAM isn't about to use q2
   reg [1:0] 		   flop_cnt_d;
   reg [1:0] 		   flop_cnt_q;
   wire 		   flop_we    = (cnt_d <= 3) & we;
   wire 		   flop_re    = (|flop_cnt_q) & re & (~mem_re_q2);

   // if data is in the ram, there is still something in q2
   // only way nothing is in ram is if q_cnt=1 and we're reading -> the 1 entry must be in flops
   wire                    cnt_d_eq0  = ~(|cnt_d);
   always @ (*) begin : flop_status
      if (reset) begin
	 flop_cnt_d = 0;
      end else begin
	 case ({flop_we, flop_re})
	   2'h0,2'h3: begin flop_cnt_d = flop_cnt_q;        end // both or none, do the same
	   2'h1:      begin flop_cnt_d = flop_cnt_q - 2'h1; end // read, no write
	   2'h2:      begin flop_cnt_d = flop_cnt_q + 2'h1; end // write, no read
	   default:   begin flop_cnt_d = 2'hx;              end
	 endcase // case ({we, re})
      end
	 
   end


   //-------------------------------------------------------------------------------------------------------------------------------
   // 4 Modes of Operation
   //  - RAM empty
   //  - RAM !empty, FLOPS draining
   //  - RAM steady state
   //  - RAM !empty, FLOPS loading
   //  

   //--------------------------------------------------------------------------*----------------------------------------------------
   // LOGIC DESCRIPTION
   //   L means same as logical value
   //   count   logic   ram action   q0 action   q1 action  q2 action
   //   q  d    we re   we re src    we re src   we re src  we re src
   //   0 +1     1  x    0  L  -      0  x  q0    0  x q1    L  x in   don't write ram,      out=in,               inc  cnt

   //   1 +1     1  0    0  L  -      0  x  q0    1  x in    0  x q2   don't write ram,       q1=in, keep q2,      inc  cnt
   //   1 +0     1  1    0  0  -      0  x  q0    0  x q1    1  x in   don't read/write ram, out=in,               keep cnt
   //   1 -1     0  1    L  0  -      0  x  q0    0  x q1    0  x q2   don't read ram,       out=in,               dec  cnt

   //   2 +1     1  0    0  L  -      1  x  in    0  x q1    0  x q2   don't write ram,       q0=in, keep q1,q2    inc  cnt
   //   2 +0     1  1    0  0  -      0  x  q0    1  x in    1  x q1   don't read/write ram,  q1=in, q2=q1         keep cnt
   //   2 -1     0  1    L  0  -      0  x  q0    0  x q1    1  x q1   don't read ram,        q2=q1,               dec  cnt

   //   3 +0     1  1    0  0  -      1  x  in    1  x q0    1  x q1   don't read/write ram,  q0=in,q1=q0,q2=q1    keep cnt
   //   3 -1     0  1    L  0  -      0  x  q0    1  x q0    1  x q1   don't read ram,              q1=q0,q2=q1    dec  cnt

   // >=3 +1     1  0    1  L  -      0  x  q0    0  x q1    0  x q2   write ram              keep q0,q1,q2        inc  cnt

   // >=4 +0     1  1    L  L  -      1  x ram    1  x q0    1  x q1   write/read ram,        q0=ram,q1=q0,q2=q1   keep cnt
   // >=4 -1     0  1    L  L  -      1  x ram    1  x q0    1  x q1   read ram,              q0=ram,q1=q0,q2=q1   dec cnt

   // Note that once the RAM is in use, we need to drain out the overflow, so that q2 gets q2->q1->q0->ram and stays on ram until
   // entire FIFO is empty
   
   //---------------------------------------------------------------------------
   // MEMORY INTERFACE
   // No need for pre-read
   //---------------------------------------------------------------------------

   // RAM control, fed to ati_fifo_ctrl_pipe inputs
   wire 		   mem_we_d  = (cnt_d > 3) & we;  // >=3, we=1
   wire 		   mem_re_d  = (cnt_q > 3) & re;  // >=4, re=1

   // q2 control (determing when q2_data is needed by data from RAM
   wire 		   mem_re_d1 = reset ? 1'h0 : (re ? mem_re_d  : mem_re_q1);
   wire 		   mem_re_d2 = reset ? 1'h0 : (re ? mem_re_q1 : mem_re_q2);

   reg [ADDR_WIDTH-1:0]    d_mem_waddr;
   reg [ADDR_WIDTH-1:0]    q_mem_waddr;
   reg [ADDR_WIDTH-1:0]    d_mem_raddr;
   reg [ADDR_WIDTH-1:0]    q_mem_raddr;

   wire 		   waddr_max = (q_mem_waddr == (FIFO_DEPTH-1));
   wire 		   raddr_max = (q_mem_raddr == (FIFO_DEPTH-1));

   always @ (*) begin
      if      (reset | (mem_we_d & waddr_max)) begin d_mem_waddr = 0;               end
      else if (mem_we_d)                       begin d_mem_waddr = q_mem_waddr + 1; end
      else                                     begin d_mem_waddr = q_mem_waddr;     end

      if      (reset | (mem_re_d & raddr_max)) begin d_mem_raddr = 0;               end
      else if (mem_re_d)                       begin d_mem_raddr = q_mem_raddr + 1; end
      else                                     begin d_mem_raddr = q_mem_raddr;     end
   end // always @ begin


   //---------------------------------------------------------------------------
   // SKID STAGE
   //---------------------------------------------------------------------------
   
   // Unlike mem_re_*, skid_re_d1 does not need to be held by re
   //  mem_re  is held since re affects when the RAM data needs data_q2
   //  skid_re doesn't need to be held since we want to load the skid as soon
   //  as mem_re is de-asserted (pulse/transition detect)
   // Stall term is ONLY for power. if both d,q are same, no need to update
   reg                     skid_re_q1;
   wire                    skid_re_d1        = reset ? 1'h0 : ((mem_re_d ^ skid_re_q1) ? mem_re_d : skid_re_q1);

   reg 			   skid_data_valid_q;
   wire 		   skid_data_valid_d = (skid_re_q1 | skid_data_valid_q) & ~re; // (set | hold) & ~clr;

   wire                    ld_skid_data_d    = skid_re_q1 & ~mem_re_d;
   reg [FIFO_WIDTH-1:0]    skid_data_q;
   wire [FIFO_WIDTH-1:0]   skid_data_d       = ld_skid_data_d ? mem_data : skid_data_q;

   wire [FIFO_WIDTH-1:0]   mem_data_synced   = skid_data_valid_q ? skid_data_q : mem_data;



   //---------------------------------------------------------------------------
   // FLOP STAGES
   //---------------------------------------------------------------------------
   
   // q0 flop stage
   reg [FIFO_WIDTH-1:0]    data_q0;
   wire [FIFO_WIDTH-1:0]   data_d0 = (({flop_cnt_d, flop_we} == 3'h7) ? write_data : // flop count to be 3, write input here
				      data_q0);                                      // hold all other cases
   
   // q1 flop stage
   reg [FIFO_WIDTH-1:0]    data_q1;
   wire [FIFO_WIDTH-1:0]   data_d1 = (({flop_cnt_q, flop_re} == 3'h7) ? data_q0    : // flop count 3 and reading, read q0, writes go elsewhere
				      ({flop_cnt_d, flop_we} == 3'h5) ? write_data : // flop count to be 2, write input here
				      data_q1);                                      // hold all other cases

   // q2/output flop stage
   // Because the read behaves like a 2-stage stallable pipe, easier to
   // just create a stalled mem_re_d delayed by 2 valid cycles
   // and use that to select RAM output, vs. trying to figure it out from
   // the counts and what-not
   reg [FIFO_WIDTH-1:0]    data_q2;
   wire [FIFO_WIDTH-1:0]   data_d2 = ((       mem_re_q2 &      re) ? mem_data_synced : // RAM read output has priority
				      (  flop_cnt_q[1]  & flop_re) ? data_q1         : // >1 flop count, read q1, writes go elsewhere
				      ((~flop_cnt_d[1]) & flop_we) ? write_data      : // writing and there will only be 1 entry
				      data_q2);

   
   // Cannot use ati_fifo_flopper because the low-latency path contains more
   //  sources to the output flop
   always @(posedge clk) begin

      // State for FAST/FLOP and OUTPUT STAGE
      data_q0           <= data_d0;
      data_q1           <= data_d1;
      data_q2           <= data_d2;

      // State for SKID
      skid_re_q1        <= skid_re_d1;
      skid_data_valid_q <= skid_data_valid_d;
      skid_data_q       <= skid_data_d;

      // State for FIFO control
      cnt_q             <= cnt_d;
      flop_cnt_q        <= flop_cnt_d;

      // RAM addresses
      q_mem_waddr       <= d_mem_waddr;
      q_mem_raddr       <= d_mem_raddr;

      // State for read data tracking
      mem_re_q1        <= mem_re_d1;
      mem_re_q2        <= mem_re_d2;

      // Status
      full_q            <= (~reset) & full_d;
      empty_q           <= empty_d;
   end

   assign                  read_data   = data_q2;
   assign                  mem_we      = mem_we_d;
   assign                  mem_re      = mem_re_d;
   assign                  mem_waddr   = q_mem_waddr;
   assign                  mem_raddr   = q_mem_raddr;
   assign                  full        = full_q;
   assign                  empty       = empty_q;
   assign                  busy        = ~empty_q;
endmodule // ati_fifo_ctrl_pipe_flopped_fast
// Local Variables:
// verilog-library-directories:("." "common" "../shared")
// verilog-library-extensions:(".v")
// End:
//-//ee [xxxxxxxxxxxxxxxx] name................ -------------------------------*----------------------------------------------------
