/*
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 *  Implementation for an IEEE round to nearest even (RNE) floating point add circuit
 * 
 */

module ati_fp_add (
   // Outputs
   result,

   // Inputs
   clk,
   en0,
   en1,
   allow_input_denorms,
   allow_output_denorms,
   src0,
   src1
   );

   parameter              LOG2_WORDS = 1; // 0 = FP16, 1 = FP32, 2 = FP64
   parameter              SRC_WIDTH = (LOG2_WORDS == 2) ? 64 : (LOG2_WORDS == 1) ? 32 : 16;
   parameter              EXP_WIDTH = (LOG2_WORDS == 2) ? 11 : (LOG2_WORDS == 1) ? 8  : 5;

   input                  clk;
   input                  en0;
   input                  en1;

   input                  allow_input_denorms;
   input                  allow_output_denorms;
   input [SRC_WIDTH-1:0]  src0;
   input [SRC_WIDTH-1:0]  src1;

   output [SRC_WIDTH-1:0] result;

   wire                   align_nan_inf;
   wire [1:0]             align_sgn;
   wire [EXP_WIDTH-1:0]   align_exp;
   wire [SRC_WIDTH-1:0]   align_man0;
   wire [SRC_WIDTH-1:0]   align_man1;

   ati_fp_add_align #(LOG2_WORDS) u_align (.allow_input_denorms(allow_input_denorms),
                                           .src0(src0),
                                           .src1(src1),
                                           .align_nan_inf(align_nan_inf),
                                           .align_sgn(align_sgn),
                                           .align_exp(align_exp),
                                           .align_man0(align_man0),
                                           .align_man1(align_man1)
                                           );

   reg                    q0_allow_output_denorms;
   reg                    q0_align_nan_inf;
   reg [1:0]              q0_align_sgn;
   reg [EXP_WIDTH-1:0]    q0_align_exp;
   reg [SRC_WIDTH-1:0]    q0_align_man0;
   reg [SRC_WIDTH-1:0]    q0_align_man1;

   always @ (posedge clk) begin
      if (en0) begin
         q0_allow_output_denorms <= allow_output_denorms;
         q0_align_nan_inf        <= align_nan_inf;
         q0_align_sgn            <= align_sgn;
         q0_align_exp            <= align_exp;
         q0_align_man0           <= align_man0;
         q0_align_man1           <= align_man1;
      end
   end

   wire [SRC_WIDTH-1:0]   q0_align_man = q0_align_man0 + q0_align_man1;

   reg                    q1_allow_output_denorms;
   reg                    q1_align_nan_inf;
   reg [1:0]              q1_align_sgn;
   reg [EXP_WIDTH-1:0]    q1_align_exp;
   reg [SRC_WIDTH-1:0]    q1_align_man;

   always @ (posedge clk) begin
      if (en1) begin
         q1_allow_output_denorms <= q0_allow_output_denorms;
         q1_align_nan_inf        <= q0_align_nan_inf;
         q1_align_sgn            <= q0_align_sgn;
         q1_align_exp            <= q0_align_exp;
         q1_align_man            <= q0_align_man;
      end
   end

   ati_fp_add_norm #(LOG2_WORDS) u_norm (.allow_output_denorms(q1_allow_output_denorms),
                                         .align_nan_inf(q1_align_nan_inf),
                                         .align_sgn(q1_align_sgn),
                                         .align_exp(q1_align_exp),
                                         .align_man(q1_align_man),
                                         .norm_result(result)
                                         );

endmodule // ati_fp_add
