/*
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */

module ati_fp_add_align (
   // Outputs
   align_nan_inf,
   align_sgn,
   align_exp,
   align_man0,
   align_man1,

   // Inputs
   allow_input_denorms,
   src0,
   src1
   );

   parameter                LOG2_WORDS    = 1; // 0 = FP16, 1 = FP32, 2 = FP64
   parameter                SRC_WIDTH     = (LOG2_WORDS == 2) ? 64  : (LOG2_WORDS == 1) ? 32 : 16;
   parameter                EXP_WIDTH     = (LOG2_WORDS == 2) ? 11  : (LOG2_WORDS == 1) ? 8  : 5;
   parameter                MAN_WIDTH     = (LOG2_WORDS == 2) ? 52  : (LOG2_WORDS == 1) ? 23 : 10;
   parameter                SHIFT_WIDTH   = (LOG2_WORDS == 2) ? 6   : (LOG2_WORDS == 1) ? 5  : 4;
   parameter                MAX_SHIFT     = (LOG2_WORDS == 2) ? 55  : (LOG2_WORDS == 1) ? 26 : 13;
   parameter                SHIFTED_WIDTH = (LOG2_WORDS == 2) ? 109 : (LOG2_WORDS == 1) ? 51 : 25;
   parameter                ALIGNED_WIDTH = (LOG2_WORDS == 2) ? 58  : (LOG2_WORDS == 1) ? 29 : 16;

   input                    allow_input_denorms;
   input [SRC_WIDTH-1:0]    src0;
   input [SRC_WIDTH-1:0]    src1;

   output                   align_nan_inf;
   output [1:0]             align_sgn;
   output [EXP_WIDTH-1:0]   align_exp;
   output [SRC_WIDTH-1:0]   align_man0;
   output [SRC_WIDTH-1:0]   align_man1;

   wire                     src0_sgn = src0[SRC_WIDTH-1];
   wire [EXP_WIDTH-1:0]     src0_exp = src0[SRC_WIDTH-2:MAN_WIDTH];
   wire [MAN_WIDTH-1:0]     src0_man = src0[MAN_WIDTH-1:0];
   wire                     src0_exp_zero = !(|src0_exp);
   wire [MAN_WIDTH+1:0]     src0_man_denorm = {1'b0, !src0_exp_zero, src0_man & {MAN_WIDTH{!src0_exp_zero || allow_input_denorms}}};
   wire [EXP_WIDTH-1:0]     src0_exp_denorm = (src0_exp_zero && |src0_man) ? 1 : src0_exp;
   wire                     src0_inf = &src0_exp && !(|src0_man);
   wire                     src0_nan = &src0_exp && (|src0_man);

   wire                     src1_sgn = src1[SRC_WIDTH-1];
   wire [EXP_WIDTH-1:0]     src1_exp = src1[SRC_WIDTH-2:MAN_WIDTH];
   wire [MAN_WIDTH-1:0]     src1_man = src1[MAN_WIDTH-1:0];
   wire                     src1_exp_zero = !(|src1_exp);
   wire [MAN_WIDTH+1:0]     src1_man_denorm = {1'b0, !src1_exp_zero, src1_man & {MAN_WIDTH{!src1_exp_zero || allow_input_denorms}}};
   wire [EXP_WIDTH-1:0]     src1_exp_denorm = (src1_exp_zero && |src1_man) ? 1 : src1_exp;
   wire                     src1_inf = &src1_exp && !(|src1_man);
   wire                     src1_nan = &src1_exp && (|src1_man);

   // align mantissas
   wire [EXP_WIDTH:0]       src0_shift = src1_exp_denorm - src0_exp_denorm;
   wire [EXP_WIDTH:0]       src1_shift = src0_exp_denorm - src1_exp_denorm;

   wire [SHIFT_WIDTH-1:0]   src0_shift_clamped = src0_shift[EXP_WIDTH] ? 0 : (src0_shift > MAX_SHIFT) ? MAX_SHIFT : src0_shift;
   wire [SHIFT_WIDTH-1:0]   src1_shift_clamped = src1_shift[EXP_WIDTH] ? 0 : (src1_shift > MAX_SHIFT) ? MAX_SHIFT : src1_shift;

   wire [MAN_WIDTH+1:0]     src0_man_2comp = src1_shift[EXP_WIDTH] && (src0_sgn != src1_sgn) ? -src0_man_denorm : src0_man_denorm;
   wire [MAN_WIDTH+1:0]     src1_man_2comp = !src1_shift[EXP_WIDTH] && (src0_sgn != src1_sgn) ? -src1_man_denorm : src1_man_denorm;

   wire signed [SHIFTED_WIDTH-1:0] src0_man_extended = {src0_man_2comp, {MAX_SHIFT{1'b0}}};
   wire signed [SHIFTED_WIDTH-1:0] src1_man_extended = {src1_man_2comp, {MAX_SHIFT{1'b0}}};

   wire [SHIFTED_WIDTH-1:0] src0_man_shifted = src0_man_extended >>> src0_shift_clamped;
   wire [SHIFTED_WIDTH-1:0] src1_man_shifted = src1_man_extended >>> src1_shift_clamped;

   wire [ALIGNED_WIDTH-1:0] src0_man_aligned = {src0_man_shifted[SHIFTED_WIDTH-1], src0_man_shifted[SHIFTED_WIDTH-1:MAN_WIDTH+1], |src0_man_shifted[MAN_WIDTH:0]};
   wire [ALIGNED_WIDTH-1:0] src1_man_aligned = {src1_man_shifted[SHIFTED_WIDTH-1], src1_man_shifted[SHIFTED_WIDTH-1:MAN_WIDTH+1], |src1_man_shifted[MAN_WIDTH:0]};

   assign                   align_nan_inf = (src0_nan || src1_nan || src0_inf || src1_inf);
   assign                   align_sgn = {src0_sgn != src1_sgn, src1_shift[EXP_WIDTH] ? src1_sgn : src0_sgn};
   assign                   align_exp = src1_shift[EXP_WIDTH] ? src1_exp_denorm : src0_exp_denorm;

   // pre-width is 1-bit larger than necessary to prevent a 0-length concatenation for fp16
   wire [SRC_WIDTH:0]       pre_align_man0 = src0_nan ? {1'b0, src0 | {1'b0, {EXP_WIDTH{1'b0}}, 1'b1, {MAN_WIDTH-1{1'b0}}}} :
                                             src1_nan ? {1'b0, src1 | {1'b0, {EXP_WIDTH{1'b0}}, 1'b1, {MAN_WIDTH-1{1'b0}}}} :
                                             (src0_inf && src1_inf && (src0_sgn != src1_sgn)) ? {2'b01, {EXP_WIDTH{1'b1}}, 1'b1, {MAN_WIDTH-1{1'b0}}} :
                                             src0_inf ? {1'b0, src0} :
                                             src1_inf ? {1'b0, src1} :
                                             {{SRC_WIDTH-ALIGNED_WIDTH+1{1'b0}}, src0_man_aligned};

   wire [SRC_WIDTH:0]       pre_align_man1 = align_nan_inf ? {SRC_WIDTH{1'b0}} :
                                             {{SRC_WIDTH-ALIGNED_WIDTH+1{1'b0}}, src1_man_aligned};

   assign                   align_man0 = pre_align_man0[SRC_WIDTH-1:0];
   assign                   align_man1 = pre_align_man1[SRC_WIDTH-1:0];

endmodule // ati_fp_add_align
