/*
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */

module ati_fp_add_norm (
   // Outputs
   norm_result,

   // Inputs
   allow_output_denorms,
   align_nan_inf,
   align_sgn,
   align_exp,
   align_man
   );

   parameter                LOG2_WORDS    = 1; // 0 = FP16, 1 = FP32, 2 = FP64
   parameter                SRC_WIDTH     = (LOG2_WORDS == 2) ? 64  : (LOG2_WORDS == 1) ? 32 : 16;
   parameter                EXP_WIDTH     = (LOG2_WORDS == 2) ? 11  : (LOG2_WORDS == 1) ? 8  : 5;
   parameter                MAN_WIDTH     = (LOG2_WORDS == 2) ? 52  : (LOG2_WORDS == 1) ? 23 : 10;
   parameter                SHIFT_WIDTH   = (LOG2_WORDS == 2) ? 6   : (LOG2_WORDS == 1) ? 5  : 4;
   parameter                MAX_SHIFT     = (LOG2_WORDS == 2) ? 55  : (LOG2_WORDS == 1) ? 26 : 13;
   parameter                SHIFTED_WIDTH = (LOG2_WORDS == 2) ? 109 : (LOG2_WORDS == 1) ? 51 : 25;
   parameter                ALIGNED_WIDTH = (LOG2_WORDS == 2) ? 58  : (LOG2_WORDS == 1) ? 29 : 16;

   input                    allow_output_denorms;

   input                    align_nan_inf;
   input [1:0]              align_sgn;
   input [EXP_WIDTH-1:0]    align_exp;
   input [SRC_WIDTH-1:0]    align_man;

   output [SRC_WIDTH-1:0]   norm_result;

   // 2's comp if negative
   wire [ALIGNED_WIDTH-1:0] result_man_signed = align_man[ALIGNED_WIDTH-1:0];
   wire [ALIGNED_WIDTH-2:0] result_man_denorm = result_man_signed[ALIGNED_WIDTH-1] ? -result_man_signed : result_man_signed;

   // leading-zero detect
   integer                  i;
   reg [SHIFT_WIDTH-1:0]    result_man_norm_shift;
   always @ * begin
      result_man_norm_shift = 0;
      for (i = 0; i < ALIGNED_WIDTH-1; i = i + 1) begin
         if (result_man_denorm[i])
            result_man_norm_shift = ALIGNED_WIDTH-2-i;
      end
   end

   // normalize
   wire [SHIFT_WIDTH-1:0]   result_man_norm_shift_clamped = (result_man_norm_shift > align_exp) ? align_exp : result_man_norm_shift;
   wire [ALIGNED_WIDTH-2:0] result_man_norm = result_man_denorm << result_man_norm_shift_clamped;

   // round
   wire [3:0]               result_man_round = (result_man_norm[3] && (result_man_norm[4] || |result_man_norm[2:0])) ? 4'd8 : 4'd0;
   wire [ALIGNED_WIDTH-1:0] result_man_rounded = result_man_norm + result_man_round;

   // result
   wire [EXP_WIDTH-1:0]     result_exp = align_exp - result_man_norm_shift_clamped + result_man_rounded[ALIGNED_WIDTH-1] + 1'b1;
   wire [MAN_WIDTH:0]       result_man = result_man_rounded[ALIGNED_WIDTH-1] ? result_man_rounded[ALIGNED_WIDTH-1:ALIGNED_WIDTH-1-MAN_WIDTH] : 
                                                                               result_man_rounded[ALIGNED_WIDTH-2:ALIGNED_WIDTH-2-MAN_WIDTH];

   // clamp
   wire                     result_sgn_clamped = !(|result_man_denorm) && align_sgn[1] ? 1'b0 : align_sgn[0] ^ result_man_signed[ALIGNED_WIDTH-1];
   wire [EXP_WIDTH-1:0]     result_exp_clamped = (result_exp == 1) && !result_man[MAN_WIDTH] || !(|result_man_denorm) ? 0 : result_exp;
   wire [MAN_WIDTH-1:0]     result_man_clamped = (&result_exp) || (!(|result_exp_clamped) && !allow_output_denorms) ? {MAN_WIDTH{1'b0}} : 
                                                                                                                      result_man[MAN_WIDTH-1:0];

   assign                   norm_result = align_nan_inf ? align_man : {result_sgn_clamped, result_exp_clamped, result_man_clamped};

endmodule // ati_fp_add_norm
