//---------------------------------------------------------------------------
//  RB Config registers
//  
//  Each register has two versions: one written by the RCU (write once)
//  and one written by the user (driver).  The user one can only
//  further disable rb-pipes, not re-enable ones disabled by the RCU.
//
//  Output is a mask of which RB pipes are enabled.
//
// Currently this module supports 8 RB-pipes.
//---------------------------------------------------------------------------

// include VGT regs from $ROOT/$BRANCH/cmn/inc/reg
`ifdef CC_RB_BACKEND_DISABLE_WRITE_DIS
`else
`include "gc__gb_reg.v"
`endif

`ifdef CC_RB_BACKEND_DISABLE
`else
`include "gc_register_addr.v"
`endif

module ati_rb_backend_cfg_reg (
    // outputs
    rb_active_pipe_mask,
    reg_rd_valid,
    reg_rd_data,

    // inputs
    reg_wr,
    reg_wr_addr,
    reg_wr_data,
    reg_rd,
    reg_rd_addr,
    reg_instance_select,
    clk,
    hard_resetB
);

parameter IS_SYSTEM_INSTANCE = 0;
// ctxclock clk

//----------------------------------------------
output  [7:0] rb_active_pipe_mask;  // bit mask per rb. 1 = active, 0 = inactive
output        reg_rd_valid;         // register read address matches
output [31:0] reg_rd_data;          // register read data.  set to zero if no reg-addr match.

input         reg_wr;       // register write enable
input  [15:0] reg_wr_addr;  // register write addr [17:2]
input  [31:0] reg_wr_data;  // register write data
input         reg_rd;       // register read enable:  only VGT should use this, others tie to zero.
input  [15:0] reg_rd_addr;  // register read addr [17:2]
input         reg_instance_select; // this se selected
input         clk;
input         hard_resetB;   // active low reset -- this must be hard-reset, not soft reset

//----------------------------------------------

// (abbreviated "rb_pipe_config" to "rbcfg", user version to "urbcfg")
reg        gc_rbcfg_write_dis;
reg  [7:0] gc_rbcfg_inactive_pipe;
reg  [7:0] gc_rbcfg_inactive_pipe_user;

reg  [7:0] rb_active_pipe_mask;
reg        reg_rd_valid;
reg [31:0] reg_rd_data;  

//----------------------------------------------
// Both disable registers are identical in every way, except one is in the system register domain and the other is in the graphics
// register domain.

wire [17:0] selected_disable_address = `CC_RB_BACKEND_DISABLE;
wire [17:0] selected_disable_address_user = `GC_USER_RB_BACKEND_DISABLE;

//----------------------------------------------

// Register-write 
always @(posedge clk) begin
    if (!hard_resetB) begin
        gc_rbcfg_write_dis          <= `CC_RB_BACKEND_DISABLE_WRITE_DIS_DEFAULT;
        gc_rbcfg_inactive_pipe      <= `CC_RB_BACKEND_DISABLE_BACKEND_DISABLE_DEFAULT;
        gc_rbcfg_inactive_pipe_user <= 8'h00;
    end
    else if (reg_wr && reg_instance_select && !gc_rbcfg_write_dis && {reg_wr_addr, 2'b00} == selected_disable_address) begin
        // this register can only be written once
        gc_rbcfg_write_dis      <= reg_wr_data[`CC_RB_BACKEND_DISABLE_WRITE_DIS];
        gc_rbcfg_inactive_pipe  <= reg_wr_data[`CC_RB_BACKEND_DISABLE_BACKEND_DISABLE];
    end
    else if (reg_wr && reg_instance_select && {reg_wr_addr, 2'b00} == selected_disable_address_user) begin
        // this register can only be written once
        gc_rbcfg_inactive_pipe_user  <= reg_wr_data[`CC_RB_BACKEND_DISABLE_BACKEND_DISABLE];
    end
    // Find the number of ACTIVE RB pipes (not inactive ones)
    rb_active_pipe_mask <= ~(gc_rbcfg_inactive_pipe | gc_rbcfg_inactive_pipe_user);
end

// Register-read 
always @(*) begin

    reg_rd_valid = 1'b0;
    reg_rd_data  = 32'h0;

    if (reg_rd && reg_instance_select && {reg_rd_addr, 2'b00} == selected_disable_address) begin
        reg_rd_valid = 1'b1;
        reg_rd_data[`CC_RB_BACKEND_DISABLE_WRITE_DIS]       = gc_rbcfg_write_dis;
        reg_rd_data[`CC_RB_BACKEND_DISABLE_BACKEND_DISABLE] = gc_rbcfg_inactive_pipe;
    end
    else if (reg_rd && reg_instance_select && {reg_rd_addr, 2'b00} == selected_disable_address_user) begin
        reg_rd_valid = 1'b1;
        reg_rd_data[`CC_RB_BACKEND_DISABLE_BACKEND_DISABLE] = gc_rbcfg_inactive_pipe_user;
    end
end
         
endmodule
