//---------------------------------------------------------------------------
//  RB Config registers
//  
//  Each register has two versions: one written by the RCU (write once)
//  and one written by the user (driver).  The user one can only
//  further disable rb-pipes, not re-enable ones disabled by the RCU.
//
//  Output is a mask of which RB pipes are enabled.
//
// Currently this module supports 8 RB-pipes.
//---------------------------------------------------------------------------

// include VGT regs from $ROOT/$BRANCH/cmn/inc/reg
`ifdef CC_RB_BACKEND_DISABLE_WRITE_DIS
`else
`include "gc__gb_reg.v"
`endif

`ifdef CC_RB_BACKEND_DISABLE
`else
`include "gc_register_addr.v"
`endif

module ati_rb_pipe_cfg_reg (
    // outputs
    rb_active_pipe_mask,
    reg_rd_valid,
    reg_rd_data,
    rb_redundancy_en,
    rb_redundancy_index,
    rb_daisy_chain,

    // inputs
    reg_wr,
    reg_wr_addr,
    reg_wr_data,
    reg_rd,
    reg_rd_addr,
    reg_instance_select,
    clk,
    hard_resetB
);

parameter IS_SYSTEM_INSTANCE = 0;
// ctxclock clk

//----------------------------------------------
output  [7:0] rb_active_pipe_mask;  // bit mask per rb. 1 = active, 0 = inactive
output        reg_rd_valid;         // register read address matches
output [31:0] reg_rd_data;          // register read data.  set to zero if no reg-addr match.
output  [2:0] rb_redundancy_index;  // points to defective rb 0-3
output        rb_redundancy_en;   // enable repair on RB/SXS
output [31:0] rb_daisy_chain;       // indicates the order of the rb daisy chain

input         reg_wr;       // register write enable
input  [15:0] reg_wr_addr;  // register write addr [17:2]
input  [31:0] reg_wr_data;  // register write data
input         reg_rd;       // register read enable:  only VGT should use this, others tie to zero.
input  [15:0] reg_rd_addr;  // register read addr [17:2]
input         reg_instance_select; // this se selected
input         clk;
input         hard_resetB;   // active low reset -- this must be hard-reset, not soft reset

//----------------------------------------------

// (abbreviated "rb_pipe_config" to "rbcfg", user version to "urbcfg")
reg        gc_rbcfg_write_dis;
reg  [7:0] gc_rbcfg_inactive_pipe;
reg  [7:0] gc_rbcfg_inactive_pipe_user;

reg  [7:0] rb_active_pipe_mask;
reg        reg_rd_valid;
reg [31:0] reg_rd_data;  

reg        rb_redundancy_write_dis;   
reg  [2:0] rb_redundancy_index;          
reg        rb_redundancy_en;
reg [31:0] rb_daisy_chain;
//----------------------------------------------
// Both disable registers are identical in every way, except one is in the system register domain and the other is in the graphics
// register domain.

wire [17:0] selected_disable_address = `CC_RB_BACKEND_DISABLE;
wire [17:0] selected_disable_address_user = `GC_USER_RB_BACKEND_DISABLE;

wire [17:0] selected_redundancy_address = `CC_RB_REDUNDANCY;

wire [17:0] selected_rb_daisy_chain_address = `CC_RB_DAISY_CHAIN;

//----------------------------------------------

// Register-write 
always @(posedge clk) begin
    if (!hard_resetB) begin
        gc_rbcfg_write_dis          <= `CC_RB_BACKEND_DISABLE_WRITE_DIS_DEFAULT;
        gc_rbcfg_inactive_pipe      <= `CC_RB_BACKEND_DISABLE_BACKEND_DISABLE_DEFAULT;
        gc_rbcfg_inactive_pipe_user <= 8'h00;
        rb_redundancy_write_dis	    <= `CC_RB_REDUNDANCY_WRITE_DIS_DEFAULT;
        rb_redundancy_index 	    <= `CC_RB_REDUNDANCY_FAILED_RB0_DEFAULT;
        rb_redundancy_en	        <= `CC_RB_REDUNDANCY_EN_REDUNDANCY0_DEFAULT;
        rb_daisy_chain              <= `CC_RB_DAISY_CHAIN_DEFAULT;
    end
    else if (reg_wr && reg_instance_select && !gc_rbcfg_write_dis && {reg_wr_addr, 2'b00} == selected_disable_address) begin
        // this register can only be written once
        gc_rbcfg_write_dis      <= reg_wr_data[`CC_RB_BACKEND_DISABLE_WRITE_DIS];
        gc_rbcfg_inactive_pipe  <= reg_wr_data[`CC_RB_BACKEND_DISABLE_BACKEND_DISABLE];
    end
    else if (reg_wr && reg_instance_select && {reg_wr_addr, 2'b00} == selected_disable_address_user) begin
        // this register can only be written once
        gc_rbcfg_inactive_pipe_user  <= reg_wr_data[`CC_RB_BACKEND_DISABLE_BACKEND_DISABLE];
    end
    else if (reg_wr && reg_instance_select && !rb_redundancy_write_dis && {reg_wr_addr, 2'b00} == selected_redundancy_address) begin
        rb_redundancy_write_dis <= reg_wr_data[`CC_RB_REDUNDANCY_WRITE_DIS];
        rb_redundancy_index 	<= reg_wr_data[`CC_RB_REDUNDANCY_FAILED_RB0];
        rb_redundancy_en    	<= reg_wr_data[`CC_RB_REDUNDANCY_EN_REDUNDANCY0];
    end
    else if (reg_wr && reg_instance_select && !rb_redundancy_write_dis && {reg_wr_addr, 2'b00} == selected_rb_daisy_chain_address) begin
        rb_daisy_chain          <= reg_wr_data;
    end

    // Find the number of ACTIVE RB pipes (not inactive ones)
    rb_active_pipe_mask <= ~(gc_rbcfg_inactive_pipe | gc_rbcfg_inactive_pipe_user);
end

// Register-read 
always @(*) begin

    reg_rd_valid = 1'b0;
    reg_rd_data  = 32'h0;

    if (reg_rd && reg_instance_select && {reg_rd_addr, 2'b00} == selected_disable_address) begin
        reg_rd_valid = 1'b1;
        reg_rd_data[`CC_RB_BACKEND_DISABLE_WRITE_DIS]       = gc_rbcfg_write_dis;
        reg_rd_data[`CC_RB_BACKEND_DISABLE_BACKEND_DISABLE] = gc_rbcfg_inactive_pipe;
    end
    else if (reg_rd && reg_instance_select && {reg_rd_addr, 2'b00} == selected_disable_address_user) begin
        reg_rd_valid = 1'b1;
        reg_rd_data[`CC_RB_BACKEND_DISABLE_BACKEND_DISABLE] = gc_rbcfg_inactive_pipe_user;
    end
    else if (reg_rd && reg_instance_select && {reg_rd_addr, 2'b00} == selected_redundancy_address) begin
        reg_rd_valid = 1'b1;
        reg_rd_data[`CC_RB_REDUNDANCY_WRITE_DIS]      = rb_redundancy_write_dis;
        reg_rd_data[`CC_RB_REDUNDANCY_FAILED_RB0]     = rb_redundancy_index;
        reg_rd_data[`CC_RB_REDUNDANCY_EN_REDUNDANCY0] = rb_redundancy_en;
    end
end

 `ifdef SIMULATION
//-------------------------------------------------------------------------------
// Write Disable Checks
//-------------------------------------------------------------------------------
// VCS coverage off
localparam disable_psl_check = "disable_psl_check=";
localparam disable_psl_write_disable_check = "disable_psl_write_disable_check=";

reg check_all_psl;
reg check_write_disable_psl;

initial
begin
  check_all_psl = 1;
  check_write_disable_psl = 1;

  if ($test$plusargs({disable_psl_check,"on"})) check_all_psl = 0;
  if ($test$plusargs({disable_psl_write_disable_check,"on"})) check_write_disable_psl = 0;
end

// psl property rb_backend_write_disable_check = always ((hard_resetB && check_all_psl && check_write_disable_psl && reg_wr && reg_instance_select && {reg_wr_addr, 2'b00} == selected_disable_address)->(~rb_redundancy_write_dis)) @(posedge clk);
// psl assert rb_backend_write_disable_check report "ERROR: Trying to write CC_{SYS}_RB_BACKEND_DISABLE while write disable is set!";

// psl property rb_redundancy_write_disable_check = always ((hard_resetB && check_all_psl && check_write_disable_psl && reg_wr && reg_instance_select && {reg_wr_addr, 2'b00} == selected_redundancy_address)->(~rb_redundancy_write_dis)) @(posedge clk);
// psl assert rb_redundancy_write_disable_check report "ERROR: Trying to write CC_{SYS}_RB_REDUNDANCY while write disable is set!";
// VCS coverage on
`endif // SIMULATION
endmodule
