//--------------------------------------------------------------------------
//
// $Id: //depot/xenos/devel/parts_lib/src/common/ati_shift_fifo.v#3 $
//
// $Change: 133265 $
//
// Notes:     An alternate flop based skid fifo where the outputs are always
//            from a flop, is a bit smaller, doesnt have routing congenstion, 
//            and the latency is only 1.
//            It can be used directly on a top level block output.
//            If REGISTER_WRITE_INPUTS is set, it registers the inputs so
//            it can be also used directly on the input of a top level block.
//            It doesn't make it bigger, but will add one cycle of latency.
//
//            The only drawback should be power.
//
//            Be careful:  The arguments are not the same as other fifos.
//            I removed ADDR_WIDTH because I needed one more bit on even
//            powers of two, and I don't want to leave it up to user error.
//
//            TO DO? While interfaces are flopped, its not using ATI_DFF.
//            does it really matter?   is just ugly, thats all.
//
//            fghodrat -- increased fifo depth to 2048
//            fghodrat -- added call to ati_basic_fifo_cover_assert task
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

module ati_shift_fifo (/*AUTOARG*/
   // Outputs
   busy, full, read_data, empty, 
   // Inputs
   clk, reset, we, write_data, re
   );
   
   //============================
   // PARAMETERS
   //============================
   parameter FIFO_WIDTH = 32;
   parameter FIFO_DEPTH = 16;
   parameter FIFO_NAME  = "";
   parameter SKID_WORDS = 0;
   parameter REGISTER_WRITE_INPUTS = 1'b0;    // Set this to be able to use this fifo directly on the input of a top level block.
   parameter REGISTER_READ_ENABLE  = 1'b0;    // Set this to flop the RE, but adds mux on read data
   parameter EXTERNAL_EMPTY_REGISTER = 1'b0;  // Flop !empty&&re_d externally to make a send with the data to another top level block.
   parameter EXTERNAL_FULL_REGISTER = 1'b0;   // Not sure why you'd flop outside since full is right out of a flop anyway.
   parameter LOW_POWER = 1;                   // Setting this to 0 will reduce area, increase power, and timing should be the same.
   
   //============================
   // IO PORTS
   //============================
   input                   clk;
   input                   reset;
   output                  busy;
   input                   we;
   input [FIFO_WIDTH-1:0]  write_data;
   output                  full;
   input                   re;
   output [FIFO_WIDTH-1:0] read_data;
   output                  empty;
   
   parameter ADDR_WIDTH = ((FIFO_DEPTH < 4) ? 2 :
                           (FIFO_DEPTH < 8) ? 3 :
                           (FIFO_DEPTH < 16) ? 4 :
                           (FIFO_DEPTH < 32) ? 5 :
                           (FIFO_DEPTH < 64) ? 6 :
                           (FIFO_DEPTH < 128) ? 7 :
                           (FIFO_DEPTH < 256) ? 8 :
                           (FIFO_DEPTH < 512) ? 9 :
                           (FIFO_DEPTH < 1024) ? 10 : 11);  
             

`ifdef SIMULATION
   // VCS coverage off
   initial begin
      if ( FIFO_DEPTH-SKID_WORDS < 2 )
        `MSGIO_DISPLAY("ati_shift_fifo ERROR (%m): At least 2 non-skid entries are needed. %s has %1d.", FIFO_NAME, FIFO_DEPTH-SKID_WORDS );
      if ( FIFO_DEPTH > 2047 )
        `MSGIO_DISPLAY("ati_shift_fifo ERROR (%m): Can't support more than 2047 entries. %s is %1d deep.", FIFO_NAME, FIFO_DEPTH);
      if ( SKID_WORDS == 0 && REGISTER_WRITE_INPUTS )
        `MSGIO_DISPLAY("ati_shift_fifo ERROR (%m): %s must have at least 1 skid slot to register the write inputs.", FIFO_NAME);
   end
   // VCS coverage on
`endif
             
   //============================
   // Control
   //============================
   // flop re to eliminate some fanout.   re will tend to be late.
   reg re_q, we_q;
   reg fempty_q, was_empty;
   always @(posedge clk) begin
      if (REGISTER_READ_ENABLE)
        re_q   <= re;
      else
        re_q   <= re && !fempty_q;
      if (reset) begin
         re_q   <= 0;
      end
   end
   
// This generate is used to conditionally add the flop we_q only if REGISTER_WRITE_INPUTS == 1.
// This fixed RTL8.3 (unreachable DFF/DLAT removed) lint warnings.
   wire wen;
generate
   if (REGISTER_WRITE_INPUTS) begin
      always @(posedge clk) begin
         we_q <= we;

         if (reset) we_q <= 0;
      end

      assign wen = we_q;
   end else begin
      assign wen = we;
   end // else: !if(REGISTER_WRITE_INPUTS)
endgenerate
   
   wire ren   = (REGISTER_READ_ENABLE ? re_q : re);
   wire pop   = (!was_empty && re_q);
   wire pop0  = (REGISTER_READ_ENABLE ? pop : (!fempty_q && re));
   
   // write pointer
   reg [ADDR_WIDTH-1:0] wptr_d, wptr_q;
   wire [ADDR_WIDTH-1:0] wptr_inc = wptr_q + 1;
   wire [ADDR_WIDTH-1:0] wptr_dec = wptr_q - ((wptr_q == 0) ? 0 : 1);
   always @(/*AS*/pop or wen or wptr_dec or wptr_inc or wptr_q)
     case ({pop,wen})
       2'b00: wptr_d = wptr_q;
       2'b01: wptr_d = wptr_inc;
       2'b10: wptr_d = wptr_dec;
       2'b11: wptr_d = wptr_q;
       // VCS coverage off
       default: wptr_d = 'bx;
       // VCS coverage on
     endcase

   // full and empty
   wire full_d    = ((wptr_d >= (FIFO_DEPTH-SKID_WORDS)) && !ren);
   wire fempty_d  = (wptr_d == 0) || (pop0 && wptr_d == 1 && !REGISTER_READ_ENABLE);
   wire oneleft_d = (wptr_d == 1);


   // Clock block
   reg  empty_d, empty_q;
   reg  full_q, oneleft_q;
   always @(posedge clk) begin
      wptr_q    <= wptr_d;
      full_q    <= full_d;
      fempty_q  <= fempty_d;
      was_empty <= empty_q;

      if (reset) begin
         fempty_q  <= 1;
         was_empty <= 1;
         full_q    <= 0;
         wptr_q    <= 0;
      end
   end // always @ (posedge clk)

`ifdef SIMULATION
   // VCS coverage off
   reg fifo_written_when_full;
   always @(posedge clk) begin
     fifo_written_when_full <= !reset && wen && (wptr_q >= FIFO_DEPTH) && !pop;
   end

  // psl assert_fifo_written_when_full : assert never (fifo_written_when_full) @(posedge clk);
   // VCS coverage on
`endif

   // This generate is used to conditionally add the flop oneleft_q only if REGISTER_READ_ENABLE == 1.
   // This fixed RTL8.3 (unreachable DFF/DLAT removed) lint warnings.
generate
   if (REGISTER_READ_ENABLE) begin
      always @(posedge clk) begin
         oneleft_q <= oneleft_d;

         if (reset) oneleft_q <= 0;
      end
   end
endgenerate
      
   // Figure out where to write the next element
   integer i;
   reg [FIFO_DEPTH-1:0] waddr_eq;
   reg [FIFO_DEPTH-1:1] valid;
   reg [ADDR_WIDTH-1:0] mod_wptr;
   always @(/*AS*/pop or pop0 or wptr_dec or wptr_q) begin
      // if we are reading, write one ahead. 
      if (pop)
        mod_wptr = wptr_dec;
      else
        mod_wptr = wptr_q;

      // if we are writing to slot 1, which is being shifted, write directly to 0.
      if (pop0 && (mod_wptr == 1) && !REGISTER_READ_ENABLE)
        mod_wptr = 'b0;
   end
   
   //============================
   // Memory
   //============================

   // vectorize the write address compare
   always @(/*AS*/mod_wptr)
     for (i = 0; i < FIFO_DEPTH; i = i + 1)
        waddr_eq[i] = (i == mod_wptr);

   always @(/*AS*/wptr_q)
     for (i = 1; i < FIFO_DEPTH; i = i + 1)
        valid[i]    = (i < wptr_q);  // Only used for power saving.

   // write when enabled.
   wire [FIFO_DEPTH-1:0] write_en = (wen ? waddr_eq : 0);

   // Vectorize the load on shift signal
   wire [FIFO_DEPTH-1:0] shift_en;
   assign shift_en[0]              = (ren ? (LOW_POWER ? valid[1] : 1) : 0);
   assign shift_en[FIFO_DEPTH-1:1] = (pop ? (LOW_POWER ? {1'b0,valid[FIFO_DEPTH-1:1]} : {FIFO_DEPTH{1'b1}}) : 0);

   // Create a clock gate enable signal explicitly just to make sure it figures it out properly.
   wire [FIFO_DEPTH-1:0] clock_en = (write_en | shift_en);
   
   // Shift registers for memory
   reg [FIFO_WIDTH-1:0] reg_mem [FIFO_DEPTH-1:0];
   always @(posedge clk)
if (reset) begin
   for (i = 0; i < FIFO_DEPTH; i = i + 1)
   reg_mem[i] <= {FIFO_WIDTH{1'b0}};
end
else begin
     for (i = 0; i < FIFO_DEPTH; i = i + 1)
       // When registering write inputs. Always write to the top, and then place it.
       if (REGISTER_WRITE_INPUTS && i == FIFO_DEPTH-1)
         reg_mem[i] <= write_data;
       else
         if ( clock_en[i] )
           if (waddr_eq[i])
             // Write data
             if (REGISTER_WRITE_INPUTS)
               reg_mem[i] <= reg_mem[FIFO_DEPTH-1];  // Write from top of stack when forced to.
             else
               reg_mem[i] <= write_data;             // normally just place it from outside.
           else
             // when reading, shift from slot above.
             if (i != FIFO_DEPTH-1)                  // Top of stack has nowhere to get shifted data when reading.
               if (i == 0 && !REGISTER_READ_ENABLE)  // Bottom of stack will grab from one up or two depending on re and re_q
                 if (re_q && FIFO_DEPTH > 2)         // This is done to limit fanout by not allowing re to control the whole stack.
                   reg_mem[0] <= reg_mem[2];
                 else
                   reg_mem[0] <= reg_mem[1];
               else
                 reg_mem[i] <= reg_mem[i+1];         // Normally just shift data down from above.
end

   //===================================================================
   // Assign outputs directly from flops unless specified otherwise
   //===================================================================
   reg  [FIFO_WIDTH-1:0] read_data;
   wire [FIFO_WIDTH-1:0] reg_mem0 = reg_mem[0];
   wire [FIFO_WIDTH-1:0] reg_mem1 = reg_mem[1];
   assign full = EXTERNAL_FULL_REGISTER ? full_d : full_q;
   always @(/*AS*/fempty_d or fempty_q or oneleft_d or oneleft_q or pop or re or reg_mem0 or reg_mem1) begin
      empty_q = fempty_q;
      empty_d = fempty_d;
      read_data = reg_mem0;
      if (REGISTER_READ_ENABLE) begin
         empty_q   = (pop ? oneleft_q : fempty_q);
         read_data = (pop ? reg_mem1 : reg_mem0);
         
         // WARNING: using both REGISTER_READ_ENABLE and EXTERNAL_EMPTY_REGISTER makes the re be used directly for one mux.
         empty_d = ((!empty_q && re) ? oneleft_d : fempty_d);
      end
   end
   assign empty = EXTERNAL_EMPTY_REGISTER ? empty_d : empty_q;

   assign busy = !empty_q || (REGISTER_WRITE_INPUTS ? we_q : 1'b0);

`ifdef SIMULATION
   // Adding cover and asserts
   parameter ASSERT_ON_FULL_AND_PUSH = (SKID_WORDS == 0);
   ati_basic_fifo_cover_assert #(ASSERT_ON_FULL_AND_PUSH, 0, 1) ati_shift_basic_cover_assert(.clk(clk), .reset(reset), .empty(empty), .full(full), .push(wen), .pop(pop));
`endif
endmodule //ati_shift_fifo
