//---------------------------------------------------------------------------
// PRIM CONFIG Config registers
//   Each register has two versions: one written by the RCU (write once)
//   and one written by the user (driver).  The user one can only
//   further disable VGT-PA pipes, not re-enable ones disabled 
//   by the RCU.
//
//---------------------------------------------------------------------------

// include VGT regs from $ROOT/$BRANCH/cmn/inc/reg
`ifdef CC_GC_PRIM_CONFIG_WRITE_DIS
`else
`include "gc__ge_reg.v"
`endif

`ifdef CC_GC_PRIM_CONFIG_EXX
`else
`include "gc_register_addr_ext.v"
`endif

`include "gc_features.vh"

module ati_vgt_pa_cfg_reg (
    // outputs
    prim_cfg_inactive_ia,
    prim_cfg_inactive_vgt_pa,
    vgt_pa_active_mask,
    ia_active_mask,
    reg_rd_valid,
    reg_rd_data,
    // inputs
    reg_wr,
    reg_wr_addr,
    reg_wr_data,
    reg_rd,
    reg_rd_addr,
    clk,
    hard_resetB
);

//----------------------------------------------
output [`GC_USER_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE-1:0] vgt_pa_active_mask;
output [`GC_USER_PRIM_CONFIG_INACTIVE_IA_SIZE-1:0]  prim_cfg_inactive_ia;
output [`GC_USER_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE-1:0] prim_cfg_inactive_vgt_pa;
output [`GC_USER_PRIM_CONFIG_INACTIVE_IA_SIZE-1:0] ia_active_mask;
   
input         reg_wr;       // register write enable
input  [17:0] reg_wr_addr;  // register write addr [19:2]
input  [31:0] reg_wr_data;  // register write data
input         reg_rd;       // register read enable: only GRBM should use this, others tie to zero.
input  [17:0] reg_rd_addr;  // register read addr [19:2]
input         clk;
input         hard_resetB;   // active low reset -- this must be hard-reset, not soft reset
   
output        reg_rd_valid; // register read address matches 
output [31:0] reg_rd_data;  // register read data.  set to zero if no reg-addr match.
//----------------------------------------------

wire [`GC_USER_PRIM_CONFIG_INACTIVE_IA_SIZE-1:0]  prim_cfg_inactive_ia;
wire [`GC_USER_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE-1:0] prim_cfg_inactive_vgt_pa;

reg 				 		       gc_prim_cfg_write_dis;
   
reg [`CC_GC_PRIM_CONFIG_INACTIVE_IA_SIZE-1:0]          gc_prim_cfg_inactive_ia;
reg [`GC_USER_PRIM_CONFIG_INACTIVE_IA_SIZE-1:0]        gc_user_prim_cfg_inactive_ia;
   
reg [`CC_GC_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE-1:0]      gc_prim_cfg_inactive_vgt_pa;
reg [`GC_USER_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE-1:0]    gc_user_prim_cfg_inactive_vgt_pa;

reg [`GC_USER_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE-1:0]    vgt_pa_active_mask;
reg [`GC_USER_PRIM_CONFIG_INACTIVE_IA_SIZE-1:0]        ia_active_mask;
   
   
reg  [31:0] reg_rd_data;  
reg         reg_rd_valid;  

wire [`CC_GC_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE-1:0]  physical_vgt_pa_mask;
wire [`CC_GC_PRIM_CONFIG_INACTIVE_IA_SIZE-1:0]  physical_ia_mask;
   
   
assign prim_cfg_inactive_ia = gc_prim_cfg_inactive_ia | gc_user_prim_cfg_inactive_ia;
assign prim_cfg_inactive_vgt_pa = gc_prim_cfg_inactive_vgt_pa | gc_user_prim_cfg_inactive_vgt_pa;

// create a mask to prevent enabling IA/VGT/PAs which
// are not physically present in this config

`ifdef GC__NUM_FRONTEND__1
assign physical_ia_mask     = {`CC_GC_PRIM_CONFIG_INACTIVE_IA_SIZE{1'b1}}     << 1 ;
`else
assign physical_ia_mask     = {`CC_GC_PRIM_CONFIG_INACTIVE_IA_SIZE{1'b1}}     << `GC__NUM_FRONTEND/2 ;
`endif

assign physical_vgt_pa_mask = {`CC_GC_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE{1'b1}} << `GC__NUM_FRONTEND ;


   
// Register-write 
always @(posedge clk) begin
    if (!hard_resetB) begin
       gc_prim_cfg_write_dis            <= `CC_GC_PRIM_CONFIG_WRITE_DIS_DEFAULT;
       gc_prim_cfg_inactive_ia          <= `CC_GC_PRIM_CONFIG_INACTIVE_IA_DEFAULT;
       gc_prim_cfg_inactive_vgt_pa      <= `CC_GC_PRIM_CONFIG_INACTIVE_VGT_PA_DEFAULT;
       gc_user_prim_cfg_inactive_ia     <= `GC_USER_PRIM_CONFIG_INACTIVE_IA_DEFAULT;
       gc_user_prim_cfg_inactive_vgt_pa <= `GC_USER_PRIM_CONFIG_INACTIVE_VGT_PA_DEFAULT;
    end
    else if (reg_wr && !gc_prim_cfg_write_dis && {reg_wr_addr, 2'b00} == `CC_GC_PRIM_CONFIG_EXX) begin
        // this register can only be written once
        gc_prim_cfg_write_dis       <= reg_wr_data[`CC_GC_PRIM_CONFIG_WRITE_DIS];
        gc_prim_cfg_inactive_ia     <= reg_wr_data[`CC_GC_PRIM_CONFIG_INACTIVE_IA] | physical_ia_mask[`CC_GC_PRIM_CONFIG_INACTIVE_IA_SIZE-1:0];
        gc_prim_cfg_inactive_vgt_pa <= reg_wr_data[`CC_GC_PRIM_CONFIG_INACTIVE_VGT_PA] | physical_vgt_pa_mask[`CC_GC_PRIM_CONFIG_INACTIVE_VGT_PA_SIZE-1:0];
    end
    else if (reg_wr && {reg_wr_addr, 2'b00} == `GC_USER_PRIM_CONFIG_EXX) begin
       gc_user_prim_cfg_inactive_ia        <= reg_wr_data[`GC_USER_PRIM_CONFIG_INACTIVE_IA];
       gc_user_prim_cfg_inactive_vgt_pa    <= reg_wr_data[`GC_USER_PRIM_CONFIG_INACTIVE_VGT_PA];
    end

    vgt_pa_active_mask <= ~(gc_prim_cfg_inactive_vgt_pa | gc_user_prim_cfg_inactive_vgt_pa);
    ia_active_mask     <= ~(gc_prim_cfg_inactive_ia | gc_user_prim_cfg_inactive_ia);
end

// Register-read 
always @ * begin
    reg_rd_data  = 32'h0;
    reg_rd_valid = 0;

    if (reg_rd && {reg_rd_addr, 2'b00} == `CC_GC_PRIM_CONFIG_EXX) begin
        reg_rd_data[`CC_GC_PRIM_CONFIG_WRITE_DIS]       = gc_prim_cfg_write_dis;
        reg_rd_data[`CC_GC_PRIM_CONFIG_INACTIVE_IA]     = gc_prim_cfg_inactive_ia;
        reg_rd_data[`CC_GC_PRIM_CONFIG_INACTIVE_VGT_PA] = gc_prim_cfg_inactive_vgt_pa;
        reg_rd_valid = 1;
    end
    else if (reg_rd && {reg_rd_addr, 2'b00} == `GC_USER_PRIM_CONFIG_EXX) begin
        reg_rd_data[`GC_USER_PRIM_CONFIG_INACTIVE_IA]     = gc_user_prim_cfg_inactive_ia;
        reg_rd_data[`GC_USER_PRIM_CONFIG_INACTIVE_VGT_PA] = gc_user_prim_cfg_inactive_vgt_pa;
        reg_rd_valid = 1;
    end
end

 `ifdef SIMULATION
//-------------------------------------------------------------------------------
// Write Disable Check
//-------------------------------------------------------------------------------
// VCS coverage off
localparam disable_psl_check = "disable_psl_check=";
localparam disable_psl_write_disable_check = "disable_psl_write_disable_check=";

reg check_all_psl;
reg check_write_disable_psl;

initial
begin
  check_all_psl = 1;
  check_write_disable_psl = 1;

  if ($test$plusargs({disable_psl_check,"on"})) check_all_psl = 0;
  if ($test$plusargs({disable_psl_write_disable_check,"on"})) check_write_disable_psl = 0;
end

// psl property write_disable_check = always ((hard_resetB && check_all_psl && check_write_disable_psl && reg_wr && {reg_wr_addr, 2'b00} == `CC_GC_PRIM_CONFIG_EXX)->(~gc_prim_cfg_write_dis)) @(posedge clk);
// psl assert write_disable_check report "ERROR: Trying to write CC_GC_PRIM_CONFIG_EXX while write disable is set!";
// VCS coverage on
`endif // SIMULATION
endmodule
