//---------------------------------------------------------------------------
// Work Group Processor Config registers
//   Each register has two versions: one written by the RCU (write once)
//   and one written by the user (driver).  The user one can only
//   further disable WGPs, not re-enable ones disabled 
//   by the RCU.
//
// Output is the number of active WGPs, minus 1.
// Output is also a mask of which WGPs are enabled.
//
// Currently this module supports 16 WGPs.
//---------------------------------------------------------------------------

`include "gc__ge_reg.v"
`include "gc__sq_reg.v"
`include "gc_register_addr.v"
`include "gc_features.vh"



module ati_wgp_cfg_reg (
    // outputs
    shader_active_wgps,
    shader_active_wgp_mask,
    shader_dpfp_rate,
    sqc_balance_disable,

    // inputs
    reg_wr,
    reg_wr_addr,
    reg_wr_data,
    reg_rd,
    reg_rd_valid,
    reg_rd_addr,
    reg_rd_data,
    reg_instance_select,
    clk,
    hard_resetB
);


parameter IGNORE_GC_USER = 0 ;

//----------------------------------------------
output [3:0] shader_active_wgps;    // # of active WGPs, minus 1.
                                    // 0 = 1 WGP (WGP 0), 7 = 8 WGPs 

output [`CC_GC_SHADER_ARRAY_CONFIG_INACTIVE_WGPS_SIZE-1:0]      shader_active_wgp_mask; // bit mask per WGP.   1 = active, 0 = inactive
output [`CC_GC_SHADER_RATE_CONFIG_DPFP_RATE_SIZE-1:0]           shader_dpfp_rate;       // rate-limit double precision floating point ops
output [`CC_GC_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE_SIZE-1:0] sqc_balance_disable;    // turn off sqc balancing

input         reg_wr;       // register write enable
input  [15:0] reg_wr_addr;  // register write addr [17:2]
input  [31:0] reg_wr_data;  // register write data
input         reg_rd;       // register read enable:  only VGT should use this, others tie to zero.
input  [15:0] reg_rd_addr;  // register read addr [17:2]
output        reg_rd_valid; // register read address matches 
output [31:0] reg_rd_data;  // register read data.  set to zero if no reg-addr match.
input         reg_instance_select; // this se selected for read/write
input         clk;
input         hard_resetB;  // active low reset -- this must be hard-reset, not soft reset
//----------------------------------------------

// (abbreviated "shader_array_config" to "sacfg", user version to "usacfg")
reg                                                             gc_sacfg_write_dis;
reg [`CC_GC_SHADER_ARRAY_CONFIG_INACTIVE_WGPS_SIZE-1:0]         gc_sacfg_inactive_wgps;

reg [`GC_USER_SHADER_ARRAY_CONFIG_INACTIVE_WGPS_SIZE-1:0]       gc_usacfg_inactive_wgps;

// (abbreviated "shader_rate_config" to "srcfg", user version to "usrcfg")
reg                                                            gc_srcfg_write_dis;
reg [`CC_GC_SHADER_RATE_CONFIG_DPFP_RATE_SIZE-1:0]             gc_srcfg_dpfp_rate;
reg [`CC_GC_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE_SIZE-1:0]   gc_srcfg_sqc_balance_disable;

reg [`GC_USER_SHADER_RATE_CONFIG_DPFP_RATE_SIZE-1:0]           gc_usrcfg_dpfp_rate;
reg [`GC_USER_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE_SIZE-1:0] gc_usrcfg_sqc_balance_disable;

reg [3:0]   shader_active_wgps;
reg [15:0]  shader_active_wgp_mask;
reg [31:0]  reg_rd_data;  
reg         reg_rd_valid;  
reg         reg_wr_d1;

wire [4:0] num_wgps_active =
                            shader_active_wgp_mask[0] +
                            shader_active_wgp_mask[1] +
                            shader_active_wgp_mask[2] +
                            shader_active_wgp_mask[3] +
                            shader_active_wgp_mask[4] +
                            shader_active_wgp_mask[5] +
                            shader_active_wgp_mask[6] +
                            shader_active_wgp_mask[7] +
                            shader_active_wgp_mask[8] +
                            shader_active_wgp_mask[9] +
                            shader_active_wgp_mask[10] +
                            shader_active_wgp_mask[11] +
                            shader_active_wgp_mask[12] +
                            shader_active_wgp_mask[13] +
                            shader_active_wgp_mask[14] +
                            shader_active_wgp_mask[15] ;

// create a mask to prevent enabling WGPs which
// are not physically present in this config
wire [15:0] physical_wgp_mask = 16'hffff << `GC__NUM_WGP_PER_SA;

// Register-write 
always @(posedge clk) begin
    if (!hard_resetB) begin
        gc_sacfg_write_dis              <= `CC_GC_SHADER_ARRAY_CONFIG_WRITE_DIS_DEFAULT;
        gc_sacfg_inactive_wgps          <= `CC_GC_SHADER_ARRAY_CONFIG_INACTIVE_WGPS_DEFAULT;
        gc_usacfg_inactive_wgps         <= `GC_USER_SHADER_ARRAY_CONFIG_INACTIVE_WGPS_DEFAULT;
        gc_srcfg_write_dis              <= `CC_GC_SHADER_RATE_CONFIG_WRITE_DIS_DEFAULT;
        gc_srcfg_dpfp_rate              <= `CC_GC_SHADER_RATE_CONFIG_DPFP_RATE_DEFAULT;
        gc_srcfg_sqc_balance_disable    <= `CC_GC_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE_DEFAULT;
        gc_usrcfg_dpfp_rate             <= `GC_USER_SHADER_RATE_CONFIG_DPFP_RATE_DEFAULT;
        gc_usrcfg_sqc_balance_disable   <= `GC_USER_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE_DEFAULT;
    end
    else if (reg_wr && reg_instance_select && !gc_sacfg_write_dis && ({reg_wr_addr, 2'b00} == `CC_GC_SHADER_ARRAY_CONFIG)) begin
        // this register can only be written once
        gc_sacfg_write_dis              <= reg_wr_data[`CC_GC_SHADER_ARRAY_CONFIG_WRITE_DIS];
        gc_sacfg_inactive_wgps          <= (reg_wr_data[`CC_GC_SHADER_ARRAY_CONFIG_INACTIVE_WGPS] | physical_wgp_mask);
    end
    else if (reg_wr && reg_instance_select && ({reg_wr_addr, 2'b00} == `GC_USER_SHADER_ARRAY_CONFIG)) begin
        gc_usacfg_inactive_wgps         <= reg_wr_data[`GC_USER_SHADER_ARRAY_CONFIG_INACTIVE_WGPS];
    end
    else if (reg_wr && reg_instance_select && !gc_srcfg_write_dis && ({reg_wr_addr, 2'b00} == `CC_GC_SHADER_RATE_CONFIG)) begin
        // this register can only be written once
        gc_srcfg_write_dis              <= reg_wr_data[`CC_GC_SHADER_RATE_CONFIG_WRITE_DIS];
        gc_srcfg_dpfp_rate              <= reg_wr_data[`CC_GC_SHADER_RATE_CONFIG_DPFP_RATE];
        gc_srcfg_sqc_balance_disable    <= reg_wr_data[`CC_GC_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE];
    end
    else if (reg_wr && reg_instance_select && ({reg_wr_addr, 2'b00} == `GC_USER_SHADER_RATE_CONFIG)) begin
        gc_usrcfg_dpfp_rate             <= reg_wr_data[`GC_USER_SHADER_RATE_CONFIG_DPFP_RATE];
        gc_usrcfg_sqc_balance_disable   <= reg_wr_data[`GC_USER_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE];
    end
    // Find the number of ACTIVE WGPs (not inactive ones)
    // Take the maximum of the # of WGPs
    shader_active_wgps             <= num_wgps_active - 1;
end // always @ (posedge clk)

generate
    always @(posedge clk) begin
        // for clock gating
        reg_wr_d1 <= (reg_wr && reg_instance_select && !gc_sacfg_write_dis && ({reg_wr_addr, 2'b00} == `CC_GC_SHADER_ARRAY_CONFIG)) ||
                     (reg_wr && reg_instance_select && ({reg_wr_addr, 2'b00} == `GC_USER_SHADER_ARRAY_CONFIG));

        if (!hard_resetB || reg_wr_d1) begin
            // ignore user version of inactive_wgps based on parameter value.
            if (IGNORE_GC_USER == 0) begin
                shader_active_wgp_mask <= ~(gc_sacfg_inactive_wgps | gc_usacfg_inactive_wgps);
            end
            else begin
                shader_active_wgp_mask <= ~(gc_sacfg_inactive_wgps);
            end
        end
    end
endgenerate

// Register-read 
always @ * begin
    reg_rd_data = 32'h0;
    reg_rd_valid = 0;

    if (reg_rd && reg_instance_select && ({reg_rd_addr, 2'b00} == `CC_GC_SHADER_ARRAY_CONFIG)) begin
        reg_rd_data[`CC_GC_SHADER_ARRAY_CONFIG_WRITE_DIS]       = gc_sacfg_write_dis;
        reg_rd_data[`CC_GC_SHADER_ARRAY_CONFIG_INACTIVE_WGPS]   = gc_sacfg_inactive_wgps;
        reg_rd_valid = 1;
    end
    else if (reg_rd && reg_instance_select && ({reg_rd_addr, 2'b00} == `GC_USER_SHADER_ARRAY_CONFIG)) begin
        reg_rd_data[`GC_USER_SHADER_ARRAY_CONFIG_INACTIVE_WGPS] = gc_usacfg_inactive_wgps;
        reg_rd_valid = 1;
    end
    else if (reg_rd && reg_instance_select && ({reg_rd_addr, 2'b00} == `CC_GC_SHADER_RATE_CONFIG)) begin
        reg_rd_data[`CC_GC_SHADER_RATE_CONFIG_WRITE_DIS]           = gc_srcfg_write_dis;
        reg_rd_data[`CC_GC_SHADER_RATE_CONFIG_DPFP_RATE]           = gc_srcfg_dpfp_rate;
        reg_rd_data[`CC_GC_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE] = gc_srcfg_sqc_balance_disable;
        reg_rd_valid = 1;
    end
    else if (reg_rd && reg_instance_select && ({reg_rd_addr, 2'b00} == `GC_USER_SHADER_RATE_CONFIG)) begin
        reg_rd_data[`GC_USER_SHADER_RATE_CONFIG_DPFP_RATE]           = gc_usrcfg_dpfp_rate;
        reg_rd_data[`GC_USER_SHADER_RATE_CONFIG_SQC_BALANCE_DISABLE] = gc_usrcfg_sqc_balance_disable;
        reg_rd_valid = 1;
    end
end

assign shader_dpfp_rate     = (gc_srcfg_dpfp_rate > gc_usrcfg_dpfp_rate) ? gc_srcfg_dpfp_rate : gc_usrcfg_dpfp_rate;
assign sqc_balance_disable  = gc_srcfg_sqc_balance_disable || gc_usrcfg_sqc_balance_disable;

`ifdef SIMULATION
//-------------------------------------------------------------------------------
// Write Disable Check
//-------------------------------------------------------------------------------
// VCS coverage off
localparam disable_psl_check = "disable_psl_check=";
localparam disable_psl_write_disable_check = "disable_psl_write_disable_check=";

reg check_all_psl;
reg check_write_disable_psl;

initial
begin
  check_all_psl = 1;
  check_write_disable_psl = 1;

  if ($test$plusargs({disable_psl_check,"on"})) check_all_psl = 0;
  if ($test$plusargs({disable_psl_write_disable_check,"on"})) check_write_disable_psl = 0;
end

// psl property cc_gc_shader_array_config_write_disable_check = always ((hard_resetB && check_all_psl && check_write_disable_psl && reg_wr && reg_instance_select && {reg_wr_addr, 2'b00} == `CC_GC_SHADER_ARRAY_CONFIG)->(~gc_sacfg_write_dis)) @(posedge clk);
// psl assert cc_gc_shader_array_config_write_disable_check report "ERROR: Trying to write CC_GC_SHADER_ARRAY_CONFIG while write disable is set!";

// psl property cc_gc_shader_rate_config_write_disable_check = always ((hard_resetB && check_all_psl && check_write_disable_psl && reg_wr && reg_instance_select && {reg_wr_addr, 2'b00} == `CC_GC_SHADER_RATE_CONFIG)->(~gc_srcfg_write_dis)) @(posedge clk);
// psl assert cc_gc_shader_rate_config_write_disable_check report "ERROR: Trying to write CC_GC_SHADER_RATE_CONFIG while write disable is set!";

// VCS coverage on
`endif // SIMULATION
endmodule
