///////////////////////////////////////////////////////////////////////////////////////////
////// block_mem_power_ctrl.v
/////  This module controls the light sleep and deep sleep control for CP/RLC memories ////
//////////////////////////////////////////////////////////////////////////////////////////

module block_mem_power_ctrl 
(
      clk,
      reset,
      busy_in,
      block_ls_en,
      block_ds_en,
      block_ls_on_delay,
      block_ls_off_delay,
      dyn_clk_vld,

      //Outputs
      busy_out,
      block_mem_ls,
      block_mem_ds

);

input              clk;
input              reset;
input              busy_in;
input              dyn_clk_vld;
input              block_ls_en;
input              block_ds_en;
input [7:0]        block_ls_on_delay;
input [7:0]        block_ls_off_delay;


output             busy_out;
output             block_mem_ls;
output             block_mem_ds;
reg             busy_out;
reg             block_mem_ls;
reg             block_mem_ds;



parameter    BLK_CLK_ON_ST       = 3'h0;
parameter    MEM_LS_ON_ST        = 3'h1;
parameter    MEM_LS_ON_WAIT_ST   = 3'h2;
parameter    MEM_LS_OFF_ST       = 3'h3;
parameter    MEM_LS_OFF_WAIT_ST  = 3'h4;
parameter    BLK_CLK_ON_WAIT_ST  = 3'h5;
parameter    BLK_CLK_OFF_WAIT_ST = 3'h6;


reg   [7:0]  delay_timer;

reg   [2:0]  mem_state, mem_state_next;

reg          delay_timer_expired, load_on_delay_timer, load_off_delay_timer;
reg          busy_out_next, block_mem_ls_next, block_mem_ds_next ;


always @ (posedge clk)
begin
    if (reset)
    begin
       busy_out     <= 1'b1;
       block_mem_ls <= 1'b0;
       block_mem_ds <= 1'b0;
       mem_state    <= BLK_CLK_ON_ST;
    end
    else
    begin
       mem_state    <= mem_state_next;
       block_mem_ls <= block_mem_ls_next;
       block_mem_ds <= block_mem_ds_next;
       busy_out     <= busy_out_next;
    end
end

always @(*)
begin
      busy_out_next  = busy_out;
      load_on_delay_timer = 1'b0;
      load_off_delay_timer = 1'b0;
      block_mem_ls_next = block_mem_ls;
      block_mem_ds_next = block_mem_ds;
      mem_state_next = mem_state;


      case (mem_state)
      BLK_CLK_ON_ST: begin
                       busy_out_next = 1'b1;
                       block_mem_ls_next = 1'b0;
                       block_mem_ds_next = 1'b0;
                       if (!busy_in)
                          mem_state_next = BLK_CLK_OFF_WAIT_ST;
                          busy_out_next  = busy_in;
                      end
      BLK_CLK_OFF_WAIT_ST: begin
                           if (dyn_clk_vld)
                              begin
                                mem_state_next = BLK_CLK_OFF_WAIT_ST;
                              if (busy_in)
                                mem_state_next = BLK_CLK_ON_ST;
                              end
                           else if (busy_in)
                                mem_state_next = BLK_CLK_ON_ST;
                            else 
                               if (block_ls_en | block_ds_en)
                               begin
                                  load_on_delay_timer = 1'b1;
                                  mem_state_next   = MEM_LS_ON_WAIT_ST;
                               end
                           end
      MEM_LS_ON_WAIT_ST: begin
                              if (busy_in)
                              begin
                                 mem_state_next = MEM_LS_OFF_WAIT_ST;
                                 load_off_delay_timer = 1'b1;
                                 block_mem_ls_next = 1'b0;
                                 block_mem_ds_next = 1'b0;
                              end
                              else if (delay_timer_expired)
                              begin
                                 mem_state_next = MEM_LS_ON_ST;
                                  block_mem_ls_next = block_ls_en & ~block_ds_en;
                                  block_mem_ds_next = block_ds_en;
                              end
                              else
                               mem_state_next = MEM_LS_ON_WAIT_ST;
                          end
      MEM_LS_ON_ST       : begin
                              if (busy_in)
                              begin
                                 mem_state_next = MEM_LS_OFF_WAIT_ST;
                                 load_off_delay_timer = 1'b1;
                                 block_mem_ls_next = 1'b0;
                                 block_mem_ds_next = 1'b0;
                              end
                              else
                                  mem_state_next = MEM_LS_ON_ST;
                           end
       MEM_LS_OFF_WAIT_ST: begin
                            if (delay_timer_expired)
                               mem_state_next = MEM_LS_OFF_ST;
                            else
                               mem_state_next = MEM_LS_OFF_WAIT_ST;
                            end
       MEM_LS_OFF_ST:       begin
                              busy_out_next = 1'b1;
                              mem_state_next = BLK_CLK_ON_WAIT_ST;
                            end
       BLK_CLK_ON_WAIT_ST:  begin
                               if (dyn_clk_vld)
                                   mem_state_next = BLK_CLK_ON_ST;
                               else
                                   mem_state_next = BLK_CLK_ON_WAIT_ST;
                            end
       default: ;
      endcase
end


always @ (posedge clk)
begin
     if (reset)
       delay_timer <= 8'h0;
     else
       begin
         if (load_on_delay_timer )
            delay_timer <= block_ls_on_delay;
         else if (load_off_delay_timer)
            delay_timer <= block_ls_off_delay;
         else if ((mem_state == MEM_LS_ON_WAIT_ST) || (mem_state == MEM_LS_OFF_WAIT_ST))
            delay_timer <= delay_timer - 8'h1;
       end

end

always @(*)
begin
    if (((mem_state == MEM_LS_ON_WAIT_ST) || (mem_state == MEM_LS_OFF_WAIT_ST)) && (delay_timer == 8'h0))
        delay_timer_expired = 1'b1;
    else
        delay_timer_expired = 1'b0;
end
       







endmodule





