`include "mm_global.d"

module bpm_avfs_fsm (
  clk,
  reset,
  bpm_avfs_cfg,
  bpm_avfs_mm_ctl,
  bpm_avfs_mm_wdata_h,
  bpm_avfs_mm_wdata_l,
  MmDatIn_M4,   

  bpm_avfs_mm_rdata_h,
  bpm_avfs_mm_rdata_l,
  avfs_mm_cmd_done,
  MmBusOut_M1,   
  MmLclRdEn_M2,              
  MmLclWrEn_M2,              
  MmLclSz_M2,    
  MmState        
);

  input  clk;
  input  reset;
  input  [23:0]      bpm_avfs_cfg;
  input  [19:0]      bpm_avfs_mm_ctl;
  input  [31:0]      bpm_avfs_mm_wdata_h;
  input  [31:0]      bpm_avfs_mm_wdata_l;
  input  [`MM_DATA]  MmDatIn_M4;    //8-bit data from avfs back to M&M bus for read

  output [31:0]      bpm_avfs_mm_rdata_h;
  output [31:0]      bpm_avfs_mm_rdata_l;
  output             avfs_mm_cmd_done;
  output [`MM_DATA]  MmBusOut_M1;   //8-bit M&M bus output contains local reg addr in M2 and write data in M4
  output MmLclRdEn_M2;              //local register read indication
  output MmLclWrEn_M2;              //local register write indication
  output [`MM_SZ]    MmLclSz_M2;    //local register size indication
  output [`MM_STATE] MmState;       //M&M state for read/write byte select

reg  MmLclRdEn_M2;
reg  MmLclWrEn_M2;
reg  [`MM_SZ]    MmLclSz_M2;
reg  [`MM_DATA]  MmBusOut_M1;

reg  [31:0]      bpm_avfs_mm_rdata_h;
reg  [31:0]      bpm_avfs_mm_rdata_l;

reg  [`MM_STATE]   MmState;
reg  [`MM_STATE]   nxt_MmState;

reg  [`MM_SZ]      mm_size;
reg  [`MM_ADDR]    mm_addr;

reg  mm_rdwr_en; 
reg  avfs_mm_cmd_done;
reg  [`MM_DATA] mm_data_out;
reg  mm_wren_out;
reg  mm_rden_out;

wire avfs_node_match = bpm_avfs_mm_ctl[19] || ((bpm_avfs_mm_ctl[`MM_STOP_ID]==4'hf) ? (bpm_avfs_cfg[6:4]==bpm_avfs_mm_ctl[18:16]) : ((bpm_avfs_cfg[3:0]==bpm_avfs_mm_ctl[`MM_STOP_ID]) && (bpm_avfs_cfg[6:4]==bpm_avfs_mm_ctl[18:16])));
//force mm_size to 64 bits if mm_size is illegal
wire [`MM_SZ] valid_mm_size = ((bpm_avfs_mm_ctl[`MM_STOP_SZ]==2'b10) || (bpm_avfs_mm_ctl[`MM_STOP_SZ]==2'b11)) ? 2'b01 : bpm_avfs_mm_ctl[`MM_STOP_SZ];

always @(posedge clk)
begin
  if(reset) 
    mm_size <= 0; 
  else if(avfs_mm_cmd_done)
    mm_size <= 0;
  else if(bpm_avfs_mm_ctl[`MM_VAL] && avfs_node_match && (MmState==`MM_STATE_M0))
    mm_size <= valid_mm_size; 
end

always @(posedge clk)
begin
  if(reset) 
    mm_addr <= 0; 
  else if(avfs_mm_cmd_done)
    mm_addr <= 0;
  else if(bpm_avfs_mm_ctl[`MM_VAL] && avfs_node_match && (MmState==`MM_STATE_M0))
    mm_addr <= bpm_avfs_mm_ctl[15:8]; 
end

always @(posedge clk)
begin
  if(reset) 
    mm_rdwr_en  <= 0; 
  else if(avfs_mm_cmd_done)
    mm_rdwr_en  <= 0; 
  else if(bpm_avfs_mm_ctl[`MM_VAL] && avfs_node_match && (MmState==`MM_STATE_M0)) 
    mm_rdwr_en <= bpm_avfs_mm_ctl[`MM_RD]; 
end


always @(posedge clk)
begin
  if(reset) 
    MmState  <= `MM_STATE_M0; 
  else 
    MmState  <= nxt_MmState; 
end
   
always @(*) 
begin
  nxt_MmState  =  MmState;
  avfs_mm_cmd_done = 0;
  mm_data_out = 0;
  mm_wren_out = 0;
  mm_rden_out = 0;
  
  case(MmState) 
    `MM_STATE_M0: begin
         if(bpm_avfs_mm_ctl[`MM_VAL] && avfs_node_match) begin
           nxt_MmState = `MM_STATE_M1;
           mm_data_out = {1'b1,bpm_avfs_mm_ctl[`MM_RD],valid_mm_size,bpm_avfs_mm_ctl[`MM_STOP_ID]}; 
         end     
       end
    `MM_STATE_M1: begin
         nxt_MmState = `MM_STATE_M2;
         mm_wren_out = !mm_rdwr_en;
         mm_rden_out = mm_rdwr_en;
         mm_data_out = mm_addr;
       end 
    `MM_STATE_M2: begin
         nxt_MmState = `MM_STATE_M3;
       end 
    `MM_STATE_M3: begin
         nxt_MmState = `MM_STATE_M4;
       end 
    `MM_STATE_M4: begin
         nxt_MmState = `MM_STATE_M5;
         mm_data_out = bpm_avfs_mm_wdata_l[7:0];
       end 
    `MM_STATE_M5: begin
         nxt_MmState = `MM_STATE_M6;
         mm_data_out = bpm_avfs_mm_wdata_l[15:8];
       end 
    `MM_STATE_M6: begin
         nxt_MmState = `MM_STATE_M7;
         mm_data_out = bpm_avfs_mm_wdata_l[23:16];
       end 
    `MM_STATE_M7: begin
         mm_data_out = bpm_avfs_mm_wdata_l[31:24];
         if(mm_size==2'b00) begin
           nxt_MmState = `MM_STATE_M0;
           avfs_mm_cmd_done = 1;
         end else begin
           nxt_MmState = `MM_STATE_M8;
         end 
       end
    `MM_STATE_M8: begin
         nxt_MmState = `MM_STATE_M9;
         mm_data_out = bpm_avfs_mm_wdata_h[7:0];
       end 
    `MM_STATE_M9: begin
         nxt_MmState = `MM_STATE_M10;
         mm_data_out = bpm_avfs_mm_wdata_h[15:8];
       end 
    `MM_STATE_M10: begin
         nxt_MmState = `MM_STATE_M11;
         mm_data_out = bpm_avfs_mm_wdata_h[23:16];
       end 
    `MM_STATE_M11: begin
         mm_data_out = bpm_avfs_mm_wdata_h[31:24];
         if(mm_size==2'b01) begin
           nxt_MmState = `MM_STATE_M0;
           avfs_mm_cmd_done = 1; 
         end else begin
           nxt_MmState = `MM_STATE_M12;
         end
       end
    default: begin
         nxt_MmState[`MM_STATE] = MmState[`MM_STATE] + `MM_STATE_WIDTH'b1;
       end
  endcase
end

always @(posedge clk)
begin
  if(reset) begin
    MmBusOut_M1  <= 0; 
    MmLclRdEn_M2 <= 0;  
    MmLclWrEn_M2 <= 0;
    MmLclSz_M2   <= 0;
  end else begin
    MmBusOut_M1  <= mm_data_out;
    MmLclRdEn_M2 <= mm_rden_out;  
    MmLclWrEn_M2 <= mm_wren_out;
    MmLclSz_M2   <= mm_size;
  end
end

always @(posedge clk)
begin
  if(reset) begin
    bpm_avfs_mm_rdata_l  <= 0; 
    bpm_avfs_mm_rdata_h  <= 0;  
  end else if((MmState==`MM_STATE_M4) && mm_rdwr_en) begin
    bpm_avfs_mm_rdata_l[7:0]  <= MmDatIn_M4;
  end else if((MmState==`MM_STATE_M5) && mm_rdwr_en) begin
    bpm_avfs_mm_rdata_l[15:8]  <= MmDatIn_M4;
  end else if((MmState==`MM_STATE_M6) && mm_rdwr_en) begin
    bpm_avfs_mm_rdata_l[23:16]  <= MmDatIn_M4;
  end else if((MmState==`MM_STATE_M7) && mm_rdwr_en) begin
    bpm_avfs_mm_rdata_l[31:24]  <= MmDatIn_M4;
  end else if((MmState==`MM_STATE_M8) && mm_rdwr_en) begin
    bpm_avfs_mm_rdata_h[7:0]  <= MmDatIn_M4;
  end else if((MmState==`MM_STATE_M9) && mm_rdwr_en) begin
    bpm_avfs_mm_rdata_h[15:8]  <= MmDatIn_M4;
  end else if((MmState==`MM_STATE_M10) && mm_rdwr_en) begin
    bpm_avfs_mm_rdata_h[23:16]  <= MmDatIn_M4;
  end else if((MmState==`MM_STATE_M11) && mm_rdwr_en) begin
    bpm_avfs_mm_rdata_h[31:24]  <= MmDatIn_M4;
  end
end

endmodule
