//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//
//         If you find any issue or have any feedback, please email to sagheer.ahmad@amd.com
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps
module cgtt_local(
   // Outputs
   static_oclk, dyn_oclk, reg_oclk, dyn_oclk_vld, reg_oclk_vld,
   // Inputs
   resetb, iclk, cg_override, core_busy, reg_busy, CGTT_BLK_CTRL_REG
   );
//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cg_override; 
        
 output      static_oclk;
 output      dyn_oclk;
 output      reg_oclk;
 output      dyn_oclk_vld;
 output      reg_oclk_vld;
 input       core_busy;
 input       reg_busy;
 input [31:0] CGTT_BLK_CTRL_REG;
parameter scan_grp_dyn_oclk = 0 ; //defaulted
/////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////
//block CGTT register
/////////////////////////////////////////////////////
// ON_DELAY     = CGTT_BLK_CTRL_REG[3:0];  //dyn_oclk ON is delayed by this counter.
// OFF_DELAY    = CGTT_BLK_CTRL_REG[11:4]; //clk_oclk OFF is delayed by this counter, also if busy is shorter than OFF_DELAY,
//               then it is filtered out. OFF_DELAY must be prorgammed to be greater than ON_DELAY.
// SOFT_CORE_OVERRIDE = CGTT_BLK_CTRL_REG[30];  //busy is ignored & dyn_oclk in always turned on.
// SOFT_REG_OVERRIDE  = CGTT_BLK_CTRL_REG[31];  //busy is ignored & reg_oclk in always turned on.
/////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////
aticlk_buf u_static_clk(.Y(static_oclk), .A(iclk));
  wire override_s;
// sync usync_override(`ifdef ABV .InputChkDis(!resetb), `endif .D(cg_override), .CK(static_oclk), .Q(override_s) );
techind_sync_icd #(.depth           (3)
       )
u_techind_sync(
               .SB              (1'b1       ),
               .RB              (1'b1       ),
               .FLUSH           (1'b0       ),
               .CKsrc           (1'b0       ),
               .CKdst           (static_oclk),
               .EN              (1'b1       ),
               .Dsrc            (cg_override),
               .Qsrc            (),
               .Qdst            (override_s ),
               `ifdef ABV
               .InputChkDis     (!resetb    ),
               `endif
               .SC1             (1'b0       ),
               .SC2             (1'b0       ),
               .SE              (1'b0       ),
               .SI              (1'b0       ),
               .SO              ()
       ) ;
cgtt_local_dcg #( .scan_grp_dyn_oclk (scan_grp_dyn_oclk) )
   u_cgtt_local_dclk
  (.soft_override  (CGTT_BLK_CTRL_REG[30]),
   .soft_on_delay  (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay (CGTT_BLK_CTRL_REG[11:4]),
  
   // Outputs
   .dyn_oclk       (dyn_oclk),
   .dyn_oclk_vld   (dyn_oclk_vld),
   // Inputs
   .resetb         (resetb),
   .iclk           (iclk),
   .cg_override    (override_s),
   .core_busy      (core_busy),
   .static_oclk    (static_oclk));
parameter scan_grp_reg_oclk = 0 ; //defaulted
cgtt_local_rcg #(.scan_grp_reg_oclk (scan_grp_reg_oclk) )
   u_cgtt_local_rclk
  (.soft_override  (CGTT_BLK_CTRL_REG[31]),
  
   // Outputs
   .reg_oclk       (reg_oclk),
   .reg_oclk_vld   (reg_oclk_vld),
   // Inputs
   .resetb         (resetb),
   .iclk           (iclk),
   .cg_override    (override_s),
   .reg_busy       (reg_busy),
   .static_oclk    (static_oclk));
/////////////////////////////////////////////////////////////////////////
//To provide information about how often a block is turning its clock off
/////////////////////////////////////////////////////////////////////////
`ifdef SIMULATION
integer dyn_oclk_off_count;
integer reg_oclk_off_count;
integer static_oclk_count;
integer    msg_rate;
initial
begin
  dyn_oclk_off_count = 0;
  reg_oclk_off_count = 0;
  static_oclk_count  = 0;
end
always @(posedge static_oclk)
begin
  if(resetb && ~dyn_oclk_vld) dyn_oclk_off_count = dyn_oclk_off_count + 1;  //don't count when resetb is active
  if(resetb && ~reg_oclk_vld) reg_oclk_off_count = reg_oclk_off_count + 1;
  if(resetb) static_oclk_count = static_oclk_count + 1;
//VCS coverage off
  if( $test$plusargs("LOCAL_CLOCK_GATING_INFO") ) begin
     if(resetb && ((static_oclk_count % 1000)==0)) begin
      $display("INFO: LCG: percentage of cycles dyn-clk is off = %f, total cycles=%f; local-clock-gate instance = %m\n", dyn_oclk_off_count / static_oclk_count, static_oclk_count);
      $display("INFO: LCG: percentage of cycles reg-clk is off = %f, total cycles=%f; local-clock-gate instance = %m\n", reg_oclk_off_count / static_oclk_count, static_oclk_count);
    end
  end else if( $value$plusargs("LOCAL_CLOCK_GATING_MSG_RATE+%d", msg_rate) ) begin //message will be printed after every 'msg_rate' number of cycles
     if(resetb && ((static_oclk_count % msg_rate)==0)) begin
      $display("INFO: LCG: percentage of cycles dyn-clk is off = %f, total cycles=%f; local-clock-gate instance = %m\n", dyn_oclk_off_count / static_oclk_count, static_oclk_count);
      $display("INFO: LCG: percentage of cycles reg-clk is off = %f, total cycles=%f; local-clock-gate instance = %m\n", reg_oclk_off_count / static_oclk_count, static_oclk_count);
    end
  end
//VCS coverage on
  //TBD: how do we print at the end of sim. i.e. how can I capture end of sim condition & then print info.
  //ggreenst: most testbenches use "always @(posedge tb.pending_finish)" for this...
end
`endif
endmodule
