//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic. 
//	       dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//	       by enable signal coming from xRBM block.
//
//	       If you find any issue or have any feedback, please email to sagheer.ahmad@amd.com
//////////////////////////////////////////////////////////////////////////////////////////////////////////
module cgtt_local_1r1d_harvest_wrapper (
   resetb,
   iclk,
   cg_override,
   reg_busy,
   core_busy,
   CGTT_BLK_CTRL_REG,
   static_oclk,
   dyn_oclk,
   dyn_oclk_vld,
   reg_oclk,
   reg_oclk_vld
);
//////////////////////////
//in/outs
//////////////////////////
input       resetb;	
input       iclk;
input       cg_override; 
input       reg_busy;
input       core_busy;
input [31:0] CGTT_BLK_CTRL_REG;
output      static_oclk;
output      dyn_oclk;
output      dyn_oclk_vld;
output      reg_oclk;
output      reg_oclk_vld;
reg         reg_busy_d1;
reg         reg_busy_d2;
wire        reg_busy_extend;
wire        reg_soft_override;
reg         reg_soft_override_d1;
reg         reg_soft_override_d2;
wire        reg_soft_override_extend;
reg         core_busy_d1;
reg         core_busy_d2;
wire        core_busy_extend;
wire        core_soft_override;
reg         core_soft_override_d1;
reg         core_soft_override_d2;
wire        core_soft_override_extend;
wire [31:0] cgtt_blk_ctrl_reg_modified;
parameter	scan_grp_dyn_clk = 0;
assign reg_soft_override = CGTT_BLK_CTRL_REG[31];
assign core_soft_override = CGTT_BLK_CTRL_REG[30];
always @(posedge static_oclk)
begin
   reg_busy_d1 <= reg_busy;
   reg_busy_d2 <= reg_busy_d1;
   core_busy_d1 <= core_busy;
   core_busy_d2 <= core_busy_d1;
   reg_soft_override_d1 <= reg_soft_override;
   reg_soft_override_d2 <= reg_soft_override_d1;
   core_soft_override_d1 <= core_soft_override;
   core_soft_override_d2 <= core_soft_override_d1;
end 
assign reg_busy_extend  = reg_busy  || reg_busy_d1  || reg_busy_d2;
assign core_busy_extend = core_busy || core_busy_d1 || core_busy_d2;
assign reg_soft_override_extend  = reg_soft_override  || reg_soft_override_d1  || reg_soft_override_d2;
assign core_soft_override_extend = core_soft_override || core_soft_override_d1 || core_soft_override_d2;
assign cgtt_blk_ctrl_reg_modified = {reg_soft_override_extend, core_soft_override_extend, 
                                     CGTT_BLK_CTRL_REG[29:0]};
cgtt_local #( .scan_grp_dyn_oclk(scan_grp_dyn_clk)) uCGTT_LOCAL (
   .resetb            (resetb),
   .iclk              (iclk),
   .cg_override       (cg_override), 
   .reg_busy          (reg_busy_extend),
   .core_busy         (core_busy_extend),
   .CGTT_BLK_CTRL_REG (cgtt_blk_ctrl_reg_modified),
   .static_oclk       (static_oclk),
   .reg_oclk          (reg_oclk),
   .dyn_oclk          (dyn_oclk),
   .reg_oclk_vld      (),
   .dyn_oclk_vld      ()
);
cgtt_local_dclkon uDYN_CLK_VLD (
   .dyn_oclk_vld(dyn_oclk_vld),
   .core_busy(core_busy),
   .resetb(resetb), 
   .static_oclk(static_oclk),
   .soft_override(core_soft_override),
   .soft_on_delay(CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay(CGTT_BLK_CTRL_REG[11:4])
);
cgtt_local_rclkon uREG_CLK_VLD (
   .reg_oclk_vld(reg_oclk_vld), 
   .reg_busy(reg_busy),
   .resetb(resetb),
   .static_oclk(static_oclk),
   .soft_override(reg_soft_override)
);
endmodule
