//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//
//         If you find any issue or have any feedback, please email to sagheer.ahmad@amd.com
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps
// following defines are added to instance the clock buffers and mark them for scan grouping
`define SCAN_GRP0 0
`define SCAN_GRP1 1
`define SCAN_GRP2 2
`define SCAN_GRP3 3
`define SCAN_GRP4 4
`define SCAN_GRP5 5
`define SCAN_GRP6 6
`define SCAN_GRP7 7
module cgtt_local_dcg(
   // Outputs
   dyn_oclk, dyn_oclk_vld,
   // Inputs
   resetb, iclk, cg_override, core_busy, soft_on_delay,
   soft_off_delay, soft_override, static_oclk
   );
//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cg_override;
 input       core_busy;
 input  [3:0] soft_on_delay;
 input [11:4] soft_off_delay;
 input        soft_override;
 input       static_oclk;
 output      dyn_oclk;
 output      dyn_oclk_vld;
//////////////////////////
reg  core_busy_d1;
reg  core_busy_d2;
//reg  [3:0] core_busy_filtered;  //4 copies of this flop for PD purpose.
reg  core_busy_filtered;
reg  core_busy_filtered_d1;
reg  core_busy_filtered_d2;
reg  [3:0] ON_DELAY;
reg [11:4] OFF_DELAY;
reg        SOFT_CORE_OVERRIDE;
reg  [3:0] on_counter;
reg [11:4] off_counter;
//
//leaf (always on) clock domain logic
//
wire [3:0] on_counter_nxt = (~core_busy_d2 && core_busy_d1 ) ?  ON_DELAY : //rising edge of busy. to turn on quickly, using busy w/o registering.
          ( on_counter == 4'd0 )           ?  on_counter : (on_counter - 4'd1);
wire [7:0] off_counter_nxt= (core_busy_d2 && ~core_busy_d1 ) ?  OFF_DELAY : //falling edge of busy
          ( off_counter == 8'd0 )          ?  off_counter : (off_counter - 8'd1);
always @(posedge static_oclk)
begin
  if(~resetb) begin
    ON_DELAY     <= 4'd0;
    OFF_DELAY    <= 8'd0;
    SOFT_CORE_OVERRIDE <= 1'b1;
    on_counter   <= 4'd0;
    off_counter  <= 8'd0;
    core_busy_d1 <= 1'b1;
    core_busy_d2 <= 1'b1;
    core_busy_filtered   <= 1'b1;
    core_busy_filtered_d1<= 1'b1;
    core_busy_filtered_d2<= 1'b1;
  end else begin
    ON_DELAY     <= soft_on_delay;
    OFF_DELAY    <= soft_off_delay;
    SOFT_CORE_OVERRIDE <= soft_override;
    core_busy_d1 <= core_busy;
    core_busy_d2 <= core_busy_d1;
    on_counter   <= on_counter_nxt;
    off_counter  <= off_counter_nxt;
    if((on_counter_nxt == 4'd0) && (off_counter_nxt == 8'd0)) begin
      core_busy_filtered <= core_busy_d1 || SOFT_CORE_OVERRIDE;
    end
    core_busy_filtered_d1 <= core_busy_filtered;
    core_busy_filtered_d2 <= core_busy_filtered_d1;
  end
end
//
//clock gates
//
wire oclk_vld = core_busy_filtered_d2 && core_busy_filtered_d1 && core_busy_filtered;
wire active = core_busy_filtered | cg_override;
`ifdef SIMULATION
//reg  oclk_vld_dly;
//always @(oclk_vld)
//begin
//  oclk_vld_dly = #2 oclk_vld;
//end
assign #2 dyn_oclk_vld = oclk_vld;
`else
assign dyn_oclk_vld = oclk_vld;
`endif
wire dyn_oclk_int;
wire dyn_oclk_gate;
`ifdef SIMULATION_WITH_x

assign dyn_oclk_int =  active ? dyn_oclk_gate : 1'bx;
`else
assign dyn_oclk_int = dyn_oclk_gate;
`endif
  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(1'b0),
   .DFT_TE(dft_te_wire)
  );
ati_clock_gate u_dyn_oclk(
    .clk_out(dyn_oclk_gate),
    .clk_src(iclk),
    .clk_en(active),
    .TE(dft_te_wire)
    );
// following code is added to instance the clock buffers and mark them for scan grouping
wire [`SCAN_GRP7:0] clks;
parameter scan_grp_dyn_oclk = `SCAN_GRP0; //defaulted
aticlk_buf scan_grp0_clk(.Y(clks[`SCAN_GRP0]), .A(dyn_oclk_int));
aticlk_buf scan_grp1_clk(.Y(clks[`SCAN_GRP1]), .A(dyn_oclk_int));
aticlk_buf scan_grp2_clk(.Y(clks[`SCAN_GRP2]), .A(dyn_oclk_int));
aticlk_buf scan_grp3_clk(.Y(clks[`SCAN_GRP3]), .A(dyn_oclk_int));
aticlk_buf scan_grp4_clk(.Y(clks[`SCAN_GRP4]), .A(dyn_oclk_int));
aticlk_buf scan_grp5_clk(.Y(clks[`SCAN_GRP5]), .A(dyn_oclk_int));
aticlk_buf scan_grp6_clk(.Y(clks[`SCAN_GRP6]), .A(dyn_oclk_int));
aticlk_buf scan_grp7_clk(.Y(clks[`SCAN_GRP7]), .A(dyn_oclk_int));
assign dyn_oclk = clks[scan_grp_dyn_oclk];
`ifdef SIMULATION
cgtt_local_dcg_cov ucgtt_local_dcg_cov
(
    .resetb                (resetb),
    .iclk                  (iclk),
    .cg_override           (cg_override),
    .core_busy             (core_busy),
    .soft_on_delay         (soft_on_delay),
    .soft_off_delay        (soft_off_delay),
    .soft_override         (soft_override),
    .static_oclk           (static_oclk),
    .dyn_oclk              (dyn_oclk),
    .dyn_oclk_vld          (dyn_oclk_vld),
    .core_busy_filtered    (core_busy_filtered)
);
`endif
endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
