//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//
//         If you find any issue or have any feedback, please email to sagheer.ahmad@amd.com
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps
// following defines are added to instance the clock buffers and mark them for scan grouping
// if we determine that no design block uses scan group other than "0" for register clocks then we can remove these.
`define SCAN_GRP0 0
`define SCAN_GRP1 1
`define SCAN_GRP2 2
`define SCAN_GRP3 3
`define SCAN_GRP4 4
`define SCAN_GRP5 5
`define SCAN_GRP6 6
`define SCAN_GRP7 7
module cgtt_local_rcg(
   // Outputs
   reg_oclk, reg_oclk_vld,
   // Inputs
   resetb, iclk, cg_override, reg_busy, soft_override, static_oclk
   );
//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cg_override;
 input       reg_busy;
 input       soft_override;
 input       static_oclk;
 output      reg_oclk;
 output      reg_oclk_vld;
//////////////////////////
reg  reg_busy_d1;
reg  reg_busy_d2;
reg  reg_busy_d3;
reg        SOFT_REG_OVERRIDE;
//
//leaf (always on) clock domain logic
//
always @(posedge static_oclk)
begin
  if(~resetb) begin
    SOFT_REG_OVERRIDE  <= 1'b1;
    reg_busy_d1  <= 1'b1;
    reg_busy_d2  <= 1'b1;
    reg_busy_d3  <= 1'b1;
  end else begin
    SOFT_REG_OVERRIDE  <= soft_override;
    reg_busy_d1  <= reg_busy || SOFT_REG_OVERRIDE;
    reg_busy_d2  <= reg_busy_d1;
    reg_busy_d3  <= reg_busy_d2;
  end
end
//
//clock gates
//
wire oclk_vld  = reg_busy_d3 && reg_busy_d2 && reg_busy_d1;
wire active = reg_busy_d1 | cg_override;
`ifdef SIMULATION
//reg  oclk_vld_dly;
//always @(oclk_vld)
//begin
//  oclk_vld_dly = #2 oclk_vld;
//end
assign #2 reg_oclk_vld = oclk_vld;
`else
assign reg_oclk_vld = oclk_vld;
`endif
wire reg_oclk_int;
wire reg_oclk_gate;
`ifdef SIMULATION_WITH_x

assign reg_oclk_int = active ? reg_oclk_gate : 1'bx;
`else
assign reg_oclk_int = reg_oclk_gate;
`endif
  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(1'b0),
   .DFT_TE(dft_te_wire)
  );
ati_clock_gate u_reg_oclk(
    .clk_out (reg_oclk_gate),
    .clk_src (iclk),
    .clk_en  (active),
    .TE      (dft_te_wire)
                   );
// following code is added to instance the clock buffers and mark them for scan grouping
wire [`SCAN_GRP7   :0] clks;
parameter scan_grp_reg_oclk = `SCAN_GRP0; //defaulted
aticlk_buf scan_grp0_clk(.Y(clks[`SCAN_GRP0]), .A(reg_oclk_int));
aticlk_buf scan_grp1_clk(.Y(clks[`SCAN_GRP1]), .A(reg_oclk_int));
aticlk_buf scan_grp2_clk(.Y(clks[`SCAN_GRP2]), .A(reg_oclk_int));
aticlk_buf scan_grp3_clk(.Y(clks[`SCAN_GRP3]), .A(reg_oclk_int));
aticlk_buf scan_grp4_clk(.Y(clks[`SCAN_GRP4]), .A(reg_oclk_int));
aticlk_buf scan_grp5_clk(.Y(clks[`SCAN_GRP5]), .A(reg_oclk_int));
aticlk_buf scan_grp6_clk(.Y(clks[`SCAN_GRP6]), .A(reg_oclk_int));
aticlk_buf scan_grp7_clk(.Y(clks[`SCAN_GRP7]), .A(reg_oclk_int));
assign reg_oclk = clks[scan_grp_reg_oclk];
`ifdef SIMULATION
cgtt_local_rcg_cov ucgtt_local_rcg_cov
(
    .resetb        (resetb),
    .iclk          (iclk),
    .cg_override   (cg_override),
    .reg_busy      (reg_busy),
    .soft_override (soft_override),
    .static_oclk   (static_oclk),
    .reg_oclk      (reg_oclk),
    .reg_oclk_vld  (reg_oclk_vld)
);
`endif
endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
