//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_9to1_lane_mux.v.dpl
// Description : Mux for 1 lane of debug client. This is combo logic only.
//               Used for both Synchronous and Source-synchronous Client
//
// NOTE: Thought is to place the byte mux with 9:1 data mux may allow
//       synthesys to better optimize
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Print out some input parameters to dperl --- begin
// MKEY          = 
// synchronicity = 
// DBG_WIDTH     = 
// DBG_IN_CNT    = 
// Print out some input parameters to dperl ----- end
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

module dbg_client_9to1_lane_mux
#(parameter LANE_COUNT = 4
,           LANE_WIDTH = 8
,           LANE_INDEX = 0
)

// ==========================================================================
// interface with dbg_client_ssb__steer
// Begin --------------------------------------------------------------------
( input       [8:0]            dbg_Mux_SelLane      // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
, output      [LANE_WIDTH-1:0] dbg_L2DataMuxLane    // Debug-bus data outputs after muxing for this lane
// End ----------------------------------------------------------------------

// ==========================================================================
// Debug-bus data inputs from functional logic
// Begin --------------------------------------------------------------------
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug63
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug7
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug6
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug5
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug4
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug3
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug2
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug1
, input       [LANE_COUNT*LANE_WIDTH-1:0] idebug0
// End ----------------------------------------------------------------------
);

  localparam BUS_WIDTH  = LANE_COUNT*LANE_WIDTH;

  reg [BUS_WIDTH -1:0]  dbg_L1DataMux;
  wire[LANE_WIDTH-1:0]  dbg_L1DataMuxDefault;
  reg [LANE_WIDTH-1:0]  reg_L2DataMuxLane;

  assign dbg_L2DataMuxLane[LANE_WIDTH-1:0] = reg_L2DataMuxLane[LANE_WIDTH-1:0];

//////////////////////////////////////////////////////////////////////////|
//                       <Debug-Bus Mux - Level 2>                      //|
//                   (Level 1 is outside of dbg_common)                 //|
//////////////////////////////////////////////////////////////////////////|
// Muxing for byte selection for each of the 3 24bit debug-bus inputs   //|
// This muxing is active only if Multi-Block mode is enabled.           //|
// In multiblock mode from each of the 3 input buses one byte (segment) //|
// is selected. This selection is independant for each input bus.       //|
// In SingleBlock mode (MultiBlock is not enables) all 3 bytes will be  //|
// selected from first input bus.                                       //|
// ----                                                                 //|
// to overload 2 bit Byte Select for Dbug-bus test purposes,
// to work with dbg_client_ssb__debug, in 3 lane case we set the default//|
// so if byte select is set to 2'b11 we pass the lane output            //|
//////////////////////////////////////////////////////////////////////////|
generate
if (LANE_COUNT>3) begin
   assign dbg_L1DataMuxDefault[LANE_WIDTH-1:0]= dbg_L1DataMux[(4*LANE_WIDTH)-1:(3*LANE_WIDTH)];
end else begin
   assign dbg_L1DataMuxDefault[LANE_WIDTH-1:0]= dbg_L1DataMux[((LANE_INDEX+1)*LANE_WIDTH)-1:(LANE_INDEX*LANE_WIDTH)];
end
endgenerate

always @(*) begin
   if (!dbg_Mux_SelLane[8])
      reg_L2DataMuxLane[LANE_WIDTH-1:0] = {LANE_WIDTH {1'b0}};
   else begin
      case (dbg_Mux_SelLane[7:6])
         2'b00  : reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMux[LANE_WIDTH-1:0];
         2'b01  : reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMux[(2*LANE_WIDTH)-1:LANE_WIDTH];
         2'b10  : reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMux[(3*LANE_WIDTH)-1:(2*LANE_WIDTH)];
         2'b11  : reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMuxDefault[LANE_WIDTH-1:0];
         default: reg_L2DataMuxLane[LANE_WIDTH-1:0] = dbg_L1DataMuxDefault[LANE_WIDTH-1:0];
      endcase // case(dbg_Mux_SelLane[7:6])
   end // else if (!dbg_Mux_SelLane[8])

//////////////////////////////////////////////////////////////////////////|
//                       <Debug-Bus Mux - Level 1>                      //|
//                      Debug-bus data muxing logic                     //|
//////////////////////////////////////////////////////////////////////////|
   case (dbg_Mux_SelLane[5:0])
     6'b000000 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug0[BUS_WIDTH-1:0];
     6'b000001 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug1[BUS_WIDTH-1:0];
     6'b000010 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug2[BUS_WIDTH-1:0];
     6'b000011 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug3[BUS_WIDTH-1:0];
     6'b000100 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug4[BUS_WIDTH-1:0];
     6'b000101 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug5[BUS_WIDTH-1:0];
     6'b000110 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug6[BUS_WIDTH-1:0];
     6'b000111 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug7[BUS_WIDTH-1:0];
     6'b111111 : dbg_L1DataMux[BUS_WIDTH-1:0]  = idebug63[BUS_WIDTH-1:0];
     default   : dbg_L1DataMux[BUS_WIDTH-1:0]  = {BUS_WIDTH {1'b0}};
   endcase // case(dbg_Mux_SelLane[5:0])

end // always @(*)
endmodule // dbg_client_9to1_lane_mux
