//                              -*- Mode: Verilog -*-
// (c) 2012, 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

module dbg_client_dbus_programmer
#(parameter USE_CLK_DIV   = 0  // Divided down clock should only be used when programming SSB debug bus
,           SSB_CLK_DIV   = 4  // Clock divide factor, only 4, or 8 is suported (verified)
,           ASYNC_RESET   = 0  // 1'b1 -> global_resetb is used as async reset term (NOTE: cdc waiver will only use 1'b0 setting)
,           IS_TESTMODULE = 0  // 1'b1 -> module used as test bench module, we want to avoid issues with false assertion trigger in cdce at reset
)
// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
( input              ipclk                    // system clock
, input              global_resetb            // Global synchronous reset, active low
// End ----------------------------------------------------------------------

// ==========================================================================
// Program source
//
// programVal        _____|---------------|_______
// programDat [31:0] xxxxx< valid program >xxxxxxx
// programBusy       _______|------------|________
// Begin --------------------------------------------------------------------
, input              programVal               // Valid bit of program data input
, input       [31:0] programDat               // Program data input, held static until programBusy deassert
, output             programBusy              // Return hand shake, programmer is busy
// End ----------------------------------------------------------------------

// ==========================================================================
// Serial programming output to Steering Logic
// Begin --------------------------------------------------------------------
, output             DBUS_TARG_daisychain_cclk// Daisy-chain program output, reverse of data flow
, output       [1:0] DBUS_TARG_daisychain_ctl // Daisy-chain program output, reverse of data flow
// End ----------------------------------------------------------------------
);


  // ==========================================================================
  // Program state of Main state machine (dbgRctl and dbgCctl)
  // Begin --------------------------------------------------------------------
  localparam       DBGC_RST_P0 = 2'b00; // Reset start point
  localparam       DBGC_RST_P1 = 2'b01; // Reset start point
  localparam       DBGC_RST_P2 = 2'b10; // Reset start point
  localparam       DBGC____RUN = 2'b11; // Reset start point

  localparam       DBGC_RSTPV0_H = {1'b1, 1'b1, 2'b00, 2'b00, 1'b1}; // Packet to flush the decoder
  localparam       DBGC_RSTPV1_H = {1'b1, 1'b1, 2'b00, 2'b11, 1'b1}; // C1 == C2 == 1'b1, ClkOffSettle == 2'b00, ProgSettle == 2'b11, Valid,
  localparam       DBGC_RSTPV2_H = {1'b0, 1'b0, 2'b00, 2'b00, 1'b1}; // C1 == C2 == 1'b0, ClkOffSettle == 2'b00, ProgSettle == 2'b00, Valid,
  localparam       DBGC_RSTPV0_T = {1'b1, 3'b111, {18 {1'b1}}}; // Packet to flush the decoder
  localparam       DBGC_RSTPV1_T = {1'b1, 3'b111, {18 {1'b1}}}; // (MuxId == 3'b111) & (BlkId == 8'hFF) to      pipe reset Data pipe
  localparam       DBGC_RSTPV2_T = {1'b1, 3'b000, {18 {1'b1}}}; // (MuxId == 3'b000) & (BlkId == 8'hFF) to STOP pipe reset Data pipe

  localparam       DBGC___DONE = 3'b000; // idel/Done
  localparam       DBGC_____C1 = 3'b001; // sent clk control (C1)
  localparam       DBGC___C1_D = 3'b010; // Clock off settle count
  localparam       DBGC__START = 3'b011; // Start pulse
  localparam       DBGC____T32 = 3'b100; // transmit 32 bit programming
  localparam       DBGC___C2_D = 3'b101; // Program settle count
  localparam       DBGC_____C2 = 3'b110; // sent clk control (C2)
  localparam       DBGC___WAIT = 3'b111; // Wait for programVal deassert & count
  // End ----------------------------------------------------------------------


  wire             clkg;
  wire             intResetb;
  reg [1:0]        dbgRctl;
  reg [1:0]        dbgRctlNxt;
  reg [2:0]        dbgCctl;
  reg [2:0]        dbgCctlNxt;
  wire             dbgRctl_is_Run;
  wire             dbgCctl_isDone;

  wire             clkEn;

  wire             cntDone;
  reg              cntEn;
  reg [7:0]        cntStartC1_D;
  reg [7:0]        cntStartC2_D;
  reg [7:0]        cntStart;
  reg [7:0]        cnt;
  reg [7:0]        cntNxt;

  reg [1:0]        ctl;
  reg [31:0]       progDatInt; // internal Program Data,  include reset override
  reg              progValInt; // internal Program Valid, include reset override
  reg              progBusy;
  reg              progBusyNxt;

  wire             prog_ValClkg;
  wire  [31:0]     progDataClkg;
  wire             progBusyClkg;
  wire             clkOffOk    ;
  // ==========================================================================
  // Clock gater
  // Begin --------------------------------------------------------------------
generate
   if (USE_CLK_DIV) begin :USEING_CLK_DIV

      wire [1:0]           targClkDiv;
      wire                 drSig;
      wire                 drSigb;
//    wire                 drClk;
      wire                 drClkb;
      wire                 drClkR;
      wire                 drClkF;
      wire                 drClkRR;
      wire                 clki;
      reg  [2:0]           rstDly;
      reg  [1:0]           progMinLow;
      reg                  programValD1;
      wire                 programValF;
      wire                 programValSlow;
      wire [1:0]           cdc_ctl;
      reg  [1:0]           ctlReg;

      assign prog_ValClkg       = programValSlow   ;
      assign progDataClkg[31:0] = programDat [31:0];
      assign programBusy        = progBusyClkg     ;
      assign progBusyClkg       = progBusy | !dbgRctl_is_Run;

      if (IS_TESTMODULE) begin :TEST_BENCH_MODULE
        assign  cdc_ctl[1:0] = ctl[1:0] & {2 {drSigb}};
         case (SSB_CLK_DIV)
           2 :  begin : SSB_CLK_DIV_IS_TBM2
             assign targClkDiv[1:0] = 2'b00; // div by 2
           end
           4 :  begin : SSB_CLK_DIV_IS_TBM4
             assign targClkDiv[1:0] = 2'b01; // div by 4
           end
           6 :  begin : SSB_CLK_DIV_IS_TBM6
             assign targClkDiv[1:0] = 2'b10; // div by 6
           end
           default :   begin : SSB_CLK_DIV_IS_TBM8
             assign targClkDiv[1:0] = 2'b11; // div by 8
           end
         endcase // case (SSB_CLK_DIV)
      end else begin :REAL_MODULE_CDC
         case (SSB_CLK_DIV)
           2 :  begin : SSB_CLK_DIV_IS_2
              // use cdcebug for timing anotation and sampling window
              techind_cdcefpm_icd #(.size (2), .applyParams(1), .DestClkCnt(1), .CdcDestSetup(0))
                mrk_cdc_ctl (.A(ctl[1:0]), .EN(1'b1), .DSTCLK(ipclk), .Z(cdc_ctl[1:0])
`ifdef ABV
                            , .InputChkDis(1'b1)
`endif                                                                          
                            );
             assign targClkDiv[1:0] = 2'b00; // div by 2
           end
           4 :  begin : SSB_CLK_DIV_IS_4
              // use cdcebug for timing anotation and sampling window
              techind_cdcefpm_icd #(.size (2), .applyParams(1), .DestClkCnt(2), .CdcDestSetup(0))
                mrk_cdc_ctl (.A(ctl[1:0]), .EN(1'b1), .DSTCLK(ipclk), .Z(cdc_ctl[1:0])
`ifdef ABV
                            , .InputChkDis(1'b1)
`endif                                                                          
                            );
             assign targClkDiv[1:0] = 2'b01; // div by 4
           end
           6 :  begin : SSB_CLK_DIV_IS_6
              // use cdcebug for timing anotation and sampling window
              techind_cdcefpm_icd #(.size (2), .applyParams(1), .DestClkCnt(3), .CdcDestSetup(0))
                mrk_cdc_ctl (.A(ctl[1:0]), .EN(1'b1), .DSTCLK(ipclk), .Z(cdc_ctl[1:0])
`ifdef ABV
                            , .InputChkDis(1'b1)
`endif                                                                          
                            );
             assign targClkDiv[1:0] = 2'b10; // div by 6
           end
           default :   begin : SSB_CLK_DIV_IS_8
              // use cdcebug for timing anotation and sampling window
              techind_cdcefpm_icd #(.size (2), .applyParams(1), .DestClkCnt(4), .CdcDestSetup(0))
                mrk_cdc_ctl (.A(ctl[1:0]), .EN(1'b1), .DSTCLK(ipclk), .Z(cdc_ctl[1:0])
`ifdef ABV
                            , .InputChkDis(1'b1)
`endif                                                                          
                            );
             assign targClkDiv[1:0] = 2'b11; // div by 8
           end
         endcase // case (SSB_CLK_DIV)
      end // block: REAL_MODULE_CDC

      assign  clkEn        = prog_ValClkg | !clkOffOk | ~intResetb | ~global_resetb;

      // internal reset must be delay to give some pulse before sending reset data
//    assign intResetb          = global_resetb    ;
      assign intResetb          = rstDly[2]        ;
      always @(posedge clki)
        if (~global_resetb)
          rstDly[2:0] <= {3 {1'b0}};
        else begin
           rstDly[1:0] <= (drClkR ? (rstDly[1:0] + 2'b01) : rstDly[1:0]) | {2 {rstDly[2]}};
           rstDly[2]   <= &rstDly[1:0];
        end
      //begin ---------------------------------------------------------------------------------------------
      // BOZO --- flatten this out because of module resolution problem
//    dbg_client_drclk I_drclk (.ick(ipclk), .resetb(global_resetb), .targClkDiv(targClkDiv),.drClkEn(clkEn), .drClk(drSig), .drClkR(drClkR), .drClkF(drClkF));

      reg  [2:0]  ickCnt    ;
      wire        ickCntClr ;
      reg         drClkReg  ;
      wire        drClkNxt  ;
      wire        drClkEnInt; // need to ensure the clock always stop low

      assign      drClkEnInt = clkEn | drClkReg;
      assign      drClkNxt   = drClkEnInt & ((ickCnt[1:0] == targClkDiv[1:0]) ? ~drClkReg : drClkReg);

      assign      ickCntClr      = (ickCnt[1:0] == targClkDiv[1:0]) | ~drClkEnInt;

      wire        drClkR_raw;
      wire        drClkF_raw;
      assign      drClkR_raw = {~drClkReg &  drClkNxt}; // detect rising  edge
      assign      drClkF_raw = { drClkReg & ~drClkNxt}; // detect falling edge

      assign drClkR =  drClkR_raw;
      assign drClkF =  drClkF_raw;

       always @(posedge clki)
         if (~global_resetb) begin
            ickCnt  [1:0] <= { 2 {1'b0}};
            drClkReg      <=      1'b0  ;
         end else begin
            ickCnt  [1:0] <= (ickCnt  [1:0] + 2'b01) & {2 {~ickCntClr}};
            drClkReg      <=  drClkNxt;
         end // else: !if(~resetb)

        // Delay the BR control by 1 cycle because we want edge on posedge
        // BOZO having timing issue with the ckgate design. Go with normal flop
        //dbg_client_ckgate I_drClk (.clkIn(ick), .clkEn_AR(drClkReg), .clkEn_BR(drClkReg), .clkOut(drClk));
        //aticlk_buf  d0nt_drClk (.Y(drClk), .A(drClkReg));
        assign drSig = drClkReg;
      //end -----------------------------------------------------------------------------------------------
      assign drSigb  = ~drSig;
      assign drClkRR = ~intResetb | drClkR; // send clock while reset is active to reset local logic
      // Clock gater ($STEM/import/rtllib/src/rtl/ati_clock_gate.v)
      ati_clock_gate I_gclk (.clk_out(clkg), .clk_src(ipclk), .clk_en(drClkRR), .TE(1'b0));
      ati_clock_gate I_iclk (.clk_out(clki), .clk_src(ipclk), .clk_en(clkEn  ), .TE(1'b0));
      ati_clk_inv d0nt_cclk (.clk_out(DBUS_TARG_daisychain_cclk), .clk_in(drClkb));
      ati_clk_inv d0nt_clkb (.clk_out(drClkb                   ), .clk_in(drSig )); // Create clock on this output (Y)

      // Slow down programVal ------------------- begin
      // Slow down programVal low so it can be sampled properly by dbgCctl
      // Since dbgCctl is slow down by divided clock
      assign programValF = programValD1 & ~programVal;
      always @(posedge clki)
        if (~intResetb) begin
           programValD1    <=     1'b0  ;
           progMinLow[1:0] <= {2 {1'b1}};
        end else begin
           programValD1  <= programVal;
           casez ({programValF, drClkF, progMinLow[1:0]})
             4'b1??? : progMinLow[1:0] <= {2 {1'b0}};
             4'b0100 : progMinLow[1:0] <=  2'b01;
             4'b0101 : progMinLow[1:0] <=  2'b11;
             default : progMinLow[1:0] <=  progMinLow[1:0];
           endcase // casez ({programValF, drClkF, progMinLow[1:0]})
        end
      assign programValSlow = &progMinLow[1:0] & programVal;
      // Slow down programVal ------------------- end

      // Use a b-phase flop (no reset on this)
      always @(posedge drClkb) ctlReg[1:0] <= cdc_ctl[1:0];
      assign DBUS_TARG_daisychain_ctl[1:0] = ctlReg[1:0];

   end else begin :NO_CLK_DIV // block: USEING_CLK_DIV
      // Every thing is single clock domain
      assign prog_ValClkg       = programVal       ;
      assign progDataClkg[31:0] = programDat [31:0];
      assign programBusy        = progBusyClkg     ;

      assign intResetb = global_resetb;
      assign DBUS_TARG_daisychain_cclk = ipclk;
      // Clock gater ($STEM/import/rtllib/src/rtl/ati_clock_gate.v)
      ati_clock_gate I_gcclk (.clk_out(clkg), .clk_src(ipclk), .clk_en(clkEn), .TE(1'b0));
      assign  clkEn        = prog_ValClkg | ~intResetb | !clkOffOk;
      assign  progBusyClkg = progBusy     | !dbgRctl_is_Run;

      assign DBUS_TARG_daisychain_ctl[1:0] = ctl[1:0];
   end // block: NO_CLK_DIV
endgenerate

  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Main state machine (dbgRctl and dbgCctl)
  // Begin --------------------------------------------------------------------
  reg  progValWas0;

  assign clkOffOk       = dbgRctl_is_Run & dbgCctl_isDone;
  assign dbgRctl_is_Run = dbgRctl[1:0] == DBGC____RUN;
  assign dbgCctl_isDone = dbgCctl[1:0] == DBGC___DONE;
  always @(*) begin
     case (dbgRctl[1:0])
       DBGC_RST_P0 : dbgRctlNxt[1:0] = progValInt ? dbgRctl[1:0] : DBGC_RST_P1;
       DBGC_RST_P1 : dbgRctlNxt[1:0] = progValInt ? dbgRctl[1:0] : DBGC_RST_P2;
       DBGC_RST_P2 : dbgRctlNxt[1:0] = progValInt ? dbgRctl[1:0] : DBGC____RUN;
       DBGC____RUN : dbgRctlNxt[1:0] = dbgRctl[1:0];
       default     : dbgRctlNxt[1:0] = dbgRctl[1:0];
     endcase // case (dbgRctl[1:0])

     case (dbgCctl[2:0])
       DBGC___DONE : dbgCctlNxt[2:0] = progValInt    ? DBGC_____C1 : dbgCctl[2:0];
       DBGC_____C1 : dbgCctlNxt[2:0] =                 DBGC___C1_D;
       DBGC___C1_D : dbgCctlNxt[2:0] = cntDone       ? DBGC__START : dbgCctl[2:0];
       DBGC__START : dbgCctlNxt[2:0] =                 DBGC____T32;
       DBGC____T32 : dbgCctlNxt[2:0] = cntDone       ? DBGC___C2_D : dbgCctl[2:0];
       DBGC___C2_D : dbgCctlNxt[2:0] = cntDone       ? DBGC_____C2 : dbgCctl[2:0];
       DBGC_____C2 : dbgCctlNxt[2:0] =                 DBGC___WAIT;
       DBGC___WAIT :
         casez ({dbgRctl_is_Run, progValInt, progValWas0, cntDone})
           4'b0??? : // Need to wait for propogation on last reset
	     case(dbgRctl[1:0])
	       DBGC_RST_P2 : dbgCctlNxt[2:0] = cntDone ? DBGC___DONE : dbgCctl[2:0];
	       default     : dbgCctlNxt[2:0] = DBGC___DONE;
	     endcase // case (dbgRctl[1:0])
           4'b111?,
           4'b1011 : dbgCctlNxt[2:0] = DBGC___DONE;
           default : dbgCctlNxt[2:0] = dbgCctl[2:0];
         endcase // casez ({progValInt, progValWas0, cntDone})
       default     : dbgCctlNxt[2:0] =                               dbgCctl[2:0];
     endcase // case(dbgCctl[2:0])
     case ({cntDone, dbgCctl[2:0]})
       {1'b0, DBGC___C1_D},
       {1'b0, DBGC____T32},
       {1'b0, DBGC___C2_D},
       {1'b0, DBGC___WAIT} : cntEn = 1'b1;
       default             : cntEn = 1'b0;
     endcase // case({cntDone, dbgCctl[2:0]})
  end

  // programBusy control
  always @(*)
    case (dbgCctl[2:0])
      DBGC___DONE ,
      DBGC_____C1 : progBusyNxt  =  clkEn;
      DBGC_____C2 ,
      DBGC___WAIT : progBusyNxt  =  1'b0;
      default     : progBusyNxt  =  progBusy;
    endcase // case (dbgCctl[2:0])
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Override programming interface so we can write out reset packets
  // Begin --------------------------------------------------------------------
  always @(*) begin
     case (dbgRctl[1:0])
       DBGC_RST_P0 : begin progDatInt[31:25] = DBGC_RSTPV0_H; progDatInt[24:22] = progDataClkg[24:22]; progDatInt[21:0] = DBGC_RSTPV0_T; end
       DBGC_RST_P1 : begin progDatInt[31:25] = DBGC_RSTPV1_H; progDatInt[24:22] = progDataClkg[24:22]; progDatInt[21:0] = DBGC_RSTPV1_T; end
       DBGC_RST_P2 : begin progDatInt[31:25] = DBGC_RSTPV2_H; progDatInt[24:22] = progDataClkg[24:22]; progDatInt[21:0] = DBGC_RSTPV2_T; end
       DBGC____RUN : progDatInt[31:0] = progDataClkg[31:0];
       default     : progDatInt[31:0] = progDataClkg[31:0];
     endcase // case (dbgRctl[1:0])
     case (dbgRctl[1:0])
       DBGC_RST_P0 ,
       DBGC_RST_P1 ,
       DBGC_RST_P2 : progValInt = dbgCctl[2:0] != DBGC___WAIT;
       DBGC____RUN : progValInt = prog_ValClkg;
       default     : progValInt = prog_ValClkg;
     endcase // case (dbgRctl[1:0])
  end // always @ begin
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // 8 bit Counter
  // Begin --------------------------------------------------------------------

  assign cntDone = cnt[7:0] == {8 {1'b0}};

  always @(*) begin
     case (progDatInt[29:28])
       2'b00   : cntStartC1_D[7:0] = {8 {1'b0}};
       2'b01   : cntStartC1_D[7:0] = 8'h03;
       2'b10   : cntStartC1_D[7:0] = 8'h0F;
       2'b11   : cntStartC1_D[7:0] = 8'h3F;
       default : cntStartC1_D[7:0] = {8 {1'b0}};
     endcase
     case (progDatInt[27:26])
       2'b00   : cntStartC2_D[7:0] = {8 {1'b0}};
       2'b01   : cntStartC2_D[7:0] = 8'h03;
       2'b10   : cntStartC2_D[7:0] = 8'h0F;
       2'b11   : cntStartC2_D[7:0] = 8'h3F;
       default : cntStartC2_D[7:0] = {8 {1'b0}};
     endcase
     case (dbgCctlNxt[2:0])
       DBGC___C1_D : cntStart[7:0] = cntStartC1_D[7:0];
       DBGC____T32 : cntStart[7:0] = 8'h1F;
       DBGC___C2_D : cntStart[7:0] = cntStartC2_D[7:0];
       DBGC___WAIT : cntStart[7:0] = 8'hFF; // Clock 256 time to propogate programming
       default     : cntStart[7:0] = {8 {1'b0}};
     endcase // case(dbgCctl[2:0])
     cntNxt[7:0] = cntEn ? cnt[7:0] - 8'h01 : cntStart[7:0];
  end
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Programming out
  // Begin --------------------------------------------------------------------
  wire ctl_1;
  reg [1:0] ctlNxt;

  assign ctl_1 = progDatInt[cnt[4:0]] & progDatInt[25]; // progDatInt[25] is valid bit

  always  @(*) begin
     case (dbgCctl[2:0])
       DBGC_____C1 : ctlNxt[1] = progDatInt[31];
       DBGC__START : ctlNxt[1] = progDatInt[25]; // progDatInt[25] is valid bit - don't send program start if not valid
       DBGC____T32 : ctlNxt[1] = ctl_1;
       DBGC_____C2 : ctlNxt[1] = progDatInt[30];
       default     : ctlNxt[1] = 1'b0;
     endcase // case(dbgCctl[2:0])
     case (dbgCctl[2:0])
       DBGC_____C1 : ctlNxt[0]  =  1'b1;
       DBGC___DONE ,
       DBGC___WAIT : ctlNxt[0]  =  1'b0;
       default     : ctlNxt[0]  =  ctl[0];
     endcase // casez(dbgCctl[2:0])
  end
  // End ----------------------------------------------------------------------

generate
   if (ASYNC_RESET) begin :ASYNC_RESET0
      always @(posedge clkg or negedge intResetb)
        if (~intResetb) begin
           dbgRctl[1:0] <= DBGC_RST_P0;
           dbgCctl[2:0] <= DBGC___DONE;
           progBusy     <=     1'b1   ;
           cnt[7:0]     <= {8 {1'b0}} ;
           ctl[1:0]     <= {2 {1'b0}} ;
           progValWas0  <=     1'b0   ;
        end else begin
           dbgRctl[1:0] <= dbgRctlNxt[1:0];
           dbgCctl[2:0] <= dbgCctlNxt[2:0];
           progBusy     <= progBusyNxt;
           cnt[7:0]     <= cntNxt[7:0];
           ctl[1:0]     <= ctlNxt[1:0];
           progValWas0  <= (dbgCctl[2:0] == DBGC___WAIT) & (progValWas0 | ~progValInt);
        end
   end else begin :SYNC_RESET0
      always @(posedge clkg)
        if (~intResetb) begin
           dbgRctl[1:0] <= DBGC_RST_P0;
           dbgCctl[2:0] <= DBGC___DONE;
           progBusy     <=     1'b1   ;
           cnt[7:0]     <= {8 {1'b0}} ;
           ctl[1:0]     <= {2 {1'b0}} ;
           progValWas0  <=     1'b0   ;
        end else begin
           dbgRctl[1:0] <= dbgRctlNxt[1:0];
           dbgCctl[2:0] <= dbgCctlNxt[2:0];
           progBusy     <= progBusyNxt;
           cnt[7:0]     <= cntNxt[7:0];
           ctl[1:0]     <= ctlNxt[1:0];
           progValWas0  <= (dbgCctl[2:0] == DBGC___WAIT) & (progValWas0 | ~progValInt);
        end
   end // ! if (ASYNC_RESET) ... :ASYNC_RESET0
endgenerate

endmodule // dbg_client_dbus_programmer

