//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_lib.v
// Description : Mux for synchronous debug bus client.
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////
module dbg_client_dff_async_reset (/*AUTOARG*/
   // Outputs
   q,
   // Inputs
   clk, rstb, d
   );
   parameter size = 1;
   parameter FLOPPED = 0;
   input             clk;
   input             rstb;
   input  [size-1:0] d;
   output [size-1:0] q;

   generate
      if (FLOPPED) begin
         reg [size-1:0]    q_int;

         always @(posedge clk or negedge rstb)
           if (~rstb)
             q_int[size-1:0] <= { size {1'b0}};
           else
             q_int[size-1:0] <= d[size-1:0];

         assign q[size-1:0] = q_int[size-1:0];
      end else begin // if (FLOPPED)
         assign q[size-1:0] = d[size-1:0];
         //! unused clk
         //! unused rstb
      end // else: !if(FLOPPED)

   endgenerate

endmodule // dbg_client_dff_async_reset

module dbg_client_mrk_mux2 (/*AUTOARG*/
   // Outputs
   z,
   // Inputs
   s, i0, i1
   );
   parameter size = 1;
   input              s;
   input  [size-1:0] i0;
   input  [size-1:0] i1;
   output [size-1:0]  z;

   // river translate_off
   generate
   genvar i;
       for(i=0; i < size; i = i+1) begin : mux_i
          ati_mux2 I_dbg_client_mrk_mux2 (.z(z[i]), .s(s), .i0(i0[i]), .i1(i1[i]));
       end
   endgenerate
   // river translate_on
endmodule // dbg_client_mrk_mux2

module dbg_client_ssb__mux_laneEn
( input           dbg_Mux_SelLane_LaneEnable
, input           TEST_CLK_EN
, output          laneEn
);

  dbg_client_mrk_or d0nt_laneEn (.Y(laneEn), .A(dbg_Mux_SelLane_LaneEnable), .B(TEST_CLK_EN));

endmodule // dbg_client_ssb__mux_laneEn

module dbg_client_clkmux_2to1
#( parameter MUX_USED = 2'b00 )
( output       clk_o
, input  [1:0] clk_i
, input        sel
);

generate
   case (MUX_USED)
     2'b01 : begin : CLKMUX_2TO1_01
        ati_clk_and d0nt_clkmux_2to1_01 (.A1(clk_i[0]), .A2(~sel), .Z(clk_o));
        //!  unused clk_i[1]
     end // case MUX_USED = 2'b01
     2'b10 : begin : CLKMUX_2TO1_10
        ati_clk_and d0nt_clkmux_2to1_10 (.A1(clk_i[1]), .A2( sel), .Z(clk_o));
       //!  unused clk_i[0]
     end // case MUX_USED = 2'b01
     2'b11 : begin : CLKMUX_2TO1_11
        aticlk_mux2 d0nt_clkmux (.clk_o(clk_o), .clk_i(clk_i[1:0]), .sel  (sel));
     end // case MUX_USED = 2'b11
     default : begin : CLKMUX_2TO1_00
       assign clk_o = 1'b0;
       //! unused clk_i
       //! unused sel
     end // case MUX_USED = 2'b00
   endcase // case(MUX_USED)
endgenerate
endmodule // dbg_client_clkmux_2to1

module dbg_client_datmux_2to1
#( parameter MUX_USED = 2'b00
 , parameter WIDTH    = 8
 )
( input  [WIDTH-1:0] dat_i0
, input  [WIDTH-1:0] dat_i1
, input              sel
, output [WIDTH-1:0] dat_o
);

generate
  genvar ii;
  for (ii=0; ii < WIDTH; ii = ii +1) begin
    case (MUX_USED)
      2'b01 : begin : CLKMUX_2TO1_01
        dbg_client_mrk_and d0nt_datmux_2to1_01 (.A(dat_i0[ii]), .B(~sel), .Y(dat_o[ii]));
        //!  unused dat_i1
      end
      2'b10 : begin : CLKMUX_2TO1_10
        dbg_client_mrk_and d0nt_datmux_2to1_10 (.A(dat_i1[ii]), .B( sel), .Y(dat_o[ii]));
        //!  unused dat_i0
      end
      2'b11 : begin : CLKMUX_2TO1_11
        ati_mux2 d0nt_datmux (.i0(dat_i0[ii]), .i1(dat_i1[ii]), .s(sel), .z(dat_o[ii]));
      end
      default : begin : CLKMUX_2TO1_00
        assign dat_o = {WIDTH{1'b0}};
        //! unused dat_i0
        //! unused dat_i1
        //! unused sel
      end
    endcase
  end
endgenerate
endmodule // dbg_client_datmux_2to1

// This is a copy of dfx_ckgate fomr dfxip4_0
// /sysip/dfx/dfx-4/4.0/main/dftmisc/src/rtl/dfx_ckgate.v
module dbg_client_ckgate
( input  clkIn     // Input clock
, input  clkEn_AR  // Clock gating term of A-phase portion of pulse, 1'b1 -> corresponding output high
, input  clkEn_BR  // Clock gating term of B-phase portion of pulse, 1'b1 -> corresponding output high
, output clkOut    // Output clock
);

  wire    clkInA;
  wire    clkInB;
  assign  clkInA = clkIn;
  ati_clk_inv d0nt_clkInB (.clk_in(clkInA), .clk_out(clkInB));

  reg     clkEnA;
  reg     clkEnB;
  reg     clkEnBB;

  always @(posedge clkInA) begin
     clkEnA  <= clkEn_AR;
     clkEnBB <= clkEn_BR;
  end

  // Use the b-phase to avoid hols time issues with clock gater latch
  always @(posedge clkInB) begin
     clkEnB <= clkEnBB;
  end

  wire clkOutA;
  wire clkOutA_Inv;
  wire clkOutB;
  wire clkOutB_Inv;

  ati_clock_gate d0nt_clkOutA (.clk_src(clkInA), .clk_en(clkEnA), .TE(1'b0), .clk_out(clkOutA)); // Half cycle to this gater
  ati_clock_gate d0nt_clkOutB (.clk_src(clkInB), .clk_en(clkEnB), .TE(1'b0), .clk_out(clkOutB)); // Half cycle to this gater

  // There are no clock OR in rtllib, so compose one
  // Need to have create_generated_clock on d0nt_clkOut.ZN
  ati_clk_inv  d0nt_clkOutA_Inv (.clk_in(clkOutA), .clk_out(clkOutA_Inv));
  ati_clk_inv  d0nt_clkOutB_Inv (.clk_in(clkOutB), .clk_out(clkOutB_Inv));
  ati_clk_nand d0nt_clkOut      (.A1(clkOutA_Inv), .A2(clkOutB_Inv), .ZN(clkOut));

endmodule // dbg_client_ckgate

// This is a modify copy of dfx_ckgate from dfxip4_0
// /sysip/dfx/dfx-4/4.0/main/fuse_if/src/rtl/fuse_if_drclk.v

module dbg_client_drclk
#( parameter SIZE = 2 )

( input            ick         // input clock
, input            resetb      // input synchronous reset
, input [SIZE-1:0] targClkDiv  // input target clock divide value (50% of cycle)
, input            drClkEn     // input enable output clock (drClk)
, output           drClk       // output clock
, output           drClkR      // output marker in ick, marking drClk rising  edge
, output           drClkF      // output marker in ick, marking drClk falling edge
);

   localparam       DELAY  = 0; // Set delay to 1 cycle ahead of the edge // DELAY == 2 if using dbg_client_ckgate

   reg  [SIZE-1:0]  ickCnt    ;
   wire             ickCntClr ;
   reg              drClkReg  ;
   wire             drClkNxt  ;
   wire             drClkEnInt; // need to ensure the clock always stop low

   assign           drClkEnInt = drClkEn | drClkReg;
   assign           drClkNxt   = drClkEnInt & ((ickCnt[SIZE-1:0] == targClkDiv[SIZE-1:0]) ? ~drClkReg : drClkReg);

   assign           ickCntClr      = (ickCnt[SIZE-1:0] == targClkDiv[SIZE-1:0]) | ~drClkEnInt;

   wire             drClkR_raw;
   wire             drClkF_raw;
   assign           drClkR_raw = {~drClkReg &  drClkNxt}; // detect rising  edge
   assign           drClkF_raw = { drClkReg & ~drClkNxt}; // detect falling edge

   reg [DELAY:0]    rising_d  ; // Delay marker because latency through dfx_ckgate 3
   reg [DELAY:0]    fallingd  ; // Delay marker because latency through dfx_ckgate 3

generate
   case (DELAY)
     0 : begin :DELAY_EQ0
        assign drClkR =  drClkR_raw;
        assign drClkF =  drClkF_raw;
        //! unused rising_d
        //! unused fallingd
     end
     1 : begin :DELAY_EQ1
        assign drClkR =  rising_d[DELAY];
        assign drClkF =  fallingd[DELAY];

        always @(posedge ick)
          if (~resetb) begin
             rising_d[1] <= 1'b0;
             fallingd[1] <= 1'b0;
          end else begin
             rising_d[1] <= drClkR_raw;
             fallingd[1] <= drClkF_raw;
          end // else: !if(~resetb)
     end
     default : begin :DELAY_G1
        assign drClkR =  rising_d[DELAY];
        assign drClkF =  fallingd[DELAY];

        always @(posedge ick)
          if (~resetb) begin
             rising_d[DELAY :1] <= { DELAY {1'b0}};
             fallingd[DELAY :1] <= { DELAY {1'b0}};
          end else begin
             rising_d[DELAY :1] <= {rising_d[DELAY-1:1], drClkR_raw};
             fallingd[DELAY :1] <= {fallingd[DELAY-1:1], drClkF_raw};
          end // else: !if(~resetb)
     end // block: DELAY_G1
   endcase // case (DELAY)
endgenerate

  always @(posedge ick)
    if (~resetb) begin
       ickCnt  [SIZE-1:0] <= { SIZE  {1'b0}};
       drClkReg           <=          1'b0  ;
    end else begin
       ickCnt  [SIZE-1:0] <= (ickCnt  [SIZE-1 :0] + {{SIZE-1 {1'b0}}, 1'b1}) & {SIZE {~ickCntClr}};
       drClkReg           <=  drClkNxt;
    end // else: !if(~resetb)

   // Delay the BR control by 1 cycle because we want edge on posedge
   // BOZO having timing issue with the ckgate desgin. Go with normal flop
   //dbg_client_ckgate I_drClk (.clkIn(ick), .clkEn_AR(drClkReg), .clkEn_BR(drClkReg), .clkOut(drClk));
   aticlk_buf  d0nt_cclk (.Y(drClk), .A(drClkReg));

endmodule // dbg_client_drclk

//// Copy this from rtllib/src/rtl/variant/gf14lpe/sync3_wrapper_flush.v
//// rtllib/rtllib_soc15@2057390
//module dbg_client_sync3_flush (/*AUTOARG*/
//   // Outputs
//   Q,
//   // Inputs
//   D, CK, FLSH
//   );
//parameter WIDTH = 1;
//parameter transport_cycle = 1;
//parameter transport_delay = 0;
//
//input D;
//input CK;
//input FLSH;
//output Q;
//
//    wire ZERO=1'b0;
//    wire ONE=1'b1;
//
//`ifdef HWEMUL
//  // HWEMUL ---> Hardwae (FPGA) emulation, replace sync with flop
//  reg [2:0] reg_hwemul;
//  always @(posedge CK or posedge FLSH)
//    if (FLSH)
//      reg_hwemul[2:0] <= {3 {D}};
//    else
//      reg_hwemul[2:0] <= {reg_hwemul[1:0], D};
//  assign    Q = reg_hwemul[2];
//`else
//  // not in HWEMUL
//    hdsync3flshmsfqxss1us_rand
//      #( .WIDTH(WIDTH)
//        ,.transport_cycle(transport_cycle)
//        ,.transport_delay(transport_delay)
//       ) sync_r (
//`ifdef ABV
//`ifdef DBGC_EnableIPWC
//        .InputChkDis(ZERO),
//`else
//        .InputChkDis(ONE),
//`endif
//`endif
//        .CLK(CK),
//        .D(D),
//        .FLSH(FLSH),
//        .SE(ZERO),
//        .SI(ZERO),
//        .Q(Q)
//       );
//`endif // `ifdef HWEMUL
//
//endmodule // dbg_client_sync3_flush
