//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_ssb__aon_s2p.v
// Description : Serial to parallel conversion for mux configuration
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////

module dbg_client_ssb__aon_s2p
// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
( output          conf_clk                  // control logic clock
, input           TARG_DBUS_daisychain_cclk // Daisy-chain program input,  source-synchronous clock for config
, output          DBUS_TARG_daisychain_cclk // Daisy-chain program output, source-synchronous clock for config
, input      [7:0]DbgBlkId                  // Debug client ID - hard tie
, input           AEB_DBUS                  // Security lock down. 1'b0-> hold program to reset == data forward
// End ----------------------------------------------------------------------

// ==========================================================================
// Debug bus serial control input
// Control data flows from DSM out to debug bus chain
// Begin --------------------------------------------------------------------
, output      [1:0]DBUS_TARG_daisychain_ctl  // Daisy-chain program output, reverse of data flow
, input       [1:0]TARG_DBUS_daisychain_ctl  // Daisy-chain program input,  reverse of data flow
// End ----------------------------------------------------------------------

// ==========================================================================
// Data decoded from daisychain_ctl_d1 (serial stream control config)
// output to all syn_dbg_client_ctl instance
// Begin --------------------------------------------------------------------
, output           ctlProgClrDat             // Clear the data Pipe
, output           ctlProgClrCtl             // Clear the Control logic data
, output           ctlProgVal                // control program : control valid
, output           ctlProg_ClkOn             // control program : Clock on / off;
, output           ctlProg_ClkEnSel          // control program : control program : Clock enable select
, output      [1:0]ctlProg_ByteSel           // control program : bit Byte select // removed [2]Multi-block mode
, output      [2:0]ctlProg_MuxId             // control program : Mux ID of the current program target
, output      [1:0]ctlProg_LaneSel           // control program : Lane select of mux
, output      [5:0]ctlProg_ChannelSel        // control program : Channel select of mux
// End ----------------------------------------------------------------------
);

  // ==========================================================================
  // Program state of Main state machine (cntSt)
  // Begin --------------------------------------------------------------------
  localparam       DBGS2P__DONE = 2'b00; // idel/Done
  localparam       DBGS2P_SHIFT = 2'b01; // Shift in data
  localparam       DBGS2P_STORE = 2'b11; // Store shifted in data
  localparam       DBGS2P__WAIT = 2'b10; // Wait for 2nd excludeCtlC1_EndMrk
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Path on source-synchronous timing
  // Begin --------------------------------------------------------------------
  // Clock marker buffer
  wire            udbus_fwd_cclk;
  assign          conf_clk = udbus_fwd_cclk;

  aticlk_buf  d0nt_udbus_fwd_cclk                 (.A(TARG_DBUS_daisychain_cclk), .Y(udbus_fwd_cclk));            // NOTE: this is a marker point for create_generated_clock in timing sdc
  ati_clk_inv d0nt__inv_DBUS_TARG_daisychain_cclk (.clk_in(udbus_fwd_cclk), .clk_out(DBUS_TARG_daisychain_cclk)); // NOTE: Marker point to do 'set_clock_sense -stop_propagation ...' in timing sdc

  // These flops are on source-synchronous path
  reg [1:0]       daisychain_ctl_d1; // Only output can be used
  reg [1:0]       daisychain_ctl_d2; // Only input can be used for logic

  assign          DBUS_TARG_daisychain_ctl[1:0] = daisychain_ctl_d2[1:0];
  always @(posedge TARG_DBUS_daisychain_cclk) daisychain_ctl_d1[1:0] <= TARG_DBUS_daisychain_ctl[1:0];
  always @(posedge udbus_fwd_cclk           ) daisychain_ctl_d2[1:0] <= daisychain_ctl_d1[1:0];
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Decode on / off encoding
  // Begin --------------------------------------------------------------------
  reg [1:0]       ctl_d2;
  reg             ClkOn;

  assign          ctlProg_ClkOn = ClkOn & ~ctlProgClrDat; // Bug 645981 - clean up assertion errors during debug client reset

  always @(posedge conf_clk) begin
     ctl_d2[1:0] <= daisychain_ctl_d1[1:0];

     case ({ctl_d2[0], daisychain_ctl_d1[0]})
       2'b01   : ClkOn <= daisychain_ctl_d1[1]; // Rising  edge, sample d1 data
       2'b10   : ClkOn <= ctl_d2[1]           ; // Falling edge, sample input
       default : ClkOn <= ClkOn               ; // This is don't care, but assign a value
     endcase // case({ctl_d2[0], daisychain_ctl_d1[0]})
  end

  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Filter out on / off portion of control stream
  // Begin --------------------------------------------------------------------
  wire            excludeCtlC1_EndMrk;
  wire            dbg_ctlVal;
  wire            dbg_ctl;
  assign          excludeCtlC1_EndMrk = daisychain_ctl_d1[0] ^ ctl_d2[0];
  assign          dbg_ctlVal          = daisychain_ctl_d1[0] & ~excludeCtlC1_EndMrk;
  assign          dbg_ctl             = daisychain_ctl_d1[1];

  // End ----------------------------------------------------------------------

  // ==========================================================================
  // BOZO: This section need to use header file !!!!
  // Serial to Parallel conversion
  // Control data shift in MSB first, keep last 22 bits
  // [   21] ClkEnSel                (ctlProg_ClkEnSel       )
  // [20:18] MuxId                   (ctlProg_MuxId[2:0]     )
  // [17:16] Lane Select             (ctlProg_LaneSel[1:0]   )
  // [15:14] bit Byte select         (ctlProg_ByteSel[1:0]   )
  // [13: 8] Channel Select          (ctlProg_ChannelSel[5:0])
  // [ 7: 0] debug client BlkId[7:0]
  // NOTE:
  //  1) Multi-block mode is defeatured
  // Begin --------------------------------------------------------------------
  reg  [4:0]             cnt       ; // counter to find end of 32 bit data
  reg  [1:0]             cntSt     ; // counterenable
  reg  [1:0]             cntStNxt  ;
  wire                   cntAtMax  ;
  wire                   SampleData;
  wire                   cntEn     ;
  wire                   ClrProgClrDatC;
  wire                   ClrCntSt;

  assign                 cntAtMax = &cnt[4:0];

  reg [21:0]             ctlDat;        // Control data shift register
  reg                    ClrDatPipe;    // Clear data Pipe

  // Clear Control data if either
  // 1) DBGBLKID == 8'hFF
  // 2) ctlProg_MuxId[2:0] == 3'b111
  // 3) AEB_DBUS == 1'b0

  // Clear Data Pipe if (DBGBLKID == 8'hFF) & (ctlProg_MuxId[2:0] == 3'b111)

  wire                   DbgBlkId_Is_FF;
  wire                   MuxId_Is_7;
  assign                 DbgBlkId_Is_FF = &ctlDat[7:0];
  assign                 MuxId_Is_7     = &ctlProg_MuxId[2:0];

  // Exclude AEB_DBUS term from mux modules
  //
  assign                 ctlProgClrDat  = ClrDatPipe;
  assign                 ClrProgClrDatC = (SampleData & DbgBlkId_Is_FF ) |
                                          (ctlProgVal & MuxId_Is_7     );

  assign                 ctlProgClrCtl = ~AEB_DBUS | ClrProgClrDatC;
  assign                 ClrCntSt      = excludeCtlC1_EndMrk & ~daisychain_ctl_d1[0] | ClrProgClrDatC;
  assign                 cntEn         = (cntSt[1:0] == DBGS2P_SHIFT) & (cntStNxt[1:0] != DBGS2P__DONE);

  always @(*)
    if (dbg_ctlVal)
      case (cntSt[1:0])
        DBGS2P__DONE : cntStNxt[1:0] = dbg_ctl             ? DBGS2P_SHIFT : DBGS2P__DONE;
        DBGS2P_SHIFT : cntStNxt[1:0] = cntAtMax            ? DBGS2P_STORE : DBGS2P_SHIFT;
        DBGS2P_STORE : cntStNxt[1:0] =                       DBGS2P__WAIT               ;
        DBGS2P__WAIT : cntStNxt[1:0] = excludeCtlC1_EndMrk ? DBGS2P__DONE : DBGS2P__WAIT;
        default : cntStNxt[1:0] = DBGS2P__DONE;
      endcase // case (cntSt[1:0])
    else
      // Force cntSt to DBGS2P__DONE to exclude C2 term
      cntStNxt[1:0] = DBGS2P__DONE;

  always @(posedge conf_clk or negedge AEB_DBUS)
    if (~AEB_DBUS) begin
       cnt   [ 4:0] <= { 5{1'b0}};
       ctlDat[21:0] <= {22{1'b0}};
       cntSt [ 1:0] <= DBGS2P__DONE;
    end else begin
       cnt   [ 4:0] <=  (cntEn & ~ClrProgClrDatC) ? (cnt[4:0] + 5'b00001)   : {5 {1'b0}};
       ctlDat[21:0] <= ((cntSt[1:0] == DBGS2P_SHIFT) ? {ctlDat[20:0], dbg_ctl} : ctlDat[21:0]) & {22 {~ClrProgClrDatC}};
       cntSt [ 1:0] <=   ClrCntSt? DBGS2P__DONE : cntStNxt[1:0];
    end

  always @(posedge conf_clk) ClrDatPipe <= SampleData ? DbgBlkId_Is_FF & MuxId_Is_7 : ClrDatPipe;

  assign SampleData              =  cntSt[1:0] == DBGS2P_STORE;
  assign ctlProg_ClkEnSel        =  ctlDat[   21];
  assign ctlProg_MuxId     [2:0] =  ctlDat[20:18];
  assign ctlProg_LaneSel   [1:0] =  ctlDat[17:16];
  assign ctlProg_ByteSel   [1:0] =  ctlDat[15:14];
  assign ctlProg_ChannelSel[5:0] =  ctlDat[13: 8];
  assign ctlProgVal              =((ctlDat[ 7: 0] == DbgBlkId[7:0]) | DbgBlkId_Is_FF)& SampleData;
  // End ----------------------------------------------------------------------
endmodule // syn_dbg_client_aon_s2p
