//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_ssb__debug.v
// Description : Module that connects to 1 dbg_client_ssb__mux
//               This module is designed for debugging the debug bus
//               (revision #6 -  12 flop + 1 latch)
////////////////////////////////////////////////////////////////////////////
module dbg_client_ssb__debug
#(parameter LANE_COUNT = 4
,           LANE_WIDTH = 8
)

( input       clk            // clock to use for this module
, input [7:0] dbg_BlkId      // Debug bus ID used for debug the debug bus client

 // ==========================================================================
 // interface with dbg_client_ssb__mux
 // [7:6] Byte Select overloaded with debug output
 // 2'b00 -  iDbgBlkId[7:0] - Debug bus ID
 // 2'b01 - ~iDbgBlkId[7:0] - Inverted Debug bus ID
 // 2'b10 - 4 bit counter
 // 2'b11 - 6 bit counter - sudo counter design to test signal integrity
 // NOTE: For 24 bit wide bus, Byte select of 2'b11 will result in 6 bit
 //       counter output to the 8 bits of the corresponding lane
 // Begin --------------------------------------------------------------------
, input  [(LANE_COUNT*9-1)          :0] dbg_Mux_SelLane   // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
, output [(LANE_COUNT*LANE_WIDTH-1) :0] idebug            // Debug-bus data output. To be connected to a
 // End ----------------------------------------------------------------------

, input                                 ctlProg_ClkOn     // Async reset and clear data
);

//////////////////////////////////////////////////////////////////////////|
//                       <Debug-Bus Mux - Level 1>                      //|
//                      Debug-bus data muxing logic                     //|
//////////////////////////////////////////////////////////////////////////|

  // Get every bit to toggle every cycle to look for signal integrety issues
  // 4 bit counter
  wire [7:0]           dbgSig_4CntO;

  reg  [7:0]           dbgSig_6CntO;
  wire [3:0]           dbgSig_6CntOi;
  reg  [5:0]           dbgSig_6Cnt;

  assign               dbgSig_4CntO[7:0] = {dbgSig_6Cnt[3:0], dbgSig_6Cnt[3:0]};
  assign               dbgSig_6CntOi[3:0] = ~dbgSig_6Cnt[4:1];

  wire                 ctlProg_ClkOn_sync;
  wire 	               gclk;
  reg   [1:0]          ctlProg_ClkOn_sync_d;
  wire                 clr6cnt;
  wire  	       clkEn;

  techind_sync_icd #(.size(1), .depth(3), .TransportDelay(0), .TransportCycle(2), .asyncReset(1))
    I_ctlProg_ClkOn_sync ( .SB(1'b1), .FLUSH(1'b0), .CKsrc(1'b0), .EN(1'b1)
                       , .RB( ctlProg_ClkOn )
                       , .CKdst( clk )
                       , .Dsrc( ctlProg_ClkOn )
                       , .Qsrc(), .Qdst( ctlProg_ClkOn_sync )
                       , .SC1(1'b0), .SC2(1'b0), .SE(1'b0), .SI(1'b0), .SO()
  `ifdef ABV
                       , .InputChkDis(1'b1)
  `endif
                       );

  // Delay the start of synchronizer so Steer can catch the start
  always @(posedge clk) ctlProg_ClkOn_sync_d[1:0] <= {ctlProg_ClkOn_sync_d[0], ctlProg_ClkOn_sync};

  assign clr6cnt = ~(&{ctlProg_ClkOn_sync_d[1:0], ctlProg_ClkOn_sync});
  assign clkEn   =   |{ctlProg_ClkOn_sync_d[1:0], ctlProg_ClkOn_sync} ;

  // Clock gater (/import/rtllib/src/rtl/ati_clock_gate.v)
  ati_clock_gate I_gclk (.clk_out(gclk), .clk_src(clk), .clk_en(clkEn), .TE(1'b0));

  always @(posedge gclk)
    if (clr6cnt)
      dbgSig_6Cnt[5:0] <= {6 {1'b0}};
    else
      dbgSig_6Cnt[5:0] <= (dbgSig_6Cnt[5:0] + 6'b000001);

  always @(*)
    case ({dbgSig_6Cnt[5], dbgSig_6Cnt[0]})
      2'b00 : dbgSig_6CntO[7:0] = { dbgSig_6Cnt[4:1],  dbgSig_6Cnt[4:1]};
      2'b01 : dbgSig_6CntO[7:0] = {~dbgSig_6Cnt[4:1], ~dbgSig_6Cnt[4:1]};
      2'b10 : dbgSig_6CntO[7:0] = {~dbgSig_6Cnt[4:1],  dbgSig_6Cnt[4:1]};
      2'b11 : dbgSig_6CntO[7:0] = { dbgSig_6Cnt[4:1], ~dbgSig_6Cnt[4:1]};
    endcase // case(dbgSig_6Cnt[5], dbgSig_6Cnt[0])

  // Logic to generate the debug signals
  generate
     if (LANE_COUNT > 3) begin
        if (LANE_WIDTH > 8) begin
          assign idebug[63:0] = {~{dbg_BlkId   [7:0]}, ~{dbgSig_6CntO[7:0]},  // Move the ID to upper bits so lower 8 can get toggled
                                 ~{dbg_BlkId   [7:0]}, ~{dbgSig_4CntO[7:0]},  // Move the ID to upper bits so lower 8 can get toggled
                                   dbgSig_6CntO[7:0] ,   dbg_BlkId   [7:0] ,
                                   dbgSig_4CntO[7:0] ,   dbg_BlkId   [7:0]  };
           //! unused dbg_Mux_SelLane
        end else begin
          assign idebug[31:0] = {dbgSig_6CntO[7:0], dbgSig_4CntO[7:0], ~{dbg_BlkId[7:0]}, dbg_BlkId[7:0]};
           //! unused dbg_Mux_SelLane
        end
     end else begin
       assign idebug[ 7: 0] = (&dbg_Mux_SelLane[ 8: 6]) ? dbgSig_6CntO[7:0] : dbg_BlkId[7:0];
       assign idebug[15: 8] = (&dbg_Mux_SelLane[17:15]) ? dbgSig_6CntO[7:0] : ~dbg_BlkId[7:0];
       assign idebug[23:16] = (&dbg_Mux_SelLane[26:24]) ? dbgSig_6CntO[7:0] : dbgSig_4CntO[7:0];
       //! unused dbg_Mux_SelLane[5:0]
       //! unused dbg_Mux_SelLane[14:9]
       //! unused dbg_Mux_SelLane[23:18]
     end
  endgenerate

endmodule // dbg_client_ssb__debug
