//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_sync_aon_s2p.v
// Description : Serial to parallel conversion for mux configuration
//
// NOTE : Multi-clock case should use Source-synchronous debug bus client
//
////////////////////////////////////////////////////////////////////////////

module dbg_client_sync_aon_s2p (

// ==========================================================================
// Misc
// Begin --------------------------------------------------------------------
input             ipclk                    , // system clock
input      [7:0]  DbgBlkId                 , // Debug client ID - hard tie
input             cold_resetb              , // From rsmu, Assert for Cold, Cool reset
input             AEB_DBUS                 , // Security lock down. 1'b0-> hold program to reset == data forward
input             TEST_CLK_EN              , // Gater override for test mode

output wire       dbg_active               , // output the gater term for data pipe (xxx_xxx_daisychain_data)
// End ----------------------------------------------------------------------

// ==========================================================================
// Debug bus serial control input
// Control data flows from DSM out to debug bus chain
// Begin --------------------------------------------------------------------
output wire [1:0] DBUS_TARG_daisychain_ctl , // Daisy-chain program output, reverse of data flow
input       [1:0] TARG_DBUS_daisychain_ctl , // Daisy-chain program input,  reverse of data flow
// End ----------------------------------------------------------------------

// ==========================================================================
// Data decoded from daisychain_ctl_d1 (serial stream control config)
// output to all dbg_client_sync_ctl instance
// Begin --------------------------------------------------------------------
output wire       gcclk                    , // gated system clock (ipclk) for control logic
output wire       ctlProgClrDat            , // Clear the data Pipe
output wire       ctlProgClrCtl            , // Clear the Control logic data
output wire       ctlProgVal               , // control program : control valid
output wire [1:0] ctlProg_ByteSel          , // control program : bit Byte select // removed [2]Multi-block mode
output wire [2:0] ctlProg_MuxId            , // control program : Mux ID of the current program target
output wire [1:0] ctlProg_LaneSel          , // control program : Lane select of mux
output wire [5:0] ctlProg_ChannelSel         // control program : Channel select of mux
// End ----------------------------------------------------------------------
);

  // ==========================================================================
  // Program state of Main state machine (cntSt)
  // Begin --------------------------------------------------------------------
  localparam       DBGS2P__DONE = 3'b000; // idel/Done
  localparam       DBGS2P_SHIFT = 3'b001; // Shift in data
  localparam       DBGS2P_STORE = 3'b011; // Store shifted in data
  localparam       DBGS2P__WAIT = 3'b010; // Wait for 2nd excludeCtlC1_EndMrk
  localparam       DBGS2P__AEB0 = 3'b100; // Reset Clear
  localparam       DBGS2P__AEB1 = 3'b101; // Reset Clear
  localparam       DBGS2P__AEB2 = 3'b110; // Reset Clear
  localparam       DBGS2P__AEB3 = 3'b111; // Reset Clear
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // Control data pipe (ungated)
  // Begin --------------------------------------------------------------------
  wire [1:0]             daisychain_ctl_d1;
  reg  [1:0]             daisychain_ctl_d2;
  reg  [2:1]             aeb_dbus_d       ;
  wire                   aeb_dbus_fe      ; // Detect falling edge of AEB_DBUS
  assign                 DBUS_TARG_daisychain_ctl[1:0] = daisychain_ctl_d2[1:0];
  assign                 aeb_dbus_fe                   = ~aeb_dbus_d[1] & aeb_dbus_d[2]; // bug 649331
  dbg_client_sync_aon_gclk I_dbg_active (
                                        .ipclk                    (ipclk                        ),
                                        .dbg_active               (dbg_active                   ),
                                        .daisychain_ctl_d1        (daisychain_ctl_d1[1:0]       ),
                                        .TARG_DBUS_daisychain_ctl (TARG_DBUS_daisychain_ctl[1:0])
                                        );
  always @(posedge ipclk) begin
     daisychain_ctl_d2[1:0] <= daisychain_ctl_d1[1:0];
     aeb_dbus_d[2:1]        <= {aeb_dbus_d[1], AEB_DBUS};
  end
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // local gater control for the configuration logic
  // Begin --------------------------------------------------------------------
  wire                   excludeCtlC1_EndMrk; // Exclude C1 term and pulse for end of transmit mark
  wire                   dbg_ctlVal;
  wire                   dbg_ctl;
  wire                   dbg_ctlClkEn;
  reg                    aeb_dly;
  assign                 excludeCtlC1_EndMrk = daisychain_ctl_d1[0] ^ daisychain_ctl_d2[0]; // Filter valid of C1
  assign                 dbg_ctlClkEn        = daisychain_ctl_d2[0] | ~cold_resetb  | aeb_dbus_fe | aeb_dly;
  assign                 dbg_ctlVal          = daisychain_ctl_d1[0] & ~excludeCtlC1_EndMrk;
  assign                 dbg_ctl             = daisychain_ctl_d1[1];
  // Clock gater ($STEM/import/rtllib/src/rtl/ati_clock_gate.v)
  ati_clock_gate I_gcclk (.clk_out(gcclk), .clk_src(ipclk), .clk_en(dbg_ctlClkEn), .TE(TEST_CLK_EN));
  // End ----------------------------------------------------------------------

  // ==========================================================================
  // BOZO: This section need to use header file !!!!
  // Serial to Parallel conversion
  // Control data shift in MSB first, keep last 20 bits
  // [20:18] MuxId                   (ctlProg_MuxId[2:0]     )
  // [17:16] Lane Select             (ctlProg_LaneSel[1:0]   )
  // [15:14] bit Byte select         (ctlProg_ByteSel[1:0]   )
  // [13: 8] Channel Select          (ctlProg_ChannelSel[5:0])
  // [ 7: 0] debug client BlkId[7:0]
  // NOTE: Multi-block mode is defeatured
  // Begin --------------------------------------------------------------------
  reg  [4:0]             cnt       ; // counter to find end of 32 bit data
  reg  [2:0]             cntSt     ; // counterenable
  reg  [2:0]             cntStNxt  ;
  wire                   cntAtMax  ;
  wire                   SampleData;
  wire                   cntEn     ;
  wire                   ClrProgClrDatC;
  wire                   ClrCntSt;

  assign                 cntAtMax = &cnt[4:0];

  reg [20:0]             ctlDat;        // Control data shift register
  reg                    ClrDatPipe;    // Clear data Pipe


  // Clear data if either
  // 1) DBGBLKID == 8'hFF
  // 2) ctlProg_MuxId[2:0] == 3'b111
  // 3) cold_resetb == 1'b0
  // 4) AEB_DBUS == 1'b0

  // Clear Data Pipe if (DBGBLKID == 8'hFF) & (ctlProg_MuxId[2:0] == 3'b111)

  wire                   DbgBlkId_Is_FF;
  wire                   MuxId_Is_7;
  assign                 DbgBlkId_Is_FF = &ctlDat[7:0];
  assign                 MuxId_Is_7     = &ctlProg_MuxId[2:0];

  // Exclude AEB_DBUS term from mux modules
  // This way AEB_DBUS setting does not result in clocking forced on
  assign                 ctlProgClrDat  = ClrDatPipe;
  assign                 ClrProgClrDatC = (SampleData & DbgBlkId_Is_FF ) |
                                          (ctlProgVal & MuxId_Is_7     ) |
                                          ~cold_resetb;

  assign                 ctlProgClrCtl = ClrProgClrDatC |  aeb_dly;
  assign                 clearCtlProg  = ctlProgClrCtl  | ~aeb_dbus_d[1]; // bug 649331
  assign                 ClrCntSt      = excludeCtlC1_EndMrk & ~daisychain_ctl_d1[0] | ClrProgClrDatC;
  assign                 cntEn         = (cntSt[2:0] == DBGS2P_SHIFT) & (cntStNxt[2:0] != DBGS2P__DONE);

  always @(*) begin
     if (dbg_ctlVal)
       case (cntSt[2:0])
         DBGS2P__DONE : cntStNxt[2:0] = dbg_ctl             ? DBGS2P_SHIFT : DBGS2P__DONE;
         DBGS2P_SHIFT : cntStNxt[2:0] = cntAtMax            ? DBGS2P_STORE : DBGS2P_SHIFT;
         DBGS2P_STORE : cntStNxt[2:0] =                       DBGS2P__WAIT               ;
         DBGS2P__WAIT : cntStNxt[2:0] = excludeCtlC1_EndMrk ? DBGS2P__DONE : DBGS2P__WAIT;
         DBGS2P__AEB0 : cntStNxt[2:0] = DBGS2P__AEB1;
         DBGS2P__AEB1 : cntStNxt[2:0] = DBGS2P__AEB2;
         DBGS2P__AEB2 : cntStNxt[2:0] = DBGS2P__AEB3;
         DBGS2P__AEB3 : cntStNxt[2:0] = DBGS2P__DONE;
         default : cntStNxt[2:0] = DBGS2P__DONE;
       endcase // case (cntSt[2:0])
     else
       case (cntSt[2:0])
         DBGS2P__AEB0 : cntStNxt[2:0] = DBGS2P__AEB1;
         DBGS2P__AEB1 : cntStNxt[2:0] = DBGS2P__AEB2;
         DBGS2P__AEB2 : cntStNxt[2:0] = DBGS2P__AEB3;
         DBGS2P__AEB3 : cntStNxt[2:0] = DBGS2P__DONE;
         default : cntStNxt[2:0] = DBGS2P__DONE;
       endcase // case (cntSt[2:0])

     case (cntSt[2:0])
       DBGS2P__AEB0 : aeb_dly = 1'b1;
       DBGS2P__AEB1 : aeb_dly = 1'b1;
       DBGS2P__AEB2 : aeb_dly = 1'b1;
       DBGS2P__AEB3 : aeb_dly = 1'b1;
       default      : aeb_dly = 1'b0;
     endcase // case (cntSt[2:0])
  end // always @ (*)

  always @(posedge gcclk) begin
     if (clearCtlProg) begin
        cnt[4:0]     <= { 5{1'b0}};
        ctlDat[20:0] <= {21{1'b0}};
     end else begin
        cnt[4:0]     <= cntEn                      ? (cnt[4:0] + 5'b00001) : {5 {1'b0}};
        ctlDat[20:0] <= cntSt[2:0] == DBGS2P_SHIFT ? {ctlDat[19:0], dbg_ctl} : ctlDat[20:0];
     end
     if (ClrCntSt)
       cntSt[2:0]    <= DBGS2P__DONE;
     else if (aeb_dbus_fe)
       cntSt[2:0]    <= DBGS2P__AEB0;
     else
       cntSt[2:0]    <= cntStNxt[2:0];
  end // always @ (posedge gcclk)

  always @(posedge gcclk) ClrDatPipe <= SampleData ? DbgBlkId_Is_FF & MuxId_Is_7 : ClrDatPipe;

  assign SampleData              =  cntSt[2:0] == DBGS2P_STORE;
  assign ctlProg_MuxId[2:0]      =  ctlDat[20:18];
  assign ctlProg_LaneSel[1:0]    =  ctlDat[17:16];
  assign ctlProg_ByteSel[1:0]    =  ctlDat[15:14];
  assign ctlProg_ChannelSel[5:0] =  ctlDat[13: 8];
  assign ctlProgVal              =((ctlDat[ 7: 0] == DbgBlkId[7:0]) | DbgBlkId_Is_FF) & SampleData;
  // End ----------------------------------------------------------------------
endmodule // dbg_client_sync_aon_s2p
