//                              -*- Mode: Verilog -*-
// (c) 2015, 2016 Advanced Micro Devices, Inc. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//
// Filename    : dbg_client_sync_mux_byte.v
// Description : Mux for debug client. This mux only have the byte select portion.
//               This block is pure combonation logic.
//
////////////////////////////////////////////////////////////////////////////
module dbg_client_sync_mux_byte #(parameter
LANE_COUNT   = 4,          // Number of Lanes, only 3 or 4 is currently supported
LANE_WIDTH   = 8,          // Bit width of each Lane, only 8 or 16 is currently supported
FLOPPED_OUTPUT = 0,   // flag to add flop after  the mux
FLOPPED_INPUTS = 0    // flag to add flop before the mux
)(
input       ipclk         , // system clock

// ==========================================================================
// interface with ${synchronicity}_dbg_client_steer
// Begin --------------------------------------------------------------------
   input                                     dbg_active      , // Clock gater control
   input       [(LANE_COUNT*9-1):0]          dbg_Mux_SelLane , // LANE_COUNT * ([8] Lane Enable; [7:6] Byte Select; [5:0] Channel Select;)
   output wire [(LANE_COUNT*LANE_WIDTH-1):0] dbg_Mux_Data    , // Debug-bus data outputs after muxing
// End ----------------------------------------------------------------------

// ==========================================================================
// Debug-bus data inputs from functional logic
// Begin --------------------------------------------------------------------
   input       [(LANE_COUNT*LANE_WIDTH-1):0] idebug ,
// End ----------------------------------------------------------------------

input                            dbg_rst       , // Combined global reset with command decoded reset
input                            TEST_CLK_EN     // Gater override for test mode
);

  wire                           gclk       ; // Gated clock for potential pipe stage
reg  [(LANE_COUNT*LANE_WIDTH-1):0] reg_MuxData;
reg  [(LANE_COUNT*LANE_WIDTH-1):0] dbg_L2DataMuxLane;
wire [(LANE_WIDTH-1):0]            dbg_L1DataMuxDefault[(LANE_COUNT-1):0];

assign dbg_Mux_Data[(LANE_COUNT*LANE_WIDTH-1):0] = reg_MuxData[(LANE_COUNT*LANE_WIDTH-1):0];

//////////////////////////////////////////////////////////////////////////|
//                        Debug-Bus PipeLine                            //|
//////////////////////////////////////////////////////////////////////////|
generate

   if (FLOPPED_OUTPUT | FLOPPED_INPUTS) begin
     // Clock gater ($STEM/import/rtllib/src/rtl/ati_clock_gate.v)
     ati_clock_gate I_gclk (.clk_out(gclk), .clk_src(ipclk), .clk_en(dbg_active | dbg_rst), .TE(TEST_CLK_EN));
   end else begin
     assign gclk = 1'b0;
     //! unused dbg_active
     //! unused dbg_rst
     //! unused ipclk
     //! unused TEST_CLK_EN
   end

   if (FLOPPED_OUTPUT) begin
      always @(posedge gclk)
        if (dbg_rst)
          reg_MuxData[(LANE_COUNT*LANE_WIDTH-1):0] <= {LANE_COUNT*LANE_WIDTH {1'b0}};
        else
          reg_MuxData[(LANE_COUNT*LANE_WIDTH-1):0] <= dbg_L2DataMuxLane[(LANE_COUNT*LANE_WIDTH-1):0];
   end else begin
      always @(*) reg_MuxData[(LANE_COUNT*LANE_WIDTH-1):0] = dbg_L2DataMuxLane[(LANE_COUNT*LANE_WIDTH-1):0];
   end
endgenerate

//////////////////////////////////////////////////////////////////////////|
//                       <Debug-Bus Mux - Level 2>                      //|
//                   (Level 1 is outside of dbg_common)                 //|
//////////////////////////////////////////////////////////////////////////|
// Muxing for byte selection for each of the 3 24bit debug-bus inputs   //|
// This muxing is active only if Multi-Block mode is enabled.           //|
// In multiblock mode from each of the 3 input buses one byte (segment) //|
// is selected. This selection is independant for each input bus.       //|
// In SingleBlock mode (MultiBlock is not enables) all 3 bytes will be  //|
// selected from first input bus.                                       //|
// ----                                                                 //|
// Also this muxing is responsible for selecting AA, 55 BlkID for       //|
// Dbug-bus test purposes                                               //|
//////////////////////////////////////////////////////////////////////////|
reg [(LANE_COUNT*LANE_WIDTH-1):0] idebugPre;

generate
if(LANE_COUNT>3) begin
  assign dbg_L1DataMuxDefault[0][(LANE_WIDTH-1):0] = idebugPre[(LANE_WIDTH*4-1):(LANE_WIDTH*3)];
  assign dbg_L1DataMuxDefault[1][(LANE_WIDTH-1):0] = idebugPre[(LANE_WIDTH*4-1):(LANE_WIDTH*3)];
  assign dbg_L1DataMuxDefault[2][(LANE_WIDTH-1):0] = idebugPre[(LANE_WIDTH*4-1):(LANE_WIDTH*3)];
  assign dbg_L1DataMuxDefault[3][(LANE_WIDTH-1):0] = idebugPre[(LANE_WIDTH*4-1):(LANE_WIDTH*3)];
end else begin
  assign dbg_L1DataMuxDefault[0][(LANE_WIDTH-1):0] = idebugPre[(LANE_WIDTH*1-1):            0 ];
  assign dbg_L1DataMuxDefault[1][(LANE_WIDTH-1):0] = idebugPre[(LANE_WIDTH*2-1):(LANE_WIDTH*1)];
  assign dbg_L1DataMuxDefault[2][(LANE_WIDTH-1):0] = idebugPre[(LANE_WIDTH*3-1):(LANE_WIDTH*2)];
end // else if(LANE_COUNT>3)

always @(*) begin

   // Lane0 ------------------------------ Begin
   case (dbg_Mux_SelLane[ 7: 6])
      2'b00  : dbg_L2DataMuxLane[(LANE_WIDTH*1-1):            0 ] = idebugPre[(LANE_WIDTH*1-1):            0 ];
      2'b01  : dbg_L2DataMuxLane[(LANE_WIDTH*1-1):            0 ] = idebugPre[(LANE_WIDTH*2-1):(LANE_WIDTH*1)];
      2'b10  : dbg_L2DataMuxLane[(LANE_WIDTH*1-1):            0 ] = idebugPre[(LANE_WIDTH*3-1):(LANE_WIDTH*2)];
      2'b11  : dbg_L2DataMuxLane[(LANE_WIDTH*1-1):            0 ] = dbg_L1DataMuxDefault[0][(LANE_WIDTH-1):0];
      default: dbg_L2DataMuxLane[(LANE_WIDTH*1-1):            0 ] = dbg_L1DataMuxDefault[0][(LANE_WIDTH-1):0];
   endcase // case (dbg_Mux_SelLane[ 7: 6])
   // Lane0 -------------------------------- End
   // Lane1 ------------------------------ Begin
   case (dbg_Mux_SelLane[16:15])
      2'b00  : dbg_L2DataMuxLane[(LANE_WIDTH*2-1):(LANE_WIDTH*1)] = idebugPre[(LANE_WIDTH*1-1):            0 ];
      2'b01  : dbg_L2DataMuxLane[(LANE_WIDTH*2-1):(LANE_WIDTH*1)] = idebugPre[(LANE_WIDTH*2-1):(LANE_WIDTH*1)];
      2'b10  : dbg_L2DataMuxLane[(LANE_WIDTH*2-1):(LANE_WIDTH*1)] = idebugPre[(LANE_WIDTH*3-1):(LANE_WIDTH*2)];
      2'b11  : dbg_L2DataMuxLane[(LANE_WIDTH*2-1):(LANE_WIDTH*1)] = dbg_L1DataMuxDefault[1][(LANE_WIDTH-1):0];
      default: dbg_L2DataMuxLane[(LANE_WIDTH*2-1):(LANE_WIDTH*1)] = dbg_L1DataMuxDefault[1][(LANE_WIDTH-1):0];
   endcase // case (dbg_Mux_SelLane[16:15])
   // Lane1 -------------------------------- End
   // Lane2 ------------------------------ Begin
   case (dbg_Mux_SelLane[25:24])
      2'b00  : dbg_L2DataMuxLane[(LANE_WIDTH*3-1):(LANE_WIDTH*2)] = idebugPre[(LANE_WIDTH*1-1):            0 ];
      2'b01  : dbg_L2DataMuxLane[(LANE_WIDTH*3-1):(LANE_WIDTH*2)] = idebugPre[(LANE_WIDTH*2-1):(LANE_WIDTH*1)];
      2'b10  : dbg_L2DataMuxLane[(LANE_WIDTH*3-1):(LANE_WIDTH*2)] = idebugPre[(LANE_WIDTH*3-1):(LANE_WIDTH*2)];
      2'b11  : dbg_L2DataMuxLane[(LANE_WIDTH*3-1):(LANE_WIDTH*2)] = dbg_L1DataMuxDefault[2][(LANE_WIDTH-1):0];
      default: dbg_L2DataMuxLane[(LANE_WIDTH*3-1):(LANE_WIDTH*2)] = dbg_L1DataMuxDefault[2][(LANE_WIDTH-1):0];
   endcase // case (dbg_Mux_SelLane[25:24])
   // Lane2 -------------------------------- End
  if (LANE_COUNT>3) begin
   // Lane3 ------------------------------ Begin
   case (dbg_Mux_SelLane[34:33])
      2'b00  : dbg_L2DataMuxLane[(LANE_WIDTH*4-1):(LANE_WIDTH*3)] = idebugPre[(LANE_WIDTH*1-1):            0 ];
      2'b01  : dbg_L2DataMuxLane[(LANE_WIDTH*4-1):(LANE_WIDTH*3)] = idebugPre[(LANE_WIDTH*2-1):(LANE_WIDTH*1)];
      2'b10  : dbg_L2DataMuxLane[(LANE_WIDTH*4-1):(LANE_WIDTH*3)] = idebugPre[(LANE_WIDTH*3-1):(LANE_WIDTH*2)];
      2'b11  : dbg_L2DataMuxLane[(LANE_WIDTH*4-1):(LANE_WIDTH*3)] = dbg_L1DataMuxDefault[3][(LANE_WIDTH-1):0];
      default: dbg_L2DataMuxLane[(LANE_WIDTH*4-1):(LANE_WIDTH*3)] = dbg_L1DataMuxDefault[3][(LANE_WIDTH-1):0];
   endcase // case (dbg_Mux_SelLane[34:33])
   // Lane3 -------------------------------- End
  end // if (LANE_COUNT>3)

end // always @(*)
endgenerate

//////////////////////////////////////////////////////////////////////////|
//                      Flop Mux Input                                  //|
//////////////////////////////////////////////////////////////////////////|
generate
   if (FLOPPED_INPUTS) begin
      always @(posedge gclk)
        if (dbg_rst)
          idebugPre[(LANE_COUNT*LANE_WIDTH-1):0] <= {LANE_COUNT*LANE_WIDTH {1'b0}};
        else
          idebugPre[(LANE_COUNT*LANE_WIDTH-1):0] <= idebug[(LANE_COUNT*LANE_WIDTH-1):0];
   end else begin
      always @(*) idebugPre[(LANE_COUNT*LANE_WIDTH-1):0] = idebug[(LANE_COUNT*LANE_WIDTH-1):0];
   end
endgenerate

endmodule // dbg_client_sync_mux_byte
