//--------------------------------------------------------------------------
//
// $Id: //tools/internal/packages/design/rtllib/main/src/rtl/ati_fifo.v#2 $
//
// $Change: 562798 $
//
// Notes:     This is a generic design of a register based fifo. It's size is determned
//            by the paramters: FIFO_WIDTH, ADDR_WIDTH, FIFO_DEPTH and SKID_WORKDS. When
//            SKID_WORDS = 0 ... this represents a fifo with a FULL flag. SKID_WORDS
//            controls the setting of full flag based on FULL = FIFO_DEPTH - SKID_WORDS - 1
//
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2002, ATI Technologies, Inc., (unpublished)
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
//--------------------------------------------------------------------------
`include "dftmisc_assert_control.vh"

module dfx_ati_fifo_gc10 ( /*AUTOARG*/
  // Outputs
  full, read_data, empty,
  // Inputs
  write_data, we, re, clk, reset
  );
    //============================
    // PARAMETERS
    //============================
    parameter FIFO_WIDTH = 0;
    parameter ADDR_WIDTH = 0;
    parameter FIFO_DEPTH = 0;
    parameter SKID_WORDS = 0;
    parameter FIFO_NAME  = "";
    parameter EXTERNAL_FULL_REGISTER  = 0;   // set this parameter if you which to register the full signal external to this block
`ifdef ABV
    parameter CYCLES_BEFORE_OVERFLOW_CHECK = 40;
    integer    cycle_count;
`endif
    //============================
    // IO PORTS
    //============================
    input    [FIFO_WIDTH-1:0] write_data;
    input                     we;
    output                    full;
    output   [FIFO_WIDTH-1:0] read_data;
    output                    empty;
    input                     re;
    input                     clk;
    input                     reset;

    //============================
    // INTERNAL SIGNALS
    //============================
    // read data register
//  wire [FIFO_WIDTH-1:0]  read_data;

    // signals to the mem
    wire [ADDR_WIDTH-1:0]  reg_waddr;
    wire [ADDR_WIDTH-1:0]  reg_raddr;

    reg  [FIFO_WIDTH-1:0]  reg_mem [FIFO_DEPTH-1:0];

    // ***********************************
    // I/O Definitions
    // ***********************************

    // signals used in this module
    // The address width of the fifo_cnt_q signal will be 1 bit too wide
    // if the depth is not a power of two; however, this case works for all values.
    wire                    fifo_we;    // internal we, clocks write ptr
    wire                    fifo_re;    // internal re, clocks read ptr
    reg   [ADDR_WIDTH-1:0]  fifo_wptr_d; //, fifo_wptr_q, fifo_rptr_d, fifo_rptr_q
    reg   [ADDR_WIDTH-1:0]  fifo_wptr_q;
    reg   [ADDR_WIDTH-1:0]  fifo_rptr_d;
    reg   [ADDR_WIDTH-1:0]  fifo_rptr_q;
    reg   [ADDR_WIDTH:0]    fifo_cnt_d; // , fifo_cnt_q
    reg   [ADDR_WIDTH:0]    fifo_cnt_q;

    reg                     fifo_full_d, fifo_full_q;
    reg                     int_fifo_empty_d, int_fifo_empty_q;   // empty for the internal fifo

    integer i;

    // -------------------------------------------
    // concurrent assignments
    // -------------------------------------------
    assign fifo_we = we;
    assign fifo_re = re & ~int_fifo_empty_q;
    assign empty = int_fifo_empty_q;
    assign full = ( EXTERNAL_FULL_REGISTER == 0 ? fifo_full_q : fifo_full_d);
    assign reg_waddr[ADDR_WIDTH-1:0] = fifo_wptr_q[ADDR_WIDTH-1:0];
    assign reg_raddr[ADDR_WIDTH-1:0] = fifo_rptr_q[ADDR_WIDTH-1:0];    // note not registered
    assign read_data[FIFO_WIDTH-1:0] = reg_mem[reg_raddr[ADDR_WIDTH-1:0]][FIFO_WIDTH-1:0];

    // -------------------------------------------
    // clock process
    // -------------------------------------------
    always @(posedge clk) begin
        fifo_wptr_q[ADDR_WIDTH-1:0] <= fifo_wptr_d[ADDR_WIDTH-1:0];
        fifo_rptr_q[ADDR_WIDTH-1:0] <= fifo_rptr_d[ADDR_WIDTH-1:0];
        fifo_cnt_q [ADDR_WIDTH  :0] <= fifo_cnt_d [ADDR_WIDTH  :0];
        fifo_full_q      <= fifo_full_d;
        int_fifo_empty_q <= int_fifo_empty_d;
    end // always clk

`ifdef ABV
// VCS coverage off
   wire MISC_ABV_ALWAYS_EN;
   wire MISC_ABV_PWROK_EN ;
   wire MISC_ABV_DFT_EN   ;
   wire MISC_ABV_FUNC_EN  ;
   wire bozo_keep;

   assign  MISC_ABV_ALWAYS_EN = `DFTMISC_ABV_ALWAYS_EN;
   assign  MISC_ABV_PWROK_EN  = `DFTMISC_ABV_PWROK_EN ;
   assign  MISC_ABV_DFT_EN    = `DFTMISC_ABV_DFT_EN   ;
   assign  MISC_ABV_FUNC_EN   = `DFTMISC_ABV_FUNC_EN  ;
   assign  bozo_keep   =  MISC_ABV_ALWAYS_EN ^ MISC_ABV_PWROK_EN  ^ MISC_ABV_DFT_EN   ^ MISC_ABV_FUNC_EN  ;
   ati_buffer0 d0nt_bozo_i (.A(bozo_keep), .Y());

// Origional test_expr: if ( (cycle_count > CYCLES_BEFORE_OVERFLOW_CHECK) && (reset == 1'b0) && fifo_we && ~fifo_re && (fifo_cnt_q == FIFO_DEPTH ) )
ovl_never #(
   .msg("Error: DFX_ATI_Fifo ERROR: fifo written when full -- data dropped"))
assert_DFX_ATI_FifoOverflow (
   .clock     (clk),
   .reset     (reset),
   .enable    (MISC_ABV_FUNC_EN),
   .test_expr ((cycle_count > CYCLES_BEFORE_OVERFLOW_CHECK) && fifo_we && ~fifo_re && (fifo_cnt_q[ADDR_WIDTH:0] == FIFO_DEPTH )),
   .fire      ());
// VCS coverage on
`endif

    // setup the register input
    always @ ( /*AUTOSENSE*/fifo_cnt_q or fifo_full_q or fifo_re
              or fifo_rptr_q or fifo_we or fifo_wptr_q
              or int_fifo_empty_q or reset) begin

        // assign default inputs
        fifo_wptr_d[ADDR_WIDTH-1:0] = fifo_wptr_q[ADDR_WIDTH-1:0];
        fifo_rptr_d[ADDR_WIDTH-1:0] = fifo_rptr_q[ADDR_WIDTH-1:0];
        fifo_cnt_d[ADDR_WIDTH:0] = fifo_cnt_q[ADDR_WIDTH:0];
        fifo_full_d = fifo_full_q;
        int_fifo_empty_d = int_fifo_empty_q;

        // clock the immediate data fifo registers on the write side
        if ( fifo_we ) begin
            if ( fifo_wptr_q[ADDR_WIDTH-1:0] == ( FIFO_DEPTH - 1 ) ) begin
                fifo_wptr_d[ADDR_WIDTH-1:0] = 0;
            end else begin
                fifo_wptr_d[ADDR_WIDTH-1:0] = fifo_wptr_q[ADDR_WIDTH-1:0] + 1;
            end
        end

        // clock the immediate data fifo registers on the read side
        if ( fifo_re ) begin
            if ( fifo_rptr_q[ADDR_WIDTH-1:0] == ( FIFO_DEPTH - 1 ) ) begin
                fifo_rptr_d[ADDR_WIDTH-1:0] = 0;
            end else begin
                fifo_rptr_d[ADDR_WIDTH-1:0] = fifo_rptr_q[ADDR_WIDTH-1:0] + 1;
            end
        end

        // immediate data fifo empty/full logic
        case ( { fifo_we, fifo_re } )
            2'b10 :
                // write, no read
                begin
                    if ( fifo_cnt_q[ADDR_WIDTH:0] == FIFO_DEPTH - SKID_WORDS - 1 ) begin
                        fifo_full_d = 1;
                    end
                    fifo_cnt_d[ADDR_WIDTH:0] = fifo_cnt_q[ADDR_WIDTH:0] + 1;
                    int_fifo_empty_d = 0;
                end
            2'b01 :
                // read, no write
                begin
                    if ( fifo_cnt_q[ADDR_WIDTH:0] == {{ADDR_WIDTH {1'b0}}, 1'b1} ) begin
                        int_fifo_empty_d = 1;
                    end
                    if ( fifo_cnt_q[ADDR_WIDTH:0] == FIFO_DEPTH - SKID_WORDS) begin
                        fifo_full_d = 0;
                    end
                    fifo_cnt_d[ADDR_WIDTH:0] = fifo_cnt_q[ADDR_WIDTH:0] - 1;
                end
            default: begin
                fifo_cnt_d[ADDR_WIDTH:0] = fifo_cnt_q[ADDR_WIDTH:0];
                fifo_full_d = fifo_full_q;
                int_fifo_empty_d = int_fifo_empty_q;
            end
        endcase

        // -------------------------------------------
        // reset-able registers
        // -------------------------------------------
        // put this last in the clock process to override
        // anything that was assigned above
        if ( reset ) begin
            // immediate data fifo registers
            fifo_wptr_d[ADDR_WIDTH-1:0] = 0;
            fifo_rptr_d[ADDR_WIDTH-1:0] = 0;
            fifo_cnt_d[ADDR_WIDTH:0] = 0;
            fifo_full_d = 0;
            int_fifo_empty_d = 1;
        end

    end

    //============================
    // RESIGER BASED STORAGE
    // This particular coding was used in the hopes that synopsys and power compiler
    // would be able to recognize the enabled nature of these flops.
    // see following article for more information
    // Author: thomas.esposito.com
    // Date: 2001-08-17 08:15:08 PST
    // Subject: Synthesis of Verilog array assignment with Synopsys
    // Newsgroups: comp.cad.synthesis
    //============================
    always @(posedge clk)
    begin
        if (we) begin
            for (i = 0; i < FIFO_DEPTH; i = i + 1) begin
                if (i == reg_waddr[ADDR_WIDTH-1:0]) begin
                    reg_mem[i][FIFO_WIDTH-1:0] <= write_data[FIFO_WIDTH-1:0];
                end
            end
        end
    end // always clk

`ifdef ABV
  initial begin
    cycle_count <= 0;
    forever begin
      @(posedge clk);
      if (cycle_count <= CYCLES_BEFORE_OVERFLOW_CHECK) begin
        cycle_count <= cycle_count + 1;
      end
    end
  end
`endif
endmodule //dfx_ati_fifo
