//--------------------------------------------------------------------------
//
// Notes:	Emulates a memory macro
// Copyright:	Trade secret of ATI Technologies, Inc.
//		Copyright 2001-2002, ATI Technologies, Inc., (unpublished)
//
//		All rights reserved.  This notice is intended as a precaution against
//		inadvertent publication and does not imply publication or any waiver
//		of confidentiality.  The year included in the foregoing notice is the
//		year of creation of the work.
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

module dum_mem
(/*AUTOARG*/
   // Outputs
   oQ,
   // Inputs
   iRCLK, iWCLK, iMER, iMEW, iWEN, iWADR, iRADR, iD, iWMSK, iOUTREG_EN
   );

   parameter DATA_WIDTH = 8;
   parameter WORDS = 1;
   parameter IS_MASKABLE = 0;
   parameter OUTPUT_REG = 2;  // 0 = None, 1 = Non-piped output enable, 2 = Pipelined output enable.
   parameter DEBUG = 2;   // debug level 1 reports errors, debug level 2 reports warnings.
   parameter WARN_ON_PARTIAL_READ_WRITE_COLLISION = 1;
   parameter WARN_ON_WRITTEN_X = 1;

   parameter ADDR_WIDTH = ((WORDS <= 1<<0) ? 0 :
                           (WORDS <= 1<<1) ? 1 :
                           (WORDS <= 1<<2) ? 2 :
                           (WORDS <= 1<<3) ? 3 :
                           (WORDS <= 1<<4) ? 4 :
                           (WORDS <= 1<<5) ? 5 :
                           (WORDS <= 1<<6) ? 6 :
                           (WORDS <= 1<<7) ? 7 :
                           (WORDS <= 1<<8) ? 8 :
                           (WORDS <= 1<<9) ? 9 :
                           (WORDS <= 1<<10) ? 10 :
                           (WORDS <= 1<<11) ? 11 :
                           (WORDS <= 1<<12) ? 12 :
                           (WORDS <= 1<<13) ? 13 :
                           (WORDS <= 1<<14) ? 14 :
                           (WORDS <= 1<<15) ? 15 :
                           (WORDS <= 1<<16) ? 16 : 17);

   input                  iRCLK, iWCLK, iMER, iMEW, iWEN;
   input [ADDR_WIDTH-1:0] iWADR, iRADR;
   input [DATA_WIDTH-1:0] iD;
   input [DATA_WIDTH-1:0] iWMSK;
   input                  iOUTREG_EN;

   output [DATA_WIDTH-1:0] oQ;
// For early designs this may get synthesized..  Just make it wire up and be small.
`ifdef SIMULATION
   parameter               WORDS2 = WORDS;
   parameter               ADDR_WIDTH2 = ADDR_WIDTH;
`else
   parameter               WORDS2 = 2;
   parameter               ADDR_WIDTH2 = 1;
`endif

   parameter               NO_OUTPUT_REG = 0;
   parameter               NON_PIPED_OUTPUT_EN = 1;
   parameter               PIPED_OUTPUT_EN = 2;

   // flop all read inputs (to time like a virage memory)
   reg                     q_iMER, q_iWEN, q_iMEW;
   reg [ADDR_WIDTH2-1:0]   q_iRADR, q_iWADR;
   reg [DATA_WIDTH-1:0]    q_iD;
   reg [DATA_WIDTH-1:0]    q_iWMSK;
   reg                     q_iOUTREG_EN;
   always @(posedge iRCLK)
     begin
        q_iOUTREG_EN <= iOUTREG_EN;
        q_iMER <= iMER;
        if (iMER)
          q_iRADR <= iRADR;
     end

   always @(posedge iWCLK)
     begin
        q_iWEN <= iWEN;
        q_iMEW <= iMEW;
        q_iWADR <= iWADR;
        q_iD <= iD;
        q_iWMSK <= iWMSK;
     end

`ifdef SIMULATION
   wire [ADDR_WIDTH2-1:0]  iWADR2 = q_iWADR;
   wire [ADDR_WIDTH2-1:0]  iRADR2 = q_iRADR;
`else
   wire [ADDR_WIDTH2-1:0]  iWADR2 = ^q_iWADR;
   wire [ADDR_WIDTH2-1:0]  iRADR2 = ^q_iRADR;
`endif

   reg [DATA_WIDTH-1:0]    bram [WORDS2 - 1:0];

   wire [DATA_WIDTH-1:0]   WMSK = IS_MASKABLE ? q_iWMSK : {DATA_WIDTH{1'b1}};

   // write
   always @(posedge iWCLK ) begin
      if ( q_iWEN & q_iMEW ) begin
         bram[ iWADR2 ] <= ( q_iD & WMSK ) | ( bram[ iWADR2 ] & ~WMSK );
`ifdef SIMULATION
   // VCS coverage off
         if (DEBUG && (iWADR2 >= WORDS2))
           `MSGIO_DISPLAY("ERROR: Writing address that does not exist (iWADR = %x) in %m at %t", iWADR2, $time);
   // VCS coverage on
`endif
      end
   end

   // read

   // If a read/write addr collision happens, bits being written (in WMSK) are read as X.  Bits not being written are returned.
   integer i;
   wire [DATA_WIDTH-1:0] read = bram[iRADR2] ;
   reg [DATA_WIDTH-1:0]  collision;
   always @(/*AUTOSENSE*/WMSK or read) begin
      for (i=0; i<DATA_WIDTH; i=i+1) begin
         collision[i] = WMSK[i] ? 1'bX : read[i];
      end
   end

`ifdef SIMULATION
   // VCS coverage off
   // Check for the writing of X.
   integer j;
   reg    writing_x;
   always @(/*AUTOSENSE*/WMSK or q_iD) begin
      writing_x = 0;
      for (j=0; j<DATA_WIDTH; j=j+1) begin
         if (WMSK[j] && (q_iD === 1'bx))
           writing_x = 1;
      end
      if (writing_x && (DEBUG > 1) && WARN_ON_WRITTEN_X)
        `MSGIO_DISPLAY("WARNING: Writing X in %m at %t", $time);
   end
   // VCS coverage on
`endif

   reg [DATA_WIDTH-1:0] mem_data;
   reg [DATA_WIDTH-1:0] mem_data_q;
`ifdef SIMULATION  // Make simulation latch the data to be like the memory.
   // VCS coverage off
   always @(/*AS*/collision or iRADR2 or iRCLK or iWADR2 or q_iMER or q_iMEW or q_iWEN or read) begin
      if (iRCLK && q_iMER)
        if ( (iRADR2 == iWADR2) & (q_iMEW & q_iWEN) ) begin
           mem_data = collision;
        end else
          mem_data = read;
   end // always @ (posedge iRCLK)
   // VCS coverage on
`else  // Make synthesis run the data through the mux and then out.  Synth can't use latches like the mem has.
   always @(/*AS*/collision or iRADR2 or iWADR2 or q_iMEW or q_iWEN or read) begin
      if ( (iRADR2 == iWADR2) & (q_iMEW & q_iWEN) ) begin
         mem_data = collision;
      end else
        mem_data = read;
   end // always @ (posedge iRCLK)
`endif // !`ifdef SIMULATION
   


`ifdef SIMULATION
   // VCS coverage off
   always @(posedge iRCLK) begin
      if (q_iMER)
	if ( (iRADR2 == iWADR2) & (q_iMEW & q_iWEN) ) begin
           if (DEBUG && (iRADR2!=0)) begin
              if (~WMSK ==0)
                `MSGIO_DISPLAY("WARNING: reading and writing same location with full write mask (ADR = %x, WMSK = %x) in %m at %0t", iRADR2, WMSK, $time);
              else if ((DEBUG > 1) && (iRADR2!=0) && WARN_ON_PARTIAL_READ_WRITE_COLLISION)
                `MSGIO_DISPLAY("WARNING: reading and writing same location (ADR = %x, WMSK = %x) in %m at %0t", iRADR2, WMSK, $time);
	      if (DEBUG && (iRADR2 >= WORDS2))
                `MSGIO_DISPLAY("ERROR: Reading address that does not exist (iRADR = %x) in %m at %0t", iRADR2, $time);
           end
        end
   end // always @ (posedge iRCLK)
   // VCS coverage on
`endif //  `ifdef SIMULATION

   always @(posedge iRCLK)
     case (OUTPUT_REG)
       NON_PIPED_OUTPUT_EN: if (iOUTREG_EN)   mem_data_q <= mem_data;
       PIPED_OUTPUT_EN:     if (q_iOUTREG_EN) mem_data_q <= mem_data;
       default:                               mem_data_q <= 'bx;
     endcase

   assign oQ = (OUTPUT_REG != NO_OUTPUT_REG) ? mem_data_q : mem_data;


`ifdef SIMULATION
   // VCS coverage off
   wire [15:0] sim_width = DATA_WIDTH; 
   wire [15:0] sim_depth = WORDS;
   // VCS coverage on
`endif
          
endmodule // dum_mem
