//                              -*- Mode: Verilog -*-
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of ATI Technologies, Inc.
 *  Copyright 2005, ATI Technologies, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : fuse_if_fk_smsg
// Description : Fake SMSg for simulation only
//               The intention of this model is not to accurately test fuse_if.
//               The intention of this model is to help IP consumming RDFT a way to test
//               without having SMSg inserted in their IP, or offer a shortem MBIST test.
//               The intended usage if to cover the RSMU & RDFT connection.
//
//               repair_byte_count[ 7:0] : 0-> no repair
//               mbist_delay	  [11:0] : # of wrckLeaf to run for MBIST
//               mbist_fail	         : 1-> report fuse_if_sms_bist_fail == 1'b1
//               mbist_delay_ack  [11:0] : # of wrckLeaf to delay before returning Ack
////////////////////////////////////////////////////////////////////////////
// Perforce revision     : : #2 n// Perforce changelist   : : 2885168 n// Perforce date         : : 2017/06/27 n// Perforce Last edit by : : couyang n////////////////////////////////////////////////////////////////////////////
`define DEFAULT_REPAIR_BYTE_COUNT_GC10 8'h03
`define DEFAULT_MBIST_DELAY_GC10       12'h004
`define DEFAULT_MBIST_FAIL_GC10        1'b0
`define DEFAULT_MBIST_DELAY_ACK_GC10   12'h002

module fuse_if_fk_smsg_gc10 (
input              wrckLeaf         ,

output wire        smsg_ef_req      , // (Per SMSg) - Every toggle is a new request
output wire [ 6:0] smsg_ef_addr     , // (Per SMSg) - 7 bit byte address
input              smsg_ef_ready    , // (Per SMSg) - every toggle is ack for request
input       [ 7:0] smsg_ef_byte     , // (Per SMSg) - 8 bit efuse data, valid at smsg_ack toggle

output wire        smsg_sfp_ready   , // (Per SMSg) - SMSg ready to accept command
output wire        smsg_sfp_fail    , // (Per SMSg) - SMSg BIST or BISR fail status
input              smsg_sfp_request , // (Per SMSg) - Trigger signal for bihr, bist, and bisr
input              smsg_sfp_resetb  , // (Per SMSg) - Signal to reset SFP of SMSg (does not clear fuse)

input              smsg_bihr        , // launch Built In Hard Repair
input              smsg_bist        , // launch Built In Self Test
input              smsg_bisr        , // launch Built In Soft Repair
input              smsg_fclken        // Clock gater term for functional clock, synchroonizer in smsg
);
   

   // ==========================================================================
   // Program state of Main state machine (fkCtl)
   // Begin --------------------------------------------------------------------
   localparam       FKSMS_____READY = 4'b0000; // Reset start point
   localparam       FKSMS______DLY1 = 4'b0001; // Delay ack
   localparam       FKSMS_______ACK = 4'b0010; // Return not ready

   localparam       FKSMS_BIHR_DLY0 = 4'b1000; //
   localparam       FKSMS_BIHR__REQ = 4'b1001; //
   localparam       FKSMS_BIHR_SAVE = 4'b1010; //
   localparam       FKSMS_BIHR_DLY1 = 4'b1011; //
   localparam       FKSMS_BIHR_ADDR = 4'b1100; //
   localparam       FKSMS_BIHR_SHFT = 4'b1101; //

   localparam       FKSMS_MBISTDLY0 = 4'b0100; //
   localparam       FKSMS_MBISTFAIL = 4'b0101; //
   localparam       FKSMS_MBISTDLY1 = 4'b0110; //

   localparam       FKSMS___N___REQ = 4'b0011; // Wait for request 1'b1->1'b0
   // End ----------------------------------------------------------------------

   // ==========================================================================
   // Setup some parameters for test settings
   // Begin --------------------------------------------------------------------
   reg  [ 7:0]     repair_byte_count;
   reg  [11:0]     mbist_delay;
   reg             mbist_fail;
   reg  [11:0]     mbist_delay_ack;

`ifdef LEDA_TOOL
  always @(posedge wrckLeaf) begin
     repair_byte_count[ 7:0] <= { 8 {1'b0}};
     mbist_fail              <=      1'b0  ;
     mbist_delay      [11:0] <= {12 {1'b0}};
     mbist_delay_ack  [11:0] <= {12 {1'b0}};
  end
`else

   initial begin
      if ($value$plusargs("repair_byte_count+%d", repair_byte_count)) begin
      end else begin
         repair_byte_count[ 7:0] = `DEFAULT_REPAIR_BYTE_COUNT_GC10;
      end
      if ($value$plusargs("mbist_delay+%d", mbist_delay)) begin
      end else begin
         mbist_delay      [11:0] = `DEFAULT_MBIST_DELAY_GC10;
      end
      if ($value$plusargs("mbist_fail+%d", mbist_fail)) begin
      end else begin
         mbist_fail              = `DEFAULT_MBIST_FAIL_GC10;
      end
      if ($value$plusargs("mbist_delay_ack+%d", mbist_delay_ack)) begin
      end else begin
         mbist_delay_ack  [11:0] = `DEFAULT_MBIST_DELAY_ACK_GC10;
      end
   end

`endif // ifdef LEDA_TOOL

   // End ----------------------------------------------------------------------

   reg  [2047:0]   repairDat   ;
   wire [2047:0]   repairDatNxt;
   reg  [3:0]      fkCtl    ;
   reg  [3:0]      fkCtlNxt ;

   reg             req      ;
   reg  [7:0]      addr     ;
   reg             ready    ;
   reg             fail     ;

   reg             isBihr   ;
   reg             ef_readyTog ;
   reg             ef_ready_d1;
   wire [7:0]      addrMax  ;
   wire [7:0]      addrNxt  ;
   wire            addrAtMax;

   reg  [11:0]     cnt      ;
   reg  [11:0]     cntStart ;
   reg             cntEn    ;
   wire            cntDone  ;
   wire [11:0]     cntNxt  ;

   // ==========================================================================
   // Main state machine (fkCtl)
   // Begin --------------------------------------------------------------------
   always @(*) begin
      case (fkCtl[3:0])
        FKSMS_____READY : fkCtlNxt[3:0] =  smsg_sfp_request ? FKSMS______DLY1 : FKSMS_____READY;
        FKSMS______DLY1 : fkCtlNxt[3:0] =  cntDone          ? FKSMS______DLY1 : FKSMS_______ACK;
        FKSMS_______ACK : fkCtlNxt[3:0] =  isBihr           ? FKSMS_BIHR_DLY0 : FKSMS_MBISTDLY0;

        FKSMS_BIHR_DLY0 :
          case ({cntDone, (|addrMax[7:0])})
            2'b10   : fkCtlNxt[3:0] = FKSMS___N___REQ;
            2'b11   : fkCtlNxt[3:0] = FKSMS_BIHR__REQ;
            default : fkCtlNxt[3:0] = FKSMS_BIHR_DLY0;
          endcase // case ({cntDone, (|addrMax[7:0])})
        FKSMS_BIHR__REQ : fkCtlNxt[3:0] =                     FKSMS_BIHR_SAVE;
        FKSMS_BIHR_SAVE : fkCtlNxt[3:0] =  ef_readyTog      ? FKSMS_BIHR_DLY1 : FKSMS_BIHR_SAVE;
        FKSMS_BIHR_DLY1 : fkCtlNxt[3:0] =  cntDone          ? FKSMS_BIHR_SHFT : FKSMS_BIHR_ADDR;
        FKSMS_BIHR_ADDR : fkCtlNxt[3:0] =  addrAtMax        ? FKSMS_BIHR_SHFT : FKSMS_BIHR__REQ;
        FKSMS_BIHR_SHFT : fkCtlNxt[3:0] =  cntDone          ? FKSMS___N___REQ : FKSMS_BIHR_SHFT;

        FKSMS_MBISTDLY0 : fkCtlNxt[3:0] =  cntDone          ? FKSMS_MBISTFAIL : FKSMS_MBISTDLY0;
        FKSMS_MBISTFAIL : fkCtlNxt[3:0] =                     FKSMS_MBISTDLY1;
        FKSMS_MBISTDLY1 : fkCtlNxt[3:0] =  cntDone          ? FKSMS___N___REQ : FKSMS_MBISTDLY1;

        FKSMS___N___REQ : fkCtlNxt[3:0] =  smsg_sfp_request ? FKSMS___N___REQ : FKSMS_____READY;
        default         : fkCtlNxt[3:0] =  fkCtl[3:0];
      endcase // case (fkCtl[3:0])
   end

   always @(posedge wrckLeaf or negedge smsg_sfp_resetb)
     if (~smsg_sfp_resetb)
       fkCtl[3:0] <= FKSMS_____READY;
     else
       fkCtl[3:0] <= fkCtlNxt[3:0];
   // End ----------------------------------------------------------------------

   // ==========================================================================
   // outputs + Misc
   // Begin --------------------------------------------------------------------
   assign smsg_sfp_ready      = ready ;
   assign smsg_sfp_fail        = fail ;
   assign smsg_ef_req          = req  ;
   assign smsg_ef_addr   [6:0] = addr[6:0];
   assign repairDatNxt[2047:0] = {smsg_ef_byte[7:0], repairDat[2047:8]};

   always @(posedge wrckLeaf or negedge smsg_sfp_resetb) 
     if (~smsg_sfp_resetb) begin
        ready       <=     1'b1;
        fail        <=     1'b0;
        req         <=     1'b0;
        addr[7:0]   <= {8 {1'b0}};
        isBihr      <=     1'b0;
        ef_readyTog <=     1'b0;
        ef_ready_d1 <=     1'b0;
        repairDat[2047:0] <= {2048 {1'b0}};
     end else begin
        ready     <= (fkCtl[3:0] == FKSMS_____READY) | (fkCtl[3:0] == FKSMS___N___REQ) | (fkCtl[3:0] == FKSMS______DLY1);
        fail      <= (fkCtl[3:0] == FKSMS_MBISTFAIL) ? mbist_fail   : fail;
        req       <= (fkCtl[3:0] == FKSMS_BIHR__REQ) ? ~req         : req ;
        addr[7:0] <= (fkCtl[3:0] == FKSMS_BIHR__REQ) ? addrNxt[7:0] : addr[7:0];
        repairDat[2047:0] <= ef_readyTog ? repairDatNxt[2047:0] : repairDat[2047:0];
        if (fkCtl[3:0] == FKSMS_____READY)
          case ({smsg_bihr, smsg_bist, smsg_bisr})
            3'b100  : isBihr    <= 1'b1;
            3'b010  : isBihr    <= 1'b0;
            default : isBihr    <= 1'bx;
          endcase // case ({smsg_bihr, smsg_bist, smsg_bisr})
        else
          isBihr    <= isBihr;
        ef_ready_d1 <= smsg_ef_ready;
        ef_readyTog <= smsg_ef_ready ^ ef_ready_d1;
     end

   assign addrAtMax    = addr[7:0] == addrMax[7:0];
   assign addrMax[7:0] = repair_byte_count[7:0];
   assign addrNxt[7:0] = addr[7:0] + {{7 {1'b0}}, 1'b1};
   // End ----------------------------------------------------------------------

   // ==========================================================================
   // counter (cnt)
   // Begin --------------------------------------------------------------------
   assign cntDone      = cnt[11:0] == {12 {1'b0}};
   assign cntNxt[11:0] = cnt[11:0] - {{11 {1'b0}}, 1'b1};
   always @(*) begin
      case (fkCtl[3:0])
        FKSMS_MBISTDLY1 : cntStart[11:0] = mbist_delay[11:0];
        FKSMS_BIHR_SHFT : cntStart[11:0] = {{4 {1'b0}}, repair_byte_count[7:0]} << 4; // assmue each byte takes 16 cycle to shift out
        default         : cntStart[11:0] = mbist_delay_ack[11:0];
      endcase // case (fkCtl[3:0])
   end
   always @(posedge wrckLeaf or negedge smsg_sfp_resetb)
     if (~smsg_sfp_resetb) begin
        cnt[11:0] <= cntStart[11:0];
        cntEn    <= 1'b0;
     end else begin
        cnt[11:0] <= cntEn ? cntNxt[11:0] : cntStart[11:0];
        case (fkCtl[3:0])
          FKSMS_____READY, FKSMS_______ACK,
          FKSMS_BIHR__REQ, FKSMS_BIHR_SAVE,
          FKSMS_BIHR_ADDR, FKSMS_MBISTFAIL,
          FKSMS___N___REQ                  : cntEn    <= 1'b0;

          FKSMS______DLY1, FKSMS_BIHR_DLY0,
          FKSMS_BIHR_DLY1, FKSMS_BIHR_SHFT,
          FKSMS_MBISTDLY0, FKSMS_MBISTDLY1 : cntEn    <= 1'b1;
          default : cntEn    <= cntEn;
        endcase // case (fkCtl[3:0])
     end
   // End ----------------------------------------------------------------------

endmodule // fuse_if_fk_smsg_gc10
