`include "grbm_defs.vh"
`include "register_addr.v"

module gc_didt_stall_release_cntl (
    //input
    //note that didt_stall_delay & didt_release_delay should not be zero
    stall_in,
    didt_stall_release_cntl_en,
    didt_stall_delay,
    didt_max_stalls_allowed,
    didt_release_delay,
    didt_base_release_allowed,
    didt_incr_release_allowed,

    clk,
    resetb,
    TEST_CLK_EN, //reserved

    //output
    stall_out,
    didt_stall_release_cntl_fsm_state
);

    parameter  NUM_INSTANCE = 16; //will not be 1 or 0. If 1, don't instance this block in gc_didt_cntl.v
    localparam DIDT_IDLE = 2'b00;
    localparam DIDT_STALL = 2'b01;
    localparam DIDT_RELEASE = 2'b10;

    input       stall_in;
    input       didt_stall_release_cntl_en;
    input reg [5:0]     didt_stall_delay;
    input reg [5:0]     didt_max_stalls_allowed;
    input reg [10:0]    didt_release_delay;
    input reg [4:0]     didt_base_release_allowed;
    input reg [4:0]     didt_incr_release_allowed;

    input       TEST_CLK_EN;
    input       clk;
    input       resetb;

    output [NUM_INSTANCE-1:0] stall_out;
    output [1:0] didt_stall_release_cntl_fsm_state;

    reg  stall_in_d;
    wire stall_raise;
    wire stall_fall;

    always @(posedge clk) begin
      if(resetb) begin
        stall_in_d <= 1'b0;
      end else if (didt_stall_release_cntl_en) begin
        stall_in_d <= stall_in;
      end else begin
        stall_in_d <= 1'b0;
      end
    end

    assign stall_raise = stall_in && ~stall_in_d;
    assign stall_fall = ~stall_in && stall_in_d;

    wire [5:0] didt_max_stalls_allowed_clamped;
    assign didt_max_stalls_allowed_clamped = (didt_max_stalls_allowed > NUM_INSTANCE) ? NUM_INSTANCE : didt_max_stalls_allowed;

   reg  [5:0]  curr_stall_nums;
   reg  [NUM_INSTANCE-1:0] stall_on_mask;
   reg  [NUM_INSTANCE-1:0] stall_on_mask_d;
   reg  [NUM_INSTANCE-1:0] stall_on_mask_t;


   integer j;
   always @*
   begin
     curr_stall_nums = 6'b0;
     for (j = 0; j < NUM_INSTANCE; j=j+1) begin
       if (stall_on_mask[j] == 1) begin
         curr_stall_nums = curr_stall_nums + 1'b1;
       end
     end
   end

   reg [5:0] stall_end_index; 
   reg [5:0] stall_start_index;

   integer k;
   always @*
   begin
     stall_end_index = 6'b0;
     for (k = 0; k < (NUM_INSTANCE - 1); k=k+1) begin
       if ((stall_on_mask[k] == 1) && (stall_on_mask[k+1] == 0)) begin
         stall_end_index = k;
       end
     end
     if ((stall_on_mask == {NUM_INSTANCE{1'b1}}) || ((stall_on_mask[NUM_INSTANCE - 1] == 1'b1) && (stall_on_mask[0] == 1'b0))) begin
       stall_end_index = NUM_INSTANCE-1;
     end 
   end

   integer p;
   always @*
   begin
     stall_start_index = 6'b0;
     for (p = 1; p < NUM_INSTANCE; p=p+1) begin
       if ((stall_on_mask[p] == 1) && (stall_on_mask[p-1] == 0)) begin
         stall_start_index = p;
       end
     end
   end

   integer i, m, n;
   reg  [5:0]  stall_counter;
   reg  [5:0]  didt_release_num_d;
   reg  [5:0]  didt_release_num;
   reg  [1:0]  next_fsm_state,curr_fsm_state, curr_fsm_state_dly;
   reg  [10:0] didt_stall_release_delay;
   reg  [10:0] didt_delay_counter_d;
   reg  [10:0] didt_delay_counter;
   wire [5:0]  next_stall_nums;

   assign next_stall_nums = (didt_max_stalls_allowed_clamped > curr_stall_nums) ? (didt_max_stalls_allowed_clamped - curr_stall_nums) : 6'b0;
   
   // FSM 
   always @*
   begin

     next_fsm_state = curr_fsm_state;
     didt_delay_counter_d = didt_delay_counter;
     stall_on_mask_d = stall_on_mask;
     stall_on_mask_t = stall_on_mask;
     didt_stall_release_delay = 11'b0;
     didt_release_num_d = didt_release_num;
     stall_counter = 6'b0;

     case(curr_fsm_state)
       DIDT_IDLE:
                begin
                  stall_on_mask_d = {NUM_INSTANCE{1'b0}};
                  didt_stall_release_delay = 11'b0;
                  didt_delay_counter_d = 11'b0;

                  if (stall_raise && didt_stall_release_cntl_en) begin
                    next_fsm_state = DIDT_STALL;
                  end
                end
       DIDT_STALL:
                begin
                  didt_stall_release_delay = {5'b0, didt_stall_delay};
                  didt_delay_counter_d = didt_delay_counter + 1'b1;

                  if (didt_max_stalls_allowed_clamped > curr_stall_nums) begin
                    if (stall_on_mask == 0) begin
                      stall_on_mask_t = (1 << didt_max_stalls_allowed_clamped) - 1;
                    end else begin
                      for (i = 0; i < NUM_INSTANCE; i=i+1) begin
                        if (i < next_stall_nums) begin
                          if ((stall_end_index + i + 1) >= NUM_INSTANCE) begin
                            stall_counter = stall_end_index + i + 1 - NUM_INSTANCE;
                          end else begin
                            stall_counter = stall_end_index + i + 1;
                          end
                          stall_on_mask_t[stall_counter] = 1'b1;
                        end
                      end
                    end
                    if (didt_delay_counter_d >= didt_stall_release_delay) begin
                      stall_on_mask_d = {stall_on_mask_t[NUM_INSTANCE-2:0], stall_on_mask_t[NUM_INSTANCE-1]};
                    end else begin
                      stall_on_mask_d = stall_on_mask_t;
                    end
                  end else if (didt_max_stalls_allowed_clamped < curr_stall_nums) begin
                    if (didt_delay_counter_d >= didt_stall_release_delay) begin
                      stall_on_mask_d[stall_start_index] = 1'b0;
                    end
                  end else begin
                    if (didt_delay_counter_d >= didt_stall_release_delay) begin
                      stall_on_mask_d = {stall_on_mask[NUM_INSTANCE-2:0], stall_on_mask[NUM_INSTANCE-1]};
                    end
                  end
                  if (didt_stall_release_cntl_en) begin
                    if (stall_fall) begin
                      next_fsm_state = DIDT_RELEASE;
                      didt_delay_counter_d = 11'b0;
                    end
                  end else begin
                    next_fsm_state = DIDT_IDLE;
                  end
                end
       DIDT_RELEASE:
                begin
                  didt_stall_release_delay = didt_release_delay;
                  didt_delay_counter_d = didt_delay_counter + 1'b1;

                  if (curr_fsm_state != curr_fsm_state_dly) begin
                    didt_delay_counter_d = 11'b0;
                    didt_release_num_d = {1'b0, didt_base_release_allowed};
                    if (curr_stall_nums >= didt_base_release_allowed) begin
                      for (m = 0; m < NUM_INSTANCE; m=m+1) begin
                        if (m < didt_release_num_d) begin
                          if ((stall_start_index + m) >= NUM_INSTANCE) begin
                            stall_counter = stall_start_index + m - NUM_INSTANCE;
                          end else begin
                            stall_counter = stall_start_index + m;
                          end
                          stall_on_mask_d[stall_counter] = 1'b0;
                        end
                      end
                    end
                  end else if (didt_delay_counter_d >= didt_stall_release_delay) begin
                    if ((didt_release_num + didt_incr_release_allowed) > NUM_INSTANCE) begin
                      didt_release_num_d = NUM_INSTANCE;
                    end else begin
                      didt_release_num_d = didt_release_num + {1'b0, didt_incr_release_allowed};
                    end
                    if (curr_stall_nums <= didt_release_num_d) begin
                      stall_on_mask_d = {NUM_INSTANCE{1'b0}};
                    end else begin
                      for (n = 0; n < NUM_INSTANCE; n=n+1) begin
                        if (n < didt_release_num_d) begin
                          if ((stall_start_index + n) >= NUM_INSTANCE) begin
                            stall_counter = stall_start_index + n - NUM_INSTANCE;
                          end else begin
                            stall_counter = stall_start_index + n;
                          end
                          stall_on_mask_d[stall_counter] = 1'b0;
                        end
                      end
                    end
                  end

                  if (didt_stall_release_cntl_en) begin
                    if (stall_raise) begin
                      next_fsm_state = DIDT_STALL;
                      didt_delay_counter_d = 11'b0;                      
                    end 
                  end else begin
                    next_fsm_state = DIDT_IDLE;
                  end
                end
       default: 
                begin
                  stall_on_mask_d = {NUM_INSTANCE{1'b0}};
                  didt_stall_release_delay = 11'b0;
                  didt_delay_counter_d = 11'b0;

                  if (stall_raise && didt_stall_release_cntl_en) begin
                    next_fsm_state = DIDT_STALL;
                  end
                end
     endcase
   end

    always @(posedge clk) begin
        if(resetb) begin
          didt_delay_counter <= 11'b0;
        end else begin
          if ((didt_delay_counter >= (didt_stall_release_delay - 1)) || (didt_stall_release_cntl_en == 1'b0)) begin
            didt_delay_counter <= 11'b0;
          end else begin
            didt_delay_counter <= didt_delay_counter_d;
          end
        end
    end

    always @(posedge clk) begin
        if(resetb) begin
          didt_release_num <= 6'b0;
        end else if (didt_stall_release_cntl_en) begin
          didt_release_num <= didt_release_num_d;
        end else begin
          didt_release_num <= 6'b0;
        end
    end

    always @(posedge clk) begin
        if(resetb) begin
          curr_fsm_state <= DIDT_IDLE;
        end else if (didt_stall_release_cntl_en) begin
          curr_fsm_state <= next_fsm_state;
        end else begin
          curr_fsm_state <= DIDT_IDLE;
        end
    end

    always @(posedge clk) begin
        if(resetb) begin
          curr_fsm_state_dly <= DIDT_IDLE;
        end else if (didt_stall_release_cntl_en) begin
          curr_fsm_state_dly <= curr_fsm_state;
        end else begin
          curr_fsm_state_dly <= DIDT_IDLE;
        end
    end

    always @(posedge clk) begin
        if(resetb) begin
          stall_on_mask <= {NUM_INSTANCE{1'b0}};
        end else if (didt_stall_release_cntl_en) begin
          stall_on_mask <= stall_on_mask_d;
        end else begin
          stall_on_mask <= {NUM_INSTANCE{1'b0}};
        end
    end

    assign stall_out = didt_stall_release_cntl_en ? stall_on_mask : {NUM_INSTANCE{stall_in}};
    assign didt_stall_release_cntl_fsm_state = didt_stall_release_cntl_en ? curr_fsm_state : DIDT_IDLE;

endmodule
