//--------------------------------------------------------------------------
//
// $Id: $
//
// $Change: $
//
// Notes: This is a module that contains the logic for the consumer module,
//        but does NOT include the actual FIFO to store data. This was done so
//        that the individual designer could taylor this as needed. As a result,
//        there are some additional ins/outs than one might not have expected.
//        DATA_WIDTH, FIFO_DEPTH, and LOW_LATENCY are parameters which 
//        characterize this file. LOW_LATENCY=1 (default) specifies to use
//        "Consumer Option 2" as specified in the Pele interfaces doc.  
//
// Copyright: Trade secret of ATI Technologies, Inc.
//           (C) Copyright 2001-2004, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
// Author: Brian Rerecich
//--------------------------------------------------------------------------

module gfx_ati_consumer_ctrl (
			  // Outputs
			  data_out,
			  free,
			  rts,
			  
			  data_in_q,
			  push,
			  pop,
			  
			  // Inputs
			  data_in,
			  send,
			  rtr,
			  
			  fifo_data_out,
			  fifo_empty,
			  
			  clk,
			  reset
			  );
   //**************
   //* Parameters *
   //**************

   parameter DATA_WIDTH  = 1;
   parameter FIFO_DEPTH  = 2;
   parameter LOW_LATENCY = 1;
   parameter CG_EN       = 0; // adds if(EN) .. structure... so that clock gating is added by the tool
    	    	    	      //
    	    	    	      // Added this new parameter so that all existing designs will not results 
			      // in timing/congestion issue 	
    	
   parameter ADDR_WIDTH  = ((FIFO_DEPTH <= 4) ? 2 :
                            (FIFO_DEPTH <= 8) ? 3 :
                            (FIFO_DEPTH <= 16) ? 4 :
                            (FIFO_DEPTH <= 32) ? 5 :
                            (FIFO_DEPTH <= 64) ? 6 :
                            (FIFO_DEPTH <= 128) ? 7 :
                            (FIFO_DEPTH <= 256) ? 8 :
                            (FIFO_DEPTH <= 512) ? 9 : 
			    (FIFO_DEPTH <= 1024) ? 10 :
			    (FIFO_DEPTH <= 2048) ? 11 : 12);  // Bigger than 4096???  i hope not.
   
   //***********************
   //* Output Declarations *
   //***********************
   output [DATA_WIDTH-1:0] data_out;  // Data bus from FIFO
   output 		   free;      // Signal to PRODUCER to "free" a credit
   output 		   rts;       // Ready To Send (There's some data ready to be consumed)

   output [DATA_WIDTH-1:0] data_in_q; // Flopped data bus
   output 		   push;      // Push data_in_q to FIFO
   output 		   pop;       // Pop data from FIFO
   
   //**********************
   //* Input Declarations *
   //**********************
   input [DATA_WIDTH-1:0]  data_in;       // Data bus from PRODUCER delayed by N cycles
   input 		   send;          // Signal from PRODUCER thats says it's sent data
   input 		   rtr;           // Ready To Receive (Pop top entry in the FIFO)

   input [DATA_WIDTH-1:0]  fifo_data_out; // Data from FIFO
   input 		   fifo_empty;    // empty signal from FIFO
  
   input 		   clk;
   input 		   reset;

   //***************************
   //* Output REG Declarations *
   //***************************
   reg [DATA_WIDTH-1:0]    data_out;
   reg 			   free;
   reg 			   rts;

   reg [DATA_WIDTH-1:0]    data_in_q;
   reg 			   push;
   reg 			   pop;
   
   //*****************************
   //* Internal REG Declarations *
   //*****************************   
   reg 			   send_q;             // Flopped version of send

   reg 			   free_d;             // D Pin of output flop for "free"

   //reg 			   skip_fifo_mux_sel;  // Mux Selection for data output (From FIFO or Input FF)
   reg                     empty_and_rtr;      // FIFO is empty and CONSUMER is Ready To Receive
   reg                     rtr_and_send_q;     // CONSUMER is Ready To Receive and PRODUCER is sending
   
   //*********************
   //* Wire Declarations *
   //*********************
   wire [DATA_WIDTH-1:0]   fifo_data_out;      // Read data from FIFO
    		   
   //***********************
   //* Combinatorial Logic *
   //***********************
   always @(/*AUTOSENSE*/fifo_empty or rtr or send_q) begin
      
      // There are common terms here, so bring it out for DC. 
      empty_and_rtr     =  fifo_empty & rtr;
      rtr_and_send_q    =  rtr & send_q;
      
      //skip_fifo_mux_sel =  empty_and_rtr & send_q;

      pop               = ~fifo_empty & rtr;
      push              = LOW_LATENCY ? (~empty_and_rtr & send_q)              : send_q;
      free_d            = LOW_LATENCY ? (pop | rtr_and_send_q)                 : pop; 
      rts               = LOW_LATENCY ? (~fifo_empty | rtr_and_send_q)         : ~fifo_empty;
      
   end

   always @(/*AUTOSENSE*/data_in_q or fifo_data_out or fifo_empty) begin
      data_out = LOW_LATENCY ? (fifo_empty ? data_in_q : fifo_data_out) : fifo_data_out;
   end

   //*********************
   //* FFs without reset *
   //*********************
   always @ (posedge clk) begin
      
       case(CG_EN)
        1'd1: if(send) data_in_q <= data_in;
        1'd0: 	       data_in_q <= data_in;
       endcase
       
   end
   
   //******************
   //* FFs with reset *
   //******************
   always @ (posedge clk) begin
      if (reset) begin
	 send_q  <= 1'b0;
	 free    <= 1'b0;
      end
      else begin
	 send_q  <= send;
	 free    <= free_d;
      end
   end // always @ (posedge clk)   

`ifdef SIMULATION
  //VCS coverage off

  ati_fifo_cover_assert #(FIFO_DEPTH) ati_fifo_cover_assert 
        ( .clk(clk), 
          .resetb(~reset), 
          .push(send), 
          .pop(free) );

   // VCS coverage on


`endif

endmodule
