//--------------------------------------------------------------------------
//
// Notes:     This is a generic design of a register based fifo. It's size is determned
//            by the paramters: FIFO_WIDTH, ADDR_WIDTH, FIFO_DEPTH and SKID_WORKDS. When
//            SKID_WORDS = 0 ... this represents a fifo with a FULL flag. SKID_WORDS 
//            controls the setting of full flag based on FULL = FIFO_DEPTH - SKID_WORDS - 1
//
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

module gfx_ati_fifo_regout (
   we,
   re,
   full,
   empty,
   write_data,
   read_data, 

   clk,
   reset
);
    //============================
    // PARAMETERS
    //============================
    parameter FIFO_WIDTH = 0;        
    parameter ADDR_WIDTH = 0;        
    parameter FIFO_DEPTH = 0;
    parameter SKID_WORDS = 0;
    parameter FIFO_NAME  = "";
    parameter EXTERNAL_FULL_REGISTER  = 0;   // set this parameter if you which to register the full signal external to this block
    //============================
    // IO PORTS
    //============================
    input    [FIFO_WIDTH-1:0] write_data;
    input                     we;
    output                    full;
    output   [FIFO_WIDTH-1:0] read_data;
    output                    empty;
    input                     re;
    input                     clk;
    input                     reset;

    //============================
    // INTERNAL SIGNALS
    //============================
    // read data register
    wire [FIFO_WIDTH-1:0]  read_data;

    // signals to the mem
    wire [ADDR_WIDTH-1:0]  reg_waddr;
    wire [ADDR_WIDTH-1:0]  reg_raddr;

    reg  [FIFO_WIDTH-1:0]  reg_mem [FIFO_DEPTH - 2:0];
    reg  [FIFO_WIDTH-1:0]  reg_out_q;
    wire [FIFO_WIDTH-1:0]  reg_out_d;
    reg 		   reg_out_valid_q;
    reg 		   reg_out_valid_d;

    // ***********************************
    // I/O Definitions
    // ***********************************

    // signals used in this module
    // The address width of the fifo_cnt_q signal will be 1 bit too wide
    // if the depth is not a power of two; however, this case works for all values.
    reg                     fifo_we;    // internal we, clocks write ptr
    reg                     fifo_re;    // internal re, clocks read ptr

    reg   [ADDR_WIDTH-1:0]  fifo_wptr_d, fifo_wptr_q, fifo_rptr_d, fifo_rptr_q;
    reg   [ADDR_WIDTH:0]    fifo_cnt_d, fifo_cnt_q;

    reg                     fifo_full_d, fifo_full_q;
    reg                     int_fifo_empty_d, int_fifo_empty_q;   // empty for the internal fifo

    integer i;

    // -------------------------------------------
    // concurrent assignments
    // -------------------------------------------
//    assign fifo_we = we;
//    assign fifo_re = re & ~int_fifo_empty_q;
//    assign empty = int_fifo_empty_q;
    assign empty = !reg_out_valid_q;
    assign full = ( EXTERNAL_FULL_REGISTER == 0 ? fifo_full_q : fifo_full_d);
    assign reg_waddr = fifo_wptr_q;
    assign reg_raddr = fifo_rptr_q;    // note not registered
//    assign read_data = reg_mem[reg_raddr];
    assign read_data = reg_out_q;

    // -------------------------------------------
    // clock process
    // -------------------------------------------
    always @(posedge clk) begin
        fifo_wptr_q <= fifo_wptr_d;
        fifo_rptr_q <= fifo_rptr_d;
        fifo_cnt_q <= fifo_cnt_d;
        fifo_full_q <= fifo_full_d;
        int_fifo_empty_q <= int_fifo_empty_d;
	reg_out_valid_q  <= reg_out_valid_d;
	reg_out_q        <= reg_out_d;

`ifdef SIMULATION
	// VCS coverage off
        // error checking
        if ( fifo_we && ~fifo_re && (fifo_cnt_q == (FIFO_DEPTH-1) ) ) begin
	   // leda DCVER_274 off
            `MSGIO_DISPLAY("ATI_Fifo ERROR (%m): %s fifo written when full -- data dropped at time %t", FIFO_NAME, $time);
           // leda DCVER_274 on
	end
	// VCS coverage on
`endif
    end // always clk

    // setup the register input
    always @ ( we or
               re or
               fifo_cnt_q or
               fifo_rptr_q or
               fifo_wptr_q or
               fifo_full_q or
               int_fifo_empty_q or
               reg_out_valid_q or
               reset
             ) begin

        // assign default inputs
        fifo_wptr_d = fifo_wptr_q;
        fifo_rptr_d = fifo_rptr_q;
        fifo_cnt_d = fifo_cnt_q;
        fifo_full_d = fifo_full_q;
        int_fifo_empty_d = int_fifo_empty_q;
	reg_out_valid_d  = reg_out_valid_q;
        fifo_we = we;
        fifo_re = re & ~int_fifo_empty_q;

        // If the reg_out is empty then store directly into outgoing register, saves on latency.
	if ( we & !reg_out_valid_q ) begin
	    reg_out_valid_d = 1'b1;
	    fifo_we = 1'b0;
	end
	// If the fifo is empty and we are reading and writing then write directly into reg_out.
	else if( we & re & int_fifo_empty_q ) begin
	    reg_out_valid_d = 1'b1;
	    fifo_we = 1'b0;
	end
        else if ( we ) begin
            if ( fifo_wptr_q == ( FIFO_DEPTH - 2 ) ) begin
                fifo_wptr_d = 'b0;
            end else begin
                fifo_wptr_d = fifo_wptr_q + 1;
            end
	    fifo_we = 1'b1;
	end
	else begin
	    fifo_we = 1'b0;
        end

        // If reading only from reg_out don't change fifo.
	if ( !we & re & int_fifo_empty_q & reg_out_valid_q ) begin
            reg_out_valid_d = 1'b0;
	    fifo_re = 1'b0;
        end
	else if ( !we & re & int_fifo_empty_q & !reg_out_valid_q ) begin
	    // nothing to read or write.
	    fifo_re = 1'b0;
        end
	else if ( we & re & int_fifo_empty_q ) begin	// If nothing to read. 
	    // already handled above.
	    fifo_re = 1'b0;
	end
        else if ( re ) begin
	    fifo_re = 1'b1;
            if ( fifo_rptr_q == ( FIFO_DEPTH - 2 ) ) begin
                fifo_rptr_d = 'b0;
            end else begin
                fifo_rptr_d = fifo_rptr_q + 1;
            end
	    // When reading out load registered output with next entry.
	    reg_out_valid_d = 1'b1;
        end

        // immediate data fifo empty/full logic
        case ( { fifo_we, fifo_re } )
            2'b10 :
                // write, no read
                begin
                    if ( fifo_cnt_q == FIFO_DEPTH - 2 - SKID_WORDS ) begin
                        fifo_full_d = 1'b1;
                    end
                    fifo_cnt_d = fifo_cnt_q + 1;
                    int_fifo_empty_d = 1'b0;
                end
            2'b01 : 
                // read, no write
                begin
                    if ( fifo_cnt_q == 1 ) begin
                        int_fifo_empty_d = 1'b1;
                    end
                    if ( fifo_cnt_q == FIFO_DEPTH - 2 - SKID_WORDS + 1) begin
                    	fifo_full_d = 1'b0;
                    end
                    fifo_cnt_d = fifo_cnt_q - 1;
                end
            default: begin
                fifo_cnt_d = fifo_cnt_q;
                fifo_full_d = fifo_full_q;
                int_fifo_empty_d = int_fifo_empty_q;
            end
        endcase

        // -------------------------------------------
        // reset-able registers
        // -------------------------------------------
        // put this last in the clock process to override
        // anything that was assigned above
        if ( reset ) begin
            // immediate data fifo registers
            fifo_wptr_d = 'b0;
            fifo_rptr_d = 'b0;
            fifo_cnt_d = 'b0;
            fifo_full_d = 1'b0;
            int_fifo_empty_d = 1'b1;
            reg_out_valid_d = 1'b0;
        end

    end

   //HM timing fix. Data is selected concurrently not gated by fifo_re
   wire [FIFO_WIDTH-1:0] array_data = reg_mem[reg_raddr];
   
    assign reg_out_d = (( we & !reg_out_valid_q ) || ( we & re & int_fifo_empty_q )) ? write_data :
		          fifo_re ?  array_data : reg_out_q;

    //============================
    // RESIGER BASED STORAGE
    // This particular coding was used in the hopes that synopsys and power compiler
    // would be able to recognize the enabled nature of these flops.
    // see following article for more information
    // Author: thomas.esposito@sandburst.com
    // Date: 2001-08-17 08:15:08 PST
    // Subject: Synthesis of Verilog array assignment with Synopsys 
    // Newsgroups: comp.cad.synthesis
    //============================
    always @(posedge clk)
    begin
        if (we) begin
            for (i = 0; i <= FIFO_DEPTH - 2; i = i + 1) begin
                if (i == reg_waddr) begin
                    reg_mem[i] <= write_data;
                end
            end
        end
    end // always clk

endmodule //ati_fifo
