//--------------------------------------------------------------------------
//
// Notes:     Simply holds data for N stages.  Can be collapsible.
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            Copyright 2001-2002, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

module gfx_ati_pipe_data (/*AUTOARG*/
   // Outputs
   busy, in_rtr, out_valid, out_data, 
   // Inputs
   clk, reset, in_valid, in_push, in_data, out_rtr, out_pop
   );
   
   //============================
   // PARAMETERS
   //============================
   parameter WIDTH = 0;
   parameter DEPTH = 0;
   parameter COLLAPSIBLE = 1'b0;
   parameter LOW_POWER = 1;                // Setting this to 0 will reduce area, increase power, and timing should be the same.
   
   //============================
   // IO PORTS
   //============================
   input              clk;
   input              reset;
   output             busy;

   output             in_rtr;
   input              in_valid;    // Used for non collapsible pipes.
   input              in_push;     // Used for collapsible pipes.
   input [WIDTH-1:0]  in_data;

   output             out_valid;
   input              out_rtr;     // Used for non collapsible pipes.           
   input              out_pop;     // Used for collapsible pipes.
   output [WIDTH-1:0] out_data;
   
   //============================
   // Control
   //============================

   reg [DEPTH-1:0] valid_q;
   wire [DEPTH:0]  valid = {valid_q, (COLLAPSIBLE ? in_push : in_valid) };
   reg [DEPTH:0]   rtr;
   integer         i;
   always @(/*AS*/out_pop or out_rtr or valid) begin
      rtr[DEPTH] = (COLLAPSIBLE ? out_pop : out_rtr);
      for (i = DEPTH-1; i >= 0; i = i - 1)
        rtr[i] = rtr[i+1] || (COLLAPSIBLE && !valid[i+1]);
   end
   
   wire [DEPTH:0] clk_en = LOW_POWER ? (rtr & valid) : rtr;
   
   integer         k;
   reg [WIDTH-1:0] data[DEPTH-1:0];
   always @(posedge clk) begin
      for (k = 0; k < DEPTH; k = k + 1) begin
         if (rtr[k])
           valid_q[k] <= valid[k];
         
         if (clk_en[k])
           if (k == 0)
             data[k] <= in_data;
           else
             data[k] <= data[k-1];
      end
      
      if (reset)
        valid_q <= 'b0;

      // error checking
`ifdef SIMULATION
      // VCS coverage off
      if ( COLLAPSIBLE && in_push && !in_rtr ) begin
         `MSGIO_DISPLAY("gfx_ati_pipe_data ERROR (%m): pushed when pipe was full at time %0t", $time);
      end
      if ( COLLAPSIBLE && out_pop && !out_valid ) begin
         `MSGIO_DISPLAY("gfx_ati_pipe_data ERROR (%m): popped when no data was valid at time %0t", $time);
      end
      // VCS coverage on
`endif
   end

   
   //===================================================================
   // Assign outputs
   //===================================================================
   assign in_rtr    = rtr[0];
   assign out_valid = valid[DEPTH];
   assign out_data  = data[DEPTH-1];
   assign busy      = |valid_q;
   
endmodule //gfx_ati_pipe_data
