//--------------------------------------------------------------------------
//
// $Id: $
//
// $Change: $
//
// Notes:     This is a module that contains the logic for the producer module,
//            as specified in the Pele Interfaces Spec. DATA_WIDTH, RST_CREDIT_VAL,
//            and CREDIT_WIDTH are parameters that specify the width of the data 
//            bus, the value of the credit counter at reset, and the width of the
//            value for credit. 
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2004, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
// Author: Brian Rerecich
//
//--------------------------------------------------------------------------
module gfx_ati_producer(
		    // Outputs
		    data_out,
		    send,
		    rtr,
		    
		    // Inputs
		    data_in,
		    free,
		    rts,
		    rbbm_credit,
		    rbbm_credit_we,
		    
		    clk,
		    reset
		    );
   //**************
   //* Parameters *
   //**************

   parameter DATA_WIDTH     = 1;
   parameter RST_CREDIT_VAL = 1;
   parameter CREDIT_WIDTH   = 2;
   parameter FREE_INC_COUNT = 1;

   //***********************
   //* Output Declarations *
   //***********************
   output [DATA_WIDTH-1:0]  data_out;       // Flopped version on data_in
   output 		    send;           // Data is being send to consumer
   output 		    rtr;            // Ready To Receive (more like consumer is ABLE to receive)	   
   
   //**********************
   //* Input Declarations *
   //**********************
   input [DATA_WIDTH-1:0]   data_in;        // Data to flop and send to consumer
   input 		    free;           // Consumer signals it "freed" a credit. 		   
   input 		    rts;            // Ready To Send (Valid data on the data_in bus)
   input [CREDIT_WIDTH-1:0] rbbm_credit;    // Value to load into Credit FF
   input 		    rbbm_credit_we; // Write Enable for rbbm_credit value
   
   input 		    clk;
   input 		    reset;

   //***************************
   //* Output REG Declarations *
   //***************************
   reg [DATA_WIDTH-1:0]    data_out;
   reg 			   send;
   reg 			   rtr;
		
   //*****************************
   //* Internal REG Declarations *
   //*****************************   
   reg [CREDIT_WIDTH-1:0]  credit_q;
   reg [CREDIT_WIDTH-1:0]  credit_d;
   reg 			   credit_non_zero;
   reg 			   send_d;
   reg 			   free_q;
			   
   //*********************
   //* Wire Declarations *
   //*********************


   //***********************
   //* Combinatorial Logic *
   //***********************
   always @(/*AUTOSENSE*/credit_q or free_q or rts) begin
  
      credit_non_zero = |(credit_q[CREDIT_WIDTH-1:0]);

      rtr    = credit_non_zero | free_q;
      send_d = rts & rtr;

   end
   always @(/*AUTOSENSE*/credit_non_zero or credit_q or free_q or rbbm_credit or rbbm_credit_we or rts) begin
      casez({rbbm_credit_we,rts,credit_non_zero,free_q})
	4'b00?1: credit_d = credit_q + FREE_INC_COUNT; // ~rbbm_we &  free_q & ~send
        4'b0101: credit_d = credit_q + FREE_INC_COUNT - 1; // ~rbbm_we & free_q & send
	4'b0110: credit_d = credit_q - 1; // ~rbbm_we & ~free_q & send
        4'b0111: credit_d = credit_q + FREE_INC_COUNT - 1; // ~rbbm_we & free_q & send
	4'b1???: credit_d = rbbm_credit;  //  rbbm_we
        default: credit_d = credit_q;     //  All other cases 
      endcase // casez({rbbm_credit_we,rts,credit_non_zero,free_q})
   end
   
   //*********************
   //* FFs without reset *
   //*********************
   always @ (posedge clk) begin
      if (send_d || reset) // reset is here because this eliminates some X's on some portion of the design (in particular, bif)
	data_out <= data_in;
   end
   
   //******************
   //* FFs with reset *
   //******************
   always @ (posedge clk) begin
      if (reset) begin
	 send     <= 1'b0;
	 free_q   <= 1'b0;
	 credit_q <= RST_CREDIT_VAL;
      end
      else begin
	 send     <= send_d;
	 free_q   <= free;
	 credit_q <= credit_d;
      end
   end // always @ (posedge clk)   

endmodule // gfx_ati_producer
