/////////////////////////////////////////////////////////////////////////
//
//                            Copyright 2001
//                            ATI Research, Inc.
//                            All Rights Reserved
//
//  Filename: gfx_ati_rd_rtn_32toN.v
//
//  Filetype   : Verilog
//  Description: Converts a 32bit input into a serialized data stream. 
//  The transfer size is parameterized with N powers of 2 payload clocks 
//  (1,2,4,8,16) and one clock for the header.
//
//  Heiarchy:
//
//  Additional Files
//
/////////////////////////////////////////////////////////////////////////
//                        Modification History
//
//   Date       Author         Revision    Comments
//   -------    -----------    --------    -----------------------------
//              Alexander Ashkar
/////////////////////////////////////////////////////////////////////////

module  gfx_ati_rd_rtn_32toN (

        iDATA,
        iVALID,
        oSERREAD_DATA,
        iSRST,
        iSCLK
        );

parameter SERIALIZED_WIDTH = 4;
parameter COUNTER_WIDTH = 3;
parameter DESERIALIZED_WIDTH = 32;
parameter [31:0] NUM_DATA_CYCLES = (DESERIALIZED_WIDTH/SERIALIZED_WIDTH)-1;

output  [SERIALIZED_WIDTH-1:0]   oSERREAD_DATA; // Serialized Output
reg     [SERIALIZED_WIDTH-1:0]   oSERREAD_DATA;

input   [DESERIALIZED_WIDTH-1:0] iDATA; // Input data stream
reg     [DESERIALIZED_WIDTH-1:0] qDATA;
input                            iVALID; // Valid for input data stream

input                            iSRST; //Synchronized Reset
input                            iSCLK; //Clock

reg                              XFER_START;

reg     [COUNTER_WIDTH-1:0]      COUNT;
wire    [COUNTER_WIDTH:0]        tmp_COUNT;

// Hold the Data
always @(posedge iSCLK)
begin
  if (iVALID)
    qDATA <= iDATA;
  else
  if (XFER_START)
    qDATA <= {qDATA[(DESERIALIZED_WIDTH-SERIALIZED_WIDTH-1):0],{SERIALIZED_WIDTH{1'b0}}}; // Shift data up by the serialized width until the transfer is complete.
end

always @(posedge iSCLK)
begin
  if (iSRST)
    oSERREAD_DATA <= {SERIALIZED_WIDTH{1'b0}};
  else
  if (iVALID)
    oSERREAD_DATA <= {SERIALIZED_WIDTH{1'b1}}; // Send the Header on all lines
  else
  if (XFER_START)
    oSERREAD_DATA <= qDATA[(DESERIALIZED_WIDTH-1):(DESERIALIZED_WIDTH-SERIALIZED_WIDTH)]; // Send the data out highest bit to lowest in chunks of serialized width.
  else
    oSERREAD_DATA <= {SERIALIZED_WIDTH{1'b0}}; // Zero lines when not sending
end

always @(posedge iSCLK)
begin
  if (iSRST || (COUNT == {COUNTER_WIDTH{1'b0}}))
    XFER_START <= 1'b0;
  else
  if (iVALID && (COUNT == NUM_DATA_CYCLES)) 
    XFER_START <= 1'b1;
end

// Down counter for payload.
assign tmp_COUNT = COUNT + {COUNTER_WIDTH{1'b1}};

always @(posedge iSCLK)
begin
    if (iSRST)
       COUNT <= NUM_DATA_CYCLES[COUNTER_WIDTH-1:0];
    else
    if (XFER_START)
       COUNT <= tmp_COUNT[COUNTER_WIDTH-1:0];
end

//-------------------------------------------------------------------------------
// Multiple valid check
//-------------------------------------------------------------------------------

// psl property multiple_valid_check = always ((~iSRST) -> !(iVALID && XFER_START)) @(posedge iSCLK);
// psl assert multiple_valid_check report "GRBM read return serializer is getting an unexpected valid during a transfer!";

endmodule
