/////////////////////////////////////////////////////////////////////////
//
//                            Copyright 2001
//                            ATI Research, Inc.
//                            All Rights Reserved
//
//  Filename: gfx_ati_rd_rtn_Nto32.v
//
//  Filetype   : Verilog
//  Description: Converts a serialized data stream into a 32-bit output with 
//  a valid. The transfer size is parameterized with N powers of 2 payload clocks (1,2,4,8,16) and one
//  clock for the header.
//                          
//  Heiarchy:
//
//  Additional Files
//
/////////////////////////////////////////////////////////////////////////
//                        Modification History
//
//   Date       Author         Revision    Comments
//   -------    -----------    --------    -----------------------------
//              Alexander Ashkar
/////////////////////////////////////////////////////////////////////////

module  gfx_ati_rd_rtn_Nto32 (

        iSERREAD_DATA,
        oDATA,
        oVALID,
        iSRST,
        iSCLK
        );

parameter SERIALIZED_WIDTH = 4;
parameter COUNTER_WIDTH = 3;
parameter DESERIALIZED_WIDTH = 32;
parameter NUM_DATA_CYCLES = (DESERIALIZED_WIDTH/SERIALIZED_WIDTH)-1;

input   [SERIALIZED_WIDTH-1:0]    iSERREAD_DATA; // Serialized input data stream

output  [DESERIALIZED_WIDTH-1:0]  oDATA;   // output data

reg     [DESERIALIZED_WIDTH-1:0]  DATA;

output                            oVALID;  // Valid for output data
reg                               oVALID;
wire                              LAST_DATA_XFER;

input                             iSRST;     //Synchronized Reset
input                             iSCLK;     //Clock

reg                               XFER_START;

reg     [COUNTER_WIDTH-1:0]       COUNT;

assign oDATA = (oVALID) ? DATA : {DESERIALIZED_WIDTH{1'b0}};

assign LAST_DATA_XFER = (COUNT == {COUNTER_WIDTH{1'b0}}) & XFER_START;

always @(posedge iSCLK)
begin
  if (iSRST)
  begin
    oVALID <= 1'b0;
    DATA   <= {DESERIALIZED_WIDTH{1'b0}};
  end
  else
  begin
    oVALID <= LAST_DATA_XFER;
  
    if (XFER_START)
      DATA <= {DATA[(DESERIALIZED_WIDTH-SERIALIZED_WIDTH-1):0],iSERREAD_DATA}; // Shift Register
  end
end

always @(posedge iSCLK)
begin
  if (iSRST || LAST_DATA_XFER)
    XFER_START <= 1'b0;
  else
  if (&iSERREAD_DATA && (COUNT == NUM_DATA_CYCLES[COUNTER_WIDTH-1:0])) // Idle and Header
    XFER_START <= 1'b1;
end

// Down counter for payload.
always @(posedge iSCLK)
begin
    if (LAST_DATA_XFER)
       COUNT <= NUM_DATA_CYCLES[COUNTER_WIDTH-1:0];
    else if (XFER_START)
       COUNT <= COUNT-1;
    if (iSRST)
       COUNT <= NUM_DATA_CYCLES[COUNTER_WIDTH-1:0];
end

endmodule
