// AVFS CPO
// Author: Stephen Kosonocky
// 

//==================================================================================================
//! disable one_module_per_file
// Disable the Leda one-per-file check
// leda E_001 off

`timescale 1ps/1ps

`include "gfx_avfs3_cpo_lib.d"
`include "gfx_avfs_psm_lib.d"
`include "gfx_avfs_lib.d"

module gfx_AVFS3_GATE_REPLICA_PATH #(parameter 
path_count = 16,
path_delay = 1000 // modeling gate or wire path delay for simulation
)
  (
   input       [path_count-1:0]    AvfsLfsr,
   output wire [path_count-1:0]    AvfsReplicaPath
  );

   // instantiate critical paths extracted from tile here.
   // logically the paths are just buffers
`ifdef AVFS_BEHAV
   reg [path_count-1:0] AvfsReplicaPathReg;
   always @(*)
   begin
   AvfsReplicaPathReg <= #path_delay AvfsLfsr[path_count-1:0]; // Making it a non-blocking delay assign, Fix UBTS 585653
   end
   assign AvfsReplicaPath[path_count-1:0] = AvfsReplicaPathReg;
`else
//direct assignment
   assign AvfsReplicaPath[path_count-1:0] = AvfsLfsr[path_count-1:0];
`endif
endmodule // AVFS3_GATE_REPLICA_PATH

module gfx_AVFS3_WIRE_REPLICA_PATH #(parameter 
path_count = 16,
path_delay = 1000 // modeling gate or wire path delay for simulation
)
  (
   input       [path_count-1:0]    AvfsLfsr,
   output wire [path_count-1:0]    AvfsReplicaPath
  );

`ifdef AVFS_BEHAV
   reg [path_count-1:0] AvfsReplicaPathReg;
   always @(*)
   begin
   AvfsReplicaPathReg <= #path_delay AvfsLfsr[path_count-1:0]; // Making it a non-blocking delay assign, Fix UBTS 585653
   end
   assign AvfsReplicaPath[path_count-1:0] = AvfsReplicaPathReg;
`else
   // instantiate critical paths extracted from tile here
   assign AvfsReplicaPath[path_count-1:0] = AvfsLfsr[path_count-1:0];
`endif
endmodule // AVFS3_WIRE_REPLICA_PATH

module gfx_AVFS3_ReplicaPathMux #(parameter size=1)
  (
   input [size-1:0] in1,
   input [size-1:0] in0,
   input [size-1:0] sel,
   input en,
   output wire [size-1:0] out
   );

   wire [size-1:0] muxSel;

   assign muxSel[size-1:0] = sel[size-1:0] & {size{en}};   
   //assign muxSel[size-1:0] = sel[size-1:0] & en;
    
   //assign out = (muxSel) ? ~in1[size-1:0] : in0[size-1:0];

   //assign muxSel[size-1:0] = sel[size-1:0] & {size{en}};   

   generate
     genvar i;
     for (i=0; i<size;i=i+1) begin : MUX
       gfx_AVFS_mux2 I_AVFS_mux2 (.Z(out[i]),.S(muxSel[i]),.D1(~in1[i]), .D0(in0[i]));
     end
   endgenerate

endmodule // AVFS3_ReplicaPathMux


// Asynchronous Ripple Counter
module gfx_AVFS3_RippleCnt #(parameter num_bits = 24,
CCLK_TO_CPO_SYNC_TD_DELAY = 5000,
CDC_RESET_TO_CPOCLK_DLY = 5000,
CDC_TST_PROP_DLY=0
)
  (input                           CntIn,     // Counter input clock
   input                           CntRst,    // Counter fuctional reset
   input                           Reset,     // Power-On Reset signal
   input                           CntEn,     // Counter Enable
   output                          CntEn_CPOCLK,
   output [num_bits-1:0]           CntOut     // Counter output
   );

   wire CDC_Rst;
   wire CDC_AsyncRst;
   wire     AsyncRst;
   wire [num_bits-1:0] rcntBits;
   wire [num_bits-1:0] rcntBits_N;
   wire [num_bits-1:0] rcntClks;

   wire CDC_CntRst;
   wire CDC_Reset;
   wire CDC_CntEn;
   wire CntIn_1;
   wire CntIn_buf;
   
   // Added this here as a "marker buffer" to be used by a timing constraint. This buffer will *not* be optimized away by synthesis.
  gfx_AVFS_clkbuf I_AVFS_clkbuf (.A(CntIn), .Y(CntIn_buf));

   
   //CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY)) I_CDC_JTAG_ClkEn (.A(Reset), .Z(CDC_Reset), .EN(1'b1), .DSTCLK(CntIn) );   // JM: likely needs to be defined in terms of ".Cycles()" rather than ".Setup()"

   // Using non-Enabled versions of CDCBUF (3 of them) with BufMetaWaiveMode bit ON to avoid Xs
   // randomize 0->1 and 1->0 transition delay from (0 .. `CDC_JTAG_X_DLY_CYCLES)
   // By design destination clock (CntIn - ripple counter clock) can't run until
   // we are out of Reset|CntRst. These is no risk of metastability on
   // - CntRst
   // - Reset
   // - CntEn
   
   // change this to CDCENBUF
  gfx_CORE_CDCBUF #(.size(1), .Setup(CDC_RESET_TO_CPOCLK_DLY), .Cycles(0), .BufMetaWaiveMode(2)) I_CDC_Reset (.A(Reset), .Z(CDC_Reset), .DSTCLK(CntIn_buf) );   // JM: likely needs to be defined in terms of ".Cycles()" rather than ".Setup()"
   //CORE_CDCBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES), .BufMetaWaiveMode(1)) I_CDC_Reset (.A(Reset), .Z(CDC_Reset), .DSTCLK(CntIn) );   // JM: likely needs to be defined in terms of ".Cycles()" rather than ".Setup()"
   // To avoid Xs in the beginning of time, we need at a minimum ~Reset
   //CORE_CDCENBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES), .Setup(0)) I_CDC_Reset (.A(Reset), .EN(~Reset), .Z(CDC_Reset), .DSTCLK(CntIn) ); 


   // Here is a CDC
   // CntIn is a clock, CPO oscillator clock
   // CntEn, CntRst, and Reset - come from CCLK domain into CPO_CLK
   // CntOut goes from CPO_CLK domain to CCLK, there is a CDCENBUF already on these outputs

   //CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY)) I_CDC_CntRst (.A(CntRst), .EN(1'b1), .Z(CDC_CntRst), .DSTCLK(CntIn) ); // Move to CDC_JTAG_X_DLY_CYCLES?  // JM: I think we want to define this using ".Cycles()" rather than ".Setup()"; the comment about CDC_JTAG_X_DLY_CYCLES is probably just a ".Cycles()" statement; also, I believe .EN(CntEn) is needed after modifying state machine to insert gap between CntRst and CntEn pulses

  gfx_CORE_CDCBUF #(.size(1), .Setup(CDC_RESET_TO_CPOCLK_DLY), .Cycles(0), .BufMetaWaiveMode(2)) I_CDC_CntRst (.A(CntRst), .Z(CDC_CntRst), .DSTCLK(CntIn_buf) );
   //CORE_CDCBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES), .BufMetaWaiveMode(1)) I_CDC_CntRst (.A(CntRst), .Z(CDC_CntRst), .DSTCLK(CntIn) );
   //CORE_CDCENBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES), .Setup(0)) I_CDC_CntRst (.A(CntRst), .EN(~CntEn&~(Reset|ScanShiftEn)), .Z(CDC_CntRst), .DSTCLK(CntIn)); 
   //CORE_CDCENBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES), .Setup(0)) I_CDC_CntRst (.A(CntRst), .EN(~CntEn&~Reset), .Z(CDC_CntRst), .DSTCLK(CntIn)); 
   //assign CDC_CntRst = CntRst ;

   //assign FlushRst = Reset | CntEn;
   // Removed CNtEn as it complicates timing closure. CntEn becomes data & clock to the "Flush" synchronizer
   // Zen/Aspen projects are having to setup complex timing consttaints to make PT time the path correctly
   // Along with removing CntEn from "Flush" term we need to run the "Flush-Synchronizer" with free running
   // clock - Intent is to propagate this fix to Zen/Aspen RevB, Greenland (Vega10), Raven, Tao etc
   assign AsyncRst = Reset ; 

// bypassing this due to bug with the techind_cdcfpm module (having to do with the BufMetaWaiveMode), this isn't required anyway
   //CORE_CDCBUF #(.size(1), .Setup(CCLK_TO_CPO_SYNC_TD_DELAY), .Cycles(0), .BufMetaWaiveMode(2)) I_CDC_AsyncRst (.A(AsyncRst), .Z(CDC_AsyncRst), .DSTCLK(CntIn_buf) );
assign CDC_AsyncRst = AsyncRst;

   // New flush term based on CntEn, min of 2-3 cycles
   // Reset -> Reset|CntEn to make sure we cover the case of very slow CPOCLK could cause async deassert with CntEn itself
   // Use a "ps" number as CPO clock freq is variable/unknown // Make it a param
//   AVFS_SYNC_NOCG_FLUSH #(.TD(CCLK_TO_CPO_SYNC_TD_DELAY)) I_CntEn_CPOCLK (.SampleClk(CntIn), .Reset(CDC_FlushRst), .SSE(1'b0), .DataIn(CntEn), .DataOut(CntEn_CPOCLK) );
   wire [`CLKBUS_SIZE-1:0] clkbus_CPO;
   //AVFS_SYNC_NOCG_FLUSH #(.TD(CCLK_TO_CPO_SYNC_TD_DELAY)) I_CntEn_CPOCLK (.SampleClk(clkbus_CPO[`C_PIN]), .Reset(CDC_FlushRst), .SSE(1'b0), .DataIn(CntEn), .DataOut(CntEn_CPOCLK) );
// Setting IC=0 to avoid invalid checker errors. The error is caused by the sample clock not being free-running. This is by design: clock only runs when a CPO is enabled.
  gfx_AVFS_SYNC_NOCG_AR #(.TD(CCLK_TO_CPO_SYNC_TD_DELAY), .IC(0)) I_CntEn_CPOCLK (.SampleClk(CntIn_buf), .Reset(CDC_AsyncRst), .SSE(1'b0), .DataIn(CntEn), .DataOut(CntEn_CPOCLK) );

   assign CDC_Rst = CDC_Reset | CDC_CntRst;
   assign rcntBits_N = ~rcntBits;

   //assign CntIn_1 = CntIn & (CntEn|CntEn_CPOCLK_B) ; // async assert, synchronous to CPO_CLK deassert 
   //assign CntIn_1 = CntIn & CntEn_CPOCLK_B ; 

   // Add B-Latch to register CntEn_CPOCLK and safely clock gate CPOCLK
   // wire CntEn_CPOCLK_B ;
   // lat2 I_CntEn_CPOCLK_B (.clk(CntIn), .d(CntEn_CPOCLK), .q(CntEn_CPOCLK_B));
   // Replacing "B-latch + and" gate with library gater
  gfx_CORE_CG I_cg_CPOCLK (.clkbus(clkbus_CPO[`CLKBUS_SIZE-1:0]), .C(CntIn_buf), .G(CntEn_CPOCLK), .R(1'b0), .SSE(1'b0));
   assign CntIn_1 = clkbus_CPO[`C_PIN] ; 
   
  //modified by Miguel to stop the count when CntEn is low
   assign rcntClks = {rcntBits[num_bits-2:0],CntIn_1}; 

   genvar index;
   generate
     for(index=0;index<num_bits; index=index+1)
     begin:flops
  gfx_srdff I_srdff_ripplecnt (.q(rcntBits[index]),.d(rcntBits_N[index]),.clk(rcntClks[index]),.rst(1'b0),.set(CDC_Rst));
     end  
   endgenerate

   assign CntOut = ~rcntBits;
   
endmodule // AVFS3_RippleCnt


module gfx_AVFS3_GATEPACK #(parameter 
path_count = `AVFS3_GATE_PATHS_NUM,
CCLK_TO_CPO_SYNC_TD_DELAY = 5000,
CDC_RESET_TO_CPOCLK_DLY = 5000,
CDC_TST_PROP_DLY = 0,
path_delay = 1000
) (
   output                          CntEn_CPOCLK,   // New CntEn port
   input  [path_count-1:0]         AvfsPathState,
   input                           AvfsPackEn,     // Gate/Wire pack enable signal (gates all operations)
   input                           AvfsCntRst,     // oscillator counter reset
   input                           AvfsOscEn,      // enables oscillator mode of operation
   input                           extendOscEn,
   input  [path_count-1:0]         AvfsOscSel,     // One-hot Oscillator Selection signal
   input                           CCLK,           // core clock
   input                           AvfsStateWrite, // Enables writing override value for AVFS replica path state
   input                           Reset,          // reset signal to reset flops on power-up
   input                           LBistEn,        // LBistEn to keep ripple counter flops in reset
   input                           ScanShiftEn,    // sse
   output [`AVFS3_RCNT_WIDTH-1:0]                   AvfsCpoCnt      // Asynchronous 14b Counter output 
   );

   wire [path_count-1:0]   AvfsLfsr;              // signal name historical, is not an LFSR
   wire [path_count-1:0]   AvfsReplicaPath;
   wire                    AvfsReplicaPathFB;
   wire                    AvfsReplicaPathFB_mcp;
   wire [path_count-1:0]   AvfsReplicaPathInput;
   wire [path_count-1:0]   AvfsReplicaPathFlopOut;         

   // assign        AvfsReplicaPathInput = (AvfsStateWrite)? AvfsPathState : AvfsReplicaPathFB;
   assign AvfsReplicaPathInput = (AvfsStateWrite)? AvfsPathState : {path_count{AvfsReplicaPathFB_mcp}};

   wire CPTimingEn;
   wire CPTimingTestIn;
   wire [`CLKBUS_SIZE-1:0] clkbus_scan_only_CPTimingEn;

   // adding scan only flop to control CPTimingEn in LBistEn mode
   assign CPTimingEn = LBistEn ? CPTimingTestIn : AvfsPackEn&AvfsCntRst; // Removing &AvfsCntRst as LBistEn will keep this in reset and limit stuck-at-fault coverage

  gfx_CORE_CG I_cg_CPTimingEn_scan_only (.clkbus(clkbus_scan_only_CPTimingEn[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(LBistEn), .R(1'b0), .SSE(ScanShiftEn));
  gfx_CORE_FLOP_DGATE I_CPTiming_ScanOnly_DFF (.clkbus(clkbus_scan_only_CPTimingEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(1'b0), .Q(CPTimingTestIn));

  gfx_CORE_FLOP_GATED_RESET #(.size(path_count)) I_AvfsCPTiming_DFF (.C(CCLK), .D(AvfsReplicaPathInput), .G(CPTimingEn), .GC(1'b0), .R(Reset), .SSE(ScanShiftEn), .Q(AvfsReplicaPathFlopOut)); 

  gfx_AVFS3_ReplicaPathMux #(.size(path_count)) I_Avfs_ReplicaPathMux (.in1(AvfsReplicaPath), .in0(AvfsReplicaPathFlopOut), .sel(AvfsOscSel), .en((extendOscEn)&AvfsPackEn), .out(AvfsLfsr));


  gfx_AVFS3_GATE_REPLICA_PATH  #(.path_count(path_count),.path_delay(path_delay)) 
     I_AVFS_GATE_REPLICA_PATH
       (
        .AvfsLfsr        (AvfsLfsr[path_count-1:0] ), 
        .AvfsReplicaPath (AvfsReplicaPath[path_count-1:0])
        );

   assign AvfsReplicaPathFB =  |(AvfsReplicaPath & (AvfsOscSel&{path_count{AvfsPackEn}}) ) ;

`ifdef AVFS3_MCENBUF_ON
  // Multi-cycle path added to help with timing, Feebback to CP_Timing_DF is going to have more than CCLK period delay
  // given the fact that there is apporx. CP delay due to CPO path in the way. Early PD runs are showing the delay is closer to 2 cycles
  // Making it a 2-cycle path so that PD tools don't prioritize working on this path
  wire mcbufEn;
  assign mcbufEn = CPTimingEn&~AvfsStateWrite&~Reset;
  gfx_CORE_MCENBUF #(.size(1), .Cycles(10)) I_AvfsReplicaPathFB_mcp (.DSTCLK(CCLK), .EN(mcbufEn), .A(AvfsReplicaPathFB), .Z(AvfsReplicaPathFB_mcp));
`else
  gfx_CORE_MCBUF #(.size(1), .Cycles(10)) I_AvfsReplicaPathFB_mcp (.DSTCLK(CCLK), .A(AvfsReplicaPathFB), .Z(AvfsReplicaPathFB_mcp));
`endif

   // Removed &AvfsPackEn from .CntEn to avoid combo logic between flop to flop CDC issue. AvfsOscEn should be a single
   // signal from CCLK to CPOCLK
  gfx_AVFS3_RippleCnt #(.num_bits(`AVFS3_RCNT_WIDTH),.CDC_TST_PROP_DLY(CDC_TST_PROP_DLY),.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY), .CDC_RESET_TO_CPOCLK_DLY(CDC_RESET_TO_CPOCLK_DLY)) I_AVFS_RippleCnt (.CntIn(AvfsReplicaPathFB), .CntRst(AvfsCntRst), .Reset(ScanShiftEn|Reset|LBistEn), .CntEn(AvfsPackEn&AvfsOscEn), .CntOut(AvfsCpoCnt), .CntEn_CPOCLK(CntEn_CPOCLK));
 
endmodule // AVFS3_GATEPACK

module gfx_AVFS3_WIREPACK #(parameter 
path_count = `AVFS3_WIRE_PATHS_NUM,
CDC_TST_PROP_DLY = 0,
CCLK_TO_CPO_SYNC_TD_DELAY = 5000,
CDC_RESET_TO_CPOCLK_DLY = 5000,
path_delay=1000
) (
   output                          CntEn_CPOCLK,   // New CntEn port
   input  [path_count-1:0]         AvfsPathState,
   input                           AvfsPackEn,     // Gate/Wire pack enable signal (gates all operations)
   input                           AvfsCntRst,     // oscillator counter reset
   input                           AvfsOscEn,      // enables oscillator mode of operation
   input                           extendOscEn,      // enables oscillator mode of operation
   input  [path_count-1:0]         AvfsOscSel,     // One-hot Oscillator Selection signal
   input                           CCLK,           // core clock
   input                           LBistEn,        // LBistEn to keep ripple counter flops in reset
   input                           AvfsStateWrite, // Enables writing override value for AVFS replica path state
   input                           Reset,          // reset signal to reset flops on power-up
   input                           ScanShiftEn,    // sse
   output [`AVFS3_RCNT_WIDTH-1:0]                   AvfsCpoCnt      // Asynchronous 14b Counter output 
   );

   wire [path_count-1:0]   AvfsLfsr;              // signal name historical, is not an LFSR
   wire [path_count-1:0]   AvfsReplicaPath;
   wire                    AvfsReplicaPathFB;
   wire                    AvfsReplicaPathFB_mcp;
   wire [path_count-1:0]   AvfsReplicaPathInput;
   wire [path_count-1:0]   AvfsReplicaPathFlopOut;         

//assign        AvfsReplicaPathInput = (AvfsStateWrite)? AvfsPathState : AvfsReplicaPathFB;
   assign        AvfsReplicaPathInput = (AvfsStateWrite)? AvfsPathState : {path_count{AvfsReplicaPathFB_mcp}};

   wire CPTimingEn;
   wire CPTimingTestIn;
   wire [`CLKBUS_SIZE-1:0] clkbus_scan_only_CPTimingEn;

   // adding scan only flop to control CPTimingEn in LBistEn mode
   assign CPTimingEn = LBistEn ? CPTimingTestIn : AvfsPackEn&AvfsCntRst; // Removing &AvfsCntRst as LBistEn will keep this in reset and limit stuck-at-fault coverage

  gfx_CORE_CG I_cg_CPTimingEn_scan_only (.clkbus(clkbus_scan_only_CPTimingEn[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(LBistEn), .R(1'b0), .SSE(ScanShiftEn));
  gfx_CORE_FLOP_DGATE I_CPTiming_ScanOnly_DFF (.clkbus(clkbus_scan_only_CPTimingEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(1'b0), .Q(CPTimingTestIn));

  gfx_CORE_FLOP_GATED_RESET #(.size(path_count)) I_AvfsCPTiming_DFF (.C(CCLK), .D(AvfsReplicaPathInput), .G(CPTimingEn), .GC(1'b0), .R(Reset), .SSE(ScanShiftEn), .Q(AvfsReplicaPathFlopOut)); 

  gfx_AVFS3_ReplicaPathMux #(.size(path_count)) I_Avfs_ReplicaPathMux (.in1(AvfsReplicaPath), .in0(AvfsReplicaPathFlopOut), .sel(AvfsOscSel), .en((extendOscEn)&AvfsPackEn), .out(AvfsLfsr));


  gfx_AVFS3_WIRE_REPLICA_PATH  #(.path_count(path_count),.path_delay(path_delay)) // 600 = 2*300ps
     I_AVFS_WIRE_REPLICA_PATH
       (
        .AvfsLfsr        (AvfsLfsr[path_count-1:0] ), 
        .AvfsReplicaPath (AvfsReplicaPath[path_count-1:0])
        );

   //assign AvfsReplicaPathFB = |(AvfsReplicaPath & (AvfsOscSel&{path_count{AvfsPackEn}}&{path_count{AvfsOscEn}}) );
   assign AvfsReplicaPathFB = |(AvfsReplicaPath & (AvfsOscSel&{path_count{AvfsPackEn}}) );

`ifdef AVFS3_MCENBUF_ON
  // Multi-cycle path added to help with timing, Feebback to CP_Timing_DF is going to have more than CCLK period delay
  // given the fact that there is apporx. CP delay due to CPO path in the way. Early PD runs are showing the delay is closer to 2 cycles
  // Making it a 2-cycle path so that PD tools don't prioritize working on this path
  wire mcbufEn;
  assign mcbufEn = CPTimingEn&~AvfsStateWrite&~Reset;
  gfx_CORE_MCENBUF #(.size(1), .Cycles(10)) I_AvfsReplicaPathFB_mcp (.DSTCLK(CCLK), .EN(mcbufEn), .A(AvfsReplicaPathFB), .Z(AvfsReplicaPathFB_mcp));
`else
  gfx_CORE_MCBUF #(.size(1), .Cycles(10)) I_AvfsReplicaPathFB_mcp (.DSTCLK(CCLK), .A(AvfsReplicaPathFB), .Z(AvfsReplicaPathFB_mcp));
`endif

   // Removed &AvfsPackEn from .CntEn to avoid combo logic between flop to flop CDC issue. AvfsOscEn should be a single
   // signal from CCLK to CPOCLK
  gfx_AVFS3_RippleCnt #(.num_bits(`AVFS3_RCNT_WIDTH),.CDC_TST_PROP_DLY(CDC_TST_PROP_DLY),.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY), .CDC_RESET_TO_CPOCLK_DLY(CDC_RESET_TO_CPOCLK_DLY)) I_AVFS_RippleCnt (.CntIn(AvfsReplicaPathFB), .CntRst(AvfsCntRst), .Reset(ScanShiftEn|Reset|LBistEn), .CntEn(AvfsPackEn&AvfsOscEn), .CntOut(AvfsCpoCnt), .CntEn_CPOCLK(CntEn_CPOCLK));
 
endmodule // AVFS3_WIREPACK

// NOTE: THERE IS NO INPUT SELECT AND STATE REST LOGIC FOR THE THERMPACK
module gfx_AVFS3_THERMPACK #(parameter path_count = 7,
CCLK_TO_CPO_SYNC_TD_DELAY = 5000,
CDC_RESET_TO_CPOCLK_DLY = 5000,
CDC_TST_PROP_DLY = 0
)
  (
   output                          CntEn_CPOCLK,   // New CntEn port
   input                           AvfsPackEn,     // Gate/Wire/Therm pack enable signal (gates all operations)
   input                           AvfsCntRst,     // oscillator counter reset
   input                           AvfsOscEn,      // enables oscillator mode of operation
   input                           extendOscEn,      // enables oscillator mode of operation
   input  [path_count-1:0]         AvfsOscSel,     // One-hot Oscillator Selection signal
   input                           Reset,          // reset signal to reset flops on power-up
   input                           ScanShiftEn,    // SSE
   input                           LBistEn,        // LBistEn to keep ripple counter flops in reset
   output [`AVFS3_RCNT_WIDTH-1:0]                   AvfsCpoCnt      // Asynchronous 14b Counter output 3
   );

   wire [path_count-1:0]   AvfsThermalROout;
   wire [path_count-1:0]   AvfsThermSel;

   assign        AvfsThermSel = (AvfsOscSel & ({path_count{AvfsPackEn & extendOscEn}} & {path_count{~Reset}}));

   // 3 macros are combined into one now
  gfx_AVFS3_THERMALRO I_AVFS_THERMALRO (
                                   // .slvt_ro_en(AvfsThermSel[0]&(extendOscEn)), // Ravi - in Non-repeat mode ROs keep running, gating with OscEn
                                   // .lvt_ro_en( AvfsThermSel[1]&(extendOscEn)), 
                                   // .rvt_ro_en( AvfsThermSel[2]&(extendOscEn)), 
                                   // .slvt_ro_out(AvfsThermalROout[0]),
                                   // .lvt_ro_out(AvfsThermalROout[1]),
                                   // .rvt_ro_out(AvfsThermalROout[2])
                                    .en(AvfsThermSel),
                                    .out(AvfsThermalROout)
                                    );   
              
   assign AvfsThermalROMuxOut = |(AvfsThermalROout & (AvfsOscSel&{path_count{AvfsPackEn}}));

   // Removed &AvfsPackEn from .CntEn to avoid combo logic between flop to flop CDC issue. AvfsOscEn should be a single
   // signal from CCLK to CPOCLK
  gfx_AVFS3_RippleCnt #(.num_bits(`AVFS3_RCNT_WIDTH),.CDC_TST_PROP_DLY(CDC_TST_PROP_DLY),.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY), .CDC_RESET_TO_CPOCLK_DLY(CDC_RESET_TO_CPOCLK_DLY)) I_AVFS_RippleCnt (.CntIn(AvfsThermalROMuxOut), .CntRst(AvfsCntRst), .Reset(ScanShiftEn|Reset|LBistEn), .CntEn(AvfsPackEn&AvfsOscEn), .CntOut(AvfsCpoCnt), .CntEn_CPOCLK(CntEn_CPOCLK));
 
endmodule // AVFS3_THERMPACK

module gfx_AVFS3_decoder_4_16
  (
   input [3:0]   in,
   output [15:0] out
   );

   assign out  = (in == 4'b0000 ) ? 16'b0000_0000_0000_0001 : 
          (in == 4'b0001 ) ? 16'b0000_0000_0000_0010 : 
          (in == 4'b0010 ) ? 16'b0000_0000_0000_0100 : 
          (in == 4'b0011 ) ? 16'b0000_0000_0000_1000 : 
          (in == 4'b0100 ) ? 16'b0000_0000_0001_0000 : 
          (in == 4'b0101 ) ? 16'b0000_0000_0010_0000 : 
          (in == 4'b0110 ) ? 16'b0000_0000_0100_0000 : 
          (in == 4'b0111 ) ? 16'b0000_0000_1000_0000 : 

          (in == 4'b1000 ) ? 16'b0000_0001_0000_0000 : 
          (in == 4'b1001 ) ? 16'b0000_0010_0000_0000 : 
          (in == 4'b1010 ) ? 16'b0000_0100_0000_0000 : 
          (in == 4'b1011 ) ? 16'b0000_1000_0000_0000 : 
          (in == 4'b1100 ) ? 16'b0001_0000_0000_0000 : 
          (in == 4'b1101 ) ? 16'b0010_0000_0000_0000 : 
          (in == 4'b1110 ) ? 16'b0100_0000_0000_0000 : 
          (in == 4'b1111 ) ? 16'b1000_0000_0000_0000 : 16'hxx;
   
endmodule // AVFS3_decoder_4_16

module gfx_AVFS3_decoder_3_7
  (
   input [2:0]   in,
   output [6:0] out
   );

   assign out  = (in == 3'b000 ) ? 7'b000_0001 : 
          (in == 3'b001 ) ? 7'b000_0010 : 
          (in == 3'b010 ) ? 7'b000_0100 : 
          (in == 3'b011 ) ? 7'b000_1000 : 
          (in == 3'b100 ) ? 7'b001_0000 : 
          (in == 3'b101 ) ? 7'b010_0000 : 
          (in == 3'b110 ) ? 7'b100_0000 : 
          (in == 3'b111 ) ? 7'b100_0000 : 7'bxxx_xxxx;
endmodule // gfx_AVFS3_decoder_3_7


module gfx_AVFS3_CPO_FSM (
   input  [`AVFS3_RCNT_WIDTH-1:0]     AvfsSmpStart,   // oscillator enable start count
   input  [`AVFS3_RCNT_WIDTH-1:0]     AvfsSmpMax,     // oscillator stop count
   input                           AvfsGo,         // avfs go pulse to start the FSM sequence
   input                           AvfsEn,         // AVFS FSM Enable signal
   input                           AvfsRepeat,     // if set, run AVFS in repeat/continuous mode
   input                           CCLK,           // core clock
   input                           Reset,          // reset signal to reset flops on power-up
   input                           ScanShiftEn,    // SSE
   input                           CntEn_Syncd_CCLK,// New input back from PACKs to make sure we cleanly stop sampling
   output                          CntRst_d1,         // cpo counter reset signal
   output                          OscEn,          // cpo oscillator enable
   output                          extendOscEn,          // cpo oscillator enable
   output                          AvfsCapture     // cpo counter capture signal
   );

   wire          smpRst;
   wire [3:0]    smpPulse;
   wire [11:0]    smpCapPulse;
   wire [11:0]    oscPulse;
   
   wire AvfsCapture_0;
   wire AvfsCapture_1;
   wire AvfsCapture_2;
   wire AvfsCapture_3;
   wire smpRst_D;

   // Merging gaters per bug : http://bugs.amd.com/ubts/prod/issue/view?id=602121
   wire [`CLKBUS_SIZE-1:0] clkbus;
   wire [`CLKBUS_SIZE-1:0] clkbus_smpCapPulse;

  gfx_CORE_CG I_cg (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(AvfsEn), .R(Reset), .SSE(ScanShiftEn));
   //CORE_CG I_cg_smpCapPulse (.clkbus(clkbus_smpCapPulse[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(AvfsEn), .R(Reset), .SSE(ScanShiftEn));

   //AVFS3_CPO_FLOP #(.size(4)) I_smpRst_DFF (.D({AvfsGo,smpPulse[3:1]}), .Q(smpPulse[3:0]), .C(clkbus[`C_PIN]), .R(Reset));
  gfx_CORE_FLOP_DGATE #(.size(4)) I_smpRst_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D({AvfsGo,smpPulse[3:1]}), .Q(smpPulse[3:0]));

// added by Miguel to guarantee that we reset the counter at startup too
   // Ravi Rachala - adding repeat mode logic, when AvfsRpeat=1'b1, kickoff the AVFS loop again (run AVFS continuously)
   assign          smpRst_D = (AvfsCapture_3 & AvfsRepeat) | (AvfsGo & ~smpPulse[0]);
   // Fix for: UBTS [Bug 586524 NEW] X generated when CPO_FSM is in repeat mode in [CPLKIT][AVFS3]
   // Sample counter goes X after Reset deasserts as smpRst=X 
   // Adding a flop to keep smpRst=0 in Reset and while AvfsEn=0
   //AVFS3_CPO_FLOP #(.size(1)) I_smpRst2_DFF (.D(smpRst_D), .Q(smpRst), .C(clkbus[`C_PIN]), .R(Reset));
  gfx_CORE_FLOP_DGATE #(.size(1)) I_smpRst2_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(smpRst_D), .Q(smpRst));


   wire [`AVFS3_RCNT_WIDTH-1:0] smpCnt;
   wire [`AVFS3_RCNT_WIDTH-1:0] smpCnt_Nxt;
   wire smpCntGate, satSmpCnt;
   wire CntRst;
   wire CntRst_d2;
   wire CntRst_d3;
   wire OscEn_d1;

   assign          smpCnt_Nxt = smpRst? 24'h000000 : smpCnt + 1;
   
   // negedge of OscEn create a long enough pulse and OR with OscEn to keep the ripple counter
   // running for a while (until CntEn_Syncd sees the fall edge), basically 3 cycles of CPOCLK
   // CPOCLK is long especially for THERMPACK, basically wait for CntEn_Syncd_CCLK negedge
  gfx_CORE_FLOP_DGATE I_CntEn_d1 (           .clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(OscEn_0),               .Q(OscEn));
  gfx_CORE_FLOP_DGATE I_CntEn_d2 (           .clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(OscEn),                 .Q(OscEn_d1));
  gfx_CORE_FLOP_DGATE I_CntEn_Syncd_CCLK_d1 (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CntEn_Syncd_CCLK),      .Q(CntEn_Syncd_CCLK_d1));

   // sticky flop capturing the negedge on OscEn/CntEn

   wire negEdgeDetected_CntEn_Syncd_CCLK;

   // Create extendOscEn to keep the CPO running until we register CntEn_Syncd (into CPOCLK) back into CCLK
//   CORE_FLOP_GATED_RESET I_extendOscEn_DFF (.C(CCLK), .D(1'b1), .G(negEdgeDetected_CntEn|Reset), .GC(1'b0), .R(CntRst|Reset|negEdgeDetected_CntEn_Syncd_CCLK), .SSE(ScanShiftEn), .Q(extendOscEn)); 
   assign extendOscEnIn = (Reset|CntRst|negEdgeDetected_CntEn_Syncd_CCLK) ? 1'b0 :
                          (OscEn_d1) ? 1'b1 :  // extendOscEn will assert 2 cycle after OscEn
                          extendOscEn;
  gfx_CORE_FLOP_DGATE I_extendOscEn_DFF (    .clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(extendOscEnIn),         .Q(extendOscEn)); 

   // Look for de-assertion of CntEn_Syncd_CCLK, indicates OscEn from CCLK to CPOCLK is registered (back in CCLK domain, confusing!! Yes.)
   // We can now safely reset extednOscEn which will turn off the CPO oscillators, until now the async timing loop
   // in the PACKs should be running. So "Enable" term to AVFS3_ReplicaPathMux gets disabled by this "resetExtendOscEn"
   //CORE_FLOP_GATED_RESET I_CntEn_Syncd_CCLK_DFF (.C(CCLK), .D(1'b1), .G(negEdgeDetected_CntEn_Syncd_CCLK), .GC(1'b0), .R(Reset), .SSE(ScanShiftEn), .Q(resetExtendOscEn)); 
   assign negEdgeDetected_CntEn_Syncd_CCLK = ~CntEn_Syncd_CCLK & CntEn_Syncd_CCLK_d1 ; // this will reset extendOscEn
   
   // modified by Miguel to get the counter to stop after a test
   assign          smpCntGate = (AvfsEn & ~satSmpCnt) | smpRst;
   
   // Data Reset logic is above here, and inside, synthesis should be able to optimize 
   //AVFS3_CPO_FLOP_GATED #(.size(`AVFS3_RCNT_WIDTH)) I_smpCnt_DFF (.D(smpCnt_Nxt), .Q(smpCnt), .G(smpCntGate), .C(CCLK), .R(smpRst|Reset));
  gfx_CORE_FLOP_GATED_RESET #(.size(`AVFS3_RCNT_WIDTH)) I_smpCnt_DFF (.C(CCLK), .D(smpCnt_Nxt), .G(smpCntGate), .GC(1'b0), .R(smpRst|Reset), .SSE(ScanShiftEn), .Q(smpCnt)); 

   assign          CntRst = (smpCnt <= AvfsSmpStart)? 1'b1 : 1'b0; // Zijian pointed out this as a potential issue resulting in CntRst=1, and OscEn=0
  gfx_CORE_FLOP_DGATE #(.size(1),.rv(1'b1)) I_CntRst_d1_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CntRst), .Q(CntRst_d1));
  gfx_CORE_FLOP_DGATE #(.size(1),.rv(1'b1)) I_CntRst_d2_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CntRst_d1), .Q(CntRst_d2));
  gfx_CORE_FLOP_DGATE #(.size(1),.rv(1'b1)) I_CntRst_d3_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CntRst_d2), .Q(CntRst_d3));
   
   //assign          stopSmp  = (smpCnt == AvfsSmpMax)? 1'b1 : 1'b0;
   //modified by Miguel to avoid the oscEn to be on after the count
   assign          stopSmp  = (smpCnt >= AvfsSmpMax) ? 1'b1 : 1'b0 ; // After syncing CntEn to CPO_CLK for clean deassertion
   assign          OscEn_0 = (~smpRst & (~CntRst_d3 & ~stopSmp)) ;                                  // we need to take care of cleanly stopping the sampling
                                                                                                // by asserting stopSmp. This adds more count, 3 more CPO cycles
   wire          satSmpCntNxt;                                                                  // worth, and that changes with each PACK and path
   
   assign          satSmpCntNxt = (stopSmp|satSmpCnt) & ~smpRst;
   // OscEn Flop to make sure turning ON vs OFF and syncing up with CP clock is clean
   // 
   //CORE_FLOP_GATED_RESET I_OscEn_DFF (.C(CCLK), .D(OscEn_0), .G(), .GC(1'b0), .R(Reset), .SSE(ScanShiftEn), .Q(OscEn)); 
   
   //AVFS3_CPO_FLOP #(.size(1))  I_satCnt_DFF (.D(satSmpCntNxt), .Q(satSmpCnt), .C(clkbus[`C_PIN]), .R(Reset));
  gfx_CORE_FLOP_DGATE #(.size(1)) I_satCnt_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(satSmpCntNxt), .Q(satSmpCnt));

   // Increasing the flop count to 12 to allow (total of 12 cycles) for the ripple to settle at cclk speeds
   // Apply 1/2 cycle max timing constraint for the prop delay between flops 
   //AVFS3_CPO_FLOP #(.size(12)) I_smpCap_DFF (.D({stopSmp,smpCapPulse[11:1]}), .Q(smpCapPulse[11:0]), .C(clkbus[`C_PIN]), .R(Reset));
  gfx_CORE_FLOP_DGATE #(.size(12)) I_smpCap_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D({~CntEn_Syncd_CCLK,smpCapPulse[11:1]}), .Q(smpCapPulse[11:0]));

   assign          AvfsCapture = smpCapPulse[4] & ~smpCapPulse[0];

   // Ravi Rachala - for AvfsRepeat mode - delay AvfsCapture by 4 cycles
  gfx_CORE_FLOP_DGATE #(.size(1)) I_AvfsCapture_dly0_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsCapture), .Q(AvfsCapture_0));
  gfx_CORE_FLOP_DGATE #(.size(1)) I_AvfsCapture_dly1_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsCapture_0), .Q(AvfsCapture_1));
  gfx_CORE_FLOP_DGATE #(.size(1)) I_AvfsCapture_dly2_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsCapture_1), .Q(AvfsCapture_2));
  gfx_CORE_FLOP_DGATE #(.size(1)) I_AvfsCapture_dly3_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsCapture_2), .Q(AvfsCapture_3));

endmodule // AVFS3_CPO_FSM

module gfx_AVFS3_CPO #(parameter 
CDC_TST_PROP_DLY = 0,
CPO_TO_CCLK_SYNC_TC_CYCLES = 1,
CDC_RESET_TO_CPOCLK_DLY = 5000,
CCLK_TO_CPO_SYNC_TD_DELAY = 5000
)(
   input                     LBistEn,                   // LBistEn
   input  [`AVFS3_RCNT_WIDTH-1:0]     AvfsSmpStart,     // oscillator enable start count
   input  [`AVFS3_RCNT_WIDTH-1:0]     AvfsSmpMax,       // oscillator stop count
   input                     AvfsGo,           // avfs go pulse to start the FSM sequence
   input                     AvfsEn,           // AVFS FSM Enable signal
   input                     AvfsRepeat,       // AVFS FSM Enable signal
   input                     AvfsGateEn0,       // Gatepack-0 Enable
   input                     AvfsGateEn1,       // Gatepack-1 Enable
   input                     AvfsWireEn,       // Wirepack Enable
   input                     AvfsThermEn,      // Thermpack Enable
   input                     AvfsStateWrite,   // Enables writing override value for AVFS replica path state (normally 1'b1)
   input [3:0]               AvfsGatePackSel0, // 1/16 Selection of Gatepack0 path
   input [3:0]               AvfsGatePackSel1, // 1/16 Selection of Gatepack1 path
   input [3:0]               AvfsWirePackSel,  // 1/16 Selection of Wirepack path
   input [2:0]               AvfsThermPackSel, // 1/7 Selection of Thermal Oscillator
   input [15:0]              AvfsGatePackState0,// Rest state of the AVFS Gate paths (Aging Control Bit)
   input [15:0]              AvfsGatePackState1,// Rest state of the AVFS Gate paths (Aging Control Bit)
   input [15:0]              AvfsWirePackState,// Rest state of the AVFS Wire paths (Aging Control Bit)
   input                     CCLK,             // core clock
   input                     SkipPhase,        // SkipPhase signal (toggle indicates a full cycle of CCLK stretch)
   input                     Reset,            // reset signal to reset flops on power-up
   input                     ScanShiftEn,      // reset signal to reset flops on power-up
   output wire [`AVFS3_RCNT_WIDTH-1:0]     GatePackCnt0,     // Flopped version of Gatepack0 Cnt
   output wire [`AVFS3_RCNT_WIDTH-1:0]     GatePackCnt1,     // Flopped version of Gatepack1 Cnt
   output wire [`AVFS3_RCNT_WIDTH-1:0]     WirePackCnt,      // Flopped version of Wirepack Cnt   
   output wire [`AVFS3_RCNT_WIDTH-1:0]     ThermPackCnt,     // Flopped version of Thermpack Cnt
   output wire [`AVFS3_RCNT_WIDTH-1:0]     SkipPhaseCnt      // Count the toggles on SkipPhase
   );
  
   wire                     AvfsCapture;
   wire                     CntRst;
   wire                     OscEn;

   wire [`AVFS3_RCNT_WIDTH-1:0]     GatePackCntNxt0;     
   wire [`AVFS3_RCNT_WIDTH-1:0]     GatePackCntNxt1;     
   wire [`AVFS3_RCNT_WIDTH-1:0]     WirePackCntNxt;      
   wire [`AVFS3_RCNT_WIDTH-1:0]     ThermPackCntNxt;      

   wire [15:0]      AvfsGatePackSelDec0;
   wire [15:0]      AvfsGatePackSelDec1;
   wire [15:0]      AvfsWirePackSelDec;
   wire [6:0]       AvfsThermPackSelDec;
   wire [`AVFS3_RCNT_WIDTH-1:0]    SkipPhaseCnt_d0;
   wire [`CLKBUS_SIZE-1:0] clkbus;
   wire [`CLKBUS_SIZE-1:0] clkbus_CntEn;

   //wire [`CLKBUS_SIZE-1:0] clkbus_free;
   //wire Reset_d1;
   
   //GatePack0
  gfx_AVFS3_decoder_4_16 I_AVFS_decoder_GATE0_4_16
     (
      .in(AvfsGatePackSel0),
      .out(AvfsGatePackSelDec0));
   //GatePack1
  gfx_AVFS3_decoder_4_16 I_AVFS_decoder_GATE1_4_16
     (
      .in(AvfsGatePackSel1),
      .out(AvfsGatePackSelDec1));
   //Wire Pack
  gfx_AVFS3_decoder_4_16 I_AVFS_decoder_WIRE_4_16
     (
      .in(AvfsWirePackSel),
      .out(AvfsWirePackSelDec));
   //Therm pack
  gfx_AVFS3_decoder_3_7 I_AVFS_decoder_THERM_3_7
     (
      .in(AvfsThermPackSel),
      .out(AvfsThermPackSelDec));

   wire CntEn_CPOCLK_GP0;
   wire CntEn_CPOCLK_GP1;
   wire CntEn_CPOCLK_WP;
   wire CntEn_CPOCLK_TP;
   wire CntEn_Syncd_GP0;
   wire CntEn_Syncd_GP1;
   wire CntEn_Syncd_WP;
   wire CntEn_Syncd_TP;

   // 16 path gate pack
  gfx_AVFS3_GATEPACK #(.path_count(16),.path_delay(300),.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY)) I_AVFS_GATEPACK_0 // 2*300ps, osc count = "sample size" / 2
     ( .AvfsPathState(AvfsGatePackState0),
       .CntEn_CPOCLK(CntEn_CPOCLK_GP0),
       .AvfsPackEn(AvfsGateEn0),
       .AvfsCntRst(CntRst),
       .AvfsOscEn(OscEn),
       .extendOscEn(extendOscEn),
       .AvfsOscSel(AvfsGatePackSelDec0&{16{~LBistEn}}), // LBistEn=1 drives 16'h0000 to AvfsOscSel to break the loop showing up in DFT runs 
       .CCLK(CCLK),
       .AvfsStateWrite(AvfsStateWrite),
       .Reset(Reset),
       .LBistEn(LBistEn),                          // Splitting LBistEn out, so it only resets ripple counter and not CPTiming_DFF
       .ScanShiftEn(ScanShiftEn),
       .AvfsCpoCnt(GatePackCntNxt0));

  // 16 path gate pack
  gfx_AVFS3_GATEPACK #(.path_count(16),.path_delay(450),.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY)) I_AVFS_GATEPACK_1 // 3*300ps, osc count = "sample size" / 3
     ( .AvfsPathState(AvfsGatePackState1),
       .CntEn_CPOCLK(CntEn_CPOCLK_GP1),
       .AvfsPackEn(AvfsGateEn1),
       .AvfsCntRst(CntRst),
       .AvfsOscEn(OscEn),
       .extendOscEn(extendOscEn),
       .AvfsOscSel(AvfsGatePackSelDec1&{16{~LBistEn}}), // LBistEn=1 drives 16'h0000 to AvfsOscSel to break the loop showing up in DFT runs
       .CCLK(CCLK),
       .AvfsStateWrite(AvfsStateWrite),
       .Reset(Reset),
       .LBistEn(LBistEn),                          // Splitting LBistEn out, so it only resets ripple counter and not CPTiming_DFF
       .ScanShiftEn(ScanShiftEn),
       .AvfsCpoCnt(GatePackCntNxt1));

   // 16 path wire pack
  gfx_AVFS3_WIREPACK #(.path_count(16),.path_delay(600),.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY)) I_AVFS_WIREPACK // 4*300ps, osc count = "sample size" / 4
     ( .AvfsPathState(AvfsWirePackState),
       .CntEn_CPOCLK(CntEn_CPOCLK_WP),
       .AvfsPackEn(AvfsWireEn),
       .AvfsCntRst(CntRst),
       .AvfsOscEn(OscEn),
       .extendOscEn(extendOscEn),
       .AvfsOscSel(AvfsWirePackSelDec&{16{~LBistEn}}), // LBistEn=1 drives 16'h0000 to AvfsOscSel to break the loop showing up in DFT runs
       .CCLK(CCLK),
       .AvfsStateWrite(AvfsStateWrite),
       .Reset(Reset),
       .LBistEn(LBistEn),                          // Splitting LBistEn out, so it only resets ripple counter and not CPTiming_DFF
       .ScanShiftEn(ScanShiftEn),
       .AvfsCpoCnt(WirePackCntNxt));

   // 3 path therm pack
  gfx_AVFS3_THERMPACK #(.CCLK_TO_CPO_SYNC_TD_DELAY(CCLK_TO_CPO_SYNC_TD_DELAY)) I_AVFS_THERMPACK
   ( .CntEn_CPOCLK(CntEn_CPOCLK_TP),
      .AvfsPackEn(AvfsThermEn),
      .AvfsCntRst(CntRst),
      .AvfsOscEn(OscEn),
      .extendOscEn(extendOscEn),
      .AvfsOscSel(AvfsThermPackSelDec),
      .Reset(Reset),
      .LBistEn(LBistEn),                          // Splitting LBistEn out, so it only resets ripple counter and not CPTiming_DFF
      .ScanShiftEn(ScanShiftEn),
      .AvfsCpoCnt(ThermPackCntNxt));

   wire CntEn_Syncd_CCLK;

  gfx_CORE_CG I_cg_CntEn (.clkbus(clkbus_CntEn[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(extendOscEn), .R(Reset), .SSE(ScanShiftEn)); 
  gfx_AVFS_SYNC_NOCG #(.TC(CPO_TO_CCLK_SYNC_TC_CYCLES), .IC(0)) I_CntEn_Syncd_GP0 (.SampleClk(clkbus_CntEn[`C_PIN]), .Reset(Reset), .SSE(ScanShiftEn), .DataIn(CntEn_CPOCLK_GP0), .DataOut(CntEn_Syncd_GP0) );
  gfx_AVFS_SYNC_NOCG #(.TC(CPO_TO_CCLK_SYNC_TC_CYCLES), .IC(0)) I_CntEn_Syncd_GP1 (.SampleClk(clkbus_CntEn[`C_PIN]), .Reset(Reset), .SSE(ScanShiftEn), .DataIn(CntEn_CPOCLK_GP1), .DataOut(CntEn_Syncd_GP1) );
  gfx_AVFS_SYNC_NOCG #(.TC(CPO_TO_CCLK_SYNC_TC_CYCLES), .IC(0)) I_CntEn_Syncd_WP (.SampleClk(clkbus_CntEn[`C_PIN]), .Reset(Reset), .SSE(ScanShiftEn), .DataIn(CntEn_CPOCLK_WP), .DataOut(CntEn_Syncd_WP) );
  gfx_AVFS_SYNC_NOCG #(.TC(CPO_TO_CCLK_SYNC_TC_CYCLES), .IC(0)) I_CntEn_Syncd_TP (.SampleClk(clkbus_CntEn[`C_PIN]), .Reset(Reset), .SSE(ScanShiftEn), .DataIn(CntEn_CPOCLK_TP), .DataOut(CntEn_Syncd_TP) );

   assign CntEn_Syncd_CCLK = CntEn_Syncd_GP0 | CntEn_Syncd_GP1 | CntEn_Syncd_WP | CntEn_Syncd_TP;
  gfx_AVFS3_CPO_FSM I_AVFS_CPO_FSM
     (
      .AvfsSmpStart(AvfsSmpStart),
      .AvfsSmpMax(AvfsSmpMax),
      .AvfsGo(AvfsGo),
      .AvfsEn(AvfsEn),
      .CCLK(CCLK),
      .AvfsRepeat(AvfsRepeat),
      .Reset(Reset),
      .ScanShiftEn(ScanShiftEn),
      .CntEn_Syncd_CCLK(CntEn_Syncd_CCLK),
      .CntRst_d1(CntRst),
      .OscEn(OscEn),
      .extendOscEn(extendOscEn),
      .AvfsCapture(AvfsCapture));


   // Moving this flop to AVFS3_TOP level, there is an error in the following tests
   // about data mismatch @150 where model (24'h0) and DUT (24'hxxxx) are mismatched 
   // - avfs3_cpo_top_rg
   // - avfs3_packs_rg

   // // Adding a flop on Reset to block Xs at ZAK level simulations as the core methodology
   // // drives Xs initially before clocks are toggling. Flop Reset here drive Reset_d1 downstream
   // CORE_CGU I_cgu (.clkbus(clkbus_free[`CLKBUS_SIZE-1:0]), .C(CCLK), .R(1'b0), .SSE(1'b0)); 
   // CORE_FLOP_DGATE I_Reset_d1_DFF (.clkbus(clkbus_free[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(Reset), .Q(Reset_d1));

   // GatePackCntNxt0 is asynchronous with a 8 cycle guarantee setup
   // GatePackCntNxt1 is asynchronous with a 8 cycle guarantee setup
   // WirePackCntNxt is asynchronous with a 8 cycle guarantee setup
   // ThermPackCntNxt is asynchronous with a 8 cycle guarantee setup

   wire [`AVFS3_RCNT_WIDTH-1:0]     CDC_GatePackCntNxt0;     
   wire [`AVFS3_RCNT_WIDTH-1:0]     CDC_GatePackCntNxt1;     
   wire [`AVFS3_RCNT_WIDTH-1:0]     CDC_WirePackCntNxt;      
   wire [`AVFS3_RCNT_WIDTH-1:0]     CDC_ThermPackCntNxt;      

  // xfer CPO_CLK to CCLK (CDC)
  // CDCENBUF with oscEn as the enable
  wire cdcbufEn;
  //assign cdcbufEn = ~CntRst & AvfsCapture ;
  assign cdcbufEn = ~CntRst & AvfsCapture & ~Reset & ~ScanShiftEn;  // Added Reset and ScanShitEn terms for bug DECPLCPLK-1825
  gfx_CORE_CDCENBUF #(.size(`AVFS3_RCNT_WIDTH), .Cycles(`CDC_JTAG_X_DLY_CYCLES)) I_CDC_GatePackCntNxt0 (.A(GatePackCntNxt0), .EN(cdcbufEn), .Z(CDC_GatePackCntNxt0), .DSTCLK(CCLK) );
  gfx_CORE_CDCENBUF #(.size(`AVFS3_RCNT_WIDTH), .Cycles(`CDC_JTAG_X_DLY_CYCLES)) I_CDC_GatePackCntNxt1 (.A(GatePackCntNxt1), .EN(cdcbufEn), .Z(CDC_GatePackCntNxt1), .DSTCLK(CCLK) );
  gfx_CORE_CDCENBUF #(.size(`AVFS3_RCNT_WIDTH), .Cycles(`CDC_JTAG_X_DLY_CYCLES)) I_CDC_WirePackCntNxt  (.A(WirePackCntNxt),  .EN(cdcbufEn), .Z(CDC_WirePackCntNxt), .DSTCLK(CCLK) );
  gfx_CORE_CDCENBUF #(.size(`AVFS3_RCNT_WIDTH), .Cycles(`CDC_JTAG_X_DLY_CYCLES)) I_CDC_ThermPackCntNxt (.A(ThermPackCntNxt), .EN(cdcbufEn), .Z(CDC_ThermPackCntNxt), .DSTCLK(CCLK) );

   // Merging gaters per bug : http://bugs.amd.com/ubts/prod/issue/view?id=602121
  gfx_CORE_CG I_cg (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(AvfsCapture|Reset), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE #(.size(`AVFS3_RCNT_WIDTH)) I_gateOscCnt0_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CDC_GatePackCntNxt0), .Q(GatePackCnt0));
  gfx_CORE_FLOP_DGATE #(.size(`AVFS3_RCNT_WIDTH)) I_gateOscCnt1_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CDC_GatePackCntNxt1), .Q(GatePackCnt1));
  gfx_CORE_FLOP_DGATE #(.size(`AVFS3_RCNT_WIDTH)) I_wireOscCnt_DFF  (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CDC_WirePackCntNxt), .Q(WirePackCnt));
  gfx_CORE_FLOP_DGATE #(.size(`AVFS3_RCNT_WIDTH)) I_thermOscCnt_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CDC_ThermPackCntNxt), .Q(ThermPackCnt));

   reg [`AVFS3_RCNT_WIDTH-1:0] next_skip_phase_cnt;
   wire SkipPhase_d1;
   wire SkipPhase_d2;
   wire posEdgeSkipPhase;
   wire negEdgeSkipPhase;

  // posEdge & negEdge detect on SkipPhase
  gfx_CORE_FLOP_GATED_RESET  I_SkipPhase_d1_DFF (.C(CCLK), .D(SkipPhase), .G(OscEn), .GC(1'b0), .R(Reset), .SSE(ScanShiftEn), .Q(SkipPhase_d1)); 
  gfx_CORE_FLOP_GATED_RESET  I_SkipPhase_d2_DFF (.C(CCLK), .D(SkipPhase_d1), .G(OscEn), .GC(1'b0), .R(Reset), .SSE(ScanShiftEn), .Q(SkipPhase_d2)); 
  assign posEdgeSkipPhase = SkipPhase_d1 & ~SkipPhase_d2;
  assign negEdgeSkipPhase = ~SkipPhase_d1 & SkipPhase_d2;

  always @(*) begin
   if ((ScanShiftEn==1'b1) || (Reset==1'b1) )  
     next_skip_phase_cnt[`AVFS3_RCNT_WIDTH-1:0] = {`AVFS3_RCNT_WIDTH{1'b0}};
   else if(OscEn==1'b1)   
     next_skip_phase_cnt[`AVFS3_RCNT_WIDTH-1:0] =  (posEdgeSkipPhase==1'b1) ? (SkipPhaseCnt_d0 + 24'h000001) : ((negEdgeSkipPhase==1'b1) ? (SkipPhaseCnt_d0 + 24'h000001) : SkipPhaseCnt_d0) ;
   else 
     next_skip_phase_cnt[`AVFS3_RCNT_WIDTH-1:0] =  SkipPhaseCnt_d0 ;
  end  

   //// This logic got messed up, SkipPhase was not even in the sensitivity list, skip_phase_cnt was incrementing as long as OscEn was high, Ugh!
   //// FIX to bug : http://bugs.amd.com/ubts/prod/issue/view?id=642651#c11  
   //always @(*)
   //  begin
   //    //if ((ScanShiftEn==1'b1) || (Reset==1'b1) || (OscEn==1'b0)) 
   //    if ((ScanShiftEn==1'b1) || (Reset==1'b1) || (CntRst==1'b1)) 
   //      begin
   //        next_skip_phase_cnt[`AVFS3_RCNT_WIDTH-1:0] = {`AVFS3_RCNT_WIDTH{1'b0}};
   //      end
   //    else if (OscEn==1'b1)
   //      begin
   //        next_skip_phase_cnt[`AVFS3_RCNT_WIDTH-1:0] =  (SkipPhase==1'b1) ? (SkipPhaseCnt + 24'h000001) : ((SkipPhase==1'b0) ? (SkipPhaseCnt + 24'h000001) : 24'hxxxxxx) ;
   //      end
   //  end

  gfx_CORE_FLOP_GATED_RESET #(.size(`AVFS3_RCNT_WIDTH)) I_SkipPhaseCnt_d0_DFF (.C(CCLK), .D(next_skip_phase_cnt), .G(OscEn), .GC(1'b0), .R(CntRst|Reset), .SSE(ScanShiftEn), .Q(SkipPhaseCnt_d0)); 
   // Need to capture the skip_phase_counter value here
   //CORE_FLOP_GATED_RESET #(.size(`AVFS3_RCNT_WIDTH)) I_SkipPhaseCnt_DFF (.C(CCLK), .D(SkipPhaseCnt_d0), .G(AvfsCapture), .GC(1'b0), .R(Reset), .SSE(ScanShiftEn), .Q(SkipPhaseCnt)); 
  gfx_CORE_FLOP_DGATE #(.size(`AVFS3_RCNT_WIDTH)) I_skipPhaseCnt_DFF (.clkbus(clkbus[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(SkipPhaseCnt_d0), .Q(SkipPhaseCnt));

endmodule // AVFS3_CPO
//==================================================================================================
//! enable one_module_per_file
// leda E_001 on
//==================================================================================================

