//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps

`include "gc_features.vh"
`include "mem_features.vh"

module gfx_cgtt_local(/*AUTOARG*/
   // Outputs
   static_oclk, dyn_oclk, reg_oclk, dyn_oclk_vld, reg_oclk_vld, p1_mem_power_ctrl0,
   // Inputs
   Cpl_GFXCLK, dft_clk_gate_ctrl, dft_shift_clk,
   resetb, iclk, cgcg_en, cg_override, core_busy, reg_busy, mgls_override, mgls_current_frequency_code, TTOP_P1_mem_power_ctrl, CGTT_BLK_CTRL_REG, stall
   );

//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cgcg_en;
 input       cg_override; /*functional or testmode override. testmode override is incorporated in cg*/
 input        Cpl_GFXCLK;
 input  [1:0] dft_clk_gate_ctrl;
 input        dft_shift_clk;
        /*it is an async signal*/

 output      static_oclk;
 output      dyn_oclk;
 output      reg_oclk;
 output      dyn_oclk_vld;
 output      reg_oclk_vld;

 output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl0;

 input       core_busy;
 input       reg_busy;
 input       stall;

 input        mgls_override;
 input [3:0]  mgls_current_frequency_code;
 input [`MEM__MEM_POWER_CTRL -1 : 0] TTOP_P1_mem_power_ctrl;

 input [31:0] CGTT_BLK_CTRL_REG;

parameter switch_code = 0;
parameter scan_grp_dyn_oclk = 0 ; //defaulted
parameter enable_dftcgmg_gate_merge = 0;
parameter use_dcg_lite = 0;

/////////////////////////////////////////////////////
// Medium Grain Light Sleep control
/////////////////////////////////////////////////////
reg ls_override;
reg override_s;

always @(posedge static_oclk) begin
  if(~resetb) begin
    ls_override <= 1'b1;
    override_s  <= 1'b1;
  end else begin
    ls_override <= mgls_override || (mgls_current_frequency_code > CGTT_BLK_CTRL_REG[3:0]); // if current frequency code is greater than ON_DELAY, the disable medium grain Light Sleep
    override_s  <= cg_override;
  end
end


wire mgls0;

reg [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl0;

always @(*) begin
   p1_mem_power_ctrl0 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl0[`MEM__MEM_POWER_CTRL_LS] = mgls0 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
end

/////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////
//block CGTT register
/////////////////////////////////////////////////////
// ON_DELAY     = CGTT_BLK_CTRL_REG[3:0];  //dyn_oclk ON is delayed by this counter.
// OFF_DELAY    = CGTT_BLK_CTRL_REG[11:4]; //clk_oclk OFF is delayed by this counter, also if busy is shorter than OFF_DELAY,
//               then it is filtered out. OFF_DELAY must be prorgammed to be greater than ON_DELAY.
// SOFT_CORE_OVERRIDE = CGTT_BLK_CTRL_REG[30];  //busy is ignored & dyn_oclk in always turned on.
// SOFT_REG_OVERRIDE  = CGTT_BLK_CTRL_REG[31];  //busy is ignored & reg_oclk in always turned on.
/////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////

wire        sclk;
reg         n0scan_static_oclk_en;

`ifdef GC__CGCG_MGCG_MERGE__1
  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(dft_shift_clk),
   .DFT_TE(dft_te_wire)
  );

reg                     n0scan_cgcg_en_d1;
always @(posedge sclk) begin
  n0scan_cgcg_en_d1 <= cgcg_en;
end

generate
  if(enable_dftcgmg_gate_merge == 1)
ati_clock_gate u_static_oclk(
    .clk_out (static_oclk),
    .clk_src (sclk),
    .clk_en  (n0scan_static_oclk_en),
    .TE      (dft_te_wire)
   );
  else
ati_clock_gate u_static_oclk(
    .clk_out (static_oclk),
    .clk_src (sclk),
    .clk_en  (n0scan_cgcg_en_d1),
    .TE      (dft_te_wire)
   );
endgenerate

`else
aticlk_buf u_static_clk(.Y(static_oclk), .A(iclk));
`endif

reg  [1:0]   n0scan_dft_clk_gate_ctrl_d1;
wire [1:0]   wdft_clk_gate_ctrl;

generate
  if(enable_dftcgmg_gate_merge == 1) begin
    always @(posedge Cpl_GFXCLK) begin
      n0scan_dft_clk_gate_ctrl_d1 <= dft_clk_gate_ctrl[1:0];
      n0scan_static_oclk_en       <= (n0scan_cgcg_en_d1 & ~n0scan_dft_clk_gate_ctrl_d1[1]) | n0scan_dft_clk_gate_ctrl_d1[0];
    end
    assign sclk = Cpl_GFXCLK;
    assign wdft_clk_gate_ctrl = n0scan_dft_clk_gate_ctrl_d1;
  end
  else begin
    assign sclk = iclk;
    assign wdft_clk_gate_ctrl = 2'b00;
  end
endgenerate

//  wire override_s;
//sync usync_override( .D(cg_override), .CK(static_oclk), .Q(override_s) );

parameter enable_dcg = 1; // defaulted
generate
if(enable_dcg == 1 ) begin : dyn_oclk_gen
gfx_cgtt_local_dcg #( .scan_grp_dyn_oclk (scan_grp_dyn_oclk) )
   u_gfx_cgtt_local_dclk
  (.soft_override  (CGTT_BLK_CTRL_REG[30]),
   .soft_on_delay  (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay (CGTT_BLK_CTRL_REG[11:4]),
   .soft_stall_override (CGTT_BLK_CTRL_REG[22]),
   .stall          (stall),
  /*AUTOINST*/
   // Outputs
   .dyn_oclk       (dyn_oclk),
   .dyn_oclk_vld   (dyn_oclk_vld),
   // Inputs
   .resetb         (resetb),
   .iclk           (sclk),
   .cgcg_en        (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override    (override_s),
   .core_busy      (core_busy),
   .ls_override    (ls_override),
   .mgls           (mgls0),
   .static_oclk    (static_oclk));
end
else if (use_dcg_lite == 1) begin : dyn_oclk_lite_gen
gfx_cgtt_local_dcg_lite #( .scan_grp_dyn_oclk (scan_grp_dyn_oclk) )
   u_gfx_cgtt_local_dclk
  (.soft_override  (CGTT_BLK_CTRL_REG[30]),
   .soft_stall_override (CGTT_BLK_CTRL_REG[22]),
   .stall          (stall),
  /*AUTOINST*/
   // Outputs
   .dyn_oclk       (dyn_oclk),
   .dyn_oclk_vld   (dyn_oclk_vld),
   // Inputs
   .resetb         (resetb),
   .iclk           (sclk),
   .cgcg_en        (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override    (override_s),
   .core_busy      (core_busy),
   .ls_override    (ls_override),
   .mgls           (mgls0),
   .static_oclk    (static_oclk));
end
else begin
assign dyn_oclk     = 1'b0;
assign dyn_oclk_vld = 1'b0;
end
endgenerate

parameter scan_grp_reg_oclk = 0 ; //defaulted

parameter enable_rcg = 1; // defaulted

generate
if(enable_rcg == 1)
begin : reg_oclk_gen
gfx_cgtt_local_rcg #(.scan_grp_reg_oclk (scan_grp_reg_oclk) )
   u_gfx_cgtt_local_rclk
  (.soft_override  (CGTT_BLK_CTRL_REG[31]),
  /*AUTOINST*/
   // Outputs
   .reg_oclk       (reg_oclk),
   .reg_oclk_vld   (reg_oclk_vld),
   // Inputs
   .resetb         (resetb),
   .iclk           (sclk),
   .cgcg_en        (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override    (override_s),
   .reg_busy       (reg_busy),
   .static_oclk    (static_oclk));
end
else
begin
  assign reg_oclk     = 1'b0;
  assign reg_oclk_vld = 1'b0;
end
endgenerate
/////////////////////////////////////////////////////////////////////////
//To provide information about how often a block is turning its clock off
/////////////////////////////////////////////////////////////////////////
`ifdef SIMULATION
integer dyn_oclk_off_count;
integer reg_oclk_off_count;
integer static_oclk_count;
integer    msg_rate;

initial
begin
  dyn_oclk_off_count = 0;
  reg_oclk_off_count = 0;
  static_oclk_count  = 0;
end

always @(posedge static_oclk)
begin
  if(resetb && ~dyn_oclk_vld) dyn_oclk_off_count = dyn_oclk_off_count + 1;  //don't count when resetb is active
  if(resetb && ~reg_oclk_vld) reg_oclk_off_count = reg_oclk_off_count + 1;

  if(resetb) static_oclk_count = static_oclk_count + 1;

//VCS coverage off
//  if( $test$plusargs("LOCAL_CLOCK_GATING_INFO") ) begin
//
//     if(resetb && ((static_oclk_count % 1000)==0)) begin
//      $display("INFO: LCG: percentage of cycles dyn-clk is off = %f, total cycles=%f; local-clock-gate instance = %m\n", dyn_oclk_off_count / static_oclk_count, static_oclk_count);
//      $display("INFO: LCG: percentage of cycles reg-clk is off = %f, total cycles=%f; local-clock-gate instance = %m\n", reg_oclk_off_count / static_oclk_count, static_oclk_count);
//    end
//
//  end else if( $value$plusargs("LOCAL_CLOCK_GATING_MSG_RATE+%d", msg_rate) ) begin //message will be printed after every 'msg_rate' number of cycles
//
//     if(resetb && ((static_oclk_count % msg_rate)==0)) begin
//      $display("INFO: LCG: percentage of cycles dyn-clk is off = %f, total cycles=%f; local-clock-gate instance = %m\n", dyn_oclk_off_count / static_oclk_count, static_oclk_count);
//      $display("INFO: LCG: percentage of cycles reg-clk is off = %f, total cycles=%f; local-clock-gate instance = %m\n", reg_oclk_off_count / static_oclk_count, static_oclk_count);
//    end
//  end
//VCS coverage on

  //TBD: how do we print at the end of sim. i.e. how can I capture end of sim condition & then print info.
  //ggreenst: most testbenches use "always @(posedge tb.pending_finish)" for this...

end

`endif

endmodule
