//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three type of clocks: static_oclk, dyn_oclki*, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk* is gated based on your 'busy' signals. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps

`include "gc_features.vh"
`include "mem_features.vh"

module gfx_cgtt_local_1r1d(/*AUTOARG*/
   // Outputs
   dyn_oclk0,
   dyn_oclk_vld0,
   static_oclk, reg_oclk, reg_oclk_vld,
   p1_mem_power_ctrl0,
   // Inputs
   Cpl_GFXCLK, dft_clk_gate_ctrl, dft_shift_clk,
   resetb, iclk, cgcg_en, cg_override,
   core_busy0, reg_busy, stall,
   mgls_override,
   mgls_current_frequency_code,
   TTOP_P1_mem_power_ctrl,
   CGTT_BLK_CTRL_REG
   );

//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cgcg_en;
 input       cg_override; /*functional or testmode override. testmode override is incorporated in cg*/
 input        Cpl_GFXCLK;
 input  [1:0] dft_clk_gate_ctrl;
 input        dft_shift_clk;
        /*it is an async signal*/

 output      dyn_oclk0;
 output      dyn_oclk_vld0;
 output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl0;

 input       core_busy0;
 input       stall;

 output      static_oclk;
 output      reg_oclk;
 output      reg_oclk_vld;
 input       reg_busy;

 input        mgls_override;
 input [3:0]  mgls_current_frequency_code;
 input [31:0] CGTT_BLK_CTRL_REG;
 input [`MEM__MEM_POWER_CTRL -1 : 0] TTOP_P1_mem_power_ctrl;

parameter switch_code = 0;
parameter enable_dftcgmg_gate_merge = 0;

/////////////////////////////////////////////////////
// Medium Grain Light Sleep control
/////////////////////////////////////////////////////
reg ls_override;
reg override_s;

always @(posedge static_oclk) begin
  if(~resetb) begin
    ls_override <= 1'b1;
    override_s  <= 1'b1;
  end else begin
    ls_override <= mgls_override || (mgls_current_frequency_code > CGTT_BLK_CTRL_REG[3:0]); // if current frequency code is greater than ON_DELAY, the disable medium grain Light Sleep
    override_s  <= cg_override;
  end
end


wire mgls0;

reg [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl0;

always @(*) begin
   p1_mem_power_ctrl0 = TTOP_P1_mem_power_ctrl;
   p1_mem_power_ctrl0[`MEM__MEM_POWER_CTRL_LS] = mgls0 | TTOP_P1_mem_power_ctrl[`MEM__MEM_POWER_CTRL_LS];
end


/////////////////////////////////////////////////////
//instantiate base module
/////////////////////////////////////////////////////

wire        sclk;
reg         n0scan_static_oclk_en;

`ifdef GC__CGCG_MGCG_MERGE__1
  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(dft_shift_clk),
   .DFT_TE(dft_te_wire)
  );

reg                     n0scan_cgcg_en_d1;
always @(posedge sclk) begin
  n0scan_cgcg_en_d1 <= cgcg_en;
end

generate
  if(enable_dftcgmg_gate_merge == 1)
ati_clock_gate u_static_oclk(
    .clk_out (static_oclk),
    .clk_src (sclk),
    .clk_en  (n0scan_static_oclk_en),
    .TE      (dft_te_wire)
   );
  else
ati_clock_gate u_static_oclk(
    .clk_out (static_oclk),
    .clk_src (sclk),
    .clk_en  (n0scan_cgcg_en_d1),
    .TE      (dft_te_wire)
   );
endgenerate

`else
aticlk_buf u_static_clk(.Y(static_oclk), .A(iclk));
`endif

reg  [1:0]   n0scan_dft_clk_gate_ctrl_d1;
wire [1:0]   wdft_clk_gate_ctrl;

generate
  if(enable_dftcgmg_gate_merge == 1) begin
    always @(posedge Cpl_GFXCLK) begin
      n0scan_dft_clk_gate_ctrl_d1 <= dft_clk_gate_ctrl[1:0];
      n0scan_static_oclk_en       <= (n0scan_cgcg_en_d1 & ~n0scan_dft_clk_gate_ctrl_d1[1]) | n0scan_dft_clk_gate_ctrl_d1[0];
    end
    assign sclk = Cpl_GFXCLK;
    assign wdft_clk_gate_ctrl = n0scan_dft_clk_gate_ctrl_d1;
  end
  else begin
    assign sclk = iclk;
    assign wdft_clk_gate_ctrl = 2'b00;
  end
endgenerate

parameter scan_grp_dyn_oclk0 = 0; //defaulted

parameter enable_dcg = 1; // defaulted
parameter use_dcg_lite = 0; // defaulted

generate
if(enable_dcg == 1) begin : dyn_oclk_gen
gfx_cgtt_local_dcg #(
          .scan_grp_dyn_oclk (scan_grp_dyn_oclk0)
        )
        u_cgtt_dclk0
  (.dyn_oclk      (dyn_oclk0),
   .dyn_oclk_vld      (dyn_oclk_vld0),
   .core_busy     (core_busy0),
   .soft_on_delay     (CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay    (CGTT_BLK_CTRL_REG[11:4]),
   .soft_override     (CGTT_BLK_CTRL_REG[30]),
   .soft_stall_override     (CGTT_BLK_CTRL_REG[22]),
   .stall             (stall),
   /*AUTOINST*/
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .ls_override     (ls_override),
   .mgls            (mgls0),
   .static_oclk     (static_oclk));
end
else if (use_dcg_lite == 1) begin : dyn_oclk_lite_gen
gfx_cgtt_local_dcg_lite #( .scan_grp_dyn_oclk (scan_grp_dyn_oclk0) )
    u_cgtt_dclk0
  (.soft_override  (CGTT_BLK_CTRL_REG[30]),
   .soft_stall_override (CGTT_BLK_CTRL_REG[22]),
   .stall          (stall),
  /*AUTOINST*/
   // Outputs
   .dyn_oclk       (dyn_oclk),
   .dyn_oclk_vld   (dyn_oclk_vld),
   // Inputs
   .resetb         (resetb),
   .iclk           (sclk),
   .cgcg_en        (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override    (override_s),
   .core_busy      (core_busy),
   .ls_override    (ls_override),
   .mgls           (mgls0),
   .static_oclk    (static_oclk));
end
else begin
assign dyn_oclk0     = 1'b0;
assign dyn_oclk_vld0 = 1'b0;
end
endgenerate

parameter scan_grp_reg_oclk = 0; //defaulted

parameter enable_rcg = 1; // defaulted

generate
if(enable_rcg == 1)
begin : reg_oclk_gen
gfx_cgtt_local_rcg #(
          .scan_grp_reg_oclk (scan_grp_reg_oclk)
        )
        u_cgtt_rclk
  (.soft_override   (CGTT_BLK_CTRL_REG[31]),
   /*AUTOINST*/
   // Outputs
   .reg_oclk      (reg_oclk),
   .reg_oclk_vld      (reg_oclk_vld),
   // Inputs
   .resetb      (resetb),
   .iclk        (sclk),
   .cgcg_en         (cgcg_en),
   .dft_clk_gate_ctrl (wdft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk),
   .cg_override     (override_s),
   .reg_busy      (reg_busy),
   .static_oclk     (static_oclk));
end
else
begin
  assign reg_oclk     = 1'b0;
  assign reg_oclk_vld = 1'b0;
end
endgenerate


endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////
