//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic. 
//	       dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//	       by enable signal coming from xRBM block.
//
//	       If you find any issue or have any feedback, please email to sagheer.ahmad@amd.com
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps

`include "gc_features.vh"
`include "mem_features.vh"

module gfx_cgtt_local_1r1d_wrapper (
   resetb,
   iclk,
   cgcg_en,
   Cpl_GFXCLK,
   dft_clk_gate_ctrl,
   dft_shift_clk,
   cg_override,
   reg_busy,
   core_busy,
   stall,
   mgls_override,
   mgls_current_frequency_code,
   TTOP_P1_mem_power_ctrl,
   CGTT_BLK_CTRL_REG,

   static_oclk,
   dyn_oclk,
   dyn_oclk_vld,
   p1_mem_power_ctrl0,
   reg_oclk,
   reg_oclk_vld
);

//////////////////////////
//in/outs
//////////////////////////
input       resetb;	
input       iclk;
input       cgcg_en;
input       cg_override; 
input       Cpl_GFXCLK;
input [1:0] dft_clk_gate_ctrl;
input       dft_shift_clk;

input       reg_busy;
input       core_busy;
input       stall;
input [31:0] CGTT_BLK_CTRL_REG;
input        mgls_override;
input [3:0]  mgls_current_frequency_code;
input [`MEM__MEM_POWER_CTRL -1 : 0] TTOP_P1_mem_power_ctrl;

output      static_oclk;
output      dyn_oclk;
output      dyn_oclk_vld;
output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl0;
output      reg_oclk;
output      reg_oclk_vld;

reg         reg_busy_d1;
reg         reg_busy_d2;
wire        reg_busy_extend;
wire        reg_soft_override;
reg         reg_soft_override_d1;
reg         reg_soft_override_d2;
wire        reg_soft_override_extend;

reg         core_busy_d1;
reg         core_busy_d2;
wire        core_busy_extend;
wire        core_soft_override;
reg         core_soft_override_d1;
reg         core_soft_override_d2;
wire        core_soft_override_extend;

wire [31:0] cgtt_blk_ctrl_reg_modified;

assign reg_soft_override = CGTT_BLK_CTRL_REG[31];
assign core_soft_override = CGTT_BLK_CTRL_REG[30];
assign soft_stall_override = CGTT_BLK_CTRL_REG[22];

wire unstall_clk = ~stall | soft_stall_override;

always @(posedge static_oclk)
begin
   reg_busy_d1 <= reg_busy;
   reg_busy_d2 <= reg_busy_d1;

   core_busy_d1 <= core_busy && unstall_clk;
   core_busy_d2 <= core_busy_d1;

   reg_soft_override_d1 <= reg_soft_override;
   reg_soft_override_d2 <= reg_soft_override_d1;

   core_soft_override_d1 <= core_soft_override;
   core_soft_override_d2 <= core_soft_override_d1;
end 
 
assign reg_busy_extend  = reg_busy  || reg_busy_d1  || reg_busy_d2;
assign core_busy_extend = core_busy || core_busy_d1 || core_busy_d2;
assign reg_soft_override_extend  = reg_soft_override  || reg_soft_override_d1  || reg_soft_override_d2;
assign core_soft_override_extend = core_soft_override || core_soft_override_d1 || core_soft_override_d2;

assign cgtt_blk_ctrl_reg_modified = {reg_soft_override_extend, core_soft_override_extend, 
                                     CGTT_BLK_CTRL_REG[29:0]};

parameter switch_code = 0;
parameter enable_dftcgmg_gate_merge = 0;

gfx_cgtt_local #(.switch_code(switch_code), .enable_dftcgmg_gate_merge(enable_dftcgmg_gate_merge)) uCGTT_LOCAL (
   .resetb            (resetb),
   .iclk              (iclk),
   .cgcg_en           (cgcg_en),
   .Cpl_GFXCLK        (Cpl_GFXCLK       ),
   .dft_clk_gate_ctrl (dft_clk_gate_ctrl),
   .dft_shift_clk     (dft_shift_clk    ),
   .cg_override       (cg_override), 
   .reg_busy          (reg_busy_extend),
   .core_busy         (core_busy_extend),
   .stall             (stall),
   .CGTT_BLK_CTRL_REG (cgtt_blk_ctrl_reg_modified),
   .mgls_override               (mgls_override),
   .mgls_current_frequency_code (mgls_current_frequency_code),
   .TTOP_P1_mem_power_ctrl      (TTOP_P1_mem_power_ctrl),
   .static_oclk       (static_oclk),
   .reg_oclk          (reg_oclk),
   .dyn_oclk          (dyn_oclk),
   .reg_oclk_vld      (),
   .dyn_oclk_vld      (),
   .p1_mem_power_ctrl0(p1_mem_power_ctrl0)
);

gfx_cgtt_local_dclkon uDYN_CLK_VLD (
   .dyn_oclk_vld(dyn_oclk_vld),
   .core_busy(core_busy),
   .unstall_clk(unstall_clk),
   .resetb(resetb), 
   .static_oclk(static_oclk),
   .soft_override(core_soft_override),
   .soft_on_delay(CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay(CGTT_BLK_CTRL_REG[11:4])
);

gfx_cgtt_local_rclkon uREG_CLK_VLD (
   .reg_oclk_vld(reg_oclk_vld), 
   .reg_busy(reg_busy),
   .resetb(resetb),
   .static_oclk(static_oclk),
   .soft_override(reg_soft_override)
);

endmodule
