//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic. 
//	       dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//	       by enable signal coming from xRBM block.
//
//	       If you find any issue or have any feedback, please email to sagheer.ahmad@amd.com
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps

`include "gc_features.vh"
`include "mem_features.vh"

module gfx_cgtt_local_1r4d_wrapper (
   resetb,
   iclk,
   cgcg_en,
   Cpl_GFXCLK,
   dft_clk_gate_ctrl,
   dft_shift_clk,
   cg_override,
   reg_busy,
   core_busy0,
   core_busy1,
   core_busy2,
   core_busy3,
   stall0,
   stall1,
   stall2,
   stall3,
   CGTT_BLK_CTRL_REG,
   mgls_override,
   mgls_current_frequency_code,
   TTOP_P1_mem_power_ctrl,

   static_oclk,
   dyn_oclk0,
   dyn_oclk_vld0,
   dyn_oclk1,
   dyn_oclk_vld1,
   dyn_oclk2,
   dyn_oclk_vld2,
   dyn_oclk3,
   dyn_oclk_vld3,
   reg_oclk,
   reg_oclk_vld,
   p1_mem_power_ctrl0,
   p1_mem_power_ctrl1,
   p1_mem_power_ctrl2,
   p1_mem_power_ctrl3
);

//////////////////////////
//in/outs
//////////////////////////
input       resetb;	
input       iclk;
input       cgcg_en;
input       cg_override; 
input       Cpl_GFXCLK;
input [1:0] dft_clk_gate_ctrl;
input       dft_shift_clk;

input       reg_busy;
input       core_busy0;
input       core_busy1;
input       core_busy2;
input       core_busy3;
input       stall0;
input       stall1;
input       stall2;
input       stall3;
input [31:0] CGTT_BLK_CTRL_REG;
input       mgls_override;
input [3:0] mgls_current_frequency_code;
input [`MEM__MEM_POWER_CTRL -1 : 0] TTOP_P1_mem_power_ctrl;

output      static_oclk;
output      dyn_oclk0;
output      dyn_oclk_vld0;
output      dyn_oclk1;
output      dyn_oclk_vld1;
output      dyn_oclk2;
output      dyn_oclk_vld2;
output      dyn_oclk3;
output      dyn_oclk_vld3;
output      reg_oclk;
output      reg_oclk_vld;
output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl0;
output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl1;
output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl2;
output [`MEM__MEM_POWER_CTRL -1 : 0] p1_mem_power_ctrl3;

reg         reg_busy_d1;
reg         reg_busy_d2;
wire        reg_busy_extend;
wire        reg_soft_override;
reg         reg_soft_override_d1;
reg         reg_soft_override_d2;
wire        reg_soft_override_extend;

reg         core_busy0_d1;
reg         core_busy0_d2;
wire        core_busy0_extend;
wire        core_soft_override0;
reg         core_soft_override0_d1;
reg         core_soft_override0_d2;
wire        core_soft_override0_extend;

reg         core_busy1_d1;
reg         core_busy1_d2;
wire        core_busy1_extend;
wire        core_soft_override1;
reg         core_soft_override1_d1;
reg         core_soft_override1_d2;
wire        core_soft_override1_extend;

reg         core_busy2_d1;
reg         core_busy2_d2;
wire        core_busy2_extend;
wire        core_soft_override2;
reg         core_soft_override2_d1;
reg         core_soft_override2_d2;
wire        core_soft_override2_extend;

reg         core_busy3_d1;
reg         core_busy3_d2;
wire        core_busy3_extend;
wire        core_soft_override3;
reg         core_soft_override3_d1;
reg         core_soft_override3_d2;
wire        core_soft_override3_extend;

wire [31:0] cgtt_blk_ctrl_reg_modified;

assign reg_soft_override = CGTT_BLK_CTRL_REG[31];
assign core_soft_override0 = CGTT_BLK_CTRL_REG[30];
assign core_soft_override1 = CGTT_BLK_CTRL_REG[29];
assign core_soft_override2 = CGTT_BLK_CTRL_REG[28];
assign core_soft_override3 = CGTT_BLK_CTRL_REG[27];
assign soft_stall0_override = CGTT_BLK_CTRL_REG[22];
assign soft_stall1_override = CGTT_BLK_CTRL_REG[21];
assign soft_stall2_override = CGTT_BLK_CTRL_REG[20];
assign soft_stall3_override = CGTT_BLK_CTRL_REG[19];


wire unstall_clk0 = ~stall0 | soft_stall0_override;
wire unstall_clk1 = ~stall1 | soft_stall1_override;
wire unstall_clk2 = ~stall2 | soft_stall2_override;
wire unstall_clk3 = ~stall3 | soft_stall3_override;

always @(posedge static_oclk)
begin
   reg_busy_d1 <= reg_busy;
   reg_busy_d2 <= reg_busy_d1;

   reg_soft_override_d1 <= reg_soft_override;
   reg_soft_override_d2 <= reg_soft_override_d1;

   core_busy0_d1 <= core_busy0 && unstall_clk0;
   core_busy0_d2 <= core_busy0_d1;
   core_busy1_d1 <= core_busy1 && unstall_clk1;
   core_busy1_d2 <= core_busy1_d1;
   core_busy2_d1 <= core_busy2 && unstall_clk2;
   core_busy2_d2 <= core_busy2_d1;
   core_busy3_d1 <= core_busy3 && unstall_clk3;
   core_busy3_d2 <= core_busy3_d1;

   core_soft_override0_d1 <= core_soft_override0;
   core_soft_override0_d2 <= core_soft_override0_d1;
   core_soft_override1_d1 <= core_soft_override1;
   core_soft_override1_d2 <= core_soft_override1_d1;
   core_soft_override2_d1 <= core_soft_override2;
   core_soft_override2_d2 <= core_soft_override2_d1;
   core_soft_override3_d1 <= core_soft_override3;
   core_soft_override3_d2 <= core_soft_override3_d1;

end 
 
assign reg_busy_extend   = reg_busy   || reg_busy_d1   || reg_busy_d2;
assign core_busy0_extend = core_busy0 || core_busy0_d1 || core_busy0_d2;
assign core_busy1_extend = core_busy1 || core_busy1_d1 || core_busy1_d2;
assign core_busy2_extend = core_busy2 || core_busy2_d1 || core_busy2_d2;
assign core_busy3_extend = core_busy3 || core_busy3_d1 || core_busy3_d2;

assign reg_soft_override_extend   = reg_soft_override   || reg_soft_override_d1   || reg_soft_override_d2;
assign core_soft_override0_extend = core_soft_override0 || core_soft_override0_d1 || core_soft_override0_d2;
assign core_soft_override1_extend = core_soft_override1 || core_soft_override1_d1 || core_soft_override1_d2;
assign core_soft_override2_extend = core_soft_override2 || core_soft_override2_d1 || core_soft_override2_d2;
assign core_soft_override3_extend = core_soft_override3 || core_soft_override3_d1 || core_soft_override3_d2;

assign cgtt_blk_ctrl_reg_modified = {reg_soft_override_extend, core_soft_override0_extend, core_soft_override1_extend, core_soft_override2_extend, core_soft_override3_extend,
                                     CGTT_BLK_CTRL_REG[26:0]};

parameter switch_code = 0;
parameter enable_dftcgmg_gate_merge = 0;

gfx_cgtt_local_1r4d  #(.switch_code(switch_code), .enable_dftcgmg_gate_merge(enable_dftcgmg_gate_merge))
  uCGTT_LOCAL (
   .resetb            (resetb),
   .iclk              (iclk),
   .cgcg_en           (cgcg_en),
   .Cpl_GFXCLK(Cpl_GFXCLK),
   .dft_clk_gate_ctrl(dft_clk_gate_ctrl),
   .dft_shift_clk(dft_shift_clk),
   .cg_override       (cg_override), 
   .reg_busy          (reg_busy_extend),
   .core_busy0        (core_busy0_extend),
   .core_busy1        (core_busy1_extend),
   .core_busy2        (core_busy2_extend),
   .core_busy3        (core_busy3_extend),
   .CGTT_BLK_CTRL_REG (cgtt_blk_ctrl_reg_modified),
   .stall0            (stall0),
   .stall1            (stall1),
   .stall2            (stall2),
   .stall3            (stall3),
   .mgls_override               (mgls_override),
   .mgls_current_frequency_code (mgls_current_frequency_code),
   .TTOP_P1_mem_power_ctrl      (TTOP_P1_mem_power_ctrl),
   .static_oclk       (static_oclk),
   .reg_oclk          (reg_oclk),
   .dyn_oclk0         (dyn_oclk0),
   .dyn_oclk1         (dyn_oclk1),
   .dyn_oclk2         (dyn_oclk2),
   .dyn_oclk3         (dyn_oclk3),
   .reg_oclk_vld      (),
   .dyn_oclk_vld0     (),
   .dyn_oclk_vld1     (),
   .dyn_oclk_vld2     (),
   .dyn_oclk_vld3     (),
   .p1_mem_power_ctrl0(p1_mem_power_ctrl0),
   .p1_mem_power_ctrl1(p1_mem_power_ctrl1),
   .p1_mem_power_ctrl2(p1_mem_power_ctrl2),
   .p1_mem_power_ctrl3(p1_mem_power_ctrl3));

gfx_cgtt_local_dclkon uDYN_CLK_VLD0 (
   .dyn_oclk_vld(dyn_oclk_vld0),
   .core_busy(core_busy0),
   .unstall_clk(unstall_clk0),
   .resetb(resetb), 
   .static_oclk(static_oclk),
   .soft_override(core_soft_override0),
   .soft_on_delay(CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay(CGTT_BLK_CTRL_REG[11:4])
);

gfx_cgtt_local_dclkon uDYN_CLK_VLD1 (
   .dyn_oclk_vld(dyn_oclk_vld1),
   .core_busy(core_busy1),
   .unstall_clk(unstall_clk1),
   .resetb(resetb), 
   .static_oclk(static_oclk),
   .soft_override(core_soft_override1),
   .soft_on_delay(CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay(CGTT_BLK_CTRL_REG[11:4])
);

gfx_cgtt_local_dclkon uDYN_CLK_VLD2 (
   .dyn_oclk_vld(dyn_oclk_vld2),
   .core_busy(core_busy2),
   .unstall_clk(unstall_clk2),
   .resetb(resetb), 
   .static_oclk(static_oclk),
   .soft_override(core_soft_override2),
   .soft_on_delay(CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay(CGTT_BLK_CTRL_REG[11:4])
);

gfx_cgtt_local_dclkon uDYN_CLK_VLD3 (
   .dyn_oclk_vld(dyn_oclk_vld3),
   .core_busy(core_busy3),
   .unstall_clk(unstall_clk3),
   .resetb(resetb), 
   .static_oclk(static_oclk),
   .soft_override(core_soft_override3),
   .soft_on_delay(CGTT_BLK_CTRL_REG[3:0]),
   .soft_off_delay(CGTT_BLK_CTRL_REG[11:4])
);

gfx_cgtt_local_rclkon uREG_CLK_VLD (
   .reg_oclk_vld(reg_oclk_vld), 
   .reg_busy(reg_busy),
   .resetb(resetb),
   .static_oclk(static_oclk),
   .soft_override(reg_soft_override)
);

endmodule
