//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps
// following defines are added to instance the clock buffers and mark them for scan grouping

`define SCAN_GRP0 0
`define SCAN_GRP1 1
`define SCAN_GRP2 2
`define SCAN_GRP3 3
`define SCAN_GRP4 4
`define SCAN_GRP5 5
`define SCAN_GRP6 6
`define SCAN_GRP7 7
`define SCAN_GRP8 8

`include "gc_features.vh"

module gfx_cgtt_local_dcg(/*AUTOARG*/
   // Outputs
   dyn_oclk, dyn_oclk_vld, mgls,
   // Inputs
   dft_clk_gate_ctrl, dft_shift_clk,
   resetb, iclk, cgcg_en, cg_override, core_busy, soft_on_delay,
   soft_off_delay, soft_override, static_oclk, ls_override, soft_stall_override, stall
   );
//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cg_override;
 input       cgcg_en;
 input       core_busy;
 input  [3:0] soft_on_delay;
 input [11:4] soft_off_delay;
 input        soft_override;
 input       static_oclk;
 input       ls_override;
 input       soft_stall_override;
 input       stall;
 input  [1:0] dft_clk_gate_ctrl;
 input        dft_shift_clk;



 output		dyn_oclk;
 output		dyn_oclk_vld;
 output		mgls;
//////////////////////////

reg  core_busy_d1;
reg  core_busy_d2;
reg  core_busy_filtered;
reg  core_busy_filtered_d1;

reg  [3:0] on_counter;
reg [11:4] off_counter;

reg        core_busy_extended_by_offdelay_d, core_busy_extended_by_offdelay_q;
reg        core_busy_filtered_by_ondelay_redge_d1;
reg        core_busy_filtered_by_ondelay_d,core_busy_filtered_by_ondelay_q;

reg        n0scan_active;
`ifdef GC__PIPE_MGCG_GATER__1
reg        n0scan_active_d1;
reg        mgls;
wire       mgls_pre;
`endif
`ifdef GC__CGCG_MGCG_MERGE__1
reg        n0scan_cgcg_en_d1;
`endif

wire       core_busy_extended_by_offdelay_redge; 

//
//leaf (always on) clock domain logic
//
wire       combined_core_busy = ((core_busy || soft_override) &&    // core_busy || local override core_busy
                                 (~stall || soft_stall_override));    // stall core_busy || local override stall

wire       core_busy_d1_redge = ~core_busy_d2 && core_busy_d1;  //rising edge of busy
wire [3:0] on_counter_nxt = core_busy_extended_by_offdelay_redge ?  soft_on_delay :    //rising edge of busy. to turn on quickly, using busy w/o registering.
          ( on_counter == 4'd0 )           ?  on_counter : (on_counter - 4'd1);

wire       core_busy_d1_fedge = core_busy_d2 && ~core_busy_d1;  //falling edge of busy
wire [7:0] off_counter_nxt= core_busy_d1_redge ? 8'd0 :         //rising edge of busy, clear off_delay counter 
                            (core_busy_d1_fedge ?  soft_off_delay :  //falling edge of busy, 
                             ( off_counter == 8'd0 )          ?  off_counter : (off_counter - 8'd1));

// extend core_busy by off_delay
always @* begin
  core_busy_extended_by_offdelay_d = core_busy_extended_by_offdelay_q;

  if(core_busy_d1_fedge && (off_counter_nxt != 8'd0))
    core_busy_extended_by_offdelay_d = 1'b1;
  else if(off_counter_nxt == 8'd0)
    core_busy_extended_by_offdelay_d = 1'b0;
end

wire       core_busy_extended_by_offdelay = core_busy_d1 | core_busy_extended_by_offdelay_d;

assign     core_busy_extended_by_offdelay_redge = ~(core_busy_d2 | core_busy_extended_by_offdelay_q) && (core_busy_d1 | core_busy_extended_by_offdelay_d); //rising edge

// filter core_busy_extended_by_offdelay by on delay, 
always @* begin
  core_busy_filtered_by_ondelay_d = core_busy_filtered_by_ondelay_q;
  if(core_busy_extended_by_offdelay && (on_counter_nxt == 4'd0)) //note: off_delay >= on_delay
    core_busy_filtered_by_ondelay_d = 1'b1;
  else if(core_busy_extended_by_offdelay == 1'b0)
    core_busy_filtered_by_ondelay_d = 1'b0;
end


// extend core_busy_extended_by_offdelay by one cycle pulse
wire       core_busy_filtered_by_ondelay_redge = ~core_busy_filtered_by_ondelay_q && core_busy_filtered_by_ondelay_d;  //rising edge
wire       core_busy_extended_by_one_pulse = core_busy_filtered_by_ondelay_d | core_busy_filtered_by_ondelay_redge_d1;

`ifdef GC__PIPE_MGCG_GATER__1
wire                oclk_vld_pre;
reg                 oclk_vld;
`else
wire                oclk_vld;
`endif

always @(posedge static_oclk)
begin
  if(~resetb) begin
    //ON_DELAY     <= 4'd0;
    //OFF_DELAY    <= 8'd0;

    //SOFT_CORE_OVERRIDE <= 1'b1;
    //SOFT_STALLCLK_OVERRIDE <= 1'b1;

    on_counter   <= 4'd0;
    off_counter  <= 8'd0;


    core_busy_d1 <= 1'b1;
    core_busy_d2 <= 1'b1;
    core_busy_filtered    <= 1'b1; 
    core_busy_filtered_d1 <= 1'b1;

    core_busy_extended_by_offdelay_q       <= 1'b1;
    core_busy_filtered_by_ondelay_q        <= 1'b1;
    core_busy_filtered_by_ondelay_redge_d1 <= 1'b0;

`ifdef GC__PIPE_MGCG_GATER__1
    mgls      <= 1'b0;
`endif

  end else begin

    core_busy_d1 <= combined_core_busy;
    core_busy_d2 <= core_busy_d1;

    if(on_counter != on_counter_nxt) begin
      on_counter   <= on_counter_nxt;
    end
    if(off_counter != off_counter_nxt) begin
      off_counter  <= off_counter_nxt;
    end

    core_busy_filtered    <= core_busy_extended_by_one_pulse; 
    core_busy_filtered_d1 <= core_busy_filtered;

    core_busy_extended_by_offdelay_q       <= core_busy_extended_by_offdelay_d;
    core_busy_filtered_by_ondelay_q        <= core_busy_filtered_by_ondelay_d;
    core_busy_filtered_by_ondelay_redge_d1 <= core_busy_filtered_by_ondelay_redge;

`ifdef GC__PIPE_MGCG_GATER__1
    mgls      <= mgls_pre;
`endif
  end
end

`ifdef GC__CGCG_MGCG_MERGE__1
always @(posedge iclk) begin
  if(!resetb) begin
    oclk_vld    <= 1'b1;
  end
  else begin
    oclk_vld    <= oclk_vld_pre;
  end
end

always @(posedge iclk) begin
  n0scan_active <= n0scan_cgcg_en_d1 && (core_busy_extended_by_one_pulse | cg_override);
`ifdef GC__PIPE_MGCG_GATER__1
  n0scan_active_d1   <= (n0scan_active & ~dft_clk_gate_ctrl[1]) | dft_clk_gate_ctrl[0];
`endif
end

`endif

parameter deassert_clk_vld_early = 0;
//
//clock gates
//

`ifdef GC__CGCG_MGCG_MERGE__1

always @(posedge iclk) begin
  n0scan_cgcg_en_d1 <= cgcg_en;
end

`ifdef GC__PIPE_MGCG_GATER__1
generate
  if(deassert_clk_vld_early == 1)
      assign oclk_vld_pre = n0scan_cgcg_en_d1 && ((core_busy_filtered_d1 && core_busy_filtered && core_busy_extended_by_one_pulse) | cg_override);
  else
      assign oclk_vld_pre = n0scan_cgcg_en_d1 && ((core_busy_filtered_d1 && core_busy_filtered) | cg_override);
endgenerate
`else
      assign oclk_vld = n0scan_cgcg_en_d1 && (core_busy_filtered_d1 && core_busy_filtered);
`endif

`else
//assign active = core_busy_filtered | cg_override;
`ifdef GC__PIPE_MGCG_GATER__1
generate
  if(deassert_clk_vld_early == 1)
assign oclk_vld_pre = core_busy_filtered_d1 && core_busy_filtered && core_busy_extended_by_one_pulse;
  else
assign oclk_vld_pre = core_busy_filtered_d1 && core_busy_filtered;
endgenerate
`else
assign oclk_vld = core_busy_filtered_d1 && core_busy_filtered;
`endif
`endif

//
//mgls
//
`ifdef GC__PIPE_MGCG_GATER__1
assign mgls_pre = ~((core_busy_extended_by_one_pulse | core_busy_filtered_d1) || ls_override); //natually has 2 cycle margin between MGLS and dyn_oclk
`else
wire mgls = ~((core_busy_extended_by_one_pulse | core_busy_filtered_d1) || ls_override); //natually has 2 cycle margin between MGLS and dyn_oclk
`endif

`ifdef SIMULATION
assign #2 dyn_oclk_vld = oclk_vld;
`else
assign dyn_oclk_vld = oclk_vld;
`endif

wire dyn_oclk_int;
wire dyn_oclk_gate;

`ifdef SIMULATION_WITH_x
`ifdef GC__PIPE_MGCG_GATER__1
assign dyn_oclk_int =  n0scan_active_d1 ? dyn_oclk_gate : 1'bx;
`else
assign dyn_oclk_int =  active ? dyn_oclk_gate : 1'bx;
`endif
`else
assign dyn_oclk_int = dyn_oclk_gate;
`endif

  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(dft_shift_clk),
   .DFT_TE(dft_te_wire)
  );

ati_clock_gate u_dyn_oclk(
    .clk_out(dyn_oclk_gate),
    .clk_src(iclk),
`ifdef GC__PIPE_MGCG_GATER__1
    .clk_en(n0scan_active_d1),
`else
    .clk_en(active),
`endif
    .TE(dft_te_wire)
    );

// following code is added to instance the clock buffers and mark them for scan grouping

wire [`SCAN_GRP8:0] clks;

parameter scan_grp_dyn_oclk = `SCAN_GRP0; //defaulted

generate
  if(scan_grp_dyn_oclk == `SCAN_GRP0)
    begin
      aticlk_buf scan_grp0_clk(.Y(clks[`SCAN_GRP0]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP1)
    begin
      aticlk_buf scan_grp1_clk(.Y(clks[`SCAN_GRP1]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP2)
    begin
      aticlk_buf scan_grp2_clk(.Y(clks[`SCAN_GRP2]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP3)
    begin
      aticlk_buf scan_grp3_clk(.Y(clks[`SCAN_GRP3]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP4)
    begin
      aticlk_buf scan_grp4_clk(.Y(clks[`SCAN_GRP4]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP5)
    begin
      aticlk_buf scan_grp5_clk(.Y(clks[`SCAN_GRP5]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP6)
    begin
      aticlk_buf scan_grp6_clk(.Y(clks[`SCAN_GRP6]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP7)
    begin
      aticlk_buf scan_grp7_clk(.Y(clks[`SCAN_GRP7]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP8)
    begin
      aticlk_buf scan_grp8_clk(.Y(clks[`SCAN_GRP8]), .A(dyn_oclk_int));
    end
endgenerate

assign dyn_oclk = clks[scan_grp_dyn_oclk];

`ifndef GFX_CG_MONITOR_DISABLE
`ifdef SIMULATION

gfx_cgtt_local_dcg_cov ucgtt_local_dcg_cov
(
    .resetb                (resetb),
    .iclk                  (iclk),
    .cgcg_en               (n0scan_cgcg_en_d1),
    .cg_override           (cg_override),
    .core_busy             (combined_core_busy),
    .soft_override         (soft_override),
    .soft_stall_override   (soft_stall_override),
    .static_oclk           (static_oclk),
    .dyn_oclk              (dyn_oclk),
    .dyn_oclk_vld          (dyn_oclk_vld),
	.soft_off_delay        (soft_off_delay),
	.soft_on_delay		   (soft_on_delay),
    .mgls                  (mgls)
);
`endif
`endif //  `ifdef GFX_CG_MONITOR_DISABLE

endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////




