//////////////////////////////////////////////////////////////////////////////////////////////////////////
//Description: Local (aka medium-grain) clock gating module. It generates three clocks: static_oclk, dyn_oclk, reg_oclk.
//             static_oclk is same as incoming (iclk) clock & is expected to drive your (flow control) interface logic.
//         dyn_oclk is gated based on your 'busy' signal. reg_oclk is to drive your xRBM registers. reg_oclk is enabled
//         by enable signal coming from xRBM block.
//////////////////////////////////////////////////////////////////////////////////////////////////////////
`timescale 1ps/1ps
// following defines are added to instance the clock buffers and mark them for scan grouping

`define SCAN_GRP0 0
`define SCAN_GRP1 1
`define SCAN_GRP2 2
`define SCAN_GRP3 3
`define SCAN_GRP4 4
`define SCAN_GRP5 5
`define SCAN_GRP6 6
`define SCAN_GRP7 7
`define SCAN_GRP8 8

`include "gc_features.vh"

module gfx_cgtt_local_dcg_lite(/*AUTOARG*/
   // Outputs
   dyn_oclk, dyn_oclk_vld, mgls,
   // Inputs
   dft_clk_gate_ctrl, dft_shift_clk,
   resetb, iclk, cgcg_en, cg_override, core_busy,
   soft_override, static_oclk, ls_override, soft_stall_override, stall
   );
//////////////////////////
//in/outs
//////////////////////////
 input       resetb;
 input       iclk;
 input       cg_override;
 input       cgcg_en;
 input       core_busy;
 input        soft_override;
 input       static_oclk;
 input       ls_override;
 input       soft_stall_override;
 input       stall;
 input  [1:0] dft_clk_gate_ctrl;
 input        dft_shift_clk;



 output   mgls;
 output		dyn_oclk;
 output		dyn_oclk_vld;
//////////////////////////

reg  core_busy_d1;
wire active;

wire       mgls = 1'b0;
wire       combined_core_busy = ((core_busy || soft_override) &&    // core_busy || local override core_busy
                                 (~stall || soft_stall_override));    // stall core_busy || local override stall


reg n0scan_cgcg_en_d1;
reg n0scan_active_d1;

reg oclk_vld;

always @(posedge static_oclk)
begin
  if(~resetb) begin
    core_busy_d1 <= 1'b1;
  end else begin
    core_busy_d1 <= combined_core_busy;
  end
end

always @(posedge iclk) begin
  if(!resetb) begin
    oclk_vld    <= 1'b1;
  end
  else begin
    oclk_vld    <= n0scan_cgcg_en_d1 && (core_busy_d1 | cg_override);
  end
end

assign active = n0scan_cgcg_en_d1 && (combined_core_busy | core_busy_d1 | cg_override);
always @(posedge iclk) begin
  n0scan_active_d1   <= (active & ~dft_clk_gate_ctrl[1]) | dft_clk_gate_ctrl[0];
end

always @(posedge iclk) begin
  n0scan_cgcg_en_d1 <= cgcg_en;
end

parameter deassert_clk_vld_early = 0;

`ifdef SIMULATION
assign #2 dyn_oclk_vld = oclk_vld;
`else
assign dyn_oclk_vld = oclk_vld;
`endif

wire dyn_oclk_int;
wire dyn_oclk_gate;

`ifdef SIMULATION_WITH_x
assign dyn_oclk_int =  n0scan_active_d1 ? dyn_oclk_gate : 1'bx;
`else
assign dyn_oclk_int = dyn_oclk_gate;
`endif

  wire dft_te_wire;
ati_TE_driver u_TE_driver
  (
   .TEST_CLK_EN(dft_shift_clk),
   .DFT_TE(dft_te_wire)
  );

ati_clock_gate u_dyn_oclk(
    .clk_out(dyn_oclk_gate),
    .clk_src(iclk),
`ifdef GC__PIPE_MGCG_GATER__1
    .clk_en(n0scan_active_d1),
`else
    .clk_en(active),
`endif
    .TE(dft_te_wire)
    );

// following code is added to instance the clock buffers and mark them for scan grouping

wire [`SCAN_GRP8:0] clks;

parameter scan_grp_dyn_oclk = `SCAN_GRP0; //defaulted

generate
  if(scan_grp_dyn_oclk == `SCAN_GRP0)
    begin
      aticlk_buf scan_grp0_clk(.Y(clks[`SCAN_GRP0]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP1)
    begin
      aticlk_buf scan_grp1_clk(.Y(clks[`SCAN_GRP1]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP2)
    begin
      aticlk_buf scan_grp2_clk(.Y(clks[`SCAN_GRP2]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP3)
    begin
      aticlk_buf scan_grp3_clk(.Y(clks[`SCAN_GRP3]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP4)
    begin
      aticlk_buf scan_grp4_clk(.Y(clks[`SCAN_GRP4]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP5)
    begin
      aticlk_buf scan_grp5_clk(.Y(clks[`SCAN_GRP5]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP6)
    begin
      aticlk_buf scan_grp6_clk(.Y(clks[`SCAN_GRP6]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP7)
    begin
      aticlk_buf scan_grp7_clk(.Y(clks[`SCAN_GRP7]), .A(dyn_oclk_int));
    end
  else if(scan_grp_dyn_oclk == `SCAN_GRP8)
    begin
      aticlk_buf scan_grp8_clk(.Y(clks[`SCAN_GRP8]), .A(dyn_oclk_int));
    end
endgenerate

assign dyn_oclk = clks[scan_grp_dyn_oclk];

//`ifndef GFX_CG_MONITOR_DISABLE
//`ifdef SIMULATION
//
//gfx_cgtt_local_dcg_cov ucgtt_local_dcg_cov
//(
//    .resetb                (resetb),
//    .iclk                  (iclk),
//    .cgcg_en               (n0scan_cgcg_en_d1),
//    .cg_override           (cg_override),
//    .core_busy             (combined_core_busy),
//    .soft_override         (soft_override),
//    .static_oclk           (static_oclk),
//    .dyn_oclk              (dyn_oclk),
//    .dyn_oclk_vld          (dyn_oclk_vld),
//	.soft_off_delay        (soft_off_delay),
//	.soft_on_delay		   (soft_on_delay),
//    .mgls                  (mgls)
//);
//`endif
//`endif //  `ifdef GFX_CG_MONITOR_DISABLE

endmodule
//////////////////////////////////////////////////////////////////////////////////////////////////////////




