// Used in dbgu_gfx. This module is used by the DRB subcomponent in dbgu_gfx in 
//  the buffer override scheme. It overrides the default flop array with whatever 
//  gfx chooses to implement.
//
// It works in conjunction with the following defines: 
//   - GFX_DRB_IP_OVERRIDE_BUFFER: When defined the default flop array in the DRB IP delivery is ifdef'd out, and the following module must be defined by gfx9.
//   - GFX_DRB_IP_MISC_IO_PORTS_MACRO: Used to pass ports down to the buffer module
//   - GFX_DRB_IP_MISC_IO_INST_MACRO: Used to connect ports that are passed down to the buffer module.

`include "gc_features.vh"
`include "mem_features.vh"
`ifdef SIMULATION
 `include "msgio_defines.vh"
`endif

module GFX_DRB_IP_BUFFER #(parameter
  METADATA_W = 9,
  DATA_W = 64,
  DEPTH = 1024,
  LOG2DEPTH = 10
)(
  // {{{ IO Ports
  // ------ Test,Scan and Clock --------
  input             CLK,                    
  input             SC1,                   
  input             SC2,                   
  input             IReset_resetb,                
  input             ScanShiftEn,
  input             CGater01_CG0,

  input             DRBIP_MemSd,

  input   [41:0]    mem_power_ctrl,
  
  //`GFX_DRB_IP_MISC_IO_PORTS_MACRO{{{
  input MiscBufferPassthrus_ZZZ_PGMEM_ZZZ, 
  output wire MiscBufferPassthrus_XXX_PGMEM_XXX,
  //}}}

  input                        Raw_WrEn,
  input        [LOG2DEPTH-1:0] Raw_WrIndex,
  input       [METADATA_W-1:0] Raw_WrMetadata,
  input           [DATA_W-1:0] Raw_WrData,
                                              
  input                        Raw_RdEn,
  input        [LOG2DEPTH-1:0] Raw_RdIndex,
  output wire [METADATA_W-1:0] Raw_RdMetadata_D2,
  output wire     [DATA_W-1:0] Raw_RdData_D2
  // }}}

);

  // -- These will never be used by gfx ------------
  //! unused SC1
  //! unused SC2
  //! unused ScanShiftEn
  //! unused CGater01_CG0
  // -----------------------------------------------

  reg [`MEM__MEM_POWER_CTRL-1:0] MEM_POWER_CTRL_INT;

  assign MiscBufferPassthrus_XXX_PGMEM_XXX = 1'b0;
  
  //locally tied to 0, the debug unit memeory needs to function when all other memories are in deep sleep or shut down
  always @(*) begin
        MEM_POWER_CTRL_INT = {`MEM__MEM_POWER_CTRL{1'b0}};
        MEM_POWER_CTRL_INT[`MEM__MEM_POWER_CTRL_SD] = DRBIP_MemSd;
        MEM_POWER_CTRL_INT[`MEM__MEM_POWER_CTRL_PDP_BC1] = 1'b1;
        MEM_POWER_CTRL_INT[`MEM__MEM_POWER_CTRL_HD_BC1] = 1'b1;
  end


`ifdef GC__DBGU_GFX_RAM_DEPTH__1024
        dbgu_mem2p1024x73qg dbgu_gfx_drb_buffer (
            .MEM_POWER_CTRL (MEM_POWER_CTRL_INT),
            .CLKA           (CLK),
            .RSTA           (!IReset_resetb),
            .DA             ({Raw_WrData, Raw_WrMetadata}),
            .ADRA           (Raw_WrIndex),
            .MEA            (Raw_WrEn),
            .WEA            (Raw_WrEn),
            .QB             ({Raw_RdData_D2, Raw_RdMetadata_D2}),
            .ADRB           (Raw_RdIndex),
            .MEB            (Raw_RdEn),
            .PIPEMEB        (Raw_RdEn),
            .CLKB           (CLK),
            .RSTB           (!IReset_resetb));
`elsif GC__DBGU_GFX_RAM_DEPTH__512
`else
        //Error
        `ifdef SIMULATION
         VCS coverage off
           initial
           begin
              `MSGIO_DISPLAY("Error: illegal parameter settings in gfx_drb_ip_buffer.v");
              #100 (2);
           end
         VCS coverage on
        `endif
        assign Raw_RdMetadata_D2[METADATA_W-1:0] = {METADATA_W{1'b0}};
        assign Raw_RdData_D2    [DATA_W    -1:0] = {DATA_W    {1'b0}};
`endif

endmodule
