/*
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 *  NOTE: This *must* be instanced in the always-on power domain and attached to the static clk
 *
 *  Common module for DSM (Debug State Machine) irritator bus
 * 
 *  Each block has some number (M) of block irritators that can be driven by any of the irritator 
 *  input bits.
 * 
 *  Associated with each block, there is a N-bit (N=3*M) control register that controls whether a 
 *  given block irritator is enabled. In each 3 bit group, the lower two bits are the same as the 
 *  non-sticky irritator. The third bit enables sticky mode.
 * 
 *  Block irritators are used to affect functionality to aid in validation and debug.  Example uses 
 *  include, but are not limited to, fifo back-pressure, forcing EDC errors, and wavefront stalling.
 * 
 * This sticky irritator is set by the rising edge of the local irritator
 * and stays high until reset using the event_occured signal.
 * 
 * If blk_enable_sticky_mode == 0, then this functions as a normal irritator.
 * 
 */

// Include gc_defs.vh to get DSM_TARG_IRRITATOR_DATA defines
`include "gc_defs.vh"

module gfx_dsm_sticky_irritator (
    clk,
    reset,
    DSM_BLK_irritator_data,
    blk_dsm_cntl,
    event_occured,
    BLK_TARG_irritator_data,
    blk_irritator
);

   parameter                               BLK_IRRITATORS = 1;
   parameter                               BLK_DSM_CNTL_WIDTH = BLK_IRRITATORS * 3;
   
   input                                   clk;
   input                                   reset;
   input [`DSM_TARG_IRRITATOR_DATA_RANGE]  DSM_BLK_irritator_data;  // Top-level irritator chain input
   input [BLK_DSM_CNTL_WIDTH-1:0]          blk_dsm_cntl;            // Block-specific DSM irritator control from register bus (e.g. GRBM)
   input [BLK_IRRITATORS-1:0]              event_occured;           // the irritation event has occured, reset the sticky state

   output [`DSM_TARG_IRRITATOR_DATA_RANGE] BLK_TARG_irritator_data; // Top-level irritator chain output
   output [BLK_IRRITATORS-1:0]             blk_irritator;           // Local block irritator signal, enabled when corresponding control bit set

   reg [`DSM_TARG_IRRITATOR_DATA_RANGE]    BLK_TARG_irritator_data;

   wire [BLK_IRRITATORS-1:0]               unsticky_blk_irritator;

   integer i, k, m;

   reg  [(BLK_IRRITATORS * `DSM_TARG_IRRITATOR_DATA_WIDTH)-1:0] local_dsm_cntl;
   reg  [BLK_IRRITATORS-1:0]                                    local_blk_enable_sticky_mode;
   always @ (*) begin
     for (i = 0; i < BLK_IRRITATORS; i=i+1) begin
       local_dsm_cntl[i*2]             = blk_dsm_cntl[i*3];
       local_dsm_cntl[i*2+1]           = blk_dsm_cntl[i*3+1];
       local_blk_enable_sticky_mode[i] = blk_dsm_cntl[i*3+2];
     end
   end 

   gfx_dsm_irritator #(.BLK_IRRITATORS(BLK_IRRITATORS)) irritator(
      .clk(clk),
      .DSM_BLK_irritator_data(DSM_BLK_irritator_data),
      .blk_dsm_cntl(local_dsm_cntl),
      .BLK_TARG_irritator_data(BLK_TARG_irritator_data),
      .blk_irritator(unsticky_blk_irritator)
   );


   reg  [BLK_IRRITATORS-1:0] blk_irritator;
   reg  [BLK_IRRITATORS-1:0] unsticky_blk_irritator_delay;
   reg  [BLK_IRRITATORS-1:0] sticky_state;
   reg  [BLK_IRRITATORS-1:0] irritator_rise_edge;
   reg  [BLK_IRRITATORS-1:0] next_sticky_state;



   always @ (*) begin
     for (k = 0; k < BLK_IRRITATORS; k=k+1) begin
       irritator_rise_edge[k] = unsticky_blk_irritator[k] & ~unsticky_blk_irritator_delay[k];
       next_sticky_state[k] = (sticky_state[k] & ~event_occured[k]) | (local_blk_enable_sticky_mode[k] & irritator_rise_edge[k]);
       blk_irritator[k] = local_blk_enable_sticky_mode[k] ? sticky_state[k] : unsticky_blk_irritator[k];
     end
   end

   always @ (posedge clk) begin
     for (m = 0; m < BLK_IRRITATORS; m=m+1) begin
       if (reset | ~local_blk_enable_sticky_mode[m]) begin
         sticky_state[m] <= 1'b0;
       end else begin
         sticky_state[m] <= next_sticky_state[m];
       end 
       unsticky_blk_irritator_delay[m] <= unsticky_blk_irritator[m];
     end
   end

endmodule // gfx_dsm_sticky_irritator
