//--------------------------------------------------------------------------
//
// $Id: $
//
// $Change: $
//
// Notes:     This module contains a register based FIFO that uses a circular FIFO
//            to store (FIFO_DEPTH-1) entries and another register to store the 'top'
//            entry. This top entry is guaranteed to always have the oldest data
//            entry. FIFO_WIDTH, FIFO_DEPTH, SKID_WORDS, and SKID_FULL are the user
//            defined parameters. SKID_WORDS defines the number of SKID entries to have.
//            SKID_FULL=1 will make 'full' de-assert if (SKID entries are being used 
//            AND 're' is asserted). 
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2004, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
// Author: Brian Rerecich
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

module gfx_new_ati_fifo_prog_depth_reg_out (// Outputs
                             full,
                             empty,
                             read_data,
                             
                             // Inputs
                             we,
                             re,
                             write_data,
                             
                             clk,
                             reset,

                             prog_depth
                             );
   
   //**************
   //* Parameters *
   //**************
   parameter FIFO_WIDTH = 1;
   parameter FIFO_DEPTH = 2;
   parameter SKID_WORDS = 0;
   parameter SKID_FULL  = 0;
   parameter FIFO_NAME  = "";
   parameter ADDR_WIDTH = ((FIFO_DEPTH <= 4) ? 2 :
                           (FIFO_DEPTH <= 8) ? 3 :
                           (FIFO_DEPTH <= 16) ? 4 :
                           (FIFO_DEPTH <= 32) ? 5 :
                           (FIFO_DEPTH <= 64) ? 6 :
                           (FIFO_DEPTH <= 128) ? 7 :
                           (FIFO_DEPTH <= 256) ? 8 :
                           (FIFO_DEPTH <= 512) ? 9 :
                           (FIFO_DEPTH <= 1024) ? 10 :
                           (FIFO_DEPTH <= 2048) ? 11 : 12);  // Bigger than 4095???  i hope not.
`ifdef SIMULATION
    parameter CYCLES_BEFORE_OVERFLOW_CHECK = 40;
    integer    cycle_count;
`endif
   
   //***********************
   //* Output Declarations *
   //***********************
   output [FIFO_WIDTH-1:0] read_data;     // FIFO Output Rd Data
   output                  full;          // FIFO Full signal
   output                  empty;         // FIFO Empty signal
   
   //**********************
   //* Input Declarations *
   //**********************
   input [FIFO_WIDTH-1:0]  write_data;    // FIFO Input Wr Data
   input                   we;            // Write Enable
   input                   re;            // Read Enable
   input                   clk;            
   input                   reset;
   
   input [ADDR_WIDTH:0]    prog_depth;    // Programmable fifo depth

   //***************************
   //* Output REG Declarations *
   //***************************
   reg                     full;
   reg                     full_d;
   reg                     empty;
   reg                     empty_d;

   //*****************************
   //* Internal REG Declarations *
   //*****************************   
   reg [FIFO_WIDTH-1:0]    fifo_q[FIFO_DEPTH-2:0];  // Actual FIFO storage (FFs) except top
   reg [FIFO_WIDTH-1:0]    reg_out_q;               // Top Entry of FIFO   
   reg                     reg_out_valid_q;         // The contents of the reg_out FF are valid
   reg [ADDR_WIDTH-1:0]    read_ptr_q;              // Read Pointer
   reg [ADDR_WIDTH-1:0]    write_ptr_q;             // Write Pointer
   reg                     int_fifo_empty_q;        // Circular FIFO empty (not including top entry - reg_out_q)
   reg [ADDR_WIDTH:0]      cnt_q;                   // Number of used entries in circular FIFO (not including the reg_out)
   reg                     fifo_we;                 // Write Enable (abstractly for RAM + output reg)
   reg                     fifo_re;                 // Read Enable (abstractly for RAM + output reg)
   reg                     outreg_wrdata_ld;        // Load the output register with write data.
      
   reg [FIFO_WIDTH-1:0]    fifo_d[FIFO_DEPTH-2:0];
   reg [FIFO_WIDTH-1:0]    reg_out_d;               
   reg                     reg_out_valid_d;
   reg [ADDR_WIDTH-1:0]    read_ptr_d;            
   reg [ADDR_WIDTH-1:0]    write_ptr_d;
   reg                     int_fifo_empty_d;             
   reg [ADDR_WIDTH:0]      cnt_d;
   
   reg                     read_fifo;               // Read from circular FIFO (only for read_ptr logic)
   reg                     write_fifo;              // Write to circular FIFO (only for write_ptr logic)
   reg                     cnt_gt_skid_limit;       // cnt_q is in ANY of the "SKID" area of the FIFO
   reg                     cnt_eq_skid_limit;       // cnt_q is EXACTLY starting to enter "SKID" area
   reg                     read_last;               // read_ptr -> last entry in FIFO
   reg                     write_last;              // write_ptr -> last entry in FIFO
   
   integer                 i;

   //*********************
   //* Wire Declarations *
   //*********************
   wire [FIFO_WIDTH-1:0]   read_data;               // Data Output of FIFO 
   wire [FIFO_WIDTH-1:0]   fifo_out;                // Top entry in sub-fifo (the circular part)            
   
   //***********************
   //* Combinatorial Logic *
   //***********************   

   // Assign read_data to the top of the FIFO
   assign read_data = reg_out_q;
   assign fifo_out  = fifo_q[read_ptr_q[ADDR_WIDTH-1:0]];

   always @ (/*AUTOSENSE*/fifo_out or int_fifo_empty_q or re
             or reg_out_q or reg_out_valid_q or we or write_data) begin
      
      // Logic for reg_out_valid, fifo_re, and fifo_we
      reg_out_valid_d = we |                           // Writing to FIFO                      OR
                        ( re & ~int_fifo_empty_q) |    // Reading and RAM not empty            OR
                        (~re &  reg_out_valid_q);      // Not Reading and prev. had valid data
      
      fifo_we         = we & reg_out_valid_q &         // Writing, output reg is already valid AND
                        ~(re & int_fifo_empty_q);      // not (reading with an empty RAM)
      
      fifo_re         = re & ~int_fifo_empty_q;        // reading and the RAM fifo isn't empty.
   
      // Logic for reg_out_q
      outreg_wrdata_ld = (we & ~reg_out_valid_q) |     // Writing and nothing valid in reg_out    OR
                         (we & re & int_fifo_empty_q); // Writing, Reading, and the FIFO is empty
      
      case(1'b1)
        outreg_wrdata_ld: reg_out_d = write_data;
        fifo_re:          reg_out_d = fifo_out;
        default:          reg_out_d = reg_out_q;
      endcase // casez(1'b1)
   end // always @ (...

   always @ (/*AUTOSENSE*/fifo_re or fifo_we or read_ptr_q
             or write_ptr_q) begin

      // Write Pointer logic
      write_last = (write_ptr_q == ((FIFO_DEPTH-1) - 1));
      casez({fifo_we, write_last})
        2'b0?: write_ptr_d = write_ptr_q;
        2'b10: write_ptr_d = write_ptr_q + 1;
        2'b11: write_ptr_d = 0;
      endcase // casez({we,write_last})
      
      // Read Pointer logic
      read_last = (read_ptr_q == ((FIFO_DEPTH-1) - 1));
      casez({fifo_re,read_last})
        2'b0?: read_ptr_d = read_ptr_q;
        2'b10: read_ptr_d = read_ptr_q + 1;
        2'b11: read_ptr_d = 0;
      endcase // casez({fifo_re,read_last})
      //       
   end // always @ (...
   
   always_comb begin
        
      casez({fifo_we, fifo_re})
        // both or none, do the same
        2'b00, 
        2'b11: begin
           int_fifo_empty_d = int_fifo_empty_q;
           cnt_d            = cnt_q;
        end
        // read, no write
        2'b01: begin
           int_fifo_empty_d = (cnt_q == 1);
           cnt_d            = cnt_q - 1;
        end
        // write, no read
        2'b10: begin
           int_fifo_empty_d = 0;
           cnt_d            = cnt_q + 1;
        end     
      endcase // casez({fifo_we, fifo_re}) 

      // Logic for Internal Circular FIFO (Empty,Count) and External (Full, Empty)
      
      // Internal Circular FIFO non-empty location count and empty signal - 
      // NOTE: cnt_q is the number of used locations in the internal circular FIFO - this equals FIFO_DEPTH-1
  
      cnt_gt_skid_limit = (cnt_q >  ((prog_depth-1) - SKID_WORDS - 1));
      cnt_eq_skid_limit = (cnt_q == ((prog_depth-1) - SKID_WORDS - 1));

      // Logic for FULL is based on the different SKID parameters

      // Toggle FULL once in SKID territory if an entry is read
      if (SKID_FULL)
        full_d = (cnt_eq_skid_limit & ~fifo_re & fifo_we) |  // hit limit, writing, and not reading
                 (cnt_gt_skid_limit & ~fifo_re);             // over limit and not reading. 

      // Keep FULL asserted once FIFO has entered the SKID entries
      else if (SKID_WORDS > 0)
        full_d = (cnt_eq_skid_limit & ~fifo_re & fifo_we) |  // hit limit, writing, and not reading
                 cnt_gt_skid_limit;                          // over limit
      
      // No SKID entries in FIFO
      else
        full_d = (prog_depth==1) ? reg_out_valid_d :
                 ((cnt_eq_skid_limit & ~fifo_re & fifo_we) |  // hit limit, writing, and not reading
                  (cnt_d > (prog_depth - SKID_WORDS - 2)) |
                  (full & ~fifo_we & ~fifo_re & (cnt_d > (prog_depth - SKID_WORDS - 2))) | // store value (no read, no write)
                  (full &  fifo_we &  fifo_re));               // store value (read, write)
      
      empty_d = ~reg_out_valid_d;
      
   end // always @ (...
   
   //******************
   //* FFs with reset *
   //******************
   always @ (posedge clk) begin
      if (reset) begin
         write_ptr_q      <= 0;
         read_ptr_q       <= 0;
         cnt_q            <= 0;
         full             <= 0;
         empty            <= 1;
         int_fifo_empty_q <= 1;
         reg_out_valid_q  <= 0;
      end
      else begin
        if (fifo_we) begin
            write_ptr_q      <= write_ptr_d;
         end
         if (fifo_re) begin
            read_ptr_q       <= read_ptr_d;
         end
         if (fifo_we | fifo_re) begin
            cnt_q            <= cnt_d;
            int_fifo_empty_q <= int_fifo_empty_d;
         end
         reg_out_valid_q  <= reg_out_valid_d;
         empty            <= empty_d;
         full             <= full_d;
      end // else begin
   end //@always
   
   //*********************
   //* FFs without reset *
   //*********************
   always @ (posedge clk) begin
      reg_out_q <= reg_out_d;
   end
      
   always @ (posedge clk) begin

      // Logic was put here instead of above with the rest of combo logic
      // because there is no way (that I know of) to put a 2D array (fifo_q)
      // into a sensitivity list when it's depth is a parameter.
      if (fifo_we) begin
         for (i = 0; i < (FIFO_DEPTH-1); i = i + 1) begin
            if (i == write_ptr_q) begin
               fifo_q[i] <= write_data;
            end
         end
      end
      
`ifdef SIMULATION
// VCS coverage off      
      // error checking
      case (1'b1)
        (cycle_count > CYCLES_BEFORE_OVERFLOW_CHECK) &
        ((reset == 1'b0) & ((prog_depth==1) ? reg_out_valid_q : (cnt_q == (prog_depth-1))) & we & ~re) : begin
           // leda DCVER_274 off - Verilog system task is not supported
           `MSGIO_DISPLAY("ATI_Fifo ERROR (%m): %s fifo written when full and cnt_q = (prog_depth-1) -- data dropped at time %t", FIFO_NAME, $time);
           // leda DCVER_274 on
        end
        ((reset == 1'b0) & ((prog_depth==1) ? reg_out_valid_q : (cnt_q >= (prog_depth-1))) & we & ~re) : begin
           // leda DCVER_274 off - Verilog system task is not supported
           `MSGIO_DISPLAY("ATI_Fifo ERROR (%m): %s fifo written when full and cnt_q > or = (prog_depth-1) -- data dropped at time %t", FIFO_NAME, $time);
           // leda DCVER_274 on
        end
      endcase // case(1'b1)
// VCS coverage on
`endif
      
   end
endmodule // fifo
