//--------------------------------------------------------------------------
//
// $Id: $
//
// $Change: $
//
// Notes:     This is a generic design of a register based FIFO. It's size is determned
//            by the paramters: FIFO_WIDTH, FIFO_DEPTH, SKID_WORDS, and SKID_FULL. It's 
//            a circular FIFO with Read and Write pointers that only increment. They 
//            flip around when they are equal to FIFO_DEPTH. To generate Full and Empty, 
//            a counter is used to count the distance between read and write pointers. 
//            SKID_WORDS is the number of SKID entries to use in the FIFO. SKID_FULL=1
//            will make full de-assert if the FIFO is using SKID entries, but 're' is
//            asserted.
//
// Copyright: Trade secret of ATI Technologies, Inc.
//            (C) Copyright 2001-2004, ATI Technologies, Inc., (unpublished) 
//
//        All rights reserved.  This notice is intended as a precaution against
//        inadvertent publication and does not imply publication or any waiver
//        of confidentiality.  The year included in the foregoing notice is the
//        year of creation of the work.
//
// Author: Brian Rerecich
//
// Notes:     This design is based on gfx_new_ati_fifo, using kc floparray to 
//            replace normal floparray when your design is in TSMC 7nm and has defined 
//            TECH__TECH__TSMC7N. You need include global_features.vh to use this macro 
//            if you want to use kc floparray, otherwise it will also use normal floparray.
//
//--------------------------------------------------------------------------
`ifdef SIMULATION
`include "msgio_defines.vh"
`endif

`include "tech_features.vh"

module gfx_new_kc_fifo (// Outputs
		     full,
		     empty,
		     read_data,
		     
		     // Inputs
		     we,
		     re,
		     write_data,
		     
		     clk,
		     reset
		     );
   
   //**************
   //* Parameters *
   //**************
   parameter FIFO_WIDTH = 1;
   parameter FIFO_DEPTH = 2;
   parameter SKID_WORDS = 0;
   parameter SKID_FULL  = 0;
   
   parameter FIFO_NAME  = ""; // spyglass disable W175 -- declared but not used
   parameter ADDR_WIDTH = ((FIFO_DEPTH <= 2) ? 1 :
                           (FIFO_DEPTH <= 4) ? 2 :
                           (FIFO_DEPTH <= 8) ? 3 :
                           (FIFO_DEPTH <= 16) ? 4 :
                           (FIFO_DEPTH <= 32) ? 5 :
                           (FIFO_DEPTH <= 64) ? 6 :
                           (FIFO_DEPTH <= 128) ? 7 :
                           (FIFO_DEPTH <= 256) ? 8 :
                           (FIFO_DEPTH <= 512) ? 9 : 
                           (FIFO_DEPTH <= 1024) ? 10 : 
                           (FIFO_DEPTH <= 2048) ? 11 : 12);  // Bigger than 4095???  i hope not.

			  
`ifdef SIMULATION
    parameter CYCLES_BEFORE_OVERFLOW_CHECK = 40;
    integer    cycle_count;
`endif

// I wish I could use named parameters :(
   parameter ASSERT_FULL_AND_PUSH = 1;
   parameter ASSERT_EMPTY_AND_POP = 1;
   parameter COVER_PUSH_AND_POP = 1;
   parameter COVER_FULL = 1;
   parameter COVER_FULL_EMPTY_FULL = 1;
   parameter ASSERT_EMPTY_EOS = 1;   
   parameter COVER_HALF_FULL = 1;
   parameter COVER_FULL_MINUS_TWO = 1;
   parameter COVER_FULL_MINUS_ONE = 1;
  
   
   //***********************
   //* Output Declarations *
   //***********************
   output [FIFO_WIDTH-1:0] read_data;     // FIFO Output Rd Data
   output 		   full;          // FIFO Full signal
   output 		   empty;         // FIFO Empty signal
   
   //**********************
   //* Input Declarations *
   //**********************
   input [FIFO_WIDTH-1:0]  write_data;    // FIFO Input Wr Data
   input 		   we;            // Write Enable
   input 		   re;            // Read Enable
   input 		   clk;            
   input 		   reset;
   
   //***************************
   //* Output REG Declarations *
   //***************************
   reg 			   full;          // FIFO is full
   reg 			   empty;         // FIFO is empty

   //*****************************
   //* Internal REG Declarations *
   //*****************************
`ifndef TECH__TECH__TSMC7N
   reg [FIFO_WIDTH-1:0]    fifo_q[FIFO_DEPTH-1:0];  // Actual FIFO storage (FFs)
`endif
   reg [ADDR_WIDTH-1:0]    read_ptr_q;              // Read Pointer
   reg [ADDR_WIDTH-1:0]    write_ptr_q;             // Write Pointer
   
//   reg [FIFO_WIDTH-1:0]    fifo_d[FIFO_DEPTH-1:0];
   reg [ADDR_WIDTH-1:0]    read_ptr_d;            
   reg [ADDR_WIDTH-1:0]    write_ptr_d;
   reg 			   full_d;
   reg 			   empty_d;
   
   reg [ADDR_WIDTH:0] 	   cnt_q;                   // Number of entries currently in the FIFO
   reg [ADDR_WIDTH:0] 	   cnt_d;

   reg 			   write_last;              // Write Pointer is at last location in FIFO 
   reg 			   read_last;               // Read Pointer is at last location in FIFO 
   reg 			   cnt_gt_skid_limit;       // FIFO has entries in SKID area.
   reg 			   cnt_eq_skid_limit;       // Next entry written enters the SKID area.

   integer 		   i;
   
   //*********************
   //* Wire Declarations *
   //*********************
   wire [FIFO_WIDTH-1:0]   read_data;
   
   //***********************
   //* Combinatorial Logic *
   //***********************   

   // Assign read_data to the top of the FIFO
`ifndef TECH__TECH__TSMC7N
   assign read_data = fifo_q[read_ptr_q[ADDR_WIDTH-1:0]];
`endif

   always @ (/*AUTOSENSE*/empty or re or read_last or read_ptr_q or we
	     or write_last or write_ptr_q) begin
      
   // Logic for Read and Write Pointers
      read_last = (read_ptr_q == (FIFO_DEPTH-1)); // spyglass disable W362 -- rhs is an integer
      casez({(re & ~empty),read_last})
	2'b0?: read_ptr_d = read_ptr_q;
	2'b10: read_ptr_d = read_ptr_q + 1;
	2'b11: read_ptr_d = 0;
      endcase // casez({re,read_last})

      write_last = (write_ptr_q == (FIFO_DEPTH-1)); // spyglass disable W362 -- rhs is an integer
      casez({we,write_last})
	2'b0?: write_ptr_d = write_ptr_q;
	2'b10: write_ptr_d = write_ptr_q + 1;
	2'b11: write_ptr_d = 0;
      endcase // casez({we,write_last})
   
   end // always @ (...

   always @ (/*AUTOSENSE*/cnt_eq_skid_limit or cnt_gt_skid_limit
	     or cnt_q or empty or full or re or we) begin

      // Logic for Full

      // NOTE: It is assumed that Synopsys DC will optimize out the logic
      //       for cnt_{eq/gt}_skid_limit if it isn't used.
      
      cnt_gt_skid_limit = (cnt_q >  (FIFO_DEPTH - SKID_WORDS - 1)); // spyglass disable W362 -- rhs is an integer
      cnt_eq_skid_limit = (cnt_q == (FIFO_DEPTH - SKID_WORDS - 1)); // spyglass disable W362 -- rhs is an integer

      // Logic for FULL is based on the different SKID parameters

      // Toggle FULL once in SKID territory if an entry is read
      if (SKID_FULL) // spyglass disable W159 W490 -- constant expression
	full_d = (cnt_eq_skid_limit & ~re & we) |  // hit limit, writing, and not reading
		 (cnt_gt_skid_limit & ~re);        // over limit and not reading. 
      
      // Keep FULL asserted once FIFO has entered the SKID entries
      else if (SKID_WORDS > 0) // spyglass disable W159 W490 -- constant expression
	full_d = (cnt_eq_skid_limit & ~re & we) |
		 cnt_gt_skid_limit;
      
      // No SKID entries in FIFO
      else
	full_d = (cnt_eq_skid_limit & ~re & we) |
		 (full & ~we & ~re) |
		 (full &  we &  re);
      
      // Logic for Empty and Count
      
      casez({we, (re & ~empty)})
	// both or none, do the same
	2'b00, 
	2'b11: begin
	   empty_d = empty;
	   cnt_d   = cnt_q;
	end
	// read, no write
	2'b01: begin
	   empty_d = (cnt_q == 1);
	   cnt_d   = cnt_q - 1;
	end
	// write, no read
	2'b10: begin
	   empty_d = 0;
	   cnt_d   = cnt_q + 1;
	end	
      endcase // casez({we, re})
   
   end // always @ (...
   
   //******************
   //* FFs with reset *
   //******************
   always @ (posedge clk) begin
      if (reset) begin
	    read_ptr_q  <= 0;
	    write_ptr_q <= 0;
	    empty       <= 1;
	    full        <= 0;
	    cnt_q       <= 0;
      end
      else begin
        if (re & ~empty) begin
	        read_ptr_q  <= read_ptr_d;
        end

        if (we) begin
	        write_ptr_q <= write_ptr_d;
        end

        if ((re & ~empty) | we) begin
	        empty       <= empty_d;
	        cnt_q       <= cnt_d;
        end
        full        <= full_d;
      end
   end // always @ (posedge clk)
   
    //*********************
   //* FFs without reset *
   //*********************

`ifdef TECH__TECH__TSMC7N
    kc1r1w_flop #(.DEPTH(FIFO_DEPTH), .WIDTH(FIFO_WIDTH)) kc_floparray (
        .ADRA (write_ptr_q),
        .ADRB (read_ptr_q),
        .DA (write_data),
		.WEA (we),
		.CLKA (clk),
		.CLKB (clk),
		.RSTA (reset),
		.RSTB (reset),
		.MEA (we),
		.MEB (1'b1),
		.WEMA (1'b1),
		.PIPEMEB (1'b0),
		.QB (read_data)
        );
`else //`ifdef TECH__TECH__TSMC7N
   always @ (posedge clk) begin

      // Logic was put here instead of above with the rest of combo logic
      // because there is no way (that I know of) to put a 2D array (fifo_q)
      // into a sensitivity list when it's depth is a parameter.
      
      if (we) begin
      	 for (i = 0; i < FIFO_DEPTH; i = i + 1) begin
            if (i == write_ptr_q) begin
               fifo_q[i] <= write_data;
            end
         end
      end
   end
`endif //`ifdef TECH__TECH__TSMC7N
      
`ifdef SIMULATION
    always @ (posedge clk) begin
	// VCS coverage off
      // leda DFT_022 off - Incomplete case statement
      // error checking
      case (1'b1)
        (cycle_count > CYCLES_BEFORE_OVERFLOW_CHECK) &
	((reset == 1'b0) & (cnt_q == FIFO_DEPTH) & we & ~re) : begin
	   // leda DCVER_274 off - Verilog system task is not supported
           `MSGIO_DISPLAY("ATI_Fifo ERROR (%m): fifo written when full -- data dropped at time %t", $time);
	   // leda DCVER_274 on
	end
      endcase // case(1'b1)
      // leda DFT_022 on
	// VCS coverage on
    end

   // VCS coverage off
   // leda  DCVER_192 off - Initial statement not supported
   initial begin
      cycle_count <= 0;
   end
   // leda  DCVER_192 on
   
   always @(posedge clk) begin
      if (cycle_count != 32'hffffffff) begin
	 cycle_count <= cycle_count + 1;
      end
   end


  ati_fifo_cover_assert #(FIFO_DEPTH,
			  ASSERT_FULL_AND_PUSH,
			  ASSERT_EMPTY_AND_POP,
			  COVER_PUSH_AND_POP,
			  COVER_FULL,
			  COVER_FULL_EMPTY_FULL,
			  ASSERT_EMPTY_EOS,
			  COVER_HALF_FULL,
			  COVER_FULL_MINUS_TWO,
			  COVER_FULL_MINUS_ONE
			  ) ati_fifo_cover_assert 
    ( .clk(clk), 
      .resetb(~reset), 
      .push(we), 
      .pop(re) );
   
   // VCS coverage on
`endif

endmodule // gfx_new_kc_fifo



