
/* Store all incoming commands for power down and power up and pass it on through a fifo to the fsm which does the sequencing */

`include "gfx_pgfsm_defines.h"

module gfx_pgfsm_cmd_fifo ( /*autoarg*/
   // Outputs
   isolation_pd, pg_reset_pd, mother_sleep_pd, daughter_sleep_pd,
   mem_mother_shutdown_pd, mem_daughter_shutdown_pd, pg_status_pd, 
   pgfsm_fsm_state, pgfsm_cmd_done,
   // Inputs
   TEST_CLK_EN,
   reset, hard_resetb, refclk, refclk_en, iclk,
   new_cmd, pd_sel, pwrup_cmd, pwrdn_cmd,
   use_extra_delay_regs_en,
   pgfsm_delay0_reg, pgfsm_delay1_reg, pgfsm_delay2_reg, pgfsm_delay3_reg, pgfsm_delay4_reg, pgfsm_delay5_reg,
   pgfsm_fsm_1_reg,  pgfsm_fsm_2_reg,  pgfsm_fsm_3_reg,  pgfsm_fsm_4_reg,  pgfsm_fsm_5_reg,  pgfsm_fsm_6_reg
   );

/**************************************************/
/*               Signal Definition                */
/**************************************************/

   // set for pd
   output                     isolation_pd;
   output                     pg_reset_pd;
   output                     mother_sleep_pd;
   output                     daughter_sleep_pd;
   output                     mem_mother_shutdown_pd;
   output                     mem_daughter_shutdown_pd;
   output [1:0]               pg_status_pd; 
   
   output [4:0]               pgfsm_fsm_state;  
   output                     pgfsm_cmd_done;
 
   // Clock, reset and DFT signals 
   input		      TEST_CLK_EN; 
   input                      reset;
   input                      hard_resetb;
   input                      refclk;
   input		      refclk_en;
   input                      iclk;

   // Command inputs
   input                      new_cmd; 
   input                      pd_sel;
   input                      pwrup_cmd, pwrdn_cmd;

   // Delay register values and FSM register values
   input                      use_extra_delay_regs_en;
   input [23:0]               pgfsm_delay0_reg;
   input [23:0]               pgfsm_delay1_reg;
   input [23:0]               pgfsm_delay2_reg;
   input [23:0]               pgfsm_delay3_reg;
   input [23:0]               pgfsm_delay4_reg;
   input [23:0]               pgfsm_delay5_reg;

   input [23:0]               pgfsm_fsm_1_reg;
   input [23:0]               pgfsm_fsm_2_reg;
   input [23:0]               pgfsm_fsm_3_reg;
   input [23:0]               pgfsm_fsm_4_reg;
   input [23:0]               pgfsm_fsm_5_reg;
   input [23:0]               pgfsm_fsm_6_reg;
   
   //------------------------------------------------------------------------

   reg                        go_pwrdown, go_pwrup; 
   reg                        pd_in_transition;  
   reg                        isolation_pd;
   reg                        pg_reset_pd;
   reg                        mother_sleep_pd;
   reg                        daughter_sleep_pd;
   reg                        mem_mother_shutdown_pd;
   reg                        mem_daughter_shutdown_pd;
   reg [1:0]                  pg_status_pd; 
   reg                        pgfsm_cmd_done;

   // using a fifo of depth 4 for cmds. 
   // When both pd should be powered off/on its considered two commands. 
   reg [1:0]                  wptr, rptr; 
   reg [3:0]                  updown_cmd_fifo; 
   reg                        hard_resetb_ff, pwrup_on_reset;

   //------------------------------------------------------------------------
   // If posedge of hard_resetb is detected, go thru pwrup sequence
   //------------------------------------------------------------------------

   wire  empty =  (wptr == rptr);
   wire  full  =  (wptr == rptr - 2'b1);

   always @(posedge iclk) begin
      if (reset) begin
         hard_resetb_ff       <= 1'b0;
         pwrup_on_reset       <= 1'b0;
      end else begin
         hard_resetb_ff       <= hard_resetb;
         pwrup_on_reset       <= (~hard_resetb & hard_resetb_ff) ? 1'b1 :
                                 (empty & ~new_cmd)              ? 1'b0 : pwrup_on_reset;
      end
   end

  // 4-deep fifo to schedule commands. 
   always @(posedge iclk) begin
      if (reset) begin 
         wptr                   <= 2'b00;
      end else if (new_cmd & pd_sel & (pwrdn_cmd | pwrup_cmd) & ~full) begin
         wptr                   <= wptr + 2'b01;
         updown_cmd_fifo[wptr]  <= pwrup_cmd ? 1'b1 :
                                   pwrdn_cmd ? 1'b0 : 1'b0;
      end else if (empty & pwrup_on_reset) begin // pd power on case
         wptr                   <= wptr + 2'b01;
         updown_cmd_fifo[wptr]  <= 1'b1;
      end
   end

   wire                       isolation;
   wire                       pg_reset;
   wire                       mother_sleep;
   wire                       daughter_sleep;
   wire                       mem_mother_shutdown;
   wire                       mem_daughter_shutdown;
   wire                       fsm_idle; 
   wire [1:0]                 pg_status; 
   wire                       fsm_cmd_done;
      
   // Popping from the fifo
   always @(posedge iclk) begin
      if (reset) begin
         rptr                    <= 2'b0; 
         go_pwrdown              <= 1'b0;
         go_pwrup                <= 1'b0;
         pd_in_transition        <= 1'b0; 
      end else if (pd_in_transition & fsm_cmd_done) begin
         go_pwrdown              <= 1'b0;
         go_pwrup                <= 1'b0;
         pd_in_transition        <= 1'b0; 
      end else if (~empty & fsm_idle & ~pd_in_transition) begin
            case (updown_cmd_fifo[rptr])
              1'b0: // power down pd
                begin
                   rptr                  <= rptr + 2'b1; 
                   if (pg_status_pd == `GFX_PGFSM_BLOCK_POWER_ON) begin // power down only if previously on. 
                      go_pwrdown         <= 1'b1;
                      go_pwrup           <= 1'b0;
                      pd_in_transition   <= 1'b1; 
                   end
                end
              1'b1: // power up pd
                begin
                   rptr                  <= rptr + 2'b1; 
                   if (pg_status_pd == `GFX_PGFSM_BLOCK_POWER_OFF) begin // power up only if previously off. 
                      go_pwrdown         <= 1'b0;
                      go_pwrup           <= 1'b1;
                      pd_in_transition   <= 1'b1; 
                   end
                end
              default: begin
                 rptr                    <= 2'b0; 
                 go_pwrdown              <= 1'b0;
                 go_pwrup                <= 1'b0;
                 pd_in_transition        <= 1'b0; 
              end // case: default
            endcase // case(updown_cmd_fifo[rptr])
      end // if (fsm_idle && !pd_in_transition)
   end // always @ (posedge iclk)

   gfx_pgfsm_fsm pgfsm_fsm (
                        .isolation(isolation),
                        .pg_reset(pg_reset),
                        .mother_sleep(mother_sleep),
                        .daughter_sleep(daughter_sleep),
                        .mem_mother_shutdown(mem_mother_shutdown),
                        .mem_daughter_shutdown(mem_daughter_shutdown),
                        .pgfsm_state(pgfsm_fsm_state),
                        .pg_status(pg_status),
                        .fsm_idle(fsm_idle),
                        .pgfsm_cmd_done(fsm_cmd_done),
                        //inputs
			.TEST_CLK_EN(TEST_CLK_EN),
                        .reset(reset),
                        .refclk(refclk),
			.refclk_en(refclk_en),
                        .iclk(iclk),
                        .powerdown(go_pwrdown),
                        .powerup(go_pwrup),
                        .pgfsm_delay0_reg(pgfsm_delay0_reg),
                        .pgfsm_delay1_reg(pgfsm_delay1_reg),
                        .pgfsm_delay2_reg(pgfsm_delay2_reg),
                        .pgfsm_delay3_reg(pgfsm_delay3_reg),
                        .pgfsm_delay4_reg(pgfsm_delay4_reg),
                        .pgfsm_delay5_reg(pgfsm_delay5_reg),
			.use_extra_delay_regs_en(use_extra_delay_regs_en),
                        .pgfsm_fsm_1_reg(pgfsm_fsm_1_reg),
                        .pgfsm_fsm_2_reg(pgfsm_fsm_2_reg),
                        .pgfsm_fsm_3_reg(pgfsm_fsm_3_reg),
                        .pgfsm_fsm_4_reg(pgfsm_fsm_4_reg),
                        .pgfsm_fsm_5_reg(pgfsm_fsm_5_reg),
                        .pgfsm_fsm_6_reg(pgfsm_fsm_6_reg)
                        );
   
   always @(posedge iclk) begin
      if (reset) begin
         isolation_pd             <= 1'b0;
         pg_reset_pd              <= 1'b0;
         mother_sleep_pd          <= 1'b0;
         daughter_sleep_pd        <= 1'b0;
         mem_mother_shutdown_pd   <= 1'b0;
         mem_daughter_shutdown_pd <= 1'b0;
      end else if (pd_in_transition & (pg_status_pd == `GFX_PGFSM_BLOCK_POWERING_OFF | pg_status_pd == `GFX_PGFSM_BLOCK_POWERING_ON)) begin
         isolation_pd             <= isolation;
         pg_reset_pd              <= pg_reset;
         mother_sleep_pd          <= mother_sleep;
         daughter_sleep_pd        <= daughter_sleep;
         mem_mother_shutdown_pd   <= mem_mother_shutdown;
         mem_daughter_shutdown_pd <= mem_daughter_shutdown;
      end 
   end

   always @(posedge iclk) begin
      if (reset)  pg_status_pd    <= `GFX_PGFSM_BLOCK_POWER_ON;
      else        pg_status_pd    <= pd_in_transition ? pg_status : pg_status_pd ; 
   end

   always @(posedge iclk) begin
      if (reset)  pgfsm_cmd_done  <= 1'b0;
      else        pgfsm_cmd_done  <= fsm_cmd_done;
   end

endmodule

