////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : gfx_pgfsm_deser.v
// Description : This module de-serializes data into a parallel word

////////////////////////////////////////////////////////////////////////////

`include "gfx_pgfsm_defines.h"

// De-serializer Module for PGFSM block
// Note that this block only works when the number of bits is a power of 2.  

module gfx_pgfsm_deser ( /*autoarg*/
   // Outputs
   data_out, new_data,
   // Inputs
   serial_in, iclk, reset, hard_resetb
   ); 

parameter width = 32;
parameter counter_width = 5;
parameter short_width = 8;
parameter short_counter_width = 3;

output [width-1:0] data_out; 
output new_data; 
input serial_in; 

input iclk; 
input reset, hard_resetb; 


reg serial_in_ff;
reg new_data; 
wire [width-1:0] long_data;
wire [width-1:0] short_data;
wire [width-1:0] data_out; 
reg  [width-1:0] data_out_reg; 

reg [counter_width-1:0] counter; 
reg [short_counter_width-1:0] short_cnt; 
reg counter_ff1; 
reg short_cnt_ff1;

reg short_cmd_ff;

always @(posedge iclk) begin
    if (reset) 
      data_out_reg <= {width{1'b0}};
    else if (new_data)
      data_out_reg <= {width{1'b0}};
    else if ((|counter) | counter_ff1)
      data_out_reg <= {serial_in, data_out_reg[width-1:1]};
end

// Long Counter
always @(posedge iclk) begin
    if      (reset)                    counter <= {counter_width{1'b0}};
    else if (new_data)                 counter <= {counter_width{1'b0}};
    else if (|counter)                 counter <= counter + {{counter_width-1{1'b0}},1'b1}; 
    else if (serial_in & ~counter_ff1) counter <= {{counter_width-1{1'b0}},1'b1};
end

// Short Counter
always @(posedge iclk) begin
    if      (reset)                      short_cnt <= {short_counter_width{1'b0}};
    else if (new_data)                   short_cnt <= {short_counter_width{1'b0}};
    else if (|short_cnt)                 short_cnt <= short_cnt + {{short_counter_width-1{1'b0}},1'b1};
    else if (serial_in & ~short_cnt_ff1) short_cnt <= {{short_counter_width-1{1'b0}},1'b1};
end

always @(posedge iclk) counter_ff1   <= &counter; 
always @(posedge iclk) short_cnt_ff1 <= &short_cnt; 

// Detect if Short or Long Command
always @(posedge iclk) begin
  if   (reset) serial_in_ff  <= 1'b0;
  else         serial_in_ff  <= serial_in;
end

wire long_cmd_done  = counter_ff1 & hard_resetb;
wire short_cmd_done = short_cnt_ff1 & hard_resetb & 
                      (data_out_reg[width-short_width+1 : width-short_width] == 2'b10);

always @(posedge iclk) begin
  if (reset)    
     short_cmd_ff  <= 1'b0;
  else if (new_data) 
     short_cmd_ff  <= 1'b0;
  else if (short_cmd_done)
     short_cmd_ff  <= 1'b1;
end
  
//always @(posedge iclk) new_data <= short_cmd_done ? short_cmd_done : long_cmd_done;
always @(posedge iclk) new_data <= long_cmd_done;


// Modify de-serialized data as per command format
  assign short_data[`GFX_PGFSM_CMD_ADDRESS] = 8'hFF;  // broadcast address
  // discard data_out_reg[data_out_reg[width-short_width + `GFX_PGFSM_CMD_TYPE] bit
  assign short_data[`GFX_PGFSM_CMD_PWRDN]   = data_out_reg[width-short_width + `GFX_PGFSM_SCMD_PWRDN];
  assign short_data[`GFX_PGFSM_CMD_PWRUP]   = data_out_reg[width-short_width + `GFX_PGFSM_SCMD_PWRUP];
  assign short_data[`GFX_PGFSM_CMD_P1]      = data_out_reg[width-short_width + `GFX_PGFSM_SCMD_P1];
  assign short_data[`GFX_PGFSM_CMD_P2]      = data_out_reg[width-short_width + `GFX_PGFSM_SCMD_P2];
  // all other bits are 1'b0
  // assign short_data[width-1 : width-short_width+`GFX_PGFSM_SCMD_P2+1] = {width - 12{1'b0}};
  assign short_data[width-1 : 12] = {width - 12{1'b0}};

  // discard data_out_reg[`GFX_PGFSM_CMD_TYPE] bit
  //assign long_data[`GFX_PGFSM_CMD_RDDATA_RESET: `GFX_PGFSM_CMD_ADDRESS_LO] = 
  //        data_out_reg[`GFX_PGFSM_LCMD_RDDATA_RESET : `GFX_PGFSM_LCMD_ADDRESS_LO]; // long_data[14:0] = data_out_reg[15:1]
  //assign long_data[`GFX_PGFSM_CMD_SHORT_FORMAT] = 1'b0; // long_data[15]
  // all other bits are as-is
  //assign long_data[width-1 : `GFX_PGFSM_CMD_SHORT_FORMAT+1] = data_out_reg[width -1: `GFX_PGFSM_CMD_SHORT_FORMAT+1];
  assign long_data[width -1 : 0] = data_out_reg[width -1 : 0];


  //assign data_out[width -1 : 0] = short_cmd_ff ? short_data : long_data;
  assign data_out[width -1 : 0] = long_data[width -1 : 0];

endmodule

