////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
// Filename    : gfx_pgfsm_fsm.v
// FSM to control the Power Down and Power Up sequence
////////////////////////////////////////////////////////////////////////////

`include "gfx_pgfsm_defines.h"

module gfx_pgfsm_fsm (/*autoarg*/
   // Outputs
   pg_reset, isolation, mother_sleep, daughter_sleep, mem_mother_shutdown, mem_daughter_shutdown,
   pgfsm_state, pg_status, fsm_idle, pgfsm_cmd_done,
   // Inputs
   TEST_CLK_EN,
   reset, refclk, refclk_en, iclk, powerdown, powerup, pgfsm_delay0_reg,
   pgfsm_delay1_reg, pgfsm_fsm_1_reg, pgfsm_fsm_2_reg,
   pgfsm_fsm_3_reg, pgfsm_fsm_4_reg, pgfsm_fsm_5_reg, pgfsm_fsm_6_reg,
   pgfsm_delay2_reg, pgfsm_delay3_reg, pgfsm_delay4_reg, pgfsm_delay5_reg, use_extra_delay_regs_en
   ); 

   /**************************************************/
   /*               State Parameters                 */
   /**************************************************/

    parameter PGFSM_STATE_BITS = 5;

    parameter IDLE             = 5'b00000;

    //Power Down states  
    parameter S_PWRUP_DELAY    = 5'b01000;	//wait pwrup delay	
    parameter S1               = 5'b00001;	//assert reset
    parameter S2               = 5'b00011;	//assert iso
    parameter S3_M             = 5'b00010;	//assert mem mother
    parameter S3_D             = 5'b00110;	//assert mem daughter
    parameter S4               = 5'b00111;	//assert mother sleep
    parameter S5               = 5'b00101;	//assert daughter sleep
//  parameter S6               = 5'd7;  //wait reset delay
//  parameter S7               = 5'd8;	//wait reset delay

    //Power Up states
    parameter S8               = 5'b01001;	//assert reset
    parameter S9               = 5'b01011;	//de-assert daughter
    parameter S10              = 5'b01010;	//de-assert mother
    parameter S11_D            = 5'b01110;	//de-assert mem daughter SD
    parameter S11_M            = 5'b01111; //de-assert mem mother SD
    parameter S12              = 5'b01101;	//de-assert iso
    parameter S13              = 5'b01100;	//de-assert reset
    parameter S14              = 5'b00100;	//wait-reset delay

   /**************************************************/
   /*               Signal Definition                */
   /**************************************************/

   input				TEST_CLK_EN;   
   input                                reset; 
   input                                refclk; 
   input				refclk_en;
   input                                iclk; 
   input                                powerdown; 
   input                                powerup; 

   input [23:0]                         pgfsm_delay0_reg; 
   input [23:0]                         pgfsm_delay1_reg;
   input [23:0]                         pgfsm_delay2_reg;
   input [23:0]                         pgfsm_delay3_reg;
   input [23:0]                         pgfsm_delay4_reg;
   input [23:0]                         pgfsm_delay5_reg;
   input				use_extra_delay_regs_en;

   input [23:0]                         pgfsm_fsm_1_reg;
   input [23:0]                         pgfsm_fsm_2_reg;
   input [23:0]                         pgfsm_fsm_3_reg;
   input [23:0]                         pgfsm_fsm_4_reg;
   input [23:0]                         pgfsm_fsm_5_reg;
   input [23:0]                         pgfsm_fsm_6_reg;

   output                               pg_reset; 
   output                               isolation; 
   output                               mother_sleep; 
   output                               daughter_sleep; 
   output                               mem_mother_shutdown; 
   output                               mem_daughter_shutdown;


   output [PGFSM_STATE_BITS-1:0]        pgfsm_state; 
   output [1:0]                         pg_status; 
   output                               fsm_idle; 
   output                               pgfsm_cmd_done;
   
   
   reg [PGFSM_STATE_BITS-1:0]           pgfsm_state, next_pgfsm_state; 
   reg [1:0]                            pg_status, next_pg_status;
   reg                                  pgfsm_cmd_done;

   reg                                  pg_reset, next_pg_reset; 
   reg                                  isolation, next_isolation; 
   reg                                  mother_sleep, next_mother_sleep; 
   reg                                  daughter_sleep, next_daughter_sleep; 
   reg                                  mem_mother_shutdown, next_mem_mother_shutdown; 
   reg                                  mem_daughter_shutdown, next_mem_daughter_shutdown;

   wire                                 counter_done; 
   wire                                 zero_delay_pre;
   reg                                  zero_delay;

   wire  [23:0]                         pgfsm_delay0_reg; 
   wire  [23:0]                         pgfsm_delay1_reg;
   wire  [23:0]                         pgfsm_delay2_reg;
   wire  [23:0]                         pgfsm_delay3_reg;
   wire  [23:0]                         pgfsm_delay4_reg;
   wire  [23:0]                         pgfsm_delay5_reg;

   wire  [23:0]                         pgfsm_fsm_1_reg;
   wire  [23:0]                         pgfsm_fsm_2_reg;
   wire  [23:0]                         pgfsm_fsm_3_reg;
   wire  [23:0]                         pgfsm_fsm_4_reg;
   wire  [23:0]                         pgfsm_fsm_5_reg;
   wire  [23:0]                         pgfsm_fsm_6_reg;

   /**************************************************/
   /*                   Detect rising edge           */
   /**************************************************/
   reg                                  powerup_ff1, powerdown_ff1; 
   wire                                 powerup_pulse, powerdown_pulse;
   always @(posedge iclk) begin
      powerup_ff1         <= powerup; 
      powerdown_ff1       <= powerdown; 
   end
   
   assign powerup_pulse    = powerup & ~powerup_ff1;
   assign powerdown_pulse  = powerdown & ~powerdown_ff1;

   //FSM status
   assign fsm_idle         = pgfsm_state == IDLE; 

   // FSM state advancing condition
   wire fsm_advance        = zero_delay | counter_done;
   
   /**************************************************/
   /*                   FSM                          */
   /**************************************************/
   
   always @(posedge iclk) begin
      if (reset) begin
         pgfsm_state             <= {PGFSM_STATE_BITS{1'b0}}; 
         pg_status               <= 2'b0;
         pgfsm_cmd_done          <= 1'b0;

         pg_reset                <= 1'b0;
         isolation               <= 1'b0;
         mother_sleep            <= 1'b0;
         daughter_sleep          <= 1'b0;
         mem_mother_shutdown     <= 1'b0;
         mem_daughter_shutdown   <= 1'b0;


      end else begin
         pgfsm_state             <= next_pgfsm_state; 
         pg_status               <= next_pg_status; 
       //pgfsm_cmd_done          <= fsm_advance & ((pgfsm_state == S7) | (pgfsm_state == S14));
         pgfsm_cmd_done          <= fsm_advance & ((pgfsm_state == S5) | (pgfsm_state == S14));
         pg_reset                <= next_pg_reset;
         isolation               <= next_isolation;
         mother_sleep            <= next_mother_sleep;
         daughter_sleep          <= next_daughter_sleep;
         mem_mother_shutdown     <= next_mem_mother_shutdown;
         mem_daughter_shutdown   <= next_mem_daughter_shutdown;
      end
   end
   
   always @(*) begin
      next_pgfsm_state                            =  pgfsm_state;
      next_pg_status                              =  pg_status; 

      next_pg_reset                               =  pg_reset;      
      next_isolation                              =  isolation;  
      next_mother_sleep                           =  mother_sleep;  
      next_daughter_sleep                         =  daughter_sleep;
      next_mem_mother_shutdown                    =  mem_mother_shutdown;
      next_mem_daughter_shutdown                  =  mem_daughter_shutdown;

      case (pgfsm_state) 
        IDLE: begin
           if (powerdown_pulse) begin
              next_pgfsm_state                    = S1; 
              next_pg_status                      = `GFX_PGFSM_BLOCK_POWERING_OFF;
           end else if (powerup_pulse) begin
              next_pgfsm_state                    = S_PWRUP_DELAY;
              next_pg_status                      = `GFX_PGFSM_BLOCK_POWERING_ON;
           end
        end 
        //-------------------------------
        //Power Down states
        //--------------------------------
        S1: begin    
           next_pg_reset                          =  pgfsm_fsm_1_reg[0];      
           next_isolation                         =  pgfsm_fsm_1_reg[1];  
           next_mother_sleep                      =  pgfsm_fsm_1_reg[2];
           next_daughter_sleep                    =  pgfsm_fsm_1_reg[3];
           next_mem_mother_shutdown               =  pgfsm_fsm_1_reg[4];
           next_mem_daughter_shutdown             =  pgfsm_fsm_1_reg[5];
           if (fsm_advance) next_pgfsm_state      = S2; 
        end
        S2: begin    
           next_pg_reset                          =  pgfsm_fsm_1_reg[8];      
           next_isolation                         =  pgfsm_fsm_1_reg[9];  
           next_mother_sleep                      =  pgfsm_fsm_1_reg[10];
           next_daughter_sleep                    =  pgfsm_fsm_1_reg[11];
           next_mem_mother_shutdown               =  pgfsm_fsm_1_reg[12];
           next_mem_daughter_shutdown             =  pgfsm_fsm_1_reg[13];
           if (fsm_advance) next_pgfsm_state      = S3_M; 
        end
        S3_M: begin    
           next_pg_reset                          =  pgfsm_fsm_1_reg[16];      
           next_isolation                         =  pgfsm_fsm_1_reg[17];  
           next_mother_sleep                      =  pgfsm_fsm_1_reg[18];
           next_daughter_sleep                    =  pgfsm_fsm_1_reg[19];
           next_mem_mother_shutdown               =  pgfsm_fsm_1_reg[20];
           next_mem_daughter_shutdown             =  pgfsm_fsm_1_reg[21];
           if (fsm_advance) next_pgfsm_state      = S3_D; 
        end
        S3_D: begin
           next_pg_reset                          =  pgfsm_fsm_6_reg[0];
           next_isolation                         =  pgfsm_fsm_6_reg[1];
           next_mother_sleep                      =  pgfsm_fsm_6_reg[2];
           next_daughter_sleep                    =  pgfsm_fsm_6_reg[3];
           next_mem_mother_shutdown               =  pgfsm_fsm_6_reg[4];
           next_mem_daughter_shutdown             =  pgfsm_fsm_6_reg[5];
           if (fsm_advance) next_pgfsm_state      = S4;
        end
        S4: begin    
           next_pg_reset                          =  pgfsm_fsm_2_reg[0];      
           next_isolation                         =  pgfsm_fsm_2_reg[1];  
           next_mother_sleep                      =  pgfsm_fsm_2_reg[2];
           next_daughter_sleep                    =  pgfsm_fsm_2_reg[3];
           next_mem_mother_shutdown               =  pgfsm_fsm_2_reg[4];
           next_mem_daughter_shutdown             =  pgfsm_fsm_2_reg[5];
           if (fsm_advance) next_pgfsm_state      = S5; 
        end
        S5: begin    
           next_pg_reset                          =  pgfsm_fsm_2_reg[8];      
           next_isolation                         =  pgfsm_fsm_2_reg[9];  
           next_mother_sleep                      =  pgfsm_fsm_2_reg[10];
           next_daughter_sleep                    =  pgfsm_fsm_2_reg[11];
           next_mem_mother_shutdown               =  pgfsm_fsm_2_reg[12];
           next_mem_daughter_shutdown             =  pgfsm_fsm_2_reg[13];
         //if (fsm_advance) next_pgfsm_state      = S6; 
           if (fsm_advance) begin 
              next_pgfsm_state                    = IDLE; 
              next_pg_status                      = `GFX_PGFSM_BLOCK_POWER_OFF;
           end
        end

//////////////////////////////////////////////////////////////////////////
// Removed as per UBTS 414869
//////////////////////////////////////////////////////////////////////////
//      S6: begin    
//         next_pg_reset                          =  pgfsm_fsm_2_reg[16];      
//         next_isolation                         =  pgfsm_fsm_2_reg[17];  
//         next_mother_sleep                      =  pgfsm_fsm_2_reg[18];
//         next_daughter_sleep                    =  pgfsm_fsm_2_reg[19];
//         next_mem_mother_shutdown               =  pgfsm_fsm_2_reg[20];
//         next_mem_daughter_shutdown             =  pgfsm_fsm_2_reg[21];
//         if (counter_done) next_pgfsm_state     = S7; 
//      end
//      S7: begin    
//         next_pg_reset                          =  pgfsm_fsm_3_reg[0];      
//         next_isolation                         =  pgfsm_fsm_3_reg[1];  
//         next_mother_sleep                      =  pgfsm_fsm_3_reg[2];
//         next_daughter_sleep                    =  pgfsm_fsm_3_reg[3];
//         next_mem_mother_shutdown               =  pgfsm_fsm_3_reg[4];
//         next_mem_daughter_shutdown             =  pgfsm_fsm_3_reg[5];
//         if (counter_done) begin 
//            next_pgfsm_state                    = IDLE; 
//            next_pg_status                      = `GFX_PGFSM_BLOCK_POWER_OFF;
//         end
//      end
//////////////////////////////////////////////////////////////////////////

        //-------------------------------
        //Power UP states
        //--------------------------------
        S_PWRUP_DELAY: begin
           next_pg_reset                          =  pgfsm_fsm_6_reg[16];
           next_isolation                         =  pgfsm_fsm_6_reg[17];
           next_mother_sleep                      =  pgfsm_fsm_6_reg[18];
           next_daughter_sleep                    =  pgfsm_fsm_6_reg[19];
           next_mem_mother_shutdown               =  pgfsm_fsm_6_reg[20];
           next_mem_daughter_shutdown             =  pgfsm_fsm_6_reg[21];
           if (fsm_advance) next_pgfsm_state      = S8;
        end
        S8: begin    
           next_pg_reset                          =  pgfsm_fsm_3_reg[8];      
           next_isolation                         =  pgfsm_fsm_3_reg[9];  
           next_mother_sleep                      =  pgfsm_fsm_3_reg[10];
           next_daughter_sleep                    =  pgfsm_fsm_3_reg[11];
           next_mem_mother_shutdown               =  pgfsm_fsm_3_reg[12];
           next_mem_daughter_shutdown             =  pgfsm_fsm_3_reg[13];
           if (fsm_advance) next_pgfsm_state      = S9; 
        end
        S9: begin    
           next_pg_reset                          =  pgfsm_fsm_3_reg[16];      
           next_isolation                         =  pgfsm_fsm_3_reg[17];  
           next_mother_sleep                      =  pgfsm_fsm_3_reg[18];
           next_daughter_sleep                    =  pgfsm_fsm_3_reg[19];
           next_mem_mother_shutdown               =  pgfsm_fsm_3_reg[20];
           next_mem_daughter_shutdown             =  pgfsm_fsm_3_reg[21];
           if (fsm_advance) next_pgfsm_state      = S10; 
        end
        S10: begin    
           next_pg_reset                          =  pgfsm_fsm_4_reg[0];      
           next_isolation                         =  pgfsm_fsm_4_reg[1];  
           next_mother_sleep                      =  pgfsm_fsm_4_reg[2];
           next_daughter_sleep                    =  pgfsm_fsm_4_reg[3];
           next_mem_mother_shutdown               =  pgfsm_fsm_4_reg[4];
           next_mem_daughter_shutdown             =  pgfsm_fsm_4_reg[5];
           if (fsm_advance) next_pgfsm_state      = S11_D; 
        end
        S11_D: begin    
           next_pg_reset                          =  pgfsm_fsm_6_reg[8];      
           next_isolation                         =  pgfsm_fsm_6_reg[9];  
           next_mother_sleep                      =  pgfsm_fsm_6_reg[10];
           next_daughter_sleep                    =  pgfsm_fsm_6_reg[11];
           next_mem_mother_shutdown               =  pgfsm_fsm_6_reg[12];
           next_mem_daughter_shutdown             =  pgfsm_fsm_6_reg[13];
           if (fsm_advance) next_pgfsm_state      = S11_M; 
        end
        S11_M: begin
           next_pg_reset                          =  pgfsm_fsm_4_reg[8];
           next_isolation                         =  pgfsm_fsm_4_reg[9];
           next_mother_sleep                      =  pgfsm_fsm_4_reg[10];
           next_daughter_sleep                    =  pgfsm_fsm_4_reg[11];
           next_mem_mother_shutdown               =  pgfsm_fsm_4_reg[12];
           next_mem_daughter_shutdown             =  pgfsm_fsm_4_reg[13];
           if (fsm_advance) next_pgfsm_state      = S12;
        end
        S12: begin    
           next_pg_reset                          =  pgfsm_fsm_4_reg[16];      
           next_isolation                         =  pgfsm_fsm_4_reg[17];  
           next_mother_sleep                      =  pgfsm_fsm_4_reg[18];
           next_daughter_sleep                    =  pgfsm_fsm_4_reg[19];
           next_mem_mother_shutdown               =  pgfsm_fsm_4_reg[20];
           next_mem_daughter_shutdown             =  pgfsm_fsm_4_reg[21];
           if (fsm_advance) next_pgfsm_state     = S13; 
        end
        S13: begin    
           next_pg_reset                          =  pgfsm_fsm_5_reg[0];      
           next_isolation                         =  pgfsm_fsm_5_reg[1];  
           next_mother_sleep                      =  pgfsm_fsm_5_reg[2];
           next_daughter_sleep                    =  pgfsm_fsm_5_reg[3];
           next_mem_mother_shutdown               =  pgfsm_fsm_5_reg[4];
           next_mem_daughter_shutdown             =  pgfsm_fsm_5_reg[5];
           if (fsm_advance) next_pgfsm_state      = S14; 
        end
        S14: begin    
           next_pg_reset                          =  pgfsm_fsm_5_reg[8];      
           next_isolation                         =  pgfsm_fsm_5_reg[9];  
           next_mother_sleep                      =  pgfsm_fsm_5_reg[10];
           next_daughter_sleep                    =  pgfsm_fsm_5_reg[11];
           next_mem_mother_shutdown               =  pgfsm_fsm_5_reg[12];
           next_mem_daughter_shutdown             =  pgfsm_fsm_5_reg[13];
           if (fsm_advance) begin 
              next_pgfsm_state                    = IDLE; 
              next_pg_status                      = `GFX_PGFSM_BLOCK_POWER_ON;
           end
        end // case: S14
        default: begin
           next_pgfsm_state                       = IDLE; 
        end
      endcase 
   end // always @ (*)
   
   wire rst_delay_pulse_pre    = (pgfsm_state != next_pgfsm_state) & 
                                 ( (next_pgfsm_state == S1) | (next_pgfsm_state == S13) |
                                 //(next_pgfsm_state == S6) | (next_pgfsm_state == S7)  |
                                   (next_pgfsm_state == S8) | (next_pgfsm_state == S14)
                                 );

   wire pwrup_delay_pulse_pre  = (pgfsm_state != next_pgfsm_state) & (next_pgfsm_state == S_PWRUP_DELAY);

   wire iso_delay_pulse_pre    = (pgfsm_state != next_pgfsm_state) & 
                                 ( (next_pgfsm_state == S2) | (next_pgfsm_state == S12)
                                 );

   wire mem_mother_delay_pulse_pre = (pgfsm_state != next_pgfsm_state) & 
                                     ( (next_pgfsm_state == S3_M) | (next_pgfsm_state == S11_M)
                                     );

   wire mem_daughter_delay_pulse_pre = (pgfsm_state != next_pgfsm_state) & 
                                       ( (next_pgfsm_state == S3_D) | (next_pgfsm_state == S11_D)
                                       );

   wire msleep_delay_pulse_pre = (pgfsm_state != next_pgfsm_state) & 
                                 ( (next_pgfsm_state == S4) | (next_pgfsm_state == S10)
                                 );

   wire dsleep_delay_pulse_pre = (pgfsm_state != next_pgfsm_state) & 
                                 ( (next_pgfsm_state == S5) | (next_pgfsm_state == S9)
                                 );

   wire delay_pulse_pre =  rst_delay_pulse_pre | pwrup_delay_pulse_pre | iso_delay_pulse_pre | 
                           mem_mother_delay_pulse_pre | mem_daughter_delay_pulse_pre |
                           msleep_delay_pulse_pre     | dsleep_delay_pulse_pre;
                       

   reg delay_pulse;

   // flop to take care of CDC issues
   always @(posedge iclk) begin
     if (reset)
       delay_pulse <= 1'b0;
     else 
       // UBTS 414869: do not send REFCLK pulse if programmed delay == 0
       delay_pulse <= delay_pulse_pre & ~zero_delay_pre;
   end

   //Add support for extra delays to have seperate delays for powering up and powering down sequence
   //if use_extra_delay_regs_en == 0, then use delay0/delay1 for powering up/down
   //if use_extra_delay_regs_en == 1, then use delay0/delay1 for powering down and delay2/delay3 for powering up 
   wire use_extra_delay             = use_extra_delay_regs_en & ~(next_pg_status == `GFX_PGFSM_BLOCK_POWERING_OFF);

   wire [23:0] new_pgfsm_delay0_reg = use_extra_delay ? pgfsm_delay2_reg : pgfsm_delay0_reg;
   wire [23:0] new_pgfsm_delay1_reg = use_extra_delay ? pgfsm_delay3_reg : pgfsm_delay1_reg;

   wire [7:0] mem_mother_delay      = use_extra_delay ? 
                                      pgfsm_delay4_reg[`GFX_PGFSM_MEM_SD_M_PWRUP_DELAY] :
                                      pgfsm_delay4_reg[`GFX_PGFSM_MEM_SD_M_PWRDN_DELAY];

  // mux delay register based on respective _pulse_pre signal
  wire [23:0] delay_pre = rst_delay_pulse_pre   ? {16'b0, new_pgfsm_delay0_reg[`GFX_PGFSM_RESET_DELAY]} :
                          iso_delay_pulse_pre   ? {16'b0, new_pgfsm_delay0_reg[`GFX_PGFSM_ISOLATION_DELAY]} :
                          pwrup_delay_pulse_pre ? pgfsm_delay5_reg :
                          mem_daughter_delay_pulse_pre ? {16'b0, new_pgfsm_delay0_reg[`GFX_PGFSM_MEM_SD_DELAY]} :
                          mem_mother_delay_pulse_pre   ? {16'b0, mem_mother_delay} :
                          dsleep_delay_pulse_pre ? {12'b0, new_pgfsm_delay1_reg[`GFX_PGFSM_DAUGHTER_DELAY]} :
                          msleep_delay_pulse_pre ? {12'b0, new_pgfsm_delay1_reg[`GFX_PGFSM_MOTHER_DELAY]} : 
                          24'b0;

  // UBTS 414869: Check if programmed delay value for next state is zero
  assign zero_delay_pre = ~(|delay_pre[23:0]);

reg [23:0] delay;

always @(posedge iclk) begin
    if (reset) begin
      delay      <= 24'b0;
      zero_delay <= 1'b0;
    end
    else if (delay_pulse_pre) begin
      delay      <= delay_pre;
      zero_delay <= zero_delay_pre;
    end
end
 
   // REFCLK based PGFSM counter 
   gfx_pgfsm_counter pgfsm_counter(
                               // outputs
                               .counter_done(counter_done),
                               // inputs
			       .TEST_CLK_EN(TEST_CLK_EN),
                               .iclk(iclk),
                               .reset(reset),
                               .irefclk(refclk),
			       .refclk_en(refclk_en),
                               .delay_pulse(delay_pulse),
                               .delay(delay)
                               ); 

endmodule


