
////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of AMD, Inc.
 *  Copyright 2012, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : gfx_pgfsm_serdes_master.v
////////////////////////////////////////////////////////////////////////////


/* PGFSM SerDes Master block */

`include "gfx_pgfsm_defines.h"
`include "gc_features.vh"

module gfx_pgfsm_serdes_master ( /*autoarg*/
   // Outputs
   PGFSM_SD_next_daisy_cmd, PGFSM_read_data_new, PGFSM_read_data,
   SERDES_SMU_power_status, SERDES_busy,
   // Inputs
   PGFSM_SD_next_daisy_status, SRBM_PGFSM_wdata, SRBM_PGFSM_control,
   SRBM_PGFSM_start_tr, IP_PGFSM_wdata, IP_PGFSM_control,
   IP_PGFSM_start_tr, iclk, reset
   ); 

   //----------------------------------------------------------------------

   parameter POWER_STATUS_PER_PGFSM = 0; 

   // When the block is instantiated, the parameter should be set to the number of tiles in the daisy chain for this IP
   parameter status_reg_count = 4;
   
   parameter POWER_STATUS_WIDTH = 1;

   localparam CMD_WIDTH = 24;
   localparam DATA_WIDTH = 32;
   localparam SHORT_CMD_WIDTH = 8;
   localparam ADDR_WIDTH = 8;

   parameter READ_DATA_WIDTH = 24;

   //----------------------------------------------------------------------

   input        iclk;
   input        reset; 
   

   output    PGFSM_SD_next_daisy_cmd; 
   input     PGFSM_SD_next_daisy_status; 
   
   input                     SRBM_PGFSM_start_tr; 
   input  [DATA_WIDTH -1 :0] SRBM_PGFSM_wdata; 
   input  [CMD_WIDTH -1 :0]  SRBM_PGFSM_control; 
   
   input                     IP_PGFSM_start_tr; 
   input  [DATA_WIDTH -1 :0] IP_PGFSM_wdata; 
   input  [CMD_WIDTH -1 :0]  IP_PGFSM_control; 
   
   // parametrized inputs //
   output [ status_reg_count-1 : 0] PGFSM_read_data_new; 
   output [ status_reg_count*READ_DATA_WIDTH-1 :0] PGFSM_read_data; 

   // Power status output -Status for one SERDES master
   // 1=All tiles connected to SERDES master are Powered-down,
   // 0=All tiles connected to SERDES master are powered-up
   output [POWER_STATUS_WIDTH-1 : 0] SERDES_SMU_power_status; //Status for one SERDES master 
   
   //0=SERDES is ready to receive new command.
   //1=SERDES is transmitting previous command.
   output                       SERDES_busy;
   
   //----------------------------------------------------------------------
   wire                         SERDES_busy;

   wire                         use_short_cmd;
   wire [CMD_WIDTH + DATA_WIDTH -1 :0]     par_in;

  ///////////////////////////////////////////////////////////////////////////
  // Command Field Decoding 
  ///////////////////////////////////////////////////////////////////////////

   //wire  start_transmit = SRBM_PGFSM_start_tr | (~SRBM_PGFSM_control[`GFX_PGFSM_CMD_SRBM_OVERRIDE] & IP_PGFSM_start_tr);
   //wire  srbm_transmit  = SRBM_PGFSM_start_tr |   SRBM_PGFSM_control[`GFX_PGFSM_CMD_SRBM_OVERRIDE]; 
   wire  start_transmit = SRBM_PGFSM_start_tr;
   wire  srbm_transmit  = SRBM_PGFSM_start_tr;

   // Decode command bits  
   wire [CMD_WIDTH  -1 :0]        cmd = srbm_transmit ? SRBM_PGFSM_control : IP_PGFSM_control;
   wire [ADDR_WIDTH -1 :0] pgfsm_addr = cmd[`GFX_PGFSM_CMD_ADDRESS];

   wire bcast_addr     = (pgfsm_addr == `GFX_PGFSM_BCAST_ADDR);
   wire pwr_down_cmd   = cmd[`GFX_PGFSM_CMD_PWRDN];
   wire pwr_up_cmd     = cmd[`GFX_PGFSM_CMD_PWRUP];
   wire rd_cmd         = cmd[`GFX_PGFSM_CMD_READ];
   wire wr_cmd         = cmd[`GFX_PGFSM_CMD_WRITE];
  
   // UBTS 408265 : reset rd_data upon new command, if Reset RdData bit is set
   wire rddata_rst_cmd = cmd[`GFX_PGFSM_CMD_RDDATA_RESET];

   // UBTS 414869 : use SerDes short command format for Bcast PU/PD commands, if this Short Command bit is set
   // This will not be used if Rd/Wr bit is set
   assign use_short_cmd = cmd[`GFX_PGFSM_CMD_SHORT_FORMAT] & bcast_addr & (pwr_down_cmd | pwr_up_cmd) & ~(rd_cmd | wr_cmd);

  ///////////////////////////////////////////////////////////////////////////
  // Power Status Registers
  ///////////////////////////////////////////////////////////////////////////

//`ifdef GC__GFX_ISLAND_POWER_GATING__1 
   reg [POWER_STATUS_WIDTH-1:0] SERDES_SMU_power_status; //Status for one SERDES master 
//`else
//   wire [POWER_STATUS_WIDTH-1 : 0] SERDES_SMU_power_status;
//`endif

//`ifdef GC__GFX_ISLAND_POWER_GATING__1 
integer n; 
generate 
   if (POWER_STATUS_PER_PGFSM == 0) begin
      always @(posedge iclk) begin
         if (reset)
            SERDES_SMU_power_status[0] <= 1'b0;
         else
            SERDES_SMU_power_status[0] <= start_transmit & bcast_addr & pwr_down_cmd ? 1'b1 :
                                          start_transmit & bcast_addr & pwr_up_cmd   ? 1'b0 :
                                          SERDES_SMU_power_status[0];
      end // always @ (posedge iclk)
    end else begin
      always @(posedge iclk) begin
         for (n=0;n<status_reg_count;n=n+1) begin
            if (reset)
               SERDES_SMU_power_status[n] <= 1'b0;
            else
               SERDES_SMU_power_status[n] <= start_transmit & (bcast_addr | (pgfsm_addr == n)) & pwr_down_cmd ? 1'b1 :
                                             start_transmit & (bcast_addr | (pgfsm_addr == n)) & pwr_up_cmd   ? 1'b0 :
                                             SERDES_SMU_power_status[n];
         end  
      end
    end  
endgenerate    
//`else // !`ifdef GC__GFX_ISLAND_POWER_GATING__1
//   assign		 SERDES_SMU_power_status = {POWER_STATUS_WIDTH{1'b0}};
//`endif

   wire [DATA_WIDTH -1 :0] par_wdata   = srbm_transmit ? SRBM_PGFSM_wdata   : IP_PGFSM_wdata;
   wire [CMD_WIDTH -1 :0]  par_control = srbm_transmit ? SRBM_PGFSM_control : IP_PGFSM_control;

   // Modify data for Short commands
   wire [DATA_WIDTH -1 :0] short_wdata;
   wire [CMD_WIDTH -1 :0]  short_control;
   
   assign short_wdata   = {DATA_WIDTH{1'b0}};

   assign short_control[CMD_WIDTH-1: SHORT_CMD_WIDTH]       = {CMD_WIDTH - SHORT_CMD_WIDTH{1'b0}}; // bits[31:8]
   assign short_control[`GFX_PGFSM_SCMD_RSVD]                   = 3'b0;                                // bits[7:5]
   assign short_control[`GFX_PGFSM_SCMD_P2 : `GFX_PGFSM_SCMD_PWRDN] = par_control[`GFX_PGFSM_CMD_P2 : `GFX_PGFSM_CMD_PWRDN]; // bit[4:1]
   assign short_control[`GFX_PGFSM_CMD_TYPE]                    = `GFX_PGFSM_CMD_TYPE_SHORT;               // bit[0]

   // Modify data for Long Command bits
   wire [CMD_WIDTH -1 :0]  long_control;
   wire [DATA_WIDTH -1 :0] long_wdata;

   assign long_wdata = par_wdata;
   
   assign long_control[`GFX_PGFSM_LCMD_REG_ADDR]      = par_control[`GFX_PGFSM_CMD_REG_ADDR];      //  bits[23:16]
   assign long_control[`GFX_PGFSM_LCMD_BPM_ADDR]      = par_control[`GFX_PGFSM_CMD_BPM_ADDR];      //  bits[15:3]
   assign long_control[`GFX_PGFSM_LCMD_BPM_LONG_CMD]  = par_control[`GFX_PGFSM_CMD_BPM_LONG_CMD];  //  bit[2]
   assign long_control[`GFX_PGFSM_LCMD_BPM_WRITE]     = par_control[`GFX_PGFSM_CMD_BPM_WRITE];     //  bit[1]
   assign long_control[`GFX_PGFSM_LCMD_BPM_BROADCAST] = par_control[`GFX_PGFSM_CMD_BPM_BROADCAST]; //  bit[0]

   // CMD_SHORT_FORMAT bit is dropped to accomodate the extra CMD_TYPE bit
   //assign long_control[`GFX_PGFSM_LCMD_RDDATA_RESET : `GFX_PGFSM_LCMD_ADDRESS_LO] =
   //        par_control[`GFX_PGFSM_CMD_RDDATA_RESET  :  `GFX_PGFSM_CMD_ADDRESS_LO];                 // bits[15:1]
   //assign long_control[`GFX_PGFSM_CMD_TYPE]          = `GFX_PGFSM_CMD_TYPE_LONG;                   // bit[0];

   // Select Short versus Long format as per CMD_SHORT_FORMAT bit
   assign par_in = use_short_cmd ? { short_wdata, short_control} : {long_wdata, long_control};

  ///////////////////////////////////////////////////////////////////////////
  // Serializer and De-serializer Instances
  ///////////////////////////////////////////////////////////////////////////
 
   gfx_pgfsm_ser #(64,6) pgfsm_ser  (
                               // outputs
                               .ser_busy   (SERDES_busy),
                               .ser_data   (PGFSM_SD_next_daisy_cmd),
                               // inputs
                               //.par_data   ({8'h00, par_in}),
                               .par_data   ({par_in[55:24], 8'h00, par_in[23:0]}),
                               .par_valid  (start_transmit ),
                               .short_cmd  (use_short_cmd),
                               .clk(iclk),
                               .reset(reset)
                               ); 
   
//`ifdef GC__GFX_ISLAND_POWER_GATING__1
   reg [status_reg_count-1 :0]        PGFSM_read_data_new;
   reg [status_reg_count*READ_DATA_WIDTH -1 :0]    PGFSM_read_data;

   wire                         new_data;
   wire [31:0]                  data_out;

   gfx_pgfsm_status_deser #(32,5) deserializer  (
                                      // outputs
                                      .data_out(data_out),
                                      .new_data(new_data),
                                      // inputs
                                      .serial_in(PGFSM_SD_next_daisy_status),
                                      .iclk(iclk),
                                      .reset(reset),
                                      .hard_resetb(1'b1)
                                      ); 
   
  ///////////////////////////////////////////////////////////////////////////
  // PGFSM Read Data Registers
  // This block assumes local address of PGFSM is from 0 to No_of_tiles - 1
  ///////////////////////////////////////////////////////////////////////////

   generate
      genvar i; 
      if(status_reg_count == 1) begin  //status_reg_count==1 is used by GFX chains so that there is only ONE status register instead of one status register per tile in the chain. This is done to save area since there are many pgfsm instances (hundreds) in GFX. This also works for System (Non-GFX) chains with just one tile because we dont really care about the address.
           always @(posedge iclk) begin
             if (reset) begin
                PGFSM_read_data_new[0]   <= 1'b0;
                PGFSM_read_data[READ_DATA_WIDTH-1:0]    <= {READ_DATA_WIDTH{1'b0}};
             // UBTS 408265: reset PGFSM_read_data if Read Data Reset bit is set
             end else if (start_transmit & rddata_rst_cmd) begin
                PGFSM_read_data[READ_DATA_WIDTH-1:0]    <= {READ_DATA_WIDTH{1'b0}};
             end else if (new_data) begin
                PGFSM_read_data_new[0]   <= new_data;
                PGFSM_read_data[READ_DATA_WIDTH-1:0]    <= data_out[READ_DATA_WIDTH-1:0];
             end else begin
                // the PGFSM_read_data_new should be a pulse signal.
                PGFSM_read_data_new[0]   <= 1'b0;
             end
           end // always @ (posedge iclk)
      end else begin //status_reg_count != 1
        for (i=0; i < status_reg_count; i=i+1)
          begin: readgen
             always @(posedge iclk) begin
               if (reset) begin
                  PGFSM_read_data_new[i]                  <= 1'b0;
                  PGFSM_read_data[(24*(i+1)) -1: 24*i]    <= 24'b0;
               // UBTS 408265: reset PGFSM_read_data if Read Data Reset bit is set
               end else if (start_transmit & rddata_rst_cmd & (pgfsm_addr == i)) begin
                  PGFSM_read_data[(24*(i+1)) -1: 24*i]    <= 24'b0;
               end else if ({24'd0, data_out[31:24]} == i && new_data) begin
                  PGFSM_read_data_new[i]                  <= new_data;
                  PGFSM_read_data[(24*(i+1)) -1: 24*i]    <= data_out[23:0];
               end else if ({24'd0, data_out[31:24]} == i) begin
                  // the PGFSM_read_data_new should be a pulse signal.
                  PGFSM_read_data_new[i]                  <= 1'b0;
               end
             end // always @ (posedge iclk)
          end // block: readgen
      end //status_reg_count != 1
   endgenerate
   
//`else // !`ifdef GC__GFX_ISLAND_POWER_GATING__1
//   wire [status_reg_count-1 :0] PGFSM_read_data_new; 
//   wire [status_reg_count*24 -1 :0] PGFSM_read_data;
//   assign                     PGFSM_read_data_new = {(status_reg_count){1'b0}};
//   assign                     PGFSM_read_data     = {(status_reg_count*24){1'b0}};
//`endif

  ///////////////////////////////////////////////////////////////////////////
  // Simulation Only code
  ///////////////////////////////////////////////////////////////////////////

`ifdef SIMULATION
 `ifdef ABV
  `include "msgio_defines.vh"

  `ifdef GFX_EN_PGFSM_ASSERT
   always @ (posedge iclk) begin
      if (reset) begin
      end else begin
         if(SERDES_busy & (IP_PGFSM_start_tr | SRBM_PGFSM_start_tr)) begin
            `MSGIO_DISPLAY("--(PGFSM-SerDes Master)-- : ERROR: IP_PGFSM_start_tr OR SRBM_PGFSM_start_tr is active when SERDES_busy is active. %m, TIME=%t \n", $time);
         end
      end // else: !if(reset)
   end // always @ (posedge iclk)
      
  `endif
 `endif
`endif 

endmodule

//----------------------------------------------------------------------
