////
//
//  Trade secret of ATI Technologies, Inc.
//  Copyright 2005, ATI Technologies, Inc., (unpublished)
//
//  All rights reserved.  This notice is intended as a precaution against
//  inadvertent publication and does not imply publication or any waiver
//  of confidentiality.  The year included in the foregoing notice is the
//  year of creation of the work.
//
////

`include "msgio_defines.vh"



module ifrit_delay_neg(clk, datain, dataout);

    parameter instance_name         = "name";
    parameter WIDTH                 = 1;
    parameter use_args              = 0;
    parameter MAXDEPTH              = 20;
    parameter MAXSCYCLES            = 0;
    parameter MINDEPTH              = 0;
    localparam instance_plusarg     = { instance_name,  "_transmit_delay=%d" };
    localparam driver_randomization = { instance_name, "_randomization="};

    input              clk;
    input [WIDTH-1:0]  datain;

    output [WIDTH-1:0] dataout;
    wire [WIDTH-1:0]   dataout;

    reg [WIDTH-1:0]    fifo[MAXDEPTH-1:0];
    integer            scycles;
    reg [31:0]         global_seed;
    reg [31:0]         local_seed;
    integer            mtrand_handle;

    initial begin

    if (use_args == 1) begin
        if (!$value$plusargs("seed=%d", global_seed)) begin
            global_seed  = 0;
        end
        local_seed      = simple_hash_2(instance_name + global_seed);
        mtrand_handle      = $ati_new_mtrand(local_seed);

        if ($value$plusargs(instance_plusarg, scycles)) begin
            $display("%s: delay set to %0d cycles", instance_name, scycles);
        end else if ($test$plusargs({driver_randomization, "on"})) begin
            scycles = $ati_mtrand_int32(mtrand_handle) % MAXDEPTH;
            if (scycles < 0) scycles = 0;
            $display("%s: delay set randomly to %0d cycles", instance_name, scycles);
        end else if ($test$plusargs("ifrit_delay_random")) begin
            if  (MINDEPTH > 0) scycles = $ati_mtrand_int32_range(mtrand_handle, MINDEPTH, (MAXDEPTH - 32'd1));
            else               scycles = $ati_mtrand_int32      (mtrand_handle) % MAXDEPTH;

            if (scycles < 0) scycles = 0;
            $display("%s: delay set randomly to %0d cycles", instance_name, scycles);
        end else begin
            scycles = MAXSCYCLES;
        end
    end else begin // if (use_args == 1)
        scycles = MAXSCYCLES;
    end // else: !if(use_args == 1)

    end // initial begin

    integer           i;
    always @(negedge clk) begin
        for (i=MAXDEPTH-1;i>0;i=i-1) begin
            fifo[i] <= fifo[i-1];
        end
        fifo[0] <= datain;
    end

    assign dataout = (scycles == 0) ? datain : fifo[scycles - 1];

    function [31:0] simple_hash_2;
        input [100*8-1:0] str;
        integer           i;
        reg [7:0]         step;
        begin
            simple_hash_2 = 32'hdeadbeef;
            for(i=0;i<100;i=i+8) begin
                step = (str >> (i*8)) & 8'hff;
                simple_hash_2 = (   ( (simple_hash_2 & 32'hff000000) - ((step << 24) & 32'hff000000) )
                                  | ( (simple_hash_2 & 32'h00ff0000) + ((step << 16) & 32'h00ff0000) )
                                  | ( (simple_hash_2 & 32'h0000ff00) ^~((step <<  8) & 32'h0000ff00) )
                                  | ( (simple_hash_2 & 32'h000000ff) ^ ((step <<  0) & 32'h000000ff) )
                                  );
            end
        end
    endfunction

endmodule // ifrit_delay_neg

module ifrit_delay_pos(clk, datain, dataout);

    parameter instance_name          = "name";
    parameter WIDTH                  = 1;
    parameter use_args               = 0;
    parameter MAXDEPTH               = 20;
    parameter MAXSCYCLES             = 0;
    parameter MINDEPTH               = 0;
    localparam instance_plusarg      = { instance_name,  "_transmit_delay=%d" };
    localparam driver_randomization  = { instance_name, "_randomization="};

    input              clk;
    input [WIDTH-1:0]  datain;

    output [WIDTH-1:0] dataout;
    wire [WIDTH-1:0]   dataout;

    reg [WIDTH-1:0]    fifo[MAXDEPTH-1:0];
    integer        scycles;
    reg [31:0]     global_seed;
    reg [31:0]     local_seed;
    integer        mtrand_handle;

    initial begin

    if (use_args == 1) begin
        if (!$value$plusargs("seed=%d", global_seed)) begin
            global_seed = 0;
        end
        local_seed = simple_hash_2(instance_name + global_seed);
        mtrand_handle = $ati_new_mtrand(local_seed);
        
        if ($value$plusargs(instance_plusarg, scycles)) begin
            $display("%s: delay set to %0d cycles", instance_name, scycles);
        end else if ($test$plusargs({driver_randomization, "on"})) begin
            scycles = $ati_mtrand_int32(mtrand_handle) % MAXDEPTH;
            if (scycles < 0) scycles = 0;
            $display("%s: delay set randomly to %0d cycles", instance_name, scycles);
        end else if ($test$plusargs("ifrit_delay_random")) begin
            if  (MINDEPTH > 0) scycles = $ati_mtrand_int32_range(mtrand_handle, MINDEPTH, (MAXDEPTH - 32'd1));
            else               scycles = $ati_mtrand_int32      (mtrand_handle) % MAXDEPTH;

            if (scycles < 0) scycles = 0;
            $display("%s: delay set randomly to %0d cycles", instance_name, scycles);
        end else begin
            scycles  = MAXSCYCLES;
        end
    end else begin // if (use_args == 1)
        scycles = MAXSCYCLES;
    end // else: !if(use_args == 1)

    end // initial begin

    integer           i;
    always @(posedge clk) begin
        for (i=MAXDEPTH-1;i>0;i=i-1) begin
            fifo[i] <= fifo[i-1];
        end
        fifo[0] <= datain;
    end

    assign dataout = (scycles == 0) ? datain : fifo[scycles - 1];

    function [31:0] simple_hash_2;
        input [100*8-1:0] str;
        integer           i;
        reg [7:0]         step;
        begin
            simple_hash_2 = 32'hdeadbeef;
            for(i=0;i<100;i=i+8) begin
                step = (str >> (i*8)) & 8'hff;
                simple_hash_2 = ( (   (simple_hash_2 & 32'hff000000) - ((step << 24) & 32'hff000000) )
                                  | ( (simple_hash_2 & 32'h00ff0000) + ((step << 16) & 32'h00ff0000) )
                                  | ( (simple_hash_2 & 32'h0000ff00) ^~((step <<  8) & 32'h0000ff00) )
                                  | ( (simple_hash_2 & 32'h000000ff) ^ ((step <<  0) & 32'h000000ff) )
                                  );
            end
        end
    endfunction

endmodule // ifrit_delay_pos

module ifrit_delay(clk, datain, dataout);

    parameter instance_name     = "name";
    parameter WIDTH             = 1;
    parameter MAXDEPTH          = 20;
    parameter MAXSCYCLES        = 0;
    localparam instance_plusarg = { instance_name,  "_transmit_delay=%d" };
    localparam driver_randomization = { instance_name, "_randomization="};

    input              clk;
    input [WIDTH-1:0]  datain;

    output [WIDTH-1:0] dataout;
    wire [WIDTH-1:0]   dataout;

    reg [WIDTH-1:0]    fifo[MAXDEPTH-1:0];
    integer            scycles;
    reg [31:0]         global_seed;
    reg [31:0]         local_seed;
    integer            mtrand_handle;

    initial begin

        if (!$value$plusargs("seed=%d", global_seed)) begin
            global_seed = 0;
        end
        local_seed = simple_hash_2(instance_name + global_seed);
        mtrand_handle = $ati_new_mtrand(local_seed);

        if ($value$plusargs(instance_plusarg, scycles)) begin
            $display("%s: delay set to %0d cycles", instance_name, scycles);
        end else if ($test$plusargs({driver_randomization, "on"})) begin
            scycles = $ati_mtrand_int32(mtrand_handle) % MAXDEPTH;
            if (scycles < 0) scycles = 0;
            $display("%s: delay set randomly to %0d cycles", instance_name, scycles);
        end else if ($test$plusargs("ifrit_delay_random")) begin
            scycles = $ati_mtrand_int32(mtrand_handle) % MAXDEPTH;
            if (scycles < 0) scycles = 0;
            $display("%s: delay set randomly to %0d cycles", instance_name, scycles);
        end else begin
            scycles = MAXSCYCLES;
        end
    end // initial begin

    integer           i;
    always @(posedge clk) begin
        for (i=MAXDEPTH-1;i>0;i=i-1) begin
            fifo[i] <= fifo[i-1];
        end
        fifo[0] <= datain;
    end

    assign dataout = (scycles == 0) ? datain :
                     fifo[scycles - 1];

    function [31:0] simple_hash_2;
        input [100*8-1:0] str;
        integer           i;
        reg [7:0]         step;
        begin
            simple_hash_2 = 32'hdeadbeef;
            for(i=0;i<100;i=i+8) begin
                step = (str >> (i*8)) & 8'hff;
                simple_hash_2 = (   ( (simple_hash_2 & 32'hff000000) - ((step << 24) & 32'hff000000) )
                                  | ( (simple_hash_2 & 32'h00ff0000) + ((step << 16) & 32'h00ff0000) )
                                  | ( (simple_hash_2 & 32'h0000ff00) ^~((step <<  8) & 32'h0000ff00) )
                                  | ( (simple_hash_2 & 32'h000000ff) ^ ((step <<  0) & 32'h000000ff) )
                                  );
            end
        end
    endfunction

endmodule // ifrit_delay
