//Parameterized kc flop array wrapper 

//Set input and output pipeline mode
// Z type  #(.INPUT_IS_PIPELINED(0),.Q_IS_PIPELINED(0), .PIPEME_MODE(0)
// N type  #(.INPUT_IS_PIPELINED(1),.Q_IS_PIPELINED(0), .PIPEME_MODE(0))
// P type  #(.INPUT_IS_PIPELINED(1),.Q_IS_PIPELINED(1), .PIPEME_MODE(1))
// L type  #(.INPUT_IS_PIPELINED(1),.Q_IS_PIPELINED(1), .PIPEME_MODE(2)
// More detail:http://twiki.amd.com/twiki/bin/view/SOCDM/BuildMyRamManual#Pipeline_Modes 

//`include "gpu_features.vh"

module kc1r1w_flop #(parameter 
  DEPTH=1,               //address depth of flop array
  WIDTH=1,               //width of flop array
  WORDWIDTH=0,           //word width, set to 0 when no bit-mask, set to correct number for bit-mask wordwidth
  INPUT_IS_PIPELINED=0,
  Q_IS_PIPELINED=0,
  PIPEME_MODE=0
)(
  ADRA,
  ADRB,
  DA,
  WEA,
  CLKA,
  CLKB,
  RSTA,
  RSTB,
  MEA,
  MEB,
  WEMA,
  PIPEMEB,
  QB
);

localparam WORDS = WIDTH/(WORDWIDTH == 0 ? WIDTH : WORDWIDTH);
localparam LOGDEPTH  = ((DEPTH >4096) ? 13 :
     (DEPTH >2048) ? 12 :
     (DEPTH >1024) ? 11 :
     (DEPTH >512) ? 10 :
     (DEPTH >256) ? 9 :
     (DEPTH >128) ? 8 :
     (DEPTH > 64) ? 7 :
     (DEPTH > 32) ? 6 :
     (DEPTH > 16) ? 5 :
     (DEPTH >  8) ? 4 :
     (DEPTH >  4) ? 3 :
     (DEPTH >  2) ? 2 : 1);
localparam ADDR_WIDTH = LOGDEPTH;

input                 [LOGDEPTH-1:0] ADRA;
input                 [LOGDEPTH-1:0] ADRB;
input                    [WIDTH-1:0] DA;
input                                WEA;
input                                CLKA;
input                                CLKB;
input                                RSTA;
input                                RSTB;
input                                MEA;
input                                MEB;
input                    [WORDS-1:0] WEMA;
input                                PIPEMEB;
output logic             [WIDTH-1:0] QB;

logic                                CLK;
logic                                WE;
logic                                wrapper_WE;
logic                                wrapper_MEB;
logic                                wrapper_PIPEMEB;
logic                 [LOGDEPTH-1:0] wrapper_ADRA;
logic                 [LOGDEPTH-1:0] wrapper_ADRB;
logic                    [WORDS-1:0] wrapper_WEMA;
logic                    [WIDTH-1:0] wrapper_DA;
logic                    [WIDTH-1:0] internal_DA;
logic                    [WIDTH-1:0] QB_prelatch;
logic                    [WIDTH-1:0] QB_latched;
logic                    [WIDTH-1:0] QB_flopped;
logic                    [WIDTH-1:0] read_data;
logic         [WORDS-1:0][DEPTH-1:0] PgenWrite;
logic                    [WORDS-1:0] word_enable;
logic                    [DEPTH-1:0] WrSel;
logic                    [DEPTH-1:0] RdSel;

assign CLK = CLKA;
assign WE = WEA & MEA;

genvar k;

generate
  if (INPUT_IS_PIPELINED) begin:pipe_input
    `ifndef SYNTHESIS
        always @(posedge CLK) begin
          wrapper_DA <= WE ? DA : wrapper_DA;
          wrapper_WEMA <= WE ? WEMA : wrapper_WEMA;
          wrapper_ADRA <= WE ? ADRA : wrapper_ADRA;
          wrapper_ADRB <= MEB ? ADRB : wrapper_ADRB;
          wrapper_WE <= WE;
          wrapper_MEB <= MEB;
        end
    `else
        logic WE_CLK_out;
        logic MEB_CLK_out;
        kc_clock_gate clk_we (.clk_en(WE), .TE(1'b0), .clk_src(CLK), .clk_out(WE_CLK_out)); 
        kc_clock_gate clk_meb (.clk_en(MEB), .TE(1'b0), .clk_src(CLK), .clk_out(MEB_CLK_out)); 
        kc_dff #(WIDTH) da_dff (.q(wrapper_DA),.clk(WE_CLK_out),.d(DA),.SSE(1'b0));
        kc_dff #(WORDS) wema_dff (.q(wrapper_WEMA),.clk(WE_CLK_out),.d(WEMA),.SSE(1'b0));
        kc_dff #(LOGDEPTH) adra_dff (.q(wrapper_ADRA),.clk(WE_CLK_out),.d(ADRA),.SSE(1'b0));
        kc_dff #(LOGDEPTH) adrb_dff (.q(wrapper_ADRB),.clk(MEB_CLK_out),.d(ADRB),.SSE(1'b0));
        kc_dff #(1) we_dff (.q(wrapper_WE),.clk(CLK),.d(WE),.SSE(1'b0));
        kc_dff #(1) meb_dff (.q(wrapper_MEB),.clk(CLK),.d(MEB),.SSE(1'b0));
    `endif

    assign WrSel[DEPTH-1:0] = wrapper_WE ? ({{DEPTH-1{1'b0}}, 1'b1}<<wrapper_ADRA[ADDR_WIDTH-1:0]):{DEPTH{1'b0}};
    assign RdSel[DEPTH-1:0] = {{DEPTH-1{1'b0}}, 1'b1}<<wrapper_ADRB[ADDR_WIDTH-1:0];
    assign internal_DA = wrapper_DA;

  end 
  else begin:non_pipe_input
    assign WrSel[DEPTH-1:0] = WE ? ({{DEPTH-1{1'b0}}, 1'b1}<<ADRA[ADDR_WIDTH-1:0]):{DEPTH{1'b0}};
    assign RdSel[DEPTH-1:0] = {{DEPTH-1{1'b0}}, 1'b1} << ADRB[ADDR_WIDTH-1:0];
    assign internal_DA = DA;

  end

  if (Q_IS_PIPELINED) begin:pipe_output
    `ifndef SYNTHESIS
      always @(posedge CLK) begin
        QB_latched <= wrapper_MEB ? QB_prelatch:QB_latched;
        if (PIPEME_MODE == 1)
          QB_flopped <= PIPEMEB ? QB_latched:QB_flopped;
      end
    `else
      logic wrapper_MEB_CLK_out;
      logic PIPEMEB_CLK_out;
      kc_clock_gate clk_wrapper_meb (.clk_en(wrapper_MEB), .TE(1'b0), .clk_src(CLK), .clk_out(wrapper_MEB_CLK_out)); 
      kc_dff #(WIDTH) qb_latched_dff (.q(QB_latched),.clk(wrapper_MEB_CLK_out),.d(QB_prelatch),.SSE(1'b0));
      if (PIPEME_MODE == 1) begin
        kc_clock_gate clk_pipemeb (.clk_en(PIPEMEB), .TE(1'b0), .clk_src(CLK), .clk_out(PIPEMEB_CLK_out)); 
        kc_dff #(WIDTH) qb_dff (.q(QB_flopped),.clk(PIPEMEB_CLK_out),.d(QB_latched),.SSE(1'b0));
      end
    `endif

    if (PIPEME_MODE == 1)
      assign QB = QB_flopped;
    else if (PIPEME_MODE == 2)
      assign QB = QB_latched;

  end
  else begin:non_pipe_output
    assign QB = QB_prelatch;
  end


  if (WORDS>1 || (WORDS == 1 && WORDWIDTH > 0)) begin:bit_mask
    if (INPUT_IS_PIPELINED)
      assign word_enable[WORDS-1:0] = wrapper_WEMA[WORDS-1:0];
    else
      assign word_enable[WORDS-1:0] = WEMA[WORDS-1:0];

    logic         [WORDS-1:0][DEPTH-1:0] CLK_latch; 
    always_comb for (int i=0; i<WORDS; i++) for (int j=0; j<DEPTH; j++) PgenWrite[i][j] = WrSel[j] & word_enable[i];
    kc_fgcg  #(DEPTH) I_RCLK_AR[WORDS-1:0](.Q(CLK_latch),.CLK(CLK),.EN(PgenWrite));

    `ifndef SYNTHESIS
    kc_floparray #(
      .WIDTH(WORDWIDTH),
      .SIZE(DEPTH),
      .RDPORTS(1),
      .RDPARTS(1),
      .WRPORTS(1),
      .LOOPBACK(0),
      .CAMPORTS(0)
    `else
    kc_floparray_GV #(
      .WIDTH(WORDWIDTH),
      .SIZE(DEPTH),
      .RDPORTS(1),
      .RDPARTS(1),
      .WRPORTS(1),
      .LOOPBACK(0),
      .QUAD(1),
      .CAMPORTS(0),
      .READAOI(1),
      .FULLCAM(0)
    `endif
    ) floparray[WORDS-1:0] (
      .RCLK_AR(CLK_latch),
      .SSE(1'b0),
      .WrEn(1'b1),
      //.WrSel(WrSel[DEPTH-1:0]),
      .WrSel({DEPTH{1'b1}}),
      .WrData(internal_DA),
      .CamEn({(DEPTH*2){1'b0}}),
      .CamMask({(WORDWIDTH*2){1'b0}}),
      .CamData({(WORDWIDTH*2){1'b0}}),
      .RdSel(RdSel[DEPTH-1:0]),
      .RdData(QB_prelatch),
      .CamMatch(),
      .Data()
    );
  end
  else begin:no_bit_mask
    logic         [DEPTH-1:0] CLK_latch; 
    for (k=0; k<DEPTH; k++) begin:cg_loop
      `ifndef SYNTHESIS
        logic [DEPTH-1:0] WrSel_enable;
        always @ (CLK or WrSel[k])
          if (!CLK)
            WrSel_enable[k] <= WrSel[k];
        assign CLK_latch[k] = WrSel_enable[k] & CLK;
      `else
        kc_clock_gate clk_cg (.clk_en(WrSel[k]), .TE(1'b0), .clk_src(CLK), .clk_out(CLK_latch[k])); 
      `endif
    end

    `ifndef SYNTHESIS 
    kc_floparray #(
      .WIDTH(WIDTH),
      .SIZE(DEPTH),
      .RDPORTS(1),
      .RDPARTS(1),
      .WRPORTS(1),
      .LOOPBACK(0),
      .CAMPORTS(0)
    `else
    kc_floparray_GV #(
      .WIDTH(WIDTH),
      .SIZE(DEPTH),
      .RDPORTS(1),
      .RDPARTS(1),
      .WRPORTS(1),
      .LOOPBACK(0),
      .QUAD(1),
      .CAMPORTS(0),
      .READAOI(1),
      .FULLCAM(0)
    `endif
    ) floparray (
      .RCLK_AR(CLK_latch[DEPTH-1:0]),
      .SSE(1'b0),
      .WrEn(1'b1),
      //.WrSel(WrSel[DEPTH-1:0]),
      .WrSel({DEPTH{1'b1}}),
      .WrData(internal_DA),
      .CamEn({(DEPTH*2){1'b0}}),
      .CamMask({(WIDTH*2){1'b0}}),
      .CamData({(WIDTH*2){1'b0}}),
      .RdSel(RdSel[DEPTH-1:0]),
      .RdData(QB_prelatch),
      .CamMatch(),
      .Data()
    );

  end
endgenerate
endmodule

module kc_fgcg #(parameter DEPTH = 1) (
  input                       CLK,  // midgrain gated clock signal
  input           [DEPTH-1:0] EN, // fine grain enable
  output logic    [DEPTH-1:0] Q // gated clock
);
genvar i;
generate
  for (i=0; i<DEPTH; i++) begin:cg_loop
   `ifndef SYNTHESIS
     logic [DEPTH-1:0] EN_enable;
     always @ (CLK or EN[i])
       if (!CLK)
         EN_enable[i] <= EN[i];
     assign Q[i] = EN_enable[i] & CLK;

   `else
     kc_clock_gate clk_cg (.clk_en(EN[i]), .TE(1'b0), .clk_src(CLK), .clk_out(Q[i]));
   `endif
  end
endgenerate

endmodule


