////////////////////////////////////////////////////////////////////////////
/*
 *
 *  Trade secret of AMD, Inc.
 *  Copyright 2005, AMD, Inc., (unpublished)
 *
 *  All rights reserved.  This notice is intended as a precaution against
 *  inadvertent publication and does not imply publication or any waiver
 *  of confidentiality.  The year included in the foregoing notice is the
 *  year of creation of the work.
 *
 */
////////////////////////////////////////////////////////////////////////////
//
// Filename    : local_cac.v
// Description : This module considers CAC (power activity of GPU) activity indicators from several block instances and compresses them into one combined activity indicator.
//               block_id - identifies the target block: TD, TD, SQ, TCP, LDS, etc. Each of them has a unique ID.
//               signal_id - signal id number assigned to this type of activity signal in the lcac wrapper. this is used to identify which local cac
//           is been accessed by SMU.
//               activity  - it supports up to 32 instances. For each type of signal, one of this modules should be instantiated in the lcac wrapper
//          list of activity indicators from all instances of a particular block that are being combined in a tile
//          {
//        instance31_activity0,
//        ...
//        instance2_activity0,
//        instance1_activity0,
//        instance0_activity0
//          }
//      eg: one instance for: {LDS2_CAC_read_active, LDS1_CAC_read_active, LDS0_CAC_read_active}
//                      eg: one instance for: {LDS2_CAC_read_active, LDS1_CAC_read_active, LDS0_CAC_read_active}
//
//               activity_comb - combined activity indicator for a given activity signal type
//          {
//        combined_activity0
//          }
//
////////////////////////////////////////////////////////////////////////////
`include "grbm_defs.vh"
`include "gc__gc_cac_reg.v"

module local_cac_gc (
	resetb,
	clk,
	activity_signals,
	enable,
	threshold,
	ovr_sel,
	ovr_val,

	activity_comb
);

parameter ACT_WIDTH = 32;
parameter OVR_WIDTH = 32;

input                                     resetb;
input                                     clk;

input  [31:0]                             activity_signals;

input 			              enable;
input [`GC_CAC_CNTL_CAC_THRESHOLD_SIZE-1:0]  threshold;
input [OVR_WIDTH-1:0]         ovr_sel;
input [OVR_WIDTH-1:0]         ovr_val;

output reg                                activity_comb;

wire [31:0] activity_ovr;

//Set controls

assign activity_ovr = (activity_signals & (~ovr_sel)) | ( ovr_val & ovr_sel ) ;

//reg [5:0] num_active ;

wire activity_value;
assign activity_value = |activity_ovr;

reg activity_value_d;
always @(posedge clk) begin
	if(~resetb) begin
		activity_value_d <= 1'b0;
	end
	else if(~enable) begin
		activity_value_d <= 1'b0;
	end else begin
		activity_value_d <= activity_value;
	end
end

wire num_active_op;
assign num_active_op = activity_value_d || activity_value;

////Count number of active & inactive inputs from all instances per Activity type
//always @(posedge clk) begin
//    if (~resetb) begin
//        num_active  <= 6'h0;
//	end
//	else if(~enable ) begin
//        num_active  <= 6'h0;
//    end else if(num_active_op) begin
//        num_active  <= {5'b0,activity_ovr[0]}  + {5'b0,activity_ovr[1]}  + {5'b0,activity_ovr[2]}  + {5'b0,activity_ovr[3]}  +
//                       {5'b0,activity_ovr[4]}  + {5'b0,activity_ovr[5]}  + {5'b0,activity_ovr[6]}  + {5'b0,activity_ovr[7]}  +
//                       {5'b0,activity_ovr[8]}  + {5'b0,activity_ovr[9]}  + {5'b0,activity_ovr[10]} + {5'b0,activity_ovr[11]} +
//                       {5'b0,activity_ovr[12]} + {5'b0,activity_ovr[13]} + {5'b0,activity_ovr[14]} + {5'b0,activity_ovr[15]} +
//                       {5'b0,activity_ovr[16]} + {5'b0,activity_ovr[17]} + {5'b0,activity_ovr[18]} + {5'b0,activity_ovr[19]} +
//                       {5'b0,activity_ovr[20]} + {5'b0,activity_ovr[21]} + {5'b0,activity_ovr[22]} + {5'b0,activity_ovr[23]} +
//                       {5'b0,activity_ovr[24]} + {5'b0,activity_ovr[25]} + {5'b0,activity_ovr[26]} + {5'b0,activity_ovr[27]} +
//                       {5'b0,activity_ovr[28]} + {5'b0,activity_ovr[29]} + {5'b0,activity_ovr[30]} + {5'b0,activity_ovr[31]};
//    end // else: !if(~resetb | ~enable )
//end // always @ (posedge clk)


   localparam WIDTH_LOG2 = ACT_WIDTH > 31 ? 6 : ACT_WIDTH > 15 ? 5 : ACT_WIDTH > 7 ? 4 : ACT_WIDTH > 3 ? 3 : ACT_WIDTH >1 ? 2 : 1;
   reg [WIDTH_LOG2 - 1 :0] num_active_t ;
   reg [WIDTH_LOG2 - 1 :0] num_active ;

  integer i; 
   always @*
     begin
        num_active_t = 'd0;
        for (i = 0; i < ACT_WIDTH; i=i+1) begin
           if(num_active_op)
              num_active_t = num_active_t + activity_ovr[i];
           else
              num_active_t = num_active_t;
          end
     end

   always @(posedge clk) begin
     if (~resetb) begin
        num_active <= 'd0;
     end else begin
       if (enable) begin 
            num_active <= num_active_t;           
       end else begin
             num_active <= 'd0;
       end
   	end
  end



wire  [19:0] tot_active; //1 more bit for potential overflow
reg   [18:0] rlo_active;

wire rlo_active_op;
//assign rlo_active_op = |activity_ovr;
//
//reg  rlo_active_op_q;
//always @(posedge clk) begin
//	if(~resetb | ~enable) begin
//		rlo_active_op_q <= 0;
//	end else begin
//		rlo_active_op_q <= rlo_active_op;
//	end
//end
assign rlo_active_op = activity_value_d;

assign tot_active = rlo_active + {13'h0, num_active};

wire[19:0] tot_active_sub = tot_active - {{(20-`GC_CAC_CNTL_CAC_THRESHOLD_SIZE){1'b0}}, threshold};

always @(posedge clk) begin
    if (~resetb) begin
        rlo_active   <= 19'h0;
	end
	else if(~enable ) begin
        rlo_active   <= 19'h0;
    end else  begin
		if(!rlo_active_op & (tot_active < {{(20-`GC_CAC_CNTL_CAC_THRESHOLD_SIZE){1'b0}}, threshold}))
			rlo_active <= rlo_active;
		else
        rlo_active <=  ( tot_active < {{(20-`GC_CAC_CNTL_CAC_THRESHOLD_SIZE){1'b0}}, threshold} ) ?  tot_active[18:0]  : (tot_active_sub[19] ? 19'h7ffff : tot_active_sub[18:0]);
    	end 
end

//FIXME: below legacy code, was comparing tot_active[18:0] with threshold[15:0]
//Assert combined active signals per Activity type when the total(actives+rollover) exceeds the threshold
always @(posedge clk) begin
    if (~resetb) begin
        activity_comb <= 1'b0;
	end
	else if(~enable ) begin
        activity_comb <= 1'b0;
    end else if(~(~resetb | ~enable)) begin
        activity_comb <= (tot_active < {{(20-`GC_CAC_CNTL_CAC_THRESHOLD_SIZE){1'b0}}, threshold}) ? 1'b0 : 1'b1;
    end
end

endmodule
