//==================================================================================================
//
// madroopdetect.v
// 
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights
// reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any
// form, or by any means manual, electric, electronic, mechanical,
// electro-magnetic, chemical, optical, or otherwise, without prior
// explicit written permission from Advanced Micro Devices, Inc.
//
//==================================================================================================
//
// INTERESTED PARTIES
//
// Circuit              Ju Tung Ng
// RTL                  Ricky Lau 
// Verification         Sean Sequeira 
//
//==================================================================================================
//
// Module Description
//
// Voltage droop detect block used in discrete gpu Volcanic Islands products
//
//==================================================================================================

`timescale 1ps/1ps

module madroopdetect (

input           VDD,
input           VSS,

input           ENABLE_DD,
input           STRETCH_ENABLE,
input           staticEnable,
input           masterReset,
input           dfs_clk,
input     [1:0] CONSERVATIVE_LOCK,
input     [6:0] LOW_BW_CONFIG,
input     [6:0] LOW_BW_CONFIG_UP,
input     [2:0] LOW_BW_SKEW_EN,
input     [2:0] STRETCH_THRESH,
input     [3:0] FSM_WAIT_CYCLES,    // New: number of cycles to wait after each delay change
input     [15:0] RESERVE,            // [7:0]=not used
input     [3:0] SAMPLE_SIZE,           // New: Number of samples to take to determine Lead/Lag
input           NO_EXTRA_COARSE_STEP, // New: If 1, FSM will increase delay by one coarse step going into FINE_DEC state.  Default = 1.  Default = 0.
input     [1:0] FREQ_MODE,          // Selects the frequency range supported
input     [1:0] PD_SYNC_SEL,        // Selects the level of synchronization from PD output

input     [3:0] CHARZ_SEL_A,
input     [3:0] CHARZ_SEL_B,
input     [1:0] CHARZ_DIV_A,
input     [1:0] CHARZ_DIV_B,
input     [1:0] FSM_STEP_SIZE,
input     [1:0] FSM_CODE_MARGIN,
input           CHARZ_DEBUG_MODE,
input     [1:0] FSM_DEBUG_SEL,
input           FSM_OVERRIDE_EN,
input     [3:0] FSM_OVERRIDE_VAL,

input     [4:0] UNSTRETCH_THRESH,        // Specifies trigger threshold when DFSSTRETCH=1. (FREQ_MODE=10, CPU only)
input           DFSSTRETCH,              // 1 means dfs_clk is stretched    
input     [3:0] DFSSTRETCH_AMOUNT,       // specifies stretch percentage seen on dfs_clk. (FREQ_MODE=10, CPU only)
input           CT_CpuPwrOkVDD,          // Isolation for the dfs_clk to be usd by CPU else connected to 1'b1 at higher level

output          charz_observ_a,
output          charz_observ_b,

output wire     droopdetected,
output wire     dd_dll_lock,
output wire [7:0] phase_lead_lag,
output    [3:0] debug_c0,
output    [3:0] debug_c19,
output    [3:0] debug_c29,
output    [3:0] debug_c39
);

`ifndef MACRO_HEADER_ONLY

`ifndef HWEMUL

wire    [3:0] c0P;
wire    [3:0] c1P;
wire    [3:0] c2P;
wire    [3:0] c3P;
wire    [3:0] c4P;
wire    [3:0] c5P;
wire    [3:0] c6P;
wire    [3:0] c7P;
wire    [3:0] c8P;
wire    [3:0] c9P;
wire    [3:0] c10P;
wire    [3:0] c11P;
wire    [3:0] c12P;
wire    [3:0] c13P;
wire    [3:0] c14P;
wire    [3:0] c15P;
wire    [3:0] c16P;
wire    [3:0] c17P;
wire    [3:0] c18P;
wire    [3:0] c19P;
wire    [3:0] c20P;
wire    [3:0] c21P;
wire    [3:0] c22P;
wire    [3:0] c23P;
wire    [3:0] c24P;
wire    [3:0] c25P;
wire    [3:0] c26P;
wire    [3:0] c27P;
wire    [3:0] c28P;
wire    [3:0] c29P;
wire    [3:0] c30P;
wire    [3:0] c31P;
wire    [3:0] c32P;
wire    [3:0] c33P;
wire    [3:0] c34P;
wire    [3:0] c35P;
wire    [3:0] c36P;
wire    [3:0] c37P;
wire    [3:0] c38P;
wire    [3:0] c39P;

wire            lead_ph2840;
wire            lead_ph2739;
wire            lead_ph2638;
wire            lead_ph2537;
wire            lead_ph2436;
wire            lead_ph2335;
wire            lead_ph2234;
wire            lead_ph2133;
wire             lag_ph2840;
wire             lag_ph2739;
wire             lag_ph2638;
wire             lag_ph2537;
wire             lag_ph2436;
wire             lag_ph2335;
wire             lag_ph2234;
wire             lag_ph2133;

wire            dfs_clk_gated;

wire            test_clk_ref;
wire            test_clk_fb40;
wire    [1:0]   test_clk_sel;


//==================================================================
mbdroop_sapr_top I_sapr_top (
        .VDD (VDD), 
        .VSS (VSS),
        
        .dfs_clk (dfs_clk),
        .ENABLE_DD (ENABLE_DD),
        .STRETCH_ENABLE (STRETCH_ENABLE),
        .staticEnable (staticEnable),
        .masterReset (masterReset),
        
        .STRETCH_THRESH (STRETCH_THRESH),
        .CONSERVATIVE_LOCK (CONSERVATIVE_LOCK),
        .LOW_BW_CONFIG (LOW_BW_CONFIG),
        .LOW_BW_CONFIG_UP (LOW_BW_CONFIG_UP),
        .LOW_BW_SKEW_EN (LOW_BW_SKEW_EN),
        .FSM_WAIT_CYCLES (FSM_WAIT_CYCLES),
        .NO_EXTRA_COARSE_STEP (NO_EXTRA_COARSE_STEP),
        .SAMPLE_SIZE (SAMPLE_SIZE),
        .FREQ_MODE (FREQ_MODE),
        .PD_SYNC_SEL (PD_SYNC_SEL),
        .RESERVE (RESERVE),
        
        .CHARZ_SEL_A(CHARZ_SEL_A),
        .CHARZ_SEL_B(CHARZ_SEL_B),
        .CHARZ_DIV_A(CHARZ_DIV_A),
        .CHARZ_DIV_B(CHARZ_DIV_B),
        .FSM_STEP_SIZE(FSM_STEP_SIZE),
        .FSM_CODE_MARGIN(FSM_CODE_MARGIN),
        .CHARZ_DEBUG_MODE(CHARZ_DEBUG_MODE),
        .FSM_DEBUG_SEL(FSM_DEBUG_SEL),
        .FSM_OVERRIDE_EN(FSM_OVERRIDE_EN),
        .FSM_OVERRIDE_VAL(FSM_OVERRIDE_VAL),
        .test_clk_ref(test_clk_ref),
        .test_clk_fb40(test_clk_fb40),

        .UNSTRETCH_THRESH (UNSTRETCH_THRESH),
        .DFSSTRETCH (DFSSTRETCH),
        .DFSSTRETCH_AMOUNT (DFSSTRETCH_AMOUNT),
        .CT_CpuPwrOkVDD    (CT_CpuPwrOkVDD), 
        .charz_observ_a(charz_observ_a),
        .charz_observ_b(charz_observ_b),
        .test_clk_sel(test_clk_sel),
        .phase_lead_lag(phase_lead_lag),
        .lead_ph40_mf (lead_ph40_mf),
        .lead_ph39_mf (lead_ph39_mf),
        .lead_ph38_mf (lead_ph38_mf),
        .lead_ph37_mf (lead_ph37_mf),
        .lead_ph36_mf (lead_ph36_mf),
        .lead_ph35_mf (lead_ph35_mf),
        .lead_ph34_mf (lead_ph34_mf),
        .lead_ph33_mf (lead_ph33_mf),
        .lead_ph50_hf (lead_ph50_hf),
        .lead_ph49_hf (lead_ph49_hf),
        .lead_ph48_hf (lead_ph48_hf),
        .lead_ph47_hf (lead_ph47_hf),
        .lead_ph46_hf (lead_ph46_hf),
        .lead_ph45_hf (lead_ph45_hf),
        .lead_ph44_hf (lead_ph44_hf),
        .lead_ph43_hf (lead_ph43_hf),
        .lead_ph42_hf (lead_ph42_hf),
        .lead_ph41_hf (lead_ph41_hf),
        .lead_ph40_hf (lead_ph40_hf),
        .lead_ph39_hf (lead_ph39_hf),
        .lead_ph38_hf (lead_ph38_hf),
        .lead_ph37_hf (lead_ph37_hf),
        .lead_ph36_hf (lead_ph36_hf),
        .lead_ph35_hf (lead_ph35_hf),
        .lead_ph34_hf (lead_ph34_hf),
        .lead_ph33_hf (lead_ph33_hf),
        
        .c0P (c0P),
        .c1P (c1P),
        .c2P (c2P),
        .c3P (c3P),
        .c4P (c4P),
        .c5P (c5P),
        .c6P (c6P),
        .c7P (c7P),
        .c8P (c8P),
        .c9P (c9P),
        .c10P (c10P),
        .c11P (c11P),
        .c12P (c12P),
        .c13P (c13P),
        .c14P (c14P),
        .c15P (c15P),
        .c16P (c16P),
        .c17P (c17P),
        .c18P (c18P),
        .c19P (c19P),
        .c20P (c20P),
        .c21P (c21P),
        .c22P (c22P),
        .c23P (c23P),
        .c24P (c24P),
        .c25P (c25P),
        .c26P (c26P),
        .c27P (c27P),
        .c28P (c28P),
        .c29P (c29P),
        .c30P (c30P),
        .c31P (c31P),
        .c32P (c32P),
        .c33P (c33P),
        .c34P (c34P),
        .c35P (c35P),
        .c36P (c36P),
        .c37P (c37P),
        .c38P (c38P),
        .c39P (c39P),
 
        .dd_dll_lock (dd_dll_lock),
        .debug_c0 (debug_c0),
        .debug_c19 (debug_c19),
        .debug_c29 (debug_c29),
        .debug_c39 (debug_c39),
        
        .droopdetected (droopdetected),
        .dfs_clk_gated_mf (dfs_clk_gated_mf),
        .dfs_clk_gated_hf (dfs_clk_gated_hf)
        );


//==================================================================
clkds_dd_delaychain_top I_delaychain_top (
        // INPUTS
        .ckin_mf   (dfs_clk_gated_mf),
        .ckin_hf   (dfs_clk_gated_hf),

        .c0P  (c0P),
        .c1P  (c1P),
        .c2P  (c2P),
        .c3P  (c3P),
        .c4P  (c4P),
        .c5P  (c5P),
        .c6P  (c6P),
        .c7P  (c7P),
        .c8P  (c8P),
        .c9P  (c9P),
        .c10P (c10P),
        .c11P (c11P),
        .c12P (c12P),
        .c13P (c13P),
        .c14P (c14P),
        .c15P (c15P),
        .c16P (c16P),
        .c17P (c17P),
        .c18P (c18P),
        .c19P (c19P),
        .c20P (c20P),
        .c21P (c21P),
        .c22P (c22P),
        .c23P (c23P),
        .c24P (c24P),
        .c25P (c25P),
        .c26P (c26P),
        .c27P (c27P),
        .c28P (c28P),
        .c29P (c29P),
        .c30P (c30P),
        .c31P (c31P),
        .c32P (c32P),
        .c33P (c33P),
        .c34P (c34P),
        .c35P (c35P),
        .c36P (c36P),
        .c37P (c37P),
        .c38P (c38P),
        .c39P (c39P),
        .test_clk_sel (test_clk_sel),
        // OUTPUTS
        .test_clk_ref (test_clk_ref),
        .test_clk_fb40(test_clk_fb40),
                 
        .lead_ph40_mf (lead_ph40_mf),           
        .lead_ph39_mf (lead_ph39_mf),           
        .lead_ph38_mf (lead_ph38_mf),           
        .lead_ph37_mf (lead_ph37_mf),           
        .lead_ph36_mf (lead_ph36_mf),           
        .lead_ph35_mf (lead_ph35_mf),           
        .lead_ph34_mf (lead_ph34_mf),           
        .lead_ph33_mf (lead_ph33_mf),
        
        .lead_ph50_hf (lead_ph50_hf),           
        .lead_ph49_hf (lead_ph49_hf),           
        .lead_ph48_hf (lead_ph48_hf),           
        .lead_ph47_hf (lead_ph47_hf),           
        .lead_ph46_hf (lead_ph46_hf),           
        .lead_ph45_hf (lead_ph45_hf),           
        .lead_ph44_hf (lead_ph44_hf),           
        .lead_ph43_hf (lead_ph43_hf),           
        .lead_ph42_hf (lead_ph42_hf),           
        .lead_ph41_hf (lead_ph41_hf),           
                   
        .lead_ph40_hf (lead_ph40_hf),           
        .lead_ph39_hf (lead_ph39_hf),           
        .lead_ph38_hf (lead_ph38_hf),           
        .lead_ph37_hf (lead_ph37_hf),           
        .lead_ph36_hf (lead_ph36_hf),           
        .lead_ph35_hf (lead_ph35_hf),           
        .lead_ph34_hf (lead_ph34_hf),           
        .lead_ph33_hf (lead_ph33_hf),           

        // PWR
        .VDD    (VDD),
        .VSS    (VSS)
);


//====================================================================
cap_madroopdetect I_toplevel_cap (
        .VDD            (VDD),
        .VSS            (VSS)
);


//==================================================================
//==================================================================
//====================================================================
// Assertions
`ifdef ABV
`include "mbdroop_assertions.v"
`endif //ABV

`else

assign droopdetected = 1'b0;
assign dd_dll_lock = 1'b0;

assign debug_c0 = 4'd0;
assign debug_c19 = 4'd0;
assign debug_c29 = 4'd0;
assign debug_c39 = 4'd0;

`endif  // HWEMUL

`endif   // MACRO_HEADER_ONLY

endmodule // madroopdetect
