
// We have a special asynchronous version of the flop memory.  We 
// need this so the PD tools can set special constraints for the 
// asynchronous flop paths.

// these params are for the CDC compliance.   The first 6 go into the techind_cdcefpm module
// the last one, NotCdcCompliant controls whether CDC buffers are instantiated.
// Use these to change the defaults for all the async flop arrays.


module mem2p_flop
  (
   /*AUTOARG*/
   // Outputs
   QB, 
   // Inputs
   ADRA, ADRB, DA, WEA, CLKA, CLKB, RSTA, RSTB, MEA, MEB, WEMA, 
   PIPEMEB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=1;
   parameter Q_IS_PIPELINED=0;
   parameter PIPEME_MODE=0;
   parameter INPUT_IS_PIPELINED=1;
   localparam LOGDEPTH  = ((DEPTH >4096) ? 13 :
        (DEPTH >2048) ? 12 :
        (DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);
   
   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;
   input [WIDTH/(WORDWIDTH == 0 ? WIDTH : WORDWIDTH)-1:0] WEMA;
   input           PIPEMEB;

// river translate_off
   wire WE;
   reg      WE_q, MEB_q, PIPEMEB_q;
   reg [LOGDEPTH-1:0]   ADRA_q, ADRB_q;
   reg [WIDTH/(WORDWIDTH == 0 ? WIDTH : WORDWIDTH)-1:0] WEMA_q;
   reg [WIDTH-1:0]  DA_q;

   reg [WIDTH-1:0]  data[0:DEPTH-1];

   integer    i;

   assign WE = WEA & MEA;
   always @(posedge CLKA) begin
      WE_q <= WE;
      if (WE) WEMA_q <= WEMA;
      if (WE) DA_q <= DA;
      if (WE) ADRA_q <= ADRA;


      if (INPUT_IS_PIPELINED ? WE_q : WE) begin
        if (WORDWIDTH == 0) begin
          if (INPUT_IS_PIPELINED)
            data[ADRA_q] <= DA_q;
          else
            data[ADRA] <= DA;
        end 
        else begin
          for (i = 0; i < WIDTH; i=i+1) begin
            if (INPUT_IS_PIPELINED ? WEMA_q[i/WORDWIDTH] : WEMA[i/WORDWIDTH]) begin
              if (INPUT_IS_PIPELINED) 
                data[ADRA_q][i] <= DA_q[i];
              else
                data[ADRA][i] <= DA[i];
            end
          end
        end 
      end
   end

   reg  [WIDTH-1:0] QB_flopped;
   reg  [WIDTH-1:0] read_data_latched;

   wire [WIDTH-1:0] read_data_pre_latch = data[INPUT_IS_PIPELINED ? ADRB_q : ADRB];

   wire [WIDTH-1:0] read_data = INPUT_IS_PIPELINED ? (MEB_q ? read_data_pre_latch : read_data_latched) : read_data_pre_latch;
   
   always @(posedge CLKB) begin
      MEB_q <= MEB;
      PIPEMEB_q <= PIPEMEB;

      if(MEB)
        ADRB_q <= ADRB;

      if(MEB_q)
        read_data_latched <= read_data_pre_latch;

      if ((PIPEME_MODE == 2) ? PIPEMEB_q : PIPEMEB)
        QB_flopped <= read_data;

   end

   assign QB = (Q_IS_PIPELINED) ? ((PIPEME_MODE > 0) ? QB_flopped : read_data_latched) : read_data;
// river translate_on

endmodule


module mem2pa_flop
  (
   /*AUTOARG*/
   // Outputs
   QB, 
   // Inputs
   ADRA, ADRB, DA, WEA, CLKA, CLKB, RSTA, RSTB, MEA, MEB, WEMA, 
   PIPEMEB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=1;
   parameter Q_IS_PIPELINED=0;
   parameter PIPEME_MODE=0;
   parameter INPUT_IS_PIPELINED=1;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >4096) ? 13 :
        (DEPTH >2048) ? 12 :
        (DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);
   
   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;
   input [WIDTH/(WORDWIDTH == 0 ? WIDTH : WORDWIDTH)-1:0] WEMA;
   input           PIPEMEB;

// river translate_off
   wire WE;
   reg      WE_q, MEB_q, PIPEMEB_q;
   reg [LOGDEPTH-1:0]   ADRA_q, ADRB_q;
   reg [WIDTH/(WORDWIDTH == 0 ? WIDTH : WORDWIDTH)-1:0] WEMA_q;
   reg [WIDTH-1:0]  DA_q;

   reg [WIDTH-1:0]  CDCSE_data[0:DEPTH-1];

   integer    i;

   assign WE = WEA & MEA;
   always @(posedge CLKA) begin
      WE_q <= WE;
      if (WE) WEMA_q <= WEMA;
      if (WE) DA_q <= DA;
      if (WE) ADRA_q <= ADRA;


      if (INPUT_IS_PIPELINED ? WE_q : WE) begin
        if (WORDWIDTH == 0) begin
          if (INPUT_IS_PIPELINED)
            CDCSE_data[ADRA_q] <= DA_q;
          else
            CDCSE_data[ADRA] <= DA;
        end 
        else begin
          for (i = 0; i < WIDTH; i=i+1) begin
            if (INPUT_IS_PIPELINED ? WEMA_q[i/WORDWIDTH] : WEMA[i/WORDWIDTH]) begin
              if (INPUT_IS_PIPELINED) 
                CDCSE_data[ADRA_q][i] <= DA_q[i];
              else
                CDCSE_data[ADRA][i] <= DA[i];
            end
          end
        end 
      end
   end

   reg  [WIDTH-1:0] QB_flopped;
   reg  [WIDTH-1:0] read_data_latched;

   reg  [WIDTH-1:0] read_data_pre_latch;

   // cdc buffers - one per bit per register
   generate 
     if (NotCdcCompliant) begin
       always @(*) begin
         read_data_pre_latch = CDCSE_data[INPUT_IS_PIPELINED ? ADRB_q : ADRB];
       end
     end else begin
       genvar j;
       wire [WIDTH-1:0] read_data_pre[0:DEPTH-1];

       // read enables - one per register location
       wire [DEPTH-1:0] rden = INPUT_IS_PIPELINED ? (MEB_q << ADRB_q) :
                                                    (MEB << ADRB);

       for (j=0; j<DEPTH; j=j+1) begin :cdcbuf
         techind_cdcefpm #(.size         (WIDTH),
                           .applyParams  (applyParams),
                           .writeParams  (writeParams),
                           .CdcDestSetup (CdcDestSetup),
                           .DestClkCnt   (DestClkCnt),
                           .AssertEnable (AssertEnable),
                           .driveSel     (driveSel))
           U_cdc          (.DSTCLK(CLKB),
                           .A(CDCSE_data[j]),
                           .Z(read_data_pre[j]),
                           .EN(rden[j]));
       end

       // OR the read data together
       integer     rdnum;
       always @(*) begin
         read_data_pre_latch = 0;
         for (rdnum=0; rdnum<DEPTH; rdnum= rdnum+1)
           read_data_pre_latch = read_data_pre_latch | read_data_pre[rdnum]; // lint ignore RTL7.3
       end
     end
   endgenerate


   wire [WIDTH-1:0] read_data = INPUT_IS_PIPELINED ? (MEB_q ? read_data_pre_latch : read_data_latched) : read_data_pre_latch;
   
   always @(posedge CLKB) begin
      MEB_q <= MEB;
      PIPEMEB_q <= PIPEMEB;

      if(MEB)
        ADRB_q <= ADRB;

      if(MEB_q)
        read_data_latched <= read_data_pre_latch;

      if ((PIPEME_MODE == 2) ? PIPEMEB_q : PIPEMEB)
        QB_flopped <= read_data;

   end

   assign QB = (Q_IS_PIPELINED) ? ((PIPEME_MODE > 0) ? QB_flopped : read_data_latched) : read_data;
// river translate_on

endmodule







module mem1pn_flop
  (
   CLK, D, Q, ADR, ME, WE,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;


  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem1pbn_flop
  (
   CLK, D, Q, ADR, ME, WE,
   WEM,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;

   input [WIDTH/WORDWIDTH-1:0] WEM;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(WEM),
      .PIPEMEB(1'b1),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem2pn_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;


  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pan_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;


  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pbn_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pabn_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;

  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem1pp_flop
  (
   CLK, D, Q, ADR, ME, WE,
   PIPEME,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;

   input PIPEME;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(1),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(1'b1),
      .PIPEMEB(PIPEME),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem1pbp_flop
  (
   CLK, D, Q, ADR, ME, WE,
   WEM,
   PIPEME,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;

   input [WIDTH/WORDWIDTH-1:0] WEM;
   input PIPEME;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(1),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(WEM),
      .PIPEMEB(PIPEME),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem2pp_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   PIPEMEB,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input    PIPEMEB;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(1),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(PIPEMEB),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pap_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   PIPEMEB,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input    PIPEMEB;

  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(1),
      .INPUT_IS_PIPELINED(1),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(PIPEMEB),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pbp_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   PIPEMEB,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;
   input    PIPEMEB;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(1),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(PIPEMEB),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pabp_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   PIPEMEB,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;
   input    PIPEMEB;

  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(1),
      .INPUT_IS_PIPELINED(1),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(PIPEMEB),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem1pq_flop
  (
   CLK, D, Q, ADR, ME, WE,
   PIPEME,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;

   input PIPEME;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(2),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(1'b1),
      .PIPEMEB(PIPEME),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem1pbq_flop
  (
   CLK, D, Q, ADR, ME, WE,
   WEM,
   PIPEME,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;

   input [WIDTH/WORDWIDTH-1:0] WEM;
   input PIPEME;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(2),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(WEM),
      .PIPEMEB(PIPEME),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem2pq_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   PIPEMEB,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input    PIPEMEB;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(2),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(PIPEMEB),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2paq_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   PIPEMEB,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input    PIPEMEB;

  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(2),
      .INPUT_IS_PIPELINED(1),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(PIPEMEB),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pbq_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   PIPEMEB,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;
   input    PIPEMEB;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(2),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(PIPEMEB),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pabq_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   PIPEMEB,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;
   input    PIPEMEB;

  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(2),
      .INPUT_IS_PIPELINED(1),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(PIPEMEB),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem1pr_flop
  (
   CLK, D, Q, ADR, ME, WE,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;


  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem1pbr_flop
  (
   CLK, D, Q, ADR, ME, WE,
   WEM,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;

   input [WIDTH/WORDWIDTH-1:0] WEM;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(WEM),
      .PIPEMEB(1'b1),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem2pr_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;


  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2par_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;


  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pbr_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pabr_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;

  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(1),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(1),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem1pz_flop
  (
   CLK, D, Q, ADR, ME, WE,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;


  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(0)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem1pbz_flop
  (
   CLK, D, Q, ADR, ME, WE,
   WEM,
   RST
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] Q;
   input [LOGDEPTH-1:0] ADR;
   input [WIDTH-1:0]  D;
   input    WE;
   input    CLK, RST;
   input    ME;

   input [WIDTH/WORDWIDTH-1:0] WEM;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(0)
      ) core
     (
      .CLKA(CLK),
      .CLKB(CLK),
      .QB(Q),
      .ADRA(ADR),
      .ADRB(ADR),
      .DA(D),
      .MEA(ME),
      .MEB(ME),
      .WEA(WE),
      .WEMA(WEM),
      .PIPEMEB(1'b1),
      .RSTA(RST),
      .RSTB(RST)
);

endmodule




module mem2pz_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;


  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(0)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2paz_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;


  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(0),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(1'b1),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pbz_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;

  mem2p_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(0)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule



module mem2pabz_flop
  (
   CLKA, CLKB, DA, QB, ADRA, ADRB, MEA, MEB, WEA,
   WEMA,
   RSTA, RSTB
   );

   parameter DEPTH=2;
   parameter WIDTH=1;
   parameter WORDWIDTH=WIDTH;
   parameter applyParams  = 0;
   parameter writeParams  = 1;
   parameter CdcDestSetup = 11000;
   parameter DestClkCnt   = 0;
   parameter AssertEnable = 1;
   parameter driveSel     = 3;
   parameter NotCdcCompliant = 1;
   localparam LOGDEPTH  = ((DEPTH >1024) ? 11 :
        (DEPTH >512) ? 10 :
        (DEPTH >256) ? 9 :
        (DEPTH >128) ? 8 :
        (DEPTH > 64) ? 7 :
        (DEPTH > 32) ? 6 :
        (DEPTH > 16) ? 5 :
        (DEPTH >  8) ? 4 :
        (DEPTH >  4) ? 3 :
        (DEPTH >  2) ? 2 : 1);

   output [WIDTH-1:0] QB;
   input [LOGDEPTH-1:0] ADRA, ADRB;
   input [WIDTH-1:0]  DA;
   input    WEA;
   input    CLKA, CLKB, RSTA, RSTB;
   input    MEA, MEB;

   input [WIDTH/WORDWIDTH-1:0] WEMA;

  mem2pa_flop
    #(.DEPTH(DEPTH),
      .WIDTH(WIDTH),
      .WORDWIDTH(WORDWIDTH),
      .Q_IS_PIPELINED(0),
      .PIPEME_MODE(0),
      .INPUT_IS_PIPELINED(0),
      .applyParams  (applyParams),
      .writeParams  (writeParams),
      .CdcDestSetup (CdcDestSetup),
      .DestClkCnt   (DestClkCnt),
      .AssertEnable (AssertEnable),
      .driveSel     (driveSel),
      .NotCdcCompliant(NotCdcCompliant)
      ) core
     (
      .CLKA(CLKA),
      .CLKB(CLKB),
      .QB(QB),
      .ADRA(ADRA),
      .ADRB(ADRB),
      .DA(DA),
      .MEA(MEA),
      .MEB(MEB),
      .WEA(WEA),
      .WEMA(WEMA),
      .PIPEMEB(1'b1),
      .RSTA(RSTA),
      .RSTB(RSTB)
);

endmodule

