//*************************************************************************************************
//
// FILE_OWNER: james.mossman@amd.com
//
//  Copyright  2013 Advanced Micro Devices, Inc.
//  All rights reserved.
//
//  This file is protected by Federal Copyright Law, with all rights
//  reserved. No part of this file may be reproduced, stored in a
//  retrieval system, translated, transcribed, or transmitted, in any
//  form, or by any means manual, electric, electronic, mechanical,
//  electro-magnetic, chemical, optical, or otherwise, without prior
//  explicit written permission from Advanced Micro Devices, Inc.
//
//*************************************************************************************************
// **************************************************************************************************
//                       Zen RTL design file
// **************************************************************************************************
//                  AMD PROPRIETARY AND CONFIDENTIAL
//               (c) 2013 Advanced Micro Devices, Inc.
// **************************************************************************************************
// Module Description:
//
// Power monitor & management (M&M) bus stop:
//
// The power monitor & management bus (M&M) is a 8-bit bus that winds its way through the core & L2,
// to support direct communication of each core+L2 with SOC devices, such as the system management unit
// (SMU). The primary anticipated function of this bus is to expedite power management requests to
// and from the core+L2. The M&M bus is a single-master bus, with the master controller being in the
// part of the core pervasive logic (CPL) that is in the same voltage domain (VDDCORE) as core+L2.
// This is referrred to as the CPLB block. The M&M bus is similar to the SRB bus in that it has a
// single-master, and connects each core+L2 block that has a stop, via a ring. The M&M bus on Zen
// replaces and consolidates functionality of the SPMI bus, and the 24 x 1-bit DPM point-to-point
// busses used to transfer Cac values to the foundation unit (FU) in Excavator (XV).

`timescale 1ps / 1ps

`include "mm_assert_control.d"
`include "mm_global.d"

// **************************************************************************************************
// Transactions
// **************************************************************************************************
// Bus Encoding:
//   Cycle   Bit  Field        Description
//     0      7     v          valid bit, 
//     0      6     r/w        1->read, 0->write
//     0      5:4   size       0->32-bit, 1->64-bit, 2->128-bit, 3->256-bit
//     0      3:0   addr:stop  4-bit stop id, 4'b1111->all stops
//     1      7:0   addr:reg   7-bit local register addr at stop id
//
// Bus Transactions
// A read or write transaction takes 'n+4' cycles where 'n' is the #bytes specified in the 'size' field.
// A new read or write transaction can be initiated by CPLB on the M&M bus every 'n+4' cycles, where
// 'n' is the size of the previous transaction. The timing diagrams below show a 64-bit read, followed
// by a 64-bit write transaction, where n=8.
//
// Read:
//
//   CCLK             |  0   |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |  9  | 10  | 11  |  12 |
//   MmBusIn_M0[7]    |  v   |  a  |     |     |     |     |     |     |     |     |     |     |  v  |
//   MmBusIn_M0[6]    |  r   |  d  |     |     |     |     |     |     |     |     |     |     |  r  |
//   MmBusIn_M0[5:4]  | 64b  |  d  |     |     |     |     |     |     |     |     |     |     | 64b |
//   MmBusIn_M0[3:0]  | id   |  r  |     |     |     |     |     |     |     |     |     |     | id  |
//   -------------------------------------------------------------------------------------------------
//   MmBusOut_M1[7]   |      |  v  |  a  |     |     |     |     |     |     |     |     |     |     |
//   MmBusOut_M1[6]   |      |  r  |  d  |     |     |     |     |     |     |     |     |     |     |
//   MmBusOut_M1[5:4] |      | 64b |  d  |     |     |  b0 |  b1 |  b2 |  b3 |  b4 |  b5 |  b6 |  b7 |
//   MmBusOut_M1[3:0] |      | id  |  r  |     |     |     |     |     |     |     |     |     |     |
//   -------------------------------------------------------------------------------------------------
//   MmState[4:0]     |  M0  |  M1 |  M2 |  M3 |  M4 |  M5 |  M6 |  M7 |  M8 |  M9 | M10 | M11 | M0  |
//   -------------------------------------------------------------------------------------------------
//   MmLclRdEn_M2     |  0   |  0  |  1  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |
//   MmLclWrEn_M2     |  0   |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |
//   MmLclSz_M2       |  0   |  0  | 64b | 64b | 64b | 64b | 64b | 64b | 64b | 64b | 64b | 64b | 64b |
//   MmBusOut_M1[6:0] |      |     |addr |     |     |     |     |     |     |     |     |     |     |
//   MmDatIn_M4[7:0]  |      |     |     |     |  b0 |  b1 |  b2 |  b3 |  b4 |  b5 |  b6 |  b7 |     |
//
// Write:
//   CCLK             |  0   |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |  9  | 10  | 11  |  12 |
//   MmBusIn_M0[7]    |  v   |  a  |     |     |     |     |     |     |     |     |     |     |  v  |
//   MmBusIn_M0[6]    |  w   |  d  |     |     |     |     |     |     |     |     |     |     |  w  |
//   MmBusIn_M0[5:4]  | 64b  |  d  |     |     |  b0 |  b1 |  b2 |  b3 |  b4 |  b5 |  b6 |  b7 | 64b |
//   MmBusIn_M0[3:0]  | id   |  r  |     |     |     |     |     |     |     |     |     |     | id  |
//   -------------------------------------------------------------------------------------------------
//   MmBusOut_M1[7]   |      |  v  |  a  |     |     |     |     |     |     |     |     |     |     |
//   MmBusOut_M1[6]   |      |  w  |  d  |     |     |     |     |     |     |     |     |     |     |
//   MmBusOut_M1[5:4] |      | 64b |  d  |     |     |  b0 |  b1 |  b2 |  b3 |  b4 |  b5 |  b6 |  b7 |
//   MmBusOut_M1[3:0] |      | id  |  r  |     |     |     |     |     |     |     |     |     |     |
//   -------------------------------------------------------------------------------------------------
//   MmState[4:0]     |  M0  |  M1 |  M2 |  M3 |  M4 |  M5 |  M6 |  M7 |  M8 |  M9 | M10 | M11 | M0  |
//   -------------------------------------------------------------------------------------------------
//   MmLclRdEn_M2     |  0   |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |
//   MmLclWrEn_M2     |  0   |  0  |  1  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |
//   MmLclSz_M2       |  0   |  0  | 64b | 64b | 64b | 64b | 64b | 64b | 64b | 64b | 64b | 64b | 64b |
//   MmBusOut_M1[6:0] |      |     |addr |     |  b0 |  b1 |  b2 |  b3 |  b4 |  b5 |  b6 |  b7 |     |
//   MmDatIn_M4[7:0]  |      |     |     |     |     |     |     |     |     |     |     |     |     |

// **************************************************************************************************
// I/O
// **************************************************************************************************

module mm_stop

#(parameter                                     // parameters
   DISABLE_ASSERT      = 0,                     // disables asserts
   unit_msk            = 4'hF,                  // mask for decode of 4-address bits for stop id 

   MmLclRdEn_M2_COPY   = 1,                     // How many copies of output flops & ports? (for timing/PD)
   MmLclWrEn_M2_COPY   = 1,                     //  Each copy of an output will have cloned driving flop(s)
   MmLclSz_M2_COPY     = 1,                     //  and the output copies will be vectors;
   MmState_COPY        = 1                      //  i.e. MmState[(MmState_COPY*`MM_STATE_WIDTH)-1:0]
  )

(

// Global inputs
input                   KCLK_Free_AR,           // clock from ungated coarse gater
input                   SSE,                    // scan shift enable

input        [3:0]      unit_val,               // 4-bit unique stop id

// M&M bus data in & out
input   [`MM_DATA]      MmBusIn_M0,             // 8-bit M&M bus input
output  [`MM_DATA]      MmBusOut_M1,            // 8-bit M&M bus output
                                                //   contains local reg addr in M2
                                                //   contains local write data in M4
// to/from local stop
input  [`MM_DATA]       MmDatIn_M4,                          // read data from local register
output wire [MmLclRdEn_M2_COPY-1:0]            MmLclRdEn_M2, // local register read indication, valid in M2
output wire [MmLclWrEn_M2_COPY-1:0]            MmLclWrEn_M2, // local register write indication, valid in M2
output wire [MmLclSz_M2_COPY*`MM_SZ_WIDTH-1:0] MmLclSz_M2,   // local register size indication, valid in M2
output wire [MmState_COPY*`MM_STATE_WIDTH-1:0] MmState       // M&M state for read/write byte select

);

// include autoreg file
// `include "incl/mm_stop.reg.d"

// local reg/wire definitions
reg                  CgenMmActive;
wire                 FgenMmVal_M0;
wire                 CgenMmVal_M1M2;
wire                 RCLK_MmActive_AR;
wire                 RCLK_MmVal_M0_AR;
wire                 RCLK_MmVal_M1M2_AR;
reg                  MmVal_M0;
reg                  MmVal_M1;
reg                  MmVal_M2;
reg  [`MM_STATE]     MmStateNxt;
reg  [`MM_DATA]      MmBusNxt_M0;
wire [`MM_SZ]        MmSize_M1;
reg                  MmShared_M1;
wire                 MmShared_M1_dgate;
reg                  MmLclRdEn_M1;
wire                 MmLclRdEn_M1_dgate;
reg                  MmLclWrEn_M1;
//wire                 MmLclWrEn_M1_dgate; // unused/dead-end flop; see bug 636868
reg                  MmLclIdDec_M1;
reg  [`MM_SZ]        MmLclSz_M1;
wire [`MM_SZ]        MmLclSz_M1_dgate;
wire                 MmShared_M2;
wire                 MmLclRdEnInt_M2;
//wire                 MmLclWrEnInt_M2; // unused/dead-end flop; see bug 636868
reg  [15:0]          MmDatInShft_M4;
reg  [`MM_DATA]      MmDatInOr;
reg                  MmStopNxt;
wire                 MmStop;

// M&M stop stdcell count:
//   21 flops
//    3 fine gaters
// M&M stop timing paths:
//   MMBusIn_M0[7]   route from previous node -> or gate -> RCLK_MmActive_AR/en
//   MMBusIn_M0[7:0] route from previous node -> 3-1 mux -> MMBus_M1[7:0]/D

// default fine gater for all M&M bus flops when there is no valid transaction
cg I_RCLK_MmActive_AR (.gclk (RCLK_MmActive_AR), .clk(KCLK_Free_AR), .en(CgenMmActive), .SSE(SSE));
// functional fine gater for M0 cycle
assign FgenMmVal_M0 = MmVal_M0;
cg I_RCLK_MmVal_M0_AR     (.gclk(RCLK_MmVal_M0_AR), .clk(KCLK_Free_AR), .en(FgenMmVal_M0), .SSE(SSE));
// functional or power fine gater for M1 cycle
assign CgenMmVal_M1M2 = MmVal_M1 | MmVal_M2;
cg I_RCLK_MmVal_M1M2_AR     (.gclk(RCLK_MmVal_M1M2_AR), .clk(KCLK_Free_AR), .en(CgenMmVal_M1M2), .SSE(SSE));

// M&M active is set if incoming transaction or state machine is busy
//   MmStop is added to turn on the ck-gater for one more cycle after
//   the transaction ends to reset MmBusOut_M1[0] which would be
//   interpreted as a valid bit by the next stop.
always @* begin
  MmVal_M0 = MmBusIn_M0[`MM_VAL] & (MmState[`MM_STATE] == `MM_STATE_M0);
  MmVal_M1 = (MmState[`MM_STATE] == `MM_STATE_M1);
  MmVal_M2 = (MmState[`MM_STATE] == `MM_STATE_M2);
  CgenMmActive = MmVal_M0 | (MmState[`MM_STATE] != `MM_STATE_M0) | MmStop;
end

// 3-1 mux an input of M&M bus output flops to select between
//   repeat inbound data for write or remote read
//   local read data 
//   local read data left shifted by stop id for shared read
always @* begin
  MmDatInShft_M4[15:0] = {15'b0, MmDatIn_M4[0]} << (unit_val[3:0] & unit_msk);
  MmDatInOr[`MM_DATA] = ({`MM_DATA_WIDTH{(MmState[`MM_STATE] == `MM_STATE_M4)}} & MmDatInShft_M4[7:0]) | 
                        ({`MM_DATA_WIDTH{(MmState[`MM_STATE] == `MM_STATE_M5)}} & MmDatInShft_M4[15:8]);
  case (MmState[`MM_STATE])
    `MM_STATE_M0,   // M0 -> valid, r/w, size, node id
    `MM_STATE_M1,   // M1 -> local reg addr
    `MM_STATE_M2,   // M2 -> delay
    `MM_STATE_M3    // M3 -> delay
                  : MmBusNxt_M0[`MM_DATA] = MmBusIn_M0[`MM_DATA];
    default       : // M4-M19 -> select read data
                    //   shared reg id  -> 'or' in read data shifted by local id
                    casez ({MmShared_M2, MmLclRdEnInt_M2})
                      2'b?0 : MmBusNxt_M0[`MM_DATA] = MmBusIn_M0[`MM_DATA];                           // any write or remote read
                      2'b01 : MmBusNxt_M0[`MM_DATA] = MmDatIn_M4[`MM_DATA];                           // local read
                      2'b11 : MmBusNxt_M0[`MM_DATA] = MmBusIn_M0[`MM_DATA] | MmDatInOr[`MM_DATA];     // local shared read
                    endcase
  endcase
end

// MmStop is asserted for one cycle following the last cycle of the transaction
//   it is used to turn on the ck-gater so that MmBusOut_M1[0], which is the
//   valid bit, can be reset to 1'b0 if there is no new transaction
always @* begin
  MmStopNxt = ((MmState[`MM_STATE] == `MM_STATE_M7)  & (MmSize_M1[`MM_SZ] == `MM_SZ_32) | 
               (MmState[`MM_STATE] == `MM_STATE_M11) & (MmSize_M1[`MM_SZ] == `MM_SZ_64) | 
               (MmState[`MM_STATE] == `MM_STATE_M15) & (MmSize_M1[`MM_SZ] == `MM_SZ_96) | 
               (MmState[`MM_STATE] == `MM_STATE_M19) & (MmSize_M1[`MM_SZ] == `MM_SZ_128)
              );
end

// flop inbound M&M bus data to output to next node & for write data to local node
// flop MmStop which indicates end of transaction
//   RCLK_MmActive used for power savings
dff #(.WIDTH(`MM_DATA_WIDTH)) I_MmBusOut_M1 (.q(MmBusOut_M1[`MM_DATA]), .clk(RCLK_MmActive_AR), .d(MmBusNxt_M0[`MM_DATA]), .SSE(SSE));
dff #(.WIDTH(1))              I_MmStop      (.q(MmStop),                .clk(RCLK_MmActive_AR), .d(MmStopNxt),             .SSE(SSE));

// M&M transaction size
//   enable in first cycle, and hold for rest of transaction
//   use `MM_STOP_SZ instead of `MM_SZ to reference bits 5:4, not bits 1:0
dff #(.WIDTH(2)) I_MmSize_M1 (.q (MmSize_M1[`MM_SZ]), .clk(RCLK_MmVal_M0_AR), .d(MmBusIn_M0[`MM_STOP_SZ]), .SSE (SSE));

// M&M state machine control
//   M20-M31 can never be reached
always @* begin
  case (MmState[`MM_STATE])
    `MM_STATE_M0  : // change to M1 if new transaction incoming
                    case (MmVal_M0)
                      1'b1      : MmStateNxt[`MM_STATE] = `MM_STATE_M1;
                      1'b0      : MmStateNxt[`MM_STATE] = `MM_STATE_M0;
                    endcase
    `MM_STATE_M7  : // return to M0 if last data beat of 32-bit transaction
                    case (MmSize_M1[`MM_SZ])
                      `MM_SZ_32 : MmStateNxt[`MM_STATE] = `MM_STATE_M0;
                      default   : MmStateNxt[`MM_STATE] = `MM_STATE_M8;
                    endcase
    `MM_STATE_M11 : // return to M0 if last data beat of 64-bit transaction
                    case (MmSize_M1[`MM_SZ])
                      `MM_SZ_64 : MmStateNxt[`MM_STATE] = `MM_STATE_M0;
                      default   : MmStateNxt[`MM_STATE] = `MM_STATE_M12;
                    endcase
    `MM_STATE_M15 : // return to M0 if last data beat of 96-bit transaction
                    case (MmSize_M1[`MM_SZ])
                      `MM_SZ_96  : MmStateNxt[`MM_STATE] = `MM_STATE_M0;
                      default    : MmStateNxt[`MM_STATE] = `MM_STATE_M16;
                    endcase
    `MM_STATE_M19 : // return to M0
                    MmStateNxt[`MM_STATE] = `MM_STATE_M0;
    default       : MmStateNxt[`MM_STATE] = MmState[`MM_STATE] + `MM_STATE_WIDTH'b1;
  endcase
end

// M&M 5-bit state flops which indicate M0 - M19
//   RCLK_MmActive used for power savings only
dff #(.WIDTH(MmState_COPY*`MM_STATE_WIDTH)) I_MmState (.q(MmState[MmState_COPY*`MM_STATE_WIDTH-1:0]), .clk(RCLK_MmActive_AR), .d({MmState_COPY{MmStateNxt[`MM_STATE]}}), .SSE(SSE)); // FYI: MmState[`MM_STATE] bits have a high fannout due to logic within mm_stop;  even using MmState_COPY=2 will not reduce MmState[`MM_STATE] fannout;  this could be a potential future static timing/PD optimization to balance fanout across copies of MmState flops


// M1
//   output read enable, write enable and transfer size to local stop 
always @* begin
  // service transaction at current stop if
  //    incoming stop id matches local node id, qualified with param 'unit_msk'
  //    incoming stop id is the shared id, 4'b1111, which is used for shared writes or or-d reads
  MmShared_M1 = (MmBusOut_M1[`MM_STOP_ID] == `MM_STOP_ALL);
  MmLclIdDec_M1 = (
                   ((MmBusOut_M1[`MM_STOP_ID] & unit_msk) == (unit_val[3:0] & unit_msk)) | 
                    MmShared_M1 
                   );
  MmLclRdEn_M1 = MmVal_M1 & MmLclIdDec_M1 & MmBusOut_M1[`MM_RD];
  MmLclWrEn_M1 = MmVal_M1 & MmLclIdDec_M1 & ~MmBusOut_M1[`MM_RD];
  // Use `MM_STOP_SZ instead of `MM_SZ to reference bits 5:4, not bits 1:0
  MmLclSz_M1[`MM_SZ] = MmBusOut_M1[`MM_STOP_SZ];
end

// M2 flops
//   flop shared id for internal use later
//     RCLK_MmVal_M1M2_AR used for functional reasons (only needed M1, but sharing M1M2 to avoid extra CG)
//   flop data size transfer indication to send to local node
//     RCLK_MmVal_M1M2_AR used for power savings      (only needed M1, but sharing M1M2 to avoid extra CG)
//   flop read & write enables to send to local node
//     RCLK_MmVal_M1M2_AR used as functional gater to hold MmLclRd(Wr)EnInt_M2 for internal use
//   flop 
assign MmShared_M1_dgate        = MmVal_M1 ? MmShared_M1        : MmShared_M2;        // change on M1, then hold for rest of transaction
assign MmLclSz_M1_dgate[`MM_SZ] = MmVal_M1 ? MmLclSz_M1[`MM_SZ] : MmLclSz_M2[`MM_SZ]; // change on M1, then hold for rest of transaction
assign MmLclRdEn_M1_dgate       = MmVal_M1 ? MmLclRdEn_M1       : MmLclRdEnInt_M2;    // change on M1, then hold for rest of transaction
//assign MmLclWrEn_M1_dgate       = MmVal_M1 ? MmLclWrEn_M1       : MmLclWrEnInt_M2;    // change on M1, then hold for rest of transaction // unused/dead-end flop; see bug 636868
dff #(.WIDTH(1))                            I_MmShared_M2     (.q(MmShared_M2),                                  .clk(RCLK_MmVal_M1M2_AR), .d(MmShared_M1_dgate),                           .SSE(SSE));
dff #(.WIDTH(MmLclSz_M2_COPY*`MM_SZ_WIDTH)) I_MmLclSz_M2      (.q(MmLclSz_M2[MmLclSz_M2_COPY*`MM_SZ_WIDTH-1:0]), .clk(RCLK_MmVal_M1M2_AR), .d({MmLclSz_M2_COPY{MmLclSz_M1_dgate[`MM_SZ]}}), .SSE(SSE));
dff #(.WIDTH(1))                            I_MmLclRdEnInt_M2 (.q(MmLclRdEnInt_M2),                              .clk(RCLK_MmVal_M1M2_AR), .d(MmLclRdEn_M1_dgate),                          .SSE(SSE));
//dff #(.WIDTH(1))                            I_MmLclWrEnInt_M2 (.q(MmLclWrEnInt_M2),                              .clk(RCLK_MmVal_M1M2_AR), .d(MmLclWrEn_M1_dgate),                          .SSE(SSE)); // unused/dead-end flop; see bug 636868

// MmLclRd(Wr)En_M2 flops will automatically create read/write pulse to local node in M2  (because RCLK_MmVal_M1M2_AR allows for assert@M2 and de-assert@M3)
dff #(.WIDTH(MmLclRdEn_M2_COPY*1))          I_MmLclRdEn_M2    (.q(MmLclRdEn_M2[MmLclRdEn_M2_COPY*1-1:0]),        .clk(RCLK_MmVal_M1M2_AR), .d({MmLclRdEn_M2_COPY{MmLclRdEn_M1}}),           .SSE(SSE));
dff #(.WIDTH(MmLclWrEn_M2_COPY*1))          I_MmLclWrEn_M2    (.q(MmLclWrEn_M2[MmLclWrEn_M2_COPY*1-1:0]),        .clk(RCLK_MmVal_M1M2_AR), .d({MmLclWrEn_M2_COPY{MmLclWrEn_M1}}),           .SSE(SSE));

// Assertions
`ifdef ABV

  // M&M state machine in unreachable state
  ovl_never #(.msg ("MM stop state machine in illegal state"))
    I_MmStopStateIllegal (
      .clock (KCLK_Free_AR), .reset (1'b1), .enable (`MM_ABV_FUNC_EN & (DISABLE_ASSERT == 0)),
      .test_expr ( (MmState[`MM_STATE] > `MM_STATE_M19)
                 ),
     .fire ()
  );

  // M&M state machine is x
  ovl_never_unknown #(.msg ("MM stop state machine is x"), .width (`MM_STATE_WIDTH))
    I_MmStopStateUnknown (
      .clock (KCLK_Free_AR), .reset (1'b1), .enable (`MM_ABV_FUNC_EN & (DISABLE_ASSERT == 0)), .qualifier (1'b1),
      .test_expr (MmState[`MM_STATE]),
      .fire ()
  );

  // M&M bus conflict if new valid bit received while still processing a previous transaction
  ovl_never #(.msg ("MM stop bus conflict"))
    I_MmStopBusConflict (
      .clock (KCLK_Free_AR), .reset (1'b1), .enable (`MM_ABV_FUNC_EN & (DISABLE_ASSERT == 0)),
      .test_expr ( MmVal_M0 & (MmState[`MM_STATE] != `MM_STATE_M0)
                 ),
     .fire ()
    );

  // FIXME_KG - commented out because MmStopIdIsRsvd(x) is not found
  // M&M transaction uses reserved stop ID
  // ovl_never #(.msg ("MM stop id is reserved"))
  //  I_MmStopIdChk (
  //    .clock (KCLK_Free_AR), .reset (1'b1), .enable (`MM_ABV_FUNC_EN & (DISABLE_ASSERT == 0)),
  //    .test_expr ( MmVal_M0 & MmStopIdIsRsvd(MmBusIn_M0[`MM_STOP_ID])
  //               ),
  //   .fire ()
  //  );
`endif

endmodule
