//================================================================================================================================
//
// Copyright (c) 2002 by Advanced Micro Devices, Inc.
//
// This file is protected by Federal Copyright Law, with all rights reserved. No part of this file may be reproduced, stored in a
// retrieval system, translated, transcribed, or transmitted, in any form, or by any means manual, electric, electronic,
// mechanical, electro-magnetic, chemical, optical, or otherwise, without prior explicit written permission from Advanced Micro
// Devices, Inc.
//
//================================================================================================================================

//! disable one_module_per_file
// leda B_1001 off
// leda NTL_CLK03 off
//~ format_rtl 1
//~ format_inst narrow

//================================================================================================================================
// <DFXIP4_GC10_JTAGTDR>
// Description: Test Data Register with both a shift and update register
// Usage:       Instantiate as a TDR for a particular 1149.1 instruction
//              OR as a "segment" for a particular 1687 SIB.
//================================================================================================================================

module DFXIP4_GC10_JTAGTDR (RSTVAL, CAP_D, TDR_Q, CAP, SHF, UPD, TRST, TCK, TDI, TDO, SEL);

parameter LENGTH = 1;

// 1149.1 interface
  input        [LENGTH-1:0] RSTVAL; // Reset value for TDR
  input        [LENGTH-1:0] CAP_D;  // Capture value into TDR
  output   reg [LENGTH-1:0] TDR_Q;  // Update value from TDR

  input                     CAP;    // TAP state machine is in the Capture-DR
  input                     SHF;    // TAP state machine is in the Shift-DR
  input                     UPD;    // TAP state machine is in the Update-DR
  input                     TRST;   // Test reset
  input                     TCK;    // Test clock
  input                     TDI;    // Test data input
  output  wire              TDO;    // Test data output
  input                     SEL;    // Connect to 1687 SIB or 1149.1 decoded instruction or just 1'b1

  reg                       TdrShf;
  reg                       TdrCap;
  reg                       TdrUpd;
  reg          [LENGTH-1:0] ShTdrDin;
  reg          [LENGTH-1:0] ShTdrNxt;
  wire         [LENGTH-1:0] ShTdr;
  reg          [LENGTH-1:0] TdrNxt;
  wire         [LENGTH-1:0] Tdr;
  reg                       TDO_Q;

always @* begin
  TdrShf = SHF & SEL;  // Only shift this TDR segment if the SEL for this TDR is set
  TdrCap = CAP & SEL;  // Only capture this TDR segment if the SEL for this TDR is set
  TdrUpd = UPD & SEL;  // Only update this TDR segment if the SEL for this TDR is set
end

always @*
  {ShTdrDin[LENGTH-1:0], TDO_Q} = {TDI, ShTdr[LENGTH-1:0]};

always @*
  ShTdrNxt[LENGTH-1:0] = TdrShf ? ShTdrDin[LENGTH-1:0]
                       : TdrCap ? CAP_D[LENGTH-1:0]
                                : ShTdr[LENGTH-1:0];

DFXIP4_GC10_IP_JTAGFLOP I_ShTDR[LENGTH-1:0] (
  .ASYNCCLK({LENGTH{TRST}}),
  .ASYNCVAL(RSTVAL[LENGTH-1:0]),
  .CLK({LENGTH{TCK}}),
  .D(ShTdrNxt[LENGTH-1:0]),
  .Q(ShTdr[LENGTH-1:0])
);

// Hold flop at the end of the shift chain
DFXIP4_GC10_IP_JTAGFLOP I_HOLD (
  .ASYNCCLK(TRST),
  .ASYNCVAL(1'b0),
  .CLK(~TCK),
  .D(TDO_Q),
  .Q(TDO)
);

always @*
  TdrNxt[LENGTH-1:0] = TdrUpd ? ShTdr[LENGTH-1:0] : Tdr[LENGTH-1:0];

DFXIP4_GC10_IP_JTAGFLOP I_TDR[LENGTH-1:0] (
  .ASYNCCLK({LENGTH{TRST}}),
  .ASYNCVAL(RSTVAL[LENGTH-1:0]),
  .CLK({LENGTH{~TCK}}),
  .D(TdrNxt[LENGTH-1:0]),
  .Q(Tdr[LENGTH-1:0])
);

always @*
  TDR_Q[LENGTH-1:0] = Tdr[LENGTH-1:0];

endmodule

//================================================================================================================================
// <DFXIP4_GC10_JTAGTDR_NO_UPD>
// Description: Test Data Register w/o the update register
// Usage:       Instantiate as a TDR for a particular 1149.1 instruction
//              OR as a "segment" for a particular 1687 SIB.
//              Use caution when instantiating this since there are no update
//              register, so shifting will toggle the TDR_Q output.
//================================================================================================================================

module DFXIP4_GC10_JTAGTDR_NO_UPD (RSTVAL, CAP_D, TDR_Q, CAP, SHF, TRST, TCK, TDI, TDO, SEL);

parameter LENGTH = 1;

// 1149.1 interface
  input        [LENGTH-1:0] RSTVAL; // Reset value for TDR
  input        [LENGTH-1:0] CAP_D;  // Capture value into TDR
  output   reg [LENGTH-1:0] TDR_Q;  // Shift value from TDR

  input                     CAP;    // TAP state machine is in the Capture-DR
  input                     SHF;    // TAP state machine is in the Shift-DR
  input                     TRST;   // Test reset
  input                     TCK;    // Test clock
  input                     TDI;    // Test data input
  output  wire              TDO;    // Test data output
  input                     SEL;    // Connect to 1687 SIB or 1149.1 decoded instruction or just 1'b1

  reg                       TdrShf;
  reg                       TdrCap;
  reg          [LENGTH-1:0] ShTdrDin;
  reg          [LENGTH-1:0] ShTdrNxt;
  wire         [LENGTH-1:0] ShTdr;
  reg                       TDO_Q;

always @* begin
  TdrShf = SHF & SEL;  // Only shift this TDR segment if the SEL for this TDR is set
  TdrCap = CAP & SEL;  // Only capture this TDR segment if the SEL for this TDR is set
end

always @*
  {ShTdrDin[LENGTH-1:0], TDO_Q} = {TDI, ShTdr[LENGTH-1:0]};

always @*
  ShTdrNxt[LENGTH-1:0] = TdrShf ? ShTdrDin[LENGTH-1:0]
                       : TdrCap ? CAP_D[LENGTH-1:0]
                                : ShTdr[LENGTH-1:0];

DFXIP4_GC10_IP_JTAGFLOP I_ShTDR[LENGTH-1:0] (
  .ASYNCCLK({LENGTH{TRST}}),
  .ASYNCVAL(RSTVAL[LENGTH-1:0]),
  .CLK({LENGTH{TCK}}),
  .D(ShTdrNxt[LENGTH-1:0]),
  .Q(ShTdr[LENGTH-1:0])
);

// Hold flop at the end of the shift chain
DFXIP4_GC10_IP_JTAGFLOP I_HOLD (
  .ASYNCCLK(TRST),
  .ASYNCVAL(1'b0),
  .CLK(~TCK),
  .D(TDO_Q),
  .Q(TDO)
);

always @*
  TDR_Q[LENGTH-1:0] = ShTdr[LENGTH-1:0];

endmodule

//================================================================================================================================
// <DFXIP4_GC10_JTAGTDR_UPD>
// Description: Test Data Register update register only
// Usage:       Instantiate as a TDR for a particular 1149.1 instruction
//              OR as a "segment" for a particular 1687 SIB.
//              This should be used in combination with the
//              <DFXIP4_GC10_JTAGTDR_NO_UPD
//================================================================================================================================

module DFXIP4_GC10_JTAGTDR_UPD (RSTVAL, UPD_D, TDR_Q, UPD, TRST, TCK, SEL);

parameter LENGTH = 1;

// 1149.1 interface
  input        [LENGTH-1:0] RSTVAL; // Reset value for TDR
  input        [LENGTH-1:0] UPD_D;  // Shift value into TDR
  output   reg [LENGTH-1:0] TDR_Q;  // Update value from TDR

  input                     UPD;    // TAP state machine is in the Update-DR
  input                     TRST;   // Test reset
  input                     TCK;    // Test clock
  input                     SEL;    // Connect to 1687 SIB or 1149.1 decoded instruction or just 1'b1

  reg                       TdrUpd;
  reg          [LENGTH-1:0] TdrNxt;
  wire         [LENGTH-1:0] Tdr;

always @* begin
  TdrUpd = UPD & SEL;  // Only update this TDR segment if the SEL for this TDR is set
end

always @*
  TdrNxt[LENGTH-1:0] = TdrUpd ? UPD_D[LENGTH-1:0] : Tdr[LENGTH-1:0];

DFXIP4_GC10_IP_JTAGFLOP I_TDR[LENGTH-1:0] (
  .ASYNCCLK({LENGTH{TRST}}),
  .ASYNCVAL(RSTVAL[LENGTH-1:0]),
  .CLK({LENGTH{~TCK}}),
  .D(TdrNxt[LENGTH-1:0]),
  .Q(Tdr[LENGTH-1:0])
);

always @*
  TDR_Q[LENGTH-1:0] = Tdr[LENGTH-1:0];

endmodule

//================================================================================================================================
// <DFXIP4_GC10_STAC_PIPELINE>
// Description: Pipeline flop to daisy chain on Wsi or Wso for timing purposes
// Usage:       Instantiate as a timing repeater between Wsi or Wso of other STACs
//================================================================================================================================

module stac_pipeline_gc10 (
  input       Stac_Wrck,
  input       Stac_Wsi,
  output wire Stac_Wso,
  input       Stac_PipelineEn );

wire pipelined_wso;

DFXIP4_GC10_IP_MUX STAC_PIPELINE_MUX (
  .Sel ( Stac_PipelineEn ),
  .In0 ( Stac_Wsi        ),
  .In1 ( pipelined_wso   ),
  .Out ( Stac_Wso        )
);

DFXIP4_GC10_JTAGTDR_NO_UPD STAC_PIPELINE_FLOP (
  .RSTVAL ( 1'b0            ),
  .CAP_D  ( 1'b0            ),
  .TDR_Q  (                 ),
  .CAP    ( 1'b0            ),
  .SHF    ( 1'b1            ),
  .TRST   ( 1'b0            ),
  .TCK    ( Stac_Wrck       ),
  .TDI    ( Stac_Wsi        ),
  .TDO    ( pipelined_wso   ),
  .SEL    ( Stac_PipelineEn )
);

endmodule

// leda B_1001 on

//================================================================================================================================
// <DFXIP4_GC10_IP_JTAGFLOP>
// Description: Wrapper around a JTAG flop.
//              TODO: instantiate library block?
//================================================================================================================================

module DFXIP4_GC10_IP_JTAGFLOP (
  input        ASYNCCLK,
  input        ASYNCVAL,
  input        CLK,
  input        D,
  output  wire Q
);
    reg Qreg;
    always @(posedge CLK or posedge ASYNCCLK) begin
        if(ASYNCCLK==1'b1) begin
            Qreg <= ASYNCVAL;
        end else begin
            if(~(ASYNCCLK==1'b1)) begin
                Qreg <= D;
            end

            //vrq begin
            // VCS coverage off
            else begin
                Qreg <= 1'hx;
            end

            // VCS coverage on
            //vrq end
        end
    end
    assign   Q=Qreg;

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_CLKGATER>
// Description: Wrapper around a clock gater.
//================================================================================================================================

module DFXIP4_GC10_IP_CLKGATER (
  input        C,
  input        D,
  output  wire CG
);

 ati_clock_gate DFXIP4_GC10_IP_CLKGATER (
  .clk_out(CG),
  .clk_src(C),
  .clk_en(D),
  .TE(1'b0)
 );

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_SYNC>
// Description: Wrapper around a synchronizer.
//================================================================================================================================

module DFXIP4_GC10_IP_SYNC (
  input        SampleClk,
  input        DataIn,
  output  wire DataOut
);

//based on //sysip/dfx/dfx-4/4.0/main/dftmisc/src/rtl/dfx_sync3.v code
techind_sync_icd
# (.size(1),
   .depth(3),
   .sourceFlop(0),
   .asyncSet(0),
   .asyncReset(0),
   .asyncFlush(0),
   .clockEnable(0),
   .scan_muxD(1),
   .scan_LSSD(0),
   .MultiVolt(0),
   .TransportDelay(0), //Delay between src and dest flop in ps
   .TransportCycle(1), //propagation time from source to destination in term of destination cycle
   .InputCheck(1),
   .MetaWindowMode(0)
) DFXIP4_GC10_IP_SYNC (
    .SB(1'b0),
    .RB(1'b0),
    .FLUSH(1'b0),
`ifdef ABV
    .InputChkDis(1'b1), // always disable, ok to miss pulses here,
`endif
    .CKsrc(1'b0),
    .CKdst(SampleClk),
    .EN(1'b1),
    .Dsrc(DataIn),
    .Qsrc(),
    .Qdst(DataOut),

    .SC1(1'b0),
    .SC2(1'b0),
    .SE(1'b0),
    .SI(1'b0),
    .SO()
);
// above intends to be functionally equivalent to
//tools/internal/packages/design/rtllib/branches/rtllib_soc15/src/rtl/variant/*/sync.gv

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_RST_SYNC>
// Description: Wrapper around an active high reset synchronizer.
//================================================================================================================================

module DFXIP4_GC10_IP_RST_SYNC (
  input        SampleClk,
  input        RstIn,
  output  wire RstOut
);

//based on //sysip/dfx/dfx-4/4.0/main/dftmisc/src/rtl/dfx_async_set_sync3.v code
techind_sync_icd
# (.size(1),
   .depth(3),
   .sourceFlop(0),
   .asyncSet(1),
   .asyncReset(0),
   .asyncFlush(0),
   .clockEnable(0),
   .scan_muxD(1),
   .scan_LSSD(0),
   .MultiVolt(0),
   .TransportDelay(0),  //10000ps (100Mhz)
   .TransportCycle(3), //3 destination cycle
   .InputCheck(1),
   .MetaWindowMode(0)
) DFXIP4_GC10_IP_RST_SYNC (
    .SB(~RstIn),
    .RB(1'b0),
    .FLUSH(1'b0),
`ifdef ABV
    .InputChkDis(1'b1), // always disable, ok to miss pulses here,
`endif
    .CKsrc(1'b0),
    .CKdst(SampleClk),
    .EN(1'b1),
    .Dsrc(1'b0),
    .Qsrc(),
    .Qdst(RstOut),

    .SC1(1'b0),
    .SC2(1'b0),
    .SE(1'b0),
    .SI(1'b0),
    .SO()
);
// above intends to be functionally equivalent to
//tools/internal/packages/design/rtllib/branches/rtllib_soc15/src/rtl/variant/*/async_rst_sync_activehi.v

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_CLKMUX>
// Description: Wrapper for a clock mux.
//================================================================================================================================

module DFXIP4_GC10_IP_CLKMUX (
  input        ClkSel,
  input        ClkIn0,
  input        ClkIn1,
  output  wire ClkOut
);

 aticlk_mux2 d0nt_DFXIP4_GC10_IP_CLKMUX (
  .clk_o ( ClkOut           ),
  .clk_i ( {ClkIn1, ClkIn0} ),
  .sel   ( ClkSel           )
 );

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_MUX>
// Description: Wrapper for a marker mux.
//================================================================================================================================

module DFXIP4_GC10_IP_MUX (
  input        Sel,
  input        In0,
  input        In1,
  output  wire Out
);

 ati_mux2 d0nt_ati_mux2 (
  .z  ( Out ),
  .i0 ( In0 ),
  .i1 ( In1 ),
  .s  ( Sel )
 );

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_MARKER>
// Description: Wrapper for a marker cell.
//================================================================================================================================

module DFXIP4_GC10_IP_MARKER (
  input        A,
  output  wire Z
);
 ati_buffer1 mrk_DFXIP4_GC10_IP_MARKER (.A(A), .Y(Z));

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_CLKMARKER>
// Description: Wrapper for a clock marker cell.
//================================================================================================================================

module DFXIP4_GC10_IP_CLKMARKER (
  input        A,
  output  wire Z
);
 aticlk_buf mrk_DFXIP4_GC10_IP_CLKMARKER (.A(A), .Y(Z));

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_SRLATCH>
// Description: Wrapper around a set/reset latch.
//================================================================================================================================

module DFXIP4_GC10_IP_SRLATCH (
  input        S,
  input        R,
  output  reg  Q
);
// leda SYN9_5 off
// leda W410 off

latch_wrapper SRLATCH (
 .Q   ( Q ),
 .D   ( S ),
 .CLK ( S | R )
);

// leda SYN9_5 on
// leda W410 on

endmodule

//================================================================================================================================
// <DFXIP4_GC10_IP_ASYNCMSFLOP>
// Description: Wrapper around an async reset flop.  Copied ASYNCMSFLOP from rtl_stdlib.
//              TODO: find a library block to instantiate? used only in legacy resync logic
//================================================================================================================================

module DFXIP4_GC10_IP_ASYNCMSFLOP (
  input         E, // enable
  input         R, // reset
  input         D, // data
  input         C, // clock
  output  reg   Q // output
);
 reg  QM;  // master latch
// leda SYN9_5 off
// leda W410 off

//  the master latch with an asynchronous clear
//! latch
 always @* begin
  if      (R)      QM <= 1'b0;
  else if (~C & E) QM <= D;
 end
//  the slave latch with an asynchronous clear
//! latch
 always @* begin
  if      (R) Q <= 1'b0;
  else if (C) Q <= QM;
 end
// leda SYN9_5 on
// leda W410 on

endmodule

//! enable one_module_per_file
// leda B_1001 on
// leda NTL_CLK03 on
