

`include "gc__gb_reg.v"
`include "grbm_defs.vh"

module signature_generator 

  #(parameter SIGNATURE_REG_ADDR = 16'hBBBB,
  parameter WIDTH	       = 32)

(
  // Signature contribution
  input                      signature_bus_vld,   
  input [WIDTH-1:0] signature_bus_data,

  // GRBM write interface
  input [`GRBM_TARG_REG_SEND_RANGE] grbm_reg_send,
  input [`GRBM_TARG_REG_OP_RANGE]   grbm_reg_op,
  input [`GRBM_TARG_REG_ADDR_RANGE] grbm_reg_addr,
  input [`GRBM_TARG_REG_WD_RANGE]   grbm_reg_wd,

  // read interface
  output        signature_read_vld,
  output [31:0] signature_read_data,

  // output if the sig block is enabled (allows power savings for external logic)
  output        signature_block_enabled,

  input reset,
  input clk
);

// WARNING: WIDTH must be less than 1024.. 
// TBD-- add psl for this check.

localparam MOD = WIDTH % 32;
localparam MOD_32 = MOD != 0 ? MOD : 32;
localparam WIDTH_BY_32 = MOD != 0 ? (WIDTH/32) : ((WIDTH-1)/32);

// psl signature_bus_data_is_unknown: assert always !(signature_bus_vld && isunknown(signature_bus_data) && (reset===1'b0))@(posedge clk);

  // -------------------
  // --- GRBM Decode ---
  // -------------------
  wire state_sel_signature_reg     = (grbm_reg_addr == SIGNATURE_REG_ADDR)               && grbm_reg_send;
  wire state_sel_signature_control = (grbm_reg_addr == `RAS_SIGNATURE_CONTROL_ABSD_ADDR) && grbm_reg_send;
  wire state_sel_signature_mask    = (grbm_reg_addr == `RAS_SIGNATURE_MASK_ABSD_ADDR)    && grbm_reg_send;

  reg [`RAS_SIGNATURE_CONTROL_ENABLE_SIZE-1:0]         signature_enabled;
  always @(posedge clk) begin
    if(reset) begin
      signature_enabled        <= `RAS_SIGNATURE_CONTROL_ENABLE_DEFAULT;
    end
    else if(state_sel_signature_control && grbm_reg_op) begin
      signature_enabled        <= grbm_reg_wd[`RAS_SIGNATURE_CONTROL_ENABLE];
    end
  end

  reg [WIDTH_BY_32:0] signature_input_bus_mask;
  always @(posedge clk) begin
    if(reset) begin
      signature_input_bus_mask <= `RAS_SIGNATURE_MASK_INPUT_BUS_MASK_DEFAULT; // lint waiver needed here.
    end
    else if(state_sel_signature_mask && grbm_reg_op) begin
      signature_input_bus_mask <= grbm_reg_wd[WIDTH_BY_32:0];
    end
  end


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // -------------------------
  // --- Signature Masking ---
  // -------------------------
  wire [WIDTH-1:0] masked_signature_bus_data;

generate
 if (WIDTH_BY_32 > 0) begin
 genvar i;
  for(i=0; i<WIDTH_BY_32; i=i+1) begin : sig_mask_i // WIDTH = 330, then WIDTH_BY_32 = 10, i = 0..10
    assign masked_signature_bus_data[(32*i+31):(32*i)] = {32{signature_input_bus_mask[i]}} & signature_bus_data[(32*i+31):(32*i)];
    //ati_buffer1 mrk_sig_mask[(32*i+31):(32*i)] ( .A( ( {32{signature_input_bus_mask[i]}} & signature_bus_data[(32*i+31):(32*i)] )), .Y( masked_signature_bus_data[(32*i+31):(32*i)]) );
  end
 end
endgenerate

    assign masked_signature_bus_data[(WIDTH-1):(32*WIDTH_BY_32)] = {(MOD_32){signature_input_bus_mask[WIDTH_BY_32]}} & signature_bus_data[WIDTH-1:(32*WIDTH_BY_32)];
    //ati_buffer1 mrk_sig_mask_last[WIDTH-1:32*WIDTH_BY_32] (.A( ( {(MOD_32){signature_input_bus_mask[WIDTH_BY_32]}} & signature_bus_data[WIDTH-1:(32*WIDTH_BY_32)]) ), .Y(masked_signature_bus_data[(WIDTH-1):(32*WIDTH_BY_32)] ) );



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // -----------------
  // --- Compactor ---
  // -----------------
  wire [31:0] compacted_signature_data;

  wire  [31:0] comp_data [WIDTH_BY_32:0]; // few bits of last column of array might be un-used (depend of WIDTH number.) 
					// lint waiver needs to be added for those bits.
  wire  [WIDTH_BY_32:0] comp_data_reverse [31:0];

generate
 if (WIDTH_BY_32 > 0) begin
 genvar k;

   for(k=0; k<WIDTH_BY_32; k=k+1) begin : comp_array_k // 0, 32, 64, 96
     assign comp_data[k] = masked_signature_bus_data[k*32+:32];
     //ati_buffer1 mrk_comp_array[31:0] (.A(masked_signature_bus_data[k*32+:32]), .Y(comp_data[k]));
   end
 end   
 endgenerate

    // tie unused bit of array to 0.
    assign comp_data[WIDTH_BY_32] = { {(32-MOD){1'b0}},  masked_signature_bus_data[WIDTH_BY_32*32+: MOD_32] };  // [<starting_bit>+:<width>]
    //ati_buffer1 mrk_comp_array[31:0] (.A( { {(32-MOD){1'b0}},  masked_signature_bus_data[WIDTH_BY_32*32+:MOD] } ), .Y(comp_data[WIDTH_BY_32]));



generate
 genvar l;
 genvar m;
  for(l=0; l<32; l=l+1) begin : comp_rev_array_l
  for(m=0; m<=WIDTH_BY_32; m=m+1) begin : comp_rev_array_m
  assign comp_data_reverse[l][m] = comp_data[m][l];
  end
  end
endgenerate

generate
 genvar n;
  for(n=0; n<32; n=n+1) begin : comp_sig_n
      assign compacted_signature_data[n] = ^comp_data_reverse[n];
      //ati_buffer1 mrk_comp_sig (.A(^comp_data_reverse[n]), .Y(compacted_signature_data[n]));
  end
endgenerate

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  // ------------------------
  // --- Signature Update ---
  // ------------------------
  reg [31:0] signature;
  always @(posedge clk) begin
    // Set signature to value updated by GRBM
    if(state_sel_signature_reg && grbm_reg_op) begin
      signature <= grbm_reg_wd;
    end
    else if(signature_bus_vld & signature_enabled) begin
      signature <= signature + compacted_signature_data;
    end
    else begin
      signature <= signature;
    end
  end

  // ----------------------
  // --- Signature Read ---
  // ----------------------
  // Force read data to zero when the read is not valid so it can
  // easily be ORed into an external read bus without extra conditioning.
  assign signature_read_vld  = state_sel_signature_reg && !grbm_reg_op;
  assign signature_read_data = signature_read_vld ? signature : 32'b0;

  
  // -------------------------------
  // --- Signature Enable Output ---
  // -------------------------------
  assign signature_block_enabled = signature_enabled;

endmodule
