`timescale 1ps/1ps
`include "rtllib_features.vh"
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Module Name    : techind_cdcefpm_single
///  
///  Revision       : $Revision$
///  Release Status : $State$
///  Author         : $Author$
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    ABSTRACT
///
///    This wrapper module implements a technology independent means to directly instantiate
///    a technology-library false-path marker cell into RTL during synthesis. With appropriate
///    switches, the correct marker cell from the target library is instantiated in the design
///    using a naming convention 'd0nt_CDEFPM' that is meaningful to the downstream tool flow.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SYNTHESIS USAGE:
///
///    In order for synthesis to pick up an appropriate cell from the target library, the
///    following value must be defined:
///
///        `define  SYNTHESIS
///
///    In addition to the 'SYNTHESIS' define, one technology library must also be specified.
///    This revision of the wrapper currently supports the following libraries [note, this
///    list must be maintained as we move forward from one process to the next in order to
///    ensure that the marker cells are properly instantiated in future designs]:
///
///        `define    GPU__CHIP__TECH__AMD_32N_SOI
///        `define    RTLLIB__TECH__AMD_28SHP
///        `define    RTLLIB__TECH__GF28G
///        `define    RTLLIB__TECH__TSMC_28HP
///        `define    RTLLIB__TECH__TSMC_16FF
///        `define    RTLLIB__TECH__TSMC_14XM
///
///    Note, as was mentioned earlier, this wrapper is only part of a larger flow and should
///    not be instantiated directly by designers. In order to heavily discourage such abuse,
///    when the 'SYNTHESIS' define is not set, this wrapper module will provide no internal 
///    functionality which will yield erroneous simulation results.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    Copyright (C) 2009,2010,2011
///      by Advanced Micro Devices (AMD) Corporation
///  
///    All rights reserved. This source file is considered proprietary and confidential
///    information of AMD Corp. As such, this source file may not be distributed by any
///    means, in whole, or in part, without the express, written approval of AMD Corp.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Revision History :
///  
///  $Log$
///
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
module techind_cdcefpm_single ( A, EN, DSTCLK, ZB ); 
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  parameter   driveSel           = 3;
  parameter   DestClkCnt         = 0;
  parameter   CdcDestSetup       = 11000;
  parameter   HoldCheck          = 0;
  parameter   SrcPath            = 0;
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  input               A;
  input               EN;
  input               DSTCLK;
  output              ZB; 
// river translate_off
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  As indicated earlier, we use a variety of defines to guide synthesis down the desired
  ///  code-path to instantiate the technology-specific marker cell.
  `ifndef SYNTHESIS
     ///  should this path be taken, the designer must have used this wrapper incorrectly. So,
     ///  we are creating a simulation exposure. Note, it would have been preferable to cause
     ///  a hard compile failure, but the VRQ X-uglification tool chokes when we try to do
     ///  that.
  `else  
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     //////                                                        /////////////////////
     //////    THE FOLLOWING CODE SHOULD BE ACTIVELY MAINTAINED    /////////////////////
     //////    AS NEW TECHNOLOGY LIBRARIES ARE TO BE SUPPORTED     /////////////////////
     //////                                                        /////////////////////
     //////    * note, as new 'define values are supported, they   /////////////////////
     //////      should be captured in the 'SYNTHESIS USAGE'       /////////////////////
     //////      documentation at the top of this file.            /////////////////////
     //////                                                        /////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     ///
     ///  For technology libraries that do support a physical cell to represent the
     ///  CDCBUFENCLK, we instantiate the cell from the library. As an initial guess we [by
     ///  default] select a cell with moderate drive strength. It is expected that the tool
     ///  flow will upsize/downsize the cell as required to optimize the timing path. However,
     ///  we also provide a means to specify an alternate drive strength through a parameter
     ///  in order to better seed the synthesis run.
     ///
     ///  Also, in support of the synthesis and constraint flow, we provide a means to
     ///  communicate back to the flow the need to "preserve" this module. This preservation
     ///  is necessary in order to ensure that the 'DSTCLK' pin can be located and traced
     ///  since it is virtual and does not map onto the actual technology-specific marker cell.
     ///  This is accomplished by setting the following define for the flow to query:
     ///
     ///     SUPRA__FEINT__NEED_PRESERVE_CDCEFPM_SINGLE
     ///
     ///  Note, it is necessary to communicate this information back to the flow so that the
     ///  preserve contstraint is only applied when we have actual technology specific marker
     ///  cells instantiated. Failure to preserve the hierarchy will result in the flow being
     ///  incapable of calculating clock-based delay constraints.
     ///
     ///  by contrast, when the clause with the [default] RTL statement is taken, we do not set
     ///  the above define. Doing so while the RTL is called out will result in a synthesis
     ///  failure since the RTL will not have been mapped to library cells when the preserve
     ///  constraint is applied.
       generate
         if      ( driveSel == 1 )    begin
           if (HoldCheck) begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC1_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
            end else begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_1 d0nt_CDCEFPM_HC0_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
            end
         end else if ( driveSel == 2 )    begin
        if (HoldCheck) begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
        end else begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
        end
         end else if ( driveSel == 3 )    begin
        if (HoldCheck) begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
           end else begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
            end
         end else if ( driveSel == 4 )    begin
           if (HoldCheck) begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC1_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
            end else begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_4 d0nt_CDCEFPM_HC0_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
            end
         end else                         begin
            if (HoldCheck) begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC1_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
        end else begin
           case (DestClkCnt)
             0: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             1: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             2: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             3: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             4: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             5: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             6: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             7: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             8: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             9: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             10: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             11: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             12: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             13: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             14: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             15: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             16: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             17: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             18: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             19: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             20: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             21: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             22: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             23: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             24: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             25: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             26: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             27: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             28: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             29: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             30: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             31: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             32: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             33: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             34: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             35: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             36: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             37: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             38: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             39: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             40: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             41: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             42: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             43: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             44: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             45: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             46: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             47: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             48: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             49: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
             default: HDBLVT08_ND2_CDC_2 d0nt_CDCEFPM_HC0_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .EN(EN), .X(ZB));
           endcase
        end
         end
       endgenerate
       `ifndef SUPRA__FEINT__NEED_PRESERVE_CDCEFPM_SINGLE
         `define  SUPRA__FEINT__NEED_PRESERVE_CDCEFPM_SINGLE
       `endif
       /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  `endif  ///  SYNTHESIS
// river translate_on
endmodule
