`timescale 1ps/1ps
`include "rtllib_features.vh"
//////////////////////////////////////////////////////////////////////////////
//  
//  Module Name    : techind_cdcfpm_single
//  
//  Revision       : $Revision$
//  Release Status : $State$
//  Author         : $Author$
//  
//   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  
//    ABSTRACT
//
//    This wrapper module implements a technology independent means to embed
//    a false-path marker into RTL code. With appropriate switches, simulation
//    is augmented with the means to aid the verification of false-path
//    assumptions by incorporating a special simulation model.
//
//    Logically, the model has the function of a simple buffer. The "A" pin is
//    pin through which the false-path propagates.
//
//    It should be noted that this wrapper only represents part of a larger
//    flow to ensure that false-paths are defined and constrained properly.
//    In addition to this wrapper and the cell contained within, a means to
//    specify timing constraints to both the simulation model as well as the
//    physical design flow must be supported in conjunction with this wrapper,
//    keying off of the 'd0nt_CDCFPM' instance name.
//
//   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
//    SYNTHESIS USAGE:
//
//    in order for synthesis to pick up an appropriate cell from the target
//    library, the following value must be defined:
//
//        `define  SYNTHESIS
//
//    in addition to the 'SYNTHESIS' define, one technology library must also
//    be specified. This revision of the wrapper currently supports the
//    following libraries [note, this list must be maintained as we move
//    forward from one process to the next]:
//
//        `define    GPU__CHIP__TECH__AMD_32N_SOI
//        `define    RTLLIB__TECH__AMD_28SHP
//        `define    RTLLIB__TECH__GF28G
//        `define    RTLLIB__TECH__TSMC_28HP
//        `define    RTLLIB__TECH__TSMC_16FF
//        `define    RTLLIB__TECH__GF_14XM
//
//   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
//    SIMULATION USAGE:
//
//    by default, RTL simulations will expect that the CDCBUFCLK model exists
//    in the library and will call it out. This behaviour is selected as the
//    default so that a minimally specified defines file will operate
//    in a more aggressive manner. However, in order to accommodate early
//    deployment and portability to a program that does not currently support
//    the CDCBUFCLK model in the library, the following define is used to
//    temporarily substitute a simple, equivalent RTL equation.
//
//        `define  TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
//
//    note, this define is explicitly declared to avoid using the CDCBUFCLK
//    cell model. It is hoped that the presence of an undesireable, temporary
//    define will be more noticable, and therefore more likely to be corrected
//    than the absense of a define used to include the CDCBUFCLK model.
//
//   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  
//    Copyright (C) 2009,2010,2011
//      by Advanced Micro Devices (AMD) Corporation
//  
//    All rights reserved. This source file is considered proprietary and
//    confidential information of AMD Corp. As such, this source file may
//    not be distributed by any means, in whole, or in part, without the
//    express, written approval of AMD Corp.
//  
//   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//  
//  Revision History :
//  
//  $Log$
//  
//////////////////////////////////////////////////////////////////////////////
module techind_cdcfpm_single ( A, DSTCLK, Z ); 
  parameter   applyParams        = 0;
  parameter   CdcDestSetup       = 11000;
  parameter   DestClkCnt         = 0;
  parameter   driveSel           = 3;
  parameter   mode01             = 0;
  parameter   BufMetaWaiveMode   = 0;
  parameter   SrcPath            = 0;
  input               A;
  input               DSTCLK;
  output              Z; 
// river translate_off
  //  we use a variety of defines that indicate the degree of library support,
  //  technology library, and tool-chain to determine which representation for the
  //  CDC enabled false-path marker shall be selected
  `ifndef SYNTHESIS
    `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
       CDCBUFCLK
         #(
            .Setup   ( CdcDestSetup ),
            .Cycles  ( DestClkCnt   ),
            .BufMetaWaiveMode (BufMetaWaiveMode),
            .mode01  ( mode01       ),
            .SrcPath ( SrcPath      )
          )  d0nt_CDCFPM  (
            .A       ( A      ),
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
            .DSTCLK  ( DSTCLK ),
           `endif
            .Z       ( Z      )
          );
       //  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       //
       //  using this wrapper, we support two, mutually exclusive flows for
       //  providing a single source of timing configuration information into
       //  simulations as well as the downstream tool flow:
       //
       //    1.  the first of these flows is the application of the configuration
       //        file as a collection of IP top-level force statements that are
       //        developed ahead of time. When applying this technique, the
       //        "applyParams" parameter must be set to 0 (default value) so that
       //        the parameters to this wrapper are not pushed down into the
       //        target cell. This leaves the controls within the target cell
       //        clear for the IP top-level force statements to be applied
       //        without introducing a force conflict. This will allow the
       //        configuration values to be applied for simulation, and
       //        separately delivered to the downstream tool flow.
       //
       //    2.  in the absense of an IP top-level force statement, the
       //        configuration file can be generated from the paramters that are
       //        passed into the instance. When using this technique, setting the
       //        "applyParams" parameter to 1 will apply (through forces) the
       //        parameters to the target cell. This will allow the simulation to
       //        use the configuration information propagated via parameters.
       //
       //        however, in order to satisfy the downstream tool flow, the
       //        parameter information must be dumped out. By dumping out the
       //        parameter information, we can avoid problems of inconsistency
       //        between multiple sources for a given piece of configuration
       //        information. Since we anticipate having many thousands of these
       //        cells, dumping may take some time. In order to avoid the
       //        simulation impact on the vast majority of the simulations, the
       //        "writeParam" parameter must also be set in order for the
       //        parameters to be dumped out.
       //
       //        However, the means for dumping will include the entire
       //        hierarchical path, and is not realtive to the IP top-level (or
       //        tile top-level). In order to deal with this, a post processing
       //        phase will be required to lift the configuation information and
       //        manipulate the path name to provide a top-level relative name
       //        that will be consumed by the downstream tool flow.
       //  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       //
       // synopsys translate_off
       // novas translate_off
       `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
         ///  `ifndef AMDFORCE
         ///    `define AMDFORCE(signal, value) force signal = value;
         ///  `endif
         initial begin
           if ( applyParams ) begin
             ///  `AMDFORCE( d0nt_CDCFPM.CDCBUF_GEN.CdcDestSetup,    CdcDestSetup )
             ///  `AMDFORCE( d0nt_CDCFPM.CDCBUF_GEN.DestClkCnt,      DestClkCnt )
             if ( $test$plusargs("logCdcBufTiming")) begin
               $display ( "info: CDCBUFCLK %m.d0nt_CDCFPM.techindMarker : %t", $time );
               $display ( "`AMDFORCE(%m.d0nt_CDCFPM.CdcDestSetup, %1d);", CdcDestSetup );
               $display ( "`AMDFORCE(%m.d0nt_CDCFPM.DestClkCnt, %1d);",   DestClkCnt );
             end
           end
         end
       `endif  //  TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
       // synopsys translate_on
       // novas translate_on
    `else  
       //  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       //
       //  if the simulation model for the enabled false-path marker is not
       //  available, we want to have an equivalent functionality present so
       //  so that our simulations behave in the expected manner while we are
       //  in the early stages of a project. However, we should have our checks
       //  in place to ensure that the simulation model is eventually selected
       //  as early in the program as possible.
       //
       //  as mentioned earlier, it is hoped that the presence of the define
       //  above will make it simpler to identify shortcomings. For instance,
       //  greping for the existence of such bypasses would yield a list of
       //  unsupported features that would need to be addressed.
       assign Z = A;
    `endif  //  TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  `else  
     /////////////////////////////////////////////////////////////////////////
     /////////////////////////////////////////////////////////////////////////
     /////////////////////////////////////////////////////////////////////////
     ////                                                        /////////////
     ////    THE FOLLOWING CODE SHOULD BE ACTIVELY MAINTAINED    /////////////
     ////    AS NEW TECHNOLOGY LIBRARIES ARE TO BE SUPPORTED     /////////////
     ////                                                        /////////////
     ////    * note, as new 'define values are supported, they   /////////////
     ////      should be captured in the 'SYNTHESIS USAGE'       /////////////
     ////      documentation at the top of this file             /////////////
     ////                                                        /////////////
     /////////////////////////////////////////////////////////////////////////
     // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
     //
     //  for technology libraries that do support a physical cell to represent
     //  the CDCBUFCLK we instantiate the cell from the library. As an initial guess
     //  we select [by default] a cell with moderate drive strength. It is expected that
     //  the tool flow will upsize/downsize the cell as required to optimize the timing
     //  path. However, we also provide a means to specify an alternate drive strenght
     //  through a parameter in order to better seed the synthesis run.
     //
     //  also, in support of the synthesis and constraint flow, we provide a means
     //  to communicate back to the flow the need to "preserve" this module. This
     //  preservation is necessary in order to ensure that the 'DSTCLK' pin can be
     //  located and traced -- since it is virtual and does not map onto the actual
     //  technology-specific marker cell. This is accomplished by setting the following
     //  define for the flow to query:
     //
     //     SUPRA__FEINT__NEED_PRESERVE_CDCFPM_SINGLE
     //
     //  note, it is necessary to communicate this information back to the flow so
     //  that the preserve contstraint is only applied when we have actual technology
     //  specific marker cells instantiated. Failure to preserve the hierarchy will
     //  result in the flow being incapable of calculating clock-based delay constraints.
     //
     //  by contrast, when the clause with the [default] RTL statement is taken, we do
     //  not set the above define. Doing so while the RTL is called out will result in
     //  a synthesis failure since the RTL will not have been mapped to library cells
     //  when the preserve constraint is applied.
       generate
         if      ( driveSel == 1 )    begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_1 d0nt_CDCFPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end else if ( driveSel == 2 )    begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end else if ( driveSel == 3 )    begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end else if ( driveSel == 4 )    begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_4 d0nt_CDCFPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end else                         begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_2 d0nt_CDCFPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end
       endgenerate
       `ifndef SUPRA__FEINT__NEED_PRESERVE_CDCFPM_SINGLE
         `define  SUPRA__FEINT__NEED_PRESERVE_CDCFPM_SINGLE
       `endif
       //  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  `endif  //  SYNTHESIS
  // river translate_on
endmodule
