`timescale 1ps/1ps
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Module Name    : techind_iso_cdcefpm
///  
///  Revision       : $Revision$
///  Release Status : $State$
///  Author         : $Author$
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    ABSTRACT
///
///    This wrapper module implements a technology independent means to embed a bus-width
///    of enabled false-path markers into RTL code. With appropriate switches, simulation
///    is augmented with the means to aid the verification of false-path assumptions by
///    incorporating a special simulation model. Also with [other] appropriate switches,
///    synthesis runs that target a specific technology library can call out specific physical
///    marker cells to which the enabled false-path markers are to be mapped.
///
///    Logically, the  model has the function of a two-input NAND gate. The "A" pin is the
///    pin through which the false-path must propagate and the "EN" pin is used to control
///    when the false-path is sampled. However, in a bid to minimize the incidence rate of
///    logic errors [introduced by designers] that would stem from the inverting function of
///    the NAND, this wrapper incorporates an optional [used by default] post-cell inversion
///    in order to present the more intuitive, non-inverting AND function.
///
///    It should be noted that this wrapper only represents part of a larger flow to ensure
///    that false-paths are defined and constrained properly. In addition to this wrapper
///    and the cell model contained within, a means to specify timing constraints to both
///    the simulation model as well as the physical design flow must be supported in
///    conjunction with this wrapper, keying off of the 'd0nt_CDCEFPM' instance name.
///
///    In support of the timing constraint flow, we want to preserve the DSTCLK pin on the
///    layer above the actual library marker cell instance. In order to accomplish this, we
///    encapsulate the actual technology specific cell instantiation within a lower-level
///    wrapper onto which we can apply a preserve command. Within this upper-level wrapper,
///    we either instantiate the simulation model, or the bussed instantiation of the
///    lower-level wrapper onto which we shall apply the preserve command.
///
///    For more details, refer to the 'techind_iso_cdcefpm_single.v' wrapper file.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SYNTHESIS USAGE:
///
///    In order for synthesis to pick up an appropriate marker cell from the target library,
///    the following define must be set within the synthesis setup in order to ensure that
///    the synthesis tool takes the appropriate path through this wrapper code.
///
///        `define  SYNTHESIS
///
///    In addition to the 'SYNTHESIS' define, a technology library must also be specified
///    in order to directly instantiate an appropriate marker cell from the target technology
///    cell library. For a current list of libraries that are supported, please refer to the
///    lower-level wrapper file:
///
///        'techind_iso_cdcefpm_single.v'
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SIMULATION USAGE:
///
///    When not performing synthesis [ie. the 'SYNTHESIS' define is not set], the compiled
///    code-path is geared for simulation. In this mode, by default, RTL simulations will
///    expect that  the CDCBUFENCLK model exists in the library and will call it out. This
///    behaviour is selected as the default so that a minimally specified defines file will
///    operate in a more aggressive manner. However, in order to accommodate early deployment
///    and portability to a program that does not currently support the CDCBUFENCLK model in
///    the library, the following define is used to temporarily substitute a simple,
///    equivalent RTL equation.
///
///        `define  TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
///
///    Note, this define is explicitly declared to avoid using the CDCBUFENCLK cell model.
///    It is hoped that the presence of an undesireable, temporary define will be more
///    noticable, and therefore more likely to be corrected than the absense of a define used
///    to include the CDCBUFENCLK model.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    Copyright (C) 2009,2010,2011
///      by Advanced Micro Devices (AMD) Corporation
///  
///    All rights reserved. This source file is considered proprietary and confidential
///    information of AMD Corp. As such, this source file may not be distributed by any
///    means, in whole, or in part, without the express, written approval of AMD Corp.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Revision History :
///  
///  $Log$
///  
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////

  module techind_iso_cdcefpm ( A, EN, DSTCLK, Z);

  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  This module has several parameters that can be used to control the simulation behaivour
  ///  of the underlying CDCBUFENCLK model. In addition, some of these parameters may be dumped
  ///  into the simulation log file as a means for mining the instance-specific information
  ///  and presenting it to the physical implementation and timing flows.
  ///
  ///  Here we shall present each of the parameters. After the parameter declarations, a
  ///  description is provided that indicates how each parameter is used.
  parameter   size               = 1;
  parameter   applyParams        = 0;
  parameter   writeParams        = 1;   ///  deprecated. DO NOT USE. Only provided here for temporary backward compatibility.
  parameter   CdcDestSetup       = 11000;
  parameter   ClockBasedDelay    = 0;   ///  deprecated. DO NOT USE. Only provided here for temporary backward compatibility.
  parameter   DestClkCnt         = 0;
  parameter   AssertEnable       = 1;
  parameter   driveSel           = 3;
  parameter   andGateModel       = 1;
  parameter   HoldCheck          = 0;
  parameter   SrcPath            = 0;
  ///    .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .
  ///
  ///  'size'          :  used to indicate the width/size of the bus on which the false-path
  ///        marker is to be instantiated. The default is set for a single-bit. In particular,
  ///        this parameter specifies the bus range [size-1:0] for the 'A' and 'Z' ports.
  ///
  ///
  ///  'applyParams'   :  default 0. When set to '1' the following three parameters are applied
  ///        to the underlying CDCBUFENCLK model using local, relative force statements, to
  ///        control the CDCBUFENCLK behaviour.
  ///
  ///
  ///  'CdcDestSetup'  :  passed along to the CDCBUFENCLK model. Specifies the number of ps
  ///        [picoseconds] after a change at the input 'A' for which the output 'Z' will be
  ///        driven 'X' to model propagation uncertainty, metastability, and glitch risk.
  ///
  ///
  ///  'DestClkCnt'    :  passed along to the CDCBUFENCLK model. Specifies the number of DSTCLK
  ///        rising edges after a change at the 'A' input for which the output 'Z' will be
  ///        driven 'X' to model propagation uncertainty, metastability, and glitch risk.
  ///
  ///
  ///  'AssertEnable'  :  passed along to the CDCBUFENCLK model. Disables the assertion check
  ///        of 'X' on each rising edge of DSTCLK. Note, disabling the assertion invalidates
  ///        the status of "customer synchronizer" normally conferred to the CDCBUFENCLK model.
  ///        Setting a '0' on this parameter will trigger a design audit. Please contact the
  ///        CDCWG (CDC Work Group) for guidance if it is felt that there is a need to set this
  ///        parameter to '0'.
  ///
  ///
  ///  'driveSel'      :  passed along to the library instantiation. Selects an alternate
  ///        drive strength for the library-specific marker cell. Generally, it is expected that
  ///        the default value will be sufficient, but it may be tweaked at the user's discretion
  ///        if better seeding for synthesis is warranted.
  ///
  ///
  ///  'andGateModel'  :  default '1'. When set to '0', the RTL inversion that abstracts the
  ///        CDCBUFENCLK to a two-input AND view is bypassed, thereby exposing the underlying
  ///        two-input NAND view of the actual CDCBUFENCLK. It is expected that the vast majority
  ///        of users will deploy the AND view, but some designers needing lower-level control
  ///        over their data-path can opt out of the AND-rectifying inversion.
  ///
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////

  input   [size-1:0]  A;
  input               EN;
  input               DSTCLK;
  output  [size-1:0]  Z; 

    `ifdef ABV
    reg InputChkDis;
    initial InputChkDis = 1'b0; // disabling InputChkDis by default, if needd techind_sync_icd should be used.
    `endif

   techind_iso_cdcefpm_icd #(
    .size            (size          ),
    .applyParams     (applyParams   ),
    .writeParams     (writeParams   ),
    .CdcDestSetup    (CdcDestSetup  ),
    .ClockBasedDelay (ClockBasedDelay),
    .DestClkCnt      (DestClkCnt    ),
    .AssertEnable    (AssertEnable  ),
    .driveSel        (driveSel      ),
    .andGateModel    (andGateModel  ),
    .HoldCheck       (HoldCheck     ),
    .SrcPath         (SrcPath       )
   )  u_tis_cdcefpm ( 
    .A              (A),
    .EN             (EN),
    .DSTCLK         (DSTCLK),
    `ifdef ABV
    .InputChkDis    (InputChkDis),
    `endif
    .Z              (Z )
    ); 

endmodule

// icd stands for InputChkDis, this module can be used for cases where designer wants to control when to enable/disable
// assertion check. Please be very cautious, the designer is fully responsible for properly controlling InputChkDis,
// and should never be completely disabled.

module techind_iso_cdcefpm_icd ( A, EN, DSTCLK, 
              `ifdef ABV
              InputChkDis,
              `endif
              Z ); 
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  This module has several parameters that can be used to control the simulation behaivour
  ///  of the underlying CDCBUFENCLK model. In addition, some of these parameters may be dumped
  ///  into the simulation log file as a means for mining the instance-specific information
  ///  and presenting it to the physical implementation and timing flows.
  ///
  ///  Here we shall present each of the parameters. After the parameter declarations, a
  ///  description is provided that indicates how each parameter is used.
  parameter   size               = 1;
  parameter   applyParams        = 0;
  parameter   writeParams        = 1;   ///  deprecated. DO NOT USE. Only provided here for temporary backward compatibility.
  parameter   CdcDestSetup       = 11000;
  parameter   ClockBasedDelay    = 0;   ///  deprecated. DO NOT USE. Only provided here for temporary backward compatibility.
  parameter   DestClkCnt         = 0;
  parameter   AssertEnable       = 1;
  parameter   driveSel           = 3;
  parameter   andGateModel       = 1;
  parameter   HoldCheck          = 0;
  parameter   SrcPath            = 0;
  ///    .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .
  ///
  ///  'size'          :  used to indicate the width/size of the bus on which the false-path
  ///        marker is to be instantiated. The default is set for a single-bit. In particular,
  ///        this parameter specifies the bus range [size-1:0] for the 'A' and 'Z' ports.
  ///
  ///
  ///  'applyParams'   :  default 0. When set to '1' the following three parameters are applied
  ///        to the underlying CDCBUFENCLK model using local, relative force statements, to
  ///        control the CDCBUFENCLK behaviour.
  ///
  ///
  ///  'CdcDestSetup'  :  passed along to the CDCBUFENCLK model. Specifies the number of ps
  ///        [picoseconds] after a change at the input 'A' for which the output 'Z' will be
  ///        driven 'X' to model propagation uncertainty, metastability, and glitch risk.
  ///
  ///
  ///  'DestClkCnt'    :  passed along to the CDCBUFENCLK model. Specifies the number of DSTCLK
  ///        rising edges after a change at the 'A' input for which the output 'Z' will be
  ///        driven 'X' to model propagation uncertainty, metastability, and glitch risk.
  ///
  ///
  ///  'AssertEnable'  :  passed along to the CDCBUFENCLK model. Disables the assertion check
  ///        of 'X' on each rising edge of DSTCLK. Note, disabling the assertion invalidates
  ///        the status of "customer synchronizer" normally conferred to the CDCBUFENCLK model.
  ///        Setting a '0' on this parameter will trigger a design audit. Please contact the
  ///        CDCWG (CDC Work Group) for guidance if it is felt that there is a need to set this
  ///        parameter to '0'.
  ///
  ///
  ///  'driveSel'      :  passed along to the library instantiation. Selects an alternate
  ///        drive strength for the library-specific marker cell. Generally, it is expected that
  ///        the default value will be sufficient, but it may be tweaked at the user's discretion
  ///        if better seeding for synthesis is warranted.
  ///
  ///
  ///  'andGateModel'  :  default '1'. When set to '0', the RTL inversion that abstracts the
  ///        CDCBUFENCLK to a two-input AND view is bypassed, thereby exposing the underlying
  ///        two-input NAND view of the actual CDCBUFENCLK. It is expected that the vast majority
  ///        of users will deploy the AND view, but some designers needing lower-level control
  ///        over their data-path can opt out of the AND-rectifying inversion.
  ///
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  input   [size-1:0]  A;
  input               EN;
  input               DSTCLK;
  `ifdef ABV
  input               InputChkDis;
  `endif
  output  [size-1:0]  Z; 
// river translate_off
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  Here we declare an intermediate wire that intercepts the output of the CDC enabled
  ///  false-path marker NAND function. Later, we will optionally invert this term, based
  ///  on a parameter [default selects the inversion], to provide this wrapper with an
  ///  overall AND function. The intention behind representing this module as an AND function
  ///  is to simplify the user deployment. However, for those wishing to hand optimize the
  ///  datapath timing, the inversion can be dispensed with.
  wire    [size-1:0]  ZB;
  ///  In the interest of simplicity, when we are not passing through synthesis [ie. we
  ///  are only simulating, we shall instantiate the CDCBUFENCLK model at this level, passing
  ///  in the size parameter to complete the bussed instantiation.
  ///
  ///  Note, in order to indicate the degree of library support for the simulation model, we
  ///  use a couple of defines around the CDCBUFENCLK instantiation as described previously
  ///  in this file:
  ///
  ///         TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  ///         TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  `ifndef SYNTHESIS
    `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
       CDCBUFENCLK
         #(
            .Setup  ( CdcDestSetup ),
            .Cycles ( DestClkCnt ),
            .size   ( size ),
            .HoldCheck  ( HoldCheck     ),
            .SrcPath    ( SrcPath       )
          )  uefpm  (
            .A      ( A      ),
            .EN     ( EN     ),
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
            .DSTCLK ( DSTCLK ),
           `endif
           `ifdef ABV
            .InputChkDis ( 1'b0 ),
           `endif
            .Z      ( ZB     )
          );
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       ///
       ///  using this wrapper, we support two, mutually exclusive flows for providing a
       ///  single source of timing configuration information into simulations as well as the
       ///  downstream tool flow:
       ///
       ///    1.  the first of these flows is the application of the configuration file as a
       ///        collection of IP top-level force statements that are developed ahead of time.
       ///        When applying this technique, the "applyParams" parameter must be set to 0
       ///        (default value) so that the parameters to this wrapper are not pushed down
       ///        into the target cell. This leaves the controls within the target cell clear
       ///        for the IP top-level force statements to be applied without introducing a
       ///        force conflict. This will allow the configuration values to be applied for
       ///        simulation, and separately delivered to the downstream tool flow.
       ///
       ///    2.  in the absense of an IP top-level force statement, the configuration file
       ///        can be generated from the paramters that are passed into the instance. When
       ///        using this technique, setting the "applyParams" parameter to 1 will apply
       ///        (through forces) the parameters to the target cell. This will allow the
       ///        simulation to use the configuration information propagated via parameters.
       ///
       ///        However, in order to satisfy the downstream tool flow, the parameter
       ///        information must be dumped out. By dumping out the parameter information,
       ///        we can avoid problems of inconsistency between multiple sources for a given
       ///        piece of configuration information. Since we anticipate having many
       ///        thousands of these cells, dumping may take some time, and is expected to
       ///        significantly bloat the simulation log file. In order to avoid the
       ///        simulation impact on the vast majority of the simulations, a command-line
       ///        switch '+logCdcBufTiming' is supported which, when supplied on the run-time
       ///        command-line, will cause the parameters to be dumped out.
       ///
       ///        It should be noted that the means for dumping will include the entire
       ///        hierarchical path, and is not realtive to the IP top-level (or tile top-level).
       ///        In order to deal with this, a post processing phase will be required to lift
       ///        the configuation information and manipulate the path name to provide a top-level
       ///        relative name that can be consumed by the downstream tool flow.
       ///
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       ///
       // synopsys translate_off
       // novas translate_off
       `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
         `ifndef AMDFORCE
           `define AMDFORCE(signal, value) force signal = value;
         `endif
         initial begin
           if ( applyParams ) begin
             if ( $test$plusargs("logCdcBufTiming")) begin
               $display ( "info: CDCBUFCLK %m.uefpm.techindMarkerSize\t = %d : %t", size, $time );
               $display ( "`AMDFORCE(%m.uefpm.CdcDestSetup, %1d);", CdcDestSetup );
               $display ( "`AMDFORCE(%m.uefpm.DestClkCnt, %1d);",   DestClkCnt );
               $display ( "`AMDFORCE(%m.uefpm.AssertEnable, %1d);", AssertEnable );
             end
           end
         end
       `endif  ///  TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
       // novas translate_on
       // synopsys translate_on
       ///
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
    `else  
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       ///
       ///  if the simulation model for the enabled false-path marker is not available, we
       ///  want to have an equivalent functionality present so that our simulations behave in
       ///  the expected manner while we are in the early stages of a project. However, we
       ///  should have our checks in place to ensure that the simulation model is eventually
       ///  selected as early in the program as possible.
       ///
       ///  As mentioned earlier, it is hoped that the presence of the define above will make
       ///  it simpler to identify shortcomings. For instance, greping for the existence of
       ///  such bypasses would yield a list of unsupported features that would need to be
       ///  addressed.
       assign ZB = ~(EN ? A : 0);
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
    `endif  //  TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  `else  
     /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ///
     ///  When we are taking the synthesis route, we provide a layer of indirection that helps
     ///  us resolve a couple of flow issue:
     ///
     ///      o   as part of the flow, we need to propagate the clock connection through synthesis
     ///          but not to the leaf cell. By creating this extra layer, we can preserve the
     ///          wrapper with the clock connection, but it will not actually be connected to the
     ///          leaf cell.
     ///
     ///      o   we can bit-blast the technology-independent wrapper to ensure that the lowest
     ///          level instantiation is performed such that the special 'd0nt_' naming convention
     ///          is not perturbed.
     techind_iso_cdcefpm_single
       #(
          .driveSel       ( driveSel     ),
          .DestClkCnt     ( DestClkCnt     ),
          .CdcDestSetup   ( CdcDestSetup     ),
          .HoldCheck      ( HoldCheck   ),
          .SrcPath        ( SrcPath     )
        )  uefpm [size-1:0]  (
          .A       ( A      ),
          .EN      ( EN     ),
          .DSTCLK  ( DSTCLK ),
          .ZB      ( ZB     )
        );
     /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  `endif  ///  SYNTHESIS
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  Now that we have generated the appropriate substitution for the NAND function of the
  ///  of the CDC enabled false-path marker, we shall [optionally] invert the output, using
  ///  an RTL statement, before sending it out. In this way, we present the more logical overall
  ///  fucnction of an AND gate, but only restrict the path as minimally as possible (the NAND),
  ///  allowing boundary optimizing synthesis to relocate the [soft] RTL inversion along the
  ///  downstream logic path, as required.
  ///
  ///  In this way, we provide the designer with the more logical, non-inverting AND function,
  ///  but do not lock down any more of the logic path than is absolutely necessary.
  ///
  ///  However, for the meticulous designer that needs to hand optimize the datapath, and for
  ///  whom the RTL inversion simply gets in their way, an option has been provided to bypass
  ///  the normalizing inversion and fully expose the raw NAND2 function of the wrapper.
  generate
    if ( andGateModel )  assign  Z = ~ZB;
    else                 assign  Z =  ZB;
  endgenerate
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  // river translate_on
endmodule
