`timescale 1ps/1ps
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Module Name    : techind_mcpm
///  
///  Revision       : $Revision$
///  Release Status : $State$
///  Author         : $Author$
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    ABSTRACT
///
///    This wrapper module implements a technology independent means to embed a bus-width
///    of multi-cycle path markers into RTL code. With appropriate switches, simulation is
///    augmented with the means to aid the verification of multi-cycle path assumptions by
///    incorporating a special simulation model. Also with [other] appropriate switches,
///    synthesis runs that target a specific technology library can call out specific physical
///    marker cells to which the multi-cycle path markers are to be mapped.
///
///    Logically, the model has the function of a buffer or wire with the "A" being the pin
///    through which the multi-cycle path must propagate.
///
///    It should be noted that this wrapper only represents part of a larger flow to ensure
///    that multi-cycle paths are defined and constrained properly. In addition to this wrapper
///    and the cell model contained within, a means to specify timing constraints to both
///    the simulation model as well as the physical design flow must be supported in
///    conjunction with this wrapper, keying off of the 'd0nt_MCPM' instance name.
///
///    In support of the timing constraint flow, we want to preserve the DSTCLK pin on the
///    layer above the actual library marker cell instance. In order to accomplish this, we
///    encapsulate the actual technology specific cell instantiation within a lower-level
///    wrapper onto which we can apply a preserve command. Within this upper-level wrapper,
///    we either instantiate the simulation model, or the bussed instantiation of the
///    lower-level wrapper onto which we shall apply the preserve command.
///
///    For more details, refer to the 'techind_mcpm_single.v' wrapper file.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SYNTHESIS USAGE:
///
///    In order for synthesis to pick up an appropriate marker cell from the target library,
///    the following define must be set within the synthesis setup in order to ensure that
///    the synthesis tool takes the appropriate path through this wrapper code.
///
///        `define  SYNTHESIS
///
///    In addition to the 'SYNTHESIS' define, a technology library must also be specified
///    in order to directly instantiate an appropriate marker cell from the target technology
///    cell library. For a current list of libraries that are supported, please refer to the
///    lower-level wrapper file:
///
///        'techind_mcpm_single.v'
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SIMULATION USAGE:
///
///    When not performing synthesis [ie. the 'SYNTHESIS' define is not set], the compiled
///    code-path is geared for simulation. In this mode, by default, RTL simulations will
///    expect that the MCBUF model exists in the library and will call it out. This
///    behaviour is selected as the default so that a minimally specified defines file will
///    operate in a more aggressive manner. However, in order to accommodate early deployment
///    and portability to a program that does not currently support the MCBUF model in
///    the library, the following define is used to temporarily substitute a simple,
///    equivalent RTL equation.
///
///        `define  GPU__VERIF__NOT_SUPPORTED__MULTI_CYCLE_PATH_MARKER
///
///    Note, this define is explicitly declared to avoid using the MCBUF cell model.
///    It is hoped that the presence of an undesireable, temporary define will be more
///    noticable, and therefore more likely to be corrected than the absense of a define used
///    to include the MCBUF model.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    Copyright (C) 2009,2010,2011
///      by Advanced Micro Devices (AMD) Corporation
///  
///    All rights reserved. This source file is considered proprietary and confidential
///    information of AMD Corp. As such, this source file may not be distributed by any
///    means, in whole, or in part, without the express, written approval of AMD Corp.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Revision History :
///  
///  $Log$
///  
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
module techind_mcpm ( A, DSTCLK, Z ); 
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  This module has several parameters that can be used to control the simulation behaivour
  ///  of the underlying MCBUF model. In addition, some of these parameters may be dumped
  ///  into the simulation log file as a means for mining the instance-specific information
  ///  and presenting it to the physical implementation and timing flows.
  ///
  ///  Here we shall present each of the parameters. After the parameter declarations, a
  ///  description is provided that indicates how each parameter is used.
  parameter   size               = 1;
  parameter   applyParams        = 0;       // deprecated
  parameter   CdcDestSetup       = 11000;
  parameter   DestClkCnt         = 0;
  parameter   driveSel           = 3;
  parameter   mode01             = 0;       // deprecated
  parameter   BufMetaWaiveMode   = 0;
  ///    .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .
  ///
  ///  'size'          :  used to indicate the width/size of the bus on which the multi_cycle path
  ///        marker is to be instantiated. The default is set for a single-bit. In particular,
  ///        this parameter specifies the bus range [size-1:0] for the 'A' and 'Z' ports.
  ///
  ///
  ///  'applyParams'   :  default 0. When set to '1' the following three parameters are applied
  ///        to the underlying MCBUF model using local, relative force statements, to
  ///        control the MCBUF behaviour.
  ///
  ///
  ///  'CdcDestSetup'  :  passed along to the MCBUF model. Specifies the number of ps
  ///        [picoseconds] after a change at the input 'A' for which the output 'Z' will be
  ///        driven 'X' to model propagation uncertainty, metastability, and glitch risk.
  ///
  ///
  ///  'DestClkCnt'    :  passed along to the MCBUF model. Specifies the number of DSTCLK
  ///        rising edges after a change at the 'A' input for which the output 'Z' will be
  ///        driven 'X' to model propagation uncertainty, metastability, and glitch risk.
  ///
  ///
  ///  'driveSel'      :  passed along to the library instantiation. Selects an alternate
  ///        drive strength for the library-specific marker cell. Generally, it is expected that
  ///        the default value will be sufficient, but it may be tweaked at the user's discretion
  ///        if better seeding for synthesis is warranted.
  ///
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  input   [size-1:0]  A;
  input               DSTCLK;
  output  [size-1:0]  Z; 
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  // river translate_off
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  In the interest of simplicity, when we are not passing through synthesis [ie. we
  ///  are only simulating, we shall instantiate the MCBUF model at this level, passing
  ///  in the size parameter to complete the bussed instantiation.
  ///
  ///  Note, in order to indicate the degree of library support for the simulation model, we
  ///  use a couple of defines around the MCBUF instantiation as described previously
  ///  in this file:
  ///
  ///         GPU__VERIF__NOT_SUPPORTED__MULTI_CYCLE_PATH_MARKER
  ///         TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
  `ifndef SYNTHESIS
    `ifndef GPU__VERIF__NOT_SUPPORTED__MULTI_CYCLE_PATH_MARKER
       MCBUF
         #(
            .Setup  ( CdcDestSetup ),
            .Cycles ( DestClkCnt   ),
            .mode01 ( mode01       ),
            .BufMetaWaiveMode ( BufMetaWaiveMode )
          )  umcpm [size-1:0]  (
            .A      ( A      ),
           `ifndef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
            .DSTCLK ( DSTCLK ),
           `endif
            .Z      ( Z      )
          );
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       ///
       ///  using this wrapper, we support two, mutually exclusive flows for providing a
       ///  single source of timing configuration information into simulations as well as the
       ///  downstream tool flow:
       ///
       ///    1.  the first of these flows is the application of the configuration file as a
       ///        collection of IP top-level force statements that are developed ahead of time.
       ///        When applying this technique, the "applyParams" parameter must be set to 0
       ///        (default value) so that the parameters to this wrapper are not pushed down
       ///        into the target cell. This leaves the controls within the target cell clear
       ///        for the IP top-level force statements to be applied without introducing a
       ///        force conflict. This will allow the configuration values to be applied for
       ///        simulation, and separately delivered to the downstream tool flow.
       ///
       ///    2.  in the absense of an IP top-level force statement, the configuration file
       ///        can be generated from the paramters that are passed into the instance. When
       ///        using this technique, setting the "applyParams" parameter to 1 will apply
       ///        (through forces) the parameters to the target cell. This will allow the
       ///        simulation to use the configuration information propagated via parameters.
       ///
       ///        However, in order to satisfy the downstream tool flow, the parameter
       ///        information must be dumped out. By dumping out the parameter information,
       ///        we can avoid problems of inconsistency between multiple sources for a given
       ///        piece of configuration information. Since we anticipate having many
       ///        thousands of these cells, dumping may take some time, and is expected to
       ///        significantly bloat the simulation log file. In order to avoid the
       ///        simulation impact on the vast majority of the simulations, a command-line
       ///        switch '+logCdcBufTiming' is supported which, when supplied on the run-time
       ///        command-line, will cause the parameters to be dumped out.
       ///
       ///        It should be noted that the means for dumping will include the entire
       ///        hierarchical path, and is not realtive to the IP top-level (or tile top-level).
       ///        In order to deal with this, a post processing phase will be required to lift
       ///        the configuation information and manipulate the path name to provide a top-level
       ///        relative name that can be consumed by the downstream tool flow.
       ///
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       ///
       // synopsys translate_off
       // novas translate_off
       initial begin
         if ( $test$plusargs("logCdcBufTiming")) begin
           `ifdef TECH__CDCBUF__DISABLE_BEHAVIORAL_VERILOG
              $display ( "info: WARNING -- MCBUF timing parameters not behaviourally simulated" );
           `endif
           if ( applyParams ) begin
             $display ( "info: MCBUF %m.umcpm.techindMarkerSize\t = %d : %t", size, $time );
             $display ( "`AMDFORCE(%m.umcpm.CdcDestSetup, %1d);", CdcDestSetup );
             $display ( "`AMDFORCE(%m.umcpm.DestClkCnt, %1d);",   DestClkCnt );
           end
         end
       end
       // novas translate_on
       // synopsys translate_on
       ///
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
    `else  
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       ///
       ///  if the simulation model for the multi-cycle path marker is not available, we want to
       ///  have an equivalent functionality present so that our simulations behave in the
       ///  expected manner while we are in the early stages of a project. However, we should
       ///  have our checks in place to ensure that the simulation model is eventually
       ///  selected as early in the program as possible.
       ///
       ///  As mentioned earlier, it is hoped that the presence of the define above will make
       ///  it simpler to identify shortcomings. For instance, greping for the existence of
       ///  such bypasses would yield a list of unsupported features that would need to be
       ///  addressed.
       assign Z = A;
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
    `endif  //  GPU__VERIF__NOT_SUPPORTED__MULTI_CYCLE_PATH_MARKER
  `else  
     /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ///
     ///  When we are taking the synthesis route, we provide a layer of indirection that helps
     ///  us resolve a couple of flow issue:
     ///
     ///      o   as part of the flow, we need to propagate the clock connection through synthesis
     ///          but not to the leaf cell. By creating this extra layer, we can preserve the
     ///          wrapper with the clock connection, but it will not actually be connected to the
     ///          leaf cell.
     ///
     ///      o   we can bit-blast the technology-independent wrapper to ensure that the lowest
     ///          level instantiation is performed such that the special 'd0nt_' naming convention
     ///          is not perturbed.
     techind_mcpm_single
       #(
          .driveSel       ( driveSel     ),
          .DestClkCnt     ( DestClkCnt   ),
          .CdcDestSetup   ( CdcDestSetup )
        )  umcpm [size-1:0]  (
          .A       ( A      ),
          .DSTCLK  ( DSTCLK ),
          .Z       ( Z      )
        );
     /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  `endif  ///  SYNTHESIS
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  // river translate_on
endmodule
