`timescale 1ps/1ps
`include "rtllib_features.vh"
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Module Name    : techind_mcpm_single
///  
///  Revision       : $Revision$
///  Release Status : $State$
///  Author         : $Author$
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    ABSTRACT
///
///    This wrapper module implements a technology independent means to directly instantiate
///    a technology-library false-path marker cell into RTL during synthesis. With appropriate
///    switches, the correct marker cell from the target library is instantiated in the design
///    using a naming convention 'd0nt_MCPM' that is meaningful to the downstream tool flow.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SYNTHESIS USAGE:
///
///    In order for synthesis to pick up an appropriate cell from the target library, the
///    following value must be defined:
///
///        `define  SYNTHESIS
///
///    In addition to the 'SYNTHESIS' define, one technology library must also be specified.
///    This revision of the wrapper currently supports the following libraries [note, this
///    list must be maintained as we move forward from one process to the next in order to
///    ensure that the marker cells are properly instantiated in future designs]:
///
///        `define    GPU__CHIP__TECH__AMD_28SHP
///        `define    RTLLIB__TECH__AMD_28SHP
///        `define    RTLLIB__TECH__AMD_28SHP_HP
///        `define    RTLLIB__TECH__TSMC_28HD
///        `define    RTLLIB__TECH__TSMC_16FF
///        `define    RTLLIB__TECH__GF_14XM
///
///    Note, as was mentioned earlier, this wrapper is only part of a larger flow and should
///    not be instantiated directly by designers. In order to heavily discourage such abuse,
///    when the 'SYNTHESIS' define is not set, this wrapper module will provide no internal 
///    functionality which will yield erroneous simulation results.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    Copyright (C) 2011
///      by Advanced Micro Devices (AMD) Corporation
///  
///    All rights reserved. This source file is considered proprietary and confidential
///    information of AMD Corp. As such, this source file may not be distributed by any
///    means, in whole, or in part, without the express, written approval of AMD Corp.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Revision History :
///  
///  $Log$
///
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
module techind_mcpm_single ( A, DSTCLK, Z ); 
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  parameter   driveSel           = 3;
  parameter   DestClkCnt         = 0;
  parameter   CdcDestSetup       = 11000;
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  input               A;
  input               DSTCLK;
  output              Z; 
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  // river translate_off
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  As indicated earlier, we use a variety of defines to guide synthesis down the desired
  ///  code-path to instantiate the technology-specific marker cell.
  `ifndef SYNTHESIS
     ///  should this path be taken, the designer must have used this wrapper incorrectly. So,
     ///  we are creating a simulation exposure. Note, it would have been preferable to cause
     ///  a hard compile failure, but the VRQ X-uglification tool chokes when we try to do
     ///  that.
  `else  
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     //////                                                        /////////////////////
     //////    THE FOLLOWING CODE SHOULD BE ACTIVELY MAINTAINED    /////////////////////
     //////    AS NEW TECHNOLOGY LIBRARIES ARE TO BE SUPPORTED     /////////////////////
     //////                                                        /////////////////////
     //////    * note, as new 'define values are supported, they   /////////////////////
     //////      should be captured in the 'SYNTHESIS USAGE'       /////////////////////
     //////      documentation at the top of this file.            /////////////////////
     //////                                                        /////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     ///
     ///  For technology libraries that do support a physical cell to represent the
     ///  MCBUF, we instantiate the cell from the library. As an initial guess we [by
     ///  default] select a cell with moderate drive strength. It is expected that the tool
     ///  flow will upsize/downsize the cell as required to optimize the timing path. However,
     ///  we also provide a means to specify an alternate drive strength through a parameter
     ///  in order to better seed the synthesis run.
     ///
     ///  Also, in support of the synthesis and constraint flow, we provide a means to
     ///  communicate back to the flow the need to "preserve" this module. This preservation
     ///  is necessary in order to ensure that the 'DSTCLK' pin can be located and traced
     ///  since it is virtual and does not map onto the actual technology-specific marker cell.
     ///  This is accomplished by setting the following define for the flow to query:
     ///
     ///     SUPRA__FEINT__NEED_PRESERVE_MCPM_SINGLE
     ///
     ///  Note, it is necessary to communicate this information back to the flow so that the
     ///  preserve contstraint is only applied when we have actual technology specific marker
     ///  cells instantiated. Failure to preserve the hierarchy will result in the flow being
     ///  incapable of calculating clock-based delay constraints.
     ///
     ///  by contrast, when the clause with the [default] RTL statement is taken, we do not set
     ///  the above define. Doing so while the RTL is called out will result in a synthesis
     ///  failure since the RTL will not have been mapped to library cells when the preserve
     ///  constraint is applied.
       generate
         if      ( driveSel == 1 )    begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_1 d0nt_MCPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end else if ( driveSel == 2 )    begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end else if ( driveSel == 3 )    begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end else if ( driveSel == 4 )    begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_4 d0nt_MCPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end else                         begin
           case (DestClkCnt)
             0: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C0_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             1: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C1_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             2: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C2_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             3: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C3_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             4: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C4_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             5: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C5_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             6: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C6_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             7: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C7_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             8: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C8_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             9: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C9_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             10: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C10_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             11: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C11_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             12: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C12_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             13: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C13_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             14: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C14_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             15: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C15_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             16: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C16_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             17: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C17_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             18: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C18_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             19: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C19_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             20: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C20_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             21: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C21_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             22: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C22_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             23: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C23_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             24: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C24_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             25: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C25_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             26: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C26_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             27: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C27_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             28: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C28_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             29: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C29_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             30: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C30_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             31: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C31_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             32: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C32_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             33: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C33_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             34: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C34_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             35: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C35_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             36: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C36_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             37: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C37_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             38: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C38_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             39: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C39_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             40: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C40_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             41: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C41_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             42: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C42_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             43: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C43_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             44: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C44_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             45: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C45_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             46: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C46_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             47: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C47_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             48: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C48_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             49: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C49_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
             default: HDBLVT08_BUF_CDC_2 d0nt_MCPM_C_greater_than_50_D[CdcDestSetup:CdcDestSetup] ( .A(A), .X(Z));
           endcase
         end
       endgenerate
       `ifndef SUPRA__FEINT__NEED_PRESERVE_MCPM_SINGLE
         `define  SUPRA__FEINT__NEED_PRESERVE_MCPM_SINGLE
       `endif
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  `endif  ///  SYNTHESIS
  // river translate_on
endmodule
