`timescale 1ns/100ps
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Module Name    : techind_sync
///  
///  Revision       : $Revision$
///  Release Status : $State$
///  Author         : $Author$
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    ABSTRACT
///
///    This wrapper module implements a technology and [almost] implementation independent 
///    means to incorporate a bus-width of synchronizers into an RTL design. With the
///    appropriate switches, simulation is augmented with the ability to aggressively
///    model the effects of metastability and multi-bit race-conditions. With [other]
///    appropriate switches, synthesis runs that target a specific technology library can
///    construct the required synchronizer using the variety of synchronizers available in
///    the target library.
///
///       NOTE: it is the goal of this wrapper module to provide a parametric, one-stop
///       means to instantiate any flavour of synchronizer, at any depth. Ideally, this
///       module would be the only module instantiated in all AMD designs where
///       synchronizers are to b deployed.
///
///    Logically, the basic model operates as an N-stage synchronizer where the first
///    destination-clock capture stage randomizes whether to pick up a recent value-change,
///    or to wait until the next clock cycle. In a multi-bit scenario, some behavioural
///    restrictions [consistent with expectations for the physical implementation] are
///    imposed in order to claw back on the aggressiveness of the randomization effects.
///
///    In addition to the basic N-stage syncrhonizer, a variety of variations are also
///    supported. While these variations will be described in more detail further down in
///    the code, we provide an overview of the vairous options here:
///
///       o  arbitrary bus width
///       o  inclusion of a leading source-flop
///       o  inclusion of a leading negedge [destination clock] sample phase
///       o  arbitrary destination depth specification [in flop-stage equivalents]
///       o  asynchronous set / reset / flush capability
///       o  clock enable support
///       o  scan support -- mux-D or LSSD
///       o  input pulse-width checking
///       o  reduction of randomization sensitivity to a "window" around the sample edge
///
///    It should be noted that this module only represents part of a larger flow to ensure
///    that synchronizers are correctly, and consistently deployed into our designs. In
///    addition to this wrapper and the model contained within, a means to specify
///    relevant timing constraints to both the simulation model as well as the physical
///    design flow are supported by the larger flow, aided by this wrapper.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SYNTHESIS USAGE:
///
///    In order for synthesis to pick up an appropriate synchronizer cell from the target
///    library, the following define must be set within the synthesis setup in order to
///    ensure that the synthesis tool takes the appropriate path through this wrapper code:
///
///        `define  SYNTHESIS
///
///    In addition to the 'SYNTHESIS' define, a technology library must also be specified
///    in order to directly instantiate an appropriate synchronizer from the target
///    technology cell library. For a current list of libraries that are supported, please
///    refer to the lower-level wrapper file:
///
///        'techind_sync_single.v'
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SIMULATION USAGE:
///
///    When not performing synthesis [ie. the 'SYNTHESIS' define is not set], the compiled
///    code-path is geared for simulation. In this mode, by default, RTL simulations will
///    expect that the CDCGENERICSYNC model exists within the simulation libraries [and in
///    particular the 'cdc_stdlib' library] and will call it out. This behaviour is chosen
///    as the default so that a minimally specified defines file will operate in a more
///    aggressive manner.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    Copyright (C) 2011
///      by Advanced Micro Devices (AMD) Corporation
///  
///    All rights reserved. This source file is considered proprietary and confidential
///    information of AMD Corp. As such, this source file may not be distributed by any
///    means, in whole, or in part, without the express, written approval of AMD Corp.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Revision History :
///  
///  $Log$
///  
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

module techind_sync (
    SB,
    RB,
    FLUSH,
    CKsrc,
    CKdst,
    EN,
    Dsrc,
    Qsrc,
    Qdst,
    SC1,
    SC2,
    SE,
    SI,
    SO
  );

  parameter      size           =  1;
  parameter      depth          =  2;
  parameter      sourceFlop     =  0;
  parameter      halfStage      =  0;
  parameter      asyncSet       =  0;
  parameter      asyncReset     =  0;
  parameter      asyncFlush     =  0;
  parameter      clockEnable    =  0;
  parameter      scan_muxD      =  1;
  parameter      scan_LSSD      =  0;
  parameter      MultiVolt      =  0;
  parameter      TransportDelay =  0;
  parameter      TransportCycle =  1;
  parameter      InputCheck     =  1;
  parameter      MetaWindowMode =  0;
  parameter      MetaWindowSize =  0;
  parameter      IgnorePulseNotSampled = 0;
  parameter      HoldCheck = 0;
  parameter      DstClkIdleTime = 0;
  parameter      DstClkIdleCycles = 0;
  parameter      SrcPath = 0;

  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  This module has several parameters that can be used to control the simulation
  ///  behaviour of the underlying CDCGENERICSYNC model. In addition, some of these
  ///  parameters may be dumped into the simulation log file as a means for mining the
  ///  instance-specific information and presenting it to the physical implementation and
  ///  timing flows.
  ///
  ///  Here we shall present each of the parameters. After the parameter declarations, a
  ///  description is provided that indicates how each parameter is used.
  /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  ///
  ///  BASE FUNCTIONAL PARAMETERS :
  ///   ------------------------
  ///
  ///  'size'           :  used to indicate the width/size of the bus on which the
  ///        synchronizer is to be instantiated. The default is set for a single-bit. In
  ///        particular, this parameter specifies the bus range [size-1:0] for the 'Dsrc',
  ///        'Qsrc' [optional], and 'Qdst' ports.
  ///
  ///
  ///  'depth'          :  indicates the total effective depth, in destination clock
  ///        cycles or flop-stage equivalents, of the desired synchronizer. Where the
  ///        specified 'depth' exceeds the size of any of the library cells, this wrapper
  ///        will attempt to construct a chain of smaller synchronizers whose total depth
  ///        satisfies the requirement -- exactly.
  ///        
  ///
  ///  'sourceFlop'     :  indicates that a source-clock flop is required ahead of the
  ///        destination-clock synchronizer stages. By default, we assume this is not
  ///        required.
  ///
  ///
  ///  'halfStage'      :  indicates that the lead sample stage for the syncrhonizer must
  ///        be a negedge stage, for a total of an odd number of latch stages. When this
  ///        parameter is specified, the the actual synchronizer depth is ( 'depth' + 0.5 ).
  ///        By default, we assume this is not required.
  ///
  ///
  ///
  ///  SPECIAL FUNCTIONAL VARIATIONS :
  ///   ---------------------------
  ///
  ///  'asyncSet'       :  indicates that the synchronizer requires an active-low,
  ///        asynchronous set control for all destination latch stages. By default, we
  ///        assume this is not required.
  ///
  ///
  ///  'asyncReset'     :  indicates that the synchronizer requires an active-low,
  ///        asynchronous reset control for all destination latch stages. By default, we
  ///        assume this is not required.
  ///
  ///  'asyncFlush'     :  indicates that the syncrhonizer requires an active-high,
  ///        asynchronous flush control for all destination latch stages. By default, we
  ///        assume this is not required.
  ///
  ///
  ///  'clockEnable'    :  indicates that the synchronizer requires an active-high clock
  ///        enable control for all destination latch stages. By default, we assume this
  ///        is not required. [SHOULD THIS BE ALL, OR JUST THE FIRST FLOP STAGE ???]
  ///
  ///
  ///
  ///  DFT SUPPORT :
  ///   ---------
  ///
  ///  'scan_muxD'      :  indicates that the synchronizer requires support for a mux-D
  ///        style of scan shifting. By default, we assume this is required.
  ///
  ///
  ///  'scan_LSSD'      :  indicates that the synchronizer requires support for LSSD style
  ///        of scan shifting. By default, we assume this is not required.
  ///
  ///
  ///  MULTIVOLTAGE SUPPORT :
  ///   ---------
  ///
  ///  'MultiVolt'      :  indicates that the synchronizer should use an alternate
  ///                      voltage for lower tau.  For example, VDDM in the cores
  ///                      does not go as low as VDD in the slowest P-state.  This
  ///                      feature is in discussion and may not exist for any
  ///                      cell libraries, the parameter defaults to 0.
  ///
  ///
  ///  TIMING AND MODEL BEHAVIOURS :
  ///   -------------------------
  ///
  ///  'TransportDelay' :  specifies the maximum propagation time for the source signal
  ///        to arrive at the destination sampling flop/stage. By default, this time is
  ///        set to zero implying that the propagation time from source to destination is
  ///        less than one destination clock cycle. Where this single destination clock
  ///        propagation time cannot be met, this parameter must specify the actual
  ///        maximum that must be timed to. The value of this will be passed along to the
  ///        synthesis, implemetnation, and timing flows.
  ///
  ///
  ///  'TransportCycle' :  specifies the maximum propagation time for the source signal
  ///        to arrive at the destination sampling flop/stage. By default, this time is
  ///        set to zero implying that the propagation time from source to destination is
  ///        less than one destination clock cycle. Where this single destination clock
  ///        propagation time cannot be met, this parameter must specify the actual
  ///        maximum number of destination clocks that must be timed to. The value of 
  ///        this will be passed along to the synthesis, implemetnation, and timing flows.
  ///
  ///
  ///  'InputCheck'     :  indicates that the model should check the input signal pulse
  ///        duration in order to ensure that it is wide enough to be properly sampled
  ///        in the destintation domain without being "lost" due to the sampling frequency.
  ///        By default, this feature is on, and must be explicitly disabled where this
  ///        check can justifiably be dispensed with.
  ///
  ///
  ///  'MetaWindowMode' :  in rare circumstances, the aggressive behaviour of the
  ///        synchronization randomization is too aggressive. In order to reduce the
  ///        aggressiveness, when this parameter is set, randomization will only occur
  ///        if the signal transitions within a defined period prior to the destination
  ///        clock sample edge. The size of this period is specified by the following
  ///        define:
  ///
  ///            `define  TECH__SYNC__META_WINDOW_SIZE    < time in picoseconds >
  ///
  ///        Note 1: in order to ensure preservation of event ordering, as launched from
  ///        the source clock domain, the size of the specified metastability window must
  ///        be less than one source clock period. As a practical matter, since the value
  ///        is global, the size of the window should be less than the period of the
  ///        highest frequency clock in the design.
  ///
  ///        Note 2: the 'MetaWindowMode' is an instance-specific means to induce the
  ///        less aggresive randomization criteria. As an alternate, a global method for
  ///        inducing the less aggressive behaviour is provided by the following define:
  ///
  ///            `define  TECH__SYNC__META_WINDOW_USE
  ///
  ///        Note 3: Where the randomization is limited to the metastability window
  ///        mechanism, there is a verification risk that the clocking infrastructure may
  ///        not be able to trigger randomization events. In general, for this method to
  ///        be effective, there must be some certainty that the source and destination
  ///        clock edges will sweep past each other at a resolution that is finer than the
  ///        specified metastability window size. This will ensure that signal transitions
  ///        will trespass within the window.
  ///
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  input                 SB;
  input                 RB;
  input                 FLUSH;
  input                 CKsrc;
  input                 CKdst;
  input                 EN;
  input   [(size-1):0]  Dsrc;
  output  [(size-1):0]  Qsrc;
  output  [(size-1):0]  Qdst;
  input                 SC1;
  input                 SC2;
  input                 SE;
  // input   [(size-1):0]  SI;
  // output  [(size-1):0]  SO;
  input     SI;
  output    SO;

    `ifdef ABV

    reg InputChkDis;
    // om  `ifndef SYNTHESIS  // just as a precaution, ABV is not defined for synthesis
    initial InputChkDis = 
`ifdef RTLLIB_HACK_DISABLE_IPWC
  1'b1 
`else
  1'b0 
`endif
; // disabling InputChkDis by default, if needd techind_sync_icd should be used.
    // om  `endif

    `endif

techind_sync_icd #(
   .size          (size          ) ,
   .depth         (depth         ) ,
   .sourceFlop    (sourceFlop    ) ,
   .asyncSet      (asyncSet      ) ,
   .asyncReset    (asyncReset    ) ,
   .asyncFlush    (asyncFlush    ) ,
   .clockEnable   (clockEnable   ) ,
   .scan_muxD     (scan_muxD     ) ,
   .scan_LSSD     (scan_LSSD     ) ,
   .MultiVolt     (MultiVolt     ) ,
   .TransportDelay(TransportDelay) ,
   .TransportCycle(TransportCycle) ,
   .InputCheck    (InputCheck    ) ,
   .MetaWindowMode(MetaWindowMode) ,
   .MetaWindowSize(MetaWindowSize) ,
   .halfStage     (halfStage     ) ,
   .IgnorePulseNotSampled (IgnorePulseNotSampled),
   .HoldCheck     (HoldCheck     ) , 
   .DstClkIdleTime (DstClkIdleTime) , 
   .DstClkIdleCycles (DstClkIdleCycles) ,
   .SrcPath       (SrcPath      )
    ) u_tis_icd (
    .SB    (SB),
    .RB    (RB),
    .FLUSH (FLUSH),
    .CKsrc (CKsrc),
    .CKdst (CKdst),
    .EN    (EN),
    .Dsrc  (Dsrc),
    .Qsrc  (Qsrc),
    .Qdst  (Qdst),
    `ifdef ABV
    .InputChkDis (
`ifdef RTLLIB_HACK_DISABLE_IPWC
  1'b1 
`else 
  InputChkDis 
`endif
),
    `endif
    .SC1   (SC1),
    .SC2   (SC2),
    .SE    (SE),
    .SI    (SI),
    .SO    (SO)
  );

endmodule



// icd stands for InputChkDis, this module can be used for cases where designer wants to control when to enable/disable
// input data pulse width check. Please be very cautious, the designer is fully responsible for properly controlling InputChkDis,
// and should never completely disable pulse width check.

module techind_sync_icd (
    SB,
    RB,
    FLUSH,
    CKsrc,
    CKdst,
    EN,
    Dsrc,
    Qsrc,
    Qdst,
    `ifdef ABV
    InputChkDis,
    `endif
    SC1,
    SC2,
    SE,
    SI,
    SO
  );

  parameter      size           =  1;
  parameter      depth          =  2;
  parameter      sourceFlop     =  0;
  parameter      halfStage      =  0;
  parameter      asyncSet       =  0;
  parameter      asyncReset     =  0;
  parameter      asyncFlush     =  0;
  parameter      clockEnable    =  0;
  parameter      scan_muxD      =  1;
  parameter      scan_LSSD      =  0;
  parameter      MultiVolt      =  0;
  parameter      TransportDelay =  0;
  parameter      TransportCycle =  1;
  parameter      InputCheck     =  1;
  parameter      MetaWindowMode =  0;
  parameter      MetaWindowSize =  0;
  parameter      IgnorePulseNotSampled = 0;
  parameter      HoldCheck = 0;
  parameter      DstClkIdleTime = 0;
  parameter      DstClkIdleCycles = 0;
  parameter      SrcPath = 0;

  input                 SB;
  input                 RB;
  input                 FLUSH;
  input                 CKsrc;
  input                 CKdst;
  input                 EN;
  input   [(size-1):0]  Dsrc;
  output  [(size-1):0]  Qsrc;
  output  [(size-1):0]  Qdst;
`ifdef ABV
  input  InputChkDis; // OVA disable, simulation only
`endif
  input                 SC1;
  input                 SC2;
  input                 SE;
  // input   [(size-1):0]  SI;
  // output  [(size-1):0]  SO;
  input     SI;
  output    SO;
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  // river translate_off
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  In order to provide for a separation from the modelling effort and the technology
  ///  library development, we allow the technology library models to capture the basic,
  ///  logical functionality of the underlying synchronizer cell. For simulation purposes,
  ///  when we are not performing synthesis, we relegate the more verfication oriented
  ///  behaviour to a generic model. 
  ///
  ///  So, when we are not synthesizing our design, we instantiate the 'CDCGENERICSYNC'
  ///  model and pass a number of parameters down into the model in order to guide the
  ///  features as well as the augmented simulation capabilities.
  `ifndef SYNTHESIS
     ///  depending on some of our parametric features, we need to determine how to
     ///  handle some of our inputs. Since we have the use-case information availalble,
     ///  rather than assume the user is going to properly protect our model from rogue
     ///  signals, we shall take it upon ourselves to render any unused inputs inert.
     wire asyncSB    =  asyncSet    ? SB    : 1'b1;
     wire asyncRB    =  asyncReset  ? RB    : 1'b1;
     wire asyncFLUSH =  asyncFlush  ? FLUSH : 1'b0;
     wire clockEN    =  clockEnable ? EN    : 1'b1;
    `ifndef TECH__SYNC__NOT_SUPPORTED__GENERIC_SYNCHRONIZER
       ///  Our 'CDCGENERICSYNC' model is as single model that has the capability of
       ///  modelling all flavours of synchronizers deployed within AMD. Using a variety
       ///  of parameters [and some defines], we can control the features and the
       ///  simulation behaviour to reflect the actual instance requirements / environment.
       CDCGENERICSYNC  #(
           .size           ( size           ),
           .depth          ( depth          ),
           .sourceFlop     ( sourceFlop     ),
           .TransportDelay ( TransportDelay ),
           .TransportCycle ( TransportCycle ),
           .InputCheck     ( InputCheck     ),
           .MetaWindowMode ( MetaWindowMode ),
           .MetaWindowSize ( MetaWindowSize ),
           .halfStage      ( halfStage      ),
           .IgnorePulseNotSampled (IgnorePulseNotSampled),
           .HoldCheck      ( HoldCheck      ), 
           .DstClkIdleTime ( DstClkIdleTime ), 
           .DstClkIdleCycles ( DstClkIdleCycles ),
           .SrcPath        ( SrcPath        )
         )  usync  (
           .SB             ( asyncSB        ),
           .RB             ( asyncRB        ),
           .FLUSH          ( asyncFLUSH     ),
           .CKsrc          ( CKsrc          ),
           .CKdst          ( CKdst          ),
           .EN             ( clockEN        ),
           `ifdef ABV
           .InputChkDis    ( InputChkDis    ),
           `endif
           .Dsrc           ( Dsrc           ),
           .Qsrc           ( Qsrc           ),
           .Qdst           ( Qdst           ),
           .SI             ( SI             ),
           .SO             ( SO             ),
           .SE             ( SE             )
         );
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       ///
       ///  Using this wrapper, we support a parameter based method for providing a single
       ///  source for physical timing information. This timing information is used both by
       ///  the model, and by the downstream physical synthesis/timing flows.
       ///
       ///  However, since the timing information is captured as parameters, we need to
       ///  dump out the information so we can make it available to the downstream tool
       ///  flow.  By dumping out the parameter information, we can avoid problems of
       ///  inconsistency between multiple sources for a given piece of configuration
       ///  information. Since we anticipate having many thousands of these cells, dumping
       ///  may take some time, and is expected to significantly bloat the simulation log
       ///  file. In order to avoid the simulation impact on the vast majority of the
       ///  simulations, a command-line switch '+logCdcBufTiming' is supported which, when
       ///  supplied on the run-time command-line, will cause the parameters to be dumped
       ///  out.
       ///
       ///  It should be noted that the means for dumping will include the entire
       ///  hierarchical path, and is not realtive to the IP top-level (or tile top-level).
       ///  In order to deal with this, a post processing phase will be required to lift
       ///  the configuation information and manipulate the path name to provide a
       ///  top-level relative name that can be consumed by the downstream tool flow.
       ///
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
       ///
       // synopsys translate_off
       // novas translate_off
       initial begin
         if ( $test$plusargs("logCdcBufTiming")) begin
           `ifdef TECH__SYNC__NOT_SUPPORTED__GENERIC_SYNCHRONIZER
              $display ( "info: WARNING -- CDCGENERICSYNC timing parameters not behaviourally simulated" );
           `endif
           $display ( "info: CDCGENERICSYNC %m.usync.techindSyncSize\t = %d : %t", size, $time );
           $display ( "`AMDFORCE(%m.usync.TransportDelay, %1d);", TransportDelay );
           $display ( "`AMDFORCE(%m.usync.TransportCycle, %1d);", TransportCycle );
         end
       end
       // novas translate_on
       // synopsys translate_on
       ///
       ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
    `else  
       ///  If the simulation model is not supported, we cannot hope to do justice by
       ///  providing some form of stand-in functionality -- that is far too complicated
       ///  to express as simple RTL. So, we are going to make two options available. The
       ///  default option is to leave the Dsrc -> Qdst connection open. This open should
       ///  result in simulation failures.
       ///
       ///  Unfortunately, such a dramatic behaviour will likely meet serious opposition
       ///  from program teams. So, in order to preserve a push for 'forward progress', we
       ///  all a means to model the syncrhonizeer as a trivial wire -- through an
       ///  explicit `define.
      `ifdef TECH__SYNC__BYPASS__GENERIC_SYNCHRONIZER
         assign  Qdst = Dsrc;
         generate
           if ( sourceFlop )  assign  Qsrc = Dsrc;
         endgenerate
      `else  
         `ifdef RTL_ONLY
	        reg   [(size-1):0] d_d1;
	        reg   [(size-1):0] qout;
	        wire  [(size-1):0] dataIn;   
            assign  Qdst = qout;
            if ( sourceFlop ) begin
	       reg   [(size-1):0]         qSource;
	       always @( posedge CKsrc )  qSource <= Dsrc;
	       assign  Qsrc = qSource;
	       assign  dataIn = qSource;
	    end else begin
	       assign  Qsrc = {size{1'bz}};
	       assign  dataIn = Dsrc;
	    end
            generate
              if(depth==2) begin
		 if(asyncFlush) begin
	            reg   [(size-1):0] qout_b;
	            reg [(size-1):0]   d_d1b;
                    always @* begin
		       if(~asyncSB) begin
			  d_d1b  <= {size{1'b1}};
			  d_d1   <= {size{1'b1}};
			  qout_b <= {size{1'b1}};
			  qout   <= {size{1'b1}};
		       end else if(~asyncRB) begin
			  d_d1b  <= {size{1'b0}};
			  d_d1   <= {size{1'b0}};
			  qout_b <= {size{1'b0}};
			  qout   <= {size{1'b0}};
		       end else begin
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d1b  <= dataIn;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d1   <= d_d1b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) qout_b <= d_d1;
			  if( (CKdst & clockEN) | asyncFLUSH ) qout   <= qout_b;
		       end // else: !if(~asyncRB)
                    end
		 end else begin
                    always @(posedge CKdst or negedge asyncSB or negedge asyncRB) begin
                       if(~asyncSB) begin
			  d_d1 <= {size{1'b1}};
			  qout <= {size{1'b1}};
                       end else if(~asyncRB) begin
			  d_d1 <= {size{1'b0}};
			  qout <= {size{1'b0}};
                       end else if(clockEN) begin
			  d_d1 <= dataIn;
			  qout <= d_d1;
                       end
                    end // always @ (posedge CKdst or negedge asyncSB or negedge asyncRB)
		 end // else: !if(asyncFlush)
              end else if(depth==3) begin
	             reg   [(size-1):0] d_d2;
		 if(asyncFlush) begin
	            reg   [(size-1):0] qout_b;
	            reg [(size-1):0]   d_d2b;
	            reg [(size-1):0]   d_d1b;
                    always @* begin
		       if(~asyncSB) begin
			  d_d1b  <= {size{1'b1}};
			  d_d1   <= {size{1'b1}};
			  d_d2b  <= {size{1'b1}};
			  d_d2   <= {size{1'b1}};
			  qout_b <= {size{1'b1}};
			  qout   <= {size{1'b1}};
		       end else if(~asyncRB) begin
			  d_d1b  <= {size{1'b0}};
			  d_d1   <= {size{1'b0}};
			  d_d2b  <= {size{1'b0}};
			  d_d2   <= {size{1'b0}};
			  qout_b <= {size{1'b0}};
			  qout   <= {size{1'b0}};
		       end else begin
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d1b  <= dataIn;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d1   <= d_d1b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d2b  <= d_d1;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d2   <= d_d2b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) qout_b <= d_d2;
			  if( (CKdst & clockEN) | asyncFLUSH ) qout   <= qout_b;
		       end // else: !if(~asyncRB)
                    end
		 end else begin
                    always @(posedge CKdst or negedge asyncSB or negedge asyncRB) begin
                       if(~asyncSB) begin
			  d_d1 <= {size{1'b1}};
			  d_d2 <= {size{1'b1}};
			  qout <= {size{1'b1}};
                       end else if(~asyncRB) begin
			  d_d1 <= {size{1'b0}};
			  d_d2 <= {size{1'b0}};
			  qout <= {size{1'b0}};
                       end else if(clockEN) begin
			  d_d1 <= dataIn;
			  d_d2 <= d_d1;
			  qout <= d_d2;
                       end
                    end // always @ (posedge CKdst or negedge asyncSB or negedge asyncRB)
		 end // else: !if(asyncFlush)
              end else if(depth==4) begin
	             reg   [(size-1):0] d_d2;
	             reg   [(size-1):0] d_d3;
		 if(asyncFlush) begin
	            reg   [(size-1):0] qout_b;
	            reg [(size-1):0]   d_d3b;
	            reg [(size-1):0]   d_d2b;
	            reg [(size-1):0]   d_d1b;
                    always @* begin
		       if(~asyncSB) begin
			  d_d1b  <= {size{1'b1}};
			  d_d1   <= {size{1'b1}};
			  d_d2b  <= {size{1'b1}};
			  d_d2   <= {size{1'b1}};
			  d_d3b  <= {size{1'b1}};
			  d_d3   <= {size{1'b1}};
			  qout_b <= {size{1'b1}};
			  qout   <= {size{1'b1}};
		       end else if(~asyncRB) begin
			  d_d1b  <= {size{1'b0}};
			  d_d1   <= {size{1'b0}};
			  d_d2b  <= {size{1'b0}};
			  d_d2   <= {size{1'b0}};
			  d_d3b  <= {size{1'b0}};
			  d_d3   <= {size{1'b0}};
			  qout_b <= {size{1'b0}};
			  qout   <= {size{1'b0}};
		       end else begin
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d1b  <= dataIn;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d1   <= d_d1b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d2b  <= d_d1;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d2   <= d_d2b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d3b  <= d_d2;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d3   <= d_d3b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) qout_b <= d_d3;
			  if( (CKdst & clockEN) | asyncFLUSH ) qout   <= qout_b;
		       end // else: !if(~asyncRB)
                    end // always @ *
		 end else begin // if (asyncFlush)
                    always @(posedge CKdst or negedge asyncSB or negedge asyncRB) begin
                       if(~asyncSB) begin
			  d_d1 <= {size{1'b1}};
			  d_d2 <= {size{1'b1}};
			  d_d3 <= {size{1'b1}};
			  qout <= {size{1'b1}};
                       end else if(~asyncRB) begin
			  d_d1 <= {size{1'b0}};
			  d_d2 <= {size{1'b0}};
			  d_d3 <= {size{1'b0}};
			  qout <= {size{1'b0}};
                       end else if(clockEN) begin
			  d_d1 <= dataIn;
			  d_d2 <= d_d1;
			  d_d3 <= d_d2;
			  qout <= d_d3;
                       end
                    end
		 end // else: !if(asyncFlush)
              end else if(depth==5) begin
	             reg   [(size-1):0] d_d2;
	             reg   [(size-1):0] d_d3;
	             reg   [(size-1):0] d_d4;
		 if(asyncFlush) begin
	            reg   [(size-1):0] qout_b;
	            reg [(size-1):0]   d_d4b;
	            reg [(size-1):0]   d_d3b;
	            reg [(size-1):0]   d_d2b;
	            reg [(size-1):0]   d_d1b;
                    always @* begin
		       if(~asyncSB) begin
			  d_d1b  <= {size{1'b1}};
			  d_d1   <= {size{1'b1}};
			  d_d2b  <= {size{1'b1}};
			  d_d2   <= {size{1'b1}};
			  d_d3b  <= {size{1'b1}};
			  d_d3   <= {size{1'b1}};
			  d_d4b  <= {size{1'b1}};
			  d_d4   <= {size{1'b1}};
			  qout_b <= {size{1'b1}};
			  qout   <= {size{1'b1}};
		       end else if(~asyncRB) begin
			  d_d1b  <= {size{1'b0}};
			  d_d1   <= {size{1'b0}};
			  d_d2b  <= {size{1'b0}};
			  d_d2   <= {size{1'b0}};
			  d_d3b  <= {size{1'b0}};
			  d_d3   <= {size{1'b0}};
			  d_d4b  <= {size{1'b0}};
			  d_d4   <= {size{1'b0}};
			  qout_b <= {size{1'b0}};
			  qout   <= {size{1'b0}};
		       end else begin
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d1b  <= dataIn;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d1   <= d_d1b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d2b  <= d_d1;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d2   <= d_d2b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d3b  <= d_d2;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d3   <= d_d3b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) d_d4b  <= d_d3;
			  if( (CKdst & clockEN) | asyncFLUSH ) d_d4   <= d_d4b;
			  if(~(CKdst & clockEN) | asyncFLUSH ) qout_b <= d_d4;
			  if( (CKdst & clockEN) | asyncFLUSH ) qout   <= qout_b;
		       end // else: !if(~asyncRB)
                    end // always @ *
		 end else begin // if (asyncFlush)
                    always @(posedge CKdst or negedge asyncSB or negedge asyncRB) begin
                       if(~asyncSB) begin
			  d_d1 <= {size{1'b1}};
			  d_d2 <= {size{1'b1}};
			  d_d3 <= {size{1'b1}};
			  d_d4 <= {size{1'b1}};
			  qout <= {size{1'b1}};
                       end else if(~asyncRB) begin
			  d_d1 <= {size{1'b0}};
			  d_d2 <= {size{1'b0}};
			  d_d3 <= {size{1'b0}};
			  d_d4 <= {size{1'b0}};
			  qout <= {size{1'b0}};
                       end else if(clockEN) begin
			  d_d1 <= dataIn;
			  d_d2 <= d_d1;
			  d_d3 <= d_d2;
			  d_d4 <= d_d3;
			  qout <= d_d4;
                       end
                    end // always @ (posedge CKdst or negedge asyncSB or negedge asyncRB)
		 end // else: !if(asyncFlush)
	      end // if (depth==5)
            endgenerate
          `endif  ///  RTL_ONLY
      `endif  ///  TECH__SYNC__BYPASS__GENERIC_SYNCHRONIZER
    `endif    ///  TECH__SYNC__NOT_SUPPORTED__GENERIC_SYNCHRONIZER
  `else  
     /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ///
     ///  When we are taking the synthesis route, we provide a layer of indirection that
     ///  allows us to deploy a fairly generic solution. This solution allows us to
     ///  accommodate a number of abstractions, including:
     ///
     ///     o  in addition to abstracting the technology mapping, we also provide a
     ///        one-stop feature selection
     ///
     ///     o  the actual total synchronizer depth can be 'dialed' in, and we provide a
     ///        means to deliver the requirement [assuming the target technology library
     ///        has adequate support]. Where multiple cells are chained to realize the
     ///        top-level requirement, the mechanism is abstracted from the designer.
     ///
     ///     o  we bit-blast the wrapper instantiation so that the low-level 'd0nt_'
     ///        naming convention is not perturbed when we pass through synthesis.
     generate 
     if(size>1)
     begin
        wire [size-1:0] si_bus;
        wire [size-1:0] so_bus; // left floating

        assign si_bus = {size{1'b0}};

     techind_sync_single  #(
         .depth          ( depth          ),
         .sourceFlop     ( sourceFlop     ),
         .halfStage      ( halfStage      ),
         .asyncSet       ( asyncSet       ),
         .asyncReset     ( asyncReset     ),
         .asyncFlush     ( asyncFlush     ),
         .clockEnable    ( clockEnable    ),
         .TransportCycle ( TransportCycle ),
         .TransportDelay ( TransportDelay ),
         .scan_muxD      ( scan_muxD      ),
         .scan_LSSD      ( scan_LSSD      ),
         .MultiVolt      ( MultiVolt      ),
         .HoldCheck      ( HoldCheck      ),
         .SrcPath        ( SrcPath        )
       )  usync[(size-1):0]  (
         .SB     ( SB    ),
         .RB     ( RB    ),
         .FLUSH  ( FLUSH ),
         .CKsrc  ( CKsrc ),
         .CKdst  ( CKdst ),
         .EN     ( EN    ),
         .Dsrc   ( Dsrc  ),
         .Qsrc   ( Qsrc  ),
         .Qdst   ( Qdst  ),
         .SC1    ( SC1   ),
         .SC2    ( SC2   ),
         .SE     ( SE    ),
         .SI     ( si_bus),
         .SO     ( so_bus)
       );

     end

     else
     techind_sync_single  #(
         .depth          ( depth          ),
         .sourceFlop     ( sourceFlop     ),
         .halfStage      ( halfStage      ),
         .asyncSet       ( asyncSet       ),
         .asyncReset     ( asyncReset     ),
         .asyncFlush     ( asyncFlush     ),
         .clockEnable    ( clockEnable    ),
         .TransportCycle ( TransportCycle ),
         .TransportDelay ( TransportDelay ),
         .scan_muxD      ( scan_muxD      ),
         .scan_LSSD      ( scan_LSSD      ),
         .MultiVolt      ( MultiVolt      ),
         .HoldCheck      ( HoldCheck      ),
         .SrcPath        ( SrcPath        )
       )  usync[(size-1):0]  (
         .SB     ( SB    ),
         .RB     ( RB    ),
         .FLUSH  ( FLUSH ),
         .CKsrc  ( CKsrc ),
         .CKdst  ( CKdst ),
         .EN     ( EN    ),
         .Dsrc   ( Dsrc  ),
         .Qsrc   ( Qsrc  ),
         .Qdst   ( Qdst  ),
         .SC1    ( SC1   ),
         .SC2    ( SC2   ),
         .SE     ( SE    ),
         .SI     ( 1'b0  ),
         .SO     ( SO    )
       );
     endgenerate
     /// .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
  `endif  ///  SYNTHESIS
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  // river translate_on
endmodule
