`timescale 1ns/100ps
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Module Name    : techind_syncWrapper
///  
///  Revision       : $Revision$
///  Release Status : $State$
///  Author         : $Author$
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    ABSTRACT
///
///  This module implements a technology-independent, and implementation independent means
///  for constructing a single-bit synchronizer of arbitrary depth, and instantiating it
///  into RTL during synthesis. With appropriate switches [parameters and `defines], a
///  chain of one or more synchronizers will be assembled from cells in the target library.
///
///  It should be noted that this module is not a complete solution, it is merely an
///  intermediate block in the overall flow. The actual responsibility of this module is
///  to determine the collection of synchronizers [in terms of their depths] that should
///  be deployed to realize the desired, arbitrary syncrhonizer depth. While the algorithms
///  in this module are subject to technology-specific information, the actual cell mapping
///  is accomplished by a lower-level synchronizer wrapper.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SYNTHESIS USAGE:
///
///    Since this module should generally only be employed during the synthesis mapping
///    phase, the code for this module is only active when the following value is defined:
///
///        `define  SYNTHESIS
///
///    However, in order for the algorithm contained within this module to operate
///    correctly, it is necessary to provide some technology specific information to the
///    routines. So, in addition to the 'SYNTHESIS' define, one technology library must
///    also be defined. This revision of the wrapper currently supports the following
///    libraries [note, this list must be maintained as we move forward from one process
///    to the next in order to ensure that synchronizer chains continue to be constructed
///    properly in future designs]:
///
///        `define  < no libraries are currently supported >
///
///    Note, as was mentioned earlier, this wrapper is only part of a larger flow and
///    should not be instantiated directly by designers. In order to heavily discourage
///    such abuse, when the 'SYNTHESIS' define is not set, this wrapper module will
///    provide no internal  functionality which will yield erroneous simulation results.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    Copyright (C) 2011
///      by Advanced Micro Devices (AMD) Corporation
///  
///    All rights reserved. This source file is considered proprietary and confidential
///    information of AMD Corp. As such, this source file may not be distributed by any
///    means, in whole, or in part, without the express, written approval of AMD Corp.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Revision History :
///  
///  $Log$
///  
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
module  techind_sync_single (
    SB,
    RB,
    FLUSH,
    CKsrc,
    CKdst,
    EN,
    Dsrc,
    Qsrc,
    Qdst,
    SC1,
    SC2,
    SE,
    SI,
    SO
  );
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  parameter  depth       = 2;
  parameter  sourceFlop  = 0;
  parameter  halfStage   = 0;
  parameter  asyncSet    = 0;
  parameter  asyncReset  = 0;
  parameter  asyncFlush  = 0;
  parameter  clockEnable = 0;
  parameter  TransportCycle = 0;
  parameter  TransportDelay = 0;
  parameter  scan_muxD   = 0;
  parameter  scan_LSSD   = 0;
  parameter  MultiVolt   = 0;
  parameter  HoldCheck   = 0;
  parameter  SrcPath     = 0;
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  input      SB;
  input      RB;
  input      FLUSH;
  input      CKsrc;
  input      CKdst;
  input      EN;
  input      Dsrc;
  output     Qsrc;
  output     Qdst;
  input      SC1;
  input      SC2;
  input      SE;
  input      SI;
  output     SO;
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  // river translate_off
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  As indicated earlier, we use a variety of defines to guide synthesis down the
  ///  desired code-path to instantiate an appropriate collection of technology-specific 
  ///  synchronizers.
  `ifndef SYNTHESIS
     ///  should this path be taken, the designer must have used this wrapper incorrectly.
     ///  So, we are creating a simulation exposure. Note, it would have been preferable
     ///  to cause a hard compile failure, but the VRQ X-uglification tool chokes when we
     ///  try to do that.
  `else  

/// +++++++++++++ START OF FPGA EMULATION CODE +++++++++++++++++++++++++++++

          // for fpga and emulation purpose we need synthesizable rtl code.
          // the below code will be enabled only when SYNTHEIS and FPGA_EMULATION are defined
       `ifdef FPGA_EMULATION
          // the implementation is generic and implements all varities of flops in sync cell with the same code
          // varieties meaning, AS, AR, FLUSH. In asic technology cells, each variety or a combination of them
          // will be a seperate sync cell. since we just care about functionality this should do.
          reg  Dsrc_int;
          wire Qdst;
          reg  Qsrc;

          reg  [depth-1:0]     stages;

          always @ (posedge CKsrc or negedge SB or negedge RB )
          // so that Dsrc_int gets initialized during simulation start, we use SB and RB,
          // though they may not be the same for CKsrc. And we are not checking for AF,AS,AR even.
          // since we dont want to use initial statements which are synthesis friendly and not all sync
          // cells will use source flop, hence the precaution. SHould not in any way affect the functional part,
          // if it does file a UBTS.
            if      ( (~SB) )      Dsrc_int <= 1'b1;
            else if ( (~RB) )      Dsrc_int <= 1'b0;
            else                   Dsrc_int <= Dsrc;

          always @ (Dsrc or Dsrc_int)
           if (sourceFlop)
           Qsrc <= Dsrc_int;
           else
           Qsrc <= Dsrc;

          always @( posedge CKdst or negedge SB or negedge RB or posedge FLUSH) 
            if      ( (~SB) )      stages <= {depth{1'b1}};
            else if ( (~RB) )      stages <= {depth{1'b0}};
            else if ( FLUSH )      stages <= {depth{Dsrc}};
            else                   stages <= {Qsrc,stages[(depth-1):1]};
        // in the above, we are not looking for a edge detection of async*, since they should be static
        // and defined/assigned in techind_sync usage itself by respective parameters. 

          assign  Qdst = stages[0];
       `else
        // end of ifdef FPGA_EMULATION

/// +++++++++++++ END OF FPGA EMULATION CODE +++++++++++++++++++++++++++++


     ////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     //////                                                        //////////////////////
     //////    THE FOLLOWING CODE SHOULD BE ACTIVELY MAINTAINED    //////////////////////
     //////    AS NEW TECHNOLOGY LIBRARIES ARE TO BE SUPPORTED     //////////////////////
     //////                                                        //////////////////////
     //////    * note, as new 'define values are supported, they   //////////////////////
     //////      should be captured in the 'SYNTHESIS USAGE'       //////////////////////
     //////      documentation at the top of this file.            //////////////////////
     //////                                                        //////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     ///
     ///  In order for this module to function properly, we need to identify, for each
     ///  supported technology library, all of the synchronizer depths [as measured in
     ///  full flop-equivalent stages] that are supported within that particular cell
     ///  library. When we have identified the supported synchronizer depths, the
     ///  algorithms below can determine the appropriate collection of synchronizer cells
     ///  to achieve the desired total synchronizer depth.
     ///
     ///  The means for communicating the depth information is through a set of defines,
     ///  each of which identifies the ability to support a particular synchronizer depth.
     ///  For this purpose, a number of defines are supported by this module as a means of
     ///  providing pertinent information as to the collection of cells that this module
     ///  has to work with, that fit the required feature set. Each such supported define
     ///  fits the following format:
     ///
     ///      `define  TECH__SYNC__{FEATURES}__SUPPORT_DEPTHn
     ///
     ///  Where the 'n' in 'DEPTHn' can be any integer value greater than or equal to 2,
     ///  and where '{FEATURES}' is a collection of two-character feature specification
     ///  fields that satisfy the following format:
     ///
     ///      '{FEATURES}'  =  'SF_HS_{AF,AR}_AS_CE_{SD,SL}'
     ///
     ///  When a feature is present, the two-character identifier [will be discussed
     ///  shortly] is provided. when a feature is absent, the two character field shall
     ///  be replaced with '__' [to preserve feature identifier alignment]. Since some
     ///  features are mutually exclusive [such as async flush vs. async set/reset;
     ///  mux-D scan vs. LSSD scan] certain positions are 'overloaded' to keep the
     ///  character count down a little.
     ///
     ///     o  'SF'  indicates that a source-flop is supported
     ///     o  'HS'  indicates that a negedge sample, or 'halfStage' is supported
     ///     o  'AF'  indicates that an asynchronous flush is supported
     ///     o  'AR'  indicates that an asynchronous reset is supported [active low]
     ///     o  'AS'  indicates that an asynchronous set is supported [active low]
     ///     o  'CE'  indicates that a destination clock-enable is supported
     ///     o  'SD'  indicates that mux-D style of scan is supported
     ///     o  'SL'  indicates that LSSD style of scan is supported
     ///
     ///  Thus, we need to maintain a set of code that, in conjunction with lower-level
     ///  'implementation' wrappers, will identify what synchronizer depths we support
     ///  in each of our technology libraries.
     ///
     ///  ATTENTION : As noted in the algorithms below, an inability to support the
     ///  requested total synchronizer depth will degenerate into this module being mapped
     ///  as a simple wire from Dsrc to Qdst. This mapping behaviour holds true when the
     ///  library being used for synthesis is not represented in the following "actively
     ///  maintained code" section.
     ///
     ///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     ///
     ///  The heart of our ability to construct a synchronizer of arbitrary length from a
     ///  set of smaller synchronizers lies in an algorithm capable of finding such a
     ///  collection. The function below implements one such algorithm, but with a number
     ///  of simplifying considerations to keep the code from becoming too unwieldy.
     ///
     ///  The major simplification that this algorithms uses to construct an arbitrary
     ///  depth synchronizer is that of the many ways to construct it, only two depths
     ///  shall be used such that the following equation is satisfied:
     ///
     ///       ( N * depth1 ) + ( M * depth2 ) = depth
     ///
     ///  As a further restriction to minimise the solution-space, depth1 and depth2 must
     ///  be adjacent in the order of supported synchronizer depths. So, for instance, if
     ///  depths 2,3, and 5 are supported, 2/3 and 3/5 are valid combinations for 
     ///  depth1/depth2. 2/5, while plausible, is not supported by the algorithm, and will
     ///  not be considered.
     ///
     ///  As a general rule, subject to the simplifications described above, the algorithm
     ///  seeks to construct the desired depth with the largest atomic units available.
     function automatic  [31:0]  syncCellCount;
       input   [31:0] totalDepth;
       input   [31:0] syncDepth;
       reg            sizingError;        ///  set if we cannot satisfy the depth requirement
       reg     [31:0] syncSupport;        ///  a vector to indicate what base sync depths we support
       integer        instCount [31:0];   ///  instance counts for each supported depth
       integer        currDepth;          ///  the running depth accumulation
       integer        currSize;           ///  the current synchronizer depth to be added
       integer        altSize;            ///  the [deeper] syncrhonizer that we can trade
       integer        ilp;                ///  a simple loop variable
       reg            SF, HS, CE, SD;     ///  two-character acronyms for the feature selection,
       reg            AF, AR, AS, SL;     ///   used to improve code density / readability
       begin
         syncSupport = 32'h0;
         SF = sourceFlop;   AF = asyncFlush;
         HS = halfStage;    AR = asyncReset;
         CE = clockEnable;  AS = asyncSet;
         SD = scan_muxD;    SL = scan_LSSD;
         ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
         ///
         ///  Through the library definitions in the "actively maintained code" section
         ///  earlier in this file, we maintain a specification, for each library, of what
         ///  synchronizer depths are supported. Based on those synchronizer depths, we
         ///  record the support capabilities in our 'syncSupport' vector so that we may
         ///  parametrically detect what our algorithm has to work with.
         ///
         ///  Essentially, we are converting individual support information into a vector
         ///  so that we can index it like a lookup table -- to facilitate looping.
         ///
         ///  Note, since we break down our depth support right down to individual feature
         ///  sets, the code associated with this particular section is fairly lengthy. In
         ///  order to ease the pain associated with this, we have pre-converted the more
         ///  descriptive feature parameters into two-character identifiers that match
         ///  those deployed in the library support naming convention. Hopefully the rest
         ///  is straightforward.
         ///
         ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .

         ///sync2 flop definitions
         if(( SF && !HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF && !HS && !AS && !AR &&  AF && !CE && !SD && !SL) ||
            (!SF && !HS && !AS && !AR && !AF && !CE && !SD && !SL) ||
            (!SF && !HS && !AS && !AR && !AF && !CE        && !SL) ||
            ( SF &&  HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF &&  HS && !AS && !AR &&  AF && !CE        && !SL) ||
            (!SF &&  HS && !AS && !AR &&  AF && !CE        && !SL) ||
            (!SF &&  HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF &&  HS && !AS &&  AR && !AF && !CE        && !SL) ||
            (!SF &&  HS &&  AS && !AR && !AF && !CE        && !SL) ||
            (!SF && !HS && !AS &&  AR && !AF && !CE        && !SL) ||
            (!SF && !HS && !AS &&  AR && !AF && !CE        && !SL) ||
            (!SF && !HS &&  AS && !AR && !AF && !CE        && !SL)) begin
               if(depth==2) begin
                  syncSupport[2] = 1'b1;
               end
               if (depth==4) begin
                 syncSupport[4] = 1'b1;
               end
               if (depth==5) begin
                 syncSupport[5] = 1'b1;
               end
         end

         ///sync3 flop definitions
         if(( SF && !HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF && !HS && !AS && !AR &&  AF && !CE && !SD && !SL) ||
            (!SF && !HS && !AS && !AR &&  AF && !CE        && !SL) ||
            (!SF && !HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF && !HS &&  AS && !AR && !AF        && !SD && !SL) ||
            (!SF && !HS && !AS &&  AR && !AF        && !SD && !SL) ||
            (!SF && !HS &&  AS && !AR && !AF               && !SL) ||
            (!SF && !HS && !AS &&  AR && !AF               && !SL) ||
            ( SF &&  HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF &&  HS && !AS && !AR &&  AF && !CE        && !SL) ||
            (!SF &&  HS && !AS && !AR && !AF && !CE        && !SL)) begin
               if(depth==3) begin
                  syncSupport[3] = 1'b1;
               end
               if (depth==4) begin
                 syncSupport[4] = 1'b1;
               end
               if (depth==5) begin
                 syncSupport[5] = 1'b1;
               end
         end

         ///sync4 flop definitions
         if(( SF && !HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF && !HS && !AS && !AR &&  AF && !CE && !SD && !SL) ||
            (!SF && !HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF && !HS &&  AS && !AR && !AF        && !SD && !SL) ||
            (!SF && !HS && !AS &&  AR && !AF        && !SD && !SL)) begin
               if (depth==4) begin
                 syncSupport[4] = 1'b1;
               end
               if (depth==5) begin
                 syncSupport[5] = 1'b1;
               end
         end

         ///sync5 flop definitions
         if(( SF && !HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF && !HS && !AS && !AR &&  AF && !CE && !SD && !SL) ||
            (!SF && !HS && !AS && !AR && !AF && !CE        && !SL) ||
            (!SF && !HS && !AS && !AR && !AF && !CE && !SD && !SL)) begin
               if (depth==5) begin
                 syncSupport[5] = 1'b1;
               end
         end

         ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
         ///
         ///  Now, we initialize our instance counters to zero to reflect that we have not
         ///  yet done any work on figuring out how many of what we need.
         ///
         ///  Note, we algorithmically support base synchronizers up to a depth of 31 flop
         ///  stages. Essentially, this is overkill to ensure that we do not have to visit
         ///  this routine again -- assuming our selection algorithm stands the test of
         ///  time.
         for ( ilp=0; ilp < 32; ilp=ilp+1 ) instCount[ilp] = 0;
         ///  And now we set up some initial value assumptions before we commence with our
         ///  algorithm.
         sizingError     = 0;
         currDepth       = 0;
         currSize        = 31;
         altSize         = 31;
         ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
         ///
         ///  Starting with the largest base synchronizer that we support, we build up a
         ///  chain that is at least as long as the total depth that we require. From that
         ///  point, if necessary, we swap out synchronizers for shallower ones [we will
         ///  get to that shortly]. Even if we are in mid-algorithm, if we have traded
         ///  down such that our current depth is less than the required depth, we need to
         ///  build it back up to at least the required depth.
         ///
         ///  And so, the general algorithm is, build a chain at least as deep as our
         ///  requirement,  trade down if too large, build up if too small, trade down if
         ///  too large, etc. Eventually, we expect to converge on a solution. However, if
         ///  we are unable to converge on a solution, we declare a "sizing error" which
         ///  will result in our zeroing out all counts.
         // leda W182 off
         while (( currDepth != totalDepth ) & ~sizingError ) begin
         // leda W182 on
           if ( currDepth < totalDepth ) begin
             ///  If we get down to a size of zero, our algorithm must not have been able
             ///  to construct the required depth. So, we shall record the error, which
             ///  will cause us to exit.
             if      ( currSize == 0         ) sizingError = 1;
             else if ( syncSupport[currSize] ) begin
               currDepth           = currDepth + currSize;         ///  here is where we build up our
               instCount[currSize] = instCount[currSize] + 1;      ///   chain to at least the required
               if ( instCount[altSize] == 0 )  altSize = currSize; ///    depth.
             ///  Should we not support the current synchronizer size, we continue by
             ///  searching for the next largest supported synchronizer. Recall above, we
             ///  error-out if we reach a size of zero.
             end else  currSize    = currSize - 1;
           end
           ///  As indicated previously, we alternate between building up and trading down
           ///  in order to converge on a solution. Here is the code that is generally
           ///  responsible for trading down.
           if ( currDepth > totalDepth ) begin
             ///  Again, if we get down to a size of zero, our algorithm must not have
             ///  been able to construct the required depth. We shall record the error,
             ///  which will cause us to exit.
             if      ( currSize == 0         ) sizingError = 1;
             ///  In order for us to trade, we need to be considering two different depths.
             ///  So, if we are not, we must search for the next deepest supported
             ///  synchronizer.
             else if ( altSize == currSize   ) currSize = currSize - 1;
             else if ( syncSupport[currSize] ) begin
               if ( instCount[altSize] ) begin                    ///  As long as we have something to
                 currDepth = currDepth - ( altSize - currSize );  ///   trade off, we can continue to 
                 instCount[altSize]  = instCount[altSize]  - 1;   ///    collapse our chain. When we run
                 instCount[currSize] = instCount[currSize] + 1;   ///     out of stuff to trade, we must
               end else      altSize = currSize;                  ///      flag a search for a smaller trade.
             ///  In order for us to actually trade, we need to be considering a supported
             ///  depth. If we are not doing this, we must continue to search for the next
             ///  deepest synchronizer that our library supports.
             end else       currSize = currSize - 1;
           end
         end
         ///  Now that we have finished our calculations, we provide the means to report
         ///  our results back to the caller. Note, since we cannot return arrays of
         ///  multi-bit information, we have created the means to extract the appropriate
         ///  information.
         ///
         ///     o  if we could not create an appropriate depth, all series sizes will be reported as zero
         ///     o  the '0' index is a special index that requests the largest series used
         ///     o  a non-zero index returns the instance count for that specific depth
         if ( sizingError ) syncCellCount = 0;
         else if ( syncDepth == 0 ) begin
           syncCellCount = 0;                                            ///  This loop is our search for
           for ( currSize=0; currSize < 32; currSize=currSize+1 ) begin  ///   the largest depth selected.
             if ( instCount[currSize] )  syncCellCount = currSize;       ///    Basically, the last non-zero
           end                                                           ///     instance count.
         end else syncCellCount = instCount[syncDepth];
       end
     endfunction
     ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     ////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////////////////////
     ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     ///
     ///  Conceptually, we are building chains of chains of synchronizers. For the "inner"
     ///  chains, we shall delcare them in each of the 'series' generate blocks in the
     ///  code below. However, for the "outer" chain, we provide a bit-vector that shall be
     ///  used to stitch together the multiple series-chains. Since we currently only
     ///  support synchronizers of depth 5 downto 2 [depth is specified in flop stages],
     ///  we only need to declare a bit-vector of limited size.
     wire    [5:1]  DSeriesChain;             ///  if we choose to expand our range of synchronizers, we
     assign         DSeriesChain[5] = Dsrc;   ///   need to modify these lines of code accordingly.
     ///  Using the function defined earlier in this module, we can determine the number
     ///  of each type of synchronizer that is required to form our overall synchronizer
     ///  depth. Conceptually, the algorithm for instantiating our various synchronizers
     ///  could have mapped nicely into a 2-level nested-loop algorithm. The outer loop
     ///  would iterate through all of the possible synchronizer depths that we could
     ///  support, while the inner loop instantiates the requisite number of sync cells
     ///  of the current [outer loop variable] depth [where depth is measure in
     ///  flop-stage equivalents].
     ///
     ///  Unfortunately, for the nested loop algorithm to work, we need to be able to
     ///  either call a function [within the generate scope] based on the current
     ///  outer-loop variable, or index into a 2D parameter table. Since verilog does not
     ///  support either construct, we are forced to declare individual local parameters
     ///  in lieu of a 2D lookup table.
     localparam  Series5InstCnt = syncCellCount(depth,5);
     localparam  Series4InstCnt = syncCellCount(depth,4);
     localparam  Series3InstCnt = syncCellCount(depth,3);
     localparam  Series2InstCnt = syncCellCount(depth,2);
     ///  Moreover [and here is the rub], since we now lack the ability to parametrically
     ///  access relevant information, we must unroll the loop that would have leveraged
     ///  the 2D table [or function call]. The consequence of this, as will be seen below,
     ///  is UGLY code. We essentially take the same chunk of code and repeat it for every
     ///  potential synchronizer depth that we want to support -- YUCK!
     ///
     ///  In addition to the instance count parameters that are so crucial to building our
     ///  overall synchronizer chain, we know that the lead syncrhonizer is of particualar
     ///  importance. If there is a source-flop, it is only associated with the lead
     ///  syncrhonizer in the chain. If there is a leading half-stage [for determinism
     ///  reasons], it is required on the lead synchronizer in the chain. So, here we
     ///  shall define a couple of parameters that identify the lead synchronizer.
     localparam  LeadFlopSeries = syncCellCount(depth,0);
     localparam  LeadFlopInst   = syncCellCount(depth,LeadFlopSeries);
     ///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ///
     ///  The first element of our unrolled loop [sigh...] is the 5-stage syncrhonizer
     ///  series. In this generate block, using a loop construct, we instantiate the
     ///  requisite number of 5-stage synchronizers. Should we not require any 5-stage
     ///  synchronizers, the degenerate behaviour of this generate block will result in a
     ///  trivial bypass of the series.
     ///
     ///  For the interested reader, the comments that are interleaved amongst the code in
     ///  this generate block are directly applicable to each of the other generate blocks
     ///  [for the other N-stage series] where the stage depth is the only differing
     ///  detail. Consequently, we elect not to clutter the later series with near carbon
     ///  copies of the comments -- so read carefully.  :)
     genvar  gvSeries5Cnt;
     generate
       if ( Series5InstCnt ) begin : s5
         ///  Here we declare our "inner-loop" chain for the 5-stage series. This will be
         ///  used to chain a calculated number of synchronizers. However, before we begin,
         ///  we must seed the start of this "inner-loop" chain from the pertinent
         ///  position in the "outer" chain.
         wire [Series5InstCnt:0] DChain;
         assign                  DChain[Series5InstCnt] = DSeriesChain[5];
         ///  Now that we have our setup complete, we can dive into the loop that will
         ///  instantiate each synchronizer, one-by-one. As they are instantiated, we
         ///  cascade down our "inner-loop" chain until, finally, we connect down to the
         ///  [0] position of our "inner-loop" chain.
         ///
         ///  Note, both clauses instantiate the same 5-stage synchronizer wrapper with
         ///  different parameters. We do this in order to simplify this code-base [the
         ///  unrolled loop makes it nasty enough as it is]. The further complexity of
         ///  mapping the parameters onto the appropriate library syncrhonizer is pushed
         ///  off to the 5-stage synchronizer wrapper.
         for ( gvSeries5Cnt = Series5InstCnt; gvSeries5Cnt > 0; gvSeries5Cnt=gvSeries5Cnt-1 ) begin : u
           if (( 5 == LeadFlopSeries ) && ( gvSeries5Cnt == LeadFlopInst )) begin
             ///  As mentioned previously, the lead synchronizer is 'special'. Certain
             ///  attributes apply to this first syncrhonizer that do not apply to any
             ///  other syncrhonizer in the chain. For this reason we identify it
             ///  explicitly using the "Lead*" local parameters. When we find it, we
             ///  instantiate our lower-level, 5-stage synchronizer wrapper with all of
             ///  the lead-syncrhonizer features.
             techind_sync5_implementation #(
                 .SF    ( sourceFlop  ),    ///  only applies to the lead synchronizer.
                 .HR    ( halfStage   ),    ///  only applies to the lead synchronizer.
                 .AS    ( asyncSet    ),
                 .AR    ( asyncReset  ),
                 .AF    ( asyncFlush  ),
                 .CE    ( clockEnable ),
                 .TC    ( TransportCycle ),    ///  only applies to the lead synchronizer.
                 .TD    ( TransportDelay ),    ///  only applies to the lead synchronizer.
                 .SD    ( scan_muxD   ),
                 .SL    ( scan_LSSD   ),
                 .MV    ( MultiVolt   ),
                 .HC    ( HoldCheck   ),
                 .SP    ( SrcPath     )
               ) u (
                 .SB    ( SB                     ),
                 .RB    ( RB                     ),
                 .FLUSH ( FLUSH                  ),
                 .CKsrc ( CKsrc                  ),
                 .CKdst ( CKdst                  ),
                 .EN    ( EN                     ),
                 .Dsrc  ( DChain[gvSeries5Cnt  ] ),
                 .Qsrc  ( Qsrc                   ),   ///  only applies to the lead synchronizer
                 .Qdst  ( DChain[gvSeries5Cnt-1] ),
                 .SC1   ( SC1                    ),
                 .SC2   ( SC2                    ),
                 .SE    ( SE                     ),
                 .SI    ( SI                     ),
                 .SO    ( SO                     )
               );
           end else begin
             ///  However, if we are not the lead synchronizer, we can bolt on synchronizers
             ///  that are effectively mere flop-stage extensions, dropping any special
             ///  features required by the lead synchronizer.
             techind_sync5_implementation #(
                 .HP    ( 1           ),    ///  while not ideal, a half-stage is
                 .AS    ( asyncSet    ),    ///   functionally permissible
                 .AR    ( asyncReset  ),
                 .AF    ( asyncFlush  ),
                 .CE    ( clockEnable ),
                 .SD    ( scan_muxD   ),
                 .SL    ( scan_LSSD   )
               ) u (
                 .SB    ( SB                     ),
                 .RB    ( RB                     ),
                 .FLUSH ( FLUSH                  ),
                 .CKsrc ( CKsrc                  ),
                 .CKdst ( CKdst                  ),
                 .EN    ( EN                     ),
                 .Dsrc  ( DChain[gvSeries5Cnt  ] ),
                 .Qdst  ( DChain[gvSeries5Cnt-1] ),
                 .SC1   ( SC1                    ),
                 .SC2   ( SC2                    ),
                 .SE    ( SE                     ),
                 .SI    ( SI                     ),
                 .SO    ( SO                     )
               );
           end
         end
         ///  Now that we are done instantiating our 5-stage series, we need to patch our
         ///  "inner-loop" chain back into the "outer" chain so that the next series can
         ///  continue the stitching exercise.
         assign  DSeriesChain[4] = DChain[0];
         ///  As noted previously, in the degenerate case where there are zero
         ///  synchronizers from this series, the for-loop will not have performed any
         ///  rounds, and the "inner-loop" chain will merely be a single-bit bypass point,
         ///  thereby retaining the required continuity.
       end  ///  s5
       else begin : bypass5
         assign DSeriesChain[4] = DSeriesChain[5];
       end
     endgenerate
     ///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     genvar  gvSeries4Cnt;
     generate
       if ( Series4InstCnt ) begin : s4
         wire [Series4InstCnt:0] DChain;
         assign                  DChain[Series4InstCnt] = DSeriesChain[4];
         for ( gvSeries4Cnt = Series4InstCnt; gvSeries4Cnt > 0; gvSeries4Cnt=gvSeries4Cnt-1 ) begin : u
           if (( 4 == LeadFlopSeries ) && ( gvSeries4Cnt == LeadFlopInst )) begin
             techind_sync4_implementation #(
                 .SF    ( sourceFlop  ),
                 .HR    ( halfStage   ),
                 .AS    ( asyncSet    ),
                 .AR    ( asyncReset  ),
                 .AF    ( asyncFlush  ),
                 .CE    ( clockEnable ),
                 .TC    ( TransportCycle ),    ///  only applies to the lead synchronizer.
                 .TD    ( TransportDelay ),    ///  only applies to the lead synchronizer.
                 .SD    ( scan_muxD   ),
                 .SL    ( scan_LSSD   ),
                 .MV    ( MultiVolt   ),    
                 .HC    ( HoldCheck   ),
                 .SP    ( SrcPath     )
               ) u (
                 .SB    ( SB                     ),
                 .RB    ( RB                     ),
                 .FLUSH ( FLUSH                  ),
                 .CKsrc ( CKsrc                  ),
                 .CKdst ( CKdst                  ),
                 .EN    ( EN                     ),
                 .Dsrc  ( DChain[gvSeries4Cnt  ] ),
                 .Qsrc  ( Qsrc                   ),
                 .Qdst  ( DChain[gvSeries4Cnt-1] ),
                 .SC1   ( SC1                    ),
                 .SC2   ( SC2                    ),
                 .SE    ( SE                     ),
                 .SI    ( SI                     ),
                 .SO    ( SO                     )
               );
           end else begin
             techind_sync4_implementation #(
                 .HP    ( 1           ),
                 .AS    ( asyncSet    ),
                 .AR    ( asyncReset  ),
                 .AF    ( asyncFlush  ),
                 .CE    ( clockEnable ),
                 .SD    ( scan_muxD   ),
                 .SL    ( scan_LSSD   )
               ) u (
                 .SB    ( SB                     ),
                 .RB    ( RB                     ),
                 .FLUSH ( FLUSH                  ),
                 .CKsrc ( CKsrc                  ),
                 .CKdst ( CKdst                  ),
                 .EN    ( EN                     ),
                 .Dsrc  ( DChain[gvSeries4Cnt  ] ),
                 .Qdst  ( DChain[gvSeries4Cnt-1] ),
                 .SC1   ( SC1                    ),
                 .SC2   ( SC2                    ),
                 .SE    ( SE                     ),
                 .SI    ( SI                     ),
                 .SO    ( SO                     )
               );
           end
         end
         assign  DSeriesChain[3] = DChain[0];
       end  ///  s4
       else begin : bypass4
         assign DSeriesChain[3] = DSeriesChain[4];
       end
     endgenerate
     ///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     genvar  gvSeries3Cnt;
     generate
       if ( Series3InstCnt ) begin : s3
         wire [Series3InstCnt:0] DChain;
         assign                  DChain[Series3InstCnt] = DSeriesChain[3];
         for ( gvSeries3Cnt = Series3InstCnt; gvSeries3Cnt > 0; gvSeries3Cnt=gvSeries3Cnt-1 ) begin : u
           if (( 3 == LeadFlopSeries ) && ( gvSeries3Cnt == LeadFlopInst )) begin
             techind_sync3_implementation #(
                 .SF    ( sourceFlop  ),
                 .HR    ( halfStage   ),
                 .AS    ( asyncSet    ),
                 .AR    ( asyncReset  ),
                 .AF    ( asyncFlush  ),
                 .CE    ( clockEnable ),
                 .TC    ( TransportCycle ),    ///  only applies to the lead synchronizer.
                 .TD    ( TransportDelay ),    ///  only applies to the lead synchronizer.
                 .SD    ( scan_muxD   ),
                 .SL    ( scan_LSSD   ),
                 .MV    ( MultiVolt   ),    
                 .HC    ( HoldCheck   ),
                 .SP    ( SrcPath     )
               ) u (
                 .SB    ( SB                     ),
                 .RB    ( RB                     ),
                 .FLUSH ( FLUSH                  ),
                 .CKsrc ( CKsrc                  ),
                 .CKdst ( CKdst                  ),
                 .EN    ( EN                     ),
                 .Dsrc  ( DChain[gvSeries3Cnt  ] ),
                 .Qsrc  ( Qsrc                   ),
                 .Qdst  ( DChain[gvSeries3Cnt-1] ),
                 .SC1   ( SC1                    ),
                 .SC2   ( SC2                    ),
                 .SE    ( SE                     ),
                 .SI    ( SI                     ),
                 .SO    ( SO                     )
               );
           end else begin
             techind_sync3_implementation #(
                 .HP    ( 1           ),
                 .AS    ( asyncSet    ),
                 .AR    ( asyncReset  ),
                 .AF    ( asyncFlush  ),
                 .CE    ( clockEnable ),
                 .SD    ( scan_muxD   ),
                 .SL    ( scan_LSSD   )
               ) u (
                 .SB    ( SB                     ),
                 .RB    ( RB                     ),
                 .FLUSH ( FLUSH                  ),
                 .CKsrc ( CKsrc                  ),
                 .CKdst ( CKdst                  ),
                 .EN    ( EN                     ),
                 .Dsrc  ( DChain[gvSeries3Cnt  ] ),
                 .Qdst  ( DChain[gvSeries3Cnt-1] ),
                 .SC1   ( SC1                    ),
                 .SC2   ( SC2                    ),
                 .SE    ( SE                     ),
                 .SI    ( SI                     ),
                 .SO    ( SO                     )
               );
           end
         end
         assign  DSeriesChain[2] = DChain[0];
       end  ///  s3
       else begin : bypass3
         assign DSeriesChain[2] = DSeriesChain[3];
       end
     endgenerate
     ///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     genvar  gvSeries2Cnt;
     generate
       if ( Series2InstCnt ) begin : s2
         wire [Series2InstCnt:0] DChain;
         assign                  DChain[Series2InstCnt] = DSeriesChain[2];
         for ( gvSeries2Cnt = Series2InstCnt; gvSeries2Cnt > 0; gvSeries2Cnt=gvSeries2Cnt-1 ) begin : u
           if (( 2 == LeadFlopSeries ) && ( gvSeries2Cnt == LeadFlopInst )) begin
             techind_sync2_implementation #(
                 .SF    ( sourceFlop  ),
                 .HR    ( halfStage   ),
                 .AS    ( asyncSet    ),
                 .AR    ( asyncReset  ),
                 .AF    ( asyncFlush  ),
                 .CE    ( clockEnable ),
                 .TC    ( TransportCycle ),    ///  only applies to the lead synchronizer.
                 .TD    ( TransportDelay ),    ///  only applies to the lead synchronizer.
                 .SD    ( scan_muxD   ),
                 .SL    ( scan_LSSD   ),
                 .MV    ( MultiVolt   ),   
                 .HC    ( HoldCheck   ),
                 .SP    ( SrcPath     )
               ) u (
                 .SB    ( SB                     ),
                 .RB    ( RB                     ),
                 .FLUSH ( FLUSH                  ),
                 .CKsrc ( CKsrc                  ),
                 .CKdst ( CKdst                  ),
                 .EN    ( EN                     ),
                 .Dsrc  ( DChain[gvSeries2Cnt  ] ),
                 .Qsrc  ( Qsrc                   ),
                 .Qdst  ( DChain[gvSeries2Cnt-1] ),
                 .SC1   ( SC1                    ),
                 .SC2   ( SC2                    ),
                 .SE    ( SE                     ),
                 .SI    ( SI                     ),
                 .SO    ( SO                     )
               );
           end else begin
             techind_sync2_implementation #(
                 .HP    ( 1           ),
                 .AS    ( asyncSet    ),
                 .AR    ( asyncReset  ),
                 .AF    ( asyncFlush  ),
                 .CE    ( clockEnable ),
                 .SD    ( scan_muxD   ),
                 .SL    ( scan_LSSD   )
               ) u (
                 .SB    ( SB                     ),
                 .RB    ( RB                     ),
                 .FLUSH ( FLUSH                  ),
                 .CKsrc ( CKsrc                  ),
                 .CKdst ( CKdst                  ),
                 .EN    ( EN                     ),
                 .Dsrc  ( DChain[gvSeries2Cnt  ] ),
                 .Qdst  ( DChain[gvSeries2Cnt-1] ),
                 .SC1   ( SC1                    ),
                 .SC2   ( SC2                    ),
                 .SE    ( SE                     ),
                 .SI    ( SI                     ),
                 .SO    ( SO                     )
               );
           end
         end
         assign  DSeriesChain[1] = DChain[0];
       end  ///  s2
       else begin : bypass2
         assign DSeriesChain[1] = DSeriesChain[2];
       end
     endgenerate
     ///   .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ///
     ///  Now that we have instantiated and wired our chains of chains of synchronizers,
     ///  we finalize our implementation by pulling off of the tail of our chain and
     ///  driving it out for the user's consumption.
     assign Qdst = DSeriesChain[1];
     ///  Note, since we do not support synchronizers more shallow than a depth of 2, we
     ///  only need to carry our chain down to the [1] bit-position.


    `endif // FPGA_EMULATION


  `endif   ///  SYNTHESIS
   ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  // river translate_on
  ///////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  
endmodule
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
