//*************************************************************************************************
//
// FILE_OWNER: kelvin.goveas@amd.com
//
//  Copyright  2013 Advanced Micro Devices, Inc.
//  All rights reserved.
//
//  This file is protected by Federal Copyright Law, with all rights
//  reserved. No part of this file may be reproduced, stored in a
//  retrieval system, translated, transcribed, or transmitted, in any
//  form, or by any means manual, electric, electronic, mechanical,
//  electro-magnetic, chemical, optical, or otherwise, without prior
//  explicit written permission from Advanced Micro Devices, Inc.
//
//*************************************************************************************************
// *************************************************************************************************
//
//  $RCSfile:  $
//  $Revision: $
//  $Date:  $
//  $Author: $
//  $State:  $
//  $Locker:  $
//
//  Author: S.Toh
//
//  Description: 
//
//  This rtl contains a top-level AVFS wrapper module, which integrates the following components:
//     AVFS/PSM Register Interface Block
//     AVFS Critical Path Accumulator (CPA)
//     AVFS Gate Pack
//     AVFS Wire Pack
//     Power Supply Monitor (PSM)
//  The AVFS/PSM register interface block, 'AvfsWrapJtgInterface', supports SPMI, JTAG (IEEE-1500)
//  & M&M access to the internal AVFS/PSM registers. The PSM rtl is in CORE_PSM_IP_NO_JTAG and
//  contains 'CorePsmIpCore' which instantiates the PSM macro. CorePsmIpCore needs to be fast
//  enough to support the highest frequency under worst case droop. 
//
// *************************************************************************************************
//==================================================================================================
// <Module Description>
//
// This is a template for creating verilog shells.  Guidelines and a how-to guide to
// creating HLD verilog shells can be found at:
// http://mpdwww.amd.com/twiki/bin/view/BDRTL/K10VerilogShells
// All questions and comments should be directed to Gene Shen (genes@amd.com)
//
// -> Outputs are declared using V2K syntax - i.e. using combined port and data type
//    declarations.  In Verilog-1995, inputs and outputs of a module were declared in two
//    separate statements (the direction of the port and the data type).  In Verilog-2000, a
//    simpler syntax combines the declarations into one statement.
// -> Please provide a short description of the signal.
// -> Signal names should be of the form:
//        <SourceIdentifier>_<SignalName>_<Suffix>
//    where <SourceIdentifier> is a unique prefix and SignalName is a short descriptive
//    signal name.  For now, suffixes should not be used - mostly because we have not decided
//    on our suffixes.
//
//
//==================================================================================================
//==================================================================================================
//  Non-scanable registers use the following flop wrappers:
//
//    CORE_PSM_IP_FLOP
//    CORE_PSM_IP_FLOP_GATED
//
// Ravi Rachala July 2015
// Decision has been made to move all non-scan flops to scannable flops in light of dropping
// support/requirement for the feature of "PSM Running while scan is active" 
//
//==================================================================================================
//==================================================================================================
//! disable one_module_per_file
// Disable the Leda one-per-file check
// leda E_001 off
`include "gfx_core_psm_ip_cfg.d"

`ifdef AVFS_ZAK
  // conditional include of M&M defines
  `include "mm_global.d"
`endif
`include "gfx_avfs_lib.d"
`include "gfx_avfs_psm_lib.d"

// AVFS (Adaptive Voltage and Frequency Scaling) Library file


//================================================================================================================================
// POPCNT7  7 to 3 compressor
//================================================================================================================================

module gfx_POPCNT7
(
    input       [6:0] DIn,
    output wire [2:0] DOut
 );
assign DOut[2:0] = {2'b00,DIn[6]} + {2'b00,DIn[5]} + {2'b00,DIn[4]} + {2'b00,DIn[3]} + 
                   {2'b00,DIn[2]} + {2'b00,DIn[1]} + {2'b00,DIn[0]};

endmodule // POPCNT7

//================================================================================================================================
// <AVFS_LFSR3>
//================================================================================================================================
// The original LFSR was replaced with a larger sequence to maximize transitions on AVFS wire paths.  An LFSR only maximizes the 
// states the wires can take.  We would like to see all transitions. 
// Each bit can take U-up, D-down, N-no transition state, so for 2b, there are 3**2=9 transitions.  For 3b, there are 27. 

module gfx_AVFS_LFSR3 
  #( parameter width = 3                   // output width
   )
   (
//    input [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsPath,
    input                  ScanShiftEn,    // Scan shift Enable
    input                  CGater01_CG0,   // Coarse Gater
    input                  CCLK,
    input                  Reset,          // reset signal to reset flops for SOCs which don't use scan-shift-reset
    input                  AvfsPathEn,     // global enable 
    input                  AvfsAgeHigh,    // aging control 

    output wire [width-1:0]AvfsLfsr
    );


//! unused CGater01_CG0

   reg  [5:0] AvfsWireDrive_m1;
   wire [5:0] AvfsWireDrive;
   wire [8:0] AvfsWireDriveDup_m1;
   wire [width+2:0] AvfsWireDriveIn;     
   wire [width+2:0] AvfsWireDriveOut;
   wire       AvfsWireStart;
   wire       AvfsWireStopHigh;
   wire       AvfsWireStopLow;
   wire       CgenAvfsPath;
   wire       AvfsPathEn_D1;
   wire       dummy;

// This Sequence is not an LFSR, but maximizes transitions on 3 wires
// Each bit can take U-up, D-down, N-no transition state
// 2b = 0U, 0D, 0U1U, 0U1D, 0D1U, 0D1D, 1U, 1D, 1N0N = 9 cases, but 1N0N has no transition, so 8 cases
// 3b = 27 cases, but again 2N1N0N has no transition so is not used leaving 26 cases. 

// The following sequence transitions through almost all cases, but insures at least 1 transition on each bit every 4-cycle period
//    |0-2-5-2- |1-7-4-0- |6-1-6-4- |6-3-5-0- |7-0-3-6- |0-5-4-3- |4-2-6-5- |repeat   missing transitions - nud nnu
//    |  udu ndu|  ndd uun|  uud nun|  udn uuu|  nuu ddn|  nnd udd|  unn dnd|         multiple transitions - nun ndu uun dnd
//    |nun dud  |uun dnn  |ddu ndn  |dnu dnd  |ddd und  |unu duu  |dun ndu  |
// <2> 0 0 1 0   0 1 1 0   1 0 1 1   1 0 1 0   1 0 0 1   0 1 1 0   1 0 1 1
// <1> 0 1 0 1   0 1 0 0   1 0 1 0   1 1 0 0   1 0 1 1   0 0 0 1   0 1 1 0
// <0> 0 0 1 0   1 1 0 0   0 1 0 0   0 1 1 0   1 0 1 0   0 1 0 1   0 0 0 1
//     2 1 3 3=9 2 2 2 1=7 2 3 3 1=9 1 2 2 2=7 3 3 2 2=102 2 1 3=8 3 2 1 2=8   total transitions per 4-cycle period (7-10)
//    
// The 2 lsbs cover all transitions for a 2b bus.

//                                    Output
//                            unique  bits
//                                ID  ||| Transition (N=none, D=Down, U=Up)
//          wrap from bottom          101 
`define AVFS_WIRE_DND   6'h00  // 000 000 DND
`define AVFS_WIRE_NUN0  6'h02  // 000 010 NUN0
`define AVFS_WIRE_UDU   6'h05  // 000 101 UDU
`define AVFS_WIRE_DUD   6'h0a  // 001 010 DUD
`define AVFS_WIRE_NDU   6'h01  // 000 001 NDU
`define AVFS_WIRE_UUN   6'h07  // 000 111 UUN
`define AVFS_WIRE_NDD   6'h04  // 000 100 NDD
`define AVFS_WIRE_DNN   6'h08  // 001 000 DNN
`define AVFS_WIRE_UUN1  6'h06  // 000 110 UUN
`define AVFS_WIRE_DDU   6'h09  // 001 001 DDU
`define AVFS_WIRE_UUD   6'h0e  // 001 110 UUD
`define AVFS_WIRE_NDN0  6'h0c  // 001 100 NDN0
`define AVFS_WIRE_NUN1  6'h16  // 010 110 NUN1
`define AVFS_WIRE_DNU   6'h03  // 000 011 DNU
`define AVFS_WIRE_UDN   6'h0d  // 001 101 UDN
`define AVFS_WIRE_DND1  6'h10  // 010 000 DND
`define AVFS_WIRE_UUU   6'h0f  // 001 111 UUU
`define AVFS_WIRE_DDD   6'h18  // 011 000 DDD
`define AVFS_WIRE_NUU   6'h0b  // 001 011 NUU
`define AVFS_WIRE_UND   6'h1e  // 011 110 UND
`define AVFS_WIRE_DDN   6'h20  // 100 000 DDN
`define AVFS_WIRE_UNU   6'h1d  // 011 101 UNU
`define AVFS_WIRE_NND   6'h1c  // 011 100 NND
`define AVFS_WIRE_DUU   6'h13  // 010 011 DUU
`define AVFS_WIRE_UDD   6'h24  // 100 100 UDD
`define AVFS_WIRE_DUN   6'h12  // 010 010 DUN
`define AVFS_WIRE_UNN   6'h26  // 100 110 UNN
`define AVFS_WIRE_NDU1  6'h25  // 100 101 NDU


`define AVFS_WIRE_NNU   6'h01  // xxx xx1 NNU     unused transition 
`define AVFS_WIRE_NUD   6'h0a  // xxx x10 NUD     unused transition 

// Between path collection times, flops reset in all 1's or all 0's state depending on the Aging selection from SMU
`define AVFS_WIRE_HIGH  6'h3f  // 111 111
`define AVFS_WIRE_LOW   6'h00  // 000 000

// Start generating states on AvfsPathEn.  There are 7 cycles of PROP in the CPA state machine before data will be collected.
// The reset signal will assert at the end of the path collection period so flops rest in a known state for aging calculations.
   always @* begin
      casez ({AvfsWireStart, AvfsWireStopLow, AvfsWireStopHigh, AvfsWireDrive[5:0]})
        {3'b001,      6'b?????? } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_HIGH ;
        {3'b01?,      6'b?????? },                                            // StopLow or 
        {3'b1??,      6'b?????? },                                            // Start or 
        {3'b000, `AVFS_WIRE_NDU1} : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DND ;  // wrapping from NDU1 to DND force outputs to zero.
        {3'b000, `AVFS_WIRE_DND } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_NUN0;  
        {3'b000, `AVFS_WIRE_NUN0} : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UDU ;
        {3'b000, `AVFS_WIRE_UDU } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DUD ;
        {3'b000, `AVFS_WIRE_DUD } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_NDU ;
        {3'b000, `AVFS_WIRE_NDU } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UUN ;
        {3'b000, `AVFS_WIRE_UUN } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_NDD ;
        {3'b000, `AVFS_WIRE_NDD } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DNN ;
        {3'b000, `AVFS_WIRE_DNN } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UUN1;
        {3'b000, `AVFS_WIRE_UUN1} : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DDU ;
        {3'b000, `AVFS_WIRE_DDU } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UUD ;
        {3'b000, `AVFS_WIRE_UUD } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_NDN0;
        {3'b000, `AVFS_WIRE_NDN0} : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_NUN1;
        {3'b000, `AVFS_WIRE_NUN1} : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DNU ;
        {3'b000, `AVFS_WIRE_DNU } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UDN ;
        {3'b000, `AVFS_WIRE_UDN } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DND1;
        {3'b000, `AVFS_WIRE_DND1} : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UUU ;
        {3'b000, `AVFS_WIRE_UUU } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DDD ;
        {3'b000, `AVFS_WIRE_DDD } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_NUU ;
        {3'b000, `AVFS_WIRE_NUU } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UND ;
        {3'b000, `AVFS_WIRE_UND } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DDN ;
        {3'b000, `AVFS_WIRE_DDN } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UNU ;
        {3'b000, `AVFS_WIRE_UNU } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_NND ;
        {3'b000, `AVFS_WIRE_NND } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DUU ;
        {3'b000, `AVFS_WIRE_DUU } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UDD ;
        {3'b000, `AVFS_WIRE_UDD } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_DUN ;
        {3'b000, `AVFS_WIRE_DUN } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_UNN ;
        {3'b000, `AVFS_WIRE_UNN } : AvfsWireDrive_m1[5:0] = `AVFS_WIRE_NDU1;
        default : AvfsWireDrive_m1[5:0] = 6'bx;
      endcase // casez({AvfsWireStart, AvfsWireStopHigh, AvfsWireStopLow, AvfsWireDrive[5:0]})
   end // always @ *


   // duplicate the lower 3 bits so that up to 9 toggling bits can be driven from the LSFR
   assign AvfsWireDriveDup_m1[8:0] = { AvfsWireDrive_m1[2:0]
                                     , AvfsWireDrive_m1[2:0]
                                     , AvfsWireDrive_m1[2:0]
                                     };

  
   // assemble the vector that is driven into the flop
   assign  AvfsWireDriveIn[width+2:0] = { AvfsWireDrive_m1[5:3]
                                        , AvfsWireDriveDup_m1[width-1:0]   // this duplicates some of the lower bits depending on width
                                        };  
   
   // sink the unused signals. RPL doesn't like parameters in the unused statement which is why they are assigned to "dummy" first.
   assign dummy = |AvfsWireDriveDup_m1[8:width];
   //! unused dummy
   
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsPath;

  gfx_CORE_CG I_cg_CgenAvfsPath (.clkbus(clkbus_CgenAvfsPath[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenAvfsPath), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE  #(.size(width+3)) I_AvfsWireDrive (.clkbus(clkbus_CgenAvfsPath[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsWireDriveIn[width+2:0]), .Q(AvfsWireDriveOut[width+2:0]));
   
   // this goes back into the LSFR next state equation
   assign AvfsWireDrive[5:0] = { AvfsWireDriveOut[width+2:width], AvfsWireDriveOut[2:0] };
   
   // this is the output that is driven. 
   assign AvfsLfsr[width-1:0] = AvfsWireDriveOut[width-1:0];   


// Clock gater 
  gfx_CORE_FLOP_DGATE  #(.size(1)) I_AvfsPathEn_D1 (.clkbus(clkbus_CgenAvfsPath[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsPathEn), .Q(AvfsPathEn_D1));
// LFSR sequence terminates on TRAILING edge of enable in either all-high or all-low state so we can control the resting value. 
// This matters for tracking aging effects so we can control the time that transistors are 1, 0 and the frequency of transition. 
   assign AvfsWireStopHigh = ~AvfsPathEn & AvfsPathEn_D1 & AvfsAgeHigh;
   assign AvfsWireStopLow = ~AvfsPathEn & AvfsPathEn_D1 & ~AvfsAgeHigh;
// LFSR always resets to the beginning of the sequence at the leading edge of PathEn
   assign AvfsWireStart = AvfsPathEn & ~AvfsPathEn_D1;
   assign CgenAvfsPath = AvfsPathEn | AvfsPathEn_D1;

endmodule // AVFS_LFSR3

//================================================================================================================================
// <AVFS_GATE_REPLICA_PATH>
//================================================================================================================================
// An additional level of hierarchy inside AVFS_GATEPACK is created to capture the replica paths, called AVFS_GATE_REPLICA_PATH.
// The inputs & outputs of this module will match the inputs & outputs of the replica paths created by the automated tile flow.
// The LFSR launch flops that start the path and the AVFS speed flops that terminate the path are in the top-level module.
//
module gfx_AVFS_GATE_REPLICA_PATH #(parameter path_count = 41)
  (
   input       [path_count-1:0]    AvfsLfsr,
   output wire [path_count-1:0]    AvfsReplicaPath
  );

   // instantiate critical paths extracted from tile here.
   // logically the paths are just buffers
   assign AvfsReplicaPath[path_count-1:0] = AvfsLfsr[path_count-1:0];

endmodule // AVFS_GATE_REPLICA_PATH

//================================================================================================================================
// <AVFS_WIRE_REPLICA_PATH>
//================================================================================================================================
// An additional level of hierarchy inside AVFS_WIREPACK is created to capture the replica paths, called AVFS_WIRE_REPLICA_PATH.
// The inputs & outputs of this module will match the inputs & outputs of the replica paths created by the automated tile flow.
// The flow will also insert buffers into this module to match the wire delay of real critical paths in the tile. 
// The LFSR launch flops that start the path and the AVFS speed flops that terminate the path are in the top-level module.
//
module gfx_AVFS_WIRE_REPLICA_PATH #(parameter path_count = 3)
  (
   input       [path_count-1:0]    AvfsLfsr,
   output wire [path_count-1:0]    AvfsReplicaPath
  );

   // instantiate critical paths extracted from tile here
   assign AvfsReplicaPath[path_count-1:0] = AvfsLfsr[path_count-1:0];

endmodule // AVFS_WIRE_REPLICA_PATH

//================================================================================================================================
// <AVFS_GATEPACK>
//================================================================================================================================
// Gate Packs are used in the AVFS (adaptive voltage and frequency scaling) support logic. 
// They consist of LSFR trigger logic to drive pseudo random data on a series of signals that are consumed by various 
// combinatorial packs of logic that require near similar gate counts into speed flops.  The speed flops contain programmable 
// delay elements that produce redundant, but slightly slower results into second result flops.  The true result and delayed 
// result flops are compared to generate the NearMiss outputs. NearMiss outputs are accumulated in the CPA module for use
// by the SMU for voltage and frequency scaling decisions. 
//
module gfx_AVFS_GATEPACK #(parameter path_count = `AVFS_GATE_PATHS_NUM)
  (
   input                           AvfsPathEn,
   input [(3*path_count)-1:0]      AvfsPathDelay,
   input                [2:0]      AvfsGlobalPathDelay,
   output wire [path_count-1:0]    AvfsNearMiss,
   input                           AvfsTestData,
   input [1:0]                     AvfsTestMode,
   input                           AvfsAgeHigh,

   input                           ScanShiftEn,    // Scan shift Enable
   input                           CGater01_CG0,   // Coarse Gater
   input                           CCLK,
   input                           Reset           // reset signal to reset flops for SOCs which don't use scan-shift-reset
   );

   integer                 i, j;
   wire                    CgenAvfsPath;
   wire [2:0]              AvfsLfsrOut;
   wire                    AvfsPathEn_D1;
   reg  [3+path_count-1:0] AvfsLfsr;
   wire [path_count-1:0]   AvfsReplicaPath;
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsPath;

  gfx_CORE_CG I_cg_CgenAvfsPath (.clkbus(clkbus_CgenAvfsPath[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenAvfsPath), .R(Reset), .SSE(ScanShiftEn));

   // LFSR
   // It's called an LFSR, but it's really a sequence of states to maximize the transitions on the 3 output bits. 
   // Start generating on AvfsPathEn.  There are 7 cycles of PROP in the CPA state machine before data will be collected.
  gfx_AVFS_LFSR3 I_AVFS_LFSR3_GATE(
//                           .clkbus_CgenAvfsPath(clkbus_CgenAvfsPath),
                           .ScanShiftEn(ScanShiftEn),
                           .CGater01_CG0(CGater01_CG0),
                           .CCLK(CCLK),
                           .Reset(Reset),
                           .AvfsPathEn   (AvfsPathEn),
                           .AvfsAgeHigh  (AvfsAgeHigh),
                           .AvfsLfsr     (AvfsLfsrOut[2:0])
                           );

   // The following code replicates AvfsLfsrOut[2:0] in multiples of 3 across AvfsLfsr[path_count-1:0]
   //   assign AvfsLfsr[0] to final bit if path_count is 1 greater than a multple of 3 or
   //   assign AvfsLfsr[1:0] to final 2 bits if path_count is 2 greater than a multiple of 3
   always @* begin
     j = 0;
     AvfsLfsr[3+path_count-1:0] = {3'b0, {path_count{1'b0}}};
     for (i = 0; i < path_count; i=i+3) begin
       AvfsLfsr[3+path_count-1:0] = ((AvfsLfsr[3+path_count-1:0] << 3) | {{path_count{1'b0}}, AvfsLfsrOut[2:0]});
       j = j+1;
     end
     AvfsLfsr[3+path_count-1:0] = AvfsLfsr[3+path_count-1:0] | ({{path_count{1'b0}},AvfsLfsrOut[2:0]} << 3*j);
   end

  gfx_AVFS_GATE_REPLICA_PATH  #(.path_count(`AVFS_GATE_PATHS_NUM))
     I_AVFS_GATE_REPLICA_PATH
       (
        .AvfsLfsr        (AvfsLfsr[path_count-1:0]), 
        .AvfsReplicaPath (AvfsReplicaPath[path_count-1:0])
        );

   // Output Speed flops
   wire [path_count-1:0] avfs_unused;
   // Clock gater 
  gfx_CORE_FLOP_DGATE  #(.size(1)) I_AvfsPathEn_D1 (.clkbus(clkbus_CgenAvfsPath[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsPathEn), .Q(AvfsPathEn_D1));
   assign CgenAvfsPath = AvfsPathEn | AvfsPathEn_D1;
  gfx_AVFS_FLOP #(.size(path_count)) I_AvfsNearMiss (.D(AvfsReplicaPath[path_count-1:0]), .Q(avfs_unused[path_count-1:0]), .G(AvfsPathEn_D1), 
                                                .NDLY(AvfsPathDelay[(3*path_count)-1:0]), .SDLY(AvfsGlobalPathDelay[2:0]), .TM_X(~AvfsTestMode[1:0]), .TST(AvfsTestData), 
                                                .NM(AvfsNearMiss[path_count-1:0]), `CORE_FLOP_TAIL);
   //! unused avfs_unused
   
endmodule // AVFS_GATEPACK

//================================================================================================================================
// <AVFS_WIREPACK>
//================================================================================================================================
// Wire Packs are used in the AVFS (adaptive voltage and frequency scaling) support logic.
// Each wirepack consists of a bundle of 3 wire replica's, driven by an LFSR3 which tries to exercise worst-case
// SI noise between neighboring wires. These bundle of 3 wires should be routed together to maximize coupling.
// The wire pack outputs 3 wire-drive signals and receives back 3 wires. These can either be a loop-back at
// the module that instantiates the wire pack or a long distance exchange between different wirepacks.

module gfx_AVFS_WIREPACK #(parameter path_count = `AVFS_WIRE_PATHS_NUM)
  (
   input                        AvfsPathEn,
   input [(3*path_count)-1:0]   AvfsPathDelay,
   input [2:0]                  AvfsGlobalPathDelay,
   output wire [path_count-1:0] AvfsNearMiss,
   input                        AvfsTestData,
   input [1:0]                  AvfsTestMode,
   input                        AvfsAgeHigh,
   
   input                        ScanShiftEn, // Scan shift Enable
   input                        CGater01_CG0, // Coarse Gater
   input                        CCLK,
   input                        Reset         // reset signal to reset flops for SOCs which don't use scan-shift-reset

   );

   integer                 i, j;
   wire                    CgenAvfsPath;
   wire [2:0]              AvfsLfsrOut;
   wire                    AvfsPathEn_D1;
   reg  [3+path_count-1:0] AvfsLfsr;
   wire [path_count-1:0]   AvfsReplicaPath;
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsPath;

  gfx_CORE_CG I_cg_CgenAvfsPath (.clkbus(clkbus_CgenAvfsPath[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenAvfsPath), .R(Reset), .SSE(ScanShiftEn));

   // LFSR
   // It's called an LFSR, but it's really a sequence of states to maximize the transitions on the 3 output bits. 
   // Start generating on AvfsPathEn.  There are 7 cycles of PROP in the CPA state machine before data will be collected.
  gfx_AVFS_LFSR3 I_AVFS_LFSR3_WIRE(
//                           .clkbus_CgenAvfsPath(clkbus_CgenAvfsPath),
                           .ScanShiftEn(ScanShiftEn),
                           .CGater01_CG0(CGater01_CG0),
                           .CCLK(CCLK),
                           .Reset(Reset),
                           .AvfsPathEn   (AvfsPathEn),
                           .AvfsAgeHigh  (AvfsAgeHigh),
                           .AvfsLfsr     (AvfsLfsrOut[2:0])
                           );

   // The following code replicates AvfsLfsrOut[2:0] in multiples of 3 across AvfsLfsr[path_count-1:0]
   //   assign AvfsLfsr[0] to final bit if path_count is 1 greater than a multple of 3 or
   //   assign AvfsLfsr[1:0] to final 2 bits if path_count is 2 greater than a multiple of 3
   always @* begin
     j = 0;
     AvfsLfsr[3+path_count-1:0] = {3'b0, {path_count{1'b0}}};
     for (i = 0; i < path_count; i=i+3) begin
       AvfsLfsr[3+path_count-1:0] = ((AvfsLfsr[3+path_count-1:0] << 3) | {{path_count{1'b0}}, AvfsLfsrOut[2:0]});
       j = j+1;
     end
     AvfsLfsr[3+path_count-1:0] = AvfsLfsr[3+path_count-1:0] | ({{path_count{1'b0}},AvfsLfsrOut[2:0]} << 3*j);
   end

   // Additional level of hierarchy to allow PD to instantiate replica wire paths with buffers between
   // AvfsLfsr[path_count-1:0] as a startpoint, and AvfsReplicaPath[path_count-1:0] as an endpoint
  gfx_AVFS_WIRE_REPLICA_PATH  #(.path_count(`AVFS_WIRE_PATHS_NUM))
     I_AVFS_WIRE_REPLICA_PATH
     (
       .AvfsLfsr        (AvfsLfsr[path_count-1:0]), 
       .AvfsReplicaPath (AvfsReplicaPath[path_count-1:0])
     );

   // Output Speed flops
   wire [path_count-1:0] avfs_unused;

   // Clock gater 
  gfx_CORE_FLOP_DGATE  #(.size(1)) I_AvfsPathEn_D1 (.clkbus(clkbus_CgenAvfsPath[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsPathEn), .Q(AvfsPathEn_D1));
   assign CgenAvfsPath = AvfsPathEn | AvfsPathEn_D1;
  gfx_AVFS_FLOP #(.size(path_count)) I_AvfsNearMiss (.D(AvfsReplicaPath[path_count-1:0]), .Q(avfs_unused[path_count-1:0]), .G(AvfsPathEn_D1), 
                                                  .NDLY(AvfsPathDelay[(3*path_count)-1:0]), .SDLY(AvfsGlobalPathDelay[2:0]), .TM_X(~AvfsTestMode[1:0]), .TST(AvfsTestData), 
                                                  .NM(AvfsNearMiss[path_count-1:0]), `CORE_FLOP_TAIL);
   //! unused avfs_unused
endmodule // AVFS_WIREPACK
                                 
                                 
//================================================================================================================================
// <AVFS_CPA>
//================================================================================================================================
// AVFS (Adaptive Voltage and Frequency Scaling) CPA (Critical Path Accumulator) 
// The CPA module controls and gathers data from all local speed flops (SFFGSPD).  
// It has a 64-bit SRB interface that can be interfaced with SPMI/JTAG/p1500

// todo:
//   parameterize by input width 

module gfx_AVFS_CPA #(parameter
  CDC_TST_PROP_DLY  = 5000
) (
//=========================================================================
//  IO Declarations
//=========================================================================

input                  ScanShiftEn,    // Scan shift Enable
input                  CGater01_CG0,   // Coarse Gater
input                  CCLK,
input                  HostReg_CCLK,
input                  Reset,          // reset signal to reset flops for SOCs which don't use scan-shift-reset

input                  AvfsGo,         // Pulse to start the state machine
input [49:0]           AvfsNearMiss,   // AVFS NearMiss inputs from gate path speed flops
input [13:0]           PsmOutput,      // PSM delta output (diff between psm long term and short term averages)
input                  PsmClk,         // PSM clock indicating when PSM delta has changed (at least 4 cycle period)
 
output wire            CgenAvfsEn,     // gate AvfsGoPulse flops
output wire            AvfsPathEn,     // global enable for synthetic path triggers
output wire [149:0]    AvfsPathDelay,  // per path delay settings, 3b per path
output wire   [2:0]    AvfsGlobalPathDelay,  // Global delay settings, 3b per path
input                  LBistEn,        // LBistEn to be used to control multicycle protection for ATPG.
output wire            AvfsTestData,   // TestData to be used in SFFGSPD
output wire [1:0]      AvfsTestMode,   // TestMode to be used in SFFGSPD
output wire [63:0]     AvfsSrbRdDat,   // SRB read data 
output wire            AvfsAgeHigh,    // Aging state for LFSR3 path trigger logic 
input                  SelAvfsCtl1,    // SRB register decodes used for both read and write 
input                  SelAvfsCtl2,
input                  SelAvfsBucket0,
input                  SelAvfsBucket1,
input                  SelAvfsValid,
input                  SelAvfsNorm0,
input                  SelAvfsNorm1,
input                  SelAvfsNorm2,

input [63:0]           AvfsSrbWrDat_4, // SRB write data
input                  SrbLclWrEn_4    // SRB write enable 
 
);

//! unused CGater01_CG0
   wire                AvfsPathEn_m1;
   wire [49:0]         CpaStickyIn;
   wire                CgenStickyEn;
   wire [49:0]         CpaSticky;
   wire [7:0]          PsmDelta;
   wire [7:0]          PsmSignMag;
   wire                PsmOutputMaxNeg;
   wire                PsmNegOver;
   wire                PsmPosOver;
   wire                CpaStep1Active;
   wire                CpaStep2Active;
   wire                CpaStep3Active;
   wire                CpaStep4Active;
   wire                CpaStep2Adv;
   wire                CpaStep3Adv;
   wire                CpaStep4Adv;
   wire                CpaStep5Adv;
   wire                AvfsEn;
   wire                CgenCpaSyncEn;
   wire [49:0]         CpaSync;
   wire [49:0]         CpaMaskOut;
   wire [49:0]         AvfsPathMask;
   wire [49:0]         FinalPathMask;
   wire [2:0]          CpaPopCntG0;
   wire [2:0]          CpaPopCntG1;
   wire [2:0]          CpaPopCntG2;
   wire [2:0]          CpaPopCntG3;
   wire [2:0]          CpaPopCntG4;
   wire [2:0]          CpaPopCntG5;
   wire [2:0]          CpaPopCntG6;
   wire [5:0]          CpaPopCntOut;
   wire [5:0]          CpaPopCntOut_MC;
   wire                CgenCpaPopCnt3;
   wire [5:0]          CpaPopCnt_S3;
   wire                CgenCpaPopCnt4;
   wire [5:0]          CpaPopCnt_S4;
   wire [47:0]         DlyTable;
   reg  [5:0]          VirtualBucket;
   wire [2:0]          AvfsDelay;
   wire [2:0]          AvfsDelaySync;
   wire [2:0]          AvfsDelayCnt;
   wire [2:0]          AvfsDelayMul;
   wire                CgenMulEn;
   wire [31:0]         MulOut;
   wire [7:0]          PSMBucketSize;
   wire [6:0]          PSMAdjust;
   wire [6:0]          FinalVirtualBucket;
   wire                ForceBucket15;
   wire                ForceBucket0;
   wire [15:0]         FinalBucketSel;
   wire [15:0]         FinalBucketSel_MC;
   reg  [7:0]          FinalBucketOut;
   wire [7:0]          CpaBucket0;
   wire [7:0]          CpaBucket1; 
   wire [7:0]          CpaBucket2; 
   wire [7:0]          CpaBucket3; 
   wire [7:0]          CpaBucket4; 
   wire [7:0]          CpaBucket5; 
   wire [7:0]          CpaBucket6; 
   wire [7:0]          CpaBucket7; 
   wire [7:0]          CpaBucket8; 
   wire [7:0]          CpaBucket9; 
   wire [7:0]          CpaBucket10;
   wire [7:0]          CpaBucket11;
   wire [7:0]          CpaBucket12;
   wire [7:0]          CpaBucket13;
   wire [7:0]          CpaBucket14;
   wire [7:0]          CpaBucket15;
   wire [63:0]         AvfsBucket0;
   wire [63:0]         AvfsBucket1;
   wire                AvfsCountSamples;
   wire [8:0]          CpaAccum;
   wire [8:0]          CpaAccum_MC;
   wire [7:0]          CpaAccumOut;
   wire                CgenCpaBucket0;
   wire                CgenCpaBucket1; 
   wire                CgenCpaBucket2; 
   wire                CgenCpaBucket3; 
   wire                CgenCpaBucket4; 
   wire                CgenCpaBucket5; 
   wire                CgenCpaBucket6; 
   wire                CgenCpaBucket7; 
   wire                CgenCpaBucket8; 
   wire                CgenCpaBucket9; 
   wire                CgenCpaBucket10;
   wire                CgenCpaBucket11;
   wire                CgenCpaBucket12;
   wire                CgenCpaBucket13;
   wire                CgenCpaBucket14;
   wire                CgenCpaBucket15;
   wire                AvfsEn_D1;
   wire [47:0]         BucketValid;
   wire [47:0]         AvfsValid;
   wire [47:0]         BucketValidIn;
   wire [15:0]         BucketValidLock;
   wire [149:0]        AvfsPathNorm;
   wire [2:0]          ValidThreshold;
   wire [63:0]         AvfsCtl1;
   wire [62:0]         AvfsCtl2;
   wire [60:0]         AvfsNorm0;
   wire [49:0]         AvfsNorm1;
   wire [49:0]         AvfsNorm2;
   wire                CgenSrbAvfsCtl1WrEn;
   wire                CgenSrbAvfsCtl2WrEn;
   wire                CgenSrbAvfsNorm0WrEn;
   wire                CgenSrbAvfsNorm1WrEn;
   wire                CgenSrbAvfsNorm2WrEn;

   wire AvfsPropDone;
   wire PsmClkEdge;
   wire PathCollectCntDone;
   wire [6:0] AvfsPathCollectTime;
   wire [7:0] AvfsDelayEn;
   wire AvfsStart;
   wire AvfsGoSync;
   wire AvfsGoSyncIn;

   wire AvfsSmAnyProp;
   wire CgenAvfsPropCnt;
   wire [2:0] AvfsPropCntIn;
   wire [2:0]  AvfsPropCnt;

   wire AvfsSmAnyPathCollect;
   wire CgenAvfsPathCollectCnt;
   wire [12:0] AvfsPathCollectCntIn;
   wire [12:0] AvfsPathCollectCnt;

   wire CgenAvfsSm;
   reg  [4:0] AvfsStateIn;
   wire [4:0]  AvfsState;

   wire CgenPsmUpdate;

   wire AvfsGoEdge;
   wire AvfsGo_D1;   
   wire PsmClk_D1;

   wire Round;

   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsEn;
   
// General Timing
// 
// PSM drives a new output/delta every 4 cycles preceded by the PsmClk
// |        |        |        |        |        |        |        |        |
// |        |PsmDelta|        |        |        |PsmDelta|        |        |
// |PsmClk  |        |        |        |PsmClk  |        |        |        |
// |  
// 
// CPA receives the PsmClk, delays it, and uses that to gate it's 4-cycle CPA pipe (PsmClkEdge)
//   This aligns the PSM measure period with the AVFS path exercise cycles (see below)
// |        |PsmClkE1|
// |        |        |PClkEdge|        |        |        |PClkEdge|        |
//
// The CPA has 2 separate pipelines.  Both use the CPA PsmClkEdge to advance a 4-cycle per step pipeline
//   Pipe1 calculates the NearMiss total (pop count) over the 4 cycle period.
//   Pipe2 calculates the PSM Adjustment shift (multiply PSMDelta(reciprocal)*BucketSize), and add it to the current delay setting.
//
// 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2|
// 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2|
//                                            |
// CPA Pipe1                                  |
// Path exercise                              |Paths run 
// 1 2 3 4                                    |
//   SFFGSPD                                  |  Speed flops generate NearMiss outputs
//   1 2 3 4                                  |
//     Sticky(step1)                          |    Sticky bits gather NearMisses over 4 cycles
//     1 2 3 4                                |
//             Sync(step2)                    |      Sync (at least 3 cycles)
//             1 - - -                        |
//                     PopCnt(step3)          |        Pop count path NearMisses
//                     1 - - -                |
//                             Accum(step4)   |          Generate final path count
//                             1 - - -        |       
// CPA Pipe2                           Bucket(step5)       Capture path count into selected bucket reg
// Bucket select                       1 - - -|
//                                            |Edge(step6)
//                             Mul&Add        |        Multiply  PsmDelta(reciprocal) * BucketSize to generate PSMAdjust shift
//                             1 - - -        |         plus Generate final bucket selection
//                             ^              |
//                             |              |
//                             PsmDelta(@CPA) |        Note: 2-cycle skew between PSM 4-cycle period and CPA 4-cycle period)
//                             1 - - -        | 
//                         PSMOutput(PSM drv) |    Drive PSM delta to CPA
//                         1 - - -            |
//                 PSM internal flop2         |  PSM average
//                 1 - - -                    |
//         PSM internal flop1                 |
//         1 - - -                            |
// PSM measure                                |PSM measures ticks per cycle
// 1 2 3 4                                    |
//                                            |


// State Machine
// 
// Current             Condition        Next             
//  -------             ---------        ----             
// Idle                AvfsGo           DlyChk0            
// DlyChk0             ~DelayEn[0]      DlyChk1            
//                     DelayEn[0]       Prop0            
// Prop0               init done        PathCollect0     
// PathCollect0        DelayTimer=0     DlyChk1            
// DlyChk1             ~DelayEn[1]      DlyChk2
//                     DelayEn[1]       Prop1
// Prop1               init done        PathCollect1
// PathCollect1        DelayTimer=0     DlyChk2
// DlyChk2
// Prop2
// PathCollect2
// DlyChk3
// Prop3
// PathCollect3
// Test4
// Init4
// PathCollect4
// DlyChk5
// Prop5
// PathCollect5
// DlyChk6
// Prop6
// PathCollect6
// DlyChk7             ~DelayEn[7]      Idle             
//                     DelayEn[7]       Prop7            
// Prop7               init done        PathCollect7     
// PathCollect7        Idle
//    

   wire [1:0] AvfsTestMode_pre;
   wire       ScanWrapCtlMC4;
   wire       AtpgMC4Protect;

   // AvfsTestMode & AvfsTestData are scan-only signals that can be used to configure the delay elements in the
   // gate & wire packs.
   wire [`CLKBUS_SIZE-1:0] clkbus_scan_only;

  gfx_CORE_CG I_cg_scan_only (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(1'b0), .R(1'b0), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE #(.size(2)) I_AvfsTestMode_pre (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(2'b0), .Q(AvfsTestMode_pre[1:0]));
  gfx_CORE_FLOP_DGATE             I_AvfsTestData     (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(1'b0), .Q(AvfsTestData));
  gfx_CORE_FLOP_DGATE             I_ScanWrapCtlMC4   (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(1'b0), .Q(ScanWrapCtlMC4));

   wire       LBistEn_CDC;

   // CDCBUF for LBistEn (ATPG test JTAG -> CCLK)
   //CORE_CDCBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES))
  gfx_CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY))
     I_CDC_LBistEn (.A(LBistEn),
                    .Z(LBistEn_CDC),
                    .EN(1'b1),
                    .DSTCLK(CCLK) );
   
   assign AvfsTestMode[1:0] = AvfsTestMode_pre[1:0] | {2{LBistEn_CDC}};
   
   assign AtpgMC4Protect = ScanWrapCtlMC4 | LBistEn_CDC;

   assign     AvfsSmAnyProp = (AvfsState[4:0] == `AVFS_SM_PROP0) | (AvfsState[4:0] == `AVFS_SM_PROP1) |
                              (AvfsState[4:0] == `AVFS_SM_PROP2) | (AvfsState[4:0] == `AVFS_SM_PROP3) |
                              (AvfsState[4:0] == `AVFS_SM_PROP4) | (AvfsState[4:0] == `AVFS_SM_PROP5) |
                              (AvfsState[4:0] == `AVFS_SM_PROP6) | (AvfsState[4:0] == `AVFS_SM_PROP7);

   assign     CgenAvfsPropCnt = AvfsSmAnyProp;
   assign     AvfsPropCntIn[2:0] = AvfsPropCnt[2:0] + 3'b1;
   assign     AvfsPropDone = (AvfsPropCnt[2:0] == 3'h7);   // propagation time so new AvfsPathDelay is reflected in NearMiss inputs

  gfx_CORE_FLOP_GATED #(.size(3))  I_AvfsPropCnt    (.D(AvfsPropCntIn[2:0]), .Q(AvfsPropCnt[2:0]), .G(CgenAvfsPropCnt), `CORE_FLOP_TAIL);

// The PsmClk input is a single cycle pulse preceeding the PsmDelta data.  It pulses once every 4 cycles, 1000100010001
// So, we don't need an edge detect.  
// PsmClk is delayed by 2 cycles here to line things up
  gfx_CORE_FLOP_DGATE  #(.size(1))  I_PsmClkEdge  (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(PsmClk), .Q(PsmClk_D1));
  gfx_CORE_FLOP_DGATE  #(.size(1))  I_PsmClkEdge2 (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(PsmClk_D1), .Q(PsmClkEdge));

   assign     AvfsSmAnyPathCollect = (AvfsState[4:0] == `AVFS_SM_PATHCOLLECT0) | (AvfsState[4:0] == `AVFS_SM_PATHCOLLECT1) |
                                     (AvfsState[4:0] == `AVFS_SM_PATHCOLLECT2) | (AvfsState[4:0] == `AVFS_SM_PATHCOLLECT3) |
                                     (AvfsState[4:0] == `AVFS_SM_PATHCOLLECT4) | (AvfsState[4:0] == `AVFS_SM_PATHCOLLECT5) |
                                     (AvfsState[4:0] == `AVFS_SM_PATHCOLLECT6) | (AvfsState[4:0] == `AVFS_SM_PATHCOLLECT7);

   assign     CgenAvfsPathCollectCnt = AvfsSmAnyProp | AvfsSmAnyPathCollect;                                   // clr on any PROP
   assign     AvfsPathCollectCntIn[12:0] = (AvfsSmAnyPathCollect) ? AvfsPathCollectCnt[12:0] + 13'b1 : 13'h0001;  // init gets 1 (save a subtract operation)

  gfx_CORE_FLOP_GATED  #(.size(13)) I_AvfsPathCollectCnt (.D(AvfsPathCollectCntIn[12:0]), .Q(AvfsPathCollectCnt[12:0]), 
                                            .G(CgenAvfsPathCollectCnt), `CORE_FLOP_TAIL);
   assign PathCollectCntDone = (AvfsPathCollectCnt[12:0] == ({AvfsPathCollectTime[6:0], 6'b0}));


// AvfsGo is a pulse that may extend for multiple CCLKs
//   convert to an edge and sync to PsmClk
  gfx_CORE_FLOP_DGATE  #(.size(1)) I_AvfsGo_d1  (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsGo), .Q(AvfsGo_D1));
   assign AvfsGoEdge = AvfsGo & ~AvfsGo_D1;
   assign AvfsGoSyncIn = AvfsGoEdge | (AvfsGoSync & ~PsmClkEdge);   // set on AvfsGo, hold until the next PsmClk is seen
  gfx_CORE_FLOP_DGATE  #(.size(1)) I_AvfsGoSync (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsGoSyncIn), .Q(AvfsGoSync));

   assign AvfsPathEn_m1 = AvfsSmAnyProp | AvfsSmAnyPathCollect;  // output to tell path logic to start generating data 
// Note, this flop adds delay to the loop from PathEn through the path to SFFGSPD back to input NearMiss signals.  
// This total delay is covered by the Propagation delay and AvfsPropCnt before paths will be accumulated. 
  gfx_CORE_FLOP_DGATE  #(.size(1)) I_AvfsPathEn (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsPathEn_m1), .Q(AvfsPathEn));

   assign AvfsStart = AvfsGoSync & // start testing paths when SPMI input bit is set 
                      PsmClkEdge &     // and the PSM is aligned (the delta value changed)
                      AvfsEn;      // and this CPA is enabled

// ** does the state advancement around PathCollect need to be aligned with PSM clk?.. PropDone & PsmClk
//    or only advance state machine aligned with PSM clk? 

   always @*
   casez ({AvfsState[4:0], AvfsStart, AvfsDelayEn[7:0], AvfsPropDone, PathCollectCntDone})
     {`AVFS_SM_IDLE,         11'b0_????????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_IDLE;
     {5'b?????,              11'b1_????????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK0;   // restart flow from any state on AvfsGo

     {`AVFS_SM_DLYCHK0,      11'b0_???????0_?_?} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK1; 
     {`AVFS_SM_DLYCHK0,      11'b0_???????1_?_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP0; 
     {`AVFS_SM_PROP0,        11'b0_????????_0_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP0; 
     {`AVFS_SM_PROP0,        11'b0_????????_1_?} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT0; 
     {`AVFS_SM_PATHCOLLECT0, 11'b0_????????_?_0} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT0; 
     {`AVFS_SM_PATHCOLLECT0, 11'b0_????????_?_1} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK1; 

     {`AVFS_SM_DLYCHK1,      11'b0_??????0?_?_?} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK2; 
     {`AVFS_SM_DLYCHK1,      11'b0_??????1?_?_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP1; 
     {`AVFS_SM_PROP1,        11'b0_????????_0_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP1; 
     {`AVFS_SM_PROP1,        11'b0_????????_1_?} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT1; 
     {`AVFS_SM_PATHCOLLECT1, 11'b0_????????_?_0} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT1; 
     {`AVFS_SM_PATHCOLLECT1, 11'b0_????????_?_1} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK2; 

     {`AVFS_SM_DLYCHK2,      11'b0_?????0??_?_?} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK3; 
     {`AVFS_SM_DLYCHK2,      11'b0_?????1??_?_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP2; 
     {`AVFS_SM_PROP2,        11'b0_????????_0_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP2; 
     {`AVFS_SM_PROP2,        11'b0_????????_1_?} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT2; 
     {`AVFS_SM_PATHCOLLECT2, 11'b0_????????_?_0} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT2; 
     {`AVFS_SM_PATHCOLLECT2, 11'b0_????????_?_1} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK3; 

     {`AVFS_SM_DLYCHK3,      11'b0_????0???_?_?} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK4; 
     {`AVFS_SM_DLYCHK3,      11'b0_????1???_?_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP3; 
     {`AVFS_SM_PROP3,        11'b0_????????_0_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP3; 
     {`AVFS_SM_PROP3,        11'b0_????????_1_?} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT3; 
     {`AVFS_SM_PATHCOLLECT3, 11'b0_????????_?_0} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT3; 
     {`AVFS_SM_PATHCOLLECT3, 11'b0_????????_?_1} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK4; 

     {`AVFS_SM_DLYCHK4,      11'b0_???0????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK5; 
     {`AVFS_SM_DLYCHK4,      11'b0_???1????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP4; 
     {`AVFS_SM_PROP4,        11'b0_????????_0_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP4; 
     {`AVFS_SM_PROP4,        11'b0_????????_1_?} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT4; 
     {`AVFS_SM_PATHCOLLECT4, 11'b0_????????_?_0} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT4; 
     {`AVFS_SM_PATHCOLLECT4, 11'b0_????????_?_1} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK5; 

     {`AVFS_SM_DLYCHK5,      11'b0_??0?????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK6; 
     {`AVFS_SM_DLYCHK5,      11'b0_??1?????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP5; 
     {`AVFS_SM_PROP5,        11'b0_????????_0_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP5; 
     {`AVFS_SM_PROP5,        11'b0_????????_1_?} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT5;
     {`AVFS_SM_PATHCOLLECT5, 11'b0_????????_?_0} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT5; 
     {`AVFS_SM_PATHCOLLECT5, 11'b0_????????_?_1} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK6; 

     {`AVFS_SM_DLYCHK6,      11'b0_?0??????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK7; 
     {`AVFS_SM_DLYCHK6,      11'b0_?1??????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP6; 
     {`AVFS_SM_PROP6,        11'b0_????????_0_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP6; 
     {`AVFS_SM_PROP6,        11'b0_????????_1_?} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT6; 
     {`AVFS_SM_PATHCOLLECT6, 11'b0_????????_?_0} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT6; 
     {`AVFS_SM_PATHCOLLECT6, 11'b0_????????_?_1} : AvfsStateIn[4:0] = `AVFS_SM_DLYCHK7; 

     {`AVFS_SM_DLYCHK7,      11'b0_0???????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_IDLE; 
     {`AVFS_SM_DLYCHK7,      11'b0_1???????_?_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP7; 
     {`AVFS_SM_PROP7,        11'b0_????????_0_?} : AvfsStateIn[4:0] = `AVFS_SM_PROP7; 
     {`AVFS_SM_PROP7,        11'b0_????????_1_?} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT7; 
     {`AVFS_SM_PATHCOLLECT7, 11'b0_????????_?_0} : AvfsStateIn[4:0] = `AVFS_SM_PATHCOLLECT7; 
     {`AVFS_SM_PATHCOLLECT7, 11'b0_????????_?_1} : AvfsStateIn[4:0] = `AVFS_SM_IDLE;
     default: AvfsStateIn[4:0] = 5'bx;
   endcase // casez(AvfsState[4:0], AvfsStart, AvfsDelayEn[7:0], AvfsPropDone, PathCollectCntDone)

   assign CgenAvfsSm = AvfsStart | ((AvfsState[4:0] != `AVFS_SM_IDLE) & PsmClkEdge);

  gfx_CORE_FLOP_GATED  #(.size(5)) I_AvfsState (.D(AvfsStateIn[4:0]), .Q(AvfsState[4:0]), .G(CgenAvfsSm), `CORE_FLOP_TAIL);

// Generate 3b AvfsDelay value for this period of path testing
  gfx_CORE_ENC3 I_ENC3 (.A({((AvfsState[4:0] == `AVFS_SM_PATHCOLLECT7) | (AvfsState[4:0] == `AVFS_SM_PROP7)),  // 7 to 3 encoder from rtl_common_lib.v
                    ((AvfsState[4:0] == `AVFS_SM_PATHCOLLECT6) | (AvfsState[4:0] == `AVFS_SM_PROP6)),
                    ((AvfsState[4:0] == `AVFS_SM_PATHCOLLECT5) | (AvfsState[4:0] == `AVFS_SM_PROP5)),
                    ((AvfsState[4:0] == `AVFS_SM_PATHCOLLECT4) | (AvfsState[4:0] == `AVFS_SM_PROP4)),
                    ((AvfsState[4:0] == `AVFS_SM_PATHCOLLECT3) | (AvfsState[4:0] == `AVFS_SM_PROP3)),
                    ((AvfsState[4:0] == `AVFS_SM_PATHCOLLECT2) | (AvfsState[4:0] == `AVFS_SM_PROP2)),
                    ((AvfsState[4:0] == `AVFS_SM_PATHCOLLECT1) | (AvfsState[4:0] == `AVFS_SM_PROP1)),
                    ((AvfsState[4:0] == `AVFS_SM_PATHCOLLECT0) | (AvfsState[4:0] == `AVFS_SM_PROP0))}),
                .Z(AvfsDelay[2:0]));


// Receive PSM output (difference between PSM average and PSM instantaneous/4 cycle avg value)
// Convert PsmDelta output from 2's complement to sign,magnitude so we can multiply unsigned (compile issues with signed muls)
// Positive values require saturation if the input exceeds 127 (0x7f)
// Negative values need to convert the magnitude format (2's comp) if >= -127 
//   or saturate at -127 if input equals or exceeds -128 (== -128 is special case that can't be represented in sign,magnitude)
   assign PsmOutputMaxNeg = (PsmOutput[7:0] == 8'h80);                 // 2's comp max neg(-128) will become sign,mag max neg(-127)
   assign PsmNegOver      =  PsmOutput[13] & (PsmOutput[12:8] != 5'h1f); 
   assign PsmPosOver      = ~PsmOutput[13] & (PsmOutput[12:8] != 5'b0);
   assign PsmSignMag[7:0] = {PsmOutput[13],                            // sign bit (only used to select add/sub of final adjust)
                              PsmOutput[13] ? (~PsmOutput[6:0] + 7'b1) |           // Neg: 2's comp 
                                             {7{PsmOutputMaxNeg | PsmNegOver}} :  //      or saturate at -127
                             ~PsmOutput[13] ? PsmOutput[6:0] |                     // Pos: no change 
                                             {7{PsmPosOver}} : 7'bx};             //      or saturate at +127
   assign CgenMulEn = CpaStep4Adv;

   wire [`CLKBUS_SIZE-1:0] clkbus_CgenMulEn;

  gfx_CORE_CG I_cg_CgenMulEn (.clkbus(clkbus_CgenMulEn[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenMulEn), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE  #(.size(8)) I_PsmDelta (.clkbus(clkbus_CgenMulEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(PsmSignMag[7:0]), .Q(PsmDelta[7:0]));

//step1 - pathcollect all 4 cycles into sticky flops
//step2 - sync
//step3 - popcnt   Virtual bucket
//step4 - bucket  
//step5 - EdgeByte
   assign CgenPsmUpdate = AvfsEn & PsmClkEdge;
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenPsmUpdate;

  gfx_CORE_CG I_cg_CgenPsmUpdate (.clkbus(clkbus_CgenPsmUpdate[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenPsmUpdate), .R(Reset), .SSE(ScanShiftEn));

   assign CpaStep1Active = AvfsSmAnyPathCollect;
   assign CpaStep2Adv    = AvfsSmAnyPathCollect & PsmClkEdge;

  gfx_CORE_FLOP_DGATE  #(.size(1)) I_CpaStep2 (.clkbus(clkbus_CgenPsmUpdate[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CpaStep1Active), .Q(CpaStep2Active));
   assign CpaStep3Adv = CpaStep2Active & PsmClkEdge;
  gfx_CORE_FLOP_DGATE  #(.size(1)) I_CpaStep3 (.clkbus(clkbus_CgenPsmUpdate[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CpaStep2Active), .Q(CpaStep3Active));
   assign CpaStep4Adv = CpaStep3Active & PsmClkEdge;
  gfx_CORE_FLOP_DGATE  #(.size(1)) I_CpaStep4 (.clkbus(clkbus_CgenPsmUpdate[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CpaStep3Active), .Q(CpaStep4Active));
   assign CpaStep5Adv = CpaStep4Active & PsmClkEdge;

// Receive inputs from the 50 speed flops and concatenate into a 50b vector
// Update EVERY cycle, ORing input to accumulate NearMisses, but clear every 4 cycles as data advances to Sync flops
   assign CpaStickyIn[49:0] = AvfsNearMiss[49:0] | (CpaSticky[49:0] & {50{~(CpaStep2Adv | AvfsPropDone)}});
   assign CgenStickyEn = (CpaStep1Active | AvfsPropDone) & AvfsEn;

  gfx_CORE_FLOP_GATED  #(.size(50)) I_CpaSticky (.D(CpaStickyIn[49:0]), .Q(CpaSticky[49:0]), .G(CgenStickyEn), `CORE_FLOP_TAIL);

// This flop updates once every time PSM Delta updates 
   assign CgenCpaSyncEn = CpaStep2Adv;
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenCpaSyncEn;

  gfx_CORE_CG I_cg_CgenCpaSyncEn (.clkbus(clkbus_CgenCpaSyncEn[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenCpaSyncEn), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE  #(.size(50)) I_CpaSync (.clkbus(clkbus_CgenCpaSyncEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CpaSticky[49:0]), .Q(CpaSync[49:0]));
   
// Mask out unwanted/unneeded signals using a mask provided by the SRB 
   assign FinalPathMask[49:0] = AvfsPathMask[49:0];
   assign CpaMaskOut[49:0] = CpaSync[49:0] & FinalPathMask[49:0]; // mask combinatorially since this is before sync

// Pop Count compresses input data down to 4b
// break 50b into 7 sets of 7, then pop count them down to 3b each, then add the 7 plus the leftover bit to a 4b sum
  gfx_POPCNT7 I_CpaPopCntG6 (.DIn(CpaMaskOut[48:42]), .DOut(CpaPopCntG6[2:0]));
  gfx_POPCNT7 I_CpaPopCntG5 (.DIn(CpaMaskOut[41:35]), .DOut(CpaPopCntG5[2:0]));
  gfx_POPCNT7 I_CpaPopCntG4 (.DIn(CpaMaskOut[34:28]), .DOut(CpaPopCntG4[2:0]));
  gfx_POPCNT7 I_CpaPopCntG3 (.DIn(CpaMaskOut[27:21]), .DOut(CpaPopCntG3[2:0]));
  gfx_POPCNT7 I_CpaPopCntG2 (.DIn(CpaMaskOut[20:14]), .DOut(CpaPopCntG2[2:0]));
  gfx_POPCNT7 I_CpaPopCntG1 (.DIn(CpaMaskOut[13:07]), .DOut(CpaPopCntG1[2:0]));
  gfx_POPCNT7 I_CpaPopCntG0 (.DIn(CpaMaskOut[06:00]), .DOut(CpaPopCntG0[2:0]));

// Now complete the compression, max count of 50 represented in 6b, no overflow
   assign CpaPopCntOut[5:0] = {3'h0,CpaPopCntG0[2:0]} + {3'h0,CpaPopCntG1[2:0]} +
                              {3'h0,CpaPopCntG2[2:0]} + {3'h0,CpaPopCntG3[2:0]} +
                              {3'h0,CpaPopCntG4[2:0]} + {3'h0,CpaPopCntG5[2:0]} + 
                              {3'h0,CpaPopCntG6[2:0]} + {5'h00,CpaMaskOut[49]};

// The pop count has 4 cycles to complete because the CPA pipeline only advances when the PSM delta updates. 
   wire [5:0] CpaPopCntOut_MC_PreAtpgProtect;
  gfx_CORE_MCBUF #(.size(6), .Cycles(3)) I_CpaPopCnt_MC (.A(CpaPopCntOut[5:0])
                                                     ,.DSTCLK(CCLK)
                                                     ,.Z(CpaPopCntOut_MC_PreAtpgProtect[5:0]));
   assign CpaPopCntOut_MC[5:0] = CpaPopCntOut_MC_PreAtpgProtect[5:0] & {6{~AtpgMC4Protect}};
   assign CgenCpaPopCnt3 = CpaStep3Adv;
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenCpaPopCnt3;

  gfx_CORE_CG I_cg_CgenCpaPopCnt3 (.clkbus(clkbus_CgenCpaPopCnt3[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenCpaPopCnt3), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE #(.size(6)) I_CpaPopCnt_S3  (.clkbus(clkbus_CgenCpaPopCnt3[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(CpaPopCntOut_MC[5:0]), .Q(CpaPopCnt_S3[5:0]));
   assign CgenCpaPopCnt4 = CpaStep4Adv;
  gfx_CORE_FLOP_GATED #(.size(6)) I_CpaPopCnt_S4 (.D(CpaPopCnt_S3[5:0]), .Q(CpaPopCnt_S4[5:0]), .G(CgenCpaPopCnt4),  `CORE_FLOP_TAIL);

// Save AvfsDelay value that was driven into replicas during sticky stage for bucket assignment
  gfx_CORE_FLOP_DGATE #(.size(3)) I_AvfsDelaySync (.clkbus(clkbus_CgenCpaSyncEn[`CLKBUS_SIZE-1:0]),  .DG(1'b1), .D(AvfsDelay[2:0]),    .Q(AvfsDelaySync[2:0]));  // step 2
  gfx_CORE_FLOP_DGATE #(.size(3)) I_AvfsDelayCnt  (.clkbus(clkbus_CgenCpaPopCnt3[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsDelaySync[2:0]), .Q(AvfsDelayCnt[2:0])); // step 3
  gfx_CORE_FLOP_DGATE #(.size(3)) I_AvfsDelayMul  (.clkbus(clkbus_CgenMulEn[`CLKBUS_SIZE-1:0]),      .DG(1'b1), .D(AvfsDelayCnt[2:0]),  .Q(AvfsDelayMul[2:0]));      // step 4

// Early bucket selection based on Delay setting (using delay setting from ucode, not normalized delay setting sent to flops)
// DlyTable contains the bucket-delay assignment, 6b per delay setting.
// Bucket selection is done in a virtual space of 64 entries so there is room to shift into the visible space (32-47) based on PSM.
// Initial bucket selection is part of config (AvfsCtl2[DlyTable]). 
// Select a virtual bucket based on the mapping of (0-7)delay setting to (0-63)virtual buckets defined in the table.
always @*
  case (AvfsDelayMul[2:0])
    3'b000: VirtualBucket[5:0] = DlyTable[5:0];
    3'b001: VirtualBucket[5:0] = DlyTable[11:6];
    3'b010: VirtualBucket[5:0] = DlyTable[17:12];
    3'b011: VirtualBucket[5:0] = DlyTable[23:18];
    3'b100: VirtualBucket[5:0] = DlyTable[29:24];
    3'b101: VirtualBucket[5:0] = DlyTable[35:30];
    3'b110: VirtualBucket[5:0] = DlyTable[41:36];
    3'b111: VirtualBucket[5:0] = DlyTable[47:42];
    default: VirtualBucket[5:0] = 6'bx;
  endcase
  
// Functionally need to divide the PSMDelta value by the PSM bucket size config parameter to determine the bucket shift
// The implementation uses a multiplication times a reciprocal.
// Multiply a positive saturated PsmDelta (0-127) * PsmBucketSize (reciprocal of PSM ticks per bucket 0.bbbbbbbb)
// aaaaaaa * 0.bbbbbbbb = iiiiiii.ffffffff
// The max output is 1111111 * 0.11111111 = 1111110.10000001 = 1111111(after rounding)
// We need 7b of the rounded int (127-0 psm adjust range), iIIIIIII.Rfffffff(from 8b of mul output)
//! unused MulOut[31:15]
//! unused MulOut[6:0]
// leda N_014 off

// widened LHS to 32-bits to suppress ZAK vscan mismatch width warning (16b != 32b)
assign MulOut[31:0] = {1'b0, PsmDelta[6:0]} * PSMBucketSize[7:0];

// leda N_014 on
   
// Adjust bucket selection with PSM difference in virtual space so we can accomodate max PSM shift before saturating. 
//! unused Round
   assign {PSMAdjust[6:0], Round} = MulOut[14:7] + {8'b00000001};            // rounded 7 lsbs of the int adjust the bucket select
// The PSMAdjust is always positive, but can be larger than the FinalVirtualBucket range of 0-63, so if it is >= 64 (bit 6 is set),
// then we need to saturate into buckets 0 and 15 based on selection of Add or Sub.  
// Of the remaining values, the max add is 31+31 = 62(0111110) and can't overflow.
// Subtracts can wrap and must be forced into the 0 bucket.  FinalVirtualBucket[6] can be used for this. 
   assign FinalVirtualBucket[6:0] = ~PsmDelta[7] ? ({1'b0, VirtualBucket[5:0]} - {1'b0, PSMAdjust[5:0]}) :       // Sub pos delta
                                     PsmDelta[7] ? ({1'b0, VirtualBucket[5:0]} + {1'b0, PSMAdjust[5:0]}) : 7'bx; // Add neg delta
                                        
   assign ForceBucket15 = ( PsmDelta[7] & (PSMAdjust[6] | FinalVirtualBucket[6])); // Add large PSM delta 
   assign ForceBucket0  = (~PsmDelta[7] & (PSMAdjust[6] | FinalVirtualBucket[6]));  // Sub large PSM delta or wrap negative
   assign FinalBucketSel[15:0] = {(FinalVirtualBucket[5:0] > 46 & ~ForceBucket0) | ForceBucket15,   // 15 gets everything above virt bucket 46
                                  (FinalVirtualBucket[5:0] == 46) & ~ForceBucket15 & ~ForceBucket0,  // 14
                                  (FinalVirtualBucket[5:0] == 45) & ~ForceBucket15 & ~ForceBucket0,  // 13
                                  (FinalVirtualBucket[5:0] == 44) & ~ForceBucket15 & ~ForceBucket0,  // 12
                                  (FinalVirtualBucket[5:0] == 43) & ~ForceBucket15 & ~ForceBucket0,  // 11
                                  (FinalVirtualBucket[5:0] == 42) & ~ForceBucket15 & ~ForceBucket0,  // 10
                                  (FinalVirtualBucket[5:0] == 41) & ~ForceBucket15 & ~ForceBucket0,  // 9 
                                  (FinalVirtualBucket[5:0] == 40) & ~ForceBucket15 & ~ForceBucket0,  // 8 
                                  (FinalVirtualBucket[5:0] == 39) & ~ForceBucket15 & ~ForceBucket0,  // 7 
                                  (FinalVirtualBucket[5:0] == 38) & ~ForceBucket15 & ~ForceBucket0,  // 6 
                                  (FinalVirtualBucket[5:0] == 37) & ~ForceBucket15 & ~ForceBucket0,  // 5 
                                  (FinalVirtualBucket[5:0] == 36) & ~ForceBucket15 & ~ForceBucket0,  // 4 
                                  (FinalVirtualBucket[5:0] == 35) & ~ForceBucket15 & ~ForceBucket0,  // 3 
                                  (FinalVirtualBucket[5:0] == 34) & ~ForceBucket15 & ~ForceBucket0,  // 2 
                                  (FinalVirtualBucket[5:0] == 33) & ~ForceBucket15 & ~ForceBucket0,  // 1 
                                  (FinalVirtualBucket[5:0] < 33 & ~ForceBucket15) | ForceBucket0};  // 0 gets everything below virt bucket 33

// 4 cycles from PsmDelta through mul to generate this FinalBucketSel into the clock gaters 
   wire [15:0] FinalBucketSel_MC_PreAtpgProtect;
  gfx_CORE_MCBUF #(.size(16), .Cycles(3)) I_FinalBucketSel_MC (.A(FinalBucketSel[15:0])
                                                           ,.DSTCLK(CCLK)
                                                           ,.Z(FinalBucketSel_MC_PreAtpgProtect[15:0]));
   assign  FinalBucketSel_MC[15:0] =  FinalBucketSel_MC_PreAtpgProtect[15:0] & {16{~AtpgMC4Protect}}   ;
   
// Bucket clock gaters
// allow buckets to update anytime threshold=0 or if the valid bit is not yet locked (reached threshold)
   assign CgenCpaBucket15 = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[15] & (~BucketValidLock[15] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket14 = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[14] & (~BucketValidLock[14] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket13 = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[13] & (~BucketValidLock[13] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket12 = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[12] & (~BucketValidLock[12] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket11 = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[11] & (~BucketValidLock[11] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket10 = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[10] & (~BucketValidLock[10] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket9  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[9] & (~BucketValidLock[9] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket8  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[8] & (~BucketValidLock[8] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket7  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[7] & (~BucketValidLock[7] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket6  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[6] & (~BucketValidLock[6] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket5  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[5] & (~BucketValidLock[5] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket4  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[4] & (~BucketValidLock[4] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket3  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[3] & (~BucketValidLock[3] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket2  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[2] & (~BucketValidLock[2] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket1  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[1] & (~BucketValidLock[1] | (ValidThreshold[2:0] == 3'b0)));
   assign CgenCpaBucket0  = AvfsGoEdge | 
                            (CpaStep5Adv & FinalBucketSel_MC[0] & (~BucketValidLock[0] | (ValidThreshold[2:0] == 3'b0)));

// Select the final bucket for accumulation
always @*
   case (FinalBucketSel[15:0])
     16'h0001: FinalBucketOut[7:0] = CpaBucket0[7:0];
     16'h0002: FinalBucketOut[7:0] = CpaBucket1[7:0];
     16'h0004: FinalBucketOut[7:0] = CpaBucket2[7:0];
     16'h0008: FinalBucketOut[7:0] = CpaBucket3[7:0];
     16'h0010: FinalBucketOut[7:0] = CpaBucket4[7:0];
     16'h0020: FinalBucketOut[7:0] = CpaBucket5[7:0];
     16'h0040: FinalBucketOut[7:0] = CpaBucket6[7:0];
     16'h0080: FinalBucketOut[7:0] = CpaBucket7[7:0];
     16'h0100: FinalBucketOut[7:0] = CpaBucket8[7:0];
     16'h0200: FinalBucketOut[7:0] = CpaBucket9[7:0];
     16'h0400: FinalBucketOut[7:0] = CpaBucket10[7:0];
     16'h0800: FinalBucketOut[7:0] = CpaBucket11[7:0];
     16'h1000: FinalBucketOut[7:0] = CpaBucket12[7:0];
     16'h2000: FinalBucketOut[7:0] = CpaBucket13[7:0];
     16'h4000: FinalBucketOut[7:0] = CpaBucket14[7:0];
     16'h8000: FinalBucketOut[7:0] = CpaBucket15[7:0];
     default: FinalBucketOut[7:0] = 8'bx;
   endcase // casez(FinalBucketSel[15:0])

   assign CpaAccum[8:0] = {1'b0, FinalBucketOut[7:0]} +        // Add new path data to existing bucket contents, retain 10b output
                           {3'b000, AvfsCountSamples ? 6'b000001 :                  // Force count to 1 if only counting samples
                                    ~AvfsCountSamples ? CpaPopCnt_S4[5:0] : 6'bx }; // Normal case is to add the pop cnt value 
// The multiply and accum have 4 cycles to complete because the CPA pipeline only advances when the PSM delta updates.
// Apply MCBUF at level before reset logic so that reset happens in one cycle.
   wire [8:0] CpaAccum_MC_PreAtpgProtect ;
  gfx_CORE_MCBUF #(.size(9), .Cycles(3)) I_CpaAccum_MC (.A(CpaAccum[8:0])
                                                    ,.DSTCLK(CCLK)
                                                    ,.Z(CpaAccum_MC_PreAtpgProtect[8:0]));
   assign  CpaAccum_MC[8:0] = CpaAccum_MC_PreAtpgProtect[8:0]  & {9{~AtpgMC4Protect}}   ;
   
   assign CpaAccumOut[7:0] = AvfsGoEdge ? 8'b0 :                                                // clear at startup
                             (CpaAccum_MC[8] | (CpaAccum_MC[7:0] == 8'hff)) ? 8'd254 :          // saturate at 254 (255 indicates invalid)
                                                                              CpaAccum_MC[7:0]; // 

// capture the new accumulated sum of NearMisses in the selected bucket reg
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket0  (.D(CpaAccumOut[7:0]), .Q(CpaBucket0[7:0]),  .G(CgenCpaBucket0),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket1  (.D(CpaAccumOut[7:0]), .Q(CpaBucket1[7:0]),  .G(CgenCpaBucket1),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket2  (.D(CpaAccumOut[7:0]), .Q(CpaBucket2[7:0]),  .G(CgenCpaBucket2),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket3  (.D(CpaAccumOut[7:0]), .Q(CpaBucket3[7:0]),  .G(CgenCpaBucket3),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket4  (.D(CpaAccumOut[7:0]), .Q(CpaBucket4[7:0]),  .G(CgenCpaBucket4),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket5  (.D(CpaAccumOut[7:0]), .Q(CpaBucket5[7:0]),  .G(CgenCpaBucket5),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket6  (.D(CpaAccumOut[7:0]), .Q(CpaBucket6[7:0]),  .G(CgenCpaBucket6),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket7  (.D(CpaAccumOut[7:0]), .Q(CpaBucket7[7:0]),  .G(CgenCpaBucket7),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket8  (.D(CpaAccumOut[7:0]), .Q(CpaBucket8[7:0]),  .G(CgenCpaBucket8),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket9  (.D(CpaAccumOut[7:0]), .Q(CpaBucket9[7:0]),  .G(CgenCpaBucket9),  `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket10 (.D(CpaAccumOut[7:0]), .Q(CpaBucket10[7:0]), .G(CgenCpaBucket10), `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket11 (.D(CpaAccumOut[7:0]), .Q(CpaBucket11[7:0]), .G(CgenCpaBucket11), `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket12 (.D(CpaAccumOut[7:0]), .Q(CpaBucket12[7:0]), .G(CgenCpaBucket12), `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket13 (.D(CpaAccumOut[7:0]), .Q(CpaBucket13[7:0]), .G(CgenCpaBucket13), `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket14 (.D(CpaAccumOut[7:0]), .Q(CpaBucket14[7:0]), .G(CgenCpaBucket14), `CORE_FLOP_TAIL);
  gfx_CORE_FLOP_GATED #(.size(8)) I_CpaBucket15 (.D(CpaAccumOut[7:0]), .Q(CpaBucket15[7:0]), .G(CgenCpaBucket15), `CORE_FLOP_TAIL);

// force to 255 if invalid except for threshold 0 which allows buckets to count and be read
   assign AvfsBucket1[63:0] = {CpaBucket15[7:0] | {8{~BucketValidLock[15] & (ValidThreshold[2:0] != 3'b0)}}, 
                               CpaBucket14[7:0] | {8{~BucketValidLock[14] & (ValidThreshold[2:0] != 3'b0)}}, 
                               CpaBucket13[7:0] | {8{~BucketValidLock[13] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket12[7:0] | {8{~BucketValidLock[12] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket11[7:0] | {8{~BucketValidLock[11] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket10[7:0] | {8{~BucketValidLock[10] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket9[7:0] | {8{~BucketValidLock[9] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket8[7:0] | {8{~BucketValidLock[8] & (ValidThreshold[2:0] != 3'b0)}}};
   assign AvfsBucket0[63:0] = {CpaBucket7[7:0] | {8{~BucketValidLock[7] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket6[7:0] | {8{~BucketValidLock[6] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket5[7:0] | {8{~BucketValidLock[5] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket4[7:0] | {8{~BucketValidLock[4] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket3[7:0] | {8{~BucketValidLock[3] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket2[7:0] | {8{~BucketValidLock[2] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket1[7:0] | {8{~BucketValidLock[1] & (ValidThreshold[2:0] != 3'b0)}},
                               CpaBucket0[7:0] | {8{~BucketValidLock[0] & (ValidThreshold[2:0] != 3'b0)}}};

// Global enable 
  gfx_CORE_FLOP_DGATE #(.size(1)) I_AvfsEn_D1 (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsEn), .Q(AvfsEn_D1));  // Delay to create leading edge
   assign CgenAvfsEn = AvfsEn | AvfsEn_D1;   // Global Clock enable for any ungated flops

  gfx_CORE_CG I_cg_CgenAvfsEn (.clkbus(clkbus_CgenAvfsEn[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(CgenAvfsEn), .R(Reset), .SSE(ScanShiftEn));

// Bucket Valid bits 
// Buckets are marked valid if the FinalBucketSel selects them during data collection.  This is valid because we are guaranteed 
// that there will be at least one path being tested every 4 cycles which is the frequency that buckets can be written. 
// clear on AvfsGo
// clock gate to CpaStep5Adv, which is when buckets get populated
   wire CgenAvfsBucketValid;
   assign CgenAvfsBucketValid = AvfsGoEdge |                                         // only update valids until threshold reached
                                (CpaStep5Adv & ((FinalBucketSel_MC[15:0] & ~BucketValidLock[15:0]) != 16'b0)) ;   

   assign BucketValidIn[47:0] = AvfsGoEdge ? 48'b0 :                                 // clear all valid counts on AvfsGo
                                {BucketValid[47:45] + {2'b0, FinalBucketSel_MC[15]}, // incr valid counts as buckets are selected
                                 BucketValid[44:42] + {2'b0, FinalBucketSel_MC[14]}, // but stop when valid threshold is reached
                                 BucketValid[41:39] + {2'b0, FinalBucketSel_MC[13]},
                                 BucketValid[38:36] + {2'b0, FinalBucketSel_MC[12]},
                                 BucketValid[35:33] + {2'b0, FinalBucketSel_MC[11]},
                                 BucketValid[32:30] + {2'b0, FinalBucketSel_MC[10]},
                                 BucketValid[29:27] + {2'b0, FinalBucketSel_MC[9]},
                                 BucketValid[26:24] + {2'b0, FinalBucketSel_MC[8]},
                                 BucketValid[23:21] + {2'b0, FinalBucketSel_MC[7]},
                                 BucketValid[20:18] + {2'b0, FinalBucketSel_MC[6]},
                                 BucketValid[17:15] + {2'b0, FinalBucketSel_MC[5]},
                                 BucketValid[14:12] + {2'b0, FinalBucketSel_MC[4]},
                                 BucketValid[11:9] + {2'b0, FinalBucketSel_MC[3]},
                                 BucketValid[8:6] + {2'b0, FinalBucketSel_MC[2]},
                                 BucketValid[5:3] + {2'b0, FinalBucketSel_MC[1]},
                                 BucketValid[2:0] + {2'b0, FinalBucketSel_MC[0]}};

  gfx_CORE_FLOP_GATED #(.size(48)) I_AvfsValid (.D(BucketValidIn[47:0]), .Q(AvfsValid[47:0]), .G(CgenAvfsBucketValid), `CORE_FLOP_TAIL);
   assign BucketValid[47:0] = AvfsValid[47:0];

   assign BucketValidLock[15:0] = {BucketValid[47:45] == ValidThreshold[2:0],  // bucket valid stops incrementing when the 
                                   BucketValid[44:42] == ValidThreshold[2:0],  // valid threshold is reached
                                   BucketValid[41:39] == ValidThreshold[2:0],  // also, bucket increments stop except if 
                                   BucketValid[38:36] == ValidThreshold[2:0],  //       valid threshold is 0
                                   BucketValid[35:33] == ValidThreshold[2:0],
                                   BucketValid[32:30] == ValidThreshold[2:0],
                                   BucketValid[29:27] == ValidThreshold[2:0],
                                   BucketValid[26:24] == ValidThreshold[2:0],
                                   BucketValid[23:21] == ValidThreshold[2:0],
                                   BucketValid[20:18] == ValidThreshold[2:0],
                                   BucketValid[17:15] == ValidThreshold[2:0],
                                   BucketValid[14:12] == ValidThreshold[2:0],
                                   BucketValid[11:9] == ValidThreshold[2:0],
                                   BucketValid[8:6] == ValidThreshold[2:0],
                                   BucketValid[5:3] == ValidThreshold[2:0],
                                   BucketValid[2:0] == ValidThreshold[2:0]};

// generate Path-normalized Delay controls for speed flops
// Path normalization uses the characterized path data to adjust delay settings so they behave similarly wrt timing
// This used to include an addition of the norm value and the delay setting. Now there is a separate norm delay unit and control. 
   assign AvfsPathDelay[149:0] = AvfsPathNorm[149:0];  // Normalization is driven straight from the AvfsCtl Norm registers
   assign AvfsGlobalPathDelay[2:0] = AvfsDelay[2:0];   // Delay is driven from the state machine delay setting 

// Register Interfaces 
// SRB based
//  AvfsCtl1
// The SRB drives data in two 32b chunks. We need to capture both to get the 50b mask register. 

   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsCtl1;
   assign CgenSrbAvfsCtl1WrEn = SelAvfsCtl1 & SrbLclWrEn_4;

  gfx_CORE_CG I_cg_CgenAvfsCtl1 (.clkbus(clkbus_CgenAvfsCtl1[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .G(CgenSrbAvfsCtl1WrEn), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE_RESET  #(.size(64),.rv(64'h03ffffffffffffff)) I_AvfsCtl1 (.clkbus(clkbus_CgenAvfsCtl1[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsSrbWrDat_4[63:0]), .Q(AvfsCtl1[63:0]));
   assign AvfsEn = AvfsCtl1[63];    // global CPA enable
   assign AvfsCountSamples = AvfsCtl1[62];     // debug mode, only count samples (count 1 regardless of NearMiss inputs)
   assign AvfsAgeHigh = AvfsCtl1[61];
   assign AvfsDelayEn[7:0] = AvfsCtl1[57:50];
   assign AvfsPathMask[49:0] = AvfsCtl1[49:0];

// AvfsCtl1[60:58] are reserved bits - zeroes are returned for reads
//! unused AvfsCtl1[60:58]

//  AvfsCtl2: TestTime, AvfsThreshold, DlyTable
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsCtl2;
   assign CgenSrbAvfsCtl2WrEn = SelAvfsCtl2 & SrbLclWrEn_4;

  gfx_CORE_CG I_cg_CgenAvfsCtl2 (.clkbus(clkbus_CgenAvfsCtl2[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .G(CgenSrbAvfsCtl2WrEn), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE_RESET  #(.size(63),.rv(63'h0)) I_AvfsCtl2 (.clkbus(clkbus_CgenAvfsCtl2[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsSrbWrDat_4[62:0]), .Q(AvfsCtl2[62:0]));
   assign AvfsPathCollectTime[6:0] = AvfsCtl2[62:56];
   assign DlyTable[47:0] = AvfsCtl2[47:0];

//! unused AvfsCtl2[55:48]

//  AvfsNorm0: Norm17_0   16<1:0>,15-0      
//  AvfsNorm1: Norm34_17  33<0>,32-17,16<2> 
//  AvfsNorm2: Norm49_34  49-34,33<2:1>     
// Path normalization allows us to adjust the delay settings post-silicon so true path lengths are near equal for each delay value.
//   we can also introduce global delays into the normalization to move the sampling window into view especially when we are high
//   up on the load-line or we are in a situatuion where Fmax is constrained by some other core
   
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsNorm0;
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsNorm1;
   wire [`CLKBUS_SIZE-1:0] clkbus_CgenAvfsNorm2;

   assign CgenSrbAvfsNorm0WrEn = SelAvfsNorm0 & SrbLclWrEn_4;
   assign CgenSrbAvfsNorm1WrEn = SelAvfsNorm1 & SrbLclWrEn_4;
   assign CgenSrbAvfsNorm2WrEn = SelAvfsNorm2 & SrbLclWrEn_4;

  gfx_CORE_CG I_cg_CgenAvfsNorm0 (.clkbus(clkbus_CgenAvfsNorm0[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .G(CgenSrbAvfsNorm0WrEn), .R(Reset), .SSE(ScanShiftEn));
  gfx_CORE_CG I_cg_CgenAvfsNorm1 (.clkbus(clkbus_CgenAvfsNorm1[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .G(CgenSrbAvfsNorm1WrEn), .R(Reset), .SSE(ScanShiftEn));
  gfx_CORE_CG I_cg_CgenAvfsNorm2 (.clkbus(clkbus_CgenAvfsNorm2[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .G(CgenSrbAvfsNorm2WrEn), .R(Reset), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE_RESET  #(.size(61),.rv(61'h0)) I_AvfsNorm0 (.clkbus(clkbus_CgenAvfsNorm0[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsSrbWrDat_4[60:0]), .Q(AvfsNorm0[60:0]));
  gfx_CORE_FLOP_DGATE_RESET  #(.size(50),.rv(50'h0)) I_AvfsNorm1 (.clkbus(clkbus_CgenAvfsNorm1[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsSrbWrDat_4[49:0]), .Q(AvfsNorm1[49:0]));
  gfx_CORE_FLOP_DGATE_RESET  #(.size(50),.rv(50'h0)) I_AvfsNorm2 (.clkbus(clkbus_CgenAvfsNorm2[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsSrbWrDat_4[49:0]), .Q(AvfsNorm2[49:0]));
   assign AvfsPathNorm[149:0] = {AvfsNorm2[49:0], AvfsNorm1[49:0], AvfsNorm0[49:0]};
   assign PSMBucketSize[7:0] = AvfsNorm0[57:50]; 
   assign ValidThreshold[2:0] = AvfsNorm0[60:58];
   
// SRB Read interface
//   return zeroes for reserved bits
   assign AvfsSrbRdDat[63:0] = ({64{SelAvfsCtl1}}    & {AvfsCtl1[63:61], 3'b0, AvfsCtl1[57:0]}) |
                               ({64{SelAvfsCtl2}}    & {1'b0, AvfsCtl2[62:56], 8'b0, AvfsCtl2[47:0]}) |
                               ({64{SelAvfsBucket0}} & AvfsBucket0[63:0]) |
                               ({64{SelAvfsBucket1}} & AvfsBucket1[63:0]) |
                               ({64{SelAvfsValid}}   & {16'b0, AvfsValid[47:0]}) |
                               ({64{SelAvfsNorm0}}   & {3'b0, AvfsNorm0[60:0]}) |   
                               ({64{SelAvfsNorm1}}   & {14'b0, AvfsNorm1[49:0]}) |   
                               ({64{SelAvfsNorm2}}   & {14'b0, AvfsNorm2[49:0]});

endmodule // AVFS_CPA


// --------------------------------------------------------

// ------------------------------------------------------
// PSM Oscillator and Oscillator I/O
// ------------------------------------------------------

// PSM Oscillator I/O (priority encoder & mux)
// Note the following module is not parameterized

//  Design Notes: It is important to note that the prority encoder is designed such that it's
//    output starts with the value 32 corresponding to the delay of the pulse reset chain
//    inside the PSM oscillator custom block.Three bits are examined (instead of the minimum 1)
//    to avoid possible metastability issues. A metastable output from the PSM oscillator LSB bit
//    on terminal (a) will resolve to the nearest valid code that has 3 stable bits.

module gfx_CorePsmIpOscIO (
  input [31:0] psmOscLsb,
  input [7:0]  psmOscMsb0,
  input [7:0]  psmOscMsb1,
  
  output wire [5:0] psmLsb,
  output wire [7:0] psmMsb
);


//  wor [5:0]   ipsmLsb;
  wire [5:0]  ipsmLsb;
  wire        msbSel;
  
  assign ipsmLsb[5:0] =
    ( {6{({psmOscLsb[1:0],psmOscLsb[31]} == 3'b101)}}  & 6'd32 ) |  // ( ({psmOscLsb[1:0],psmOscLsb[31]} == 3'b101)  ? 6'd32    : 6'd0 ) |
    ( {6{(psmOscLsb[2:0] == 3'b100)}}                  & 6'd33 ) |  // ( (psmOscLsb[2:0] == 3'b100)                  ? 6'd33    : 6'd0 ) |
    ( {6{(psmOscLsb[3:1] == 3'b100)}}                  & 6'd34 ) |  // ( (psmOscLsb[3:1] == 3'b100)                  ? 6'd34    : 6'd0 ) |
    ( {6{(psmOscLsb[4:2] == 3'b100)}}                  & 6'd35 ) |  // ...
    ( {6{(psmOscLsb[5:3] == 3'b100)}}                  & 6'd36 ) |  // X-uglification
    ( {6{(psmOscLsb[6:4] == 3'b100)}}                  & 6'd37 ) |
    ( {6{(psmOscLsb[7:5] == 3'b100)}}                  & 6'd38 ) |
    ( {6{(psmOscLsb[8:6] == 3'b100)}}                  & 6'd39 ) |
    ( {6{(psmOscLsb[9:7] == 3'b100)}}                  & 6'd40 ) |
    ( {6{(psmOscLsb[10:8] == 3'b100)}}                 & 6'd41 ) |
    ( {6{(psmOscLsb[11:9] == 3'b100)}}                 & 6'd42 ) |
    ( {6{(psmOscLsb[12:10] == 3'b100)}}                & 6'd43 ) |
    ( {6{(psmOscLsb[13:11] == 3'b100)}}                & 6'd44 ) |
    ( {6{(psmOscLsb[14:12] == 3'b100)}}                & 6'd45 ) |
    ( {6{(psmOscLsb[15:13] == 3'b100)}}                & 6'd46 ) |
    ( {6{(psmOscLsb[16:14] == 3'b100)}}                & 6'd47 ) |
    ( {6{(psmOscLsb[17:15] == 3'b100)}}                & 6'd48 ) |
    ( {6{(psmOscLsb[18:16] == 3'b100)}}                & 6'd49 ) |
    ( {6{(psmOscLsb[19:17] == 3'b100)}}                & 6'd50 ) |
    ( {6{(psmOscLsb[20:18] == 3'b100)}}                & 6'd51 ) |
    ( {6{(psmOscLsb[21:19] == 3'b100)}}                & 6'd52 ) |
    ( {6{(psmOscLsb[22:20] == 3'b100)}}                & 6'd53 ) |
    ( {6{(psmOscLsb[23:21] == 3'b100)}}                & 6'd54 ) |
    ( {6{(psmOscLsb[24:22] == 3'b100)}}                & 6'd55 ) |
    ( {6{(psmOscLsb[25:23] == 3'b100)}}                & 6'd56 ) |
    ( {6{(psmOscLsb[26:24] == 3'b100)}}                & 6'd57 ) |
    ( {6{(psmOscLsb[27:25] == 3'b100)}}                & 6'd58 ) |
    ( {6{(psmOscLsb[28:26] == 3'b100)}}                & 6'd59 ) |
    ( {6{(psmOscLsb[29:27] == 3'b100)}}                & 6'd60 ) |
    ( {6{(psmOscLsb[30:28] == 3'b100)}}                & 6'd61 ) |
    ( {6{(psmOscLsb[31:29] == 3'b100)}}                & 6'd62 ) |
    ( {6{({psmOscLsb[0],psmOscLsb[31:30]} == 3'b000)}} & 6'd63 ) |
    ( {6{({psmOscLsb[1:0],psmOscLsb[31]} == 3'b010)}}  & 6'd0  ) |
    ( {6{(psmOscLsb[2:0] == 3'b011)}}                  & 6'd1  ) |
    ( {6{(psmOscLsb[3:1] == 3'b011)}}                  & 6'd2  ) |
    ( {6{(psmOscLsb[4:2] == 3'b011)}}                  & 6'd3  ) |
    ( {6{(psmOscLsb[5:3] == 3'b011)}}                  & 6'd4  ) |
    ( {6{(psmOscLsb[6:4] == 3'b011)}}                  & 6'd5  ) |
    ( {6{(psmOscLsb[7:5] == 3'b011)}}                  & 6'd6  ) |
    ( {6{(psmOscLsb[8:6] == 3'b011)}}                  & 6'd7  ) |
    ( {6{(psmOscLsb[9:7] == 3'b011)}}                  & 6'd8  ) |
    ( {6{(psmOscLsb[10:8] == 3'b011)}}                 & 6'd9  ) |
    ( {6{(psmOscLsb[11:9] == 3'b011)}}                 & 6'd10 ) |
    ( {6{(psmOscLsb[12:10] == 3'b011)}}                & 6'd11 ) |
    ( {6{(psmOscLsb[13:11] == 3'b011)}}                & 6'd12 ) |
    ( {6{(psmOscLsb[14:12] == 3'b011)}}                & 6'd13 ) |
    ( {6{(psmOscLsb[15:13] == 3'b011)}}                & 6'd14 ) |
    ( {6{(psmOscLsb[16:14] == 3'b011)}}                & 6'd15 ) |
    ( {6{(psmOscLsb[17:15] == 3'b011)}}                & 6'd16 ) |
    ( {6{(psmOscLsb[18:16] == 3'b011)}}                & 6'd17 ) |
    ( {6{(psmOscLsb[19:17] == 3'b011)}}                & 6'd18 ) |
    ( {6{(psmOscLsb[20:18] == 3'b011)}}                & 6'd19 ) |
    ( {6{(psmOscLsb[21:19] == 3'b011)}}                & 6'd20 ) |
    ( {6{(psmOscLsb[22:20] == 3'b011)}}                & 6'd21 ) |
    ( {6{(psmOscLsb[23:21] == 3'b011)}}                & 6'd22 ) |
    ( {6{(psmOscLsb[24:22] == 3'b011)}}                & 6'd23 ) |
    ( {6{(psmOscLsb[25:23] == 3'b011)}}                & 6'd24 ) |
    ( {6{(psmOscLsb[26:24] == 3'b011)}}                & 6'd25 ) |
    ( {6{(psmOscLsb[27:25] == 3'b011)}}                & 6'd26 ) |
    ( {6{(psmOscLsb[28:26] == 3'b011)}}                & 6'd27 ) |
    ( {6{(psmOscLsb[29:27] == 3'b011)}}                & 6'd28 ) |
    ( {6{(psmOscLsb[30:28] == 3'b011)}}                & 6'd29 ) |
    ( {6{(psmOscLsb[31:29] == 3'b011)}}                & 6'd30 ) |
    ( {6{({psmOscLsb[0],psmOscLsb[31:30]} == 3'b111)}} & 6'd31 );

  assign         msbSel      = ipsmLsb[5];
  assign         psmLsb[5:0] = ipsmLsb[5:0];
  assign         psmMsb[7:0] = msbSel ? psmOscMsb0[7:0] : (~msbSel ? psmOscMsb1[7:0] : 8'bx);

endmodule  // CorePsmIpOscIO



// ------------------------------------------------------
// PSM Core
// ------------------------------------------------------

// Gear Slipper kills mapsm oscEn input at a time which coincides with the oneShotCnt[2:0] = 001 (the 1st time).
// Releases the oscEn killing coinciding with oneShotCnt[2:0] = 000 which causes the mapsm psmOscGater counter
// to realign its pediodicity to when the full oneShotCnt[13:0] will time out and cause the oneshot  measurement.
// This module also issues dblsmpstart determinstically at the same time as the oscEn kill is released.

module gfx_CorePsmIpGearSlip (
  input [2:0] jtag_one_shot_count,
  input       jtag_discrete,  
  input [`CLKBUS_SIZE-1:0] clkbus_psmRun,
  input       trigReg2,
  output reg   slip,
  output wire  dblsmpstart  
);

  reg  [2:0]   slipCntNxt;
  wire [2:0]   slipCnt;
  wire         slipDone;
  
  always @* begin
    casez ({ trigReg2, slipDone, slipCnt[2:0]})
      5'b1_0_111: {slipCntNxt[2:0], slip} = 4'b110_0;
      5'b1_0_110: {slipCntNxt[2:0], slip} = 4'b101_0;
      5'b1_0_101: {slipCntNxt[2:0], slip} = 4'b100_0;
      5'b1_0_100: {slipCntNxt[2:0], slip} = 4'b011_0;
      5'b1_0_011: {slipCntNxt[2:0], slip} = 4'b010_0;
      5'b1_0_010: {slipCntNxt[2:0], slip} = 4'b001_0;
      5'b1_0_001: {slipCntNxt[2:0], slip} = 4'b000_1;
      5'b1_0_000: {slipCntNxt[2:0], slip} = 4'b111_0;

      5'b1_1_???: {slipCntNxt[2:0], slip} = 4'b000_0;
      5'b0_?_???: {slipCntNxt[2:0], slip} = {jtag_one_shot_count[2:0], 1'b0};
      // VCS coverage off
      default: {slipCntNxt[2:0], slip} = {4'bx};
      // VCS coverage on
    endcase
  end

  gfx_CORE_FLOP_DGATE_RESET #(.size(3)) CorePsmIpCore_SlipCntReg (.clkbus(clkbus_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(slipCntNxt[2:0]), .Q(slipCnt[2:0]));

  wire slipDoneNxt;
  wire dblsmpstartNxt;
  
  assign slipDoneNxt = (slip | slipDone) & trigReg2;

  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_SlipDoneReg (.clkbus(clkbus_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(slipDoneNxt), .Q(slipDone));

  assign dblsmpstartNxt = jtag_discrete & slip ;
   
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_DblSmpStrtReg (.clkbus(clkbus_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(dblsmpstartNxt), .Q(dblsmpstart));

endmodule  // CorePsmIpGearSlip

// PSM Core top level
module gfx_CorePsmIpCore #(parameter
  JTAG_ONE_SHOT_DELAY_W = 8,
  JTAG_CLK_DIV_W        = 2,
  JTAG_AVG_DIV_W        = 6,
  AVGPSM_W              = 14,
  JTAG_THRESHOLD_W      = 14,
  CURPSM_W              = 14,
  MINMAX_W              = 14,
  MCP_SETUP             = 1100,
  MCP_CYCLES_MAPSMRO    = 0,
  MCP_CYCLES_AVGPSMREG  = 3
)(
  input                                   ScanShiftEn,         // sse
  input                                   clk,                 // system clock
  input                                   Psm_ScanMode,        // Test Mode indcator
  input                                   Psm_TrstSyncd,       // JTAG TRST signal, sync'ed to CCLK
  input                                   psmEn,               // functional enable signal
  input                                   psmGater,            // system coarse clock gater
  input                                   psmOscGater,         // gater signal from oscillator
  input       [             CURPSM_W-1:0] curPsm_1,            // curPsm value from oscillator I/O module
  input       [             CURPSM_W-1:0] jtag_curPsm,         // jtag overide of current PSM oscillator value
  input                                   jtag_test,           // sets jtag_test mode to override PSM oscillator value
  input                                   jtag_rstPsm,         // resets internal registers
  input                                   jtag_psmEnable,      // master JTAG enable to turn-on entire PSM
  input                                   jtag_discrete,       // enables discrete measurement mode if set
  input                                   jtag_one_shot,       // one shot mode enable
  input       [JTAG_ONE_SHOT_DELAY_W-1:0] jtag_one_shot_count, // one shot offset delay
  input       [     JTAG_THRESHOLD_W-1:0] jtag_threshValue,    // min/max threshold value for alarm
  input                                   jtag_threshMaxPsm,   // if set threshold alarm value measures overshoots, default=undershoots
  input                                   jtag_measMaxPsm,     // if set tracks high water mark, default = low water mark
  input                                   jtag_deltaCmp,       // if set threshold compares curPsm - avgPsm, default = curPsm
  input                                   jtag_doubleSmp,      // enables double sample mode (not implemented yet)
  input       [       JTAG_AVG_DIV_W-1:0] jtag_avgDiv,         // average divisor/window
  input                                   jtag_avgEn,          // enables averaging circuit
  input                                   jtag_calibrate,      // enables update of the threshold register with abs or delta psm values
  input                                   jtag_threshWriteEn,  // threshold write enable
  input       [       JTAG_CLK_DIV_W-1:0] jtag_clkDiv,         // clkDiv value
  input                                   nullNxtAvg,          // bug 435656

  output wire [             CURPSM_W-1:0] curPsm,      // current PSM value
  output wire [             MINMAX_W-1:0] minMaxPsm,   // high or low water mark
  output wire                             alarm,       // sticky alarm bit
  output wire [             AVGPSM_W-1:0] avgPsm,      // average PSM output
  output wire                             oscEn,       // oscillator enable signal for oscillator

  // Additional test hooks for debug-bus observability
  output wire [     JTAG_THRESHOLD_W-1:0] thresholdReg,
  output wire [                     35:0] deltaPsmReg,
  output wire [JTAG_ONE_SHOT_DELAY_W-1:0] oneShotCnt,
  output wire [                      3:0] trigReg,
  output wire                             oneShotReg,
  
  // Needed for delta generation (i.e. used to flip jtag_curPsm[6])
  output wire                             dblSampEn_ddiv,
  
  // Porting out div_clk_gater for AVFS (optional at CORE_PSM_IP level, via `ifdef)
  output wire                             div_clk_gater
);

  // JTAG-controlled PSM reset signal
  // rstPsm goes high on posedge of jtag_rstPsm, then goes low 1 divided-CCLK cycles later
  // -- S.K. Modified so that rstPsm goes high at the beginning of the next divided-CCLK for 1 cycle
  wire jtag_rstPsm_d;
  wire jtag_rstPsm_d1;

  wire                    psmRun;
  wire [`CLKBUS_SIZE-1:0] clkbus_div_clk_gater;

  gfx_CORE_CG I_cg_div_clk_gater (.clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .C(clk), .G(div_clk_gater), .R(Psm_TrstSyncd), .SSE(ScanShiftEn));

  wire [`CLKBUS_SIZE-1:0] clkbus_avgEn_div_clk_gater;
  wire avgPsmTestIn ;
  wire [`CLKBUS_SIZE-1:0] clkbus_scan_only_CorePsmIpCore;
  assign avgEn_div_clk_gater = Psm_ScanMode ?  avgPsmTestIn : ( (jtag_avgEn & psmRun & psmOscGater) | Psm_TrstSyncd ); 
  
  // adding a scan only flop to contol the avgPsmReg clock gater in test mode. Help block multi-cycle failures (MCP-3 on avgPsmReg)
  gfx_CORE_CG I_cg_CorePsmIpCore_scan_only (.clkbus(clkbus_scan_only_CorePsmIpCore[`CLKBUS_SIZE-1:0]), .C(clk), .G(Psm_ScanMode), .R(1'b0), .SSE(ScanShiftEn));
  gfx_CORE_FLOP_DGATE I_avgPsm_ScanOnly_DFF (.clkbus(clkbus_scan_only_CorePsmIpCore[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(1'b0), .Q(avgPsmTestIn));

  gfx_CORE_CG I_cg_avgEn_div_clk_gater (.clkbus(clkbus_avgEn_div_clk_gater[`CLKBUS_SIZE-1:0]), .C(clk), .G(avgEn_div_clk_gater), .R(Psm_TrstSyncd), .SSE(ScanShiftEn));

//wire [`CLKBUS_SIZE-1:0] clkbus_discrete_div_clk_gater;
//
//assign discrete_div_clk_gater = (jtag_discrete & psmRun & psmOscGater) | Psm_TrstSyncd;
//
//CORE_CG I_cg_discrete_div_clk_gater (.clkbus(clkbus_discrete_div_clk_gater[`CLKBUS_SIZE-1:0]), .C(clk), .G(discrete_div_clk_gater), .R(Psm_TrstSyncd), .SSE(Psm_TrstSyncd));
  
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_jtag_rstPsm_d (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(jtag_rstPsm), .Q(jtag_rstPsm_d));
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_jtag_rstPsm_d1 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(~jtag_rstPsm_d), .Q(jtag_rstPsm_d1));
    
  assign rstPsm = jtag_rstPsm_d & jtag_rstPsm_d1;

  // Flop psmEn
  wire psmEn_1;
  gfx_CORE_FLOP_GATED_RESET #(.size(1)) CorePsmIpCore_psmEn_1 (.GC(1'b1), .G(1'b1), .C(clk), .D(psmEn), .R(Psm_TrstSyncd), .SSE(ScanShiftEn), .Q(psmEn_1));

  // Jtag Configurable Options
  wire                        measMaxPsm;   // if set high water mark set to max otherwise min
  wire                        deltaCmp;     // if set delta from Avg values used otherwise absolute and avgPsm = curPsm
  wire                        threshMaxPsm; // if set threshold detector works on overshoots otherwise undershoots
  wire [JTAG_AVG_DIV_W-1:0]   avgDiv;
  wire [JTAG_THRESHOLD_W-1:0] threshold; // Jtag threshold value
  wire                        doubleSmp;
  wire                        threshWriteEn;

  assign                      measMaxPsm = jtag_measMaxPsm;
  assign                      deltaCmp = jtag_deltaCmp;
  assign                      threshMaxPsm = jtag_threshMaxPsm;
  assign                      avgDiv[JTAG_AVG_DIV_W-1:0] = jtag_avgDiv[JTAG_AVG_DIV_W-1:0];
  assign                      avgEn = jtag_avgEn;
  assign                      threshold[JTAG_THRESHOLD_W-1:0] = jtag_threshValue[JTAG_THRESHOLD_W-1:0];
  assign                      doubleSmp = jtag_doubleSmp;
  assign                      threshWriteEn = jtag_threshWriteEn;
 
  // psmRun serves as clock gater for pseudo free running clock
  // assertion of Psm_TrstSyncd forces gater enable
  assign psmRun = (jtag_psmEnable & psmGater) | Psm_TrstSyncd;

  wire [`CLKBUS_SIZE-1:0] clkbus_psmRun;

  gfx_CORE_CG I_cg_psmRun (.clkbus(clkbus_psmRun[`CLKBUS_SIZE-1:0]), .C(clk), .G(psmRun), .R(Psm_TrstSyncd), .SSE(ScanShiftEn));

  // discussion with SteveKosonocky & MiguelRodriguez suggests that since DLDO will only operate in continuous mode, we can use the "jtag_discrete" to gate off unused function (i.e. oneshot, trigreg, slip, DeltaCmp, Calibrate, DoubleSmp, Discrete)
  assign discrete_psmRun = ((jtag_discrete |
                             trigReg[1]   // detect when "trigMode" de-asserts, b/c "clkbus_discrete_psmRun" must remain active for two extra cycles to reset all "trigReg[3:0]" flops
                            ) & jtag_psmEnable & psmGater) | Psm_TrstSyncd;
  wire [`CLKBUS_SIZE-1:0] clkbus_discrete_psmRun;
  
  gfx_CORE_CG I_cg_discrete_psmRun (.clkbus(clkbus_discrete_psmRun[`CLKBUS_SIZE-1:0]), .C(clk), .G(discrete_psmRun), .R(Psm_TrstSyncd), .SSE(ScanShiftEn));


  wire [CURPSM_W-1:0]       curPsmOrTest_1;
  wire                      psmGate;
  wire                      psmGate_ddiv1;
  wire                      psmGate_ddiv2;

  // Gear-slip logic
  wire                      slip;
  wire                      dblsmpstart;

  gfx_CorePsmIpGearSlip CorePsmIpCore_CorePsmIpGearSlip (
    .trigReg2             (trigReg[2]),
    .clkbus_psmRun        (clkbus_discrete_psmRun),
    .jtag_discrete        (jtag_discrete),
    .jtag_one_shot_count  (jtag_one_shot_count[2:0]),  // only taking last 3 LSBs of jtag_one_shot_count
    .slip                 (slip),
    .dblsmpstart          (dblsmpstart)
  );
 
  // Gear sliping is done by killing oscEn
  // When oscEn => 0, the mapsm psmOscGater counter is reset
  
  assign oscEn = psmRun & ~slip;

  // Mux to select curPsm from oscillator or JTAG parameter
  
  // Move the MCBUF here on curPsm_1
  //wire [CURPSM_W-1:0] curPsm_1_mcp ;
  //wire mcbufEn;
  //assign mcbufEn = (~Psm_TrstSyncd) & psmOscGater & oscEn ; 
  //CORE_MCENBUF #(.size(CURPSM_W), .Cycles(MCP_CYCLES_MAPSMRO)) I_curPsm_1_mcp (.A(curPsm_1), 
  //                                                                             .EN(mcbufEn),
  //                                                                             .DSTCLK(clk),
  //                                                                             .Z(curPsm_1_mcp));

  // would adding "OR" with LBistEn to jtag_test preclude usage of mcbufEn for fencing above
  // Recommend fencing using this structure 
  assign curPsmOrTest_1[CURPSM_W-1:0] = (jtag_test|Psm_ScanMode) ? jtag_curPsm[CURPSM_W-1:0] : (~(Psm_ScanMode|jtag_test) ? curPsm_1[CURPSM_W-1:0] : {CURPSM_W{1'bx}});
  
  //
  // Divided-clock gater
  //

  // div_clk_gater serves as clock gater for divided gated clock
  // assertion of Psm_TrstSyncd forces gater enable
  assign div_clk_gater = (psmRun & psmOscGater) | Psm_TrstSyncd;

  //
  // psmOsc Output Register, 1st pipeline point after priority encoder/mux.
  //

  wire [CURPSM_W-1:0] curPsm_2;
  wire [CURPSM_W-1:0] curPsm_2_nxt;
  wire [CURPSM_W-1:0] curPsm_2_stretch_adj;

  assign curPsm_2_nxt[CURPSM_W-1:0] = psmGate ? curPsmOrTest_1[CURPSM_W-1:0] : (~psmGate ? curPsm_2[CURPSM_W-1:0] : {CURPSM_W{1'bx}});

`ifdef CORE_PSM_IP_AVFS_ENABLE

  wire                nullNxtAvg_1;
  wire                nullNxtAvg_2;
  wire                nullNxtAvgChange;
  wire [2:0]          nullNxtAvgCnt;
  wire [2:0]          nullNxtAvgCnt_nxt;
  wire                selCurPsm_2;
  wire                selCurPsm_2_pulse;
  reg                 selCurPsm_2_nxt;

 
  gfx_CORE_FLOP_DGATE_RESET #(.size(CURPSM_W)) CorePsmIpCore_curPsm_2 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(curPsm_2_nxt[CURPSM_W-1:0]), .Q(curPsm_2[CURPSM_W-1:0]));

  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_nullNxtAvg_1 (
  .clkbus(clkbus_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(nullNxtAvg), .Q(nullNxtAvg_1)); 
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_nullNxtAvg_2 (
  .clkbus(clkbus_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(nullNxtAvg_1), .Q(nullNxtAvg_2));

  assign nullNxtAvgChange = nullNxtAvg_1 ^ nullNxtAvg_2;
  
  // 3-bit up counter
  assign nullNxtAvgCnt_nxt[2:0] = nullNxtAvgChange ? nullNxtAvgCnt[2:0] + 3'b1 : nullNxtAvgCnt[2:0];
  
  gfx_CORE_FLOP_DGATE_RESET #(.size(3)) CorePsmIpCore_nullNxtAvgCnt (
  .clkbus(clkbus_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(nullNxtAvgCnt_nxt[2:0]), .Q(nullNxtAvgCnt[2:0]));
  
  // 4:1 mux
  always @* begin
    case (jtag_clkDiv[1:0])
      2'b00: selCurPsm_2_nxt = nullNxtAvgChange;
      2'b01: selCurPsm_2_nxt = nullNxtAvgCnt[0];
      2'b10: selCurPsm_2_nxt = nullNxtAvgCnt[0] & nullNxtAvgCnt[1];
      2'b11: selCurPsm_2_nxt = nullNxtAvgCnt[2] & nullNxtAvgCnt[1] & nullNxtAvgCnt[0];
      // VCS coverage off
      default: selCurPsm_2_nxt = 1'bx;
      // VCS coverage on
    endcase
  end
  
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_selCurPsm_2 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(selCurPsm_2_nxt), .Q(selCurPsm_2));
   
  assign selCurPsm_2_pulse = ~selCurPsm_2 & selCurPsm_2_nxt;

  assign curPsm_2_stretch_adj[CURPSM_W-1:0] = selCurPsm_2_pulse ? {CURPSM_W{1'b0}} : curPsm_2[CURPSM_W-1:0];

`else

//! unused jtag_clkDiv[1:0]
//! unused nullNxtAvg

  gfx_CORE_FLOP_DGATE_RESET #(.size(CURPSM_W)) CorePsmIpCore_curPsm_2 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(curPsm_2_nxt[CURPSM_W-1:0]), .Q(curPsm_2[CURPSM_W-1:0]));

  assign curPsm_2_stretch_adj[CURPSM_W-1:0] = curPsm_2[CURPSM_W-1:0];

`endif

  // These are currently not parameterized
  // Note averaging logic may break if the following parameters are changed:
  // CURPSM_W, AVGPSM_W
  wire [35:0]          curPsm_pad;
  wire [35:0]          opA_sub;
  wire [35:0]          opB_sub;
  wire [35:0]          deltaPsm;
//  wire [34:0]          unsignDivOut;
  wire [13:0]          deltaPsmShort;
  wire                 deltaSign;
  wire [35:0]          divOut;
  wire [CURPSM_W-1:0]  curPsmOrDeltaPsm;
  wire [35:0]          signExtension;

//  wire [35:0]          deltaPsmReg;
  wire [35:0]          avgPsmReg;
  
  wire [35:0]          deltaPsmRegNxt;
  wire [35:0]          avgPsmNxt;
  
  wire  avgRst;
  wire  dblSampEn;

  // PSM Average Calculation
  // Pad PSM output to 36 bits to increase averaging period sign + 14b(PSM) + 2^21 cycles
  // LSB of the PSM must shift over n bits corresponding to max average period in terms of cycles

  assign curPsm_pad[35:0] = { 1'b0, curPsm_2_stretch_adj[CURPSM_W-1:0], {(35-CURPSM_W){1'b0}} };

  // Shared subtractor for the averager and double measure mode test for ODDI
  assign opA_sub[35:0] = curPsm_pad[35:0];
  assign opB_sub[35:0] = avgPsmReg[35:0];
  assign deltaPsm[35:0] = opA_sub[35:0] - opB_sub[35:0];

  assign deltaSign = deltaPsmReg[35];

  assign signExtension[35:0] = {deltaSign, ~({ 35 { deltaSign } } >> avgDiv[JTAG_AVG_DIV_W-1:0]) & {35 { deltaSign } }};

  assign divOut[35:0] = (deltaPsmReg[35:0] >> avgDiv[JTAG_AVG_DIV_W-1:0]) | signExtension[35:0];
  
  assign avgRst = rstPsm | deltaCmp | doubleSmp;
  
  wire [35:0] avgPsmAdd;
  assign     avgPsmAdd[35:0] = avgPsmReg[35:0] + divOut[35:0];

  // Multi-cycle path limited to the output of the adder
  wire [35:0] avgPsmAdd_mcp;
  //CORE_MCBUF #(.size(36), .Cycles(MCP_CYCLES_AVGPSMREG)) I_avgPsmReg_nxt_mcp (.DSTCLK(clk),
  //                                                                            .A(avgPsmAdd[35:0]),
  //                                                                            .Z(avgPsmAdd_mcp[35:0]));
`ifdef AVFS3_MCENBUF_ON
   wire mcbufAvgEn;
   assign mcbufAvgEn = oscEn & avgEn & (~Psm_TrstSyncd) & psmOscGater ;
  gfx_CORE_MCENBUF #(.size(36), .Cycles(MCP_CYCLES_AVGPSMREG)) I_avgPsmReg_nxt_mcp (.DSTCLK(clk),
                                                                              .A(avgPsmAdd[35:0]),
                                                                              .EN(mcbufAvgEn),
                                                                              .Z(avgPsmAdd_mcp[35:0]));
`else
  gfx_CORE_MCBUF #(.size(36), .Cycles(MCP_CYCLES_AVGPSMREG)) I_avgPsmReg_nxt_mcp (.DSTCLK(clk),
                                                                              .A(avgPsmAdd[35:0]),
                                                                              .Z(avgPsmAdd_mcp[35:0]));
`endif

  // assign avgPsmNxt[35:0] = avgRst ? curPsm_pad[35:0] : (~avgRst ? (avgPsmReg[35:0] + divOut[35:0]) : 36'bx);
  assign avgPsmNxt[35:0] = avgRst ? curPsm_pad[35:0] : (~avgRst ? avgPsmAdd_mcp[35:0] : 36'bx);

  //
  // deltaPsm : Pipe-staging deltaPsm
  //

  wire rstPsm_ddiv;

  assign deltaPsmRegNxt[35:0] = psmGate_ddiv1 ? (rstPsm ? 36'b0 : deltaPsm[35:0]) : (~psmGate_ddiv1 ? deltaPsmReg[35:0] : 36'bx);

  gfx_CORE_FLOP_DGATE_RESET #(.size(36)) CorePsmIpCore_deltaPsmReg (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(deltaPsmRegNxt[35:0]), .Q(deltaPsmReg[35:0]));

  //
  // Average Register and deltaCmp mode storage register
  //

  wire [35:0] avgPsmReg_nxt;
  assign avgPsmReg_nxt[35:0] = (psmGate_ddiv1 & avgEn) ? avgPsmNxt[35:0] : (~(psmGate_ddiv1 & avgEn) ? avgPsmReg[35:0] : 36'bx);

  gfx_CORE_FLOP_DGATE_RESET #(.size(36)) CorePsmIpCore_avgPsmReg (
  .clkbus(clkbus_avgEn_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), // mcbufAvgEn risk of adding logic, timing impact
  .D(avgPsmReg_nxt[35:0]), .Q(avgPsmReg[35:0]));

  assign avgPsm[AVGPSM_W-1:0] = avgPsmReg[34:35-AVGPSM_W];

  // Set comparator for tracking max/min of the current PSM or delta from average PSM
  assign deltaPsmShort[CURPSM_W-1:0] = deltaPsmReg[34:21];
  assign curPsmOrDeltaPsm[CURPSM_W-1:0] = deltaCmp ? deltaPsmShort[CURPSM_W-1:0] : (~deltaCmp ? curPsm_2[CURPSM_W-1:0] : {CURPSM_W{1'bx}});
  
//  wire [CURPSM_W-1:0] curPsmOrDeltaPsm_mcp;
//`ifdef CORE_PSM_IP_AVFS_ENABLE
//  // Multi-cycle path added to help with ZAK timing
//  CORE_MCBUF #(.size(CURPSM_W), .Cycles(2)) I_curPsmOrDeltaPsm_mcp (.DSTCLK(clk),
//                                                                              .A(curPsmOrDeltaPsm[CURPSM_W-1:0]),
//                                                                              .Z(curPsmOrDeltaPsm_mcp[CURPSM_W-1:0]));
//`else // DLDO, non-AVFS PSM consumers
//  assign curPsmOrDeltaPsm_mcp = curPsmOrDeltaPsm;
//`endif

  // Shared comparator for high water mark and low water mark as determined by measMaxPsm
  wire                swapCmpOps;
  wire [MINMAX_W-1:0] opA_cmp;
  wire [MINMAX_W-1:0] opB_cmp;
  wire [MINMAX_W-1:0] opA_cmpNxt;
  wire [MINMAX_W-1:0] opB_cmpNxt;
  wire                cmpPsm;
  //wire                cmpPsm_mcp;
  wire                oneShotEnd;
  wire [MINMAX_W-1:0] minMaxReg;
  reg  [MINMAX_W-1:0] minMaxReg_nxt;
  wire                minMaxSignXor;
  wire                minMaxCmpOut;

  assign opA_cmpNxt[MINMAX_W-1:0] = minMaxReg[MINMAX_W-1:0];
  assign opB_cmpNxt[MINMAX_W-1:0] = curPsmOrDeltaPsm[MINMAX_W-1:0];
  
  // Comparison between numbers of different signs (only possible during delta mode)
  assign minMaxSignXor         = deltaCmp & (curPsmOrDeltaPsm[MINMAX_W-1] ^ minMaxReg[MINMAX_W-1]);

  assign swapCmpOps            = (measMaxPsm ^ minMaxSignXor) ;
  assign opA_cmp[MINMAX_W-1:0] = swapCmpOps ? opB_cmpNxt[MINMAX_W-1:0] : (~swapCmpOps ? opA_cmpNxt[MINMAX_W-1:0] : {MINMAX_W{1'bx}});
  assign opB_cmp[MINMAX_W-1:0] = swapCmpOps ? opA_cmpNxt[MINMAX_W-1:0] : (~swapCmpOps ? opB_cmpNxt[MINMAX_W-1:0] : {MINMAX_W{1'bx}});
  assign minMaxCmpOut          = (opA_cmp[MINMAX_W-1:0] > opB_cmp[MINMAX_W-1:0]);

  assign cmpPsm                = minMaxCmpOut;

//`ifdef AVFS_MCBUF_ENABLE
//  // Multi-cycle path added to help with ZAK timing
//  CORE_MCBUF #(.size(1), .Cycles(2)) I_cmpPsm_mcp (.DSTCLK(clkbus_div_clk_gater[`C_PIN]),
//                                                   .A(cmpPsm),
//                                                   .Z(cmpPsm_mcp));
//`else // DLDO, non-AVFS PSM consumers
//  assign cmpPsm_mcp = cmpPsm;
//`endif

  //
  // MaxRange Register update rstPsm during functional mode and cmpPsm
  //

//  assign minMaxReg_nxt[MINMAX_W-1:0] = (rstPsm_ddiv & deltaCmp)                            ? {MINMAX_W{1'b0}} :
//                                       ( (rstPsm_ddiv & ~deltaCmp) |
//                                       (deltaCmp & psmGate_ddiv2 & cmpPsm & oneShotEnd) |
//                                       (~deltaCmp & psmGate_ddiv1 & cmpPsm) )              ? curPsmOrDeltaPsm[MINMAX_W-1:0] :
//                                                                                             minMaxReg[MINMAX_W-1:0];

// This is not a sequential always block. Waiving LEDA B_1419.
// leda B_1419 off
  always @* begin
    casez ({(rstPsm_ddiv & deltaCmp),
            ((rstPsm_ddiv & ~deltaCmp) | (deltaCmp & psmGate_ddiv2 & cmpPsm & oneShotEnd) | (~deltaCmp & psmGate_ddiv1 & cmpPsm))})
      2'b1?: minMaxReg_nxt[MINMAX_W-1:0] = {MINMAX_W{1'b0}};
      2'b01: minMaxReg_nxt[MINMAX_W-1:0] = curPsmOrDeltaPsm[MINMAX_W-1:0];
      2'b00: minMaxReg_nxt[MINMAX_W-1:0] = minMaxReg[MINMAX_W-1:0];
      // VCS coverage off
      default: minMaxReg_nxt[MINMAX_W-1:0] = {MINMAX_W{1'bx}};
      // VCS coverage on
    endcase
  end // always @*
// leda B_1419 on

  gfx_CORE_FLOP_DGATE_RESET #(.size(MINMAX_W)) CorePsmIpCore_minMaxReg (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(minMaxReg_nxt[MINMAX_W-1:0]), .Q(minMaxReg[MINMAX_W-1:0]));

  assign minMaxPsm[MINMAX_W-1:0] = minMaxReg[MINMAX_W-1:0];

  wire [JTAG_THRESHOLD_W-1:0] opA_thsh;
  wire [JTAG_THRESHOLD_W-1:0] opB_thsh;
  wire [JTAG_THRESHOLD_W-1:0] opA_preswap;
  wire [JTAG_THRESHOLD_W-1:0] opB_preswap;
  wire                        bothnegOvershoot; 
  wire                        bothposDroopDelta;
  wire                        suppressAlarm;
  wire                        threshAlarm;
  wire                        threshSignXor;
  wire                        threshCmpOut;

  wire threshAlarmNxt;
  wire threshAlarmReg; // Sticky register to indicate a threshold alarm
  
  wire threshAlarmAdj;

  //
  // Shared threshold storage and double Sample mode register
  //

//  wire [JTAG_THRESHOLD_W-1:0] thresholdReg;
  reg  [JTAG_THRESHOLD_W-1:0] thresholdNxt;
  
//  assign thresholdNxt[JTAG_THRESHOLD_W-1:0] = jtag_calibrate ? curPsmOrDeltaPsm[JTAG_THRESHOLD_W-1:0] :
//                                              threshWriteEn  ? threshold[JTAG_THRESHOLD_W-1:0] :
//                                                               thresholdReg[JTAG_THRESHOLD_W-1:0];

  always @* begin
    casez ({jtag_calibrate, threshWriteEn})
      2'b1?: thresholdNxt[JTAG_THRESHOLD_W-1:0] = curPsmOrDeltaPsm[JTAG_THRESHOLD_W-1:0];
      2'b01: thresholdNxt[JTAG_THRESHOLD_W-1:0] = threshold[JTAG_THRESHOLD_W-1:0];
      2'b00: thresholdNxt[JTAG_THRESHOLD_W-1:0] = thresholdReg[JTAG_THRESHOLD_W-1:0];
      // VCS coverage off
      default: thresholdNxt[JTAG_THRESHOLD_W-1:0] = {JTAG_THRESHOLD_W{1'bx}};
      // VCS coverage on
    endcase
  end // always @*

//  CORE_FLOP_DGATE #(.size(JTAG_THRESHOLD_W)) CorePsmIpCore_thresholdReg (
//  .clkbus(clkbus_discrete_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
//  .D(thresholdNxt[JTAG_THRESHOLD_W-1:0]), .Q(thresholdReg[JTAG_THRESHOLD_W-1:0]));
  gfx_CORE_FLOP_DGATE_RESET #(.size(JTAG_THRESHOLD_W)) CorePsmIpCore_thresholdReg (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(thresholdNxt[JTAG_THRESHOLD_W-1:0]), .Q(thresholdReg[JTAG_THRESHOLD_W-1:0]));

  // JTAG one-shot control wires
  wire [3:0]                       trigRegNxt;
  wire [3:0]                       trigRegNxt_2;
  wire [JTAG_ONE_SHOT_DELAY_W-1:0] oneShotCntNxt;
  wire [JTAG_ONE_SHOT_DELAY_W-1:0] oneShotCntNxt_1;
  wire [JTAG_ONE_SHOT_DELAY_W-1:0] oneShotCntNxtSat;
  wire                             oneShotRegNxt;
  wire                             oneShotRegNxt_1;
  wire                             oneShotEn;
  wire                             trigMode;
  wire                             cntTrig;

  // The alarm SHOULD NOT trigger 
  // 1) if the threshold and deltapsm or of different polarities
  // 2) If both threhold and deltapsm are negative while monitoring for overshoots
  // 3) if both threshold and deltapsm are positive while monitoring for droops
  //    in delta units mode
  assign threshSignXor     = curPsmOrDeltaPsm[JTAG_THRESHOLD_W-1] ^ thresholdReg[JTAG_THRESHOLD_W-1];
  assign bothnegOvershoot  = thresholdReg[JTAG_THRESHOLD_W-1] & curPsmOrDeltaPsm[JTAG_THRESHOLD_W-1] & threshMaxPsm;
  assign bothposDroopDelta = ~thresholdReg[JTAG_THRESHOLD_W-1] & ~curPsmOrDeltaPsm[JTAG_THRESHOLD_W-1] & ~threshMaxPsm & deltaCmp;

  assign suppressAlarm =  (threshSignXor | bothnegOvershoot | bothposDroopDelta) & deltaCmp;

  assign opA_preswap[JTAG_THRESHOLD_W-1:0] = {thresholdReg[JTAG_THRESHOLD_W-1:0]};
  assign opB_preswap[JTAG_THRESHOLD_W-1:0] = {curPsmOrDeltaPsm[JTAG_THRESHOLD_W-1:0]};
  
  // Comparator output is high if opA (threshold preswap) > opB (curpsm or deltapsm preswap)
  // So comparator output is ok if:
  // 1) in absolute mode, when measuring droops
  // 2) in delta mode, when measuring droops (both operands would be sign
  //    extended, so more negative number would be interpreted as lower in
  //    an mapsmrounsigned comparison)
  // comparison should be the other way around if:
  // 1) in absolute mode, when measuring overshoots
  // 2) in delta mode, when measuring overshoots
  // So we should swap operands if we are measuring overshoots
   
  // Note: These conditions will only work if we use the assumptions for
  //       suppressing alarm as outlined above

  assign opA_thsh[JTAG_THRESHOLD_W-1:0] = threshMaxPsm ? opB_preswap[JTAG_THRESHOLD_W-1:0] :
                                                         (~threshMaxPsm ? opA_preswap[JTAG_THRESHOLD_W-1:0] : {JTAG_THRESHOLD_W{1'bx}});
  assign opB_thsh[JTAG_THRESHOLD_W-1:0] = threshMaxPsm ? opA_preswap[JTAG_THRESHOLD_W-1:0] :
                                                         (~threshMaxPsm ? opB_preswap[JTAG_THRESHOLD_W-1:0] : {JTAG_THRESHOLD_W{1'bx}});
  
  assign threshCmpOut = (opA_thsh[JTAG_THRESHOLD_W-1:0] > opB_thsh[JTAG_THRESHOLD_W-1:0]);

  assign threshAlarm = ~suppressAlarm & threshCmpOut ; 


  // If there is cause for alarm (like curpsm < threshold during droop measurement)
  //   - Alarm triggers if we are in continuous mode
  //   - Alarm tiggers if we are in dicrete mode and the one shot measurement
  //     has been made and we were not performing calibration  
  assign threshAlarmAdj = (threshAlarm & ~jtag_discrete) | (threshAlarm & jtag_discrete & oneShotEnd & ~jtag_calibrate);

  // remove stickiness when in discrete mode
  assign threshAlarmNxt = ~rstPsm & (((threshAlarmAdj | threshAlarmReg) & ~jtag_discrete) | (threshAlarmAdj & jtag_discrete)) ;

  //
  // Threshold Alarm Register
  // Sticky bit for non-discrete modes
  //

  wire   alarmGater;
  assign alarmGater = (~deltaCmp &  psmGate_ddiv1) | (deltaCmp & psmGate_ddiv2) | rstPsm | Psm_TrstSyncd;

  // Alarm should sampled if either
  //  1) RstPsm was asserted
  //  2) if in delta mode, 2 intervals after the psm was sampled
  //  3) if not in delta mode, 1 interval after psm was sampled 

  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_threshAlarmReg (.clkbus(clkbus_psmRun[`CLKBUS_SIZE-1:0]), .DG(alarmGater), .D(threshAlarmNxt), .Q(threshAlarmReg));

  assign alarm = threshAlarmReg;

  assign curPsm[CURPSM_W-1:0] = curPsm_2[CURPSM_W-1:0];

  //
  // JTAG one-shot logic
  //

  assign trigMode = (psmEn_1 | jtag_one_shot) & jtag_discrete & ~rstPsm;

  assign oneShotEnd = (~|oneShotCnt[JTAG_ONE_SHOT_DELAY_W-1:0]) & (trigReg[2]);
  assign oneShotEn  = (~oneShotReg & oneShotEnd) & trigMode;
  assign cntTrig    = trigMode & trigReg[1];
  
  wire dblsmpstart_ddiv0;   // _ddiv0 suffix denotes pipe stage to match phase of data of curPsm_1
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_dblsmpstart_ddiv0 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(dblsmpstart), .Q(dblsmpstart_ddiv0));

  assign dblSampEn  = dblsmpstart_ddiv0 & doubleSmp;
       
  // 8 bit down counter and saturate at 0, reset when jtag_one_shot =
  assign oneShotCntNxtSat[JTAG_ONE_SHOT_DELAY_W-1:0] = (oneShotCnt[JTAG_ONE_SHOT_DELAY_W-1:0] - {{JTAG_ONE_SHOT_DELAY_W-1{1'b0}} , 1'b1});
  
  assign oneShotCntNxt   [JTAG_ONE_SHOT_DELAY_W-1:0] = trigReg[2] ? oneShotCntNxtSat[JTAG_ONE_SHOT_DELAY_W-1:0] : 
                                                     (~trigReg[2] ? jtag_one_shot_count[JTAG_ONE_SHOT_DELAY_W-1:0] : {JTAG_ONE_SHOT_DELAY_W{1'bx}});
  
  assign oneShotCntNxt_1 [JTAG_ONE_SHOT_DELAY_W-1:0] = (cntTrig & ~rstPsm & ~oneShotEnd) ? oneShotCntNxt[JTAG_ONE_SHOT_DELAY_W-1:0] :
                                                     (~(cntTrig & ~rstPsm & ~oneShotEnd) ? {JTAG_ONE_SHOT_DELAY_W{1'b0}} : {JTAG_ONE_SHOT_DELAY_W{1'bx}});

  // 8 bit counter registers
  gfx_CORE_FLOP_DGATE_RESET #(.size(JTAG_ONE_SHOT_DELAY_W)) CorePsmIpCore_oneShotCnt (
  .clkbus(clkbus_discrete_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(oneShotCntNxt_1[JTAG_ONE_SHOT_DELAY_W-1:0]), .Q(oneShotCnt[JTAG_ONE_SHOT_DELAY_W-1:0]));

  // one-shot control
  assign oneShotRegNxt   = psmOscGater ? oneShotEnd : (~psmOscGater ? oneShotReg : 1'bx);
//  assign oneShotRegNxt_1 = (~rstPsm & cntTrig) ? oneShotRegNxt : 1'b0;
  assign oneShotRegNxt_1 = (~rstPsm & cntTrig) & oneShotRegNxt;

  // one-shot control registers
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_oneShotReg (
  .clkbus(clkbus_discrete_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(oneShotRegNxt_1), .Q(oneShotReg));

  // one-shot control generation
  assign trigRegNxt[3:0]   = {trigReg[2], cntTrig, trigReg[0], trigMode};
//  assign trigRegNxt_2[3:0] = (~rstPsm & trigMode) ? trigRegNxt[3:0] : 4'b0000;
  assign trigRegNxt_2[3:0] = {4{(~rstPsm & trigMode)}} & trigRegNxt[3:0];

  gfx_CORE_FLOP_DGATE_RESET #(.size(4)) CorePsmIpCore_trigReg (
  .clkbus(clkbus_discrete_psmRun[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(trigRegNxt_2[3:0]), .Q(trigReg[3:0]));

  // when oneShot = 8'b0 capture psmOsc value

// dblSampEn is gear slip aligned to fire as the oneShot counter starts its down count. 
// dblSampEn_ddiv fires at the end of the double sample interval which is the 1st measurement interval within the down count period.
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_dblSampEn_ddiv (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(dblSampEn), .Q(dblSampEn_ddiv));

// oneShotEn is gear slip aligned to fire as the oneShot counter ends its down count. 
// oneShotEn_ddiv0 fires at the end of the measurement that starts at the end of the down count period.
// oneShotEn_ddiv1 fires at the end of the measurement that starts when oneShotEn_ddiv0 fires.

  wire oneShotEn_ddiv0;   // _ddiv0 suffix denotes pipe stage to match phase of data of curPsm_1
  wire oneShotEn_ddiv1;   // _ddiv1 suffix denotes pipe stage to match phase of data of curPsm_2

  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_oneShotEn_ddiv0 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(oneShotEn), .Q(oneShotEn_ddiv0));
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_oneShotEn_ddiv1 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(oneShotEn_ddiv0), .Q(oneShotEn_ddiv1));

// _ddiv suffix denotes delay until next div_clk_gater gated clock
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_rstPsm_ddiv (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(rstPsm), .Q(rstPsm_ddiv));

  assign psmGate = ~jtag_discrete | dblSampEn_ddiv | oneShotEn_ddiv1;
  
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_psmGate_ddiv1 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(psmGate),       .Q(psmGate_ddiv1));
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) CorePsmIpCore_psmGate_ddiv2 (
  .clkbus(clkbus_div_clk_gater[`CLKBUS_SIZE-1:0]), .DG(1'b1), 
  .D(psmGate_ddiv1), .Q(psmGate_ddiv2));

endmodule // CorePsmIpCore


// ------------------------------------------------------
// Main PSM without JTAG. All registers are brought out as inputs/outputs. Additional registers (trigReg, oneShotCnt) brought out for debug bus observability.
// ------------------------------------------------------
module gfx_CORE_PSM_IP_NO_JTAG #(parameter
  //
  PSM_PARAM_LENGTH      = 64,
  PSM_DATA_LENGTH       = 29,
  PSM_CURPSM_LENGTH     = 14,

  // JTAG inputs to PSM core (total length = PSM_PARAM_LENGTH)
  //
  //   These 4 parameters can be configured to be set by the WDR or the PsmParam register
  //      this was done to reduce BW requirements on the SPMI/P1500 serial port
  //
  //   JTAG_WDR_OVERRIDE   1'b0        1'b1
  //                       ---------   ----------
  //                       SPMI/P1500      M&M
  //   PsmEnable:          WDR[64]     PsmParam[24]
  //   RstPsm   :          WDR[65]     PsmParam[0]
  //   OneShot  :          WDR[66]     PsmParam[5]
  //   AvgEn    :          WDR[67]     PsmParam[56]
  //
  JTAG_UNUSED_H         = 63,    // UNUSED_END
  JTAG_UNUSED_L         = 59,    // UNUSED_START
  JTAG_MASK_SKIP_PHASE  = 58,    // If set allow skipPhase to go through, default is 0 to mask skipPhase -> nullNxtAvg
  JTAG_WDR_OVERRIDE     = 57,    // If set JTAG_AVGEN, JTAG_PSM_ENABLE, JTAG_PSM_RST, JTAG_ONE_SHOT override the WDR[67:64] values
  JTAG_AVGEN            = 56,    // same functionality as WDR[67]
  JTAG_PSM_BYPASS_H     = 55,
  JTAG_PSM_BYPASS_L     = 42,
  JTAG_TEST             = 41,
  JTAG_ALARMFLAGEN      = 40,
  JTAG_THRESHWRITEEN    = 39,
  JTAG_THRESHOLD_H      = 38,
  JTAG_THRESHOLD_L      = 25,
  JTAG_PSM_ENABLE       = 24,    // same functionality as WDR[64]
//   JTAG_PSM_DBG_EN      = 24,  // replaced by WDR[74]
  JTAG_ONE_SHOT_DELAY_H = 23,
  JTAG_ONE_SHOT_DELAY_L = 16,
  JTAG_AVG_DIV_H        = 15,
  JTAG_AVG_DIV_L        = 10,
  JTAG_CLK_DIV_H        = 9,
  JTAG_CLK_DIV_L        = 8,
  JTAG_MEAS_MAXPSM      = 7,
  JTAG_THRESH_MAXPSM    = 6,
  JTAG_ONE_SHOT         = 5,     // same functionality as WDR[66]
  JTAG_DELTA_CMP        = 4,
  JTAG_CALIBRATE        = 3,
  JTAG_DOUBLE_SMP       = 2,
  JTAG_DISCRETE         = 1,
  JTAG_PSM_RST          = 0,     // same functionality as WDR[65]
 
  // Output from PSM core (total length = PSM_DATA_LENGTH)
  // Excludes CurPsm, since it can also function as JTAG bypass
  AVGPSM_H              = 28,
  AVGPSM_L              = 15,
  MINMAX_H              = 14,
  MINMAX_L              = 1,
  ALARM                 = 0,
  
  // MCP settings for the PSM core
  MCP_SETUP             = 1100,
  MCP_CYCLES_MAPSMRO    = 0, // PSMRO output MCP(2) is now moved to curPsm_1 inside CorePsmIpCore, default is 0 for DLDO purposes. AVFS3_TOP and AVFS_PSM_WRAP, and CORE_PSM_IP_LS modules override this as "2"
  MCP_CYCLES_AVGPSMREG  = 3
)(
  input                             ScanShiftEn,
  input                             Psm_Rst,
  input                             Psm_ScanMode,
  input                             Psm_Gater,
  input                             JTAG_Psm_Enable,
  input                             JTAG_Psm_AvgEn,
  input                             JTAG_Psm_Rst,
  input                             JTAG_Psm_OneShot,
  input                             PsmEn,
  input                             Host_Clk,
  input                             HostReg_Clk,
  
`ifdef CORE_PSM_IP_AVFS_ENABLE

  input                             nullNxtAvg,
  output wire                       div_clk_gater,
`endif
`ifdef GFX_DROOP_DETECT
  output wire [PSM_CURPSM_LENGTH-1:0] psmCountOut, // adding new port to direct PSM count out through AVFS3 for GNL purposes
`endif
  output wire [PSM_CURPSM_LENGTH-1:0] deltaPsmRegOut,
  output wire [PSM_CURPSM_LENGTH-1:0] dbgCurPsm,
  output wire [PSM_CURPSM_LENGTH-1:0] dbgAvgPsm,
  output wire [PSM_CURPSM_LENGTH-1:0] dbgThreshold,
  output wire [PSM_CURPSM_LENGTH-1:0] dbgMinMax,
  output wire                         dbgAlarm,

  output reg                        Alarm_Flag,

  input [63:0]                      WrDat, // register write bus
  output wire [63:0]                RdDat, // register read bus
  output wire                       MM_Psm_Enable,
  input                             SelPsmParam,
  input                             SelPsmData,
  input                             WrEn,
  
  input                             psmOscGater,        //mapsmro
  input [31:0]                      psmOscLsb,          //mapsmro 
  input [7:0]                       psmOscMsb0,         //mapsmro 
  input [7:0]                       psmOscMsb1,         //mapsmro 
  output [1:0]                      clkDivider,         //mapsmro
  output                            oscEn               //mapsmro

);

  wire [  PSM_DATA_LENGTH-1:0]  PsmDataReg;
  wire [ PSM_PARAM_LENGTH-1:0]  PsmParamReg;
  wire [PSM_CURPSM_LENGTH-1:0]  CurPsm;
  wire [PSM_CURPSM_LENGTH-1:0]  CurPsmBypass;
  wire [PSM_CURPSM_LENGTH-1:0]  JtgCurPsmBypass;
//  wire                          psmOscGater;
  wire [PSM_CURPSM_LENGTH-1:0]  curPsm_1;

  // Additional PSM outputs for debug-bus observability
  wire [13:0] thresholdReg;
  wire [35:0] deltaPsmReg;
  wire        dblSampEn_ddiv;
  wire        PsmCore_Psm_Enable;

  always @* begin
    casez ({ ( PsmCore_Psm_Enable & ~Psm_ScanMode ), PsmParamReg[JTAG_ALARMFLAGEN]}) // Per spec in scan mode, Alaram_flag is forced to 0, adding "& ~Psm_ScanMode"
      2'b0?: Alarm_Flag = 1'b0;
      2'b10: Alarm_Flag = PsmParamReg[JTAG_THRESH_MAXPSM];
      2'b11: Alarm_Flag = PsmDataReg[ALARM];
      // VCS coverage off
      default: Alarm_Flag = 1'bx;
      // VCS coverage on
    endcase
  end // always @*
  
  // Delta generation logic for CurPsmBypass
  // Flip bit 6 when dblSampEn_ddiv==1, to generate a delta
  assign CurPsmBypass[13:7] = JtgCurPsmBypass[13:7];
  assign CurPsmBypass[5:0]  = JtgCurPsmBypass[5:0];
  assign CurPsmBypass[6]    = dblSampEn_ddiv ^ JtgCurPsmBypass[6];
  
`ifdef CORE_PSM_IP_AVFS_ENABLE
  wire   div_clk_gater_int;

  // Psm_ScanMode is used to isolate the PSM outputs when external logic,
  // including the AVFS_CPA, is in scan-test. There seems to be no need
  // to force div_clk_gater=1, which is equivalent to div-by-1 mode.
  assign div_clk_gater = div_clk_gater_int | Psm_ScanMode;

  // Force deltaPsmRegOut to known state because it is an input to the
  // AVFS_CPA which can be in scan-test.  
  assign deltaPsmRegOut[13:0] = deltaPsmReg[34:21] | {14{Psm_ScanMode}};
//! unused deltaPsmReg[35]
//! unused deltaPsmReg[20:0]
`else
  assign deltaPsmRegOut[13:0] = 14'b0;
//! unused deltaPsmReg[35:0]
`endif

  wire PsmCore_Psm_Rst;
  wire PsmCore_Psm_OneShot;
  wire PsmCore_Psm_AvgEn;

  // The following 4 control bits will be set either from the WDR for
  // SOCs which support SPMI, or via the PsmParam register for SOCs
  // that support M&M. These 2 versions are selected by PsmParamReg[JTAG_WDR_OVERRIDE]

  assign MM_Psm_Enable       =                                 PsmParamReg[JTAG_PSM_ENABLE];

  assign PsmCore_Psm_Enable  = PsmParamReg[JTAG_WDR_OVERRIDE]? PsmParamReg[JTAG_PSM_ENABLE] : JTAG_Psm_Enable;
  assign PsmCore_Psm_Rst     = PsmParamReg[JTAG_WDR_OVERRIDE]? PsmParamReg[JTAG_PSM_RST] : JTAG_Psm_Rst;
  assign PsmCore_Psm_OneShot = PsmParamReg[JTAG_WDR_OVERRIDE]? PsmParamReg[JTAG_ONE_SHOT] : JTAG_Psm_OneShot;
  assign PsmCore_Psm_AvgEn   = PsmParamReg[JTAG_WDR_OVERRIDE]? PsmParamReg[JTAG_AVGEN] : JTAG_Psm_AvgEn;

   assign clkDivider    = PsmParamReg[JTAG_CLK_DIV_H:JTAG_CLK_DIV_L];


  // PSM Core
  gfx_CorePsmIpCore #(
  .JTAG_ONE_SHOT_DELAY_W(JTAG_ONE_SHOT_DELAY_H-JTAG_ONE_SHOT_DELAY_L+1),
  .JTAG_THRESHOLD_W     (     JTAG_THRESHOLD_H-     JTAG_THRESHOLD_L+1),
  .JTAG_CLK_DIV_W       (       JTAG_CLK_DIV_H-       JTAG_CLK_DIV_L+1),
  .JTAG_AVG_DIV_W       (       JTAG_AVG_DIV_H-       JTAG_AVG_DIV_L+1),
  .AVGPSM_W             (             AVGPSM_H-             AVGPSM_L+1),
  .CURPSM_W             (                            PSM_CURPSM_LENGTH),
  .MINMAX_W             (             MINMAX_H-             MINMAX_L+1),
  .MCP_SETUP            (                                    MCP_SETUP),
  .MCP_CYCLES_MAPSMRO   (                           MCP_CYCLES_MAPSMRO),
  .MCP_CYCLES_AVGPSMREG (                         MCP_CYCLES_AVGPSMREG)
  ) I_PsmCore (

  // Inputs to PSM core
  .ScanShiftEn         (ScanShiftEn),
  .clk                 (Host_Clk),
  .Psm_ScanMode        (Psm_ScanMode),
  .Psm_TrstSyncd       (Psm_Rst),
  .psmEn               (PsmEn),
  .psmGater            (Psm_Gater),
  .psmOscGater         (psmOscGater),
  .curPsm_1            (curPsm_1   [                      PSM_CURPSM_LENGTH-1:0]),
  .jtag_curPsm         (CurPsmBypass[                     PSM_CURPSM_LENGTH-1:0]),
  .jtag_test           (PsmParamReg[                                  JTAG_TEST]),
  .jtag_rstPsm         (PsmCore_Psm_Rst),
  .jtag_psmEnable      (PsmCore_Psm_Enable),
  .jtag_discrete       (PsmParamReg[                              JTAG_DISCRETE]),
  .jtag_one_shot       (PsmCore_Psm_OneShot),
  .jtag_one_shot_count (PsmParamReg[JTAG_ONE_SHOT_DELAY_H:JTAG_ONE_SHOT_DELAY_L]),
  .jtag_threshValue    (PsmParamReg[     JTAG_THRESHOLD_H:     JTAG_THRESHOLD_L]),
  .jtag_threshMaxPsm   (PsmParamReg[                         JTAG_THRESH_MAXPSM]),
  .jtag_measMaxPsm     (PsmParamReg[                           JTAG_MEAS_MAXPSM]),
  .jtag_deltaCmp       (PsmParamReg[                             JTAG_DELTA_CMP]),
  .jtag_doubleSmp      (PsmParamReg[                            JTAG_DOUBLE_SMP]),
  .jtag_avgDiv         (PsmParamReg[       JTAG_AVG_DIV_H:       JTAG_AVG_DIV_L]),
  .jtag_avgEn          (PsmCore_Psm_AvgEn),
  .jtag_calibrate      (PsmParamReg[                             JTAG_CALIBRATE]),
  .jtag_threshWriteEn  (PsmParamReg[                         JTAG_THRESHWRITEEN]),
  .jtag_clkDiv         (PsmParamReg[       JTAG_CLK_DIV_H:       JTAG_CLK_DIV_L]),

  // Outputs from PSM core
  .curPsm              (CurPsm    [PSM_CURPSM_LENGTH-1:0]),
  .minMaxPsm           (PsmDataReg[MINMAX_H:    MINMAX_L]),
  .alarm               (PsmDataReg[                ALARM]),
  .avgPsm              (PsmDataReg[AVGPSM_H:    AVGPSM_L]),
  .oscEn               (oscEn),
  // Additional PSM outputs for debug-bus observability
  .thresholdReg (thresholdReg[13:0]),
  .deltaPsmReg  (deltaPsmReg [35:0]),
  .oneShotCnt   (),
  .trigReg      (),
  .oneShotReg   (),
  
  // Needed for delta generation (i.e. used to flip jtag_curPsm[6])
  .dblSampEn_ddiv  (dblSampEn_ddiv),
  
`ifdef CORE_PSM_IP_AVFS_ENABLE
  .nullNxtAvg      (nullNxtAvg&PsmParamReg[JTAG_MASK_SKIP_PHASE]), // PsmParamReg[JTAG_MASK_SKIP_PHASE] and with nullNxtAvg to mask skipPhase effect
  .div_clk_gater   (div_clk_gater_int)
`else
  .nullNxtAvg      (1'b0),
  .div_clk_gater   ()
`endif
  );

`ifdef GFX_DROOP_DETECT
  assign psmCountOut[PSM_CURPSM_LENGTH-1:0]= PsmCore_Psm_AvgEn ? PsmDataReg[AVGPSM_H:AVGPSM_L] : CurPsm[PSM_CURPSM_LENGTH-1:0] ;
`endif

  // Move the MCBUF here on mapsmro outputs coming into "encoder" logic
  //wire [31:0] psmOscLsb_mcp;
  //wire [7:0] psmOscMsb0_mcp;
  //wire [7:0] psmOscMsb1_mcp;

  //wire mcbufEn;
  //assign mcbufEn = oscEn & (~Psm_Rst) & psmOscGater ; 
  //CORE_MCENBUF #(.size(32), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscLsb_mcp (.A(psmOscLsb[31:0]), 
  //                                                                        .EN(mcbufEn),
  //                                                                        .DSTCLK(Host_Clk),
  //                                                                        .Z(psmOscLsb_mcp[31:0]));
  //CORE_MCENBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb0_mcp (.A(psmOscMsb0[7:0]), 
  //                                                                        .EN(mcbufEn),
  //                                                                        .DSTCLK(Host_Clk),
  //                                                                        .Z(psmOscMsb0_mcp[7:0]));
  //CORE_MCENBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb1_mcp (.A(psmOscMsb1[7:0]), 
  //                                                                        .EN(mcbufEn),
  //                                                                        .DSTCLK(Host_Clk),
  //                                                                        .Z(psmOscMsb1_mcp[7:0]));

  //
  // Instatiate the psmOscIO (priority encoder and mux)
  //
  gfx_CorePsmIpOscIO I_CorePsmIpOscIO(
    //.psmOscLsb  (psmOscLsb_mcp[31:0]),
    //.psmOscMsb0 (psmOscMsb0_mcp[7:0]),
    //.psmOscMsb1 (psmOscMsb1_mcp[7:0]),
    .psmOscLsb  (psmOscLsb[31:0]),
    .psmOscMsb0 (psmOscMsb0[7:0]),
    .psmOscMsb1 (psmOscMsb1[7:0]),
    .psmLsb     (curPsm_1[5:0]),
    .psmMsb     (curPsm_1[13:6])
  );


  // PSM Register Interface

  // PsmParamReg is reset on TRST or external Reset via Psm_Rst, but not on jtag_Psm_Rst.
  // jtag_Psm_Rst causes curPsm -> AvgPsm and 0 -> DeltaPsm
  wire PsmParam_Gater;
  assign PsmParam_Gater = Psm_Gater & WrEn & SelPsmParam | Psm_Rst; 
  gfx_CORE_FLOP_GATED_RESET #(.size(PSM_PARAM_LENGTH),.rv(64'h0000000000000300)) I_PsmParamReg (.C(HostReg_Clk), .D(WrDat[PSM_PARAM_LENGTH-1:0]), .G(PsmParam_Gater), .GC(1'b0), .R(Psm_Rst), .SSE(ScanShiftEn), .Q(PsmParamReg[PSM_PARAM_LENGTH-1:0])); // TODO:  HostReg_Clk also connects to scannable flops;  is this a problem for ATPG?

  // Return zeroes for reserved bits in bits 63:58 of PsmParam
  //  assign RdDat[63:0] = ({64{SelPsmParam}} & {{(64-PSM_PARAM_LENGTH) {1'b0}},PsmParamReg[PSM_PARAM_LENGTH-1:0]}) |
  //                       ({64{SelPsmData}} & {{(64-PSM_DATA_LENGTH) {1'b0}},PsmDataReg[PSM_DATA_LENGTH-1:0]});  
    assign RdDat[63:0] = ({64{SelPsmParam}} & PsmParamReg[PSM_PARAM_LENGTH-1:0]) |  // RRR: Removed {(64-PSM_PARAM_LENGTH) {1'b0} to address LEDA violation - min length of concatenation expression should be at least one 
                         ({64{SelPsmData}} & {{(64-PSM_DATA_LENGTH){1'b0}},PsmDataReg[PSM_DATA_LENGTH-1:0]}); 
  assign JtgCurPsmBypass[PSM_CURPSM_LENGTH-1:0] = PsmParamReg[JTAG_PSM_BYPASS_H:JTAG_PSM_BYPASS_L];

  //drive debug signals
  assign dbgCurPsm[PSM_CURPSM_LENGTH-1:0] = CurPsm[PSM_CURPSM_LENGTH-1:0];
  assign dbgAvgPsm[PSM_CURPSM_LENGTH-1:0] = PsmDataReg[AVGPSM_H:AVGPSM_L];
  assign dbgThreshold[PSM_CURPSM_LENGTH-1:0] = thresholdReg[13:0];
  assign dbgMinMax[PSM_CURPSM_LENGTH-1:0] = PsmDataReg[MINMAX_H:MINMAX_L];
  assign dbgAlarm = PsmDataReg[ALARM];
   
//! unused WrDat[63:57]

endmodule // PSM_IP


// ------------------------------------------------------
// Main PSM with JTAG/1500 and level shifters.
// ------------------------------------------------------
module gfx_CORE_PSM_IP_LS #(parameter
  HIGH_VOLT = 0, 
  SYNC_TC_CYCLES    = 3, 
  CDC_TST_PROP_DLY  = 5000,
  DBG_LENGTH        = 16,
  DBG_CFG_LENGTH    = 3,
  MCP_CYCLES_AVGPSMREG=3,
  MCP_CYCLES_MAPSMRO=2,
  PSM_CURPSM_LENGTH = 14,
  NEG_JTAG_TDO      = 0    // Enables TDO/WRO on negedge of JTAG clock
) (
  input                        PsmRoInIso,     // PwrOkVDD2: VDD2 domain PwrOk
  input                        PsmRoOutIso,    // PwrOkVDD1: VDD1 domain PwrOk
  input                        ScanShiftEn,    // Scan shift Enable
//input                        PsmAvfs_Gater,
  input                        Avfs_ScanMode,
//input                        Reset,
  input                        Isolate,          // indicates tile is being power gated
//input                        SkipPhase,        // indicates Host_Clk is being stretched
  input                        CCLK,             // ZAK uses mesh clock
  input                        TstCcgEn,         // ZAK test coarse gater enable (non-scannable flops would use 1'b1)
//input                        RCLK_Free_AR,     // ZAK uses free-running clock for coarse gater flops
                               
  // JTAG interface            
  input                        WRCK,         // p1500
  input                        WSI,          // p1500
  input                        ShiftWR,      // p1500
  input                        UpdateWR,     // p1500
  input                        CaptureWR,    // p1500
  input                        WDREn,        // p1500
  input                        WRSTN,        // p1500
  input                        MeasureWR,    // p1500
  output wire                  WRO,          // p1500
                               
  input                        PsmEn,
  output wire                  Psm_Alarm_Flag,
  input [DBG_CFG_LENGTH-1:0]   DbgBusSel,
  output wire [DBG_LENGTH-1:0] DbgBus
);

wire [31:0] psmOscLsb_VDD2;
wire [7:0]  psmOscMsb0_VDD2;
wire [7:0]  psmOscMsb1_VDD2;
wire [1:0]  div_VDD2;

  gfx_CORE_PSM_IP_JTAG #(.SYNC_TC_CYCLES(SYNC_TC_CYCLES),.CDC_TST_PROP_DLY(CDC_TST_PROP_DLY),.DBG_LENGTH(DBG_LENGTH),.DBG_CFG_LENGTH(DBG_CFG_LENGTH),.MCP_CYCLES_AVGPSMREG(MCP_CYCLES_AVGPSMREG),.MCP_CYCLES_MAPSMRO(MCP_CYCLES_MAPSMRO),.PSM_CURPSM_LENGTH(PSM_CURPSM_LENGTH),.NEG_JTAG_TDO(NEG_JTAG_TDO)) I_CORE_PSM_IP_JTAG (
   .PsmRoOutIso         (PsmRoOutIso       ),
   .CCLK                (CCLK              ),
   .ScanShiftEn         (ScanShiftEn       ),
   .TstCcgEn            (TstCcgEn          ),
   .Avfs_ScanMode       (Avfs_ScanMode     ),
   .Isolate             (Isolate           ),

   // IEEE1500 interface
   .WRCK                (WRCK              ),
   .WRSTN               (WRSTN             ),
   .ShiftWR             (ShiftWR           ),
   .UpdateWR            (UpdateWR          ),
   .CaptureWR           (CaptureWR         ),
   .WDREn               (WDREn             ),
   .MeasureWR           (MeasureWR         ),
   .WSI                 (WSI               ),
   .WRO                 (WRO               ),

// mapsmro interface
   .psmOscLsb_VDD2      (psmOscLsb_VDD2[31:0]),
   .psmOscMsb0_VDD2     (psmOscMsb0_VDD2[7:0]),
   .psmOscMsb1_VDD2     (psmOscMsb1_VDD2[7:0]),
   .psmOscGater_VDD2    (psmOscGater_VDD2  ),
   .oscEn_VDD1          (oscEn_VDD1        ),
   .clk_VDD2            (clk_VDD2          ),
   .div_VDD2            (div_VDD2[1:0]     ),

   // Debug interface                       
   .PsmEn               (PsmEn             ),
   .Psm_Alarm_Flag      (Psm_Alarm_Flag    ),
   .DbgBusSel           (DbgBusSel[DBG_CFG_LENGTH-1:0]),
   .DbgBus              (DbgBus[DBG_LENGTH-1:0])
);


// RRR (09/12/2015) : Note that all of the interface signals should now be VDD1 type except
// PsmRoInIso (VDD2). At the time of creation of this module there were level shifters
// at this level which needed all this redirection/renaming of signals denoting
// their voltage domain. But now as the LS cells are pushed inside the maspmro
// macro these are just redirections, not really a voltage level change

//--------------------------------------------------------------------
// PSM Oscillator Macro Instantiation (VDMM domain)
//--------------------------------------------------------------------
  gfx_CORE_PSM_IP_OSC #(
  .HIGH_VOLT (HIGH_VOLT),
  .JTAG_CLK_DIV_W (2),
  .MCP_CYCLES_MAPSMRO(MCP_CYCLES_MAPSMRO),
  .CURPSM_W       (14)
) I_CORE_PSM_IP_OSC (
  .clk                       (clk_VDD2                      ),
  .div                       (div_VDD2[1:0]                 ),
  .pwrOk                     (PsmRoInIso                    ), 
  .oscEn                     (oscEn_VDD1 & PsmRoOutIso      ), // FIX to http://bugs.amd.com/ubts/prod/issue/view?id=642993
  .psmOscLsb                 (psmOscLsb_VDD2[31:0]          ),
  .psmOscMsb0                (psmOscMsb0_VDD2[7:0]          ),
  .psmOscMsb1                (psmOscMsb1_VDD2[7:0]          ),
  .psmOscGater               (psmOscGater_VDD2              )
);

endmodule // CORE_PSM_IP_LS


// ------------------------------------------------------
// Main PSM with JTAG/1500 and level shifters.
// ------------------------------------------------------
module gfx_CORE_PSM_IP_JTAG #(parameter
  SYNC_TC_CYCLES    = 3, 
  CDC_TST_PROP_DLY  = 5000,
  DBG_LENGTH        = 16,
  DBG_CFG_LENGTH    = 3,
  MCP_CYCLES_AVGPSMREG=3,
  MCP_CYCLES_MAPSMRO=2,
  PSM_CURPSM_LENGTH = 14,
  NEG_JTAG_TDO      = 0    // Enables TDO/WRO on negedge of JTAG clock
) (
//  input                        PsmRoInIso,     // PwrOkVDD2: VDD2 domain PwrOk
  input                        PsmRoOutIso,    // PwrOkVDD1: VDD1 domain PwrOk
  input                        ScanShiftEn,    // Scan shift Enable
//input                        PsmAvfs_Gater,
  input                        Avfs_ScanMode,
//input                        Reset,
  input                        Isolate,          // indicates tile is being power gated
//input                        SkipPhase,        // indicates Host_Clk is being stretched
  input                        CCLK,             // ZAK uses mesh clock
  input                        TstCcgEn,         // ZAK test coarse gater enable (non-scannable flops would use 1'b1)
//input                        RCLK_Free_AR,     // ZAK uses free-running clock for coarse gater flops
                               
  // JTAG interface            
  input                        WRCK,         // p1500
  input                        WSI,          // p1500
  input                        ShiftWR,      // p1500
  input                        UpdateWR,     // p1500
  input                        CaptureWR,    // p1500
  input                        WDREn,        // p1500
  input                        WRSTN,        // p1500
  input                        MeasureWR,    // p1500
  output wire                  WRO,          // p1500
                               
  input                        PsmEn,
  output wire                  Psm_Alarm_Flag,

  // mapsmro interface
  input [31:0]                 psmOscLsb_VDD2,    // input from CORE_PSM_IP_OSC
  input [7:0]                  psmOscMsb0_VDD2,   // input from CORE_PSM_IP_OSC
  input [7:0]                  psmOscMsb1_VDD2,   // input from CORE_PSM_IP_OSC
  input                        psmOscGater_VDD2,  // input from CORE_PSM_IP_OSC
  output wire                  oscEn_VDD1,        // output to CORE_PSM_IP_OSC
  output wire                  clk_VDD2,          // output to CORE_PSM_IP_OSC
  output wire [1:0]            div_VDD2,          // output to CORE_PSM_IP_OSC

  input [DBG_CFG_LENGTH-1:0]   DbgBusSel,
  output wire [DBG_LENGTH-1:0] DbgBus
);

  wire                         RCLK_Free_AR;
  wire                         KCLK_Free_AR;
  wire                         TrstSyncd;
  wire                         JTAG_ClkEn_ASYNC;
  wire                         JTAG_ClkEn_CCLK;
  wire                         JTAG_Psm_Enable;
  wire                         JTAG_Psm_AvgEn;
  wire                         JTAG_Psm_Rst;
  wire                         JTAG_Psm_Rst_Reset;
  wire                         JTAG_Psm_OneShot;

  wire                         Avfs_ScanMode_CDC;
  reg  [DBG_LENGTH-1:0]        DbgBusNxt;
  reg  [DBG_LENGTH-1:0]        DbgBus_int;
  wire                         dbgEn_ScanMode;
  wire                         dbgEn;
  wire [PSM_CURPSM_LENGTH-1:0] dbgCurPsm;
  wire [PSM_CURPSM_LENGTH-1:0] dbgAvgPsm;
  wire [PSM_CURPSM_LENGTH-1:0] dbgThreshold;
  wire [PSM_CURPSM_LENGTH-1:0] dbgMinMax;
  wire                         dbgAlarm;
  wire [49:0]                  AvfsNearMiss;

  wire                         RegisterReadEnLo;
  wire                         RegisterReadEnHi;
  wire                         RegisterRWLo;
  wire                         RegisterRWHi;
  wire                         RegisterWriteEn;
  wire [3:0]                   RegisterAddress;
  wire                         GlobalSelPsm;
  wire                         SelPsmParam;
  wire                         SelPsmData;

  wire                         HostJtag_ClkEn;
  wire                         HostReg_Clk;
  wire                         Host_Clk;
  wire                         Host_CDC_Clk;
  wire                         Psm_ClkEn;
  wire                         PsmClk;
  wire                         Psm_Rst;
  wire [13:0]                  PsmDeltaOut;
  wire [PSM_CURPSM_LENGTH-1:0] PsmOut;

  wire                         clk_VDD1;            // VDD1 domain clock (CCLK)
  wire [1:0]                   div_VDD1;            // Output from CORE_PSM_IP_NO_JTAG
  wire                         oscEn_VDD2;          // Input to CORE_PSM_IP_OSC

  wire [31:0]                  psmOscLsb_VDD1;      // Input to CORE_PSM_IP_NO_JTAG
  wire [7:0]                   psmOscMsb0_VDD1;     // Input to CORE_PSM_IP_NO_JTAG
  wire [7:0]                   psmOscMsb1_VDD1;     // Input to CORE_PSM_IP_NO_JTAG
  wire                         psmOscGater_VDD1;    // Input to CORE_PSM_IP_NO_JTAG
  wire                         psmOscGater_VDD1_1;  // Mux out selectable by LBistEn

   wire [31:0] psmOscLsb_mcp;
   wire [7:0] psmOscMsb0_mcp;
   wire [7:0] psmOscMsb1_mcp;

  wire [63:0]                  RegisterWrDat;
  wire [63:0]                  PSM_RdDat;
  wire                         MM_Psm_Enable;

  wire                         Reset;  // tie low

  wire [3:0]                   LS_VDD1_to_VDD2_in;
  wire [3:0]                   LS_VDD1_to_VDD2_out;
  wire [48:0]                  LS_VDD2_to_VDD1_in;
  wire [48:0]                  LS_VDD2_to_VDD1_out;

  assign     Reset   = ScanShiftEn; //need to connect as an external pin and drive with ~PwrOk; ScanShiften will prevent ATPG+PSM concurrent operation //1'b0; // external reset not used for ZAK
  assign     JTAG_Psm_Rst_Reset = JTAG_Psm_Rst | Reset;
  assign     Psm_Rst = TrstSyncd | Reset;


// CDCBUF for Avfs_ScanMode (ATPG test JTAG -> Host_clk)
//CORE_CDCBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES)) I_Avfs_ScanMode_CDC (
  //CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY)) // add a top level param to configure X prop delay
  //   I_CDC_Avfs_ScanMode (.A(Avfs_ScanMode),
  //                  .Z(Avfs_ScanMode_CDC),
  //                  .EN(1'b1),
  //                  .DSTCLK(Host_Clk) );
   // Fix for http://bugs.amd.com/ubts/prod/issue/view?id=643502
  techind_cdcefpm_icd #(.DestClkCnt(0), .CdcDestSetup(CDC_TST_PROP_DLY), .size(1)) I_Avfs_ScanMode_CDC (
     .A(Avfs_ScanMode), 
     .DSTCLK(Host_Clk), 
     .EN(1'b1), 
     .Z(Avfs_ScanMode_CDC)
     `ifdef ABV
       ,.InputChkDis((WRSTN!==1'b1)|~PsmRoOutIso) 
     `endif
     );



assign RegisterRWLo =  RegisterReadEnLo | RegisterWriteEn;
assign RegisterRWHi =  RegisterReadEnHi | RegisterWriteEn;
assign GlobalSelPsm = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_PARAM_ADDR) |
                      (RegisterAddress[3:0] == `AVFS_WRAP_PSM_DATA_ADDR );
assign SelPsmParam  = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_PARAM_ADDR) & RegisterRWLo;
assign SelPsmData   = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_DATA_ADDR ) & RegisterRWLo;


//--------------------------------------------------------------------
// PSM interface (VDD1 domain)
//--------------------------------------------------------------------
  gfx_CORE_PSM_IP_NO_JTAG #(
  .PSM_CURPSM_LENGTH(PSM_CURPSM_LENGTH),
  .MCP_CYCLES_MAPSMRO(MCP_CYCLES_MAPSMRO),
  .MCP_CYCLES_AVGPSMREG(MCP_CYCLES_AVGPSMREG)
) I_Psm (
  .ScanShiftEn               (ScanShiftEn                   ),
  .Psm_Rst                   (Psm_Rst                       ),
  .Psm_ScanMode              (Avfs_ScanMode_CDC             ),
//.Psm_Gater                 (PsmAvfs_Gater                 ),
  .Psm_Gater                 (1'b1                          ),
  .JTAG_Psm_Enable           (JTAG_Psm_Enable               ),
  .JTAG_Psm_AvgEn            (JTAG_Psm_AvgEn                ),
  .JTAG_Psm_Rst              (JTAG_Psm_Rst_Reset            ),
  .JTAG_Psm_OneShot          (JTAG_Psm_OneShot              ),
  .PsmEn                     (PsmEn                         ),
  .Host_Clk                  (PsmClk                        ),
  .HostReg_Clk               (HostReg_Clk                   ),
//.nullNxtAvg                (SkipPhase                     ),
  .nullNxtAvg                (1'b0                          ),
//.div_clk_gater             (PsmDivClkGater                ),         
  .div_clk_gater             (                              ),         
`ifdef GFX_DROOP_DETECT
  .psmCountOut               (PsmOut[PSM_CURPSM_LENGTH-1:0] ), // directing PSM count out through AVFS3 for GNL purposes
`endif
  .deltaPsmRegOut            (PsmDeltaOut[13:0]             ),                       
  .Alarm_Flag                (Psm_Alarm_Flag                ),
  .WrDat                     (RegisterWrDat[63:0]           ),
  .RdDat                     (PSM_RdDat[63:0]               ),
  .MM_Psm_Enable             (MM_Psm_Enable                 ),
  .SelPsmParam               (SelPsmParam                   ),
  .SelPsmData                (SelPsmData                    ),
  .WrEn                      (GlobalSelPsm & RegisterWriteEn),
  .clkDivider                (div_VDD1[1:0]                 ),
  .oscEn                     (oscEn_VDD1                    ),
  .psmOscLsb                 (psmOscLsb_VDD1[31:0]          ),
  .psmOscMsb0                (psmOscMsb0_VDD1[7:0]          ),
  .psmOscMsb1                (psmOscMsb1_VDD1[7:0]          ),
  .psmOscGater               (psmOscGater_VDD1_1            ),
  .dbgCurPsm                 (dbgCurPsm[PSM_CURPSM_LENGTH-1:0]   ),
  .dbgAvgPsm                 (dbgAvgPsm[PSM_CURPSM_LENGTH-1:0]   ),
  .dbgThreshold              (dbgThreshold[PSM_CURPSM_LENGTH-1:0]),
  .dbgMinMax                 (dbgMinMax[PSM_CURPSM_LENGTH-1:0]   ),
  .dbgAlarm                  (dbgAlarm                           )
);

assign psmOscGater_VDD1_1 = Avfs_ScanMode_CDC ? 1'b1 : psmOscGater_VDD1 ; // Fix for ATPG to keep psmOscGater for being "x", PSM outputs are "X" in test mode

assign  clk_VDD1  =  CCLK;

//--------------------------------------------------------------------
// - VDD1 --> VDD2 level shifters
//       - PSM_IP_NO_JTAG outputs in VDD1 domain
//       - PSM_IP_OSC inputs in VDD2 domain
//       - 4 level-shifters
//--------------------------------------------------------------------
assign  LS_VDD1_to_VDD2_in[0]    =  clk_VDD1;
assign  LS_VDD1_to_VDD2_in[2:1]  =  div_VDD1[1:0];
assign  LS_VDD1_to_VDD2_in[3]    =  oscEn_VDD1;

//lsbuf  lsbuf__VDD1_to_VDD2[3:0] (.z(LS_VDD1_to_VDD2_out[3:0]), .a(LS_VDD1_to_VDD2_in[3:0]), .pwrok(PsmRoInIso));
// Bug #611294 (removal of level-shifters)
assign  LS_VDD1_to_VDD2_out[3:0]  =  LS_VDD1_to_VDD2_in[3:0] ;

// used now, driving mapsmro/pwrOK pin - //! unused  PsmRoInIso

assign  clk_VDD2       =  LS_VDD1_to_VDD2_out[0];
assign  div_VDD2[1:0]  =  LS_VDD1_to_VDD2_out[2:1];
assign  oscEn_VDD2     =  LS_VDD1_to_VDD2_out[3];


//--------------------------------------------------------------------
// - VDD2 --> VDD1 level shifters
//       - PSM_IP_OSC outputs in VDD2 domain
//       - PSM_IP_NO_JTAG inputs in VDD1 domain
//       - 49 level-shifters
//--------------------------------------------------------------------
//assign  LS_VDD2_to_VDD1_in[31:0]   =  psmOscLsb_VDD2[31:0];
//assign  LS_VDD2_to_VDD1_in[39:32]  =  psmOscMsb0_VDD2[7:0];
//assign  LS_VDD2_to_VDD1_in[47:40]  =  psmOscMsb1_VDD2[7:0];
assign  LS_VDD2_to_VDD1_in[31:0]   =  psmOscLsb_mcp[31:0];
assign  LS_VDD2_to_VDD1_in[39:32]  =  psmOscMsb0_mcp[7:0];
assign  LS_VDD2_to_VDD1_in[47:40]  =  psmOscMsb1_mcp[7:0];
assign  LS_VDD2_to_VDD1_in[48]     =  psmOscGater_VDD2;

//lsbuf  lsbuf__VDD2_to_VDD1[48:0] (.z(LS_VDD2_to_VDD1_out[48:0]), .a(LS_VDD2_to_VDD1_in[48:0]), .pwrok(PsmRoOutIso));
// Bug #611294 (removal of level-shifters)
assign  LS_VDD2_to_VDD1_out[48:0]  =  LS_VDD2_to_VDD1_in[48:0];

assign  psmOscLsb_VDD1[31:0]  =  LS_VDD2_to_VDD1_out[31:0];
assign  psmOscMsb0_VDD1[7:0]  =  LS_VDD2_to_VDD1_out[39:32];
assign  psmOscMsb1_VDD1[7:0]  =  LS_VDD2_to_VDD1_out[47:40];
assign  psmOscGater_VDD1      =  LS_VDD2_to_VDD1_out[48];


  // Move the MCBUF here on mapsmro outputs as Reset is not available inside CORE_PSM_IP_OSC wrapper 

`ifdef AVFS3_MCENBUF_ON
  wire mcbufEn;
  assign mcbufEn = oscEn_VDD1 & (~Psm_Rst) & psmOscGater_VDD2 ; // VDD1, VDD2 have no significance here, all of it is VDD1 domain. Earlier to pushing level shifters inside the mapsmro macro, these made sense. So, this is not a mix of two voltage domains without any level shifters etc
  gfx_CORE_MCENBUF #(.size(32), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscLsb_mcp (.A(psmOscLsb_VDD2[31:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(clk_VDD2),
                                                                          .Z(psmOscLsb_mcp[31:0]));
  gfx_CORE_MCENBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb0_mcp (.A(psmOscMsb0_VDD2[7:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(clk_VDD2),
                                                                          .Z(psmOscMsb0_mcp[7:0]));
  gfx_CORE_MCENBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb1_mcp (.A(psmOscMsb1_VDD2[7:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(clk_VDD2),
                                                                          .Z(psmOscMsb1_mcp[7:0]));
`else
  gfx_CORE_MCBUF #(.size(32), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscLsb_mcp (.A(psmOscLsb_VDD2[31:0]), 
                                                                          .DSTCLK(clk_VDD2),
                                                                          .Z(psmOscLsb_mcp[31:0]));
  gfx_CORE_MCBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb0_mcp (.A(psmOscMsb0_VDD2[7:0]), 
                                                                          .DSTCLK(clk_VDD2),
                                                                          .Z(psmOscMsb0_mcp[7:0]));
  gfx_CORE_MCBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb1_mcp (.A(psmOscMsb1_VDD2[7:0]), 
                                                                          .DSTCLK(clk_VDD2),
                                                                          .Z(psmOscMsb1_mcp[7:0]));
`endif

//--------------------------------------------------------------------
// JTAG/1500 interface (VDD1 domain)
//--------------------------------------------------------------------
  gfx_AvfsWrapJtgInterface  #(.SYNC_TC_CYCLES(SYNC_TC_CYCLES), .NEG_JTAG_TDO(NEG_JTAG_TDO)) I_AvfsWrapJtgInterface 
(
  .RegisterOutput            (RegisterWrDat[63:0]           ),
  .TrstSyncd_CCLK            (TrstSyncd                     ),
  .JTAG_ClkEn_ASYNC          (JTAG_ClkEn_ASYNC              ),
  .JTAG_ClkEn_CCLK           (JTAG_ClkEn_CCLK               ),
  .RegisterAddress           (RegisterAddress[3:0]          ),
  .RegisterWriteEn           (RegisterWriteEn               ),
  .RegisterReadEnLo          (RegisterReadEnLo              ),
  .RegisterReadEnHi          (RegisterReadEnHi              ),
  .JTAG_Psm_Enable           (JTAG_Psm_Enable               ),
  .JTAG_Psm_AvgEn            (JTAG_Psm_AvgEn                ),
  .JTAG_Psm_Rst              (JTAG_Psm_Rst                  ),
  .JTAG_Psm_OneShot          (JTAG_Psm_OneShot              ),
//.AvfsGo_OneShot            (AvfsGo                        ),
  .AvfsGo_OneShot            (                              ),
  .JTAG_DbgEn                (dbgEn                         ),
//.RegisterInput    (SrbLclAvfsRdDat[63:0] | PSM_RdDat[63:0]),
  .RegisterInput             (PSM_RdDat[63:0]               ),
  .PwrMonEnX                 (1'b1                          ),  // mux between SPMI (0) and p1500 path(1)
  .SpmiClk                   (1'b0                          ),  // ZAK does not support Spmi
  .SpmiTdi                   (1'b0                          ),  // ZAK does not support Spmi
  .SpmiShift                 (1'b0                          ),  // ZAK does not support Spmi
  .SpmiUpdate                (1'b0                          ),  // ZAK does not support Spmi
  .SpmiCapEn                 (1'b0                          ),  // ZAK does not support Spmi
  .SpmiPathEn                (1'b0                          ),  // ZAK does not support Spmi
  .SpmiTrst                  (1'b0                          ),  // ZAK does not support Spmi
  .SpmiTdo                   (                              ),  // ZAK does not support Spmi
  .SpmiMeas                  (1'b0                          ),  // ZAK does not support Spmi
  .WRCK                      (WRCK                          ),
  .WSI                       (WSI                           ),
  .ShiftWR                   (ShiftWR                       ),
  .UpdateWR                  (UpdateWR                      ),
  .CaptureWR                 (CaptureWR                     ),
  .WDREn                     (WDREn                         ),
  .WRSTN                     (WRSTN                         ),
  .MeasureWR                 (MeasureWR                     ),
  .WRO                       (WRO                           ),
//.PsmAvfs_Gater             (PsmAvfs_Gater                 ),
  .PsmAvfs_Gater             (1'b1                          ),
  .Isolate                   (Isolate                       ),
//.SkipPhase                 (SkipPhase                     ),
  .SkipPhase                 (1'b0                          ),
  .Reset                     (Reset                         ),
  .CCLK                      (Host_Clk                      ),  // Changed to Host_Clk from CCLK (Free running MESH clock) : fix for http://bugs.amd.com/ubts/prod/issue/view?id=608619
  .CCLK_CDC                  (Host_CDC_Clk                 ),  // Changed to Host_Clk from CCLK (Free running MESH clock) : fix for http://bugs.amd.com/ubts/prod/issue/view?id=608619
  .HostReg_CCLK              (HostReg_Clk                   ),
  .MmBusIn_M1                (8'b0                          ),  // MM interface is unused
  .MmLclRdEn_M2              (1'b0                          ),  // MM interface is unused
  .MmLclWrEn_M2              (1'b0                          ),  // MM interface is unused
  .MmLclSz_M2                (2'b0                          ),  // MM interface is unused
  .MmState                   (5'b0                          ),  // MM interface is unused
  .MmDatOut_M4               (                              ),  // MM interface is unused
  .ScanShiftEn               (ScanShiftEn                   ),  // MM interface is unused
  .LBistEn                   (Avfs_ScanMode_CDC             ),  // MM interface is unused
  .MmActiveEn                (                              )   // MM interface is unused
);


//-----------------------------------------------------------
// Debug interface
//-----------------------------------------------------------
assign dbgEn_ScanMode = dbgEn | Avfs_ScanMode_CDC;
assign AvfsNearMiss[49:0] = 50'b0; //DbgBusCPA: unused

  // adding debug counter for DECPLCPLK-2833
  logic [3:0] DbgCntrNxt;
  logic [3:0] DbgCntr;
  assign  DbgCntrNxt = DbgCntr + 1;
  gfx_CORE_FLOP_GATED_RESET  #(.size(4),.rv({4'b0}))  I_DbgCntr (.D(DbgCntrNxt[3:0]), .Q(DbgCntr[3:0]), .G(dbgEn_ScanMode|TrstSyncd), .GC(1'b0), .R(TrstSyncd|Reset), .SSE(ScanShiftEn), .C(Host_Clk)); // NOTE: ".GC(1'b0)" is unused

always @* begin
  casez({dbgEn_ScanMode, DbgBusSel[DBG_CFG_LENGTH-1:0]})
    4'b0_???: DbgBusNxt[DBG_LENGTH-1:0] = {DBG_LENGTH{1'b0}};
    4'b1_000: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgCurPsm[PSM_CURPSM_LENGTH-1:0], dbgAlarm};  //DbgBusPSM
    4'b1_001: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgAvgPsm[PSM_CURPSM_LENGTH-1:0], dbgAlarm};  //DbgBusPSM
    4'b1_010: DbgBusNxt[DBG_LENGTH-1:0] = {2'b0, dbgThreshold[PSM_CURPSM_LENGTH-1:0]};         //DbgBusPSM
    4'b1_011: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgMinMax[PSM_CURPSM_LENGTH-1:0], dbgAlarm};  //DbgBusPSM
//  4'b1_100: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsNearMiss[15:0]};  //DbgBusCPA
    4'b1_100: DbgBusNxt[DBG_LENGTH-1:0] = {{12'hFEA}, DbgCntr[3:0]}; //Adding PSM signature (12'hFEA), counter value
    4'b1_101: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsNearMiss[31:16]}; //DbgBusCPA
    4'b1_110: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsNearMiss[47:32]}; //DbgBusCPA
    4'b1_111: DbgBusNxt[DBG_LENGTH-1:0] = {PsmDeltaOut[13:0],AvfsNearMiss[49:48]}; //DbgBusCPA & DbgBusPSM
    // VCS coverage off
    default : DbgBusNxt[DBG_LENGTH-1:0] = {DBG_LENGTH{1'bx}};
    // VCS coverage on
  endcase
end

  gfx_CORE_FLOP_GATED_RESET  #(.size(DBG_LENGTH))  I_DbgBus (
  .D  (DbgBusNxt[DBG_LENGTH-1:0]  ),
  .Q  (DbgBus_int[DBG_LENGTH-1:0] ),
  .G  (dbgEn_ScanMode | TrstSyncd ),
  .GC (1'b0                       ), // unused
  .R  (TrstSyncd | Reset          ),
  .SSE(ScanShiftEn                ),
  .C  (Host_Clk                   )
);

// Avfs_ScanMode==1 will force DbgBus and Alarm_Flag to 0

assign DbgBus[DBG_LENGTH-1:0] = DbgBus_int[DBG_LENGTH-1:0] & {DBG_LENGTH{~Avfs_ScanMode_CDC}};


//   CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY))
//     I_CDC_JTAG_ClkEn (.A(JTAG_ClkEn_ASYNC),
//                    .Z(JTAG_ClkEn_CDC),
//                    .EN(1'b1),
//                    .DSTCLK(CCLK) );   

//-----------------------------------------------------------
// Clock gaters
//-----------------------------------------------------------
//assign  HostJtag_CDC_ClkEn = JTAG_ClkEn_CDC | JTAG_ClkEn_CCLK;  // async assertion and synchronous de-assertion
assign  HostJtag_ClkEn = JTAG_ClkEn_CCLK;  // synchronous-only assertion and de-assertion
//assign         Psm_ClkEn      = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | JTAG_Psm_Rst_Reset;
assign  Psm_ClkEn = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | Avfs_ScanMode_CDC; //Term JTAG_Psm_Rst_Reset can be removed, as PSM wakes up with Psm_Enable; added Avfs_ScanMode_CDC for DECPLCPLK-2355

// vscan off cellprop
  gfx_ccgu  I_KCLK_Free_AR         (.gclk(KCLK_Free_AR), .clk(CCLK), .TstCcgEn(TstCcgEn));
  gfx_cgu   I_RCLK_Free_AR         (.gclk(RCLK_Free_AR), .clk(KCLK_Free_AR));

//ccg   I_KCLK_HostJtag_CDC_Clk_AR (.gclk(Host_CDC_Clk),     .clk(CCLK), .fclk(RCLK_Free_AR), .en(HostJtag_CDC_ClkEn), .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccgu  I_HostClk_Free_AR      (.gclk(Host_CDC_Clk), .clk(CCLK), .TstCcgEn(TstCcgEn)); // Removing gating from Host_CDC_Clk, this really should be called Free clock, runs just one synchronizer in AvfsWrapJtgInterface
  gfx_ccg   I_KCLK_HostJtag_Clk_AR (.gclk(Host_Clk),     .clk(CCLK), .fclk(RCLK_Free_AR), .en(HostJtag_ClkEn), .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
assign HostReg_Clk = Host_Clk;  // NOTE: since there is no SPMI or MM interface, the only way to access registers is via JTAG
  gfx_ccg   I_KCLK_PsmClk_AR       (.gclk(PsmClk),       .clk(CCLK), .fclk(RCLK_Free_AR), .en(Psm_ClkEn),      .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
// vscan on cellprop


endmodule // CORE_PSM_IP_JTAG


// --------------------------------------------------------
// AVFS Wrapper JTAG Interface
// --------------------------------------------------------
module gfx_AvfsWrapJtgInterface #(parameter
  SYNC_TC_CYCLES        = 0,   // Overriding as 3 for Zen, Aspen and Zepplin at AVFS3_TOP, making the parameter programmable at AVFS3_TOP, & CORE_PSM_IP_LS
  CDC_TST_PROP_DLY      = 0,
  DATA_LENGTH           = 64,  // data register
  CTRL_LENGTH           = 16,  // ctrl register
  JTAG_ALARM            = 0,   // position relative to PsmDataReg
  JTAG_TEST             = 43,  // position relative to PsmParamReg
  CTRL_ISOLATE          = 12,  // indicates power isolation event since last AvfsGo pulse
  CTRL_SKIP_PHASE       = 11,  // indicates clock was stretched since last AvfsGo pulse
  CTRL_DBG_EN           = 10,  // Enables debug bus
  CTRL_AVFS_GO          = 9,   // Writing 1 on this bit triggers an AVFS Go, implements oneshot logic, position rel to CtrlReg
  CTRL_REG_WRITE_EN     = 8,   // Gates capture signal sent to CPA/PSM, position relative to CtrlReg
  CTRL_REG_ADDR_H       = 7,   // high bit of register address position relative to CtrlReg
  CTRL_REG_ADDR_L       = 4,   // lower bit of register address position relative to CtrlReg
  CTRL_PSM_AVG_EN       = 3,   // position relative to CtrlReg               
  CTRL_PSM_ONE_SHOT     = 2,   // position relative to CtrlReg   
  CTRL_PSM_RST          = 1,   // position relative to CtrlReg
  CTRL_PSM_ENABLE       = 0,   // position relative to CtrlReg
  NEG_JTAG_TDO          = 0    // Enables TDO/WRO on negedge of JTAG clock
)(
  output wire [DATA_LENGTH-1:0]            RegisterOutput,
  output wire                              TrstSyncd_CCLK,
  output wire                              JTAG_ClkEn_ASYNC,
  output wire                              JTAG_ClkEn_CCLK,
  output wire [3:0]                        RegisterAddress,
  output wire                              RegisterWriteEn,
  output wire                              RegisterReadEnLo,
  output wire                              RegisterReadEnHi,
  output wire                              JTAG_Psm_Enable,
  output wire                              JTAG_Psm_AvgEn,
  output wire                              JTAG_Psm_Rst,
  output wire                              JTAG_Psm_OneShot,
  output wire                              AvfsGo_OneShot,
  output wire                              JTAG_DbgEn,
  input       [DATA_LENGTH-1:0]            RegisterInput,

  input                                    PsmAvfs_Gater,
  input                                    Reset,
  input                                    Isolate,                              // indicates tile is being power gated
  input                                    SkipPhase,                            // indicates Host_Clk is being stretched

  // SPMI interface
  input                                    PwrMonEnX,
  input                                    SpmiClk,
  input                                    SpmiTdi,
  input                                    SpmiShift,
  input                                    SpmiUpdate,
  input                                    SpmiCapEn,
  input                                    SpmiPathEn,
  input                                    SpmiTrst,
  output wire                              SpmiTdo,
  input                                    SpmiMeas,

  // JTAG interface
  input                                    WRCK,
  input                                    WSI,
  input                                    ShiftWR,
  input                                    UpdateWR,
  input                                    CaptureWR,
  input                                    WDREn,
  input                                    WRSTN,
  input                                    MeasureWR,
  output wire                              WRO,

`ifdef AVFS_ZAK                                                                   // M&M interface
  input                                    MmLclRdEn_M2,                          //   local register read indication
  input                                    MmLclWrEn_M2,                          //   local register write indication
  input       [1:0]                        MmLclSz_M2,                            //   local register size indication
  input       [`MM_STATE]                  MmState,                               //   M&M state for read/write byte select
  input       [`MM_DATA]                   MmBusIn_M1,                            //   8-bit M&M bus input contains local reg addr in M2 and write data in M4
  output      [`MM_DATA]                   MmDatOut_M4,                           //   read data from local register
  output wire                              MmActiveEn,                            // feedback to top-level CCG to allow register access (i.e. wake up clocks)
`endif    // AVFS_ZAK

  input                                    ScanShiftEn,                           //   SSE needed for M&M write staging flop
  input                                    LBistEn,                               //   LBistEn added to select between scan and jtag paths

  input                                    HostReg_CCLK,
  input                                    CCLK_CDC,
  input                                    CCLK
  );

  wire    Trst;
  wire    JtgClk;
  wire    JtgUpdate;
  wire    JtgCapEn;
  wire    JtgShift;
  wire    JtgMeasure;
  wire    JtgTdi;
  wire    JtgGater;
  wire    JtgTdo;

  // spmi/jtag mux
  //   PwrMonEnX = 0 -> choose spmi
  //   PwrMonEnX = 1 -> choose jtag
  assign WRO        =  PwrMonEnX ? JtgTdo :
                      ~PwrMonEnX ? 1'b0 : 1'bx;
  assign SpmiTdo    = ~PwrMonEnX ? JtgTdo :
                       PwrMonEnX ? 1'b0 : 1'bx;

//!unused MeasureWR     -- Removed/replaced (see DECPLCPLK-2796)

  // Give priority to p1500 reset, regardless of PwrMonEnX state - based on feedback from Kamran
`ifdef AVFS_ZAK
  assign Trst       = ~WRSTN ;
  assign JtgClk     = WRCK;  // ZAK doesn't use SPMI; remove mux to avoid "flops not in clock tree" linting error
  assign JtgUpdate  = UpdateWR ;
  assign JtgCapEn   = CaptureWR ;
  assign JtgShift   = (ShiftWR & WDREn) ;
//  assign JtgMeasure = MeasureWR ; -- Removed/replaced (see DECPLCPLK-2796)
  assign JtgTdi     = WSI ;
  assign JtgGater  = WDREn ;
`else
  assign Trst       = ~WRSTN | (~PwrMonEnX & SpmiTrst);
  assign JtgClk     = ~PwrMonEnX ? SpmiClk :
                       PwrMonEnX ? WRCK : 1'bx;
  assign JtgUpdate  = ~PwrMonEnX ? SpmiUpdate :
                       PwrMonEnX ? UpdateWR : 1'bx;
  assign JtgCapEn   = ~PwrMonEnX ? SpmiCapEn :
                       PwrMonEnX ? CaptureWR : 1'bx;
  assign JtgShift   = ~PwrMonEnX ? (SpmiShift & SpmiPathEn) :
                       PwrMonEnX ? (ShiftWR & WDREn) : 1'bx;
//  assign JtgMeasure = ~PwrMonEnX ? SpmiMeas : -- Removed/replaced (see DECPLCPLK-2796)
//                       PwrMonEnX ? MeasureWR : 1'bx;
  assign JtgTdi     = ~PwrMonEnX ? SpmiTdi :
                       PwrMonEnX ? WSI : 1'bx;
  assign JtgGater  = ~PwrMonEnX ? SpmiPathEn :
                       PwrMonEnX ? WDREn : 1'bx;
`endif    // AVFS_ZAK

  // Synchronizers
  wire JtgGaterSyncd_CCLK;
  wire JtgMeasureSyncd_CCLK;
  wire JtgUpdateSyncd_CCLK;
  wire JtgMeasureSyncd_CCLK_d1;
  wire JtgUpdateSyncd_CCLK_d1;
  wire JtgUpdateSyncd_CCLK_d2;

  //wire JtgGater_ClkEn;  // async assertion and synchronous de-assertion // unused now
  //wire Trst_ClkEn;       // async assertion and synchronous de-assertion
                                 
  //wire CDC_Trst;
  //wire CDC_JtgGater;
  wire JtgMeasureSyncdEdge;
  wire JtgUpdateSyncdEdge;
  wire JtgUpdateSyncdEdge_d1;


  // Jtag Trst flop enabled
  //   when driving a jtag command
  //   Trst to enable flop when Trst transitions from 0 -> 1
  //   TrstSyncd_CCLK to enable flop when Trst transitions from 1 -> 0
  //   cold or warm reset assertion to allow flop to reset in functional mode w/o requiring Trst assertion
  wire [`CLKBUS_SIZE-1:0] clkbus_CDC;
  wire [`CLKBUS_SIZE-1:0] clkbus_sync;

  gfx_CORE_CG I_cg_clkbus_sync (.clkbus(clkbus_sync[`CLKBUS_SIZE-1:0]), .C(CCLK_CDC), .G(JtgGaterSyncd_CCLK), .R(Reset), .SSE(ScanShiftEn));
  gfx_CORE_CGU I_cgu_CDC (.clkbus(clkbus_CDC[`CLKBUS_SIZE-1:0]), .C(CCLK_CDC), .R(Reset), .SSE(ScanShiftEn));  // need to flush X's from synchronizers;  harmless for synchronizers to flush-to-zero on SSE since it will not impact retention of PSM configuration (i.e. CtrlReg)
  
  //AVFS_SYNC_NOCG #(.rv(1'b1),.TC(3)) I_TrstSyncd_CCLK (.SampleClk(clkbus_CDC[`C_PIN]), .Reset(Reset), .SSE(clkbus_CDC[`SSE_PIN]), .DataIn(Trst), .DataOut(TrstSyncd_CCLK) ); // need to reset into "inactive" (1'b1) state to avoid extra cycles of reset


  // Put disclaimers in spec that JTAG (async) signals must be glitch free 
  // Salih added a WRCK flop on Measure and WDREn/JtgGater outside of AVFS3 and DLDO
  gfx_AVFS_SYNC_NOCG #(.TC(SYNC_TC_CYCLES)) I_JtgGater_CCLK (.SampleClk(clkbus_CDC[`C_PIN]), .Reset(Reset), .SSE(clkbus_CDC[`SSE_PIN]), .DataIn(JtgGater), .DataOut(JtgGaterSyncd_CCLK) );

  assign JTAG_ClkEn_ASYNC = 1'b0 ; // keeping the port, driving a 1'b0 for convenience
  assign JTAG_ClkEn_CCLK  = Reset | JtgGaterSyncd_CCLK;  // synchronous de-assertion

  // IEEE 1500 deprecates the jtag measure signal, but SPMI (AVFS/MM Bus, we still need the measure signal for AVFS to function as the logic is today) still uses it. ZAK/KK/CM still plan to support
  // it.
  //   JtgMeasure -> transfer read data from AVFS/PSM internal register to DataReg[63:0]
  //   JtgCapEn   -> transfer read data from DataReg[63:0] to DataRegSer[63:0] (WDR[63:0])
  //
  // JTAG registers must preserve their value across warm reset
  //   assume JTAG Update, Measure and Capture are never asserted simultaneously
  //
  // Jtag Measure/Update flops enabled
  //   when driving a jtag command
  //   Jtag/SPMI reset
  //   cold or warm reset to allow flops to reset in functional mode w/o requiring Trst assertion
  //AVFS_SYNC_NOCG #(.TC(SYNC_TC_CYCLES)) I_JtgMeasureSyncd_CCLK (.SampleClk(clkbus_sync[`C_PIN]), .Reset(Reset), .SSE(clkbus_sync[`SSE_PIN]), .DataIn(JtgMeasure), .DataOut(JtgMeasureSyncd_CCLK) );  -- Removed/replaced (see DECPLCPLK-2796)


     // Adding B-phase JTAG flop on JtgUpdate, there is combo logic on JtgUpdate coming into AVFS
   // JtgGater (WDREn), and JtgMeasure are taken care of at the top level (Source: Salih Hamid)
   wire JtgUpdate_bph;
  gfx_CORE_PSM_IP_JTAGFLOP_B #(.size(1)) I_JtgUpdate_bph (
                              .ASYNCCLK (Trst),
                              .ASYNCVAL (1'b0),
                              .CLK      (JtgClk),
                              .D        (JtgUpdate),
                              .Q        (JtgUpdate_bph));

  gfx_AVFS_SYNC_NOCG #(.TC(1), .IC(0)) I_JtgUpdateSyncd_CCLK  (.SampleClk(clkbus_sync[`C_PIN]), .Reset(Reset), .SSE(clkbus_sync[`SSE_PIN]), .DataIn(JtgUpdate_bph), .DataOut(JtgUpdateSyncd_CCLK) ); 

// Below added to replace previous MeasureWR input (see DECPLCPLK-2796)
  gfx_CORE_PSM_IP_JTAGFLOP #(.size(1)) I_JtgMeasure (
                              .ASYNCCLK (Trst),
                              .ASYNCVAL (1'b0),
                              .CLK      (JtgClk),
                              .D        (JtgUpdate_bph & JtgGater),
                              .Q        (JtgMeasure));

  gfx_AVFS_SYNC_NOCG #(.TC(1), .IC(0)) I_JtgMeasure_CCLK  (.SampleClk(clkbus_sync[`C_PIN]), .Reset(Reset), .SSE(clkbus_sync[`SSE_PIN]), .DataIn(JtgMeasure), .DataOut(JtgMeasureSyncd_CCLK) ); 


  // Delayed versions of JtgMeasure & JtgUpdate syncd versions to use for read & write enables
  // to AVFS/PSM register logic
  wire [`CLKBUS_SIZE-1:0] clkbus_JtgMeasureUpdateSyncd_CCLK;
  wire                    Cgen_JtgMeasureSyncd_CCLK_d1;
  wire                    Cgen_JtgUpdateSyncd_CCLK_d1;
  wire                    Cgen_JtgUpdateSyncd_CCLK_d2;

  assign Cgen_JtgMeasureSyncd_CCLK_d1 = (JtgMeasureSyncd_CCLK   | JtgMeasureSyncd_CCLK_d1);
  assign Cgen_JtgUpdateSyncd_CCLK_d1  = (JtgUpdateSyncd_CCLK    | JtgUpdateSyncd_CCLK_d1);
  assign Cgen_JtgUpdateSyncd_CCLK_d2  = (JtgUpdateSyncd_CCLK_d1 | JtgUpdateSyncd_CCLK_d2);

  gfx_CORE_CG I_cg_JtgMeasureUpdateSyncd_CCLK_d1 (.clkbus(clkbus_JtgMeasureUpdateSyncd_CCLK[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(Cgen_JtgMeasureSyncd_CCLK_d1 | Cgen_JtgUpdateSyncd_CCLK_d1 | Cgen_JtgUpdateSyncd_CCLK_d2), .R(Reset), .SSE(ScanShiftEn)); 

  gfx_CORE_FLOP_DGATE_RESET I_JtgMeasureSyncd_CCLK_d1 (.clkbus(clkbus_JtgMeasureUpdateSyncd_CCLK[`CLKBUS_SIZE-1:0]), .DG(Cgen_JtgMeasureSyncd_CCLK_d1),
  //CORE_FLOP_DGATE_RESET I_JtgMeasureSyncd_CCLK_d1 (.clkbus(clkbus_JtgMeasureUpdateSyncd_CCLK[`CLKBUS_SIZE-1:0]), .DG(1'b1),
                                                    .D(JtgMeasureSyncd_CCLK),
                                                    .Q(JtgMeasureSyncd_CCLK_d1)
                                                   );

  gfx_CORE_FLOP_DGATE_RESET I_JtgUpdateSyncd_CCLK_d1  (.clkbus(clkbus_JtgMeasureUpdateSyncd_CCLK[`CLKBUS_SIZE-1:0]), .DG(Cgen_JtgUpdateSyncd_CCLK_d1),
  //CORE_FLOP_DGATE_RESET I_JtgUpdateSyncd_CCLK_d1  (.clkbus(clkbus_JtgMeasureUpdateSyncd_CCLK[`CLKBUS_SIZE-1:0]), .DG(1'b1),
                                                    .D(JtgUpdateSyncd_CCLK),
                                                    .Q(JtgUpdateSyncd_CCLK_d1)
                                                   );

  gfx_CORE_FLOP_DGATE_RESET I_JtgUpdateSyncd_CCLK_d2  (.clkbus(clkbus_JtgMeasureUpdateSyncd_CCLK[`CLKBUS_SIZE-1:0]), .DG(Cgen_JtgUpdateSyncd_CCLK_d2),
  //CORE_FLOP_DGATE_RESET I_JtgUpdateSyncd_CCLK_d2  (.clkbus(clkbus_JtgMeasureUpdateSyncd_CCLK[`CLKBUS_SIZE-1:0]), .DG(1'b1),
                                                    .D(JtgUpdateSyncd_CCLK_d1),
                                                    .Q(JtgUpdateSyncd_CCLK_d2)
                                                   );
  // JtgMeasureSyncdEdge is a 1-clock pulse that asserts on rising edge of JtgMeasureSyncd_CCLK
  // JtgUpdateSyncdEdge is a 1-clock pulse that asserts on rising edge of JtgUpdateSyncd_CCLK
  // JtgUpdateSyncdEdge_d1 is a 1-clock pulse that asserts 1-clock after the rising edge of JtgUpdateSyncd_CCLK
  //     the 1-clock delay is needed to allow the address to be written into CtrlReg so that
  //     RegisterAddress[3:0] aligns with RegisterWriteEn
  assign JtgMeasureSyncdEdge = JtgMeasureSyncd_CCLK & ~JtgMeasureSyncd_CCLK_d1;
  assign JtgUpdateSyncdEdge = JtgUpdateSyncd_CCLK & ~JtgUpdateSyncd_CCLK_d1;
  assign JtgUpdateSyncdEdge_d1 = JtgUpdateSyncd_CCLK_d1 & ~JtgUpdateSyncd_CCLK_d2;

  //16 bit control register, only updates
  reg  [CTRL_LENGTH-1:0] CtrlRegSerNxt;
  wire [CTRL_LENGTH-1:0] CtrlRegSer;
  wire [CTRL_LENGTH-4:0] CtrlRegNxt;
  wire [CTRL_LENGTH-4:0] CtrlReg;
  
  wire [CTRL_LENGTH-1:0] CDC_CtrlReg;
  wire [CTRL_LENGTH-1:0] CDC_CtrlRegSer;

`ifndef AVFS_ZAK
  // Declaration for M&M bus input for non-ZAK SOCs because it is not a primary input
  wire [7:0] MmBusIn_M1;
`endif

  // M&M interface signals
  //   driven to zeroes for non-ZAK SOCs
  wire                   MmAddrEn;
  wire                   MmRdEnLo;
  wire                   MmRdEnHi;
  wire                   MmRdEn;
  wire                   MmWrEn;
  wire [7:0]             MmDatWrEn;
  wire [DATA_LENGTH-1:0] MmDataRegNxt;
  wire                   JtgAbortMm;
  reg                    AvfsGoNxt;

  // Update
  
  // CDCBUF for update (TCK->CCLK)
  gfx_CORE_CDCENBUF #(.size(CTRL_LENGTH), .Cycles(`CDC_JTAG_X_DLY_CYCLES))
     I_CDC_CtrlRegSer (.A(CtrlRegSer[CTRL_LENGTH-1:0]),
                       .EN(JtgUpdateSyncdEdge),
                       .Z(CDC_CtrlRegSer[CTRL_LENGTH-1:0]),
                       .DSTCLK(CCLK) );

  // Select between M&M AVFS reg 4-bit address & jtag/spmi control
  //   M&M has priority
  //   bits 15:13 are unused, don't have flops and are driven to zeroes later
  assign CtrlRegNxt[CTRL_DBG_EN:CTRL_REG_ADDR_L] =  MmAddrEn? {{(CTRL_DBG_EN+1-8){1'b0}}, MmBusIn_M1[3:0]} : // capture address
                                                   ~MmAddrEn? CDC_CtrlRegSer[CTRL_DBG_EN:CTRL_REG_ADDR_L] :
                                                             {(CTRL_DBG_EN-CTRL_REG_ADDR_L+1){1'bx}};

  // The following 4 bits are supported both in the WDR and the PsmParamReg. However,
  // there is logic to ensure that these bits only get set to 1'b1 from one write source.
  // By default these bits are written via the WDR for SPMI/P1500 applications. 
  // For M&M Applications which don't support these extra WDR register bits, these parameters can be
  // written via by setting the PsmParam[JTAG_WDR_OVERRIDE] = 1'b1. 
  // 
  //   JTAG_WDR_OVERRIDE   1'b0        1'b1
  //                       ---------   ----------
  //                       SPMI/P1500      M&M
  //   PsmEnable:          WDR[64]     PsmParam[24]
  //   RstPsm   :          WDR[65]     PsmParam[0]
  //   OneShot  :          WDR[66]     PsmParam[5]
  //   AvgEn    :          WDR[67]     PsmParam[56]
  //
  // The motivation for moving these bits to the WDR was to reduce the number of
  // SPMI write operations. An SPMI write to the above 4 bits can be piggy-backed
  // onto another SPMI write to another register e.g AvfsCtl1 or AvfsCtl2. The RstPsm
  // and OneShot bits in the WDR shadow register, CtrlReg, are reset on a JTAG read.
  // This is done to keep the values of these bits in the WDR aligned with their
  // values in the PSM as internally a pulse is generated when there is a 0->1 transition
  // on these 2 bits. 

  // AvgEn bit
  assign CtrlRegNxt[CTRL_PSM_AVG_EN] = CDC_CtrlRegSer[CTRL_PSM_AVG_EN];

  // OneShot bit
  assign CtrlRegNxt[CTRL_PSM_ONE_SHOT] = CDC_CtrlRegSer[CTRL_PSM_ONE_SHOT] & ~JtgMeasureSyncdEdge;

  // RstPsm bit
  assign CtrlRegNxt[CTRL_PSM_RST] = CDC_CtrlRegSer[CTRL_PSM_RST] & ~JtgMeasureSyncdEdge;

  // PsmEnable bit
  //  Update function is achieved via clock gater on I_PsmParamReg0 (bug 341348)
  assign CtrlRegNxt[CTRL_PSM_ENABLE] = CDC_CtrlRegSer[CTRL_PSM_ENABLE];

   
  // Isolate bit - sticky bit that remains set if there is a power gating event
  //   asserted when tile is being power gated indicated by Isolate assertion
  //   deasserted on cycle folowing assertion of AvfsGo_OneShot
  //   cannot be written by JtAG/SPMI
  assign CtrlRegNxt[CTRL_ISOLATE] = (Isolate | CtrlReg[CTRL_ISOLATE]) & ~AvfsGo_OneShot;
   
  // SkipPhase - sticky bit that remains set if clock is stretched
  //   asserted when Host_Clk is stretched indicated by SkipPhase assertion
  //   deasserted on cycle folowing assertion of AvfsGo_OneShot
  //   cannot be written by JTAG/SPMI
  assign CtrlRegNxt[CTRL_SKIP_PHASE] = (SkipPhase | CtrlReg[CTRL_SKIP_PHASE]) & ~AvfsGo_OneShot;

  // Update function achieved via clock gater
  wire   CtrlReg_Gater;
  wire   CtrlReg_Gater_2_1;
  wire   CtrlReg_Gater_11;
  wire   CtrlReg_Gater_12;
  wire   AvfsGo_Gater;

  // Default gater for CtrlReg
  //   rising edge of JtgUpdateSyncd_CCLK
  //   Jtag/SPMI reset
  //   M&M address enable
  assign CtrlReg_Gater = JtgUpdateSyncdEdge | MmAddrEn; // TrstSyncd_CCLK

  // Unique gaters for PsmRst, OneShot, Isolate & SkipPhase
  
  // Enable PsmRst on
  //   CtrlReg_Gater
  //   rising edge of JtgMeasureSyncd
  assign CtrlReg_Gater_2_1 = CtrlReg_Gater | JtgMeasureSyncdEdge; // original

  // Enable SkipPhase/Isolate
  //   when input != output 
  //   AvfsGoGater
  assign CtrlReg_Gater_11 = (SkipPhase ^ CtrlReg[CTRL_SKIP_PHASE]) | AvfsGo_OneShot ;
  assign CtrlReg_Gater_12 = (Isolate ^ CtrlReg[CTRL_ISOLATE]) | AvfsGo_OneShot ;

  wire [`CLKBUS_SIZE-1:0] clkbus_CtrlReg_Gater;

  gfx_CORE_CG I_cg_CtrlReg_Gater (.clkbus(clkbus_CtrlReg_Gater[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .G(CtrlReg_Gater | CtrlReg_Gater_2_1 | CtrlReg_Gater_11 | CtrlReg_Gater_12), .R(Reset), .SSE(ScanShiftEn)); // TODO:  HostReg_CCLK also connects to scannable flops;  is this a problem for ATPG?  CtrlReg is shared between MM & JTAG to store 4 bit register address

  // No flops for upper 3 reserved bits
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) I_CtrlReg_12     (.clkbus(clkbus_CtrlReg_Gater[`CLKBUS_SIZE-1:0]), .DG(CtrlReg_Gater_12),
                                                       .D(CtrlRegNxt[CTRL_ISOLATE]),
                                                       .Q(CtrlReg[CTRL_ISOLATE]));

  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) I_CtrlReg_11     (.clkbus(clkbus_CtrlReg_Gater[`CLKBUS_SIZE-1:0]), .DG(CtrlReg_Gater_11),
                                                       .D(CtrlRegNxt[CTRL_SKIP_PHASE]),
                                                       .Q(CtrlReg[CTRL_SKIP_PHASE]));

  gfx_CORE_FLOP_DGATE_RESET #(.size(8)) I_CtrlReg_10to3  (.clkbus(clkbus_CtrlReg_Gater[`CLKBUS_SIZE-1:0]), .DG(CtrlReg_Gater),
                                                       .D(CtrlRegNxt[CTRL_DBG_EN:CTRL_PSM_AVG_EN]),
                                                       .Q(CtrlReg[CTRL_DBG_EN:CTRL_PSM_AVG_EN]));

  gfx_CORE_FLOP_DGATE_RESET #(.size(2)) I_CtrlReg_2_1    (.clkbus(clkbus_CtrlReg_Gater[`CLKBUS_SIZE-1:0]), .DG(CtrlReg_Gater_2_1),
                                                       .D({CtrlRegNxt[CTRL_PSM_ONE_SHOT] , CtrlRegNxt[CTRL_PSM_RST]}),
                                                       .Q({CtrlReg[CTRL_PSM_ONE_SHOT]    , CtrlReg[CTRL_PSM_RST]}));
  
  // JTAG controlled PsmEnable controls all CCLK clock gaters in the JTAG interface
  gfx_CORE_FLOP_DGATE_RESET #(.size(1)) I_CtrlReg_0      (.clkbus(clkbus_CtrlReg_Gater[`CLKBUS_SIZE-1:0]), .DG(CtrlReg_Gater),
                                                       .D(CtrlRegNxt[CTRL_PSM_ENABLE]),
                                                       .Q(CtrlReg[CTRL_PSM_ENABLE]));
  // CDCBUF for capture (CCLK->TCK)
  //   return zeroes for upper 3 reserved bits
  gfx_CORE_CDCENBUF #(.size(CTRL_LENGTH), .Cycles(`CDC_JTAG_X_DLY_CYCLES))
    I_CDC_CtrlReg (.A({3'b0, CtrlReg[CTRL_LENGTH-4:0]}),
                   .EN(JtgCapEn&(~JtgShift)),
                   .Z(CDC_CtrlReg[CTRL_LENGTH-1:0]),
                   .DSTCLK(JtgClk));

  // Shift
  always @* begin
    casez ({JtgShift, JtgCapEn})
      2'b1?: CtrlRegSerNxt[CTRL_LENGTH-1:0] = {JtgTdi, CtrlRegSer[CTRL_LENGTH-1:1]};
      2'b01: CtrlRegSerNxt[CTRL_LENGTH-1:0] = CDC_CtrlReg[CTRL_LENGTH-1:0];
      2'b00: CtrlRegSerNxt[CTRL_LENGTH-1:0] = CtrlRegSer[CTRL_LENGTH-1:0];
      // VCS coverage off
      default: CtrlRegSerNxt[CTRL_LENGTH-1:0] = {CTRL_LENGTH{1'bx}};
      // VCS coverage on
    endcase
  end // always @*

  gfx_CORE_PSM_IP_JTAGFLOP #(.size(CTRL_LENGTH)) I_CtrlRegSer (
                              .ASYNCCLK (Trst),
                              .ASYNCVAL ({CTRL_LENGTH{1'b0}}),
                              .CLK      (JtgClk),
                              .D        (CtrlRegSerNxt[CTRL_LENGTH-1:0]),
                              .Q        (CtrlRegSer[CTRL_LENGTH-1:0]));

// Logic below creates a pulse for AvfsGo -> AvfsGo_OneShot. AvfsGo_OneShot is triggered by
//   writing a "1" to AvfsGo JTAG register
//   writing AvfsNorm2 register via M&M 
// AvfsGoNxt -> AvfsGo_OneShot, AvfsGoEdge -> AvfsGoSync

  wire CtrlRegNxtAvfsGo;
  wire JtgAvfsGoWrEn;
  wire MmAvfsGoWrEn;

  assign JtgAvfsGoWrEn = JtgUpdateSyncdEdge ;
  assign CtrlRegNxtAvfsGo = CtrlRegNxt[CTRL_AVFS_GO];
  assign MmAvfsGoWrEn  = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM2) & MmWrEn;
  assign AvfsGo_Gater  = (JtgAvfsGoWrEn | MmAvfsGoWrEn | AvfsGo_OneShot);

  always @* begin
    casez ({JtgAvfsGoWrEn, MmAvfsGoWrEn, AvfsGo_OneShot})
      3'b1??  : AvfsGoNxt = CtrlRegNxtAvfsGo;  // Write AvfsGo from JTAG register
      3'b01?  : AvfsGoNxt = 1'b1;              // Write AvfsGo via M&M, inferred by write to AvfsNorm2 reg
      3'b001  : AvfsGoNxt = 1'b0;              // AvfsGo was 1'b1 in previous cycle, reset it to 0
      default : AvfsGoNxt = 1'b0;
    endcase // casez ({JtgAvfsGoWrEn, MmAvfsGoWrEn, CtrlRegAvfsGo})
  end // always @* begin

  gfx_CORE_FLOP_GATED_RESET #(.size(1)) I_AvfsGoOneShot   (.C(HostReg_CCLK), .GC(1'b0),
                                                      .G(AvfsGo_Gater),
                                                      .R(Reset), // may have to split this into CORE_CG+CORE_NOSCAN_FLOP_DGATE   
                                                      .SSE(ScanShiftEn), 
                                                      .D(AvfsGoNxt),
                                                      .Q(AvfsGo_OneShot));
                                 
// ------------------------------------------------------------
// Data Register
// Can either capture or update
// Measures into place-holder register. Updates straight to internal registers of CPA and PSM based on select address in CTRL Reg.
// ------------------------------------------------------------

  reg  [DATA_LENGTH-1:0] DataRegNxt;
  wire [DATA_LENGTH-1:0] DataReg;
  reg  [DATA_LENGTH-1:0] DataRegSerNxt;
  wire [DATA_LENGTH-1:0] DataRegSer;

  // CDCBUF for update (TCK->CCLK)
  wire [DATA_LENGTH-1:0] CDC_DataRegSer;
  wire [DATA_LENGTH-1:0] CDC_DataRegSer_Out;
  gfx_CORE_CDCENBUF #(.size(DATA_LENGTH), .Cycles(`CDC_JTAG_X_DLY_CYCLES))
    I_CDC_DataRegSer (.A(DataRegSer[DATA_LENGTH-1:0]),
                      .EN(JtgUpdateSyncdEdge),
                      .Z(CDC_DataRegSer[DATA_LENGTH-1:0]),
                      .DSTCLK(CCLK));

   // scan-only flop to allow DFT to isolate the JTAG outputs going into (scannable) CCLK paths
   // module CORE_SCAN_ONLY_FLOP #(parameter
   // 2:1 Mux .A(CDC_DataRegSer), .B(scan_only_din), .SEL(scanMode), .Z(CDC_DataRegSer_out)
   // add another wrapper with no CORE_CG so that we can share single CORE_CG between many scan_only flops

   wire [`CLKBUS_SIZE-1:0] clkbus_scan_only;

  gfx_CORE_CG I_cg_scan_only (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .C(CCLK), .G(1'b0), .R(1'b0), .SSE(ScanShiftEn));

  gfx_CORE_FLOP_DGATE I_DataRegTestIn (.clkbus(clkbus_scan_only[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(1'b0), .Q(DataRegTestIn));

   assign CDC_DataRegSer_Out[DATA_LENGTH-1:0] = LBistEn ? {DATA_LENGTH{DataRegTestIn}} : CDC_DataRegSer[DATA_LENGTH-1:0] ;

  // Mux for DataReg
  //   Jtag/Spmi read/write has higher priority that M&M read/write
  //   Jtag/Spmi read & write cannot occur at the same time.
  //   M&M read & write cannot occur at the same time. 
  //   assumes JtagRead & JtgUpdate are never asserted simultaneously
  always @* begin
    casez ({JtgUpdateSyncdEdge, JtgMeasureSyncdEdge, (|MmDatWrEn[7:0])})
      3'b10? : DataRegNxt[DATA_LENGTH-1:0] = CDC_DataRegSer_Out[DATA_LENGTH-1:0];                                           // Jtag/Spmi write data
      3'b01? : DataRegNxt[DATA_LENGTH-1:0] = RegisterInput[DATA_LENGTH-1:0];                                            // Jtag/Spmi read data
      3'b000 : DataRegNxt[DATA_LENGTH-1:0] = MmDataRegNxt[DATA_LENGTH-1:0];                                             // M&M read data
      3'b001 : case (MmDatWrEn[7:0])
                 8'h01   : DataRegNxt[DATA_LENGTH-1:0] = {56'b0, MmBusIn_M1[7:0]};                                      // M&M write data byte 0
                 8'h02   : DataRegNxt[DATA_LENGTH-1:0] = {DataReg[DATA_LENGTH-1:16], MmBusIn_M1[7:0], DataReg[7:0]};    // M&M write data byte 1
                 8'h04   : DataRegNxt[DATA_LENGTH-1:0] = {DataReg[DATA_LENGTH-1:24], MmBusIn_M1[7:0], DataReg[15:0]};   // M&M write data byte 2
                 8'h08   : DataRegNxt[DATA_LENGTH-1:0] = {DataReg[DATA_LENGTH-1:32], MmBusIn_M1[7:0], DataReg[23:0]};   // M&M write data byte 3
                 8'h10   : DataRegNxt[DATA_LENGTH-1:0] = {DataReg[DATA_LENGTH-1:40], MmBusIn_M1[7:0], DataReg[31:0]};   // M&M write data byte 4
                 8'h20   : DataRegNxt[DATA_LENGTH-1:0] = {DataReg[DATA_LENGTH-1:48], MmBusIn_M1[7:0], DataReg[39:0]};   // M&M write data byte 5
                 8'h40   : DataRegNxt[DATA_LENGTH-1:0] = {DataReg[DATA_LENGTH-1:56], MmBusIn_M1[7:0], DataReg[47:0]};   // M&M write data byte 6
                 8'h80   : DataRegNxt[DATA_LENGTH-1:0] = {MmBusIn_M1[7:0], DataReg[55:0]};                              // M&M write data byte 7
                 default : DataRegNxt[DATA_LENGTH-1:0] = {DATA_LENGTH{1'bx}};                                           // should not occur,
               endcase                                                                                                  //   only added to prevent synthesis from inferring a latch


      default : DataRegNxt[DATA_LENGTH-1:0] = {DATA_LENGTH{1'bx}};                                                      // if JtgUpdate & JtgMeasure asserted simultaneously return x's
    endcase
     
  end

  // Measure function achieved via clock gater enabled on
  //   rising edge of JtgMeasureSyncd_CCLK
  //   rising edge of JtgUpdateSyncd_CCLK
  //   Jtag/SPMI reset
  //   MmRdEn to enable for M&M read
  wire   DataReg_Gater;
  assign DataReg_Gater = JtgMeasureSyncdEdge | JtgUpdateSyncdEdge | MmRdEn | (|MmDatWrEn[7:0]);
  gfx_CORE_FLOP_GATED_RESET #(.size(DATA_LENGTH)) I_DataReg (.C(HostReg_CCLK), .G(DataReg_Gater), .GC(1'b0), .D(DataRegNxt[DATA_LENGTH-1:0]), .R(Reset), .SSE(ScanShiftEn), .Q(DataReg[DATA_LENGTH-1:0])); // DataReg & CtrlReg are "shared" between JTAG & MM

  // CDCBUF for capture (CCLK->TCK)
  wire [DATA_LENGTH-1:0] CDC_DataReg;
  gfx_CORE_CDCENBUF #(.size(DATA_LENGTH), .Cycles(`CDC_JTAG_X_DLY_CYCLES))
    I_CDC_DataReg (.A(DataReg[DATA_LENGTH-1:0]),
                   .EN(JtgCapEn&(~JtgShift)),
                   .Z(CDC_DataReg[DATA_LENGTH-1:0]),
                   .DSTCLK(JtgClk));
  // Shift
  always @* begin
    casez ({JtgShift, JtgCapEn})
      2'b1?: DataRegSerNxt[DATA_LENGTH-1:0] = {CtrlRegSer[0], DataRegSer[DATA_LENGTH-1:1]};
      2'b01: DataRegSerNxt[DATA_LENGTH-1:0] = CDC_DataReg[DATA_LENGTH-1:0];
      2'b00: DataRegSerNxt[DATA_LENGTH-1:0] = DataRegSer[DATA_LENGTH-1:0];
      // VCS coverage off
      default: DataRegSerNxt[DATA_LENGTH-1:0] = {DATA_LENGTH{1'bx}};
      // VCS coverage on
    endcase
  end // always @*

  gfx_CORE_PSM_IP_JTAGFLOP #(.size(DATA_LENGTH)) I_DataRegSer (
                              .ASYNCCLK (Trst),
                              .ASYNCVAL ({DATA_LENGTH{1'b0}}),
                              .CLK      (JtgClk),
                              .D        (DataRegSerNxt[DATA_LENGTH-1:0]),
                              .Q        (DataRegSer[DATA_LENGTH-1:0]));


  // Register R/W logic
  //   RegisterReadEnLo asserted for all AVFS/PSM regs for JTAG/SPMI/M&M
  //   RegisterReadEnHi only asserted for M&M 16-byte read (for AvfsBucket0)
  assign RegisterAddress[3:0] = CtrlReg[CTRL_REG_ADDR_H:CTRL_REG_ADDR_L];
  assign RegisterWriteEn = (CtrlReg[CTRL_REG_WRITE_EN] & JtgUpdateSyncdEdge_d1) | MmWrEn;
  assign RegisterReadEnLo = JtgMeasureSyncdEdge | MmRdEnLo;
  assign RegisterReadEnHi = MmRdEnHi;

  // Register output only when address is valid and RegisterWriteEn is valid
  assign RegisterOutput[DATA_LENGTH-1:0] = DataReg[DATA_LENGTH-1:0];

  // JTAG PSM controls
  assign JTAG_Psm_AvgEn = CtrlReg[CTRL_PSM_AVG_EN];
  assign JTAG_Psm_Enable = CtrlReg[CTRL_PSM_ENABLE];
  assign JTAG_Psm_Rst = CtrlReg[CTRL_PSM_RST];
  assign JTAG_Psm_OneShot = CtrlReg[CTRL_PSM_ONE_SHOT];
  assign JTAG_DbgEn = CtrlReg[CTRL_DBG_EN];

  wire PreJtgTdo;
  assign PreJtgTdo = DataRegSer[0];

generate
  if (NEG_JTAG_TDO == 1) begin: add_negedge_jtag_flop
  gfx_CORE_PSM_IP_JTAGFLOP_B  I_Neg_JtgTdo (.ASYNCCLK(Trst), .ASYNCVAL(1'b0), .CLK(JtgClk), .D(PreJtgTdo), .Q(JtgTdo));
  end
  else begin: no_extra_flop_needed
     assign JtgTdo = PreJtgTdo;
  end
endgenerate

  //assign TrstSyncd_CCLK = CtrlReg[CTRL_PSM_RST]; // Same as JTAG_Psm_Rst, set by software
  assign TrstSyncd_CCLK = 1'b0;

`ifdef AVFS_ZAK

  wire [1:0]             MmSz;
  wire [1:0]             MmLclSz_M2_dgate;
  wire                   MmRd;
  wire                   MmRdNxt_M2_dgate;
  wire                   MmWr;
  wire                   MmWrNxt_M2_dgate;
  wire                   MmWrEnNxt_m1;
  wire                   MmRdNxt_M2;
  wire                   MmWrNxt_M2;
  reg                    MmAddrAvfs_M2;
  reg  [DATA_LENGTH-1:0] MmDataReg;
  wire                   FgenMmAddrEn;
  wire                   FgenMmRdEn;
  wire                   FgenMmWrEn;
//wire                   FgenMmActiveEn;
  wire                   MmStop;
  wire                   MmStateUpdate;
  wire                   MmWrEn_m1;
  wire                   MmWrEnNxt_m1_dgate;
  wire                   MmWrEn_m1_dgate;
  wire                   MmRdEn_d1;
  wire                   MmRdEn_dgate;
  wire [`MM_STATE]       MmState_d1;

  assign JtgAbortMm = (JtgMeasureSyncdEdge | JtgUpdateSyncdEdge) ; // IEEE1500 transaction detected: abort any Mm transactions in progress

  // Decode whether M&M address is to AVFS/PSM register
  //   MmBusIn_M1[7:0] contains the 8b reg addr in the M2 cycle
  always @* begin
    case (MmBusIn_M1[7:0])
       `MM_REG_PSM_PARAM_0,
       `MM_REG_PSM_DATA_0,
       `MM_REG_AVFS_CTL1_0,
       `MM_REG_AVFS_CTL2_0,
       `MM_REG_AVFS_BUCKET0_0,
       `MM_REG_AVFS_BUCKET1_0,
       `MM_REG_AVFS_VALID_0,
       `MM_REG_AVFS_NORM0_0,
       `MM_REG_AVFS_NORM1_0,
       `MM_REG_AVFS_NORM2_0 : MmAddrAvfs_M2 = 1'b1;
       default              : MmAddrAvfs_M2 = 1'b0;
    endcase
  end

  // M&M address enable to jtag WDR Ctrl register
  //   don't qualify with MmAddrAvfs_M2 to allow MmRd/MmWr flops to reset
  //   for reads to other regs at same stop
  assign  MmAddrEn = (MmLclWrEn_M2 | MmLclRdEn_M2);

  // Qualify M&M read/write indications with AVFS/PSM addr
  assign  MmRdNxt_M2 = MmLclRdEn_M2 & MmAddrAvfs_M2;
  assign  MmWrNxt_M2 = MmLclWrEn_M2 & MmAddrAvfs_M2;

  // M&M write data enables to jtag WDR Data register
  //   use MmWrEn_m1 which asserts during M8 for 32b writes to turn on MmDatWrEn[3]
  //    and during M11 for 64b writes to turn on MmDatWrEn[7]
  //   since using delayed version of MmState, decode state from 1-clock earlier
  //     e.g M10 instead of M11
  assign  MmDatWrEn[7:0] = {(MmWrEn_m1 & (MmSz[1:0] == `MM_SZ_64)),
                             ( {7{MmWr}} & 
                               { ( MmState_d1[`MM_STATE] == `MM_STATE_M10),
                                 ( MmState_d1[`MM_STATE] == `MM_STATE_M9),
                                 ( MmState_d1[`MM_STATE] == `MM_STATE_M8),
                                 ((MmState_d1[`MM_STATE] == `MM_STATE_M7) |
                                  (MmWrEn_m1 & (MmSz[1:0] == `MM_SZ_32))
                                 ),
                                 ( MmState_d1[`MM_STATE] == `MM_STATE_M6),
                                 ( MmState_d1[`MM_STATE] == `MM_STATE_M5),
                                 ( MmState_d1[`MM_STATE] == `MM_STATE_M4)
                               }
                             )
                           }; 

  // MmRdEnLo asserted in M3-M10 cycle for lower 8 bytes of 4-byte, 8-byte or 16-byte read
  // MmRdEnHi asserted in M11-M18 cycle for upper 8 bytes of 16-byte read (only for AvfsBucket0)
  //   -> asserts RegisterReadEnLo in M3-M10, RegisterReadEnHi in M11-M18
  //   -> asserts SelAvfs* or SelPsm* register select to AVFS_CPA in M3-M18
  //   -> AVFS_CPA returns RegisterInput[63:0] in M3-M10
  //   -> mux to MmDataOutNxt_M3[7:0] in M3-M10, upper 8 bytes in M11-M18
  //   -> mux from lower 8 bytes to DataReg[7:0] in M3-M10
  //      mux from upper 8 bytes to DataReg[7:0] in M11-M18
  //   -> output MmDatOut_M4[7:0] from DataReg[7:0] & route to M&M stop in M4-M19
  //   since using delayed version of MmState, decode state from 1-clock earlier
  //     e.g M2 instead of M3
  //   use assign for MmRdEnLo(Hi) to allow non-ZAK code to assign RHS to zeroes
  //   non-ZAK linter complains if assigning RHS to zeroes in an always block
  assign MmRdEnLo = MmRd & 
                         ( (MmState_d1[`MM_STATE] == `MM_STATE_M2)  | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M3)  | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M4)  | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M5)  | 
                           ((MmState_d1[`MM_STATE] == `MM_STATE_M6) & (MmSz[1:0] == `MM_SZ_64 | MmSz[1:0] == `MM_SZ_128)) |
                           ((MmState_d1[`MM_STATE] == `MM_STATE_M7) & (MmSz[1:0] == `MM_SZ_64 | MmSz[1:0] == `MM_SZ_128)) |
                           ((MmState_d1[`MM_STATE] == `MM_STATE_M8) & (MmSz[1:0] == `MM_SZ_64 | MmSz[1:0] == `MM_SZ_128)) |
                           ((MmState_d1[`MM_STATE] == `MM_STATE_M9) & (MmSz[1:0] == `MM_SZ_64 | MmSz[1:0] == `MM_SZ_128))
                         );
  assign MmRdEnHi = MmRd & (MmSz[1:0] == `MM_SZ_128) & 
                         ( (MmState_d1[`MM_STATE] == `MM_STATE_M10) | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M11) | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M12) | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M13) | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M14) | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M15) | 
                           (MmState_d1[`MM_STATE] == `MM_STATE_M16) |
                           (MmState_d1[`MM_STATE] == `MM_STATE_M17)
                         );

  // AVFS/PSM reg -> M&M read data mux
  //   8-to-1 mux to select AVFS/PSM reg byte to be pushed out into LS byte of DataReg
  //   need to decode both Mi & Mi+8 states for each byte to support 128-bit AvfsBucket0 access
  //   since using delayed version of MmState, decode state from 1-clock earlier
  //     e.g M2 instead of M3
  always @* begin
    case ({MmRdEn, MmState_d1[`MM_STATE]})
      {1'b1, `MM_STATE_M2},
      {1'b1, `MM_STATE_M10} : MmDataReg[DATA_LENGTH-1:0] = {56'b0, RegisterInput[ 7:0]};
      {1'b1, `MM_STATE_M3},
      {1'b1, `MM_STATE_M11} : MmDataReg[DATA_LENGTH-1:0] = {56'b0, RegisterInput[15:8]};
      {1'b1, `MM_STATE_M4},
      {1'b1, `MM_STATE_M12} : MmDataReg[DATA_LENGTH-1:0] = {56'b0, RegisterInput[23:16]};
      {1'b1, `MM_STATE_M5},
      {1'b1, `MM_STATE_M13} : MmDataReg[DATA_LENGTH-1:0] = {56'b0, RegisterInput[31:24]};
      {1'b1, `MM_STATE_M6},
      {1'b1, `MM_STATE_M14} : MmDataReg[DATA_LENGTH-1:0] = {56'b0, RegisterInput[39:32]};
      {1'b1, `MM_STATE_M7},
      {1'b1, `MM_STATE_M15} : MmDataReg[DATA_LENGTH-1:0] = {56'b0, RegisterInput[47:40]};
      {1'b1, `MM_STATE_M8},
      {1'b1, `MM_STATE_M16} : MmDataReg[DATA_LENGTH-1:0] = {56'b0, RegisterInput[55:48]};
      {1'b1, `MM_STATE_M9},
      {1'b1, `MM_STATE_M17} : MmDataReg[DATA_LENGTH-1:0] = {56'b0, RegisterInput[63:56]};
      default               : MmDataReg[DATA_LENGTH-1:0] = {DATA_LENGTH{1'bx}};                             // should not occur,
    endcase
  end

  // use assign for MmRdEnLo(Hi) to allow non-ZAK code to assign RHS to zeroes
  // non-ZAK linter complains if assigning RHS to zeroes in an always block
  assign MmDataRegNxt[DATA_LENGTH-1:0] = MmDataReg[DATA_LENGTH-1:0];

  // MmRdEn(Lo|Hi)  - asserted in M3-M18 to turn on RegisterReadEn to AVFS/PSM &
  //                  gate write of return reg data to DataReg for M&M read
  // MmRdEn_d1      - asserted in M4-M19 to qualify DataReg data to M&M stop for M&M read
  // MmWrEn_m1      - asserted in M8(M11) to gate write of data from M&M stop into DataReg for M&M write
  // MmWrEn         - asserted in M9(M12) to turn on RegisterWriteEn to AVFS/PSM to write data
  // read into DataReg.
  assign MmRdEn = MmRdEnLo | MmRdEnHi;

  // MmWrEnNxt_m1 asserted on last byte of data transfer
  //   byte 3 for dwords, byte 7 for qwords
  assign MmWrEnNxt_m1 = MmWr &
                       (
                         (MmSz[1:0] == `MM_SZ_32) & (MmState_d1[`MM_STATE] == `MM_STATE_M6) | 
                         (MmSz[1:0] == `MM_SZ_64) & (MmState_d1[`MM_STATE] == `MM_STATE_M10)
                        );

  // Functional fine gaters for flops

  // "MmAddrEn" pulses to flop Rd/Wr/Sz indications for the whole transaction; reset Rd/Wr/Sz when [1] "MmWrEn" pulses to store accumulated value into register or [2] "MmRdEn" de-asserts at end of a read
  assign FgenMmAddrEn = MmAddrEn | MmWrEn | (MmRd & ~MmRdEn); //NOTE: "~MmRdEn" might introduce a static timing challenge; consider a direct MmState decode

  // Enable for MmWrEn_m1 & MmWrEn which is needed to clear out flops after
  //   MmWrEnNxt_m1 changes from 1 -> 0
  assign FgenMmWrEn = (MmWrEnNxt_m1 | MmWrEn_m1 | MmWrEn) & MmStateUpdate;

  // Enable for MmRdEn_d1 which is needed to clear out flops after MmRdEn changes from 1 -> 0
  assign FgenMmRdEn = MmRdEn | MmRdEn_d1;

  // Define last cycle for MM related logic (keep running 1 cycle past MmState=0 to allow downstream write to register flops)
  assign MmStop     = (MmState_d1[`MM_STATE]==`MM_STATE_M0);

  // Enable for MmState_d1
  //   it does not assert when MmState == M0 which means MmState_d1 will transition directly
  //   from M0 -> M2. There is no decode logic for MmState == M1.
//  assign FgenMmActiveEn = (MmState[`MM_STATE] != `MM_STATE_M0) |
//                         FgenMmAddrEn |
//                         FgenMmRdEn |
//                         FgenMmWrEn; // collapsing into single cg to save power (really should consider ccg_nf and cgu)

  wire [`CLKBUS_SIZE-1:0] RCLK_MmActive_AR ;

  // fine gaters for M&M Rd/Wr/Sz/WrEn
  // vscan off cellprop
  //cgu                           I_RCLK_MmActive_AR (.gclk(RCLK_MmActive_AR), .clk(HostReg_CCLK));
  gfx_CORE_CGU I_Mm_cgu (.clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .C(HostReg_CCLK), .R(Reset), .SSE(ScanShiftEn));
  // vscan on cellprop

  assign MmRdNxt_M2_dgate      = FgenMmAddrEn   ? MmRdNxt_M2                   : (MmRd       & ~JtgAbortMm & ~MmStop);
  assign MmWrNxt_M2_dgate      = FgenMmAddrEn   ? MmWrNxt_M2                   : (MmWr       & ~JtgAbortMm & ~MmStop);
//  assign MmActiveEn_dgate      = (MmState[`MM_STATE]==`MM_STATE_M1) |
  assign MmActiveEn_dgate      = ((MmState[`MM_STATE]==`MM_STATE_M2) ? (MmRdNxt_M2 | MmWrNxt_M2) : (MmActiveEn & ~JtgAbortMm & ~MmStop));
  assign MmLclSz_M2_dgate[1:0] = FgenMmAddrEn   ? MmLclSz_M2[1:0]              : MmSz[1:0];
  assign MmWrEnNxt_m1_dgate    = FgenMmWrEn     ? (MmWrEnNxt_m1 & ~JtgAbortMm) : (MmWrEn_m1  & ~JtgAbortMm & ~MmStop);
  assign MmWrEn_m1_dgate       = FgenMmWrEn     ? (MmWrEn_m1    & ~JtgAbortMm) : (MmWrEn     & ~JtgAbortMm & ~MmStop);
  assign MmRdEn_dgate          = FgenMmRdEn     ? MmRdEn                       : (MmRdEn_d1  & ~JtgAbortMm & ~MmStop);

  // Flops for rd/wr/size/state
  //    flop signals from M&M stop before using as they may have long route from stop

   //CORE_FLOP_DGATE  #(.size(50)) I_AvfsNorm2 (.clkbus(clkbus_CgenAvfsNorm2[`CLKBUS_SIZE-1:0]), .DG(1'b1), .D(AvfsSrbWrDat_4[49:0]), .Q(AvfsNorm2[49:0]));

  gfx_CORE_FLOP_DGATE #(.size(1))              I_MmRd       ( .Q(MmRd),                  .clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .D(MmRdNxt_M2_dgate),         .DG(1'b1));
  gfx_CORE_FLOP_DGATE #(.size(1))              I_MmWr       ( .Q(MmWr),                  .clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .D(MmWrNxt_M2_dgate),         .DG(1'b1));
  gfx_CORE_FLOP_DGATE #(.size(1))              I_MmActiveEn ( .Q(MmActiveEn),            .clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .D(MmActiveEn_dgate),         .DG(1'b1));
  gfx_CORE_FLOP_DGATE #(.size(2))              I_MmSz       ( .Q(MmSz[1:0]),             .clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .D(MmLclSz_M2_dgate[1:0]),    .DG(1'b1));
  gfx_CORE_FLOP_DGATE #(.size(`MM_STATE_WIDTH))I_MmState_d1 ( .Q(MmState_d1[`MM_STATE]), .clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .D(MmState[`MM_STATE]),       .DG(1'b1)); // no data gating required
  gfx_CORE_FLOP_DGATE #(.size(1))              I_MmWrEn_m1  ( .Q(MmWrEn_m1),             .clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .D(MmWrEnNxt_m1_dgate),       .DG(1'b1));
  gfx_CORE_FLOP_DGATE #(.size(1))              I_MmWrEn     ( .Q(MmWrEn),                .clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .D(MmWrEn_m1_dgate),          .DG(1'b1));
  gfx_CORE_FLOP_DGATE #(.size(1))              I_MmRdEn_d1  ( .Q(MmRdEn_d1),             .clkbus(RCLK_MmActive_AR[`CLKBUS_SIZE-1:0]), .D(MmRdEn_dgate),             .DG(1'b1));

  // pulse to identify state advancement so MM interface supports either back-to-back cycle byte updates, or writes via separate, disperse byte writes
  assign MmStateUpdate = MmState[`MM_STATE] != MmState_d1[`MM_STATE];  

  // return M&M read data from DataReg in M4 - M11
  //   qualify with MmRdEn because data gets or'd with data from other
  //   sources before returning to M&M stop
  assign MmDatOut_M4[`MM_DATA] = DataReg[`MM_DATA] & {`MM_DATA_WIDTH{MmRdEn_d1}};

`else
  assign MmBusIn_M1[7:0] = 8'b0;
  assign MmAddrEn = 1'b0;
  assign MmDatWrEn[7:0] = 8'b0;
  assign MmWrEn = 1'b0;
  assign MmRdEnLo = 1'b0;
  assign MmRdEnHi = 1'b0;
  assign MmRdEn = 1'b0;
  assign MmDataRegNxt[DATA_LENGTH-1:0] = {DATA_LENGTH{1'b0}};
  assign JtgAbortMm = 1'b0;
`endif    // AVFS_ZAK

endmodule
                                 
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// 
// 
//
// Top Level AVFS and PSM wrapper with muxed SPMI/p1500 interface and M&M Bus
//
//
//
// 
// 
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------


module gfx_AVFS_PSM_WRAP #(parameter
  SYNC_TC_CYCLES    = 3,    
  CDC_TST_PROP_DLY  = 5000,
  DBG_LENGTH        = 16,
  DBG_CFG_LENGTH    = 3,
  MCP_CYCLES_MAPSMRO = 2, // PSMRO macro output pin timing can't pass 1-cycle timign high CCLK frequencies, No one should be using this module now, but keeping it the same for consistency between AVFS3_TOP adn AVFS_PSM_WRAP. or is it better to keep it 0?
  MCP_CYCLES_AVGPSMREG = 3,
  PSM_CURPSM_LENGTH = 14
) (
//Scan options
//1. LSSD Scan with scan reset (Might not be needed - no product)
//2. MUXD with scan based reset (CM, ZAK)
//3. MUXD with async reset (KK)
  input                      ScanShiftEn,  // Scan shift Enable
  input                      CGater01_CG0, // Coarse Gater
  input                      Reset,        // reset signal to reset flops for SOCs which don't use scan-shift-reset
  input                      PwrMonEnX,    // mux between SPMI (0) and p1500 path(1)
`ifndef AVFS_ZAK
  // ZAK does not support Spmi
  input                      SpmiClk,      // SPMI
  input                      SpmiTdi,      // SPMI
  input                      SpmiShift,    // SPMI
  input                      SpmiUpdate,   // SPMI
  input                      SpmiCapEn,    // SPMI
  input                      SpmiPathEn,   // SPMI
  input                      SpmiTrst,     // SPMI
  output wire                SpmiTdo,      // SPMI
  input                      SpmiMeas,     // SPMI
`endif
  input                      WRCK,         // p1500
  input                      WSI,          // p1500
  input                      ShiftWR,      // p1500
  input                      UpdateWR,     // p1500
  input                      CaptureWR,    // p1500
  input                      WDREn,        // p1500
  input                      WRSTN,        // p1500
  input                      MeasureWR,    // p1500 (does not exist in JTAG protocol - can use Idle signal)
  output wire                WRO,          // p1500
`ifndef AVFS_ZAK
  input                      Host_Clk,     // gated clk to AVFS_CPA/GATEPACK/WIREPACK
  input                      PsmClk,       // free running clk to PSM for AC scan test
`else
  input                      CCLK,             // ZAK uses mesh clock
  input                      TstCcgEn,         // ZAK test coarse gater enable (non-scannable flops would use 1'b1)
  input                      Cgen_UnitActive,  // ZAK coarse gater enable
  input                      RCLK_Free_AR,     // ZAK uses free-running clock for coarse gater flops
`endif
  input                      LBistEn,
`ifdef GFX_DROOP_DETECT
  output wire [PSM_CURPSM_LENGTH-1:0] PsmOut, // adding new port to direct PSM count out through AVFS3 for GNL purposes
`endif
  input                      PsmEn,
  input                      PsmAvfs_Gater,
  input                      Avfs_ScanMode,
  output wire                Psm_Alarm_Flag,
  input                      SkipPhase,    // from clock stretcher module - needs to be conditionally gated at the PSM when IP is updated
  input                      Isolate,      // indicates if tile containing AVFS module is being power-gated
  input [DBG_CFG_LENGTH-1:0] DbgBusSel,
  output wire [DBG_LENGTH-1:0] DbgBus

`ifdef AVFS_ZAK                            // M&M interface
  ,input       [`MM_DATA]    MmBusIn_M1,   //   8-bit M&M bus input contains local reg addr in M2 and write data in M4
  input                      MmLclRdEn_M2, //   local register read indication
  input                      MmLclWrEn_M2, //   local register write indication
  input       [1:0]          MmLclSz_M2,   //  local register size indication
  input       [`MM_STATE]    MmState,      //   M&M state for read/write byte select
  output wire [`MM_DATA]     MmDatOut_M4   //   8-bit data back to M&M bus for read
`endif

);
  wire             JTAG_ClkEn_ASYNC;
  wire             JTAG_ClkEn_CCLK;
  wire             MmActiveEn;

  wire             TrstSyncd;
  wire             CgenAvfsEn;
  wire [49:0]      AvfsNearMiss;
  wire [13:0]      PsmDeltaOut;
  wire             PsmDivClkGater;

  wire [DBG_LENGTH-1:0] DbgBus_int;
  wire [PSM_CURPSM_LENGTH-1:0] dbgCurPsm;
  wire [PSM_CURPSM_LENGTH-1:0] dbgAvgPsm;
  wire [PSM_CURPSM_LENGTH-1:0] dbgThreshold;
  wire [PSM_CURPSM_LENGTH-1:0] dbgMinMax;
  wire dbgAlarm;
   
  wire             AvfsGo;
  wire             AvfsPathEn;
  wire             AvfsTestData;
  wire [1:0]       AvfsTestMode;
  wire [149:0]     AvfsPathDelay;
  wire [2:0]       AvfsGlobalPathDelay;
  wire             AvfsAgeHigh;
  wire             dbgEn;
  wire [63:0]      SrbLclAvfsRdDat;
  wire             SrbLclSelAvfsCtl1;
  wire             SrbLclSelAvfsCtl2;
  wire             SrbLclSelAvfsBucket0;
  wire             SrbLclSelAvfsBucket1;
  wire             SrbLclSelAvfsValid;
   wire            SrbLclSelAvfsNorm0;
   wire            SrbLclSelAvfsNorm1;
   wire            SrbLclSelAvfsNorm2;
   wire [63:0]     PSM_RdDat;
   wire            MM_Psm_Enable;
   wire            SelPsmParam;
   wire            SelPsmData;
   wire            JTAG_Psm_Enable;
   wire            JTAG_Psm_AvgEn;
   wire            JTAG_Psm_Rst;
   wire            JTAG_Psm_Rst_Reset;
   wire            JTAG_Psm_OneShot;
   wire            Psm_Rst;
   
  // Register access decoder and signals
  wire RegisterRWLo;
  wire RegisterRWHi;
  wire RegisterReadEnLo;
  wire RegisterReadEnHi;
  wire RegisterWriteEn;
  wire [3:0] RegisterAddress;
  wire [63:0] RegisterWrDat;
  wire GlobalSelPsm;
  wire GlobalSelCPA;

  assign RegisterRWLo = RegisterReadEnLo | RegisterWriteEn;
  assign RegisterRWHi = RegisterReadEnHi | RegisterWriteEn;

  //decoder
  assign SelPsmParam = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_PARAM_ADDR) & RegisterRWLo;
  assign SelPsmData = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_DATA_ADDR) & RegisterRWLo;
  assign SrbLclSelAvfsCtl1 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_CTL1) & RegisterRWLo;
  assign SrbLclSelAvfsCtl2 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_CTL2) & RegisterRWLo;
  assign SrbLclSelAvfsValid = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_VALID) & RegisterRWLo;
  assign SrbLclSelAvfsNorm0 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM0) & RegisterRWLo;
  assign SrbLclSelAvfsNorm1 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM1) & RegisterRWLo;
  assign SrbLclSelAvfsNorm2 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM2) & RegisterRWLo;
   
  // Support 16-byte read of both AvfsBucket0 & AvfsBucket1 with one M&M transaction
  //   SrbLclSelAvfsBucket0 asserted for lower 8 bytes
  //   SrbLclSelAvfsBucket1 asserted for upper 8 bytes
  assign SrbLclSelAvfsBucket0 = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET0) & RegisterRWLo;
  assign SrbLclSelAvfsBucket1 = ((RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET1) & RegisterRWLo) |
                                ((RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET0) & RegisterRWHi);

  assign GlobalSelPsm = (RegisterAddress[3:0] == `AVFS_WRAP_PSM_PARAM_ADDR) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_PSM_DATA_ADDR);
  assign GlobalSelCPA = (RegisterAddress[3:0] == `AVFS_WRAP_CPA_CTL1) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_CTL2) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_VALID) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM0) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM1) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_NORM2) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET0) |
                        (RegisterAddress[3:0] == `AVFS_WRAP_CPA_BUCKET1);

`ifdef AVFS_ZAK
// Force Spmi inputs to 1'b0 and declare output as unused
  wire           SpmiClk;
  wire           SpmiTdi;
  wire           SpmiShift;
  wire           SpmiUpdate;
  wire           SpmiCapEn;
  wire           SpmiPathEn;
  wire           SpmiTrst;
  wire           SpmiMeas;

  assign         SpmiClk = 1'b0;
  assign         SpmiTdi = 1'b0;
  assign         SpmiShift = 1'b0;
  assign         SpmiUpdate = 1'b0;
  assign         SpmiCapEn = 1'b0;
  assign         SpmiPathEn = 1'b0;
  assign         SpmiTrst = 1'b0;
  assign         SpmiMeas = 1'b0;

  wire           PsmClk;
  wire           Host_Clk;
  wire           Host_CDC_Clk;
  wire           Psm_ClkEn;
  wire           HostJtag_CDC_ClkEn;
  wire           HostJtag_ClkEn;
 // implement coarse gater for ZAK
/*
  wire                      CCLK;             // ZAK uses mesh clock
  wire                      TstCcgEn;         // ZAK test coarse gater enable (non-scannable flops would use 1'b1)
  wire                      Cgen_UnitActive;  // ZAK coarse gater enable
  wire                      RCLK_Free_AR;     // ZAK uses free-running clock for coarse gater flops
  wire                      KCLK_Free_AR;     // ZAK uses free-running clock for coarse gater flops

  assign CCLK = Host_Clk;
  assign TstCcgEn = 1'b1;
  gfx_ccgu I_KCLK_Free_AR (.gclk(KCLK_Free_AR), .TstCcgEn(TstCcgEn), .clk(CCLK));
  gfx_cgu I_RCLK_Free_AR (.gclk(RCLK_Free_AR), .clk(KCLK_Free_AR));
  assign Cgen_UnitActive = 1'b1;
*/
 
//!unused Cgen_UnitActive

   //CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY))
   //  I_CDC_JTAG_ClkEn (.A(JTAG_ClkEn_ASYNC),
   //                 .Z(JTAG_ClkEn_CDC),
   //                 .EN(1'b1),
   //                 .DSTCLK(CCLK) );   

  //assign         HostJtag_CDC_ClkEn = JTAG_ClkEn_CDC | JTAG_ClkEn_CCLK;  // async assertion and synchronous de-assertion
  assign         HostJtag_ClkEn = JTAG_ClkEn_CCLK;  // synchronous-only assertion and de-assertion
  assign         HostReg_ClkEn  = JTAG_ClkEn_CCLK | (MmState[`MM_STATE]==`MM_STATE_M2) | MmActiveEn;  // shallow enable term to feed CCG_NF (i.e. fast wakeup for MM transactions)
//assign         HostCpa_ClkEn  = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | LBistEn;
//assign         Psm_ClkEn      = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | JTAG_Psm_Rst_Reset;
  assign         HostCpa_ClkEn  = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable | LBistEn;            //Term HostReg_ClkEn is safe to be removed, as CPA wakes up with Psm_Enable
  assign         Psm_ClkEn      = Psm_Rst | MM_Psm_Enable | JTAG_Psm_Enable; //Terms HostReg_ClkEn and JTAG_Psm_Rst_Reset are safe to be removed, as PSM wakes up with Psm_Enable

  // vscan off cellprop
  //ccg    I_KCLK_HostJtag_CDC_Clk_AR (.gclk(Host_CDC_Clk), .clk(CCLK), .fclk(RCLK_Free_AR), .en(HostJtag_CDC_ClkEn), .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccgu  I_KCLK_Free_AR         (.gclk(Host_CDC_Clk), .clk(CCLK), .TstCcgEn(TstCcgEn)); // Removing gating from Host_CDC_Clk, this really should be called Free clock, runs just one synchronizer in AvfsWrapJtgInterface
  gfx_ccg    I_KCLK_HostJtag_Clk_AR (.gclk(Host_Clk),     .clk(CCLK), .fclk(RCLK_Free_AR), .en(HostJtag_ClkEn), .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccg_nf I_KCLK_HostReg_Clk_AR  (.gclk(HostReg_Clk),  .clk(CCLK),                      .en(HostReg_ClkEn),  .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccg    I_KCLK_Host_Clk_AR     (.gclk(HostCpa_Clk),  .clk(CCLK), .fclk(RCLK_Free_AR), .en(HostCpa_ClkEn),  .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  gfx_ccg    I_KCLK_PsmClk_AR       (.gclk(PsmClk),       .clk(CCLK), .fclk(RCLK_Free_AR), .en(Psm_ClkEn),      .TstCcgEn(TstCcgEn), .SSE(ScanShiftEn));
  // vscan on cellprop

  assign PsmReg_Clk   = HostReg_Clk;  // TODO:  will this cause ATPG problems?  PSM is non-scan, but "HostReg_Clk" also drives scannable flops
`else
  assign HostCpa_Clk  = Host_Clk;  // needed to allow non-AVFS_ZAK compile
  assign PsmReg_Clk   = PsmClk;    // needed to allow non-AVFS_ZAK compile
  assign MmActiveEn   = 1'b0;      // needed to allow non-AVFS_ZAK compile
//!unused JTAG_ClkEn_ASYNC
//!unused JTAG_ClkEn_CCLK
`endif

  gfx_AvfsWrapJtgInterface #(.SYNC_TC_CYCLES(SYNC_TC_CYCLES)) I_AvfsWrapJtgInterface  (
                                            .RegisterOutput(RegisterWrDat[63:0]),
                                            .TrstSyncd_CCLK(TrstSyncd),
                                            .JTAG_ClkEn_ASYNC(JTAG_ClkEn_ASYNC),
                                            .JTAG_ClkEn_CCLK(JTAG_ClkEn_CCLK),
                                            .RegisterAddress(RegisterAddress[3:0]),
                                            .RegisterWriteEn(RegisterWriteEn),
                                            .RegisterReadEnLo(RegisterReadEnLo),
                                            .RegisterReadEnHi(RegisterReadEnHi),
                                            .JTAG_Psm_Enable(JTAG_Psm_Enable),
                                            .JTAG_Psm_AvgEn(JTAG_Psm_AvgEn),
                                            .JTAG_Psm_Rst(JTAG_Psm_Rst),
                                            .JTAG_Psm_OneShot(JTAG_Psm_OneShot),
                                            .AvfsGo_OneShot(AvfsGo),
                                            .JTAG_DbgEn(dbgEn),
                                            .RegisterInput(SrbLclAvfsRdDat[63:0] | PSM_RdDat[63:0]),
                                            .PwrMonEnX(PwrMonEnX),
                                            .SpmiClk(SpmiClk),
                                            .SpmiTdi(SpmiTdi),
                                            .SpmiShift(SpmiShift),
                                            .SpmiUpdate(SpmiUpdate),
                                            .SpmiCapEn(SpmiCapEn),
                                            .SpmiTrst(SpmiTrst),
                                            .SpmiTdo(SpmiTdo),
                                            .SpmiMeas(SpmiMeas),
                                            .SpmiPathEn(SpmiPathEn),
                                            .WRCK(WRCK),
                                            .WSI(WSI),
                                            .ShiftWR(ShiftWR),
                                            .UpdateWR(UpdateWR),
                                            .CaptureWR(CaptureWR),
                                            .WDREn(WDREn),
                                            .WRSTN(WRSTN),
                                            .MeasureWR(MeasureWR),
                                            .WRO(WRO),
                                            .PsmAvfs_Gater(PsmAvfs_Gater),
                                            .Isolate(Isolate),
                                            .SkipPhase(SkipPhase),
                                            .Reset(Reset),
                                            .CCLK(Host_Clk),
                                            .CCLK_CDC(Host_CDC_Clk),              // provide CDC'd asynchronous enable for JTAG/CCLK synchronizers
                                            .HostReg_CCLK(HostReg_Clk)
                                          `ifdef AVFS_ZAK                         // M&M bus interface
                                          , .MmBusIn_M1(MmBusIn_M1[7:0]),         //   8-bit M&M bus input contains local reg addr in M2 and write data in M4
                                            .MmLclRdEn_M2(MmLclRdEn_M2),          //   local register read indication
                                            .MmLclWrEn_M2(MmLclWrEn_M2),          //   local register write indication
                                            .MmLclSz_M2(MmLclSz_M2[1:0]),         //   local register size indication
                                            .MmState(MmState[`MM_STATE]),         //   M&M state for read/write byte select
                                            .MmDatOut_M4(MmDatOut_M4[`MM_DATA]),  //   8-bit data back to M&M bus for read
                                            .ScanShiftEn(ScanShiftEn),            //   SSE needed for M&M write staging flops
                                            .LBistEn(LBistEn),                    //   SSE needed for M&M write staging flops
                                            .MmActiveEn(MmActiveEn)               // feedback to top-level CCG to allow register access (i.e. wake up clocks)
                                          `endif
                                            );

`ifdef AVFS_ZAK
// SpmiTdo is not an output for ZAK
//! unused SpmiTdo
`endif

// Default ck-gater output from AVFS_CPA but not used because there are no flops at top level
//! unused CgenAvfsEn
                                 

  gfx_AVFS_CPA I_AVFS_CPA(
          .ScanShiftEn(ScanShiftEn),
          .CGater01_CG0(CGater01_CG0),
          .Reset(Reset),
          .CCLK(HostCpa_Clk),
          .HostReg_CCLK(HostReg_Clk),
          .CgenAvfsEn(CgenAvfsEn),
          .AvfsNearMiss(AvfsNearMiss[49:0]),
          .PsmOutput(PsmDeltaOut[13:0]),
          .PsmClk(PsmDivClkGater),                                                      
          .AvfsGo(AvfsGo),
          .AvfsPathEn(AvfsPathEn),
          .LBistEn(LBistEn),
          .AvfsTestData(AvfsTestData),
          .AvfsTestMode(AvfsTestMode[1:0]),
          .AvfsPathDelay(AvfsPathDelay[149:0]),
          .AvfsGlobalPathDelay(AvfsGlobalPathDelay[2:0]),
          .AvfsAgeHigh(AvfsAgeHigh),
          .AvfsSrbRdDat(SrbLclAvfsRdDat[63:0]),
          .SelAvfsCtl1(SrbLclSelAvfsCtl1),
          .SelAvfsCtl2(SrbLclSelAvfsCtl2),
          .SelAvfsBucket0(SrbLclSelAvfsBucket0),
          .SelAvfsBucket1(SrbLclSelAvfsBucket1),
          .SelAvfsValid(SrbLclSelAvfsValid),
          .SelAvfsNorm0(SrbLclSelAvfsNorm0),
          .SelAvfsNorm1(SrbLclSelAvfsNorm1),
          .SelAvfsNorm2(SrbLclSelAvfsNorm2),
          .AvfsSrbWrDat_4(RegisterWrDat[63:0]),
          .SrbLclWrEn_4(GlobalSelCPA & RegisterWriteEn));  

   // Total number of gate paths + wire paths must equal 50 as AVFS_CPA has
   // control logic that assumes this. Relative number of gate paths versus
   // wire paths, however, can vary as long as the sum is 50.
  gfx_AVFS_GATEPACK #(.path_count(`AVFS_GATE_PATHS_NUM)) I_AVFS_GATEPACK (
                                .AvfsPathEn(AvfsPathEn),
                                .AvfsTestData(AvfsTestData),
                                .AvfsTestMode(AvfsTestMode[1:0]),
                                .AvfsPathDelay(AvfsPathDelay[3*`AVFS_GATE_PATHS_NUM-1:0]),
                                .AvfsGlobalPathDelay(AvfsGlobalPathDelay[2:0]),
                                .AvfsNearMiss(AvfsNearMiss[`AVFS_GATE_PATHS]),
                                .AvfsAgeHigh(AvfsAgeHigh),
                                .CCLK(Host_Clk),
                                .ScanShiftEn(ScanShiftEn),.CGater01_CG0(CGater01_CG0),
                                .Reset(Reset)
                               );

  gfx_AVFS_WIREPACK #(.path_count(`AVFS_WIRE_PATHS_NUM)) I_AVFS_WIREPACK (
                                .AvfsPathEn(AvfsPathEn),
                                .AvfsTestData(AvfsTestData),
                                .AvfsTestMode(AvfsTestMode[1:0]),
                                .AvfsPathDelay(AvfsPathDelay[149:3*`AVFS_GATE_PATHS_NUM]),
                                .AvfsGlobalPathDelay(AvfsGlobalPathDelay[2:0]),
                                .AvfsNearMiss(AvfsNearMiss[49:`AVFS_GATE_PATHS_NUM]),
                                .AvfsAgeHigh(AvfsAgeHigh),
                                .CCLK(Host_Clk),
                                .ScanShiftEn(ScanShiftEn),.CGater01_CG0(CGater01_CG0),
                                .Reset(Reset)
                               );

   // Added a Host Clock Reset signal to JTAG_Psm_Rst_Reset for initialization at start-up.
   assign     JTAG_Psm_Rst_Reset = JTAG_Psm_Rst | Reset;
   assign     Psm_Rst = TrstSyncd | Reset;
   //assign     Psm_Rst = Reset;

   wire Avfs_ScanMode_CDC;

   // PSM Oscillator connections
   wire [1:0]  clkDivider;
   wire        oscEn;
   wire [31:0] psmOscLsb;
   wire [7:0]  psmOscMsb0;
   wire [7:0]  psmOscMsb1;
   wire        psmOscGater;
   wire        psmOscGater_1;
   wire [31:0] psmOscLsb_mcp;
   wire [7:0] psmOscMsb0_mcp;
   wire [7:0] psmOscMsb1_mcp;

   
  gfx_CORE_PSM_IP_NO_JTAG #(
      .PSM_CURPSM_LENGTH(PSM_CURPSM_LENGTH),
      .MCP_CYCLES_MAPSMRO(MCP_CYCLES_MAPSMRO),
      .MCP_CYCLES_AVGPSMREG(MCP_CYCLES_AVGPSMREG)
   ) I_Psm (
       .ScanShiftEn(ScanShiftEn),
       .Psm_Rst(Psm_Rst),
       .Psm_ScanMode(Avfs_ScanMode_CDC),
       .Psm_Gater(PsmAvfs_Gater),  
       .JTAG_Psm_Enable(JTAG_Psm_Enable),
       .JTAG_Psm_AvgEn(JTAG_Psm_AvgEn),
       .JTAG_Psm_Rst(JTAG_Psm_Rst_Reset),
       .JTAG_Psm_OneShot(JTAG_Psm_OneShot),               
       .PsmEn(PsmEn),
       .Host_Clk(PsmClk),
       .HostReg_Clk(PsmReg_Clk),
       .nullNxtAvg(SkipPhase),
       .div_clk_gater(PsmDivClkGater),
`ifdef GFX_DROOP_DETECT
       .psmCountOut(PsmOut[PSM_CURPSM_LENGTH-1:0]), // directing PSM count out through AVFS3 for GNL purposes
`endif
       .deltaPsmRegOut(PsmDeltaOut[13:0]),                   
       .Alarm_Flag(Psm_Alarm_Flag),
       .WrDat(RegisterWrDat[63:0]),
       .RdDat(PSM_RdDat[63:0]),
       .MM_Psm_Enable(MM_Psm_Enable),
       .SelPsmParam(SelPsmParam),
       .SelPsmData(SelPsmData),
       .WrEn(GlobalSelPsm & RegisterWriteEn),
       .clkDivider(clkDivider),
       .oscEn(oscEn),
       .psmOscLsb(psmOscLsb_mcp),
       .psmOscMsb0(psmOscMsb0_mcp),
       .psmOscMsb1(psmOscMsb1_mcp),
       .psmOscGater(psmOscGater_1), 
       .dbgCurPsm(dbgCurPsm[PSM_CURPSM_LENGTH-1:0]),
       .dbgAvgPsm(dbgAvgPsm[PSM_CURPSM_LENGTH-1:0]),
       .dbgThreshold(dbgThreshold[PSM_CURPSM_LENGTH-1:0]),
       .dbgMinMax(dbgMinMax[PSM_CURPSM_LENGTH-1:0]),
       .dbgAlarm(dbgAlarm)
        );

  // PSM Oscillator Macro Instantiation
  gfx_CORE_PSM_IP_OSC #(
    .JTAG_CLK_DIV_W (2),
    .CURPSM_W       (14)
  )
  I_CORE_PSM_IP_OSC (
    .clk         (CCLK), // Fixing for http://bugs.amd.com/ubts/prod/issue/view?id=590907
    .div         (clkDivider[1:0]),
    .pwrOk       (1'b1), // No one should be using AVFS_PSM_WRAP now, which AVFS 2.0 and earlier, Just tieing it HIGH
    .oscEn       (oscEn), 
    .psmOscLsb   (psmOscLsb[31:0]),
    .psmOscMsb0  (psmOscMsb0[7:0]),
    .psmOscMsb1  (psmOscMsb1[7:0]),
    .psmOscGater (psmOscGater)
  );

  // Move the MCBUF here on mapsmro outputs as Reset is not available inside CORE_PSM_IP_OSC wrapper 

`ifdef AVFS3_MCENBUF_ON
  wire mcbufEn;
  assign mcbufEn = oscEn & (~Psm_Rst) & psmOscGater ; 
  gfx_CORE_MCENBUF #(.size(32), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscLsb_mcp (.A(psmOscLsb[31:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscLsb_mcp[31:0]));
  gfx_CORE_MCENBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb0_mcp (.A(psmOscMsb0[7:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscMsb0_mcp[7:0]));
  gfx_CORE_MCENBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb1_mcp (.A(psmOscMsb1[7:0]), 
                                                                          .EN(mcbufEn),
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscMsb1_mcp[7:0]));
`else
  gfx_CORE_MCBUF #(.size(32), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscLsb_mcp (.A(psmOscLsb[31:0]), 
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscLsb_mcp[31:0]));
  gfx_CORE_MCBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb0_mcp (.A(psmOscMsb0[7:0]), 
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscMsb0_mcp[7:0]));
  gfx_CORE_MCBUF #(.size(8), .Cycles(MCP_CYCLES_MAPSMRO)) I_psmOscMsb1_mcp (.A(psmOscMsb1[7:0]), 
                                                                          .DSTCLK(CCLK),
                                                                          .Z(psmOscMsb1_mcp[7:0]));
`endif

  assign psmOscGater_1 = Avfs_ScanMode_CDC ? 1'b1 : psmOscGater ; // Forcing psmOscGater to 1'b1 in test mode
  //assign psmOscGater_1 = psmOscGater ; 
                                 
// debug interface
  reg  [DBG_LENGTH-1:0] DbgBusNxt;
  wire dbgEn_ScanMode;
   
  // CDCBUF for Avfs_ScanMode (ATPG test JTAG -> Host_clk)
  //CORE_CDCBUF #(.size(1), .Cycles(`CDC_JTAG_X_DLY_CYCLES))
  //CORE_CDCENBUF #(.size(1), .Cycles(0), .Setup(CDC_TST_PROP_DLY)) // add a top level param to configure X prop delay
  //   I_CDC_Avfs_ScanMode (.A(Avfs_ScanMode),
  //                  .Z(Avfs_ScanMode_CDC),
  //                  .EN(1'b1),
  //                  .DSTCLK(Host_Clk) );
   // Fix for http://bugs.amd.com/ubts/prod/issue/view?id=643502
  techind_cdcefpm_icd #(.DestClkCnt(0), .CdcDestSetup(CDC_TST_PROP_DLY), .size(1)) I_Avfs_ScanMode_CDC (
     .A(Avfs_ScanMode), 
     .DSTCLK(Host_Clk), 
     .EN(1'b1), 
     .Z(Avfs_ScanMode_CDC)
     `ifdef ABV
       ,.InputChkDis((WRSTN!==1'b1)) 
     `endif
     );

  assign dbgEn_ScanMode = dbgEn | Avfs_ScanMode_CDC;

  always @* begin
    casez({dbgEn_ScanMode, DbgBusSel[DBG_CFG_LENGTH-1:0]})
      4'b0_???: DbgBusNxt[DBG_LENGTH-1:0] = {DBG_LENGTH{1'b0}};
      4'b1_000: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgCurPsm[PSM_CURPSM_LENGTH-1:0],dbgAlarm}; //DbgBusPSM
      4'b1_001: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgAvgPsm[PSM_CURPSM_LENGTH-1:0],dbgAlarm}; //DbgBusPSM
      4'b1_010: DbgBusNxt[DBG_LENGTH-1:0] = {2'b0, dbgThreshold[PSM_CURPSM_LENGTH-1:0]}; //DbgBusPSM
      4'b1_011: DbgBusNxt[DBG_LENGTH-1:0] = {1'b0, dbgMinMax[PSM_CURPSM_LENGTH-1:0], dbgAlarm}; //DbgBusPSM
      4'b1_100: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsNearMiss[15:0]}; //DbgBusCPA
      4'b1_101: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsNearMiss[31:16]}; //DbgBusCPA
      4'b1_110: DbgBusNxt[DBG_LENGTH-1:0] = {AvfsNearMiss[47:32]}; //DbgBusCPA
      4'b1_111: DbgBusNxt[DBG_LENGTH-1:0] = {PsmDeltaOut[13:0],AvfsNearMiss[49:48]}; //DbgBusCPA & DbgBusPSM
      // VCS coverage off
      default: DbgBusNxt[DBG_LENGTH-1:0]      = {DBG_LENGTH{1'bx}};
      // VCS coverage on
    endcase
  end

  gfx_CORE_FLOP_GATED_RESET  #(.size(DBG_LENGTH))  I_DbgBus (.D(DbgBusNxt[DBG_LENGTH-1:0]), .Q(DbgBus_int[DBG_LENGTH-1:0]), .G(dbgEn_ScanMode | TrstSyncd), .GC(1'b0), .R(TrstSyncd), .SSE(ScanShiftEn), .C(Host_Clk) // NOTE: ".GC(1'b0)" is unused
                                                          );

  // Avfs_ScanMode==1 will force DbgBus and Alarm_Flag to 0

  assign DbgBus[DBG_LENGTH-1:0] = DbgBus_int[DBG_LENGTH-1:0] & {DBG_LENGTH{~Avfs_ScanMode_CDC}};
  

                 
endmodule // AVFS_PSM_WRAP
//==================================================================================================
//! enable one_module_per_file
// leda E_001 on
//==================================================================================================
