`timescale 1ns/100ps
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Module Name    : techind_sync4_implementation
///  
///  Revision       : $Revision$
///  Release Status : $State$
///  Author         : $Author$
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    ABSTRACT
///
///    This wrapper module implements a technology independent means to directly instantiate
///    a technology-specific synchronizer cell into RTL during synthesis. With appropriate
///    switches [`defines and parameters], the desired synchronizer within the target library
///    is instantiated in the design using a naming convention 'd0nt_SYNC' that is meaningful
///    to the downstream tool flow.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///
///    SYNTHESIS USAGE:
///
///    In order for synthesis to pick up an appropriate synchronizer from the target library,
///    the following value must be defined:
///
///        `define  SYNTHESIS
///
///    In addition to the 'SYNTHESIS' define, one technology library must also be specified.
///    This revision of the wrapper currently supports the following libraries [note, this
///    list must be maintained as we move forward from one process technology to the next in
///    order to ensure that the synchronizer cells are properly instantiated in future designs.
///
///        `define  < no libraries are currently supported >
///
///    Note, this wrapper is only a part of a larger flow and should not be instantiated
///    directly by designers. In order to heavily discourage such abuse, when the 'SYNTHESIS'
///    define is not set, this wrapper module will provide no internal functionality, nor
///    continuity through the module. Consequently, erroneous simulation results should
///    ensue.
///
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///    Copyright (C) 2011
///      by Advanced Micro Devices (AMD) Corporation
///  
///    All rights reserved. This source file is considered proprietary and confidential
///    information of AMD Corp. As such, this source file may not be distributed by any
///    means, in whole, or in part, without the express, written approval of AMD Corp.
///  
///   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
///  
///  Revision History :
///  
///  
//  
///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
module techind_sync4_implementation (
    SB,
    RB,
    FLUSH,
    CKsrc,
    CKdst,
    EN,
    Dsrc,
    Qsrc,
    Qdst,
    SC1,
    SC2,
    SE,
    SI,
    SO
  );
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  parameter  SF = 0;  ///  source-flop required
  parameter  HR = 0;  ///  half-stage required [lead synchronizer in a chain]
  parameter  HP = 0;  ///  half-stage permitted [subsequent synchronizers in a chain]
  parameter  AS = 0;  ///  asynchronous set required
  parameter  AR = 0;  ///  asynchronous reset required
  parameter  AF = 0;  ///  asynchronous flush required
  parameter  CE = 0;  ///  clock enable required
  parameter  TC = 0;  ///  TransportCycles
  parameter  TD = 0;  ///  Transport Delay
  parameter  SD = 1;  ///  scan support, mux-D style
  parameter  SL = 0;  ///  scan supprot, LSSD style
  parameter  MV = 0;  ///  MultiVoltage cell for lower Tau
  parameter  HC = 0;  ///  Hold Check passed in rtl simulation
  parameter  SP = 0;  ///  SrcPath for TC overriden
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  input       SB;
  input       RB;
  input       FLUSH;
  input       CKsrc;
  input       CKdst;
  input       EN;
  input       Dsrc;
  output      Qsrc;
  output      Qdst;
  input       SC1;
  input       SC2;
  input       SE;
  input       SI;
  output      SO;
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  // river translate_off
  //////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////
  ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  ///
  ///  As indicated earlier, we use a variety of defines to guide synthesis down the desired
  ///  code-path to instantiate the technology-specific marker cell. The code below leverages
  ///  these defines [and parameters] to accomplish this goal.
  `ifndef SYNTHESIS
     ///  should this path be taken, the designer must have used this wrapper incorrectly. So,
     ///  we are creating a simulation exposure. Note, it would have been preferable to cause
     ///  a hard compile failure, but the VRQ X-uglification tool chokes when we try to do
     ///  that.
  `else  
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     //////                                                        /////////////////////
     //////    THE FOLLOWING CODE SHOULD BE ACTIVELY MAINTAINED    /////////////////////
     //////    AS NEW TECHNOLOGY LIBRARIES ARE TO BE SUPPORTED     /////////////////////
     //////                                                        /////////////////////
     //////    * note, as new 'define values are supported, they   /////////////////////
     //////      should be captured in the 'SYNTHESIS USAGE'       /////////////////////
     //////      documentation at the top of this file.            /////////////////////
     //////                                                        /////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     ///
     ///  For technology libraries that do support a physical synchronizer cell that matches
     ///  the feature set requested, we instantiate the appropriate cell from the target
     ///  library. However, when the selected library does not support a syncrhonizer with
     ///  the requested feature set, we shall substitute a wire.
     ///
     ///  As a consequence of this incorrect mapping selection, the overall flow must have
     ///  a means for quickly determining the quality of the mapping. An unmapped cell will
     ///  need to be flagged to the designer so that they may either alter their selection,
     ///  or work with the library team to provide the appropriate flavour of synchronizer.
     ///  For development purposes, we will model the synchronizer pipleine and use it where we
     ///  select our synchronizers. When we are closer to deployment, we shall remove this
     ///  code and replace it with real cell instantiations from the appropriate cell
     ///  libraries.
       generate
         ///  Given the plethora of different synchronizer flavours that we have, we have
         ///  defined a specific format for which syncrhonizer instantiation should fit.
         ///  The following [fake] instantiation represents the fully featured instantiation
         ///  for a D-mux scan style syncrhonizer:
         ///
         ///      syncCellType  d0nt_sync ( .CLKDST(CKdst),  .DSRC(Dsrc),     .Q(Qdst),        .CLKSRC(CKsrc),  .QSRC(Qsrc),   /// main data synchronization
         ///                                .RESETH(RST),    .SETH(SET),      .FLSH(FLUSH),                                    /// asynchronous controls
         ///                                .SENDST(SE), .SDIDST(SI), .SENSRC(SENsrc), .SDISRC(SDIsrc));                /// scan signals
         ///
         ///  When a particular signal is not required [due to a feature not supported], the
         ///  particular instantiation should simply clear the port connection field, leaving
         ///  all in-use fields where they are found in the example instantiation above.
         if(!SF && ( !HR || HP ) &&  AS && !AR && !AF &&  CE && !SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
         // for ubts 629451
         assign Qdst = Dsrc;  

            // NO SUPPORT
            // hdsync4ormsfqnsxss1us hdsync4ormsfqnsxss1us_d0nt_sync_noscan_C1 (.Q(Qdst),.CLK(CKdst),.TE(SE),.D(Dsrc),.SETB(SB),.EN(EN));

         end else         if(!SF && ( !HR || HP ) && !AS && !AR && !AF && !CE &&  SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
            if (HC) begin
            case (TC)
            0: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C0_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C1_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C2_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C3_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C4_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C5_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C6_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C7_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C8_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C9_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C10_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C11_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C12_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C13_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C14_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C15_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C16_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C17_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C18_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C19_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C20_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C21_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C22_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C23_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C24_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C25_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C26_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C27_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C28_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C29_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C30_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C31_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C32_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C33_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C34_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C35_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C36_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C37_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C38_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C39_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C40_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C41_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C42_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C43_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C44_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C45_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C46_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C47_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C48_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C49_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC1_C_greater_than_50_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
         endcase
        end else begin
            case (TC)
            0: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C0_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C1_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C2_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C3_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C4_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C5_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C6_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C7_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C8_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C9_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C10_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C11_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C12_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C13_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C14_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C15_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C16_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C17_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C18_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C19_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C20_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C21_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C22_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C23_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C24_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C25_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C26_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C27_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C28_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C29_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C30_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C31_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C32_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C33_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C34_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C35_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C36_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C37_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C38_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C39_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C40_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C41_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C42_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C43_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C44_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C45_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C46_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C47_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C48_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C49_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4MSFQ_Y4_1 hdsync4msfqxss1us_d0nt_sync_HC0_C_greater_than_50_D[TD:TD] (.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
         endcase
        end
         end else         if(!SF && ( !HR || HP ) && !AS &&  AR && !AF && !CE && !SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
         if (HC) begin
            case (TC)
            0: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C0_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C1_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C2_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C3_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C4_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C5_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C6_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C7_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C8_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C9_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C10_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C11_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C12_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C13_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C14_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C15_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C16_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C17_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C18_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C19_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C20_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C21_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C22_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C23_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C24_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C25_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C26_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C27_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C28_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C29_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C30_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C31_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C32_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C33_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C34_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C35_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C36_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C37_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C38_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C39_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C40_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C41_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C42_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C43_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C44_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C45_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C46_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C47_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C48_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C49_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC1_C_greater_than_50_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
         endcase
        end else begin
            case (TC)
            0: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C0_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C1_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C2_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C3_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C4_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C5_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C6_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C7_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C8_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C9_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C10_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C11_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C12_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C13_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C14_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C15_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C16_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C17_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C18_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C19_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C20_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C21_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C22_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C23_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C24_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C25_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C26_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C27_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C28_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C29_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C30_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C31_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C32_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C33_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C34_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C35_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C36_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C37_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C38_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C39_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C40_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C41_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C42_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C43_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C44_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C45_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C46_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C47_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C48_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C49_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_noscan_HC0_C_greater_than_50_D[TD:TD] (.RESETH(~RB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
         endcase
        end

         end else         if(!SF && ( !HR || HP ) && !AS &&  AR && !AF && !CE && SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
         if (HC) begin
            case (TC)
            0: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C0_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C1_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C2_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C3_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C4_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C5_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C6_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C7_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C8_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C9_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C10_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C11_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C12_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C13_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C14_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C15_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C16_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C17_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C18_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C19_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C20_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C21_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C22_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C23_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C24_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C25_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C26_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C27_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C28_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C29_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C30_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C31_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C32_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C33_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C34_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C35_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C36_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C37_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C38_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C39_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C40_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C41_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C42_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C43_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C44_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C45_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C46_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C47_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C48_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C49_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC1_C_greater_than_50_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
         endcase
        end else begin
            case (TC)
            0: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C0_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C1_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C2_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C3_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C4_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C5_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C6_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C7_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C8_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C9_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C10_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C11_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C12_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C13_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C14_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C15_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C16_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C17_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C18_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C19_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C20_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C21_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C22_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C23_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C24_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C25_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C26_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C27_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C28_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C29_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C30_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C31_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C32_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C33_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C34_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C35_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C36_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C37_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C38_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C39_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C40_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C41_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C42_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C43_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C44_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C45_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C46_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C47_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C48_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C49_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4RMSFQ_Y4_1 hdsync4rmsfqxss1us_d0nt_sync_HC0_C_greater_than_50_D[TD:TD] (.RESETH(~RB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
         endcase
        end

         end else         if( SF && ( !HR || HP ) && !AS && !AR && !AF && !CE &&  SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
            if (HC) begin
            case (TC)
            0: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C0_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            1: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C1_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            2: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C2_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            3: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C3_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            4: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C4_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            5: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C5_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            6: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C6_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            7: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C7_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            8: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C8_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            9: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C9_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            10: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C10_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            11: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C11_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            12: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C12_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            13: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C13_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            14: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C14_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            15: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C15_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            16: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C16_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            17: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C17_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            18: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C18_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            19: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C19_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            20: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C20_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            21: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C21_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            22: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C22_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            23: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C23_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            24: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C24_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            25: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C25_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            26: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C26_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            27: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C27_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            28: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C28_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            29: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C29_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            30: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C30_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            31: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C31_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            32: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C32_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            33: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C33_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            34: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C34_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            35: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C35_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            36: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C36_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            37: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C37_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            38: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C38_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            39: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C39_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            40: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C40_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            41: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C41_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            42: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C42_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            43: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C43_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            44: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C44_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            45: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C45_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            46: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C46_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            47: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C47_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            48: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C48_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            49: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C49_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            default: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC1_C_greater_than_50_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
         endcase
         end else begin
            case (TC)
            0: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C0_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            1: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C1_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            2: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C2_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            3: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C3_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            4: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C4_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            5: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C5_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            6: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C6_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            7: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C7_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            8: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C8_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            9: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C9_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            10: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C10_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            11: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C11_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            12: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C12_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            13: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C13_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            14: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C14_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            15: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C15_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            16: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C16_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            17: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C17_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            18: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C18_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            19: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C19_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            20: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C20_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            21: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C21_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            22: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C22_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            23: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C23_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            24: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C24_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            25: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C25_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            26: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C26_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            27: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C27_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            28: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C28_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            29: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C29_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            30: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C30_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            31: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C31_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            32: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C32_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            33: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C33_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            34: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C34_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            35: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C35_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            36: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C36_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            37: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C37_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            38: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C38_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            39: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C39_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            40: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C40_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            41: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C41_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            42: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C42_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            43: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C43_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            44: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C44_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            45: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C45_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            46: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C46_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            47: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C47_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            48: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C48_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            49: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C49_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
            default: HDBULT08_SYNC4CDCMSFQ_Y6_1 hdsync4cdcmsfqxss1us_d0nt_sync_HC0_C_greater_than_50_D[TD:TD] (.SENDST(SE),.QSRC(Qsrc),.CLKDST(CKdst),.DSRC(Dsrc),.CLKSRC(CKsrc),.SENSRC(1'b0),.Q(Qdst),.SDISRC(1'b0),.SDIDST(SI));
         endcase
        end
         end else         if(!SF && ( !HR || HP ) && !AS &&  AR && !AF &&  CE && !SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
         // for ubts 626839, no sync4 cell with async reset
         assign Qdst = Dsrc;

            // NO SUPPORT
            // hdsync4ormsfqnsxss1us derived_reset_hdsync4ormsfqnsxss1us_d0nt_sync_noscan_C1 (.Q(QdstX),.CLK(CKdst),.D(DsrcX),.TE(SE),.SETB(SB),.EN(EN));

         end else         if( SF && ( !HR || HP ) && !AS && !AR && !AF && !CE && !SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
            if (HC) begin
            case (TC)
            0: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C0_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            1: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C1_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            2: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C2_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            3: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C3_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            4: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C4_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            5: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C5_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            6: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C6_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            7: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C7_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            8: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C8_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            9: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C9_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            10: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C10_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            11: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C11_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            12: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C12_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            13: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C13_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            14: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C14_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            15: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C15_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            16: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C16_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            17: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C17_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            18: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C18_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            19: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C19_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            20: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C20_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            21: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C21_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            22: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C22_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            23: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C23_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            24: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C24_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            25: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C25_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            26: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C26_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            27: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C27_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            28: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C28_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            29: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C29_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            30: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C30_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            31: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C31_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            32: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C32_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            33: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C33_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            34: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C34_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            35: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C35_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            36: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C36_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            37: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C37_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            38: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C38_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            39: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C39_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            40: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C40_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            41: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C41_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            42: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C42_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            43: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C43_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            44: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C44_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            45: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C45_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            46: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C46_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            47: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C47_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            48: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C48_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            49: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C49_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            default: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC1_C_greater_than_50_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
         endcase
         end else begin
            case (TC)
            0: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C0_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            1: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C1_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            2: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C2_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            3: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C3_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            4: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C4_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            5: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C5_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            6: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C6_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            7: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C7_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            8: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C8_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            9: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C9_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            10: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C10_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            11: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C11_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            12: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C12_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            13: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C13_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            14: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C14_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            15: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C15_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            16: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C16_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            17: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C17_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            18: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C18_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            19: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C19_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            20: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C20_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            21: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C21_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            22: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C22_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            23: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C23_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            24: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C24_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            25: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C25_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            26: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C26_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            27: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C27_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            28: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C28_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            29: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C29_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            30: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C30_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            31: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C31_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            32: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C32_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            33: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C33_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            34: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C34_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            35: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C35_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            36: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C36_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            37: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C37_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            38: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C38_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            39: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C39_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            40: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C40_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            41: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C41_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            42: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C42_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            43: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C43_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            44: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C44_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            45: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C45_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            46: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C46_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            47: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C47_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            48: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C48_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            49: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C49_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
            default: HDBULT08_SYNC4CDCMSFQ_Y6_1 ns_hdsync4cdcmsfqxss1us_d0nt_sync_noscan_HC0_C_greater_than_50_D[TD:TD] (.CLKDST(CKdst),.QSRC(Qsrc),.SENDST(1'b0),.DSRC(Dsrc),.SENSRC(1'b0),.CLKSRC(CKsrc),.SDISRC(1'b0),.Q(Qdst),.SDIDST(1'b0));
         endcase
        end
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
         end else         if(!SF && ( !HR || HP ) &&  AS && !AR && !AF && !CE && !SD && !SL) begin
         if (HC) begin
            case (TC)
            0: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C0_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C1_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C2_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C3_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C4_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C5_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C6_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C7_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C8_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C9_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C10_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C11_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C12_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C13_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C14_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C15_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C16_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C17_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C18_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C19_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C20_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C21_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C22_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C23_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C24_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C25_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C26_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C27_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C28_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C29_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C30_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C31_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C32_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C33_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C34_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C35_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C36_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C37_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C38_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C39_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C40_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C41_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C42_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C43_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C44_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C45_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C46_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C47_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C48_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C49_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC1_C_greater_than_50_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
         endcase
        end else begin
            case (TC)
            0: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C0_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C1_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C2_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C3_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C4_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C5_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C6_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C7_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C8_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C9_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C10_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C11_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C12_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C13_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C14_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C15_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C16_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C17_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C18_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C19_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C20_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C21_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C22_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C23_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C24_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C25_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C26_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C27_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C28_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C29_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C30_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C31_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C32_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C33_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C34_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C35_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C36_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C37_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C38_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C39_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C40_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C41_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C42_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C43_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C44_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C45_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C46_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C47_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C48_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C49_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_noscan_HC0_C_greater_than_50_D[TD:TD] (.SETH(~SB),.SDI(1'b0),.Q(Qdst),.SEN(1'b0),.CLK(CKdst),.D(Dsrc));
         endcase
        end
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///

         end else         if(!SF && ( !HR || HP ) &&  AS && !AR && !AF && !CE && SD && !SL) begin
         if (HC) begin
            case (TC)
            0: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C0_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C1_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C2_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C3_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C4_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C5_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C6_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C7_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C8_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C9_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C10_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C11_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C12_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C13_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C14_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C15_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C16_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C17_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C18_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C19_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C20_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C21_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C22_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C23_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C24_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C25_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C26_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C27_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C28_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C29_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C30_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C31_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C32_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C33_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C34_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C35_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C36_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C37_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C38_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C39_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C40_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C41_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C42_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C43_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C44_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C45_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C46_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C47_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C48_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C49_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC1_C_greater_than_50_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
         endcase
        end else begin
            case (TC)
            0: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C0_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            1: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C1_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            2: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C2_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            3: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C3_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            4: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C4_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            5: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C5_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            6: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C6_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            7: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C7_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            8: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C8_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            9: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C9_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            10: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C10_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            11: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C11_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            12: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C12_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            13: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C13_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            14: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C14_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            15: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C15_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            16: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C16_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            17: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C17_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            18: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C18_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            19: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C19_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            20: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C20_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            21: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C21_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            22: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C22_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            23: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C23_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            24: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C24_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            25: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C25_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            26: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C26_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            27: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C27_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            28: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C28_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            29: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C29_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            30: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C30_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            31: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C31_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            32: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C32_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            33: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C33_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            34: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C34_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            35: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C35_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            36: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C36_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            37: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C37_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            38: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C38_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            39: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C39_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            40: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C40_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            41: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C41_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            42: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C42_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            43: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C43_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            44: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C44_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            45: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C45_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            46: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C46_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            47: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C47_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            48: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C48_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            49: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C49_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
            default: HDBULT08_SYNC4SMSFQ_Y4_1 hdsync4smsfqxss1us_d0nt_sync_HC0_C_greater_than_50_D[TD:TD] (.SETH(~SB),.SDI(SI),.Q(Qdst),.SEN(SE),.CLK(CKdst),.D(Dsrc));
         endcase
        end
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///

         end else         if(!SF && ( !HR || HP ) && !AS && !AR &&  AF && !CE && !SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///

            // NO SUPPORT
            // hdsync4flshmsfqnsxss1us hdsync4flshmsfqnsxss1us_d0nt_sync_noscan_C1 (.Q(Qdst),.CLK(CKdst),.D(Dsrc),.FLSH(FLUSH));
            assign Qdst = Dsrc;

         end else         if(!SF && ( !HR || HP ) && !AS && !AR && !AF && !CE && !SD && !SL) begin
         /// -------------------  ---------  --+----------------+----------------+----------------+----------------+----------  ///
            if (HC) begin
            case (TC)
            0: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C0_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            1: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C1_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            2: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C2_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            3: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C3_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            4: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C4_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            5: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C5_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            6: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C6_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            7: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C7_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            8: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C8_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            9: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C9_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            10: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C10_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            11: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C11_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            12: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C12_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            13: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C13_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            14: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C14_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            15: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C15_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            16: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C16_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            17: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C17_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            18: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C18_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            19: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C19_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            20: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C20_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            21: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C21_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            22: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C22_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            23: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C23_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            24: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C24_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            25: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C25_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            26: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C26_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            27: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C27_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            28: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C28_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            29: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C29_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            30: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C30_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            31: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C31_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            32: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C32_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            33: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C33_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            34: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C34_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            35: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C35_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            36: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C36_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            37: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C37_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            38: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C38_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            39: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C39_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            40: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C40_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            41: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C41_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            42: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C42_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            43: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C43_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            44: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C44_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            45: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C45_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            46: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C46_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            47: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C47_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            48: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C48_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            49: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C49_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            default: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC1_C_greater_than_50_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
        endcase
        end else begin
            case (TC)
            0: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C0_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            1: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C1_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            2: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C2_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            3: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C3_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            4: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C4_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            5: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C5_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            6: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C6_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            7: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C7_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            8: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C8_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            9: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C9_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            10: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C10_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            11: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C11_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            12: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C12_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            13: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C13_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            14: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C14_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            15: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C15_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            16: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C16_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            17: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C17_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            18: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C18_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            19: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C19_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            20: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C20_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            21: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C21_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            22: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C22_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            23: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C23_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            24: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C24_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            25: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C25_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            26: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C26_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            27: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C27_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            28: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C28_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            29: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C29_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            30: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C30_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            31: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C31_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            32: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C32_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            33: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C33_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            34: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C34_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            35: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C35_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            36: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C36_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            37: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C37_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            38: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C38_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            39: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C39_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            40: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C40_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            41: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C41_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            42: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C42_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            43: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C43_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            44: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C44_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            45: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C45_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            46: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C46_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            47: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C47_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            48: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C48_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            49: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C49_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
            default: HDBULT08_SYNC4MSFQ_Y4_1 ns_hdsync4msfqxss1us_d0nt_sync_noscan_HC0_C_greater_than_50_D[TD:TD] (.Q(Qdst),.SDI(1'b0),.CLK(CKdst),.SEN(1'b0),.D(Dsrc));
        endcase
        end
        end else begin assign Qdst = Dsrc;   end
     endgenerate
     ///  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
     ///////////////////////////////////////////////////////////////////////////////////
     //////                                         ////////////////////////////////////
     //////    END OF 'ACTIVELY' MAINTAINED CODE    ////////////////////////////////////
     //////                                         ////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
     ///////////////////////////////////////////////////////////////////////////////////
  `endif  ///  SYNTHESIS
  // river translate_on
endmodule
